@@ -5,7 +5,7 @@ ARTIFACT_PATH="/artifacts/wallet-core/${CI_COMMIT_REF}/*.deb"
 
 RSYNC_HOST="taler.host.internal"
 RSYNC_PORT=424242
-RSYNC_PATH="incoming_packages/bookworm-taler-ci/"
+RSYNC_PATH="incoming_packages/trixie-taler-ci/"
 RSYNC_DEST="rsync://${RSYNC_HOST}/${RSYNC_PATH}"
 
 
