@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2022-2024 Taler Systems S.A.
+ (C) 2022-2025 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -18,7 +18,8 @@ import {
   Codec,
   TranslatedString,
   buildCodecForObject,
-  codecForBoolean
+  codecForBoolean,
+  codecOptionalDefault,
 } from "@gnu-taler/taler-util";
 import {
   buildStorageKey,
@@ -30,19 +31,34 @@ interface Preferences {
   showDebugInfo: boolean;
   allowInsecurePassword: boolean;
   keepSessionAfterReload: boolean;
+  testingDialect: boolean;
+  preventCompression: boolean;
 }
 
 export const codecForPreferences = (): Codec<Preferences> =>
   buildCodecForObject<Preferences>()
-  .property("allowInsecurePassword", (codecForBoolean()))
-  .property("showDebugInfo", codecForBoolean())
-  .property("keepSessionAfterReload", (codecForBoolean()))
+    .property(
+      "allowInsecurePassword",
+      codecOptionalDefault(codecForBoolean(), false),
+    )
+    .property("showDebugInfo", codecOptionalDefault(codecForBoolean(), false))
+    .property("testingDialect", codecOptionalDefault(codecForBoolean(), false))
+    .property(
+      "keepSessionAfterReload",
+      codecOptionalDefault(codecForBoolean(), false),
+    )
+    .property(
+      "preventCompression",
+      codecOptionalDefault(codecForBoolean(), false),
+    )
     .build("Preferences");
 
 const defaultPreferences: Preferences = {
   allowInsecurePassword: false,
   showDebugInfo: false,
+  testingDialect: false,
   keepSessionAfterReload: false,
+  preventCompression: false,
 };
 
 const PREFERENCES_KEY = buildStorageKey(
@@ -75,16 +91,27 @@ export function getAllBooleanPreferences(): Array<keyof Preferences> {
     "showDebugInfo",
     "allowInsecurePassword",
     "keepSessionAfterReload",
+    "testingDialect",
+    "preventCompression",
   ];
 }
 
+const TALER_SCREEN_ID = 118;
+
 export function getLabelForPreferences(
   k: keyof Preferences,
   i18n: ReturnType<typeof useTranslationContext>["i18n"],
 ): TranslatedString {
   switch (k) {
-    case "showDebugInfo": return i18n.str`Show debug info`
-    case "allowInsecurePassword": return i18n.str`Allow Insecure password`
-    case "keepSessionAfterReload": return i18n.str`Keep session after reload`
+    case "showDebugInfo":
+      return i18n.str`Show debug info`;
+    case "testingDialect":
+      return i18n.str`Use testing dialect`;
+    case "allowInsecurePassword":
+      return i18n.str`Allow Insecure password`;
+    case "keepSessionAfterReload":
+      return i18n.str`Keep session after reload`;
+    case "preventCompression":
+      return i18n.str`Don't compress request`;
   }
 }
