@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2022-2024 Taler Systems S.A.
+ (C) 2022-2025 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -14,8 +14,16 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
-/*eslint quote-props: ["error", "consistent"]*/
-export const strings: { [s: string]: any } = {};
+export interface StringsType {
+  domain: string;
+  // lang: string;
+  completeness?: number;
+  // plural_forms: string;
+  locale_data: {
+    messages: Record<string, unknown>;
+  };
+}
+export const strings: Record<string, StringsType> = {};
 
 strings["de"] = {
   domain: "messages",
