@@ -19,7 +19,7 @@ import { useAnastasisContext } from "../../../context/anastasis.js";
 import { authMethods, KnownAuthMethods } from "../authMethod/index.jsx";
 import { AuthProvByStatusMap, State, testProvider } from "./index.js";
 
-interface Props {
+export interface Props {
   providerType?: KnownAuthMethods;
   onCancel: () => Promise<void>;
   notifications?: Notification[];
