@@ -20,10 +20,12 @@
  */
 
 import { ReducerState } from "@gnu-taler/anastasis-core";
-import { createExample, reducerStatesExample } from "../../utils/index.js";
+import { reducerStatesExample } from "../../utils/index.js";
 import { TruthsPayingScreen as TestedComponent } from "./TruthsPayingScreen.js";
+import * as tests from "@gnu-taler/web-util/testing";
 
 export default {
+  title: "Truths Paying",
   component: TestedComponent,
   args: {
     order: 10,
@@ -34,11 +36,12 @@ export default {
   },
 };
 
-export const Example = createExample(
+export const Example = tests.createExample(
   TestedComponent,
+  {},
   reducerStatesExample.truthsPaying,
 );
-export const WithPaytoList = createExample(TestedComponent, {
+export const WithPaytoList = tests.createExample(TestedComponent, {}, {
   ...reducerStatesExample.truthsPaying,
   payments: ["payto://x-taler-bank/bank/account"],
 } as ReducerState);
