@@ -16,12 +16,14 @@
 
 import {
   LocalNotificationBanner,
+  makeSafeCall,
   urlPattern,
   useBankCoreApiContext,
+  useChallengeHandler,
   useCurrentLocation,
-  useLocalNotification,
+  useLocalNotificationBetter,
   useNavigationContext,
-  useTranslationContext,
+  useTranslationContext
 } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
 
@@ -29,19 +31,25 @@ import {
   AbsoluteTime,
   AccessToken,
   HttpStatusCode,
-  TranslatedString,
+  TalerErrorCode,
+  TokenRequest,
   assertUnreachable,
-  createRFC8959AccessTokenEncoded,
+  createRFC8959AccessTokenEncoded
 } from "@gnu-taler/taler-util";
 import { useEffect } from "preact/hooks";
-import { useSessionState } from "./hooks/session.js";
+import {
+  useRefreshSessionBeforeExpires,
+  useSessionState,
+} from "./hooks/session.js";
 import { AccountPage } from "./pages/AccountPage/index.js";
 import { BankFrame } from "./pages/BankFrame.js";
-import { LoginForm } from "./pages/LoginForm.js";
+import { ConversionRateClassDetails } from "./pages/ConversionRateClassDetails.js";
+import { LoginForm, SESSION_DURATION } from "./pages/LoginForm.js";
+import { NewConversionRateClass } from "./pages/NewConversionRateClass.js";
 import { PublicHistoriesPage } from "./pages/PublicHistoriesPage.js";
 import { RegistrationPage } from "./pages/RegistrationPage.js";
 import { ShowNotifications } from "./pages/ShowNotifications.js";
-import { SolveChallengePage } from "./pages/SolveChallengePage.js";
+import { SolveMFAChallenges } from "./pages/SolveMFA.js";
 import { WireTransfer } from "./pages/WireTransfer.js";
 import { WithdrawalOperationPage } from "./pages/WithdrawalOperationPage.js";
 import { CashoutListForAccount } from "./pages/account/CashoutListForAccount.js";
@@ -55,14 +63,20 @@ import { ConversionConfig } from "./pages/regional/ConversionConfig.js";
 import { CreateCashout } from "./pages/regional/CreateCashout.js";
 import { ShowCashoutDetails } from "./pages/regional/ShowCashoutDetails.js";
 
+const TALER_SCREEN_ID = 100;
+
+Routing.SCREEN_ID = TALER_SCREEN_ID;
 export function Routing(): VNode {
   const session = useSessionState();
 
+  useRefreshSessionBeforeExpires();
+
   if (session.state.status === "loggedIn") {
     const { isUserAdministrator, username } = session.state;
     return (
       <BankFrame
         account={username}
+        routeNotifications={privatePages.notifications}
         routeAccountDetails={privatePages.myAccountDetails}
       >
         <PrivateRouting username={username} isAdmin={isUserAdministrator} />
@@ -70,10 +84,10 @@ export function Routing(): VNode {
     );
   }
   return (
-    <BankFrame>
+    <BankFrame routeNotifications={privatePages.notifications}>
       <PublicRounting
-        onLoggedUser={(username, token) => {
-          session.logIn({ username, token: token });
+        onLoggedUser={(username, token, expiration) => {
+          session.logIn({ username, token, expiration });
         }}
       />
     </BankFrame>
@@ -88,19 +102,24 @@ const publicPages = {
     /\/operation\/(?<wopid>[a-zA-Z0-9-]+)/,
     ({ wopid }) => `#/operation/${wopid}`,
   ),
-  solveSecondFactor: urlPattern(/\/2fa/, () => "#/2fa"),
 };
 
 function PublicRounting({
   onLoggedUser,
 }: {
-  onLoggedUser: (username: string, token: AccessToken) => void;
+  onLoggedUser: (
+    username: string,
+    token: AccessToken,
+    expiration: AbsoluteTime,
+  ) => void;
 }): VNode {
   const { i18n } = useTranslationContext();
   const location = useCurrentLocation(publicPages);
   const { navigateTo } = useNavigationContext();
+
   const { config, lib } = useBankCoreApiContext();
-  const [notification, notify, handleError] = useLocalNotification();
+  const [notification, notifyOnError] = useLocalNotificationBetter();
+  const mfa = useChallengeHandler();
 
   useEffect(() => {
     if (location === undefined) {
@@ -108,43 +127,59 @@ function PublicRounting({
     }
   }, [location]);
 
-  async function doAutomaticLogin(username: string, password: string) {
-    await handleError(async () => {
-      const resp = await lib
-        .auth(username)
-        .createAccessTokenBasic(username, password, {
-          scope: "readwrite",
-          duration: { d_us: "forever" },
-          refreshable: true,
-        });
-      if (resp.type === "ok") {
-        onLoggedUser(
-          username,
-          createRFC8959AccessTokenEncoded(resp.body.access_token),
-        );
-      } else {
-        switch (resp.case) {
-          case HttpStatusCode.Unauthorized:
-            return notify({
-              type: "error",
-              title: i18n.str`Wrong credentials for "${username}"`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case HttpStatusCode.NotFound:
-            return notify({
-              type: "error",
-              title: i18n.str`Account not found`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          default:
-            assertUnreachable(resp);
-        }
-      }
-    });
+  const tokenRequest = {
+    scope: "readwrite",
+    duration: SESSION_DURATION,
+    refreshable: true,
+  } as TokenRequest;
+
+  const [doAutomaticLogin, repeatLogin, lastCallingArgs] = mfa.withMfaHandler(
+    ({ challengeIds, onChallengeRequired }) =>
+      makeSafeCall(
+        i18n,
+        (username: string, password: string) =>
+          lib.bank.createAccessToken(
+            username,
+            { type: "basic", password },
+            tokenRequest,
+            { challengeIds },
+          ),
+        (success, username) => {
+          onLoggedUser(
+            username,
+            createRFC8959AccessTokenEncoded(success.body.access_token),
+            AbsoluteTime.fromProtocolTimestamp(success.body.expiration),
+          );
+        },
+        (fail, username) => {
+          switch (fail.case) {
+            case HttpStatusCode.Accepted: {
+              onChallengeRequired(fail.body);
+              return i18n.str`A second factor authentication is required.`;
+            }
+            case HttpStatusCode.Unauthorized:
+              return i18n.str`Wrong credentials for "${username}"`;
+            case TalerErrorCode.GENERIC_FORBIDDEN:
+              return i18n.str`You have no permission to this account.`;
+            case TalerErrorCode.BANK_ACCOUNT_LOCKED:
+              return i18n.str`This account is locked. If you have a active session you can change the password or contact the administrator.`;
+            case HttpStatusCode.NotFound:
+              return i18n.str`Account not found`;
+          }
+        },
+      ),
+  );
+
+  if (mfa.pendingChallenge && repeatLogin) {
+    return (
+      <SolveMFAChallenges
+        currentChallenge={mfa.pendingChallenge}
+        description={i18n.str`New web session`}
+        onCancel={mfa.doCancelChallenge}
+        username={lastCallingArgs[0]}
+        onCompleted={repeatLogin}
+      />
+    );
   }
 
   switch (location.name) {
@@ -166,13 +201,9 @@ function PublicRounting({
       return (
         <WithdrawalOperationPage
           operationId={location.values.wopid}
-          routeWithdrawalDetails={publicPages.operationDetails}
           origin="from-wallet-ui"
           onOperationAborted={() => navigateTo(publicPages.login.url({}))}
           routeClose={publicPages.login}
-          onAuthorizationRequired={() =>
-            navigateTo(publicPages.solveSecondFactor.url({}))
-          }
         />
       );
     }
@@ -181,26 +212,18 @@ function PublicRounting({
         <Fragment>
           <LocalNotificationBanner notification={notification} />
           <RegistrationPage
-            onRegistrationSuccesful={doAutomaticLogin}
+            onRegistrationSuccesful={notifyOnError(doAutomaticLogin)}
             routeCancel={publicPages.login}
           />
         </Fragment>
       );
     }
-    case "solveSecondFactor": {
-      return (
-        <SolveChallengePage
-          onChallengeCompleted={() => navigateTo(publicPages.login.url({}))}
-          routeClose={publicPages.login}
-        />
-      );
-    }
     default:
       assertUnreachable(location);
   }
 }
 
-export const privatePages = {
+const privatePages = {
   homeChargeWallet: urlPattern(
     /\/account\/charge-wallet/,
     () => "#/account/charge-wallet",
@@ -212,7 +235,6 @@ export const privatePages = {
   }>(/\/account\/wire-transfer/, () => "#/account/wire-transfer"),
   home: urlPattern(/\/account/, () => "#/account"),
   notifications: urlPattern(/\/notifications/, () => "#/notifications"),
-  solveSecondFactor: urlPattern(/\/2fa/, () => "#/2fa"),
   cashoutCreate: urlPattern(/\/new-cashout/, () => "#/new-cashout"),
   cashoutDetails: urlPattern<{ cid: string }>(
     /\/cashout\/(?<cid>[a-zA-Z0-9]+)/,
@@ -236,7 +258,7 @@ export const privatePages = {
   myAccountDetails: urlPattern(/\/my-profile/, () => "#/my-profile"),
   myAccountPassword: urlPattern(/\/my-password/, () => "#/my-password"),
   myAccountCashouts: urlPattern(/\/my-cashouts/, () => "#/my-cashouts"),
-  conversionConfig: urlPattern(/\/conversion/, () => "#/conversion"),
+  conversionConfig: urlPattern(/\/conversion$/, () => "#/conversion"),
   accountDetails: urlPattern<{ account: string }>(
     /\/profile\/(?<account>[a-zA-Z0-9_-]+)\/details/,
     ({ account }) => `#/profile/${account}/details`,
@@ -261,6 +283,14 @@ export const privatePages = {
     /\/operation\/(?<wopid>[a-zA-Z0-9-]+)/,
     ({ wopid }) => `#/operation/${wopid}`,
   ),
+  conversionRateClassCreate: urlPattern(
+    /\/new-conversion-rate-class/,
+    () => "#/new-conversion-rate-class",
+  ),
+  conversionRateClassDetails: urlPattern<{ classId: string }>(
+    /\/conversion-rate-class\/(?<classId>[0-9]+)\/details/,
+    ({ classId }) => `#/conversion-rate-class/${classId}/details`,
+  ),
 };
 
 function PrivateRouting({
@@ -283,13 +313,9 @@ function PrivateRouting({
       return (
         <WithdrawalOperationPage
           operationId={location.values.wopid}
-          routeWithdrawalDetails={privatePages.operationDetails}
           origin="from-wallet-ui"
           onOperationAborted={() => navigateTo(privatePages.home.url({}))}
           routeClose={privatePages.home}
-          onAuthorizationRequired={() =>
-            navigateTo(privatePages.solveSecondFactor.url({}))
-          }
         />
       );
     }
@@ -297,21 +323,9 @@ function PrivateRouting({
       return (
         <WithdrawalOperationPage
           operationId={location.values.wopid}
-          routeWithdrawalDetails={privatePages.operationDetails}
           origin="from-bank-ui"
           onOperationAborted={() => navigateTo(privatePages.home.url({}))}
           routeClose={privatePages.home}
-          onAuthorizationRequired={() =>
-            navigateTo(privatePages.solveSecondFactor.url({}))
-          }
-        />
-      );
-    }
-    case "solveSecondFactor": {
-      return (
-        <SolveChallengePage
-          onChallengeCompleted={() => navigateTo(privatePages.home.url({}))}
-          routeClose={privatePages.home}
         />
       );
     }
@@ -334,15 +348,11 @@ function PrivateRouting({
         <ShowAccountDetails
           account={location.values.account}
           onUpdateSuccess={() => navigateTo(privatePages.home.url({}))}
-          routeHere={privatePages.accountDetails}
           routeMyAccountCashout={privatePages.myAccountCashouts}
           routeMyAccountDelete={privatePages.myAccountDelete}
           routeMyAccountDetails={privatePages.myAccountDetails}
           routeMyAccountPassword={privatePages.myAccountPassword}
           routeConversionConfig={privatePages.conversionConfig}
-          onAuthorizationRequired={() =>
-            navigateTo(privatePages.solveSecondFactor.url({}))
-          }
           routeClose={privatePages.home}
         />
       );
@@ -352,16 +362,12 @@ function PrivateRouting({
         <UpdateAccountPassword
           focus
           account={location.values.account}
-          routeHere={privatePages.accountChangePassword}
           onUpdateSuccess={() => navigateTo(privatePages.home.url({}))}
           routeMyAccountCashout={privatePages.myAccountCashouts}
           routeMyAccountDelete={privatePages.myAccountDelete}
           routeMyAccountDetails={privatePages.myAccountDetails}
           routeMyAccountPassword={privatePages.myAccountPassword}
           routeConversionConfig={privatePages.conversionConfig}
-          onAuthorizationRequired={() =>
-            navigateTo(privatePages.solveSecondFactor.url({}))
-          }
           routeClose={privatePages.home}
         />
       );
@@ -370,11 +376,7 @@ function PrivateRouting({
       return (
         <RemoveAccount
           account={location.values.account}
-          routeHere={privatePages.accountDelete}
           onUpdateSuccess={() => navigateTo(privatePages.home.url({}))}
-          onAuthorizationRequired={() =>
-            navigateTo(privatePages.solveSecondFactor.url({}))
-          }
           routeCancel={privatePages.home}
         />
       );
@@ -383,7 +385,6 @@ function PrivateRouting({
       return (
         <CashoutListForAccount
           account={location.values.account}
-          routeCreateCashout={privatePages.cashoutCreate}
           routeCashoutDetails={privatePages.cashoutDetails}
           routeClose={privatePages.home}
           routeMyAccountCashout={privatePages.myAccountCashouts}
@@ -392,9 +393,6 @@ function PrivateRouting({
           routeMyAccountPassword={privatePages.myAccountPassword}
           routeConversionConfig={privatePages.conversionConfig}
           onCashout={() => navigateTo(privatePages.home.url({}))}
-          onAuthorizationRequired={() =>
-            navigateTo(privatePages.solveSecondFactor.url({}))
-          }
         />
       );
     }
@@ -402,11 +400,7 @@ function PrivateRouting({
       return (
         <RemoveAccount
           account={username}
-          routeHere={privatePages.accountDelete}
           onUpdateSuccess={() => navigateTo(privatePages.home.url({}))}
-          onAuthorizationRequired={() =>
-            navigateTo(privatePages.solveSecondFactor.url({}))
-          }
           routeCancel={privatePages.home}
         />
       );
@@ -415,16 +409,12 @@ function PrivateRouting({
       return (
         <ShowAccountDetails
           account={username}
-          routeHere={privatePages.accountDetails}
           onUpdateSuccess={() => navigateTo(privatePages.home.url({}))}
           routeMyAccountCashout={privatePages.myAccountCashouts}
           routeConversionConfig={privatePages.conversionConfig}
           routeMyAccountDelete={privatePages.myAccountDelete}
           routeMyAccountDetails={privatePages.myAccountDetails}
           routeMyAccountPassword={privatePages.myAccountPassword}
-          onAuthorizationRequired={() =>
-            navigateTo(privatePages.solveSecondFactor.url({}))
-          }
           routeClose={privatePages.home}
         />
       );
@@ -434,16 +424,12 @@ function PrivateRouting({
         <UpdateAccountPassword
           focus
           account={username}
-          routeHere={privatePages.accountChangePassword}
           onUpdateSuccess={() => navigateTo(privatePages.home.url({}))}
           routeMyAccountCashout={privatePages.myAccountCashouts}
           routeMyAccountDelete={privatePages.myAccountDelete}
           routeMyAccountDetails={privatePages.myAccountDetails}
           routeMyAccountPassword={privatePages.myAccountPassword}
           routeConversionConfig={privatePages.conversionConfig}
-          onAuthorizationRequired={() =>
-            navigateTo(privatePages.solveSecondFactor.url({}))
-          }
           routeClose={privatePages.home}
         />
       );
@@ -453,16 +439,12 @@ function PrivateRouting({
         <CashoutListForAccount
           account={username}
           routeCashoutDetails={privatePages.cashoutDetails}
-          routeCreateCashout={privatePages.cashoutCreate}
           routeMyAccountCashout={privatePages.myAccountCashouts}
           routeMyAccountDelete={privatePages.myAccountDelete}
           routeMyAccountDetails={privatePages.myAccountDetails}
           routeMyAccountPassword={privatePages.myAccountPassword}
           routeConversionConfig={privatePages.conversionConfig}
           onCashout={() => navigateTo(privatePages.home.url({}))}
-          onAuthorizationRequired={() =>
-            navigateTo(privatePages.solveSecondFactor.url({}))
-          }
           routeClose={privatePages.home}
         />
       );
@@ -472,16 +454,19 @@ function PrivateRouting({
       if (isAdmin) {
         return (
           <AdminHome
-            onAuthorizationRequired={() =>
-              navigateTo(privatePages.solveSecondFactor.url({}))
-            }
-            routeCreate={privatePages.accountCreate}
+            routeCreateAccount={privatePages.accountCreate}
             routeRemoveAccount={privatePages.accountDelete}
             routeShowAccount={privatePages.accountDetails}
             routeShowCashoutsAccount={privatePages.accountCashouts}
             routeUpdatePasswordAccount={privatePages.accountChangePassword}
             routeCreateWireTransfer={privatePages.wireTranserCreate}
             routeDownloadStats={privatePages.statsDownload}
+            routeCreateConversionRateClass={
+              privatePages.conversionRateClassCreate
+            }
+            routeShowConversionRateClass={
+              privatePages.conversionRateClassDetails
+            }
           />
         );
       }
@@ -494,13 +479,9 @@ function PrivateRouting({
           routeOperationDetails={privatePages.startOperation}
           routeChargeWallet={privatePages.homeChargeWallet}
           routeWireTransfer={privatePages.homeWireTransfer}
-          routeSolveSecondFactor={privatePages.solveSecondFactor}
           routeCashout={privatePages.myAccountCashouts}
           routeClose={privatePages.home}
           onClose={() => navigateTo(privatePages.home.url({}))}
-          onAuthorizationRequired={() =>
-            navigateTo(privatePages.solveSecondFactor.url({}))
-          }
           onOperationCreated={(wopid) =>
             navigateTo(privatePages.startOperation.url({ wopid }))
           }
@@ -511,10 +492,6 @@ function PrivateRouting({
       return (
         <CreateCashout
           account={username}
-          routeHere={privatePages.cashoutCreate}
-          onAuthorizationRequired={() =>
-            navigateTo(privatePages.solveSecondFactor.url({}))
-          }
           onCashout={() => navigateTo(privatePages.home.url({}))}
           routeClose={privatePages.home}
         />
@@ -534,10 +511,6 @@ function PrivateRouting({
           toAccount={location.values.account}
           withAmount={location.values.amount}
           withSubject={location.values.subject}
-          routeHere={privatePages.wireTranserCreate}
-          onAuthorizationRequired={() =>
-            navigateTo(privatePages.solveSecondFactor.url({}))
-          }
           routeCancel={privatePages.home}
           onSuccess={() => navigateTo(privatePages.home.url({}))}
         />
@@ -554,12 +527,8 @@ function PrivateRouting({
           routePublicAccounts={privatePages.publicAccountList}
           routeOperationDetails={privatePages.startOperation}
           routeCashout={privatePages.myAccountCashouts}
-          routeSolveSecondFactor={privatePages.solveSecondFactor}
           routeClose={privatePages.home}
           onClose={() => navigateTo(privatePages.home.url({}))}
-          onAuthorizationRequired={() =>
-            navigateTo(privatePages.solveSecondFactor.url({}))
-          }
           onOperationCreated={(wopid) =>
             navigateTo(privatePages.startOperation.url({ wopid }))
           }
@@ -591,19 +560,44 @@ function PrivateRouting({
           routeCreateWireTransfer={privatePages.wireTranserCreate}
           routePublicAccounts={privatePages.publicAccountList}
           routeOperationDetails={privatePages.startOperation}
-          routeSolveSecondFactor={privatePages.solveSecondFactor}
           routeCashout={privatePages.myAccountCashouts}
           routeClose={privatePages.home}
           onClose={() => navigateTo(privatePages.home.url({}))}
-          onAuthorizationRequired={() =>
-            navigateTo(privatePages.solveSecondFactor.url({}))
-          }
           onOperationCreated={(wopid) =>
             navigateTo(privatePages.startOperation.url({ wopid }))
           }
         />
       );
     }
+    case "conversionRateClassCreate": {
+      return (
+        <NewConversionRateClass
+          onCreated={(id) =>
+            navigateTo(
+              privatePages.conversionRateClassDetails.url({
+                classId: String(id),
+              }),
+            )
+          }
+          routeCancel={privatePages.home}
+        />
+      );
+    }
+    case "conversionRateClassDetails": {
+      const id = Number.parseInt(location.values.classId, 10);
+      if (Number.isNaN(id)) {
+        return <div>class id is not a number "{location.values.classId}"</div>;
+      }
+      return (
+        <ConversionRateClassDetails
+          classId={id}
+          routeCancel={privatePages.home}
+          onClassDeleted={() => {
+            navigateTo(privatePages.home.url({}));
+          }}
+        />
+      );
+    }
     case "notifications": {
       return <ShowNotifications />;
     }
