@@ -26,11 +26,14 @@ import {
 import {
   BankApiProvider,
   BrowserHashNavigationProvider,
+  ErrorLoading,
   Loading,
   TalerWalletIntegrationBrowserProvider,
   TranslationProvider,
+  CommonPreferenceProvider,
+  useCommonPreference,
 } from "@gnu-taler/web-util/browser";
-import { h } from "preact";
+import { h, VNode, Fragment } from "preact";
 import { useEffect, useState } from "preact/hooks";
 import { SWRConfig } from "swr";
 import { Routing } from "./Routing.js";
@@ -47,9 +50,10 @@ import {
   revalidateBusinessAccounts,
   revalidateCashouts,
   revalidateConversionInfo,
+  revalidateConversionRateClassDetails,
+  revalidateConversionRateClasses,
 } from "./hooks/regional.js";
 const WITH_LOCAL_STORAGE_CACHE = false;
-
 export function App() {
   const [settings, setSettings] = useState<UiSettings>();
   useEffect(() => {
@@ -63,52 +67,19 @@ export function App() {
       <TranslationProvider
         source={strings}
         completeness={{
+          uk: strings["uk"].completeness,
+          // tr: strings["tr"].completeness,
+          ru: strings["ru"].completeness,
+          // sv: strings["sv"].completeness,
+          it: strings["it"].completeness,
+          fr: strings["fr"].completeness,
           es: strings["es"].completeness,
           de: strings["de"].completeness,
         }}
       >
-        <BankApiProvider
-          baseUrl={new URL("/", baseUrl)}
-          frameOnError={BankFrame}
-          evictors={{
-            bank: evictBankSwrCache,
-            conversion: evictConversionSwrCache,
-          }}
-        >
-          <SWRConfig
-            value={{
-              provider: WITH_LOCAL_STORAGE_CACHE
-                ? localStorageProvider
-                : undefined,
-              // normally, do not revalidate
-              revalidateOnFocus: false,
-              revalidateOnReconnect: false,
-              revalidateIfStale: false,
-              revalidateOnMount: undefined,
-              focusThrottleInterval: undefined,
-
-              // normally, do not refresh
-              refreshInterval: undefined,
-              dedupingInterval: 2000,
-              refreshWhenHidden: false,
-              refreshWhenOffline: false,
-
-              // ignore errors
-              shouldRetryOnError: false,
-              errorRetryCount: 0,
-              errorRetryInterval: undefined,
-
-              // do not go to loading again if already has data
-              keepPreviousData: true,
-            }}
-          >
-            <TalerWalletIntegrationBrowserProvider>
-              <BrowserHashNavigationProvider>
-                <Routing />
-              </BrowserHashNavigationProvider>
-            </TalerWalletIntegrationBrowserProvider>
-          </SWRConfig>
-        </BankApiProvider>
+        <CommonPreferenceProvider showDebug={false}>
+          <SubApp baseUrl={baseUrl} />
+        </CommonPreferenceProvider>
       </TranslationProvider>
     </SettingsProvider>
   );
@@ -210,6 +181,19 @@ const evictBankSwrCache: CacheEvictor<TalerCoreBankCacheEviction> = {
       case TalerCoreBankCacheEviction.ABORT_WITHDRAWAL:
       case TalerCoreBankCacheEviction.CREATE_WITHDRAWAL:
         return;
+      case TalerCoreBankCacheEviction.UPDATE_CONVERSION_RATE_CLASS:
+      case TalerCoreBankCacheEviction.CREATE_CONVERSION_RATE_CLASS:
+      case TalerCoreBankCacheEviction.DELETE_CONVERSION_RATE_CLASS:
+        {
+          await Promise.all([
+            revalidateConversionInfo(),
+            revalidateCashouts(),
+            revalidateTransactions(),
+            revalidateConversionRateClassDetails(),
+            revalidateConversionRateClasses(),
+          ]);
+        }
+        return;
       default:
         assertUnreachable(op);
     }
@@ -229,3 +213,48 @@ const evictConversionSwrCache: CacheEvictor<TalerBankConversionCacheEviction> =
       }
     },
   };
+
+function SubApp({ baseUrl }: { baseUrl: string }) {
+  return (
+    <BankApiProvider
+      baseUrl={new URL("/", baseUrl)}
+      frameOnError={BankFrame}
+      evictors={{
+        bank: evictBankSwrCache,
+        conversion: evictConversionSwrCache,
+      }}
+    >
+      <SWRConfig
+        value={{
+          provider: WITH_LOCAL_STORAGE_CACHE ? localStorageProvider : undefined,
+          // normally, do not revalidate
+          revalidateOnFocus: false,
+          revalidateOnReconnect: false,
+          revalidateIfStale: false,
+          revalidateOnMount: undefined,
+          focusThrottleInterval: undefined,
+
+          // normally, do not refresh
+          refreshInterval: undefined,
+          dedupingInterval: 2000,
+          refreshWhenHidden: false,
+          refreshWhenOffline: false,
+
+          // ignore errors
+          shouldRetryOnError: false,
+          errorRetryCount: 0,
+          errorRetryInterval: undefined,
+
+          // do not go to loading again if already has data
+          keepPreviousData: true,
+        }}
+      >
+        <TalerWalletIntegrationBrowserProvider>
+          <BrowserHashNavigationProvider>
+            <Routing />
+          </BrowserHashNavigationProvider>
+        </TalerWalletIntegrationBrowserProvider>
+      </SWRConfig>
+    </BankApiProvider>
+  );
+}
