@@ -25,6 +25,8 @@ import { Fragment, VNode, h } from "preact";
 import { RenderAmount } from "../../pages/PaytoWireTransferForm.js";
 import { State } from "./index.js";
 
+const TALER_SCREEN_ID = 5;
+
 export function ReadyView({
   transactions,
   routeCreateWireTransfer,
@@ -47,7 +49,7 @@ export function ReadyView({
 
         <Attention type="low" title={i18n.str`No transactions yet.`}>
           <i18n.Translate>
-            You can start sending a wire transfer or withdrawing to your wallet.
+            You can make a transfer or a withdrawal to your wallet.
           </i18n.Translate>
         </Attention>
       </div>
@@ -147,7 +149,7 @@ export function ReadyView({
                                   />
                                 </span>
                               ) : (
-                                <span style={{ color: "grey" }}>
+                                <span class="text-[grey]">
                                   &lt;{i18n.str`Invalid value`}&gt;
                                 </span>
                               )}
@@ -188,11 +190,13 @@ export function ReadyView({
                               value={item.amount}
                               negative={item.negative}
                               withColor
+                              withSign
                               spec={config.currency_specification}
                             />
                           ) : (
-                            <span style={{ color: "grey" }}>
-                              &lt;{i18n.str`Invalid value`}&gt;
+                            <span class="text-[grey]">
+                              &lt;
+                              {i18n.str`Invalid value`}&gt;
                             </span>
                           )}
                         </td>
