@@ -34,23 +34,29 @@ import {
 import { useBankCoreApiContext } from "@gnu-taler/web-util/browser";
 import { useState } from "preact/hooks";
 import _useSWR, { SWRHook, mutate } from "swr";
-import { buildPaginatedResult } from "./account.js";
 import { PAGINATED_LIST_REQUEST } from "../utils.js";
+import { buildPaginatedResult } from "./account.js";
+import { TalerBankConversionHttpClient } from "@gnu-taler/taler-util";
+import { assertUnreachable } from "@gnu-taler/taler-util";
+import { TalerBankConversionErrorsByMethod } from "@gnu-taler/taler-util";
 
 // FIX default import https://github.com/microsoft/TypeScript/issues/49189
 const useSWR = _useSWR as unknown as SWRHook;
 
-export type TransferCalculation =
-  | {
-      debit: AmountJson;
-      credit: AmountJson;
-      beforeFee: AmountJson;
-    }
-  | "amount-is-too-small";
+export type TransCalc = {
+  debit: AmountJson;
+  credit: AmountJson;
+  beforeFee: AmountJson;
+};
+export type TransferCalculation = TransCalc | "amount-is-too-small";
 type EstimatorFunction = (
   amount: AmountJson,
   fee: AmountJson,
-) => Promise<TransferCalculation>;
+) => Promise<
+  | OperationOk<TransCalc>
+  | TalerBankConversionErrorsByMethod<"getCashinRate">
+  | TalerBankConversionErrorsByMethod<"getCashoutRate">
+>;
 
 type ConversionEstimators = {
   estimateByCredit: EstimatorFunction;
@@ -92,132 +98,187 @@ export function useConversionInfo() {
   return undefined;
 }
 
+export function useConversionRateForUser(
+  username: string,
+  token: AccessToken | undefined,
+) {
+  const {
+    lib: { conversionForUser },
+    config,
+  } = useBankCoreApiContext();
+
+  async function fetcher() {
+    return await conversionForUser(username).getRate(token);
+  }
+  const { data, error } = useSWR<
+    TalerBankConversionResultByMethod<"getRate">,
+    TalerHttpError
+  >(
+    !config.allow_conversion ? undefined : ["useConversionInfoForUser"],
+    fetcher,
+    {
+      refreshInterval: 0,
+      refreshWhenHidden: false,
+      revalidateOnFocus: false,
+      revalidateOnReconnect: false,
+      refreshWhenOffline: false,
+      errorRetryCount: 0,
+      errorRetryInterval: 1,
+      shouldRetryOnError: false,
+      keepPreviousData: true,
+    },
+  );
+
+  if (data) return data;
+  if (error) return error;
+  return undefined;
+}
+
+function buildEstimatorWithTheBackend(
+  conversion: TalerBankConversionHttpClient,
+  token: AccessToken | undefined,
+  estimation:
+    | "cashin-rate-from-credit"
+    | "cashin-rate-from-debit"
+    | "cashout-rate-from-credit"
+    | "cashout-rate-from-debit",
+): EstimatorFunction {
+  return async (amount, fee) => {
+    let resp;
+    switch (estimation) {
+      case "cashin-rate-from-credit": {
+        resp = await conversion.getCashinRate(token, {
+          credit: amount,
+        });
+        break;
+      }
+      case "cashin-rate-from-debit": {
+        resp = await conversion.getCashinRate(token, {
+          debit: amount,
+        });
+        break;
+      }
+      case "cashout-rate-from-credit": {
+        resp = await conversion.getCashoutRate(token, {
+          credit: amount,
+        });
+        break;
+      }
+      case "cashout-rate-from-debit": {
+        resp = await conversion.getCashoutRate(token, {
+          debit: amount,
+        });
+        break;
+      }
+      default: {
+        assertUnreachable(estimation);
+      }
+    }
+    if (resp.type === "fail") {
+      return resp;
+    }
+    const credit = Amounts.parseOrThrow(resp.body.amount_credit);
+    const debit = Amounts.parseOrThrow(resp.body.amount_debit);
+    const beforeFee = Amounts.add(credit, fee).amount;
+
+    return opFixedSuccess({
+      debit,
+      beforeFee,
+      credit,
+    });
+  };
+}
+
+function buildConversionEstimatorsWithTheBackend(
+  conversion: TalerBankConversionHttpClient,
+  direction: "cashin" | "cashout",
+): ConversionEstimators {
+  const { state } = useSessionState();
+  const token = state.status === "loggedIn" ? state.token : undefined;
+  return {
+    estimateByCredit: buildEstimatorWithTheBackend(
+      conversion,
+      token,
+      direction == "cashin"
+        ? "cashin-rate-from-credit"
+        : "cashout-rate-from-credit",
+    ),
+    estimateByDebit: buildEstimatorWithTheBackend(
+      conversion,
+      token,
+      direction == "cashin"
+        ? "cashin-rate-from-debit"
+        : "cashout-rate-from-debit",
+    ),
+  };
+}
+
 export function useCashinEstimator(): ConversionEstimators {
   const {
     lib: { conversion },
   } = useBankCoreApiContext();
-  return {
-    estimateByCredit: async (fiatAmount, fee) => {
-      const resp = await conversion.getCashinRate({
-        credit: fiatAmount,
-      });
-      if (resp.type === "fail") {
-        switch (resp.case) {
-          case HttpStatusCode.Conflict: {
-            return "amount-is-too-small";
-          }
-          // this below can't happen
-          case HttpStatusCode.NotImplemented: //it should not be able to call this function
-          case HttpStatusCode.BadRequest: //we are using just one parameter
-            throw TalerError.fromDetail(resp.detail.code, {}, resp.detail.hint);
-        }
-      }
-      const credit = Amounts.parseOrThrow(resp.body.amount_credit);
-      const debit = Amounts.parseOrThrow(resp.body.amount_debit);
-      const beforeFee = Amounts.sub(credit, fee).amount;
 
-      return {
-        debit,
-        beforeFee,
-        credit,
-      };
-    },
-    estimateByDebit: async (regionalAmount, fee) => {
-      const resp = await conversion.getCashinRate({
-        debit: regionalAmount,
-      });
-      if (resp.type === "fail") {
-        switch (resp.case) {
-          case HttpStatusCode.Conflict: {
-            return "amount-is-too-small";
-          }
-          // this below can't happen
-          case HttpStatusCode.NotImplemented: //it should not be able to call this function
-          case HttpStatusCode.BadRequest: //we are using just one parameter
-            throw TalerError.fromDetail(resp.detail.code, {}, resp.detail.hint);
-        }
-      }
-      const credit = Amounts.parseOrThrow(resp.body.amount_credit);
-      const debit = Amounts.parseOrThrow(resp.body.amount_debit);
-      const beforeFee = Amounts.add(credit, fee).amount;
-
-      return {
-        debit,
-        beforeFee,
-        credit,
-      };
-    },
-  };
+  return buildConversionEstimatorsWithTheBackend(conversion, "cashin");
 }
 
 export function useCashoutEstimator(): ConversionEstimators {
   const {
     lib: { conversion },
   } = useBankCoreApiContext();
-  return {
-    estimateByCredit: async (fiatAmount, fee) => {
-      const resp = await conversion.getCashoutRate({
-        credit: fiatAmount,
-      });
-      if (resp.type === "fail") {
-        switch (resp.case) {
-          case HttpStatusCode.Conflict: {
-            return "amount-is-too-small";
-          }
-          // this below can't happen
-          case HttpStatusCode.NotImplemented: //it should not be able to call this function
-          case HttpStatusCode.BadRequest: //we are using just one parameter
-            throw TalerError.fromDetail(resp.detail.code, {}, resp.detail.hint);
-        }
-      }
-      const credit = Amounts.parseOrThrow(resp.body.amount_credit);
-      const debit = Amounts.parseOrThrow(resp.body.amount_debit);
-      const beforeFee = Amounts.sub(credit, fee).amount;
-
-      return {
-        debit,
-        beforeFee,
-        credit,
-      };
-    },
-    estimateByDebit: async (regionalAmount, fee) => {
-      const resp = await conversion.getCashoutRate({
-        debit: regionalAmount,
-      });
-      if (resp.type === "fail") {
-        switch (resp.case) {
-          case HttpStatusCode.Conflict: {
-            return "amount-is-too-small";
-          }
-          // this below can't happen
-          case HttpStatusCode.NotImplemented: //it should not be able to call this function
-          case HttpStatusCode.BadRequest: //we are using just one parameter
-            throw TalerError.fromDetail(resp.detail.code, {}, resp.detail.hint);
-        }
-      }
-      const credit = Amounts.parseOrThrow(resp.body.amount_credit);
-      const debit = Amounts.parseOrThrow(resp.body.amount_debit);
-      const beforeFee = Amounts.add(credit, fee).amount;
-
-      return {
-        debit,
-        beforeFee,
-        credit,
-      };
-    },
-  };
+  return buildConversionEstimatorsWithTheBackend(conversion, "cashout");
 }
 
-/**
- * @deprecated use useCashoutEstimator
- */
-export function useEstimator(): ConversionEstimators {
-  return useCashoutEstimator();
+export function useCashinEstimatorForClass(
+  classId: number,
+): ConversionEstimators {
+  const {
+    lib: { conversionForClass },
+  } = useBankCoreApiContext();
+  return buildConversionEstimatorsWithTheBackend(
+    conversionForClass(classId),
+    "cashin",
+  );
+}
+
+export function useCashoutEstimatorForClass(
+  classId: number,
+): ConversionEstimators {
+  const {
+    lib: { conversionForClass },
+  } = useBankCoreApiContext();
+  return buildConversionEstimatorsWithTheBackend(
+    conversionForClass(classId),
+    "cashout",
+  );
+}
+
+export function useCashinEstimatorByUser(
+  username: string,
+): ConversionEstimators {
+  const {
+    lib: { conversionForUser },
+  } = useBankCoreApiContext();
+  return buildConversionEstimatorsWithTheBackend(
+    conversionForUser(username),
+    "cashin",
+  );
+}
+
+export function useCashoutEstimatorByUser(
+  username: string,
+): ConversionEstimators {
+  const {
+    lib: { conversionForUser },
+  } = useBankCoreApiContext();
+  return buildConversionEstimatorsWithTheBackend(
+    conversionForUser(username),
+    "cashout",
+  );
 }
 
 export async function revalidateBusinessAccounts() {
   return mutate(
-    (key) => Array.isArray(key) && key[key.length - 1] === "getAccounts",
+    (key) => Array.isArray(key) && key[key.length - 1] === "listAccounts",
     undefined,
     { revalidate: true },
   );
@@ -233,22 +294,17 @@ export function useBusinessAccounts() {
   const [offset, setOffset] = useState<number | undefined>();
 
   function fetcher([token, aid]: [AccessToken, number]) {
-    // FIXME: add account name filter
-    return api.getAccounts(
-      token,
-      {},
-      {
-        limit: PAGINATED_LIST_REQUEST,
-        offset: aid ? String(aid) : undefined,
-        order: "asc",
-      },
-    );
+    return api.listAccounts(token, {
+      limit: PAGINATED_LIST_REQUEST,
+      offset: aid ? String(aid) : undefined,
+      order: "asc",
+    });
   }
 
   const { data, error } = useSWR<
-    TalerCoreBankResultByMethod<"getAccounts">,
+    TalerCoreBankResultByMethod<"listAccounts">,
     TalerHttpError
-  >([token, offset ?? 0, "getAccounts"], fetcher, {
+  >([token, offset ?? 0, "listAccounts"], fetcher, {
     refreshInterval: 0,
     refreshWhenHidden: false,
     revalidateOnFocus: false,
@@ -372,7 +428,7 @@ export function useCashouts(account: string) {
       }),
     );
     const cashouts = all.filter(notUndefined);
-    return { type: "ok" as const, body: { cashouts } };
+    return opFixedSuccess({ cashouts });
   }
   const { data, error } = useSWR<
     | OperationOk<{ cashouts: CashoutWithId[] }>
@@ -468,6 +524,7 @@ export function useLastMonitorInfo(
     lib: { bank: api },
   } = useBankCoreApiContext();
   const { state: credentials } = useSessionState();
+
   const token =
     credentials.status !== "loggedIn" ? undefined : credentials.token;
 
@@ -505,3 +562,157 @@ export function useLastMonitorInfo(
   if (error) return error;
   return undefined;
 }
+
+export function revalidateConversionRateClasses() {
+  return mutate(
+    (key) =>
+      Array.isArray(key) && key[key.length - 1] === "useConversionRateClasses",
+    undefined,
+    { revalidate: true },
+  );
+}
+export function useConversionRateClasses() {
+  const { state: credentials } = useSessionState();
+  const token =
+    credentials.status !== "loggedIn" ? undefined : credentials.token;
+  const {
+    lib: { bank: api },
+  } = useBankCoreApiContext();
+
+  const [offset, setOffset] = useState<number | undefined>();
+
+  function fetcher([token, aid]: [AccessToken, number]) {
+    return api.listConversionRateClasses(token, {
+      limit: PAGINATED_LIST_REQUEST,
+      offset: aid ? String(aid) : undefined,
+      order: "asc",
+    });
+  }
+
+  const { data, error } = useSWR<
+    TalerCoreBankResultByMethod<"listConversionRateClasses">,
+    TalerHttpError
+  >([token, offset ?? 0, "useConversionRateClasses"], fetcher, {
+    refreshInterval: 0,
+    refreshWhenHidden: false,
+    revalidateOnFocus: false,
+    revalidateOnReconnect: false,
+    refreshWhenOffline: false,
+    errorRetryCount: 0,
+    errorRetryInterval: 1,
+    shouldRetryOnError: false,
+    keepPreviousData: true,
+  });
+
+  if (error) return error;
+  if (data === undefined) return undefined;
+  if (data.type !== "ok") return data;
+
+  return buildPaginatedResult(
+    data.body.classes,
+    offset,
+    setOffset,
+    (d) => d.conversion_rate_class_id,
+  );
+}
+
+export function revalidateConversionRateClassDetails() {
+  return mutate(
+    (key) =>
+      Array.isArray(key) &&
+      key[key.length - 1] === "useConversionRateClassDetails",
+    undefined,
+    { revalidate: true },
+  );
+}
+
+export function useConversionRateClassDetails(classId: number) {
+  const { state: credentials } = useSessionState();
+  const token =
+    credentials.status !== "loggedIn" ? undefined : credentials.token;
+  const {
+    lib: { bank: api },
+  } = useBankCoreApiContext();
+
+  async function fetcher([username, token]: [number, AccessToken]) {
+    return await api.getConversionRateClass(token, username);
+  }
+
+  const { data, error } = useSWR<
+    TalerCoreBankResultByMethod<"getConversionRateClass">,
+    TalerHttpError
+  >([classId, token, "useConversionRateClassDetails"], fetcher, {});
+
+  if (data) return data;
+  if (error) return error;
+  return undefined;
+}
+
+export function revalidateConversionRateClassUsers() {
+  return mutate(
+    (key) =>
+      Array.isArray(key) &&
+      key[key.length - 1] === "useConversionRateClassUsers",
+    undefined,
+    { revalidate: true },
+  );
+}
+
+export function useConversionRateClassUsers(
+  classId: number | undefined,
+  username?: string,
+) {
+  const { state: credentials } = useSessionState();
+  const token =
+    credentials.status !== "loggedIn" ? undefined : credentials.token;
+  const {
+    lib: { bank: api },
+  } = useBankCoreApiContext();
+
+  const [offset, setOffset] = useState<number | undefined>();
+
+  function fetcher([token, aid, username, classId]: [
+    AccessToken,
+    number,
+    string,
+    number,
+  ]) {
+    return api.listAccounts(token, {
+      limit: PAGINATED_LIST_REQUEST,
+      offset: aid ? String(aid) : undefined,
+      order: "asc",
+      account: username,
+      conversionRateId: classId,
+    });
+  }
+
+  const { data, error } = useSWR<
+    TalerCoreBankResultByMethod<"listAccounts">,
+    TalerHttpError
+  >(
+    [token, offset ?? 0, username, classId, "useConversionRateClassUsers"],
+    fetcher,
+    {
+      refreshInterval: 0,
+      refreshWhenHidden: false,
+      revalidateOnFocus: false,
+      revalidateOnReconnect: false,
+      refreshWhenOffline: false,
+      errorRetryCount: 0,
+      errorRetryInterval: 1,
+      shouldRetryOnError: false,
+      keepPreviousData: true,
+    },
+  );
+
+  if (error) return error;
+  if (data === undefined) return undefined;
+  if (data.type !== "ok") return data;
+
+  return buildPaginatedResult(
+    data.body.accounts,
+    offset,
+    setOffset,
+    (d) => d.row_id!,
+  );
+}
