@@ -19,11 +19,17 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
+import { RouteDefinition } from "@gnu-taler/web-util/browser";
 import * as tests from "@gnu-taler/web-util/testing";
+import { ComponentChildren } from "preact";
 import { BankFrame } from "./BankFrame.js";
 
 export default {
   title: "bank frame",
 };
 
-export const Ready = tests.createExample(BankFrame, {});
+export const Ready: tests.ExampleItemSetup<{
+  account?: string;
+  routeAccountDetails?: RouteDefinition;
+  children: ComponentChildren;
+}> = tests.createExample(BankFrame, {});
