@@ -14,37 +14,64 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
-import { HttpStatusCode, createRFC8959AccessTokenEncoded, createRFC8959AccessTokenPlain } from "@gnu-taler/taler-util";
+import {
+  AbsoluteTime,
+  Duration,
+  HttpStatusCode,
+  createRFC8959AccessTokenEncoded,
+} from "@gnu-taler/taler-util";
 import {
   Button,
+  ButtonBetter,
   LocalNotificationBanner,
+  makeSafeCall,
+  RouteDefinition,
   ShowInputErrorLabel,
+  useBankCoreApiContext,
+  useChallengeHandler,
+  useLocalNotificationBetter,
   useLocalNotificationHandler,
   useTranslationContext,
 } from "@gnu-taler/web-util/browser";
 import { VNode, h } from "preact";
-import { useEffect, useRef, useState } from "preact/hooks";
-import { useBankCoreApiContext } from "@gnu-taler/web-util/browser";
+import { useState } from "preact/hooks";
 import { useSessionState } from "../hooks/session.js";
-import { RouteDefinition } from "@gnu-taler/web-util/browser";
 import { undefinedIfEmpty } from "../utils.js";
 import { doAutoFocus } from "./PaytoWireTransferForm.js";
 import { USERNAME_REGEX } from "./RegistrationPage.js";
+import { TalerErrorCode } from "@gnu-taler/taler-util";
+import { useBankState } from "../hooks/bank-state.js";
+import { TokenRequest } from "@gnu-taler/taler-util";
+import { SolveMFAChallenges } from "./SolveMFA.js";
+
+const TALER_SCREEN_ID = 104;
+
+export const SESSION_DURATION = Duration.toTalerProtocolDuration(
+  Duration.fromSpec({
+    // seconds: 6,
+    minutes: 30,
+  }),
+);
 
 /**
  * Collect and submit login data.
  */
+LoginForm.SCREEN_ID = TALER_SCREEN_ID;
 export function LoginForm({
   currentUser,
   fixedUser,
   routeRegister,
+  //
 }: {
   fixedUser?: boolean;
   currentUser?: string;
   routeRegister?: RouteDefinition;
+  //
 }): VNode {
   const session = useSessionState();
 
+  const sessionState =
+    session.state.status === "loggedIn" ? session.state : undefined;
   const sessionUser =
     session.state.status !== "loggedOut" ? session.state.username : undefined;
   const [username, setUsername] = useState<string | undefined>(
@@ -53,16 +80,12 @@ export function LoginForm({
   const [password, setPassword] = useState<string | undefined>();
   const { i18n } = useTranslationContext();
   const {
-    lib: { auth: authenticator },
+    lib: { bank: api },
   } = useBankCoreApiContext();
-  const [notification, withErrorHandler] = useLocalNotificationHandler();
+  const [notification, notifyOnError] = useLocalNotificationBetter();
+  const mfa = useChallengeHandler();
   const { config } = useBankCoreApiContext();
 
-  const ref = useRef<HTMLInputElement>(null);
-  useEffect(function focusInput() {
-    ref.current?.focus();
-  }, []);
-
   const errors = undefinedIfEmpty({
     username: !username
       ? i18n.str`Missing username`
@@ -73,31 +96,72 @@ export function LoginForm({
   });
 
   async function doLogout() {
+    if (sessionState) {
+      await api.deleteAccessToken(sessionState.username, sessionState.token);
+    }
     session.logOut();
   }
 
-  const loginHandler =
-    !username || !password
-      ? undefined
-      : withErrorHandler(
-          async () =>
-            authenticator(username).createAccessTokenBasic(username, password, {
-              scope: "readwrite",
-              duration: { d_us: "forever" },
-              refreshable: true,
-            }),
-          (result) => {
-            session.logIn({ username, token: createRFC8959AccessTokenEncoded(result.body.access_token) });
-          },
-          (fail) => {
-            switch (fail.case) {
-              case HttpStatusCode.Unauthorized:
-                return i18n.str`Wrong credentials for "${username}"`;
-              case HttpStatusCode.NotFound:
-                return i18n.str`Account not found`;
+  const tokenRequest = {
+    scope: "readwrite",
+    duration: SESSION_DURATION,
+    refreshable: true,
+  } as TokenRequest;
+
+  const [doLogin, repeatLogin] = mfa.withMfaHandler(
+    ({ challengeIds, onChallengeRequired }) =>
+      makeSafeCall(
+        i18n,
+        (username: string, password: string) =>
+          api.createAccessToken(
+            username,
+            { type: "basic", password },
+            tokenRequest,
+            { challengeIds },
+          ),
+        (result, username) => {
+          session.logIn({
+            username,
+            token: createRFC8959AccessTokenEncoded(result.body.access_token),
+            expiration: AbsoluteTime.fromProtocolTimestamp(
+              result.body.expiration,
+            ),
+          });
+        },
+        (fail, username) => {
+          switch (fail.case) {
+            case HttpStatusCode.Accepted: {
+              onChallengeRequired(fail.body);
+              return i18n.str`A second factor authentication is required.`;
             }
-          },
-        );
+            case TalerErrorCode.GENERIC_FORBIDDEN:
+              return i18n.str`You have no permission to this account.`;
+            case TalerErrorCode.BANK_ACCOUNT_LOCKED:
+              return i18n.str`You have no permission to this account.`;
+            case HttpStatusCode.Unauthorized:
+              return i18n.str`Wrong credentials for "${username}"`;
+            case HttpStatusCode.NotFound:
+              return i18n.str`Account not found`;
+          }
+        },
+      ),
+  );
+  const loginHandler =
+    !username || !password || !!errors
+      ? undefined
+      : () => notifyOnError(doLogin)(username, password);
+
+  if (mfa.pendingChallenge && repeatLogin && username) {
+    return (
+      <SolveMFAChallenges
+        currentChallenge={mfa.pendingChallenge}
+        description={i18n.str`Account login.`}
+        onCancel={mfa.doCancelChallenge}
+        username={username}
+        onCompleted={repeatLogin}
+      />
+    );
+  }
 
   return (
     <div class="flex min-h-full flex-col justify-center ">
@@ -190,27 +254,25 @@ export function LoginForm({
                 <i18n.Translate>Cancel</i18n.Translate>
               </button>
 
-              <Button
+              <ButtonBetter
                 type="submit"
                 name="check"
                 class="rounded-md bg-indigo-600 disabled:bg-gray-300 px-3 py-1.5 text-sm font-semibold leading-6 text-white shadow-sm hover:bg-indigo-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-indigo-600"
-                disabled={!!errors}
-                handler={loginHandler}
+                onClick={loginHandler}
               >
                 <i18n.Translate>Check</i18n.Translate>
-              </Button>
+              </ButtonBetter>
             </div>
           ) : (
             <div>
-              <Button
+              <ButtonBetter
                 type="submit"
                 name="login"
                 class="flex w-full justify-center rounded-md bg-indigo-600 disabled:bg-gray-300 px-3 py-1.5 text-sm font-semibold leading-6 text-white shadow-sm hover:bg-indigo-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-indigo-600"
-                disabled={!!errors}
-                handler={loginHandler}
+                onClick={loginHandler}
               >
                 <i18n.Translate>Log in</i18n.Translate>
-              </Button>
+              </ButtonBetter>
             </div>
           )}
         </form>
