@@ -15,31 +15,36 @@
  */
 
 import {
-  AbsoluteTime,
   Amounts,
   HttpStatusCode,
   TalerErrorCode,
-  TranslatedString,
+  TalerUris,
   assertUnreachable,
-  stringifyWithdrawUri,
 } from "@gnu-taler/taler-util";
 import {
   Attention,
+  ButtonBetter,
   LocalNotificationBanner,
+  makeSafeCall,
   notifyInfo,
   useBankCoreApiContext,
-  useLocalNotification,
+  useChallengeHandler,
+  useLocalNotificationBetter,
   useTalerWalletIntegrationAPI,
   useTranslationContext,
 } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
 import { useEffect } from "preact/hooks";
 import { QR } from "../../components/QR.js";
-import { useBankState } from "../../hooks/bank-state.js";
 import { usePreferences } from "../../hooks/preferences.js";
+import { useSessionState } from "../../hooks/session.js";
+import { RenderAmount } from "../PaytoWireTransferForm.js";
+import { SolveMFAChallenges } from "../SolveMFA.js";
 import { ShouldBeSameUser } from "../WithdrawalConfirmationQuestion.js";
 import { State } from "./index.js";
-import { RenderAmount, doAutoFocus } from "../PaytoWireTransferForm.js";
+import { PaytoType } from "@gnu-taler/taler-util";
+
+const TALER_SCREEN_ID = 5;
 
 export function InvalidPaytoView({ payto }: State.InvalidPayto) {
   return <div>Payto from server is not valid &quot;{payto}&quot;</div>;
@@ -48,129 +53,106 @@ export function InvalidWithdrawalView({ uri }: State.InvalidWithdrawal) {
   return <div>Withdrawal uri from server is not valid &quot;{uri}&quot;</div>;
 }
 export function InvalidReserveView({ reserve }: State.InvalidReserve) {
-  return <div>Reserve from server is not valid &quot;{reserve}&quot;</div>;
+  return (
+    <div>
+      Reserve from server is not valid &quot;
+      {reserve}&quot;
+    </div>
+  );
 }
 
 export function NeedConfirmationView({
-  onAbort: doAbort,
-  onConfirm: doConfirm,
-  routeHere,
+  onAbort,
   account,
   details,
-  id,
-  onAuthorizationRequired,
+  operationId,
 }: State.NeedConfirmation) {
   const { i18n } = useTranslationContext();
   const [settings] = usePreferences();
-  const [notification, notify, errorHandler] = useLocalNotification();
-  const [, updateBankState] = useBankState();
-  const { config } = useBankCoreApiContext();
+  const [notification, notifyOnError] = useLocalNotificationBetter();
+  const { state: credentials } = useSessionState();
+  const creds = credentials.status !== "loggedIn" ? undefined : credentials;
+  const mfa = useChallengeHandler();
+  const {
+    config,
+    lib: { bank },
+  } = useBankCoreApiContext();
   const wireFee =
     config.wire_transfer_fees === undefined
       ? Amounts.zeroOfCurrency(config.currency)
       : Amounts.parseOrThrow(config.wire_transfer_fees);
 
-  async function onCancel() {
-    errorHandler(async () => {
-      if (!doAbort) return;
-      const resp = await doAbort();
-      if (!resp) return;
-      switch (resp.case) {
-        case HttpStatusCode.Conflict:
-          return notify({
-            type: "error",
-            title: i18n.str`The reserve operation has been confirmed previously and can't be aborted`,
-            description: resp.detail.hint as TranslatedString,
-            debug: resp.detail,
-            when: AbsoluteTime.now(),
-          });
-        case HttpStatusCode.BadRequest:
-          return notify({
-            type: "error",
-            title: i18n.str`The operation id is invalid.`,
-            description: resp.detail.hint as TranslatedString,
-            debug: resp.detail,
-            when: AbsoluteTime.now(),
-          });
-        case HttpStatusCode.NotFound:
-          return notify({
-            type: "error",
-            title: i18n.str`The operation was not found.`,
-            description: resp.detail.hint as TranslatedString,
-            debug: resp.detail,
-            when: AbsoluteTime.now(),
-          });
-        default:
-          assertUnreachable(resp);
-      }
-    });
-  }
+  const doAbort = !creds
+    ? undefined
+    : notifyOnError(
+        makeSafeCall(
+          i18n,
+          () => bank.abortWithdrawalById(creds, operationId),
+          (suc) => {
+            onAbort();
+          },
+          (fail) => {
+            switch (fail.case) {
+              case HttpStatusCode.Conflict:
+                return i18n.str`The reserve operation has been confirmed previously and can't be aborted`;
+              case HttpStatusCode.BadRequest:
+                return i18n.str`The operation ID is invalid.`;
+              case HttpStatusCode.NotFound:
+                return i18n.str`The operation was not found.`;
+            }
+          },
+        ),
+      );
 
-  async function onConfirm() {
-    errorHandler(async () => {
-      if (!doConfirm) return;
-      const resp = await doConfirm();
-      if (!resp) {
-        if (!settings.showWithdrawalSuccess) {
-          notifyInfo(i18n.str`Wire transfer completed!`);
-        }
-        return;
-      }
-      switch (resp.case) {
-        case TalerErrorCode.BANK_CONFIRM_ABORT_CONFLICT:
-          return notify({
-            type: "error",
-            title: i18n.str`The withdrawal has been aborted previously and can't be confirmed`,
-            description: resp.detail.hint as TranslatedString,
-            debug: resp.detail,
-            when: AbsoluteTime.now(),
-          });
-        case TalerErrorCode.BANK_CONFIRM_INCOMPLETE:
-          return notify({
-            type: "error",
-            title: i18n.str`The withdrawal operation can't be confirmed before a wallet accepted the transaction.`,
-            description: resp.detail.hint as TranslatedString,
-            debug: resp.detail,
-            when: AbsoluteTime.now(),
-          });
-        case HttpStatusCode.BadRequest:
-          return notify({
-            type: "error",
-            title: i18n.str`The operation id is invalid.`,
-            description: resp.detail.hint as TranslatedString,
-            debug: resp.detail,
-            when: AbsoluteTime.now(),
-          });
-        case HttpStatusCode.NotFound:
-          return notify({
-            type: "error",
-            title: i18n.str`The operation was not found.`,
-            description: resp.detail.hint as TranslatedString,
-            debug: resp.detail,
-            when: AbsoluteTime.now(),
-          });
-        case TalerErrorCode.BANK_UNALLOWED_DEBIT:
-          return notify({
-            type: "error",
-            title: i18n.str`Your balance is not enough.`,
-            description: resp.detail.hint as TranslatedString,
-            debug: resp.detail,
-            when: AbsoluteTime.now(),
-          });
-        case HttpStatusCode.Accepted: {
-          updateBankState("currentChallenge", {
-            operation: "confirm-withdrawal",
-            id: String(resp.body.challenge_id),
-            sent: AbsoluteTime.never(),
-            location: routeHere.url({ wopid: id }),
-            request: id,
-          });
-          return onAuthorizationRequired();
-        }
-        default:
-          assertUnreachable(resp);
-      }
-    });
+  const [doConfirm, repeatConfirm] = !creds
+    ? [undefined, undefined]
+    : mfa.withMfaHandler(({ challengeIds, onChallengeRequired }) =>
+        makeSafeCall(
+          i18n,
+          () =>
+            bank.confirmWithdrawalById(creds, {}, operationId, {
+              challengeIds,
+            }),
+          (suc) => {
+            if (!settings.showWithdrawalSuccess) {
+              notifyInfo(i18n.str`Wire transfer completed!`);
+            }
+          },
+          (fail) => {
+            switch (fail.case) {
+              case TalerErrorCode.BANK_CONFIRM_ABORT_CONFLICT:
+                return i18n.str`The withdrawal has been aborted previously and can't be confirmed`;
+              case TalerErrorCode.BANK_CONFIRM_INCOMPLETE:
+                return i18n.str`The withdrawal operation can't be confirmed before a wallet accepted the transaction.`;
+              case HttpStatusCode.BadRequest:
+                return i18n.str`The operation ID is invalid.`;
+              case HttpStatusCode.NotFound:
+                return i18n.str`The operation was not found.`;
+              case TalerErrorCode.BANK_UNALLOWED_DEBIT:
+                return i18n.str`Your balance is not sufficient for the operation.`;
+              case HttpStatusCode.Accepted: {
+                onChallengeRequired(fail.body);
+                return i18n.str`A second factor authentication is required.`;
+              }
+              case TalerErrorCode.BANK_AMOUNT_DIFFERS:
+                return i18n.str`The starting withdrawal amount and the confirmation amount differs.`;
+              case TalerErrorCode.BANK_AMOUNT_REQUIRED:
+                return i18n.str`The bank requires a bank account which has not been specified yet.`;
+            }
+          },
+        ),
+      );
+
+  if (mfa.pendingChallenge && repeatConfirm) {
+    return (
+      <SolveMFAChallenges
+        currentChallenge={mfa.pendingChallenge}
+        description={i18n.str`Confirm withdrawal.`}
+        username={details.username}
+        onCancel={mfa.doCancelChallenge}
+        onCompleted={repeatConfirm}
+      />
+    );
   }
 
   return (
@@ -194,29 +176,21 @@ export function NeedConfirmationView({
                 <div class="w-full">
                   <dl class="">
                     {((): VNode => {
-                      if (!details.account.isKnown) {
-                        return (
-                          <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
-                            <dt class="text-sm font-medium leading-6 text-gray-900">
-                              <i18n.Translate>
-                                Payment provider's account
-                              </i18n.Translate>
-                            </dt>
-                            <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
-                              {details.account.targetPath}
-                            </dd>
-                          </div>
-                        );
-                      }
                       switch (details.account.targetType) {
-                        case "iban": {
+                        case undefined:
+                        case PaytoType.TalerReserveHttp:
+                        case PaytoType.TalerReserve: {
+                          // FIXME: support wire transfer to wallet
+                          return <div>not yet supported</div>;
+                        }
+                        case PaytoType.IBAN: {
                           const name = details.account.params["receiver-name"];
                           return (
                             <Fragment>
                               <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
                                 <dt class="text-sm font-medium leading-6 text-gray-900">
                                   <i18n.Translate>
-                                    Payment provider's account number
+                                    Payment Service Provider's account number
                                   </i18n.Translate>
                                 </dt>
                                 <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
@@ -227,7 +201,7 @@ export function NeedConfirmationView({
                                 <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
                                   <dt class="text-sm font-medium leading-6 text-gray-900">
                                     <i18n.Translate>
-                                      Payment provider's name
+                                      Payment Service Provider's name
                                     </i18n.Translate>
                                   </dt>
                                   <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
@@ -238,14 +212,15 @@ export function NeedConfirmationView({
                             </Fragment>
                           );
                         }
-                        case "x-taler-bank": {
+                        case PaytoType.TalerBank: {
                           const name = details.account.params["receiver-name"];
                           return (
                             <Fragment>
                               <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
                                 <dt class="text-sm font-medium leading-6 text-gray-900">
                                   <i18n.Translate>
-                                    Payment provider's account bank hostname
+                                    Payment Service Provider's account bank
+                                    hostname
                                   </i18n.Translate>
                                 </dt>
                                 <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
@@ -255,7 +230,7 @@ export function NeedConfirmationView({
                               <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
                                 <dt class="text-sm font-medium leading-6 text-gray-900">
                                   <i18n.Translate>
-                                    Payment provider's account id
+                                    Payment Service Provider's account id
                                   </i18n.Translate>
                                 </dt>
                                 <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
@@ -266,7 +241,7 @@ export function NeedConfirmationView({
                                 <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
                                   <dt class="text-sm font-medium leading-6 text-gray-900">
                                     <i18n.Translate>
-                                      Payment provider's name
+                                      Payment Service Provider's name
                                     </i18n.Translate>
                                   </dt>
                                   <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
@@ -277,14 +252,14 @@ export function NeedConfirmationView({
                             </Fragment>
                           );
                         }
-                        case "bitcoin": {
+                        case PaytoType.Bitcoin: {
                           const name = details.account.params["receiver-name"];
                           return (
                             <Fragment>
                               <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
                                 <dt class="text-sm font-medium leading-6 text-gray-900">
                                   <i18n.Translate>
-                                    Payment provider's account address
+                                    Payment Service Provider's account address
                                   </i18n.Translate>
                                 </dt>
                                 <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
@@ -295,7 +270,36 @@ export function NeedConfirmationView({
                                 <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
                                   <dt class="text-sm font-medium leading-6 text-gray-900">
                                     <i18n.Translate>
-                                      Payment provider's name
+                                      Payment Service Provider's name
+                                    </i18n.Translate>
+                                  </dt>
+                                  <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
+                                    {name}
+                                  </dd>
+                                </div>
+                              )}
+                            </Fragment>
+                          );
+                        }
+                        case PaytoType.Ethereum: {
+                          const name = details.account.params["receiver-name"];
+                          return (
+                            <Fragment>
+                              <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
+                                <dt class="text-sm font-medium leading-6 text-gray-900">
+                                  <i18n.Translate>
+                                    Payment Service Provider's account address
+                                  </i18n.Translate>
+                                </dt>
+                                <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
+                                  {details.account.address}
+                                </dd>
+                              </div>
+                              {name && (
+                                <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
+                                  <dt class="text-sm font-medium leading-6 text-gray-900">
+                                    <i18n.Translate>
+                                      Payment Service Provider's name
                                     </i18n.Translate>
                                   </dt>
                                   <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
@@ -323,7 +327,7 @@ export function NeedConfirmationView({
                           />
                         ) : (
                           <i18n.Translate>
-                            No amount specified yet.
+                            No amount has yet been determined.
                           </i18n.Translate>
                         )}
                       </dd>
@@ -349,28 +353,22 @@ export function NeedConfirmationView({
                 </div>
               </div>
               <div class="flex items-center justify-between gap-x-6 border-t border-gray-900/10 px-4 py-4 sm:px-8">
-                <button
+                <ButtonBetter
                   type="button"
                   name="cancel"
                   class="text-sm font-semibold leading-6 text-gray-900"
-                  onClick={(e) => {
-                    e.preventDefault();
-                    onCancel();
-                  }}
+                  onClick={doAbort}
                 >
                   <i18n.Translate>Cancel</i18n.Translate>
-                </button>
-                <button
+                </ButtonBetter>
+                <ButtonBetter
                   type="submit"
                   name="transfer"
                   class="disabled:opacity-50 disabled:cursor-default cursor-pointer rounded-md bg-indigo-600 px-3 py-2 text-sm font-semibold text-white shadow-sm hover:bg-indigo-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-indigo-600"
-                  onClick={(e) => {
-                    e.preventDefault();
-                    onConfirm();
-                  }}
+                  onClick={!doConfirm ? undefined : notifyOnError(doConfirm)}
                 >
                   <i18n.Translate>Transfer</i18n.Translate>
-                </button>
+                </ButtonBetter>
               </div>
             </form>
           </ShouldBeSameUser>
@@ -388,7 +386,9 @@ export function FailedView({ error }: State.Failed) {
           type="danger"
           title={i18n.str`Unauthorized to make the operation, maybe the session has expired or the password changed.`}
         >
-          <div class="mt-2 text-sm text-red-700">{error.detail.hint}</div>
+          {!error.detail ? undefined : (
+            <div class="mt-2 text-sm text-red-700">{error.detail.hint}</div>
+          )}
         </Attention>
       );
     case HttpStatusCode.Conflict:
@@ -397,7 +397,9 @@ export function FailedView({ error }: State.Failed) {
           type="danger"
           title={i18n.str`The operation was rejected due to insufficient funds.`}
         >
-          <div class="mt-2 text-sm text-red-700">{error.detail.hint}</div>
+          {!error.detail ? undefined : (
+            <div class="mt-2 text-sm text-red-700">{error.detail.hint}</div>
+          )}
         </Attention>
       );
     case HttpStatusCode.NotFound:
@@ -406,7 +408,9 @@ export function FailedView({ error }: State.Failed) {
           type="danger"
           title={i18n.str`The operation was rejected due to insufficient funds.`}
         >
-          <div class="mt-2 text-sm text-red-700">{error.detail.hint}</div>
+          {!error.detail ? undefined : (
+            <div class="mt-2 text-sm text-red-700">{error.detail.hint}</div>
+          )}
         </Attention>
       );
     default:
@@ -450,8 +454,9 @@ export function ConfirmedView({ routeClose }: State.Confirmed) {
           <div class="mt-2">
             <p class="text-sm text-gray-500">
               <i18n.Translate>
-                The wire transfer to the Taler operator has been initiated. You
-                will soon receive the requested amount in your Taler wallet.
+                The wire transfer to the Payment Service Provider has been
+                initiated. You will shortly receive the requested amount in your
+                Taler wallet.{" "}
               </i18n.Translate>
             </p>
           </div>
@@ -508,51 +513,47 @@ export function ConfirmedView({ routeClose }: State.Confirmed) {
 export function ReadyView({
   uri,
   focus,
-  onAbort: doAbort,
+  onAbort,
+  operationId,
 }: State.Ready): VNode {
   const { i18n } = useTranslationContext();
   const walletInegrationApi = useTalerWalletIntegrationAPI();
-  const [notification, notify, errorHandler] = useLocalNotification();
+  const [notification, withErrorHandler] = useLocalNotificationBetter();
+  const { state: credentials } = useSessionState();
+  const creds = credentials.status !== "loggedIn" ? undefined : credentials;
+  const {
+    config,
+    lib: { bank },
+  } = useBankCoreApiContext();
 
-  const talerWithdrawUri = stringifyWithdrawUri(uri);
+  const parsedUri = TalerUris.createTalerWithdraw(
+    uri.bankIntegrationApiBaseUrl,
+    uri.withdrawalOperationId,
+  );
+  const talerWithdrawUri = TalerUris.toString(parsedUri);
   useEffect(() => {
     walletInegrationApi.publishTalerAction(uri);
   }, []);
 
-  async function onAbort() {
-    errorHandler(async () => {
-      const hasError = await doAbort();
-      if (!hasError) return;
-      switch (hasError.case) {
-        case HttpStatusCode.Conflict:
-          return notify({
-            type: "error",
-            title: i18n.str`The reserve operation has been confirmed previously and can't be aborted`,
-            description: hasError.detail.hint as TranslatedString,
-            debug: hasError.detail,
-            when: AbsoluteTime.now(),
-          });
-        case HttpStatusCode.BadRequest:
-          return notify({
-            type: "error",
-            title: i18n.str`The operation id is invalid.`,
-            description: hasError.detail.hint as TranslatedString,
-            debug: hasError.detail,
-            when: AbsoluteTime.now(),
-          });
-        case HttpStatusCode.NotFound:
-          return notify({
-            type: "error",
-            title: i18n.str`The operation was not found.`,
-            description: hasError.detail.hint as TranslatedString,
-            debug: hasError.detail,
-            when: AbsoluteTime.now(),
-          });
-        default:
-          assertUnreachable(hasError);
-      }
-    });
-  }
+  const doAbort = !creds
+    ? undefined
+    : makeSafeCall(
+        i18n,
+        () => bank.abortWithdrawalById(creds, operationId),
+        (suc) => {
+          onAbort();
+        },
+        (fail) => {
+          switch (fail.case) {
+            case HttpStatusCode.Conflict:
+              return i18n.str`The reserve operation has been confirmed previously and can't be aborted`;
+            case HttpStatusCode.BadRequest:
+              return i18n.str`The operation ID is invalid.`;
+            case HttpStatusCode.NotFound:
+              return i18n.str`The operation was not found.`;
+          }
+        },
+      );
 
   return (
     <Fragment>
@@ -608,7 +609,7 @@ export function ReadyView({
         <div class="px-4 py-5 sm:p-6">
           <h3 class="text-base font-semibold leading-6 text-gray-900">
             <i18n.Translate>
-              Or if you have the Taler wallet on another device
+              In case you have a Taler wallet on another device
             </i18n.Translate>
           </h3>
           <div class="mt-4 max-w-xl text-sm text-gray-500">
@@ -626,7 +627,7 @@ export function ReadyView({
             // class="disabled:opacity-50 disabled:cursor-default cursor-pointer rounded-md  px-3 py-2 text-sm font-semibold text-black shadow-sm hover:bg-red-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-red-600"
             class="text-sm font-semibold leading-6 text-gray-900"
             // handler={onAbortHandler}
-            onClick={onAbort}
+            onClick={doAbort}
           >
             <i18n.Translate>Cancel</i18n.Translate>
           </button>
