@@ -15,56 +15,52 @@
  */
 
 import {
-  AbsoluteTime,
   AmountJson,
   AmountString,
   Amounts,
   CurrencySpecification,
   FRAC_SEPARATOR,
+  HostPortPath,
   HttpStatusCode,
-  PaytoString,
-  PaytoUri,
+  IbanString,
+  PaytoType,
+  Paytos,
   TalerCorebankApi,
   TalerErrorCode,
   TranslatedString,
-  assertUnreachable,
-  buildPayto,
-  parsePaytoUri,
-  stringifyPaytoUri,
+  assertUnreachable
 } from "@gnu-taler/taler-util";
 import {
+  ButtonBetter,
   InternationalizationAPI,
   LocalNotificationBanner,
   RouteDefinition,
   ShowInputErrorLabel,
+  makeSafeCall,
   notifyInfo,
   useBankCoreApiContext,
-  useLocalNotification,
+  useChallengeHandler,
+  useLocalNotificationBetter,
   useTranslationContext,
 } from "@gnu-taler/web-util/browser";
 import { ComponentChildren, Fragment, Ref, VNode, h } from "preact";
 import { useState } from "preact/hooks";
-import { mutate } from "swr";
-import { IdempotencyRetry } from "../../../taler-util/lib/http-client/utils.js";
-import { useBankState } from "../hooks/bank-state.js";
-import { useSessionState } from "../hooks/session.js";
+import { LoggedIn, useSessionState } from "../hooks/session.js";
 import { undefinedIfEmpty, validateIBAN, validateTalerBank } from "../utils.js";
+import { SolveMFAChallenges } from "./SolveMFA.js";
+import { IntAmountJson, IntAmounts } from "./regional/CreateCashout.js";
 
-interface Props {
+const TALER_SCREEN_ID = 106;
+
+export interface Props {
   focus?: boolean;
   withAccount?: string;
   withSubject?: string;
   withAmount?: string;
   onSuccess: () => void;
-  onAuthorizationRequired: () => void;
   routeCancel?: RouteDefinition;
   routeCashout?: RouteDefinition;
-  routeHere: RouteDefinition<{
-    account?: string;
-    subject?: string;
-    amount?: string;
-  }>;
-  limit: AmountJson;
+  limit: IntAmountJson;
   balance: AmountJson;
 }
 
@@ -76,8 +72,6 @@ export function PaytoWireTransferForm({
   onSuccess,
   routeCancel,
   routeCashout,
-  routeHere,
-  onAuthorizationRequired,
   limit,
   balance,
 }: Props): VNode {
@@ -96,31 +90,30 @@ export function PaytoWireTransferForm({
   const [account, setAccount] = useState<string | undefined>(withAccount);
   const [subject, setSubject] = useState<string | undefined>(withSubject);
   const [amount, setAmount] = useState<string | undefined>(withAmount);
-  const [, updateBankState] = useBankState();
 
   const [rawPaytoInput, rawPaytoInputSetter] = useState<string | undefined>(
     undefined,
   );
   const { i18n } = useTranslationContext();
-
-  const trimmedAmountStr = amount?.trim();
-  const parsedAmount = Amounts.parse(`${limit.currency}:${trimmedAmountStr}`);
-  const [notification, notify, handleError] = useLocalNotification();
-
-  const paytoType =
-    config.wire_type === "X_TALER_BANK"
-      ? ("x-taler-bank" as const)
-      : ("iban" as const);
-
   const wireFee =
     config.wire_transfer_fees === undefined
       ? Amounts.zeroOfCurrency(config.currency)
       : Amounts.parseOrThrow(config.wire_transfer_fees);
 
-  const limitWithFee =
-    Amounts.cmp(limit, wireFee) === 1
-      ? Amounts.sub(limit, wireFee).amount
-      : Amounts.zeroOfAmount(limit);
+  const trimmedAmountStr = amount?.trim();
+  const limitWithFee = IntAmounts.from(limit)
+    .deduce(wireFee)
+    .getResultZeroIfNegative();
+  const parsedAmount = Amounts.parse(
+    `${limitWithFee.currency}:${trimmedAmountStr}`,
+  );
+  const [notification, notifyOnError] = useLocalNotificationBetter();
+  const mfa = useChallengeHandler();
+
+  const paytoType =
+    config.wire_type === "X_TALER_BANK"
+      ? ("x-taler-bank" as const)
+      : ("iban" as const);
 
   const errorsWire = undefinedIfEmpty({
     account: !account
@@ -138,165 +131,149 @@ export function PaytoWireTransferForm({
         : validateAmount(parsedAmount, limitWithFee, i18n),
   });
 
-  const parsed = !rawPaytoInput ? undefined : parsePaytoUri(rawPaytoInput);
+  const foundPayto = searchPaytoInHumanReadableText(rawPaytoInput);
+  const parsed = !foundPayto ? undefined : Paytos.fromString(foundPayto);
 
   const errorsPayto = undefinedIfEmpty({
     rawPaytoInput: !rawPaytoInput
       ? i18n.str`Required`
-      : !parsed
+      : !parsed || parsed.type === "fail"
         ? i18n.str`Does not follow the pattern`
-        : validateRawPayto(parsed, limitWithFee, url.host, i18n, paytoType),
+        : validateRawPayto(
+            parsed.body,
+            limitWithFee,
+            url.host,
+            i18n,
+            paytoType,
+          ),
   });
 
-  async function doSend() {
-    let payto_uri: PaytoString | undefined;
-    let sendingAmount: AmountString | undefined;
+  let payto_uri: Paytos.FullPaytoString | undefined;
+  let sendingAmount: AmountString | undefined;
 
-    if (credentials.status !== "loggedIn") return;
-    let acName: string | undefined;
-    if (isRawPayto) {
-      const p = parsePaytoUri(rawPaytoInput!);
-      if (!p) return;
+  let acName: string | undefined;
+  if (isRawPayto) {
+    const res = Paytos.fromString(rawPaytoInput!);
+
+    if (res && res.type === "ok") {
+      const p = res.body;
       sendingAmount = p.params.amount as AmountString;
       delete p.params.amount;
       // if this payto is valid then it already have message
-      payto_uri = stringifyPaytoUri(p);
-      acName = !p.isKnown
-        ? undefined
-        : p.targetType === "iban"
-          ? p.iban
-          : p.targetType === "bitcoin"
-            ? p.address
-            : p.targetType === "x-taler-bank"
-              ? p.account
-              : assertUnreachable(p);
-    } else {
-      if (!account || !subject) return;
-      let payto;
-      acName = account;
-      switch (paytoType) {
-        case "x-taler-bank": {
-          payto = buildPayto("x-taler-bank", url.host, account);
-
-          break;
-        }
-        case "iban": {
-          payto = buildPayto("iban", account, undefined);
-          break;
-        }
-        default:
-          assertUnreachable(paytoType);
-      }
-
-      payto.params.message = encodeURIComponent(subject);
-      payto_uri = stringifyPaytoUri(payto);
-      sendingAmount = `${limit.currency}:${trimmedAmountStr}` as AmountString;
+      payto_uri = Paytos.toFullString(p);
+      acName =
+        p.targetType === undefined
+          ? undefined
+          : p.targetType === PaytoType.IBAN
+            ? p.iban
+            : p.targetType === PaytoType.TalerReserve ||
+                p.targetType === PaytoType.TalerReserveHttp
+              ? undefined // FIXME: unsupported payto://
+              : p.targetType === PaytoType.Bitcoin
+                ? p.address
+                : p.targetType === PaytoType.Ethereum
+                  ? p.address
+                  : p.targetType === PaytoType.TalerBank
+                    ? p.account
+                    : assertUnreachable(p);
     }
-    const puri = payto_uri;
-    const sAmount = sendingAmount;
-
-    await handleError(async function createTransactionHandleError() {
-      const request: TalerCorebankApi.CreateTransactionRequest = {
-        payto_uri: puri,
-        amount: sAmount,
-      };
-      const check = IdempotencyRetry.tryFiveTimes();
-      const resp = await api.createTransaction(credentials, request, check);
-      mutate(() => true);
-      if (resp.type === "fail") {
-        switch (resp.case) {
-          case HttpStatusCode.BadRequest:
-            return notify({
-              type: "error",
-              title: i18n.str`The request was invalid or the payto://-URI used unacceptable features.`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case HttpStatusCode.Unauthorized:
-            return notify({
-              type: "error",
-              title: i18n.str`Not enough permission to complete the operation.`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case TalerErrorCode.BANK_ADMIN_CREDITOR:
-            return notify({
-              type: "error",
-              title: i18n.str`Bank administrator can't be the transfer creditor.`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case TalerErrorCode.BANK_UNKNOWN_CREDITOR:
-            return notify({
-              type: "error",
-              title: i18n.str`The destination account "${
-                acName ?? puri
-              }" was not found.`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case TalerErrorCode.BANK_SAME_ACCOUNT:
-            return notify({
-              type: "error",
-              title: i18n.str`The origin and the destination of the transfer can't be the same.`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case TalerErrorCode.BANK_UNALLOWED_DEBIT:
-            return notify({
-              type: "error",
-              title: i18n.str`Your balance is not enough.`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case HttpStatusCode.NotFound:
-            return notify({
-              type: "error",
-              title: i18n.str`The origin account "${puri}" was not found.`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case TalerErrorCode.BANK_TRANSFER_REQUEST_UID_REUSED: {
-            return notify({
-              type: "error",
-              title: i18n.str`Tried to create the transaction ${check.maxTries} times with different UID but failed.`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          }
-          case HttpStatusCode.Accepted: {
-            updateBankState("currentChallenge", {
-              operation: "create-transaction",
-              id: String(resp.body.challenge_id),
-              location: routeHere.url({
-                account: account ?? "",
-                amount,
-                subject,
-              }),
-              sent: AbsoluteTime.never(),
-              request,
-            });
-            return onAuthorizationRequired();
-          }
-          default:
-            assertUnreachable(resp);
-        }
+  } else if (account && subject) {
+    // if (!account || !subject) return;
+    let payto;
+    acName = account;
+    switch (paytoType) {
+      case "x-taler-bank": {
+        payto = Paytos.createTalerBank(url.host as HostPortPath, account);
+        break;
       }
-      notifyInfo(i18n.str`Wire transfer created!`);
-      onSuccess();
-      setAmount(undefined);
-      setAccount(undefined);
-      setSubject(undefined);
-      rawPaytoInputSetter(undefined);
-    });
+      case "iban": {
+        payto = Paytos.createIban(account as IbanString, undefined);
+        break;
+      }
+      default:
+        assertUnreachable(paytoType);
+    }
+
+    payto.params.message = encodeURIComponent(subject);
+    payto_uri = Paytos.toFullString(payto);
+    sendingAmount =
+      `${limitWithFee.currency}:${trimmedAmountStr}` as AmountString;
+  }
+  const puri = payto_uri;
+  const sAmount = sendingAmount;
+
+  const request: TalerCorebankApi.CreateTransactionRequest | undefined =
+    !payto_uri
+      ? undefined
+      : {
+          payto_uri: puri!,
+          amount: sAmount,
+        };
+
+  type reqType = TalerCorebankApi.CreateTransactionRequest;
+
+  const [doTransfer, repeatTransfer, lastCallingArgs] = mfa.withMfaHandler(
+    ({ challengeIds, onChallengeRequired }) =>
+      makeSafeCall(
+        i18n,
+        (credentials: LoggedIn, request: reqType) =>
+          api.createTransaction(credentials, request, {
+            challengeIds,
+          }),
+        (success) => {
+          notifyInfo(i18n.str`The wire transfer was successfully completed!`);
+          onSuccess();
+          setAmount(undefined);
+          setAccount(undefined);
+          setSubject(undefined);
+          rawPaytoInputSetter(undefined);
+        },
+        (fail) => {
+          switch (fail.case) {
+            case HttpStatusCode.BadRequest:
+              return i18n.str`The request was invalid or the payto://-URI used unacceptable features.`;
+            case HttpStatusCode.Unauthorized:
+              return i18n.str`Not enough permission to complete the operation.`;
+            case TalerErrorCode.BANK_ADMIN_CREDITOR:
+              return i18n.str`The bank administrator cannot be the transfer creditor.`;
+            case TalerErrorCode.BANK_UNKNOWN_CREDITOR:
+              return i18n.str`The destination account "${
+                acName ?? puri
+              }" was not found.`;
+            case TalerErrorCode.BANK_SAME_ACCOUNT:
+              return i18n.str`The origin and the destination of the transfer can't be the same.`;
+            case TalerErrorCode.BANK_UNALLOWED_DEBIT:
+              return i18n.str`Your balance is not sufficient for the operation.`;
+            case HttpStatusCode.NotFound:
+              return i18n.str`The origin account "${puri}" was not found.`;
+            case TalerErrorCode.BANK_TRANSFER_REQUEST_UID_REUSED: {
+              return i18n.str`The attempt to create the transaction has failed. Please try again.`;
+            }
+            case HttpStatusCode.Accepted: {
+              onChallengeRequired(fail.body);
+              return i18n.str`A second factor authentication is required.`;
+            }
+            default:
+              assertUnreachable(fail);
+          }
+        },
+      ),
+  );
+  const sendHandler =
+    !request || credentials.status !== "loggedIn"
+      ? undefined
+      : () => notifyOnError(doTransfer)(credentials, request);
+
+  if (mfa.pendingChallenge && repeatTransfer) {
+    return (
+      <SolveMFAChallenges
+        currentChallenge={mfa.pendingChallenge}
+        description={i18n.str`Confirm wire transfer.`}
+        onCancel={mfa.doCancelChallenge}
+        username={lastCallingArgs[0].username}
+        onCompleted={repeatTransfer}
+      />
+    );
   }
 
   return (
@@ -315,33 +292,40 @@ export function PaytoWireTransferForm({
                 type="radio"
                 name="input-type"
                 onChange={() => {
-                  if (parsed && parsed.isKnown) {
-                    switch (parsed.targetType) {
-                      case "iban": {
-                        setAccount(parsed.iban);
+                  if (parsed && parsed.type === "ok") {
+                    switch (parsed.body.targetType) {
+                      case PaytoType.Ethereum:
+                      case PaytoType.Bitcoin:
+                      case undefined:
+                      case PaytoType.TalerReserve:
+                      case PaytoType.TalerReserveHttp: {
+                        // FIXME: unsupported payto
                         break;
                       }
-                      case "x-taler-bank": {
-                        setAccount(parsed.account);
+                      case PaytoType.IBAN: {
+                        setAccount(parsed.body.iban);
                         break;
                       }
-                      case "bitcoin": {
+                      case PaytoType.TalerBank: {
+                        setAccount(parsed.body.account);
                         break;
                       }
                       default: {
-                        assertUnreachable(parsed);
+                        assertUnreachable(parsed.body);
                       }
                     }
-                    const amountStr = !parsed.params
+                    const amountStr = !parsed.body.params
                       ? undefined
-                      : parsed.params["amount"];
+                      : parsed.body.params["amount"];
                     if (amountStr) {
                       const amount = Amounts.parse(amountStr);
                       if (amount) {
                         setAmount(Amounts.stringifyValue(amount));
                       }
                     }
-                    const subject = parsed.params["message"];
+                    const subject = !parsed.body.params["message"]
+                      ? parsed.body.params["subject"]
+                      : parsed.body.params["message"];
                     if (subject) {
                       setSubject(subject);
                     }
@@ -373,12 +357,11 @@ export function PaytoWireTransferForm({
                     name="input-type"
                     onChange={() => {
                       if (account) {
-                        let payto;
+                        let payto: Paytos.URI;
                         switch (paytoType) {
                           case "x-taler-bank": {
-                            payto = buildPayto(
-                              "x-taler-bank",
-                              url.host,
+                            payto = Paytos.createTalerBank(
+                              url.host as HostPortPath,
                               account,
                             );
                             if (parsedAmount) {
@@ -391,7 +374,10 @@ export function PaytoWireTransferForm({
                             break;
                           }
                           case "iban": {
-                            payto = buildPayto("iban", account, undefined);
+                            payto = Paytos.createIban(
+                              account as IbanString,
+                              undefined,
+                            );
                             if (parsedAmount) {
                               payto.params["amount"] =
                                 Amounts.stringify(parsedAmount);
@@ -404,7 +390,7 @@ export function PaytoWireTransferForm({
                           default:
                             assertUnreachable(paytoType);
                         }
-                        rawPaytoInputSetter(stringifyPaytoUri(payto));
+                        rawPaytoInputSetter(Paytos.toFullString(payto));
                       }
                       setInputType("payto");
                     }}
@@ -424,8 +410,8 @@ export function PaytoWireTransferForm({
                       class="block text-sm text-gray-500 data-[checked=true]:text-indigo-600"
                     >
                       <i18n.Translate>
-                        A special URI that indicate the transfer amount and
-                        account target.
+                        A special URI that specifies the amount to be
+                        transferred and the destination account.
                       </i18n.Translate>
                     </span>
                   </span>
@@ -459,7 +445,7 @@ export function PaytoWireTransferForm({
                           class="block text-sm text-gray-500 data-[checked=true]:text-indigo-600"
                         >
                           <i18n.Translate>
-                            If you have a camera in this device you can import a
+                            If your device has a camera, you can import a
                             payto:// URI from a QR code.
                           </i18n.Translate>
                         </span>
@@ -470,7 +456,7 @@ export function PaytoWireTransferForm({
               </Fragment>
             )}
           </div>
-          {routeCashout ? (
+          {routeCashout && config.allow_conversion ? (
             <a
               name="do cashout"
               href={routeCashout.url({})}
@@ -501,7 +487,7 @@ export function PaytoWireTransferForm({
                         id="x-taler-bank"
                         required
                         label={i18n.str`Recipient`}
-                        help={i18n.str`Id of the recipient's account`}
+                        help={i18n.str`ID of the recipient's account`}
                         error={errorsWire?.account}
                         onChange={setAccount}
                         value={account}
@@ -538,11 +524,12 @@ export function PaytoWireTransferForm({
                   class="block text-sm font-medium leading-6 text-gray-900"
                 >
                   {i18n.str`Transfer subject`}
-                  <b style={{ color: "red" }}> *</b>
+                  <b class="text-[red]"> *</b>
                 </label>
                 <div class="mt-2">
-                  <input
-                    type="text"
+                  <textarea
+                    type="textarea"
+                    rows={3}
                     class="block w-full rounded-md border-0 py-1.5 text-gray-900 shadow-sm ring-1 ring-inset ring-gray-300 placeholder:text-gray-400 focus:ring-2 focus:ring-inset focus:ring-indigo-600 sm:text-sm sm:leading-6"
                     name="subject"
                     id="subject"
@@ -572,12 +559,12 @@ export function PaytoWireTransferForm({
                   class="block text-sm font-medium leading-6 text-gray-900"
                 >
                   {i18n.str`Amount`}
-                  <b style={{ color: "red" }}> *</b>
+                  <b class="text-[red]"> *</b>
                 </label>
                 <InputAmount
                   name="amount"
                   left
-                  currency={limit.currency}
+                  currency={limitWithFee.currency}
                   value={trimmedAmountStr}
                   onChange={(d) => {
                     setAmount(d);
@@ -600,7 +587,7 @@ export function PaytoWireTransferForm({
                   class="block text-sm font-medium leading-6 text-gray-900"
                 >
                   {i18n.str`Payto URI:`}
-                  <b style={{ color: "red" }}> *</b>
+                  <b class="text-[red]"> *</b>
                 </label>
                 <div class="mt-2">
                   <textarea
@@ -616,9 +603,9 @@ export function PaytoWireTransferForm({
                     placeholder={((): TranslatedString => {
                       switch (paytoType) {
                         case "x-taler-bank":
-                          return i18n.str`payto://x-taler-bank/[bank-host]/[receiver-account]?message=[subject]&amount=[${limit.currency}:X.Y]`;
+                          return i18n.str`payto://x-taler-bank/[bank-host]/[receiver-account]?message=[subject]&amount=[${limitWithFee.currency}:X.Y]`;
                         case "iban":
-                          return i18n.str`payto://iban/[receiver-iban]?message=[subject]&amount=[${limit.currency}:X.Y]`;
+                          return i18n.str`payto://iban/[receiver-iban]?message=[subject]&amount=[${limitWithFee.currency}:X.Y]`;
                       }
                     })()}
                     onInput={(e): void => {
@@ -633,10 +620,10 @@ export function PaytoWireTransferForm({
               </div>
             </div>
           )}
-          {Amounts.cmp(limitWithFee, balance) > 0 ? (
+          {Amounts.isNonZero(limitWithFee) ? (
             <p class="mt-2 text-sm text-gray-900">
               <i18n.Translate>
-                You can transfer{" "}
+                The maximum amount for a wire transfer is{" "}
                 <RenderAmount
                   value={limitWithFee}
                   spec={config.currency_specification}
@@ -684,18 +671,15 @@ export function PaytoWireTransferForm({
           ) : (
             <div />
           )}
-          <button
+          <ButtonBetter
             type="submit"
             name="send"
             class="disabled:opacity-50 disabled:cursor-default cursor-pointer rounded-md bg-indigo-600 px-3 py-2 text-sm font-semibold text-white shadow-sm hover:bg-indigo-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-indigo-600"
             disabled={isRawPayto ? !!errorsPayto : !!errorsWire}
-            onClick={(e) => {
-              e.preventDefault();
-              doSend();
-            }}
+            onClick={sendHandler}
           >
             <i18n.Translate>Send</i18n.Translate>
-          </button>
+          </ButtonBetter>
         </div>
         <LocalNotificationBanner notification={notification} />
       </form>
@@ -726,12 +710,14 @@ export function InputAmount(
     name,
     value,
     left,
+    placeholder,
     onChange,
   }: {
     currency: string;
     name: string;
     left?: boolean | undefined;
     value: string | undefined;
+    placeholder?: string | undefined;
     onChange?: (s: string) => void;
   },
   ref: Ref<HTMLInputElement>,
@@ -747,7 +733,7 @@ export function InputAmount(
           type="number"
           data-left={left}
           class="disabled:bg-gray-200 text-right rounded-md rounded-l-none data-[left=true]:text-left w-full py-1.5 pl-3 text-gray-900  placeholder:text-gray-400  sm:text-sm sm:leading-6"
-          placeholder="0.00"
+          placeholder={placeholder ?? "0.00"}
           aria-describedby="price-currency"
           ref={ref}
           name={name}
@@ -787,15 +773,17 @@ export function InputAmount(
 export function RenderAmount({
   value,
   spec,
-  negative,
-  withColor,
-  hideSmall,
+  negative = false,
+  withColor = false,
+  hideSmall = false,
+  withSign = false,
 }: {
   spec: CurrencySpecification;
   value: AmountJson;
   hideSmall?: boolean;
   negative?: boolean;
   withColor?: boolean;
+  withSign?: boolean;
 }): VNode {
   const neg = !!negative; // convert to true or false
 
@@ -809,7 +797,7 @@ export function RenderAmount({
       data-negative={withColor ? neg : undefined}
       class="whitespace-nowrap data-[negative=false]:text-green-600 data-[negative=true]:text-red-600"
     >
-      {negative ? "- " : undefined}
+      {withSign && negative ? "- " : undefined}
       {currency} {normal}{" "}
       {!hideSmall && small && <sup class="-ml-1">{small}</sup>}
     </span>
@@ -817,15 +805,12 @@ export function RenderAmount({
 }
 
 function validateRawPayto(
-  parsed: PaytoUri,
+  parsed: Paytos.URI,
   limit: AmountJson,
   host: string,
   i18n: InternationalizationAPI,
   type: "iban" | "x-taler-bank",
 ): TranslatedString | undefined {
-  if (!parsed.isKnown) {
-    return i18n.str`The target type is unknown, use "${type}"`;
-  }
   let result: TranslatedString | undefined;
   switch (type) {
     case "x-taler-bank": {
@@ -838,7 +823,7 @@ function validateRawPayto(
       }
 
       if (!parsed.account) {
-        return i18n.str`Missing account name`;
+        return i18n.str`Account name is missing`;
       }
       const result = validateTalerBank(parsed.account, i18n);
       if (result) return result;
@@ -866,10 +851,9 @@ function validateRawPayto(
   if (result) return result;
 
   if (!parsed.params.message) {
-    return i18n.str`Missing the "message" parameter to specify a reference text for the transfer`;
+    return i18n.str`"message" parameters to specify a reference text for the transfer are missing`;
   }
-  const subject = parsed.params.message;
-  result = validateSubject(subject, i18n);
+  result = validateSubject(parsed.params.message, i18n);
   if (result) return result;
 
   return undefined;
@@ -884,10 +868,10 @@ function validateAmount(
     return i18n.str`The only currency allowed is "${limit.currency}"`;
   }
   if (Amounts.isZero(amount)) {
-    return i18n.str`Can't transfer zero amount`;
+    return i18n.str`You cannot transfer an amount of zero.`;
   }
   if (Amounts.cmp(limit, amount) === -1) {
-    return i18n.str`Balance is not enough`;
+    return i18n.str`The balance is not sufficient`;
   }
   return undefined;
 }
@@ -897,7 +881,7 @@ function validateSubject(
   i18n: InternationalizationAPI,
 ): TranslatedString | undefined {
   if (text.length < 2) {
-    return i18n.str`Use a longer subject`;
+    return i18n.str`Please enter a longer subject`;
   }
   return undefined;
 }
@@ -946,7 +930,7 @@ export function TextField({
     <div class="sm:col-span-5">
       <label for={id} class="block text-sm font-medium leading-6 text-gray-900">
         {label}
-        {required && <b style={{ color: "red" }}> *</b>}
+        {required && <b class="text-[red]"> *</b>}
       </label>
       <div class="mt-2">
         <Wrapper withIcon={rightIcons !== undefined}>
@@ -973,3 +957,14 @@ export function TextField({
     </div>
   );
 }
+
+const PAYTO_START_REGEX = /(?:payto:\/\/[^\s]*)/;
+
+function searchPaytoInHumanReadableText(
+  text: string | undefined,
+): string | undefined {
+  if (!text) return undefined;
+  const result = PAYTO_START_REGEX.exec(text);
+  if (!result) return undefined;
+  return result[0];
+}
