@@ -27,7 +27,11 @@ import { useState } from "preact/hooks";
 import { useSettingsContext } from "../context/settings.js";
 import { undefinedIfEmpty } from "../utils.js";
 import { getRandomPassword, getRandomUsername } from "./rnd.js";
+import { usePreferences } from "../hooks/preferences.js";
 
+const TALER_SCREEN_ID = 110;
+
+RegistrationPage.SCREEN_ID = TALER_SCREEN_ID;
 export function RegistrationPage({
   onRegistrationSuccesful,
   routeCancel,
@@ -52,12 +56,13 @@ export function RegistrationPage({
 
 // eslint-disable-next-line no-useless-escape
 export const USERNAME_REGEX = /^[a-zA-Z0-9\-\.\_\~]*$/;
-export const PHONE_REGEX = /^(\+\d{1,2}\s)?\(?\d{3}\)?[\s.-]\d{3}[\s.-]\d{4}$/;
-export const EMAIL_REGEX = /^[\w-.]+@([\w-]+\.)+[\w-]{2,4}$/;
+// export const PHONE_REGEX = /^(\+\d{1,2}\s)?\(?\d{3}\)?[\s.-]\d{3}[\s.-]\d{4}$/;
+// export const EMAIL_REGEX = /^[\w-.]+@([\w-]+\.)+[\w-]{2,4}$/;
 
 /**
  * Collect and submit registration data.
  */
+RegistrationForm.SCREEN_ID = TALER_SCREEN_ID;
 function RegistrationForm({
   onRegistrationSuccesful,
   routeCancel,
@@ -73,6 +78,7 @@ function RegistrationForm({
   const [repeatPassword, setRepeatPassword] = useState<string | undefined>();
   const [notification, , handleError] = useLocalNotification();
   const settings = useSettingsContext();
+  const [pref] = usePreferences();
 
   const {
     lib: { bank: api },
@@ -81,7 +87,7 @@ function RegistrationForm({
   const { i18n } = useTranslationContext();
 
   const errors = undefinedIfEmpty({
-    name: !name ? i18n.str`Missing name` : undefined,
+    name: !name ? i18n.str`The name is missing` : undefined,
     username: !username
       ? i18n.str`Missing username`
       : !USERNAME_REGEX.test(username)
@@ -97,11 +103,15 @@ function RegistrationForm({
     //   : !EMAIL_REGEX.test(email)
     //     ? i18n.str`Use letters and numbers only, and start with a lowercase letter`
     //     : undefined,
-    password: !password ? i18n.str`Missing password` : undefined,
+    password: !password
+      ? i18n.str`Missing password`
+      : pref.allowsSimplePassword && password.length < 8
+        ? i18n.str`The password should be longer than 8 letters`
+        : undefined,
     repeatPassword: !repeatPassword
       ? i18n.str`Missing password`
       : repeatPassword !== password
-        ? i18n.str`Passwords don't match`
+        ? i18n.str`The passwords do not match`
         : undefined,
   });
 
@@ -125,7 +135,7 @@ function RegistrationForm({
             case HttpStatusCode.BadRequest:
               return i18n.str`Server replied with invalid phone or email.`;
             case HttpStatusCode.Unauthorized:
-              return i18n.str`No enough permission to create that account.`;
+              return i18n.str`You are not authorised to create this account.`;
             case TalerErrorCode.BANK_UNALLOWED_DEBIT:
               return i18n.str`Registration is disabled because the bank ran out of bonus credit.`;
             case TalerErrorCode.BANK_RESERVED_USERNAME_CONFLICT:
@@ -133,17 +143,23 @@ function RegistrationForm({
             case TalerErrorCode.BANK_REGISTER_USERNAME_REUSE:
               return i18n.str`That username is already taken.`;
             case TalerErrorCode.BANK_REGISTER_PAYTO_URI_REUSE:
-              return i18n.str`That account id is already taken.`;
+              return i18n.str`That account ID is already taken.`;
             case TalerErrorCode.BANK_MISSING_TAN_INFO:
               return i18n.str`No information for the selected authentication channel.`;
             case TalerErrorCode.BANK_TAN_CHANNEL_NOT_SUPPORTED:
               return i18n.str`Authentication channel is not supported.`;
             case TalerErrorCode.BANK_NON_ADMIN_PATCH_DEBT_LIMIT:
-              return i18n.str`Only admin is allow to set debt limit.`;
-            case TalerErrorCode.BANK_NON_ADMIN_SET_MIN_CASHOUT:
-              return i18n.str`Only the administrator can change the minimum cashout limit.`;
+              return i18n.str`Only an administrator is allowed to set the debt limit.`;
+            case TalerErrorCode.BANK_NON_ADMIN_SET_CONVERSION_RATE_CLASS:
+              return i18n.str`Only the administrator can change the conversion rate.`;
+            case TalerErrorCode.BANK_CONVERSION_RATE_CLASS_UNKNOWN:
+              return i18n.str`The conversion rate class doesn't exist.`;
             case TalerErrorCode.BANK_NON_ADMIN_SET_TAN_CHANNEL:
               return i18n.str`Only admin can create accounts with second factor authentication.`;
+            case TalerErrorCode.BANK_PASSWORD_TOO_SHORT:
+              return i18n.str`The password is too short. Can't have less than 8 characters.`;
+            case TalerErrorCode.BANK_PASSWORD_TOO_LONG:
+              return i18n.str`The password is too long. Can't have more than 64 characters.`;
           }
         });
       }
@@ -164,7 +180,9 @@ function RegistrationForm({
     const user = getRandomUsername();
 
     const password = settings.simplePasswordForRandomAccounts
-      ? "123"
+      ? pref.allowsSimplePassword
+        ? "123"
+        : "12345678"
       : getRandomPassword();
     const username = `_${user.first}-${user.second}_`;
     const name = `${capitalizeFirstLetter(user.first)} ${capitalizeFirstLetter(
@@ -200,7 +218,7 @@ function RegistrationForm({
                 class="block text-sm font-medium leading-6 text-gray-900"
               >
                 <i18n.Translate>Login username</i18n.Translate>
-                <b style={{ color: "red" }}> *</b>
+                <b class="text-[red]"> *</b>
               </label>
               <div class="mt-2">
                 <input
@@ -232,7 +250,7 @@ function RegistrationForm({
                   class="block text-sm font-medium leading-6 text-gray-900"
                 >
                   <i18n.Translate>Password</i18n.Translate>
-                  <b style={{ color: "red" }}> *</b>
+                  <b class="text-[red]"> *</b>
                 </label>
               </div>
               <div class="mt-2">
@@ -255,6 +273,16 @@ function RegistrationForm({
                   isDirty={password !== undefined}
                 />
               </div>
+              {pref.allowsSimplePassword ? undefined : (
+                <p class="mt-2 text-sm text-gray-500">
+                  <i18n.Translate>
+                    Use a strong password: 8 characters minimum, don't use any
+                    public information related to you (names, birthday, phone
+                    number, etc...) and mix lowercase, uppercase, symbols and
+                    numbers
+                  </i18n.Translate>
+                </p>
+              )}
             </div>
 
             <div>
@@ -264,7 +292,7 @@ function RegistrationForm({
                   class="block text-sm font-medium leading-6 text-gray-900"
                 >
                   <i18n.Translate>Repeat password</i18n.Translate>
-                  <b style={{ color: "red" }}> *</b>
+                  <b class="text-[red]"> *</b>
                 </label>
               </div>
               <div class="mt-2">
@@ -296,7 +324,7 @@ function RegistrationForm({
                   class="block text-sm font-medium leading-6 text-gray-900"
                 >
                   <i18n.Translate>Full name</i18n.Translate>
-                  <b style={{ color: "red" }}> *</b>
+                  <b class="text-[red]"> *</b>
                 </label>
               </div>
               <div class="mt-2">
