@@ -15,46 +15,48 @@
  */
 
 import {
-  AbsoluteTime,
   AmountJson,
   Amounts,
   HttpStatusCode,
   PaytoUri,
-  PaytoUriIBAN,
-  PaytoUriTalerBank,
   TalerErrorCode,
-  TranslatedString,
   WithdrawUriResult,
   assertUnreachable,
 } from "@gnu-taler/taler-util";
 import {
   Attention,
+  ButtonBetter,
   LocalNotificationBanner,
+  makeSafeCall,
   notifyInfo,
-  useLocalNotification,
+  MfaHandler,
+  safeFunctionCall,
+  useBankCoreApiContext,
+  useChallengeHandler,
+  useLocalNotificationBetter,
   useTranslationContext,
 } from "@gnu-taler/web-util/browser";
 import { ComponentChildren, Fragment, VNode, h } from "preact";
 import { mutate } from "swr";
-import { useBankCoreApiContext } from "@gnu-taler/web-util/browser";
-import { useBankState } from "../hooks/bank-state.js";
 import { usePreferences } from "../hooks/preferences.js";
-import { useSessionState } from "../hooks/session.js";
-import { RouteDefinition } from "@gnu-taler/web-util/browser";
+import { LoggedIn, useSessionState } from "../hooks/session.js";
 import { LoginForm } from "./LoginForm.js";
 import { RenderAmount } from "./PaytoWireTransferForm.js";
+import { SolveMFAChallenges } from "./SolveMFA.js";
+import { Paytos } from "@gnu-taler/taler-util";
+import { PaytoType } from "@gnu-taler/taler-util";
+
+const TALER_SCREEN_ID = 114;
 
 interface Props {
   onAborted: () => void;
   withdrawUri: WithdrawUriResult;
-  routeHere: RouteDefinition<{ wopid: string }>;
   details: {
-    account: PaytoUri;
+    account: Paytos.URI;
     reserve: string;
     username: string;
     amount?: AmountJson;
   };
-  onAuthorizationRequired: () => void;
 }
 /**
  * Additional authentication required to complete the operation.
@@ -63,17 +65,15 @@ interface Props {
 export function WithdrawalConfirmationQuestion({
   onAborted,
   details,
-  onAuthorizationRequired,
-  routeHere,
   withdrawUri,
 }: Props): VNode {
   const { i18n } = useTranslationContext();
   const [settings] = usePreferences();
   const { state: credentials } = useSessionState();
   const creds = credentials.status !== "loggedIn" ? undefined : credentials;
-  const [, updateBankState] = useBankState();
 
-  const [notification, notify, handleError] = useLocalNotification();
+  const [notification, notifyOnError] = useLocalNotificationBetter();
+  const mfa = useChallengeHandler();
 
   const {
     config,
@@ -85,120 +85,130 @@ export function WithdrawalConfirmationQuestion({
       ? Amounts.zeroOfCurrency(config.currency)
       : Amounts.parseOrThrow(config.wire_transfer_fees);
 
-  async function doTransfer() {
-    await handleError(async () => {
-      if (!creds) return;
-      const resp = await api.confirmWithdrawalById(
-        creds,
-        withdrawUri.withdrawalOperationId,
-      );
-      if (resp.type === "ok") {
-        mutate(() => true); // clean any info that we have
-        if (!settings.showWithdrawalSuccess) {
-          notifyInfo(i18n.str`Wire transfer completed!`);
-        }
-      } else {
-        switch (resp.case) {
-          case TalerErrorCode.BANK_CONFIRM_ABORT_CONFLICT:
-            return notify({
-              type: "error",
-              title: i18n.str`The withdrawal has been aborted previously and can't be confirmed`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case TalerErrorCode.BANK_CONFIRM_INCOMPLETE:
-            return notify({
-              type: "error",
-              title: i18n.str`The withdrawal operation can't be confirmed before a wallet accepted the transaction.`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case HttpStatusCode.BadRequest:
-            return notify({
-              type: "error",
-              title: i18n.str`The operation id is invalid.`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case HttpStatusCode.NotFound:
-            return notify({
-              type: "error",
-              title: i18n.str`The operation was not found.`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case TalerErrorCode.BANK_UNALLOWED_DEBIT:
-            return notify({
-              type: "error",
-              title: i18n.str`Your balance is not enough for the operation.`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case HttpStatusCode.Accepted: {
-            updateBankState("currentChallenge", {
-              operation: "confirm-withdrawal",
-              id: String(resp.body.challenge_id),
-              location: routeHere.url({
-                wopid: withdrawUri.withdrawalOperationId,
-              }),
-              sent: AbsoluteTime.never(),
-              request: withdrawUri.withdrawalOperationId,
-            });
-            return onAuthorizationRequired();
-          }
-          default:
-            assertUnreachable(resp);
-        }
-      }
-    });
-  }
+      /**
+       * Continue here:
+       * 
+       * Make the useNotification hook take "handler|undefine"
+       * to cover the most common use case.
+       * 
+       * Usenotification hook also should take the new SafeHandler interface
+       * 
+       * safeFunctionCall should also include the default error handler that shows
+       * the generic erros (talererror and http errors)
+       */
+  // const fn = safeFunctionCall(
+  //   (mfa: MfaHandler, creds: LoggedIn, opId: string) =>
+  //     api.confirmWithdrawalById(creds, {}, opId, mfa),
+  // );
 
-  async function doCancel() {
-    await handleError(async () => {
-      if (!creds) return;
-      const resp = await api.abortWithdrawalById(
-        creds,
-        withdrawUri.withdrawalOperationId,
-      );
-      if (resp.type === "ok") {
-        onAborted();
-      } else {
-        switch (resp.case) {
-          case HttpStatusCode.Conflict:
-            return notify({
-              type: "error",
-              title: i18n.str`The reserve operation has been confirmed previously and can't be aborted`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case HttpStatusCode.BadRequest:
-            return notify({
-              type: "error",
-              title: i18n.str`The operation id is invalid.`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case HttpStatusCode.NotFound:
-            return notify({
-              type: "error",
-              title: i18n.str`The operation was not found.`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          default: {
-            assertUnreachable(resp);
+  // fn.onSuccess = () => {
+  //   mutate(() => true); // clean any info that we have
+  //   if (!settings.showWithdrawalSuccess) {
+  //     notifyInfo(i18n.str`Wire transfer completed!`);
+  //   }
+  // };
+
+  // fn.onFail = (fail, mfa, a, b) => {
+  //   switch (fail.case) {
+  //     case TalerErrorCode.BANK_CONFIRM_ABORT_CONFLICT:
+  //       return i18n.str`The withdrawal has been aborted previously and can't be confirmed`;
+  //     case TalerErrorCode.BANK_CONFIRM_INCOMPLETE:
+  //       return i18n.str`The withdrawal operation can't be confirmed before a wallet accepted the transaction.`;
+  //     case HttpStatusCode.BadRequest:
+  //       return i18n.str`The operation ID is invalid.`;
+  //     case HttpStatusCode.NotFound:
+  //       return i18n.str`The operation was not found.`;
+  //     case TalerErrorCode.BANK_UNALLOWED_DEBIT:
+  //       return i18n.str`Your balance is not sufficient for the operation.`;
+  //     case TalerErrorCode.BANK_AMOUNT_DIFFERS:
+  //       return i18n.str`The starting withdrawal amount and the confirmation amount differs.`;
+  //     case TalerErrorCode.BANK_AMOUNT_REQUIRED:
+  //       return i18n.str`The bank requires a bank account which has not been specified yet.`;
+  //     case HttpStatusCode.Accepted: {
+  //       mfa.onChallengeRequired(fail.body);
+  //       return i18n.str`A second factor authentication is required.`;
+  //     }
+  //   }
+  // };
+
+  //   const [doConfirm2, repeatConfirm2] = mfa.addMfaHandler( fn )
+
+  const [doConfirm, repeatConfirm] = mfa.withMfaHandler(
+    ({ challengeIds, onChallengeRequired }) =>
+      makeSafeCall(
+        i18n,
+        (creds: LoggedIn, opId: string) =>
+          api.confirmWithdrawalById(creds, {}, opId, { challengeIds }),
+        (success) => {
+          mutate(() => true); // clean any info that we have
+          if (!settings.showWithdrawalSuccess) {
+            notifyInfo(i18n.str`Wire transfer completed!`);
           }
-        }
-      }
-    });
+        },
+        (fail) => {
+          switch (fail.case) {
+            case TalerErrorCode.BANK_CONFIRM_ABORT_CONFLICT:
+              return i18n.str`The withdrawal has been aborted previously and can't be confirmed`;
+            case TalerErrorCode.BANK_CONFIRM_INCOMPLETE:
+              return i18n.str`The withdrawal operation can't be confirmed before a wallet accepted the transaction.`;
+            case HttpStatusCode.BadRequest:
+              return i18n.str`The operation ID is invalid.`;
+            case HttpStatusCode.NotFound:
+              return i18n.str`The operation was not found.`;
+            case TalerErrorCode.BANK_UNALLOWED_DEBIT:
+              return i18n.str`Your balance is not sufficient for the operation.`;
+            case TalerErrorCode.BANK_AMOUNT_DIFFERS:
+              return i18n.str`The starting withdrawal amount and the confirmation amount differs.`;
+            case TalerErrorCode.BANK_AMOUNT_REQUIRED:
+              return i18n.str`The bank requires a bank account which has not been specified yet.`;
+            case HttpStatusCode.Accepted: {
+              onChallengeRequired(fail.body);
+              return i18n.str`A second factor authentication is required.`;
+            }
+          }
+        },
+      ),
+  );
+
+  const confirmHandler = !creds
+    ? undefined
+    : () => notifyOnError(doConfirm)(creds, withdrawUri.withdrawalOperationId);
+
+  const abortHandler = !creds
+    ? undefined
+    : notifyOnError(() =>
+        makeSafeCall(
+          i18n,
+          (creds, opId) => api.abortWithdrawalById(creds, opId),
+          (success) => {
+            onAborted();
+          },
+          (fail) => {
+            switch (fail.case) {
+              case HttpStatusCode.Conflict:
+                return i18n.str`The reserve operation has been confirmed previously and can't be aborted`;
+              case HttpStatusCode.BadRequest:
+                return i18n.str`The operation ID is invalid.`;
+              case HttpStatusCode.NotFound:
+                return i18n.str`The operation was not found.`;
+              default: {
+                assertUnreachable(fail);
+              }
+            }
+          },
+        )(creds, withdrawUri.withdrawalOperationId),
+      );
+
+  if (mfa.pendingChallenge && repeatConfirm) {
+    return (
+      <SolveMFAChallenges
+        currentChallenge={mfa.pendingChallenge}
+        description={i18n.str`Complete withdrawal.`}
+        onCancel={mfa.doCancelChallenge}
+        onCompleted={repeatConfirm}
+        username={details.username}
+      />
+    );
   }
 
   return (
@@ -231,22 +241,14 @@ export function WithdrawalConfirmationQuestion({
                       <div class="mt-6 border-t border-gray-100">
                         <dl class="divide-y divide-gray-100">
                           {((): VNode => {
-                            if (!details.account.isKnown) {
-                              return (
-                                <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
-                                  <dt class="text-sm font-medium leading-6 text-gray-900">
-                                    <i18n.Translate>
-                                      Payment provider's account
-                                    </i18n.Translate>
-                                  </dt>
-                                  <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
-                                    {details.account.targetPath}
-                                  </dd>
-                                </div>
-                              );
-                            }
                             switch (details.account.targetType) {
-                              case "iban": {
+                              case undefined:
+                              case PaytoType.TalerReserveHttp:
+                              case PaytoType.TalerReserve: {
+                                // FIXME: support wire transfer to wallet
+                                return <div>not yet supported</div>;
+                              }
+                              case PaytoType.IBAN: {
                                 const name =
                                   details.account.params["receiver-name"];
                                 return (
@@ -254,7 +256,8 @@ export function WithdrawalConfirmationQuestion({
                                     <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
                                       <dt class="text-sm font-medium leading-6 text-gray-900">
                                         <i18n.Translate>
-                                          Payment provider's account number
+                                          Payment Service Provider's account
+                                          number
                                         </i18n.Translate>
                                       </dt>
                                       <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
@@ -265,7 +268,7 @@ export function WithdrawalConfirmationQuestion({
                                       <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
                                         <dt class="text-sm font-medium leading-6 text-gray-900">
                                           <i18n.Translate>
-                                            Payment provider's name
+                                            Payment Service Provider's name
                                           </i18n.Translate>
                                         </dt>
                                         <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
@@ -276,7 +279,7 @@ export function WithdrawalConfirmationQuestion({
                                   </Fragment>
                                 );
                               }
-                              case "x-taler-bank": {
+                              case PaytoType.TalerBank: {
                                 const name =
                                   details.account.params["receiver-name"];
                                 return (
@@ -284,8 +287,8 @@ export function WithdrawalConfirmationQuestion({
                                     <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
                                       <dt class="text-sm font-medium leading-6 text-gray-900">
                                         <i18n.Translate>
-                                          Payment provider's account bank
-                                          hostname
+                                          Payment Service Provider's account
+                                          bank hostname
                                         </i18n.Translate>
                                       </dt>
                                       <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
@@ -295,7 +298,7 @@ export function WithdrawalConfirmationQuestion({
                                     <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
                                       <dt class="text-sm font-medium leading-6 text-gray-900">
                                         <i18n.Translate>
-                                          Payment provider's account id
+                                          Payment Service Provider's account id
                                         </i18n.Translate>
                                       </dt>
                                       <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
@@ -306,7 +309,7 @@ export function WithdrawalConfirmationQuestion({
                                       <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
                                         <dt class="text-sm font-medium leading-6 text-gray-900">
                                           <i18n.Translate>
-                                            Payment provider's name
+                                            Payment Service Provider's name
                                           </i18n.Translate>
                                         </dt>
                                         <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
@@ -317,7 +320,7 @@ export function WithdrawalConfirmationQuestion({
                                   </Fragment>
                                 );
                               }
-                              case "bitcoin": {
+                              case PaytoType.Bitcoin: {
                                 const name =
                                   details.account.params["receiver-name"];
                                 return (
@@ -325,7 +328,8 @@ export function WithdrawalConfirmationQuestion({
                                     <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
                                       <dt class="text-sm font-medium leading-6 text-gray-900">
                                         <i18n.Translate>
-                                          Payment provider's account address
+                                          Payment Service Provider's account
+                                          address
                                         </i18n.Translate>
                                       </dt>
                                       <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
@@ -336,7 +340,38 @@ export function WithdrawalConfirmationQuestion({
                                       <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
                                         <dt class="text-sm font-medium leading-6 text-gray-900">
                                           <i18n.Translate>
-                                            Payment provider's name
+                                            Payment Service Provider's name
+                                          </i18n.Translate>
+                                        </dt>
+                                        <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
+                                          {name}
+                                        </dd>
+                                      </div>
+                                    )}
+                                  </Fragment>
+                                );
+                              }
+                              case PaytoType.Ethereum: {
+                                const name =
+                                  details.account.params["receiver-name"];
+                                return (
+                                  <Fragment>
+                                    <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
+                                      <dt class="text-sm font-medium leading-6 text-gray-900">
+                                        <i18n.Translate>
+                                          Payment Service Provider's account
+                                          address
+                                        </i18n.Translate>
+                                      </dt>
+                                      <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
+                                        {details.account.address}
+                                      </dd>
+                                    </div>
+                                    {name && (
+                                      <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
+                                        <dt class="text-sm font-medium leading-6 text-gray-900">
+                                          <i18n.Translate>
+                                            Payment Service Provider's name
                                           </i18n.Translate>
                                         </dt>
                                         <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
@@ -364,7 +399,7 @@ export function WithdrawalConfirmationQuestion({
                                 />
                               ) : (
                                 <i18n.Translate>
-                                  No amount specified yet.
+                                  No amount has yet been determined.
                                 </i18n.Translate>
                               )}
                             </dd>
@@ -392,25 +427,22 @@ export function WithdrawalConfirmationQuestion({
                   </div>
 
                   <div class="flex items-center justify-between gap-x-6 border-t border-gray-900/10 px-4 py-4 sm:px-8">
-                    <button
+                    <ButtonBetter
                       type="button"
                       name="cancel"
                       class="text-sm font-semibold leading-6 text-gray-900"
-                      onClick={doCancel}
+                      onClick={abortHandler}
                     >
                       <i18n.Translate>Cancel</i18n.Translate>
-                    </button>
-                    <button
+                    </ButtonBetter>
+                    <ButtonBetter
                       type="submit"
                       name="transfer"
                       class="disabled:opacity-50 disabled:cursor-default cursor-pointer rounded-md bg-indigo-600 px-3 py-2 text-sm font-semibold text-white shadow-sm hover:bg-indigo-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-indigo-600"
-                      onClick={(e) => {
-                        e.preventDefault();
-                        doTransfer();
-                      }}
+                      onClick={confirmHandler}
                     >
                       <i18n.Translate>Transfer</i18n.Translate>
-                    </button>
+                    </ButtonBetter>
                   </div>
                 </form>
               </div>
@@ -444,7 +476,7 @@ export function ShouldBeSameUser({
       <Fragment>
         <Attention
           type="warning"
-          title={i18n.str`This operation was created with other username`}
+          title={i18n.str`This operation was created with another username`}
         />
         <LoginForm currentUser={username} fixedUser />
       </Fragment>
