@@ -19,30 +19,30 @@ import {
   HttpStatusCode,
   TalerError,
   WithdrawUriResult,
-  assertUnreachable,
-  parsePaytoUri,
-  stringifyWithdrawUri,
+  assertUnreachable
 } from "@gnu-taler/taler-util";
 import {
   Attention,
+  ErrorLoading,
   Loading,
+  RouteDefinition,
   notifyInfo,
   useTranslationContext,
 } from "@gnu-taler/web-util/browser";
 import { VNode, h } from "preact";
-import { ErrorLoadingWithDebug } from "../components/ErrorLoadingWithDebug.js";
+
+import { Paytos, TalerUris } from "@gnu-taler/taler-util";
 import { useWithdrawalDetails } from "../hooks/account.js";
-import { RouteDefinition } from "@gnu-taler/web-util/browser";
 import { QrCodeSection } from "./QrCodeSection.js";
 import { WithdrawalConfirmationQuestion } from "./WithdrawalConfirmationQuestion.js";
 
+const TALER_SCREEN_ID = 116;
+
 interface Props {
   withdrawUri: WithdrawUriResult;
   origin: "from-bank-ui" | "from-wallet-ui";
   onOperationAborted: () => void;
   routeClose: RouteDefinition;
-  routeWithdrawalDetails: RouteDefinition<{ wopid: string }>;
-  onAuthorizationRequired: () => void;
 }
 /**
  * Offer the QR code (and a clickable taler://-link) to
@@ -54,8 +54,6 @@ export function WithdrawalQRCode({
   onOperationAborted,
   routeClose,
   origin,
-  routeWithdrawalDetails,
-  onAuthorizationRequired,
 }: Props): VNode {
   const { i18n } = useTranslationContext();
   const result = useWithdrawalDetails(withdrawUri.withdrawalOperationId);
@@ -64,7 +62,7 @@ export function WithdrawalQRCode({
     return <Loading />;
   }
   if (result instanceof TalerError) {
-    return <ErrorLoadingWithDebug error={result} />;
+    return <ErrorLoading error={result} />;
   }
   if (result.type === "fail") {
     switch (result.case) {
@@ -106,8 +104,9 @@ export function WithdrawalQRCode({
             <div class="mt-2">
               <p class="text-sm text-gray-500">
                 <i18n.Translate>
-                  The wire transfer to the payment provider's account was
-                  aborted from somewhere else, your balance was not affected.
+                  The wire transfer to the Payment Service Provider's account
+                  was aborted from somewhere else, your balance was not
+                  affected.
                 </i18n.Translate>
               </p>
             </div>
@@ -125,7 +124,7 @@ export function WithdrawalQRCode({
       </div>
     );
   }
-  const talerWithdrawUri = stringifyWithdrawUri(withdrawUri);
+  const talerWithdrawUri = TalerUris.toString(withdrawUri);
 
   if (data.status === "confirmed") {
     return (
@@ -157,9 +156,9 @@ export function WithdrawalQRCode({
             <div class="mt-2">
               <p class="text-sm text-gray-500">
                 <i18n.Translate>
-                  The wire transfer to the Taler operator has been initiated.
-                  You will soon receive the requested amount in your Taler
-                  wallet.
+                  The wire transfer to the Payment Service Provider has been
+                  initiated. You will shortly receive the requested amount in
+                  your Taler wallet.{" "}
                 </i18n.Translate>
               </p>
             </div>
@@ -173,13 +172,13 @@ export function WithdrawalQRCode({
           >
             <i18n.Translate>Close</i18n.Translate>
           </a>
-          {origin === "from-wallet-ui" ? (
+          {origin === "from-wallet-ui" && false ? (
             <a
               href={talerWithdrawUri}
               name="done"
               class="inline-flex  justify-center rounded-md bg-indigo-600 px-3 py-2 text-sm font-semibold text-white shadow-sm hover:bg-indigo-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-indigo-600"
             >
-              <i18n.Translate>Go to wallet</i18n.Translate>
+              <i18n.Translate>Go to your wallet now</i18n.Translate>
             </a>
           ) : undefined}
         </div>
@@ -192,7 +191,7 @@ export function WithdrawalQRCode({
       <QrCodeSection
         withdrawUri={withdrawUri}
         onAborted={() => {
-          notifyInfo(i18n.str`Operation canceled`);
+          notifyInfo(i18n.str`Operation aborted`);
           onOperationAborted();
         }}
       />
@@ -201,44 +200,43 @@ export function WithdrawalQRCode({
 
   const account = !data.selected_exchange_account
     ? undefined
-    : parsePaytoUri(data.selected_exchange_account);
+    : Paytos.fromString(data.selected_exchange_account);
 
-  if (!data.selected_reserve_pub && account) {
+  if (!account || account.type === "fail") {
+    if (!data.selected_reserve_pub) {
+      return (
+        <Attention
+          type="danger"
+          title={i18n.str`The operation is marked as selected, but a process during the withdrawal failed`}
+        >
+          <i18n.Translate>
+            A withdrawal reserve ID was not found and no account has been
+            selected.
+          </i18n.Translate>
+        </Attention>
+      );
+    }
     return (
       <Attention
         type="danger"
-        title={i18n.str`The operation is marked as 'selected' but some step in the withdrawal failed`}
+        title={i18n.str`The operation is marked as selected, but a process during the withdrawal failed`}
       >
         <i18n.Translate>
-          The account is selected but no withdrawal identification found.
+          There is a withdrawal reserve ID but no account has been selected or
+          the selected account is invalid.
         </i18n.Translate>
       </Attention>
     );
   }
 
-  if (!account && data.selected_reserve_pub) {
+  if (!data.selected_reserve_pub) {
     return (
       <Attention
         type="danger"
-        title={i18n.str`The operation is marked as 'selected' but some step in the withdrawal failed`}
+        title={i18n.str`The operation is marked as selected, but a process during the withdrawal failed`}
       >
         <i18n.Translate>
-          There is a withdrawal identification but no account has been selected
-          or the selected account is invalid.
-        </i18n.Translate>
-      </Attention>
-    );
-  }
-
-  if (!account || !data.selected_reserve_pub) {
-    return (
-      <Attention
-        type="danger"
-        title={i18n.str`The operation is marked as 'selected' but some step in the withdrawal failed`}
-      >
-        <i18n.Translate>
-          No withdrawal ID found and no account has been selected or the
-          selected account is invalid.
+          The account was selected, but no withdrawal reserve ID was found.
         </i18n.Translate>
       </Attention>
     );
@@ -247,16 +245,14 @@ export function WithdrawalQRCode({
   return (
     <WithdrawalConfirmationQuestion
       withdrawUri={withdrawUri}
-      routeHere={routeWithdrawalDetails}
       details={{
         username: data.username,
-        account,
+        account: account.body,
         reserve: data.selected_reserve_pub,
         amount: !data.amount ? undefined : Amounts.parseOrThrow(data.amount),
       }}
-      onAuthorizationRequired={onAuthorizationRequired}
       onAborted={() => {
-        notifyInfo(i18n.str`Operation canceled`);
+        notifyInfo(i18n.str`Operation aborted`);
         onOperationAborted();
       }}
     />
@@ -299,9 +295,9 @@ export function OperationNotFound({
           <div class="mt-2">
             <p class="text-sm text-gray-500">
               <i18n.Translate>
-                This operation is not known by the server. The operation id is
-                wrong or the server deleted the operation information before
-                reaching here.
+                This process is not known to the server. The process ID is
+                incorrect or the server has deleted the process information
+                before it arrived here.
               </i18n.Translate>
             </p>
           </div>
@@ -314,7 +310,7 @@ export function OperationNotFound({
             name="continue to dashboard"
             class="inline-flex w-full justify-center rounded-md bg-indigo-600 px-3 py-2 text-sm font-semibold text-white shadow-sm hover:bg-indigo-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-indigo-600"
           >
-            <i18n.Translate>Cotinue to dashboard</i18n.Translate>
+            <i18n.Translate>Continue to dashboard</i18n.Translate>
           </a>
         </div>
       )}
