@@ -21,25 +21,25 @@ import { ProfileNavigation } from "../ProfileNavigation.js";
 import { CreateCashout } from "../regional/CreateCashout.js";
 import { RouteDefinition } from "@gnu-taler/web-util/browser";
 
+const TALER_SCREEN_ID = 117;
+
 interface Props {
   account: string;
   routeClose: RouteDefinition;
-  onAuthorizationRequired: () => void;
+
   onCashout: () => void;
   routeCashoutDetails: RouteDefinition<{ cid: string }>;
   routeMyAccountDetails: RouteDefinition;
   routeMyAccountDelete: RouteDefinition;
   routeMyAccountPassword: RouteDefinition;
   routeMyAccountCashout: RouteDefinition;
-  routeCreateCashout: RouteDefinition;
   routeConversionConfig: RouteDefinition;
 }
 
 export function CashoutListForAccount({
   account,
-  onAuthorizationRequired,
+
   onCashout,
-  routeCreateCashout,
   routeCashoutDetails,
   routeMyAccountCashout,
   routeMyAccountDelete,
@@ -76,10 +76,8 @@ export function CashoutListForAccount({
 
       <CreateCashout
         focus
-        routeHere={routeCreateCashout}
         routeClose={routeClose}
         onCashout={onCashout}
-        onAuthorizationRequired={onAuthorizationRequired}
         account={account}
       />
 
