@@ -14,58 +14,60 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 import {
-  AbsoluteTime,
-  AccountLetter,
   HttpStatusCode,
   TalerCorebankApi,
   TalerError,
   TalerErrorCode,
-  TranslatedString,
-  assertUnreachable,
-  parsePaytoUri,
+  assertUnreachable
 } from "@gnu-taler/taler-util";
 import {
+  Attention,
+  ButtonBetter,
   CopyButton,
+  ErrorLoading,
   Loading,
   LocalNotificationBanner,
   RouteDefinition,
+  makeSafeCall,
   notifyInfo,
   useBankCoreApiContext,
-  useLocalNotification,
+  useChallengeHandler,
+  useLocalNotificationBetter,
   useTranslationContext,
 } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
 import { useState } from "preact/hooks";
-import { ErrorLoadingWithDebug } from "../../components/ErrorLoadingWithDebug.js";
+
+import { Paytos } from "@gnu-taler/taler-util";
 import { useAccountDetails } from "../../hooks/account.js";
-import { useBankState } from "../../hooks/bank-state.js";
-import { useSessionState } from "../../hooks/session.js";
+import { usePreferences } from "../../hooks/preferences.js";
+import { LoggedIn, useSessionState } from "../../hooks/session.js";
+import { AccountForm } from "../admin/AccountForm.js";
 import { LoginForm } from "../LoginForm.js";
 import { ProfileNavigation } from "../ProfileNavigation.js";
-import { AccountForm } from "../admin/AccountForm.js";
-import { usePreferences } from "../../hooks/preferences.js";
+import { SolveMFAChallenges } from "../SolveMFA.js";
+
+const TALER_SCREEN_ID = 118;
 
 export function ShowAccountDetails({
   account,
   routeClose,
   onUpdateSuccess,
-  onAuthorizationRequired,
+
   routeMyAccountCashout,
   routeMyAccountDelete,
   routeMyAccountDetails,
-  routeHere,
   routeMyAccountPassword,
   routeConversionConfig,
 }: {
   routeClose: RouteDefinition;
-  routeHere: RouteDefinition<{ account: string }>;
   routeMyAccountDetails: RouteDefinition;
   routeMyAccountDelete: RouteDefinition;
   routeMyAccountPassword: RouteDefinition;
   routeMyAccountCashout: RouteDefinition;
   routeConversionConfig: RouteDefinition;
   onUpdateSuccess: () => void;
-  onAuthorizationRequired: () => void;
+
   account: string;
 }): VNode {
   const { i18n } = useTranslationContext();
@@ -83,8 +85,8 @@ export function ShowAccountDetails({
   const [submitAccount, setSubmitAccount] = useState<
     TalerCorebankApi.AccountReconfiguration | undefined
   >();
-  const [notification, notify, handleError] = useLocalNotification();
-  const [, updateBankState] = useBankState();
+  const [notification, notifyOnError] = useLocalNotificationBetter();
+  const mfa = useChallengeHandler();
 
   const result = useAccountDetails(account);
   if (!result) {
@@ -93,7 +95,7 @@ export function ShowAccountDetails({
   if (result instanceof TalerError) {
     return (
       <Fragment>
-        <ErrorLoadingWithDebug error={result} />
+        <ErrorLoading error={result} />
         <LoginForm currentUser={account} />
       </Fragment>
     );
@@ -108,123 +110,77 @@ export function ShowAccountDetails({
     }
   }
 
-  async function doUpdate() {
-    if (!submitAccount || !creds) return;
-    await handleError(async () => {
-      const resp = await bank.updateAccount(
-        {
-          token: creds.token,
-          username: account,
+  const [doUpdate, repeatUpdate] = mfa.withMfaHandler(
+    ({ challengeIds, onChallengeRequired }) =>
+      makeSafeCall(
+        i18n,
+        (creds: LoggedIn, account: TalerCorebankApi.AccountReconfiguration) =>
+          bank.updateAccount(creds, account, { challengeIds }),
+        (success) => {
+          notifyInfo(i18n.str`Account updated`);
+          onUpdateSuccess();
         },
-        submitAccount,
-      );
+        (fail) => {
+          switch (fail.case) {
+            case HttpStatusCode.Unauthorized:
+              return i18n.str`The rights to change the account are not sufficient`;
+            case HttpStatusCode.NotFound:
+              return i18n.str`The username was not found`;
+            case TalerErrorCode.BANK_NON_ADMIN_PATCH_LEGAL_NAME:
+              return i18n.str`You can't change the legal name, please contact the your account administrator.`;
+            case TalerErrorCode.BANK_NON_ADMIN_PATCH_DEBT_LIMIT:
+              return i18n.str`You can't change the debt limit, please contact the your account administrator.`;
+            case TalerErrorCode.BANK_NON_ADMIN_PATCH_CASHOUT:
+              return i18n.str`You can't change the cashout address, please contact the your account administrator.`;
+            case TalerErrorCode.BANK_MISSING_TAN_INFO:
+              return i18n.str`No information for the selected authentication channel.`;
+            case HttpStatusCode.Accepted: {
+              onChallengeRequired(fail.body);
+              return i18n.str`A second factor authentication is required.`;
+            }
+            case TalerErrorCode.BANK_TAN_CHANNEL_NOT_SUPPORTED:
+              return i18n.str`Authentication channel is not supported.`;
+            case TalerErrorCode.BANK_NON_ADMIN_SET_CONVERSION_RATE_CLASS:
+              return i18n.str`Only the administrator can change the conversion rate.`;
+            case TalerErrorCode.BANK_CONVERSION_RATE_CLASS_UNKNOWN:
+              return i18n.str`The conversion rate class doesn't exist.`;
+            case TalerErrorCode.BANK_PASSWORD_TOO_SHORT:
+              return i18n.str`The password is too short. Can't have less than 8 characters.`;
+            case TalerErrorCode.BANK_PASSWORD_TOO_LONG:
+              return i18n.str`The password is too long. Can't have more than 64 characters.`;
+          }
+        },
+      ),
+  );
 
-      if (resp.type === "ok") {
-        notifyInfo(i18n.str`Account updated`);
-        onUpdateSuccess();
-      } else {
-        switch (resp.case) {
-          case HttpStatusCode.Unauthorized:
-            return notify({
-              type: "error",
-              title: i18n.str`The rights to change the account are not sufficient`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case HttpStatusCode.NotFound:
-            return notify({
-              type: "error",
-              title: i18n.str`The username was not found`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case TalerErrorCode.BANK_NON_ADMIN_PATCH_LEGAL_NAME:
-            return notify({
-              type: "error",
-              title: i18n.str`You can't change the legal name, please contact the your account administrator.`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case TalerErrorCode.BANK_NON_ADMIN_PATCH_DEBT_LIMIT:
-            return notify({
-              type: "error",
-              title: i18n.str`You can't change the debt limit, please contact the your account administrator.`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case TalerErrorCode.BANK_NON_ADMIN_PATCH_CASHOUT:
-            return notify({
-              type: "error",
-              title: i18n.str`You can't change the cashout address, please contact the your account administrator.`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case TalerErrorCode.BANK_MISSING_TAN_INFO:
-            return notify({
-              type: "error",
-              title: i18n.str`No information for the selected authentication channel.`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case HttpStatusCode.Accepted: {
-            updateBankState("currentChallenge", {
-              operation: "update-account",
-              id: String(resp.body.challenge_id),
-              location: routeHere.url({ account }),
-              sent: AbsoluteTime.never(),
-              request: submitAccount,
-            });
-            return onAuthorizationRequired();
-          }
-          case TalerErrorCode.BANK_TAN_CHANNEL_NOT_SUPPORTED: {
-            return notify({
-              type: "error",
-              title: i18n.str`Authentication channel is not supported.`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          }
-          case TalerErrorCode.BANK_NON_ADMIN_SET_MIN_CASHOUT: {
-            return notify({
-              type: "error",
-              title: i18n.str`Only the administrator can change the minimum cashout limit.`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          }
-          default:
-            assertUnreachable(resp);
-        }
-      }
-    });
-  }
+  const updateHandler =
+    !creds || !submitAccount
+      ? undefined
+      : () => notifyOnError(doUpdate)(creds, submitAccount);
 
   const url = bank.getRevenueAPI(account);
   const baseURL = url.href;
   const revenueURL = new URL(baseURL);
   revenueURL.username = account;
   revenueURL.password;
-  const ac = parsePaytoUri(result.body.payto_uri);
-  const payto = !ac?.isKnown ? undefined : ac;
-  const accountLetter: AccountLetter | undefined = !payto
-    ? undefined
-    : {
-        accountURI: result.body.payto_uri,
-        infoURL: revenueURL.href,
-        accountToken: creds?.token,
-      };
+  const ac = Paytos.fromString(result.body.payto_uri);
+  const payto = ac.type === "fail" || !ac.body.targetType ? undefined : ac.body;
+
+  if (mfa.pendingChallenge && repeatUpdate) {
+    return (
+      <SolveMFAChallenges
+        currentChallenge={mfa.pendingChallenge}
+        description={i18n.str`Update account information.`}
+        onCancel={mfa.doCancelChallenge}
+        username={account}
+        onCompleted={repeatUpdate}
+      />
+    );
+  }
 
   return (
     <Fragment>
-      <LocalNotificationBanner notification={notification} showDebug={true} />
+      <LocalNotificationBanner notification={notification} />
       {accountIsTheCurrentUser ? (
         <ProfileNavigation
           current="details"
@@ -240,6 +196,12 @@ export function ShowAccountDetails({
         </h1>
       )}
 
+      {result.body.status !== "deleted" ? undefined : (
+        <Attention title={i18n.str`Removed`} type="info">
+          <i18n.Translate>This account can't be used.</i18n.Translate>
+        </Attention>
+      )}
+
       <div class="grid grid-cols-1 gap-x-8 gap-y-8 pt-6 md:grid-cols-3 bg-gray-100 my-4 px-4 pb-4 rounded-lg">
         <div class="px-4 sm:px-0">
           <h2 class="text-base font-semibold leading-7 text-gray-900">
@@ -271,15 +233,15 @@ export function ShowAccountDetails({
             >
               <i18n.Translate>Cancel</i18n.Translate>
             </a>
-            <button
+            <ButtonBetter
               type="submit"
               name="update"
               class="disabled:opacity-50 disabled:cursor-default cursor-pointer rounded-md bg-indigo-600 px-3 py-2 text-sm font-semibold text-white shadow-sm hover:bg-indigo-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-indigo-600"
               disabled={!submitAccount}
-              onClick={doUpdate}
+              onClick={updateHandler}
             >
               <i18n.Translate>Update</i18n.Translate>
-            </button>
+            </ButtonBetter>
           </div>
         </AccountForm>
       </div>
@@ -472,6 +434,8 @@ export function ShowAccountDetails({
                         </div>
                       );
                     }
+                    default:
+                      return `unsupported account type ${payto.targetType}`;
                   }
                 })(payto)}
 
@@ -547,19 +511,6 @@ export function ShowAccountDetails({
                 <i18n.Translate>Cancel</i18n.Translate>
               </a>
               <span></span>
-
-              {!preferences.showCopyAccount ? (
-                <span />
-              ) : (
-                <CopyButton
-                  getContent={() =>
-                    !accountLetter ? "" : JSON.stringify(accountLetter)
-                  }
-                  class="flex text-center disabled:opacity-50 disabled:cursor-default cursor-pointer rounded-md bg-indigo-600 px-3 py-2 text-sm font-semibold text-white shadow-sm hover:bg-indigo-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-indigo-600"
-                >
-                  <i18n.Translate>Copy</i18n.Translate>
-                </CopyButton>
-              )}
             </div>
           </div>
         )}
