@@ -18,6 +18,7 @@ import {
   AmountString,
   Amounts,
   CurrencySpecification,
+  Duration,
   HttpStatusCode,
   TalerCorebankApi,
   TalerError,
@@ -25,28 +26,28 @@ import {
 } from "@gnu-taler/taler-util";
 import {
   Attention,
+  ErrorLoading,
   RouteDefinition,
   useBankCoreApiContext,
   useTranslationContext,
 } from "@gnu-taler/web-util/browser";
-import {
-  format,
-  sub
-} from "date-fns";
+import { format, sub } from "date-fns";
 import { Fragment, VNode, h } from "preact";
 import { useState } from "preact/hooks";
-import { ErrorLoadingWithDebug } from "../../components/ErrorLoadingWithDebug.js";
+
 import { Transactions } from "../../components/Transactions/index.js";
 import { useConversionInfo, useLastMonitorInfo } from "../../hooks/regional.js";
 import { RenderAmount } from "../PaytoWireTransferForm.js";
 import { WireTransfer } from "../WireTransfer.js";
 import { AccountList } from "./AccountList.js";
+import { ConversionClassList } from "./ConversionClassList.js";
+
+const TALER_SCREEN_ID = 122;
 
 /**
  * Query account information and show QR code if there is pending withdrawal
  */
 interface Props {
-  routeCreate: RouteDefinition;
   routeDownloadStats: RouteDefinition;
   routeCreateWireTransfer: RouteDefinition<{
     account?: string;
@@ -54,43 +55,51 @@ interface Props {
     amount?: string;
   }>;
 
-  routeShowAccount: RouteDefinition<{ account: string }>;
+  routeCreateAccount: RouteDefinition;
   routeRemoveAccount: RouteDefinition<{ account: string }>;
+  routeShowAccount: RouteDefinition<{ account: string }>;
   routeUpdatePasswordAccount: RouteDefinition<{ account: string }>;
   routeShowCashoutsAccount: RouteDefinition<{ account: string }>;
-  onAuthorizationRequired: () => void;
+
+  routeCreateConversionRateClass: RouteDefinition;
+  routeShowConversionRateClass: RouteDefinition<{ classId: string }>;
 }
 export function AdminHome({
-  routeCreate,
+  routeCreateAccount,
   routeRemoveAccount,
   routeShowAccount,
   routeUpdatePasswordAccount,
   routeDownloadStats,
   routeCreateWireTransfer,
-  onAuthorizationRequired,
+  routeCreateConversionRateClass,
+  routeShowConversionRateClass,
 }: Props): VNode {
+  const { config } = useBankCoreApiContext();
   return (
     <Fragment>
       <Metrics routeDownloadStats={routeDownloadStats} />
-      <WireTransfer
-        routeHere={routeCreateWireTransfer}
-        onAuthorizationRequired={onAuthorizationRequired}
-      />
+      <WireTransfer />
       <Transactions
         account="admin"
         routeCreateWireTransfer={routeCreateWireTransfer}
       />
       <AccountList
-        routeCreate={routeCreate}
+        routeCreate={routeCreateAccount}
         routeRemoveAccount={routeRemoveAccount}
         routeShowAccount={routeShowAccount}
         routeUpdatePasswordAccount={routeUpdatePasswordAccount}
       />
+      {!config.allow_conversion ? undefined : (
+        <ConversionClassList
+          routeCreate={routeCreateConversionRateClass}
+          routeShowDetails={routeShowConversionRateClass}
+        />
+      )}
     </Fragment>
   );
 }
 
-function getDateForTimeframe(
+function getDateForTimeframeStart(
   date: AbsoluteTime,
   timeframe: TalerCorebankApi.MonitorTimeframeParam,
   locale: Locale,
@@ -98,7 +107,7 @@ function getDateForTimeframe(
   if (date.t_ms === "never") return "--";
   switch (timeframe) {
     case TalerCorebankApi.MonitorTimeframeParam.hour:
-      return `${format(date.t_ms, "HH", { locale })}hs`;
+      return `${format(date.t_ms, "HH:00", { locale })}hs`;
     case TalerCorebankApi.MonitorTimeframeParam.day:
       return format(date.t_ms, "EEEE", { locale });
     case TalerCorebankApi.MonitorTimeframeParam.month:
@@ -111,6 +120,62 @@ function getDateForTimeframe(
   assertUnreachable(timeframe);
 }
 
+function getDateForTimeframeEnd(
+  date: AbsoluteTime,
+  timeframe: TalerCorebankApi.MonitorTimeframeParam,
+  locale: Locale,
+): string {
+  if (date.t_ms === "never") return "--";
+  switch (timeframe) {
+    case TalerCorebankApi.MonitorTimeframeParam.hour: {
+      const end = AbsoluteTime.addDuration(
+        date,
+        Duration.fromSpec({ hours: 1 }),
+      );
+      if (end.t_ms === "never")
+        throw Error(`abs time plus 1 hour duration can't be 'never'`);
+      return `${format(end.t_ms, "HH:00", { locale })}hs`;
+    }
+    case TalerCorebankApi.MonitorTimeframeParam.day: {
+      const end = AbsoluteTime.addDuration(
+        date,
+        Duration.fromSpec({ days: 1 }),
+      );
+      if (end.t_ms === "never")
+        throw Error(`abs time plus 1 day duration can't be 'never'`);
+      return format(end.t_ms, "EEEE", { locale });
+    }
+    case TalerCorebankApi.MonitorTimeframeParam.month: {
+      const end = AbsoluteTime.addDuration(
+        date,
+        Duration.fromSpec({ months: 1 }),
+      );
+      if (end.t_ms === "never")
+        throw Error(`abs time plus 1 month duration can't be 'never'`);
+      return format(end.t_ms, "MMMM", { locale });
+    }
+    case TalerCorebankApi.MonitorTimeframeParam.year: {
+      const end = AbsoluteTime.addDuration(
+        date,
+        Duration.fromSpec({ years: 1 }),
+      );
+      if (end.t_ms === "never")
+        throw Error(`abs time plus 1 year duration can't be 'never'`);
+      return format(end.t_ms, "yyyy", { locale });
+    }
+    case TalerCorebankApi.MonitorTimeframeParam.decade: {
+      const end = AbsoluteTime.addDuration(
+        date,
+        Duration.fromSpec({ years: 10 }),
+      );
+      if (end.t_ms === "never")
+        throw Error(`abs time plus 10 years duration can't be 'never'`);
+      return format(end.t_ms, "yyyy", { locale });
+    }
+  }
+  assertUnreachable(timeframe);
+}
+
 export function getTimeframesForDate(
   time: Date,
   timeframe: TalerCorebankApi.MonitorTimeframeParam,
@@ -131,7 +196,7 @@ export function getTimeframesForDate(
           sub(time, { days: 1 }).getTime(),
         ),
         previous: AbsoluteTime.fromMilliseconds(
-          sub(time, { days: 4 }).getTime(),
+          sub(time, { days: 2 }).getTime(),
         ),
       };
     case TalerCorebankApi.MonitorTimeframeParam.month:
@@ -183,26 +248,25 @@ function Metrics({
   const resp = useLastMonitorInfo(params.current, params.previous, metricType);
   if (!resp) return <Fragment />;
   if (resp instanceof TalerError) {
-    return <ErrorLoadingWithDebug error={resp} />;
+    return <ErrorLoading error={resp} />;
   }
-  if (!respInfo) return <Fragment />;
-  if (respInfo instanceof TalerError) {
-    return <ErrorLoadingWithDebug error={respInfo} />;
+  if (respInfo && respInfo instanceof TalerError) {
+    return <ErrorLoading error={respInfo} />;
   }
-  if (respInfo.type === "fail") {
+  if (respInfo && respInfo.type === "fail") {
     switch (respInfo.case) {
       case HttpStatusCode.NotImplemented: {
         return (
-          <Attention type="danger" title={i18n.str`Cashout are disabled`}>
+          <Attention type="danger" title={i18n.str`Cashout is disabled`}>
             <i18n.Translate>
-              Cashout should be enable by configuration and the conversion rate
-              should be initialized with fee, ratio and rounding mode.
+              Cashout should be enabled in the configuration, the conversion
+              rate should be initialized with fee(s), rates and a rounding mode.
             </i18n.Translate>
           </Attention>
         );
       }
       default: {
-        assertUnreachable(respInfo.case);
+        assertUnreachable(respInfo);
       }
     }
   }
@@ -277,8 +341,10 @@ function Metrics({
           class="block w-full rounded-md border-gray-300 focus:border-indigo-500 focus:ring-indigo-500"
           onChange={(e) => {
             setMetricType(
-              parseInt(e.currentTarget
-                .value, 10) as TalerCorebankApi.MonitorTimeframeParam,
+              parseInt(
+                e.currentTarget.value,
+                10,
+              ) as TalerCorebankApi.MonitorTimeframeParam,
             );
           }}
         >
@@ -413,19 +479,20 @@ function Metrics({
 
       <div class="w-full flex justify-between">
         <h1 class="text-base text-gray-900 mt-5">
-          {i18n.str`Trading volume on ${getDateForTimeframe(
+          {i18n.str`Trading volume from ${getDateForTimeframeStart(
             params.current,
             metricType,
             dateLocale,
-          )} compared to ${getDateForTimeframe(
-            params.previous,
+          )} to ${getDateForTimeframeEnd(
+            params.current,
             metricType,
             dateLocale,
           )}`}
         </h1>
       </div>
       <dl class="mt-5 grid grid-cols-1 md:grid-cols-2  divide-y divide-gray-200 overflow-hidden rounded-lg bg-white shadow-lg md:divide-x md:divide-y-0">
-        {resp.current.body.type !== "with-conversions" ||
+        {!respInfo ||
+        resp.current.body.type !== "with-conversions" ||
         resp.previous.body.type !== "with-conversions" ? undefined : (
           <Fragment>
             <div class="px-4 py-5 sm:p-6">
@@ -438,7 +505,7 @@ function Metrics({
                   </i18n.Translate>
                 </div>
               </dt>
-              <MetricValue
+              <MetricValueAmount
                 current={resp.current.body.cashinFiatVolume}
                 previous={resp.previous.body.cashinFiatVolume}
                 spec={respInfo.body.fiat_currency_specification}
@@ -454,7 +521,7 @@ function Metrics({
                   account.
                 </i18n.Translate>
               </div>
-              <MetricValue
+              <MetricValueAmount
                 current={resp.current.body.cashoutFiatVolume}
                 previous={resp.previous.body.cashoutFiatVolume}
                 spec={respInfo.body.fiat_currency_specification}
@@ -471,7 +538,7 @@ function Metrics({
               </i18n.Translate>
             </div>
           </dt>
-          <MetricValue
+          <MetricValueAmount
             current={resp.current.body.talerInVolume}
             previous={resp.previous.body.talerInVolume}
             spec={config.currency_specification}
@@ -486,12 +553,40 @@ function Metrics({
               </i18n.Translate>
             </div>
           </dt>
-          <MetricValue
+          <MetricValueAmount
             current={resp.current.body.talerOutVolume}
             previous={resp.previous.body.talerOutVolume}
             spec={config.currency_specification}
           />
         </div>
+        <div class="px-4 py-5 sm:p-6">
+          <dt class="text-base font-normal text-gray-900">
+            <i18n.Translate>Payin</i18n.Translate>
+            <div class="text-xs text-gray-500">
+              <i18n.Translate>
+                Transferred from an account to a Taler exchange.
+              </i18n.Translate>
+            </div>
+          </dt>
+          <MetricValueNumber
+            current={resp.current.body.talerInCount}
+            previous={resp.previous.body.talerInCount}
+          />
+        </div>
+        <div class="px-4 py-5 sm:p-6">
+          <dt class="text-base font-normal text-gray-900">
+            <i18n.Translate>Payout</i18n.Translate>
+            <div class="text-xs text-gray-500">
+              <i18n.Translate>
+                Transferred from a Taler exchange to another account.
+              </i18n.Translate>
+            </div>
+          </dt>
+          <MetricValueNumber
+            current={resp.current.body.talerOutCount}
+            previous={resp.previous.body.talerOutCount}
+          />
+        </div>
       </dl>
       <div class="flex justify-end mt-4">
         <a
@@ -506,7 +601,7 @@ function Metrics({
   );
 }
 
-function MetricValue({
+function MetricValueAmount({
   current,
   previous,
   spec,
@@ -554,7 +649,7 @@ function MetricValue({
         <div class="flex flex-col">
           <div class="flex justify-end items-baseline text-2xl font-semibold text-indigo-600">
             <small class="ml-2 text-sm font-medium text-gray-500">
-              <i18n.Translate>from</i18n.Translate>{" "}
+              <i18n.Translate>previous</i18n.Translate>{" "}
               {!previous ? (
                 "-"
               ) : (
@@ -620,3 +715,93 @@ function MetricValue({
     </Fragment>
   );
 }
+
+function MetricValueNumber({
+  current,
+  previous,
+}: {
+  current: number | undefined;
+  previous: number | undefined;
+}): VNode {
+  const { i18n } = useTranslationContext();
+
+  const cmp = current && previous ? (current < previous ? -1 : 1) : 0;
+
+  const rate =
+    !current || Number.isNaN(current) || !previous || Number.isNaN(previous)
+      ? 0
+      : cmp === -1
+        ? 1 - Math.round(current) / Math.round(previous)
+        : cmp === 1
+          ? Math.round(current) / Math.round(previous) - 1
+          : 0;
+
+  const negative = cmp === 0 ? undefined : cmp === -1;
+  const rateStr = `${(Math.abs(rate) * 100).toFixed(2)}%`;
+  return (
+    <Fragment>
+      <dd class="mt-1 block ">
+        <div class="flex justify-start text-2xl items-baseline font-semibold text-indigo-600">
+          {!current ? "-" : current}
+        </div>
+        <div class="flex flex-col">
+          <div class="flex justify-end items-baseline text-2xl font-semibold text-indigo-600">
+            <small class="ml-2 text-sm font-medium text-gray-500">
+              <i18n.Translate>previous</i18n.Translate>{" "}
+              {!previous ? "-" : previous}
+            </small>
+          </div>
+          {!!rate && (
+            <span
+              data-negative={negative}
+              class="flex items-center gap-x-1.5 w-fit rounded-md bg-green-100 text-green-800 data-[negative=true]:bg-red-100 px-2 py-1 text-xs font-medium data-[negative=true]:text-red-700 whitespace-pre"
+            >
+              {negative ? (
+                <svg
+                  xmlns="http://www.w3.org/2000/svg"
+                  fill="none"
+                  viewBox="0 0 24 24"
+                  stroke-width="1.5"
+                  stroke="currentColor"
+                  class="w-6 h-6"
+                >
+                  <path
+                    stroke-linecap="round"
+                    stroke-linejoin="round"
+                    d="M12 4.5v15m0 0l6.75-6.75M12 19.5l-6.75-6.75"
+                  />
+                </svg>
+              ) : (
+                <svg
+                  xmlns="http://www.w3.org/2000/svg"
+                  fill="none"
+                  viewBox="0 0 24 24"
+                  stroke-width="1.5"
+                  stroke="currentColor"
+                  class="w-6 h-6"
+                >
+                  <path
+                    stroke-linecap="round"
+                    stroke-linejoin="round"
+                    d="M12 19.5v-15m0 0l-6.75 6.75M12 4.5l6.75 6.75"
+                  />
+                </svg>
+              )}
+
+              {negative ? (
+                <span class="sr-only">
+                  <i18n.Translate>Decreased by</i18n.Translate>
+                </span>
+              ) : (
+                <span class="sr-only">
+                  <i18n.Translate>Increased by</i18n.Translate>
+                </span>
+              )}
+              {rateStr}
+            </span>
+          )}
+        </div>
+      </dd>
+    </Fragment>
+  );
+}
