@@ -35,6 +35,8 @@ import { useSessionState } from "../../hooks/session.js";
 import { RouteDefinition } from "@gnu-taler/web-util/browser";
 import { AccountForm } from "./AccountForm.js";
 
+const TALER_SCREEN_ID = 123;
+
 export function CreateNewAccount({
   routeCancel,
   onCreateSuccess,
@@ -70,7 +72,7 @@ export function CreateNewAccount({
             return notify({
               type: "error",
               title: i18n.str`Server replied that phone or email is invalid`,
-              description: resp.detail.hint as TranslatedString,
+              description: resp.detail?.hint as TranslatedString,
               debug: resp.detail,
               when: AbsoluteTime.now(),
             });
@@ -78,7 +80,7 @@ export function CreateNewAccount({
             return notify({
               type: "error",
               title: i18n.str`The rights to perform the operation are not sufficient`,
-              description: resp.detail.hint as TranslatedString,
+              description: resp.detail?.hint as TranslatedString,
               debug: resp.detail,
               when: AbsoluteTime.now(),
             });
@@ -86,15 +88,15 @@ export function CreateNewAccount({
             return notify({
               type: "error",
               title: i18n.str`Account username is already taken`,
-              description: resp.detail.hint as TranslatedString,
+              description: resp.detail?.hint as TranslatedString,
               debug: resp.detail,
               when: AbsoluteTime.now(),
             });
           case TalerErrorCode.BANK_REGISTER_PAYTO_URI_REUSE:
             return notify({
               type: "error",
-              title: i18n.str`Account id is already taken`,
-              description: resp.detail.hint as TranslatedString,
+              title: i18n.str`Account ID is already taken`,
+              description: resp.detail?.hint as TranslatedString,
               debug: resp.detail,
               when: AbsoluteTime.now(),
             });
@@ -102,7 +104,7 @@ export function CreateNewAccount({
             return notify({
               type: "error",
               title: i18n.str`Bank ran out of bonus credit.`,
-              description: resp.detail.hint as TranslatedString,
+              description: resp.detail?.hint as TranslatedString,
               debug: resp.detail,
               when: AbsoluteTime.now(),
             });
@@ -110,15 +112,15 @@ export function CreateNewAccount({
             return notify({
               type: "error",
               title: i18n.str`Account username can't be used because is reserved`,
-              description: resp.detail.hint as TranslatedString,
+              description: resp.detail?.hint as TranslatedString,
               debug: resp.detail,
               when: AbsoluteTime.now(),
             });
           case TalerErrorCode.BANK_NON_ADMIN_PATCH_DEBT_LIMIT:
             return notify({
               type: "error",
-              title: i18n.str`Only admin is allow to set debt limit.`,
-              description: resp.detail.hint as TranslatedString,
+              title: i18n.str`Only an administrator is allowed to set the debt limit.`,
+              description: resp.detail?.hint as TranslatedString,
               debug: resp.detail,
               when: AbsoluteTime.now(),
             });
@@ -126,7 +128,7 @@ export function CreateNewAccount({
             return notify({
               type: "error",
               title: i18n.str`No information for the selected authentication channel.`,
-              description: resp.detail.hint as TranslatedString,
+              description: resp.detail?.hint as TranslatedString,
               debug: resp.detail,
               when: AbsoluteTime.now(),
             });
@@ -134,7 +136,7 @@ export function CreateNewAccount({
             return notify({
               type: "error",
               title: i18n.str`Authentication channel is not supported.`,
-              description: resp.detail.hint as TranslatedString,
+              description: resp.detail?.hint as TranslatedString,
               debug: resp.detail,
               when: AbsoluteTime.now(),
             });
@@ -142,15 +144,42 @@ export function CreateNewAccount({
             return notify({
               type: "error",
               title: i18n.str`Only admin can create accounts with second factor authentication.`,
-              description: resp.detail.hint as TranslatedString,
+              description: resp.detail?.hint as TranslatedString,
               debug: resp.detail,
               when: AbsoluteTime.now(),
             });
-          case TalerErrorCode.BANK_NON_ADMIN_SET_MIN_CASHOUT: {
+          case TalerErrorCode.BANK_NON_ADMIN_SET_CONVERSION_RATE_CLASS: {
             return notify({
               type: "error",
-              title: i18n.str`Only the administrator can change the minimum cashout limit.`,
-              description: resp.detail.hint as TranslatedString,
+              title: i18n.str`Only the administrator can change the conversion rate.`,
+              description: resp.detail?.hint as TranslatedString,
+              debug: resp.detail,
+              when: AbsoluteTime.now(),
+            });
+          }
+          case TalerErrorCode.BANK_CONVERSION_RATE_CLASS_UNKNOWN: {
+            return notify({
+              type: "error",
+              title: i18n.str`The conversion rate class doesn't exist.`,
+              description: resp.detail?.hint as TranslatedString,
+              debug: resp.detail,
+              when: AbsoluteTime.now(),
+            });
+          }
+          case TalerErrorCode.BANK_PASSWORD_TOO_SHORT: {
+            return notify({
+              type: "error",
+              title: i18n.str`The password is too short. Can't have less than 8 characters.`,
+              description: resp.detail?.hint as TranslatedString,
+              debug: resp.detail,
+              when: AbsoluteTime.now(),
+            });
+          }
+          case TalerErrorCode.BANK_PASSWORD_TOO_LONG: {
+            return notify({
+              type: "error",
+              title: i18n.str`The password is too long. Can't have more than 64 characters.`,
+              description: resp.detail?.hint as TranslatedString,
               debug: resp.detail,
               when: AbsoluteTime.now(),
             });
