@@ -34,6 +34,8 @@ import { useSessionState } from "../../hooks/session.js";
 import { RouteDefinition } from "@gnu-taler/web-util/browser";
 import { getTimeframesForDate } from "./AdminHome.js";
 
+const TALER_SCREEN_ID = 124;
+
 interface Props {
   routeCancel: RouteDefinition;
 }
@@ -383,10 +385,9 @@ export function DownloadStats({ routeCancel }: Props): VNode {
         <div>
           <div class="relative mb-5 h-5 rounded-full bg-gray-200">
             <div
-              class="h-full animate-pulse rounded-full bg-blue-500"
-              style={{
-                width: `${Math.round((lastStep.step / lastStep.total) * 100)}%`,
-              }}
+              class={`h-full animate-pulse rounded-full bg-blue-500 w-[${Math.round(
+                (lastStep.step / lastStep.total) * 100,
+              )}%]`}
             >
               <span class="absolute inset-0 flex items-center justify-center text-xs font-semibold text-white">
                 <i18n.Translate>
@@ -468,7 +469,7 @@ async function fetchAllStatus(
         : undefined;
 
       if (previous && previous.type === "fail" && options.endOnFirstFail) {
-        throw TalerError.fromUncheckedDetail(previous.detail);
+        return accumulatedMap; //skip
       }
 
       const current = await api.getMonitor(token, {
@@ -477,7 +478,7 @@ async function fetchAllStatus(
       });
 
       if (current.type === "fail" && options.endOnFirstFail) {
-        throw TalerError.fromUncheckedDetail(current.detail);
+        return accumulatedMap; //skip
       }
 
       const metricName =
