@@ -14,46 +14,48 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 import {
-  AbsoluteTime,
   Amounts,
   HttpStatusCode,
   TalerError,
   TalerErrorCode,
-  TranslatedString,
   assertUnreachable,
 } from "@gnu-taler/taler-util";
 import {
   Attention,
+  ButtonBetter,
+  ErrorLoading,
   Loading,
   LocalNotificationBanner,
+  RouteDefinition,
   ShowInputErrorLabel,
+  makeSafeCall,
   notifyInfo,
-  useLocalNotification,
+  useBankCoreApiContext,
+  useChallengeHandler,
+  useLocalNotificationBetter,
   useTranslationContext,
 } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
 import { useState } from "preact/hooks";
-import { ErrorLoadingWithDebug } from "../../components/ErrorLoadingWithDebug.js";
-import { useBankCoreApiContext } from "@gnu-taler/web-util/browser";
+
 import { useAccountDetails } from "../../hooks/account.js";
 import { useSessionState } from "../../hooks/session.js";
 import { undefinedIfEmpty } from "../../utils.js";
 import { LoginForm } from "../LoginForm.js";
 import { doAutoFocus } from "../PaytoWireTransferForm.js";
-import { useBankState } from "../../hooks/bank-state.js";
-import { RouteDefinition } from "@gnu-taler/web-util/browser";
+import { SolveMFAChallenges } from "../SolveMFA.js";
+
+const TALER_SCREEN_ID = 125;
 
 export function RemoveAccount({
   account,
   routeCancel,
   onUpdateSuccess,
-  onAuthorizationRequired,
+
   focus,
-  routeHere,
 }: {
   focus?: boolean;
-  routeHere: RouteDefinition<{ account: string }>;
-  onAuthorizationRequired: () => void;
+
   routeCancel: RouteDefinition;
   onUpdateSuccess: () => void;
   account: string;
@@ -67,8 +69,8 @@ export function RemoveAccount({
   const {
     lib: { bank: api },
   } = useBankCoreApiContext();
-  const [notification, notify, handleError] = useLocalNotification();
-  const [, updateBankState] = useBankState();
+  const [notification, notifyOnError] = useLocalNotificationBetter();
+  const mfa = useChallengeHandler();
 
   if (!result) {
     return <Loading />;
@@ -76,7 +78,7 @@ export function RemoveAccount({
   if (result instanceof TalerError) {
     return (
       <Fragment>
-        <ErrorLoadingWithDebug error={result} />
+        <ErrorLoading error={result} />
         <LoginForm currentUser={account} />
       </Fragment>
     );
@@ -119,65 +121,6 @@ export function RemoveAccount({
     );
   }
 
-  async function doRemove() {
-    if (!token) return;
-    await handleError(async () => {
-      const resp = await api.deleteAccount({ username: account, token });
-      if (resp.type === "ok") {
-        notifyInfo(i18n.str`Account removed`);
-        onUpdateSuccess();
-      } else {
-        switch (resp.case) {
-          case HttpStatusCode.Unauthorized:
-            return notify({
-              type: "error",
-              title: i18n.str`No enough permission to delete the account.`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case HttpStatusCode.NotFound:
-            return notify({
-              type: "error",
-              title: i18n.str`The username was not found.`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case TalerErrorCode.BANK_RESERVED_USERNAME_CONFLICT:
-            return notify({
-              type: "error",
-              title: i18n.str`Can't delete a reserved username.`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case TalerErrorCode.BANK_ACCOUNT_BALANCE_NOT_ZERO:
-            return notify({
-              type: "error",
-              title: i18n.str`Can't delete an account with balance different than zero.`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case HttpStatusCode.Accepted: {
-            updateBankState("currentChallenge", {
-              operation: "delete-account",
-              id: String(resp.body.challenge_id),
-              sent: AbsoluteTime.never(),
-              location: routeHere.url({ account }),
-              request: account,
-            });
-            return onAuthorizationRequired();
-          }
-          default: {
-            assertUnreachable(resp);
-          }
-        }
-      }
-    });
-  }
-
   const errors = undefinedIfEmpty({
     accountName: !accountName
       ? i18n.str`Required`
@@ -186,6 +129,49 @@ export function RemoveAccount({
         : undefined,
   });
 
+  const [doDelete, repeatDelete] =
+    !token || !!errors
+      ? [undefined, undefined]
+      : mfa.withMfaHandler(({ challengeIds, onChallengeRequired }) =>
+          makeSafeCall(
+            i18n,
+            () =>
+              api.deleteAccount({ username: account, token }, { challengeIds }),
+            (success) => {
+              notifyInfo(i18n.str`Account removed`);
+              onUpdateSuccess();
+            },
+            (fail) => {
+              switch (fail.case) {
+                case HttpStatusCode.Unauthorized:
+                  return i18n.str`No enough permission to delete the account.`;
+                case HttpStatusCode.NotFound:
+                  return i18n.str`The username was not found.`;
+                case TalerErrorCode.BANK_RESERVED_USERNAME_CONFLICT:
+                  return i18n.str`Can't delete a reserved username.`;
+                case TalerErrorCode.BANK_ACCOUNT_BALANCE_NOT_ZERO:
+                  return i18n.str`Can't delete an account with balance different than zero.`;
+                case HttpStatusCode.Accepted: {
+                  onChallengeRequired(fail.body);
+                  return i18n.str`A second factor authentication is required.`;
+                }
+              }
+            },
+          ),
+        );
+
+  if (mfa.pendingChallenge && repeatDelete) {
+    return (
+      <SolveMFAChallenges
+        currentChallenge={mfa.pendingChallenge}
+        description={i18n.str`Remove account.`}
+        username={account}
+        onCancel={mfa.doCancelChallenge}
+        onCompleted={repeatDelete}
+      />
+    );
+  }
+
   return (
     <div>
       <LocalNotificationBanner notification={notification} />
@@ -258,18 +244,14 @@ export function RemoveAccount({
             >
               <i18n.Translate>Cancel</i18n.Translate>
             </a>
-            <button
+            <ButtonBetter
               type="submit"
               name="delete"
               class="disabled:opacity-50 disabled:cursor-default cursor-pointer rounded-md bg-red-600 px-3 py-2 text-sm font-semibold text-white shadow-sm hover:bg-red-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-red-600"
-              disabled={!!errors}
-              onClick={(e) => {
-                e.preventDefault();
-                doRemove();
-              }}
+              onClick={!doDelete ? undefined : notifyOnError(doDelete)}
             >
               <i18n.Translate>Delete</i18n.Translate>
-            </button>
+            </ButtonBetter>
           </div>
         </form>
       </div>
