@@ -26,27 +26,19 @@ import {
 } from "@gnu-taler/taler-util";
 import {
   Attention,
+  ErrorLoading,
   InternationalizationAPI,
+  Loading,
   LocalNotificationBanner,
+  RouteDefinition,
   ShowInputErrorLabel,
+  useBankCoreApiContext,
   useLocalNotification,
   useTranslationContext,
   utils,
 } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
 import { useEffect, useState } from "preact/hooks";
-import { useBankCoreApiContext } from "@gnu-taler/web-util/browser";
-import { useSessionState } from "../../hooks/session.js";
-import {
-  TransferCalculation,
-  useCashinEstimator,
-  useCashoutEstimator,
-  useConversionInfo,
-} from "../../hooks/regional.js";
-import { RouteDefinition } from "@gnu-taler/web-util/browser";
-import { undefinedIfEmpty } from "../../utils.js";
-import { InputAmount, RenderAmount } from "../PaytoWireTransferForm.js";
-import { ProfileNavigation } from "../ProfileNavigation.js";
 import {
   FormErrors,
   FormStatus,
@@ -55,6 +47,20 @@ import {
   UIField,
   useFormState,
 } from "../../hooks/form.js";
+import {
+  TransferCalculation,
+  useCashinEstimator,
+  useCashoutEstimator,
+  useConversionInfo,
+} from "../../hooks/regional.js";
+import { useSessionState } from "../../hooks/session.js";
+import { undefinedIfEmpty } from "../../utils.js";
+import { InputAmount, RenderAmount } from "../PaytoWireTransferForm.js";
+import { ProfileNavigation } from "../ProfileNavigation.js";
+
+import { DescribeConversion } from "../admin/ConversionClassList.js";
+
+const TALER_SCREEN_ID = 126;
 
 interface Props {
   routeMyAccountDetails: RouteDefinition;
@@ -81,29 +87,43 @@ function useComponentState({
 }: Props): utils.RecursiveState<VNode> {
   const { i18n } = useTranslationContext();
 
-  const result = useConversionInfo();
-  const info =
-    result && !(result instanceof TalerError) && result.type === "ok"
-      ? result.body
-      : undefined;
-
   const { state: credentials } = useSessionState();
   const creds =
     credentials.status !== "loggedIn" || !credentials.isUserAdministrator
       ? undefined
       : credentials;
 
-  if (!info) {
-    return <i18n.Translate>loading...</i18n.Translate>;
-  }
-
   if (!creds) {
     return <i18n.Translate>only admin can setup conversion</i18n.Translate>;
   }
 
-  return function afterComponentLoads() {
-    const { i18n } = useTranslationContext();
+  const resp = useConversionInfo();
+  if (!resp) {
+    return <Loading />;
+  }
+  if (resp instanceof TalerError) {
+    return <ErrorLoading error={resp} />;
+  }
 
+  if (resp.type !== "ok") {
+    switch (resp.case) {
+      case HttpStatusCode.NotImplemented: {
+        return (
+          <Attention type="danger" title={i18n.str`Cashout is disabled`}>
+            <i18n.Translate>
+              Cashout should be enabled in the configuration, the conversion
+              rate should be initialized with fee(s), rates and a rounding mode.
+            </i18n.Translate>
+          </Attention>
+        );
+      }
+      default:
+        assertUnreachable(resp);
+    }
+  }
+  const info = resp.body;
+
+  return function afterComponentLoads() {
     const {
       lib: { conversion },
     } = useBankCoreApiContext();
@@ -114,18 +134,18 @@ function useComponentState({
       amount: "100",
       conv: {
         cashin_min_amount: info.conversion_rate.cashin_min_amount.split(":")[1],
-        cashin_tiny_amount:
-          info.conversion_rate.cashin_tiny_amount.split(":")[1],
         cashin_fee: info.conversion_rate.cashin_fee.split(":")[1],
         cashin_ratio: info.conversion_rate.cashin_ratio,
         cashin_rounding_mode: info.conversion_rate.cashin_rounding_mode,
+        cashin_tiny_amount:
+          info.conversion_rate.cashin_tiny_amount.split(":")[1],
         cashout_min_amount:
           info.conversion_rate.cashout_min_amount.split(":")[1],
-        cashout_tiny_amount:
-          info.conversion_rate.cashout_tiny_amount.split(":")[1],
         cashout_fee: info.conversion_rate.cashout_fee.split(":")[1],
         cashout_ratio: info.conversion_rate.cashout_ratio,
         cashout_rounding_mode: info.conversion_rate.cashout_rounding_mode,
+        cashout_tiny_amount:
+          info.conversion_rate.cashout_tiny_amount.split(":")[1],
       },
     };
 
@@ -153,9 +173,16 @@ function useComponentState({
             `${info.fiat_currency}:${form.amount.value}`,
           );
           const in_fee = Amounts.parseOrThrow(info.conversion_rate.cashin_fee);
-          const cashin = await calculateCashinFromDebit(in_amount, in_fee);
+          const respCashin = await calculateCashinFromDebit(in_amount, in_fee);
 
-          if (cashin === "amount-is-too-small") {
+          const cashin =
+            respCashin.type === "ok"
+              ? respCashin.body
+              : respCashin.case === HttpStatusCode.Conflict
+                ? ("amount-is-too-small" as const)
+                : undefined;
+
+          if (!cashin || cashin === "amount-is-too-small") {
             setCalc(undefined);
             return;
           }
@@ -163,11 +190,23 @@ function useComponentState({
           const out_fee = Amounts.parseOrThrow(
             info.conversion_rate.cashout_fee,
           );
-          const cashout = await calculateCashoutFromDebit(
+          const respCashout = await calculateCashoutFromDebit(
             cashin.credit,
             out_fee,
           );
 
+          const cashout =
+            respCashout.type === "ok"
+              ? respCashout.body
+              : respCashout.case === HttpStatusCode.Conflict
+                ? ("amount-is-too-small" as const)
+                : undefined;
+                
+          if (!cashout) {
+            setCalc(undefined); // silent failure
+            return;
+          }
+
           setCalc({ cashin, cashout });
         });
       }
@@ -205,7 +244,7 @@ function useComponentState({
               return notify({
                 type: "error",
                 title: i18n.str`Wrong credentials`,
-                description: resp.detail.hint as TranslatedString,
+                description: resp.detail?.hint as TranslatedString,
                 debug: resp.detail,
                 when: AbsoluteTime.now(),
               });
@@ -214,7 +253,7 @@ function useComponentState({
               return notify({
                 type: "error",
                 title: i18n.str`Conversion is disabled`,
-                description: resp.detail.hint as TranslatedString,
+                description: resp.detail?.hint as TranslatedString,
                 debug: resp.detail,
                 when: AbsoluteTime.now(),
               });
@@ -364,10 +403,17 @@ function useComponentState({
                 <div class="px-6 pt-6">
                   <div class="justify-between items-center flex ">
                     <dt class="text-sm text-gray-600">
-                      <i18n.Translate>Cashin ratio</i18n.Translate>
+                      <i18n.Translate>Cashin</i18n.Translate>
                     </dt>
                     <dd class="text-sm text-gray-900">
-                      {info.conversion_rate.cashin_ratio}
+                      <DescribeConversion
+                        ratio={info.conversion_rate.cashin_ratio}
+                        fee={info.conversion_rate.cashin_fee}
+                        min={info.conversion_rate.cashin_min_amount}
+                        rounding={info.conversion_rate.cashin_rounding_mode}
+                        minSpec={info.fiat_currency_specification}
+                        feeSpec={info.regional_currency_specification}
+                      />
                     </dd>
                   </div>
                 </div>
@@ -375,10 +421,17 @@ function useComponentState({
                 <div class="px-6 pt-6">
                   <div class="justify-between items-center flex ">
                     <dt class="text-sm text-gray-600">
-                      <i18n.Translate>Cashout ratio</i18n.Translate>
+                      <i18n.Translate>Cashout</i18n.Translate>
                     </dt>
                     <dd class="text-sm text-gray-900">
-                      {info.conversion_rate.cashout_ratio}
+                      <DescribeConversion
+                        ratio={info.conversion_rate.cashout_ratio}
+                        fee={info.conversion_rate.cashout_fee}
+                        min={info.conversion_rate.cashout_min_amount}
+                        rounding={info.conversion_rate.cashout_rounding_mode}
+                        minSpec={info.regional_currency_specification}
+                        feeSpec={info.fiat_currency_specification}
+                      />
                     </dd>
                   </div>
                 </div>
@@ -437,7 +490,7 @@ function useComponentState({
                               value={cashinCalc.debit}
                               negative
                               withColor
-                              spec={info.regional_currency_specification}
+                              spec={info.fiat_currency_specification}
                             />
                           </dd>
                         </div>
@@ -452,7 +505,7 @@ function useComponentState({
                             <dd class="text-sm text-gray-900">
                               <RenderAmount
                                 value={cashinCalc.beforeFee}
-                                spec={info.fiat_currency_specification}
+                                spec={info.regional_currency_specification}
                               />
                             </dd>
                           </div>
@@ -465,7 +518,7 @@ function useComponentState({
                             <RenderAmount
                               value={cashinCalc.credit}
                               withColor
-                              spec={info.fiat_currency_specification}
+                              spec={info.regional_currency_specification}
                             />
                           </dd>
                         </div>
@@ -485,7 +538,7 @@ function useComponentState({
                               value={cashoutCalc.debit}
                               negative
                               withColor
-                              spec={info.fiat_currency_specification}
+                              spec={info.regional_currency_specification}
                             />
                           </dd>
                         </div>
@@ -500,7 +553,7 @@ function useComponentState({
                             <dd class="text-sm text-gray-900">
                               <RenderAmount
                                 value={cashoutCalc.beforeFee}
-                                spec={info.regional_currency_specification}
+                                spec={info.fiat_currency_specification}
                               />
                             </dd>
                           </div>
@@ -513,7 +566,7 @@ function useComponentState({
                             <RenderAmount
                               value={cashoutCalc.credit}
                               withColor
-                              spec={info.regional_currency_specification}
+                              spec={info.fiat_currency_specification}
                             />
                           </dd>
                         </div>
@@ -616,11 +669,6 @@ function createFormValidator(
           : !cashin_min_amount
             ? i18n.str`Invalid`
             : undefined,
-        cashin_tiny_amount: !state.conv.cashin_tiny_amount
-          ? i18n.str`Required`
-          : !cashin_tiny_amount
-            ? i18n.str`Invalid`
-            : undefined,
         cashin_fee: !state.conv.cashin_fee
           ? i18n.str`Required`
           : !cashin_fee
@@ -632,11 +680,6 @@ function createFormValidator(
           : !cashout_min_amount
             ? i18n.str`Invalid`
             : undefined,
-        cashout_tiny_amount: !state.conv.cashin_tiny_amount
-          ? i18n.str`Required`
-          : !cashout_tiny_amount
-            ? i18n.str`Invalid`
-            : undefined,
         cashout_fee: !state.conv.cashin_fee
           ? i18n.str`Required`
           : !cashout_fee
@@ -660,6 +703,21 @@ function createFormValidator(
           : Number.isNaN(cashout_ratio)
             ? i18n.str`Rnvalid`
             : undefined,
+
+        cashin_tiny_amount: !state.conv.cashin_tiny_amount
+          ? i18n.str`Required`
+          : !cashin_tiny_amount
+            ? i18n.str`Invalid`
+            : +state.conv.cashin_tiny_amount == 0
+              ? i18n.str`Must be > 0`
+              : undefined,
+        cashout_tiny_amount: !state.conv.cashout_tiny_amount
+          ? i18n.str`Required`
+          : !cashout_tiny_amount
+            ? i18n.str`Invalid`
+            : +state.conv.cashout_tiny_amount == 0
+              ? i18n.str`Must be > 0`
+              : undefined,
       }),
 
       amount: !state.amount
@@ -678,30 +736,30 @@ function createFormValidator(
         cashin_min_amount: !errors?.conv?.cashin_min_amount
           ? Amounts.stringify(cashin_min_amount!)
           : undefined,
+        cashin_tiny_amount: !errors?.conv?.cashin_tiny_amount
+          ? Amounts.stringify(cashin_tiny_amount!)
+          : undefined,
         cashin_ratio: !errors?.conv?.cashin_ratio
           ? String(cashin_ratio!)
           : undefined,
         cashin_rounding_mode: !errors?.conv?.cashin_rounding_mode
           ? state.conv.cashin_rounding_mode!
           : undefined,
-        cashin_tiny_amount: !errors?.conv?.cashin_tiny_amount
-          ? Amounts.stringify(cashin_tiny_amount!)
-          : undefined,
         cashout_fee: !errors?.conv?.cashout_fee
           ? Amounts.stringify(cashout_fee!)
           : undefined,
         cashout_min_amount: !errors?.conv?.cashout_min_amount
           ? Amounts.stringify(cashout_min_amount!)
           : undefined,
+        cashout_tiny_amount: !errors?.conv?.cashout_tiny_amount
+          ? Amounts.stringify(cashout_tiny_amount!)
+          : undefined,
         cashout_ratio: !errors?.conv?.cashout_ratio
           ? String(cashout_ratio!)
           : undefined,
         cashout_rounding_mode: !errors?.conv?.cashout_rounding_mode
           ? state.conv.cashout_rounding_mode!
           : undefined,
-        cashout_tiny_amount: !errors?.conv?.cashout_tiny_amount
-          ? Amounts.stringify(cashout_tiny_amount!)
-          : undefined,
       },
     };
     return errors === undefined
@@ -710,7 +768,7 @@ function createFormValidator(
   };
 }
 
-function ConversionForm({
+export function ConversionForm({
   id,
   inputCurrency,
   outputCurrency,
@@ -719,14 +777,24 @@ function ConversionForm({
   ratio,
   rounding,
   tiny,
+  fallback_fee,
+  fallback_minimum,
+  fallback_ratio,
+  fallback_rounding,
+  fallback_tiny,
 }: {
   inputCurrency: string;
   outputCurrency: string;
   minimum: UIField | undefined;
-  tiny: UIField | undefined;
+  fallback_minimum?: string;
   fee: UIField | undefined;
+  fallback_fee?: string;
   rounding: UIField | undefined;
+  fallback_rounding?: string;
+  tiny: UIField | undefined;
+  fallback_tiny?: string;
   ratio: UIField | undefined;
+  fallback_ratio?: string;
   id: string;
 }): VNode {
   const { i18n } = useTranslationContext();
@@ -745,6 +813,7 @@ function ConversionForm({
               currency={inputCurrency}
               value={minimum?.value ?? ""}
               onChange={minimum?.onUpdate}
+              placeholder={fallback_minimum}
             />
             <ShowInputErrorLabel
               message={minimum?.error}
@@ -752,8 +821,9 @@ function ConversionForm({
             />
             <p class="mt-2 text-sm text-gray-500">
               <i18n.Translate>
-                Only cashout operation above this threshold will be allowed
+                Only cashout operation above this threshold will be allowed.
               </i18n.Translate>
+              &nbsp;
             </p>
           </div>
         </div>
@@ -778,6 +848,7 @@ function ConversionForm({
               ratio?.onUpdate(e.currentTarget.value);
             }}
             autocomplete="off"
+            placeholder={fallback_ratio ?? "1.0"}
           />
           <ShowInputErrorLabel
             message={ratio?.error}
@@ -792,8 +863,8 @@ function ConversionForm({
       <div class="px-6 pt-4">
         <Attention title={i18n.str`Example conversion`}>
           <i18n.Translate>
-            1 {inputCurrency} will be converted into {ratio?.value}{" "}
-            {outputCurrency}
+            1 {inputCurrency} will be converted into{" "}
+            {ratio?.value ?? fallback_ratio} {outputCurrency}
           </i18n.Translate>
         </Attention>
       </div>
@@ -802,26 +873,23 @@ function ConversionForm({
         <div class="grid max-w-2xl grid-cols-1 gap-x-6 gap-y-8 sm:grid-cols-6">
           <div class="sm:col-span-5">
             <label
-              for={`${id}_tiny_amount`}
               class="block text-sm font-medium leading-6 text-gray-900"
-            >{i18n.str`Rounding value`}</label>
+              for={`${id}_tiny_amount`}
+            >
+              {i18n.str`Tiny amount`}
+            </label>
             <InputAmount
               name={`${id}_tiny_amount`}
               left
-              currency={outputCurrency}
+              currency={inputCurrency}
               value={tiny?.value ?? ""}
               onChange={tiny?.onUpdate}
+              placeholder={fallback_tiny ?? "0.01"}
             />
             <ShowInputErrorLabel
               message={tiny?.error}
               isDirty={tiny?.value !== undefined}
             />
-            <p class="mt-2 text-sm text-gray-500">
-              <i18n.Translate>
-                Smallest difference between two amounts after the ratio is
-                applied.
-              </i18n.Translate>
-            </p>
           </div>
         </div>
       </div>
@@ -955,6 +1023,14 @@ function ConversionForm({
                   </svg>
                 </label>
               </div>
+              {!fallback_rounding ? undefined : (
+                <p class="mt-2 text-sm text-gray-500">
+                  <i18n.Translate>
+                    If none specified the fallback value is "{fallback_rounding}
+                    ".
+                  </i18n.Translate>
+                </p>
+              )}
             </div>
           </div>
         </div>
@@ -1133,6 +1209,7 @@ function ConversionForm({
                 <i18n.Translate>
                   With the "up" mode the value will be rounded to 1.3
                 </i18n.Translate>
+                .0
               </p>
             </details>
           </section>
@@ -1152,6 +1229,7 @@ function ConversionForm({
               currency={outputCurrency}
               value={fee?.value ?? ""}
               onChange={fee?.onUpdate}
+              placeholder={fallback_fee}
             />
             <ShowInputErrorLabel
               message={fee?.error}
