@@ -14,39 +14,44 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 import {
-  AbsoluteTime,
+  AmountJson,
   Amounts,
+  ConversionRate,
   HttpStatusCode,
+  TalerConversionInfoConfig,
   TalerError,
   TalerErrorCode,
-  TranslatedString,
   assertUnreachable,
   encodeCrock,
   getRandomBytes,
-  parsePaytoUri,
 } from "@gnu-taler/taler-util";
 import {
   Attention,
+  ButtonBetter,
+  ErrorLoading,
   Loading,
   LocalNotificationBanner,
   RouteDefinition,
   ShowInputErrorLabel,
+  makeSafeCall,
   notifyInfo,
   useBankCoreApiContext,
-  useLocalNotification,
+  useChallengeHandler,
+  useLocalNotificationBetter,
   useTranslationContext,
 } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
 import { useEffect, useState } from "preact/hooks";
-import { ErrorLoadingWithDebug } from "../../components/ErrorLoadingWithDebug.js";
+
 import { useAccountDetails } from "../../hooks/account.js";
-import { useBankState } from "../../hooks/bank-state.js";
 import {
+  TransCalc,
   TransferCalculation,
-  useCashoutEstimator,
+  useCashoutEstimatorByUser,
   useConversionInfo,
+  useConversionRateForUser,
 } from "../../hooks/regional.js";
-import { useSessionState } from "../../hooks/session.js";
+import { LoggedIn, useSessionState } from "../../hooks/session.js";
 import { TanChannel, undefinedIfEmpty } from "../../utils.js";
 import { LoginForm } from "../LoginForm.js";
 import {
@@ -54,14 +59,17 @@ import {
   RenderAmount,
   doAutoFocus,
 } from "../PaytoWireTransferForm.js";
+import { SolveMFAChallenges } from "../SolveMFA.js";
+import { TalerCorebankApi } from "@gnu-taler/taler-util";
+import { Paytos } from "@gnu-taler/taler-util";
+
+const TALER_SCREEN_ID = 127;
 
 interface Props {
   account: string;
   focus?: boolean;
-  onAuthorizationRequired: () => void;
   onCashout: () => void;
   routeClose: RouteDefinition;
-  routeHere: RouteDefinition;
 }
 
 type FormType = {
@@ -74,32 +82,15 @@ type ErrorFrom<T> = {
   [P in keyof T]+?: string;
 };
 
+const RANDOM_STRING = encodeCrock(getRandomBytes(32));
 export function CreateCashout({
   account: accountName,
-  onAuthorizationRequired,
   onCashout,
   focus,
-  routeHere,
   routeClose,
 }: Props): VNode {
   const { i18n } = useTranslationContext();
-  const resultAccount = useAccountDetails(accountName);
-  const {
-    estimateByCredit: calculateFromCredit,
-    estimateByDebit: calculateFromDebit,
-  } = useCashoutEstimator();
-  const { state: credentials } = useSessionState();
-  const creds = credentials.status !== "loggedIn" ? undefined : credentials;
-  const [, updateBankState] = useBankState();
-
-  const {
-    lib: { bank: api },
-    config,
-  } = useBankCoreApiContext();
-  const [form, setForm] = useState<Partial<FormType>>({ isDebit: true });
-  const [notification, notify, handleError] = useLocalNotification();
-  const info = useConversionInfo();
-
+  const { config } = useBankCoreApiContext();
   if (!config.allow_conversion) {
     return (
       <Fragment>
@@ -121,13 +112,17 @@ export function CreateCashout({
     );
   }
 
+  const resultAccount = useAccountDetails(accountName);
+  const { state: credentials } = useSessionState();
+  const creds = credentials.status !== "loggedIn" ? undefined : credentials;
+  const rateResp = useConversionRateForUser(accountName, creds?.token);
+  const conversionResp = useConversionInfo();
+
   if (!resultAccount) {
     return <Loading />;
-  }
-  if (resultAccount instanceof TalerError) {
-    return <ErrorLoadingWithDebug error={resultAccount} />;
-  }
-  if (resultAccount.type === "fail") {
+  } else if (resultAccount instanceof TalerError) {
+    return <ErrorLoading error={resultAccount} />;
+  } else if (resultAccount.type === "fail") {
     switch (resultAccount.case) {
       case HttpStatusCode.Unauthorized:
         return <LoginForm currentUser={accountName} />;
@@ -137,60 +132,115 @@ export function CreateCashout({
         assertUnreachable(resultAccount);
     }
   }
-  if (!info) {
-    return <Loading />;
-  }
 
-  if (info instanceof TalerError) {
-    return <ErrorLoadingWithDebug error={info} />;
-  }
-  if (info.type === "fail") {
-    switch (info.case) {
+  if (!conversionResp) {
+    return <Loading />;
+  } else if (conversionResp instanceof TalerError) {
+    return <ErrorLoading error={conversionResp} />;
+  } else if (conversionResp.type === "fail") {
+    switch (conversionResp.case) {
       case HttpStatusCode.NotImplemented: {
         return (
-          <Attention type="danger" title={i18n.str`Cashout are disabled`}>
+          <Attention type="danger" title={i18n.str`Cashout is disabled`}>
             <i18n.Translate>
-              Cashout should be enable by configuration and the conversion rate
-              should be initialized with fee, ratio and rounding mode.
+              Cashout should be enabled in the configuration, the conversion
+              rate should be initialized with fee(s), rates and a rounding mode.
             </i18n.Translate>
           </Attention>
         );
       }
       default:
-        assertUnreachable(info.case);
+        assertUnreachable(conversionResp);
     }
   }
 
-  const conversionInfo = info.body.conversion_rate;
-  if (!conversionInfo) {
+  if (!rateResp) {
+    return <Loading />;
+  } else if (rateResp instanceof TalerError) {
+    return <ErrorLoading error={rateResp} />;
+  } else if (rateResp.type === "fail") {
+    switch (rateResp.case) {
+      case HttpStatusCode.NotImplemented: {
+        return (
+          <Attention type="danger" title={i18n.str`Cashout is disabled`}>
+            <i18n.Translate>
+              Cashout should be enabled in the configuration, the conversion
+              rate should be initialized with fee(s), rates and a rounding mode.
+            </i18n.Translate>
+          </Attention>
+        );
+      }
+      default:
+        assertUnreachable(rateResp);
+    }
+  }
+  const rate = rateResp.body;
+  if (!rate) {
     return (
       <div>conversion enabled but server replied without conversion_rate</div>
     );
   }
+  if (!creds) {
+    return <div>authentication required</div>;
+  }
 
-  const {
+  return (
+    <CreateCashoutInternal
+      accountData={resultAccount.body}
+      onCashout={onCashout}
+      routeClose={routeClose}
+      focus={focus}
+      convConfig={conversionResp.body}
+      rate={rateResp.body}
+      session={creds}
+    />
+  );
+}
+
+function CreateCashoutInternal({
+  onCashout,
+  accountData,
+  focus,
+  routeClose,
+  convConfig: {
     fiat_currency,
-    regional_currency,
     fiat_currency_specification,
+    regional_currency,
     regional_currency_specification,
-  } = info.body;
+  },
+  session,
+  rate,
+}: Omit<Props, "account"> & {
+  convConfig: TalerConversionInfoConfig;
+  rate: ConversionRate;
+  session: LoggedIn;
+  accountData: TalerCorebankApi.AccountData;
+}): VNode {
+  const {
+    estimateByCredit: calculateFromCredit,
+    estimateByDebit: calculateFromDebit,
+  } = useCashoutEstimatorByUser(accountData.name);
+  const [form, setForm] = useState<Partial<FormType>>({ isDebit: true });
+  const [notification, notifyOnError] = useLocalNotificationBetter();
+  const mfa = useChallengeHandler();
+  const { i18n } = useTranslationContext();
+  const {
+    lib: { bank: api },
+  } = useBankCoreApiContext();
+
   const regionalZero = Amounts.zeroOfCurrency(regional_currency);
-  const fiatZero = Amounts.zeroOfCurrency(fiat_currency);
+  const fiatZero = Amounts.zeroOfCurrency(fiat_currency!);
 
   const account = {
-    balance: Amounts.parseOrThrow(resultAccount.body.balance.amount),
-    balanceIsDebit:
-      resultAccount.body.balance.credit_debit_indicator == "debit",
-    debitThreshold: Amounts.parseOrThrow(resultAccount.body.debit_threshold),
-    minCashout:
-      resultAccount.body.min_cashout === undefined
-        ? regionalZero
-        : Amounts.parseOrThrow(resultAccount.body.min_cashout),
+    balance: Amounts.parseOrThrow(accountData.balance.amount),
+    balanceIsDebit: accountData.balance.credit_debit_indicator == "debit",
+    debitThreshold: Amounts.parseOrThrow(accountData.debit_threshold),
   };
 
-  const limit = account.balanceIsDebit
-    ? Amounts.sub(account.debitThreshold, account.balance).amount
-    : Amounts.add(account.balance, account.debitThreshold).amount;
+  const balanceLimit = IntAmounts.toIntAmount(
+    account.balance,
+    account.balanceIsDebit,
+  ).increment(account.debitThreshold);
 
   const zeroCalc = {
     debit: regionalZero,
@@ -199,8 +249,8 @@ export function CreateCashout({
   };
   const [calculationResult, setCalculation] =
     useState<TransferCalculation>(zeroCalc);
-  const sellFee = Amounts.parseOrThrow(conversionInfo.cashout_fee);
-  const sellRate = conversionInfo.cashout_ratio;
+  const sellFee = Amounts.parseOrThrow(rate.cashout_fee);
+  const sellRate = rate.cashout_ratio;
   /**
    * can be in regional currency or fiat currency
    * depending on the isDebit flag
@@ -211,30 +261,50 @@ export function CreateCashout({
     }`,
   );
 
+  const higerThanMin = form.isDebit
+    ? Amounts.cmp(inputAmount, rate.cashout_min_amount) >= 0
+    : true;
+  const notZero = Amounts.isNonZero(inputAmount);
+
+  const conversionCalculator = makeSafeCall(
+    i18n,
+    (isDebit: boolean, input: AmountJson, fee: AmountJson) =>
+      isDebit
+        ? calculateFromDebit(input, fee)
+        : calculateFromCredit(input, fee),
+    (success) => {
+      setCalculation(success.body);
+    },
+    (fail) => {
+      switch (fail.case) {
+        case HttpStatusCode.Conflict:
+          return i18n.str`The amount is too small.`;
+        case HttpStatusCode.BadRequest:
+          return i18n.str`Server didn't like our request`;
+        case HttpStatusCode.NotImplemented:
+          return i18n.str`Conversion is not enabled.`;
+      }
+    },
+  );
+
   useEffect(() => {
     async function doAsync() {
-      await handleError(async () => {
-        const higerThanMin = form.isDebit
-          ? Amounts.cmp(inputAmount, conversionInfo.cashout_min_amount) === 1
-          : true;
-        const notZero = Amounts.isNonZero(inputAmount);
-        if (notZero && higerThanMin) {
-          const resp = await (form.isDebit
-            ? calculateFromDebit(inputAmount, sellFee)
-            : calculateFromCredit(inputAmount, sellFee));
-          setCalculation(resp);
-        } else {
-          setCalculation(zeroCalc);
-        }
-      });
+      if (notZero && higerThanMin) {
+        await conversionCalculator(form.isDebit ?? false, inputAmount, sellFee);
+      } else {
+        setCalculation(zeroCalc);
+      }
     }
     doAsync();
-  }, [form.amount, form.isDebit]);
+  }, [form.amount, form.isDebit, notZero, higerThanMin, rate.cashout_fee]);
 
   const calc =
     calculationResult === "amount-is-too-small" ? zeroCalc : calculationResult;
 
-  const balanceAfter = Amounts.sub(account.balance, calc.debit).amount;
+  const balanceAfter = IntAmounts.toIntAmount(
+    account.balance,
+    account.balanceIsDebit,
+  ).deduce(calc.debit).result;
 
   function updateForm(newForm: typeof form): void {
     setForm(newForm);
@@ -245,140 +315,112 @@ export function CreateCashout({
       ? i18n.str`Required`
       : !inputAmount
         ? i18n.str`Invalid`
-        : Amounts.cmp(limit, calc.debit) === -1
-          ? i18n.str`Balance is not enough`
-          : calculationResult === "amount-is-too-small"
-            ? i18n.str`Amount needs to be higher`
-            : Amounts.cmp(calc.debit, conversionInfo.cashout_min_amount) < 0
-              ? i18n.str`No account can't cashout less than ${
+        : calculationResult === "amount-is-too-small"
+          ? i18n.str`Amount needs to be higher`
+          : Amounts.isZero(
+                balanceLimit
+                  .deduce(calculationResult.debit)
+                  .getResultZeroIfNegative(),
+              )
+            ? i18n.str`Balance is not enough`
+            : Amounts.cmp(calculationResult.debit, rate.cashout_min_amount) < 0
+              ? i18n.str`It is not possible to cashout less than ${
                   Amounts.stringifyValueWithSpec(
-                    Amounts.parseOrThrow(conversionInfo.cashout_min_amount),
+                    Amounts.parseOrThrow(rate.cashout_min_amount),
                     regional_currency_specification,
                   ).normal
-                }`
-              : Amounts.cmp(calc.debit, account.minCashout) < 0
-                ? i18n.str`Your account can't cashout less than ${
-                    Amounts.stringifyValueWithSpec(
-                      Amounts.parseOrThrow(account.minCashout),
-                      regional_currency_specification,
-                    ).normal
-                  }`
-                : Amounts.isZero(calc.credit)
-                  ? i18n.str`The total transfer at destination will be zero`
-                  : undefined,
+                }: ${Amounts.stringify(calculationResult.debit)}`
+              : Amounts.isZero(calculationResult.credit)
+                ? i18n.str`The total transfer to the destination will be zero`
+                : undefined,
   });
   const trimmedAmountStr = form.amount?.trim();
 
-  async function createCashout() {
-    const request_uid = encodeCrock(getRandomBytes(32));
-    await handleError(async () => {
-      if (!creds || !form.subject) return;
-      const request = {
-        request_uid,
-        amount_credit: Amounts.stringify(calc.credit),
-        amount_debit: Amounts.stringify(calc.debit),
-        subject: form.subject,
-      };
-      const resp = await api.createCashout(creds, request);
-      if (resp.type === "ok") {
-        notifyInfo(i18n.str`Cashout created`);
-        onCashout();
-      } else {
-        switch (resp.case) {
-          case HttpStatusCode.Accepted: {
-            updateBankState("currentChallenge", {
-              operation: "create-cashout",
-              id: String(resp.body.challenge_id),
-              sent: AbsoluteTime.never(),
-              location: routeHere.url({}),
-              request,
-            });
-            return onAuthorizationRequired();
+  const [createCashout, repeatCashout] = mfa.withMfaHandler(
+    ({ challengeIds, onChallengeRequired }) =>
+      makeSafeCall(
+        i18n,
+        (calc: TransCalc, subject: string) =>
+          api.createCashout(
+            session,
+            {
+              request_uid: RANDOM_STRING,
+              amount_credit: Amounts.stringify(calc.credit),
+              amount_debit: Amounts.stringify(calc.debit),
+              subject,
+            },
+            { challengeIds },
+          ),
+        (success) => {
+          notifyInfo(i18n.str`Cashout created`);
+          onCashout();
+        },
+        (fail) => {
+          switch (fail.case) {
+            case HttpStatusCode.Accepted: {
+              // updateBankState("currentChallenge", {
+              //   operation: "create-cashout",
+              //   id: String(resp.body.challenge_id),
+              //   sent: AbsoluteTime.never(),
+              //   request,
+              // });
+              onChallengeRequired(fail.body);
+              return i18n.str`Second factor authentication required.`;
+            }
+            case HttpStatusCode.NotFound:
+              return i18n.str`Account not found`;
+            case TalerErrorCode.BANK_TRANSFER_REQUEST_UID_REUSED:
+              return i18n.str`Duplicated request detected, check if the operation succeeded or try again.`;
+            case TalerErrorCode.BANK_BAD_CONVERSION:
+              return i18n.str`The conversion rate was applied incorrectly`;
+            case TalerErrorCode.BANK_UNALLOWED_DEBIT:
+              return i18n.str`The account does not have sufficient funds`;
+            case HttpStatusCode.NotImplemented:
+              return i18n.str`Cashout is disabled`;
+            case TalerErrorCode.BANK_CONFIRM_INCOMPLETE:
+              return i18n.str`Missing cashout URI in the profile`;
+            case TalerErrorCode.BANK_CONVERSION_AMOUNT_TO_SMALL:
+              return i18n.str`The amount is below the minimum amount permitted.`;
+            case TalerErrorCode.BANK_TAN_CHANNEL_SCRIPT_FAILED:
+              return i18n.str`Sending the confirmation message failed, retry later or contact the administrator.`;
+            case TalerErrorCode.BANK_TAN_CHANNEL_NOT_SUPPORTED: {
+              return i18n.str`The server doesn't support the current TAN channel.`;
+            }
           }
-          case HttpStatusCode.NotFound:
-            return notify({
-              type: "error",
-              title: i18n.str`Account not found`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case TalerErrorCode.BANK_TRANSFER_REQUEST_UID_REUSED:
-            return notify({
-              type: "error",
-              title: i18n.str`Duplicated request detected, check if the operation succeeded or try again.`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case TalerErrorCode.BANK_BAD_CONVERSION:
-            return notify({
-              type: "error",
-              title: i18n.str`The conversion rate was incorrectly applied`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case TalerErrorCode.BANK_UNALLOWED_DEBIT:
-            return notify({
-              type: "error",
-              title: i18n.str`The account does not have sufficient funds`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case HttpStatusCode.NotImplemented:
-            return notify({
-              type: "error",
-              title: i18n.str`Cashout are disabled`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case TalerErrorCode.BANK_CONFIRM_INCOMPLETE:
-            return notify({
-              type: "error",
-              title: i18n.str`Missing cashout URI in the profile`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-          case TalerErrorCode.BANK_CONVERSION_AMOUNT_TO_SMALL:
-            return notify({
-              type: "error",
-              title: i18n.str`The amount is less than the minimum allowed.`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
+        },
+      ),
+  );
 
-          case TalerErrorCode.BANK_TAN_CHANNEL_SCRIPT_FAILED:
-            return notify({
-              type: "error",
-              title: i18n.str`Sending the confirmation message failed, retry later or contact the administrator.`,
-              description: resp.detail.hint as TranslatedString,
-              debug: resp.detail,
-              when: AbsoluteTime.now(),
-            });
-        }
-        assertUnreachable(resp);
-      }
-    });
+  const subject = form.subject;
+  const cashoutHandler =
+    !!errors || !subject
+      ? undefined
+      : () => notifyOnError(createCashout)(calc, subject);
+
+  const cashoutDisabled = !accountData.cashout_payto_uri;
+
+  const cashoutAccount = !accountData.cashout_payto_uri
+    ? undefined
+    : Paytos.fromString(accountData.cashout_payto_uri);
+  const cashoutAccountName = !cashoutAccount || cashoutAccount.type === "fail"
+    ? undefined
+    : cashoutAccount.body.displayName;
+
+  const cashoutLegalName = !cashoutAccount || cashoutAccount.type === "fail"
+    ? undefined
+    : cashoutAccount.body.params["receiver-name"];
+
+  if (mfa.pendingChallenge && repeatCashout) {
+    return (
+      <SolveMFAChallenges
+        currentChallenge={mfa.pendingChallenge}
+        username={accountData.name}
+        description={i18n.str`Create cashout.`}
+        onCancel={mfa.doCancelChallenge}
+        onCompleted={repeatCashout}
+      />
+    );
   }
-  const cashoutDisabled =
-    (config.supported_tan_channels ?? []).length < 1 ||
-    !resultAccount.body.cashout_payto_uri;
-
-  const cashoutAccount = !resultAccount.body.cashout_payto_uri
-    ? undefined
-    : parsePaytoUri(resultAccount.body.cashout_payto_uri);
-  const cashoutAccountName = !cashoutAccount
-    ? undefined
-    : cashoutAccount.targetPath;
-
-  const cashoutLegalName = !cashoutAccount
-    ? undefined
-    : cashoutAccount.params["receiver-name"];
 
   return (
     <div>
@@ -407,6 +449,8 @@ export function CreateCashout({
               <dd class="text-sm text-gray-900">
                 <RenderAmount
                   value={account.balance}
+                  negative={account.balanceIsDebit}
+                  withSign
                   spec={regional_currency_specification}
                 />
               </dd>
@@ -420,7 +464,9 @@ export function CreateCashout({
               <dd class="text-sm text-gray-900">
                 <RenderAmount
                   value={sellFee}
-                  spec={fiat_currency_specification}
+                  negative
+                  withSign
+                  spec={fiat_currency_specification!}
                 />
               </dd>
             </div>
@@ -444,16 +490,17 @@ export function CreateCashout({
                 </div>
                 <p class="mt-2 text-sm text-gray-500">
                   <i18n.Translate>
-                    If this name doesn't match the account holder's name your
+                    If this name doesn't match the account holder's name, your
                     transaction may fail.
                   </i18n.Translate>
                 </p>
               </Fragment>
             ) : (
               <div class="flex items-center justify-between border-t-2 afu pt-4">
-                <Attention type="warning" title={i18n.str`No cashout account`}>
+                <Attention type="warning" title={i18n.str`Unable to cashout`}>
                   <i18n.Translate>
-                    Before doing a cashout you need to complete your profile
+                    Before being able to cashout to a bank account, you need to
+                    complete your profile
                   </i18n.Translate>
                 </Attention>
               </div>
@@ -478,7 +525,7 @@ export function CreateCashout({
                   for="subject"
                 >
                   {i18n.str`Transfer subject`}
-                  <b style={{ color: "red" }}> *</b>
+                  <b class="text-[red]"> *</b>
                 </label>
                 <div class="mt-2">
                   <input
@@ -597,7 +644,7 @@ export function CreateCashout({
                     for="amount"
                   >
                     {i18n.str`Amount`}
-                    <b style={{ color: "red" }}> *</b>
+                    <b class="text-[red]"> *</b>
                   </label>
                   {/* <button
                     type="button"
@@ -623,7 +670,7 @@ export function CreateCashout({
                   <InputAmount
                     name="amount"
                     left
-                    currency={form.isDebit ? regional_currency : fiat_currency}
+                    currency={form.isDebit ? regional_currency : fiat_currency!}
                     value={trimmedAmountStr}
                     onChange={
                       cashoutDisabled
@@ -667,6 +714,8 @@ export function CreateCashout({
                       <dd class="text-sm text-gray-900">
                         <RenderAmount
                           value={balanceAfter}
+                          negative={balanceAfter.negative}
+                          withSign
                           spec={regional_currency_specification}
                         />
                       </dd>
@@ -682,7 +731,7 @@ export function CreateCashout({
                         <dd class="text-sm text-gray-900">
                           <RenderAmount
                             value={calc.beforeFee}
-                            spec={fiat_currency_specification}
+                            spec={fiat_currency_specification!}
                           />
                         </dd>
                       </div>
@@ -695,7 +744,7 @@ export function CreateCashout({
                         <RenderAmount
                           value={calc.credit}
                           withColor
-                          spec={fiat_currency_specification}
+                          spec={fiat_currency_specification!}
                         />
                       </dd>
                     </div>
@@ -714,21 +763,120 @@ export function CreateCashout({
             >
               <i18n.Translate>Cancel</i18n.Translate>
             </a>
-            <button
+            <ButtonBetter
               type="submit"
               name="cashout"
               class="disabled:opacity-50 disabled:cursor-default cursor-pointer rounded-md bg-indigo-600 px-3 py-2 text-sm font-semibold text-white shadow-sm hover:bg-indigo-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-indigo-600"
-              disabled={!!errors}
-              onClick={(e) => {
-                e.preventDefault();
-                createCashout();
-              }}
+              onClick={cashoutHandler}
             >
               <i18n.Translate>Cashout</i18n.Translate>
-            </button>
+            </ButtonBetter>
           </div>
         </form>
       </div>
     </div>
   );
 }
+export interface IntAmountJson extends AmountJson {
+  negative: boolean;
+  saturated: boolean;
+}
+
+/**
+ * Helper to manipulate amounts that can be negative.
+ *
+ */
+export class IntAmounts {
+  readonly result: IntAmountJson;
+  private constructor(
+    value: AmountJson,
+    negative: boolean = false,
+    saturated: boolean = false,
+  ) {
+    this.result = {
+      ...value,
+      negative,
+      saturated,
+    };
+  }
+
+  static from(value: IntAmountJson): IntAmounts {
+    return new IntAmounts(value, value.negative, value.saturated);
+  }
+
+  static toIntAmount(value: AmountJson, negative: boolean = false): IntAmounts {
+    return new IntAmounts(value, negative);
+  }
+
+  getResultZeroIfNegative(): IntAmountJson {
+    return this.result.negative
+      ? IntAmounts.toIntAmount(Amounts.zeroOfCurrency(this.result.currency))
+          .result
+      : this.result;
+  }
+  /**
+   * Sum this value to a positive or negative amount.
+   *
+   * @param d
+   * @returns
+   */
+  merge(d: IntAmountJson): IntAmounts {
+    if (d.negative) {
+      return this.deduce(d);
+    } else {
+      return this.increment(d);
+    }
+  }
+  /**
+   * Deduce the absolute value by the amount
+   *
+   * @param am
+   * @returns
+   */
+  deduce(am: AmountJson): IntAmounts {
+    if (this.result.negative) {
+      const { amount, saturated } = Amounts.add(this.result, am);
+      return IntAmounts.from({
+        ...amount,
+        saturated,
+        negative: true,
+      });
+    } else {
+      const negative = Amounts.cmp(this.result, am) < 0;
+      const { amount, saturated } = negative
+        ? Amounts.sub(am, this.result)
+        : Amounts.sub(this.result, am);
+      return IntAmounts.from({
+        ...amount,
+        negative,
+        saturated,
+      });
+    }
+  }
+  /**
+   * Increment the value by the amount
+   *
+   * @param am
+   * @returns
+   */
+  increment(am: AmountJson): IntAmounts {
+    if (this.result.negative) {
+      const negative = Amounts.cmp(this.result, am) > 0;
+      const { amount, saturated } = negative
+        ? Amounts.sub(this.result, am)
+        : Amounts.sub(am, this.result);
+      return IntAmounts.from({
+        ...amount,
+        negative,
+        saturated,
+      });
+    } else {
+      const { amount, saturated } = Amounts.add(this.result, am);
+      return IntAmounts.from({
+        ...amount,
+        saturated,
+        negative: false,
+      });
+    }
+  }
+}
