@@ -33,7 +33,7 @@ export interface UiSettings {
   // Useful for testing
   // default: false
   allowRandomAccountCreation?: boolean;
-  // Create all random accounts with password "123"
+  // Create all random accounts with password "12345678" or "123"
   // Useful for testing
   // default: false
   simplePasswordForRandomAccounts?: boolean;
@@ -50,6 +50,9 @@ export interface UiSettings {
   // will send a default value that the user can change.
   // default: 10
   defaultSuggestedAmount?: number;
+  // Show a "This is a demo" info in the home screen.
+  // default: false
+  showDemoDescription?: boolean;
 }
 
 /**
@@ -60,6 +63,7 @@ const defaultSettings: UiSettings = {
   iconLinkURL: undefined,
   simplePasswordForRandomAccounts: false,
   allowRandomAccountCreation: false,
+  showDemoDescription: false,
   topNavSites: {},
   defaultSuggestedAmount: 10,
 };
@@ -68,6 +72,7 @@ const codecForUISettings = (): Codec<UiSettings> =>
   buildCodecForObject<UiSettings>()
     .property("backendBaseURL", codecOptional(codecForString()))
     .property("allowRandomAccountCreation", codecOptional(codecForBoolean()))
+    .property("showDemoDescription", codecOptional(codecForBoolean()))
     .property("defaultSuggestedAmount", codecOptional(codecForNumber()))
     .property(
       "simplePasswordForRandomAccounts",
