@@ -72,7 +72,7 @@ export const codecForSessionState = (): Codec<SessionState> =>
 
 export interface SessionStateHandler {
   state: SessionState | undefined;
-  start(s: SessionId): void;
+  start(): void;
   saveAddress(type: string, address: Record<string, string>): void;
   removeAddress(index: number): void;
   sent(info: ChallengerApi.ChallengeCreateResponse): void;
