@@ -14,29 +14,36 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 import {
+  assertUnreachable,
+  ChallengerApi,
   EmptyObject,
   HttpStatusCode,
+  InternationalizationAPI,
   TalerError,
-  TranslatedString
+  TalerFormAttributes,
+  TranslatedString,
 } from "@gnu-taler/taler-util";
 import {
   Attention,
   Button,
+  countryNameList,
+  ErrorLoading,
+  FormDesign,
+  FormUI,
   LocalNotificationBanner,
   RouteDefinition,
-  ShowInputErrorLabel,
-  Time,
   useChallengerApiContext,
+  useForm,
   useLocalNotificationHandler,
-  useTranslationContext,
+  useTranslationContext
 } from "@gnu-taler/web-util/browser";
-import { Fragment, VNode, h } from "preact";
+import { Fragment, h, VNode } from "preact";
 import { useState } from "preact/hooks";
 import { useChallengeSession } from "../hooks/challenge.js";
 import { SessionId, useSessionState } from "../hooks/session.js";
-import { doAutoFocus } from "./AnswerChallenge.js";
-
-export const EMAIL_REGEX = /^[\w-.]+@([\w-]+\.)+[\w-]{2,4}$/;
+import {
+  getAddressDescriptionFromAddrType
+} from "./AnswerChallenge.js";
 
 type Props = {
   onSendSuccesful: () => void;
@@ -56,81 +63,122 @@ export function AskChallenge({
 
   const { i18n } = useTranslationContext();
   const [notification, withErrorHandler] = useLocalNotificationHandler();
-  const [address, setEmail] = useState<string | undefined>();
-  const [repeat, setRepeat] = useState<string | undefined>();
-  const [remember, setRemember] = useState<boolean>(false);
+  // const [address, setEmail] = useState<string | undefined>();
   const [addrIndex, setAddrIndex] = useState<number | undefined>();
 
-  const restrictionKeys = !config.restrictions
-    ? []
-    : Object.keys(config.restrictions);
-  const restrictionKey = !restrictionKeys.length
-    ? undefined
-    : restrictionKeys[0];
-
   const result = useChallengeSession(session);
 
-  if (!restrictionKey) {
+  if (!result) {
     return (
       <div>
-        invalid server configuration, there is no restriction in /config
+        <i18n.Translate>loading...</i18n.Translate>
       </div>
     );
   }
+  if (result instanceof TalerError) {
+    return <ErrorLoading error={result} />;
+  }
+  if (result.type === "fail") {
+    switch (result.case) {
+      case HttpStatusCode.BadRequest: {
+        return (
+          <Attention
+            type="danger"
+            title={i18n.str`Couldn't get information about the validation process`}
+          >
+            <i18n.Translate>Bad request</i18n.Translate>
+          </Attention>
+        );
+      }
+      case HttpStatusCode.NotFound: {
+        return (
+          <Attention
+            type="danger"
+            title={i18n.str`Couldn't get information about the validation process`}
+          >
+            <i18n.Translate>Not found</i18n.Translate>
+          </Attention>
+        );
+      }
+      case HttpStatusCode.NotAcceptable: {
+        return (
+          <Attention
+            type="danger"
+            title={i18n.str`Couldn't get information about the validation process`}
+          >
+            <i18n.Translate>Not acceptable</i18n.Translate>
+          </Attention>
+        );
+      }
+      case HttpStatusCode.TooManyRequests: {
+        return (
+          <Attention
+            type="danger"
+            title={i18n.str`Couldn't get information about the validation process`}
+          >
+            <i18n.Translate>Too many request</i18n.Translate>
+          </Attention>
+        );
+      }
+      case HttpStatusCode.InternalServerError: {
+        return (
+          <Attention
+            type="danger"
+            title={i18n.str`Couldn't get information about the validation process`}
+          >
+            <i18n.Translate>Server error</i18n.Translate>
+          </Attention>
+        );
+      }
+    }
+  }
 
-  const regexEmail = !config.restrictions
-    ? undefined
-    : config.restrictions[restrictionKey];
-  const regexTest =
-    regexEmail && regexEmail.regex ? new RegExp(regexEmail.regex) : EMAIL_REGEX;
-  const regexHint =
-    regexEmail && regexEmail.hint ? regexEmail.hint : i18n.str`invalid email`;
+  const lastStatus = result.body;
 
-  const lastStatus =
-    result && !(result instanceof TalerError) && result.type !== "fail"
-      ? result.body
-      : undefined;
+  const initial = lastStatus.last_address ?? {};
+  if (config.address_type === "postal-ch") {
+    initial[TalerFormAttributes.ADDRESS_COUNTRY] = "CH";
+  }
+
+  const design = getFormDesignBasedOnAddressType(
+    i18n,
+    config.address_type,
+    config.restrictions ?? {},
+    initial,
+    lastStatus.fix_address,
+  );
+  const form = useForm(design, initial);
 
   const prevAddr = !lastStatus?.last_address
     ? undefined
-    : lastStatus.last_address[restrictionKey];
+    : getAddressDescriptionFromAddrType(
+        config.address_type,
+        lastStatus.last_address,
+      );
 
-  const errors = undefinedIfEmpty({
-    address: !address
-      ? i18n.str`required`
-      : !regexTest.test(address)
-        ? regexHint
-        : prevAddr !== undefined && address === prevAddr
-          ? i18n.str`can't use the same address`
-          : undefined,
-    repeat: !repeat
-      ? i18n.str`required`
-      : address !== repeat
-        ? i18n.str`doesn't match`
-        : undefined,
-  });
-
-  const contact = address ? { [restrictionKey]: address } : undefined;
-
-  const usableAddrs =
-    !state?.lastAddress || !state.lastAddress.length
-      ? []
-      : state.lastAddress.filter((d) => !!d.address[restrictionKey]);
+  const contact =
+    form.status.status === "fail"
+      ? undefined
+      : (form.status.result as Record<string, string>);
 
   const onSend =
-    errors || !contact 
+    form.status.errors || !contact
       ? undefined
       : withErrorHandler(
           async () => {
-            return lib.challenger.challenge(session.nonce, contact);
+            const info = lastStatus.fix_address
+              ? lastStatus.last_address!
+              : contact;
+
+            return lib.challenger.challenge(session.nonce, info);
           },
           (ok) => {
             if (ok.body.type === "completed") {
               completed(ok.body);
             } else {
-              if (remember) {
-                saveAddress(config.address_type, contact);
-              }
+              // if (remember) {
+              //   saveAddress(config.address_type, contact);
+              // }
               sent(ok.body);
             }
             onSendSuccesful();
@@ -146,18 +194,14 @@ export function AskChallenge({
               case HttpStatusCode.TooManyRequests:
                 return i18n.str`There have been too many attempts to request challenge transmissions.`;
               case HttpStatusCode.InternalServerError:
-                return i18n.str`Server is not able to respond due to internal problems.`;
+                return i18n.str`Server is unable to respond due to internal problems.`;
             }
           },
         );
 
-  if (!lastStatus) {
-    return <div>no status loaded</div>;
-  }
-
   return (
     <Fragment>
-      <LocalNotificationBanner notification={notification} showDebug={true} />
+      <LocalNotificationBanner notification={notification} />
 
       <div class="isolate bg-white px-6 py-12">
         <div class="mx-auto max-w-2xl text-center">
@@ -188,19 +232,23 @@ export function AskChallenge({
           )}
         </div>
 
-        {lastStatus?.last_address && (
-          <Fragment>
-            <Attention title={i18n.str`A code has been sent to ${prevAddr}`}>
-              <i18n.Translate>
-                <a href={routeSolveChallenge.url({})} class="underline">
-                  <i18n.Translate>Complete the challenge here.</i18n.Translate>
-                </a>
-              </i18n.Translate>
-            </Attention>
-          </Fragment>
-        )}
+        {lastStatus &&
+          lastStatus.last_address &&
+          lastStatus.auth_attempts_left > 0 && (
+            <Fragment>
+              <Attention title={i18n.str`A code has been sent to ${prevAddr}`}>
+                <i18n.Translate>
+                  <a href={routeSolveChallenge.url({})} class="underline">
+                    <i18n.Translate>
+                      Complete the challenge here.
+                    </i18n.Translate>
+                  </a>
+                </i18n.Translate>
+              </Attention>
+            </Fragment>
+          )}
 
-        {!usableAddrs.length ? undefined : (
+        {/* {!usableAddrs.length ? undefined : (
           <div class="mx-auto max-w-xl mt-4">
             <h3>
               <i18n.Translate>Previous address</i18n.Translate>
@@ -223,7 +271,6 @@ export function AskChallenge({
                           onClick={() => {
                             setAddrIndex(idx);
                             setEmail(addr.address[restrictionKey]);
-                            setRepeat(addr.address[restrictionKey]);
                           }}
                           class="h-4 w-4 border-gray-300 text-indigo-600 focus:ring-indigo-600 active:ring-2 active:ring-indigo-600 active:ring-offset-2"
                         />
@@ -262,7 +309,6 @@ export function AskChallenge({
                       onClick={() => {
                         setAddrIndex(undefined);
                         setEmail(undefined);
-                        setRepeat(undefined);
                       }}
                       class="h-4 w-4 border-gray-300 text-indigo-600 focus:ring-indigo-600 active:ring-2 active:ring-indigo-600 active:ring-offset-2"
                     />
@@ -277,10 +323,9 @@ export function AskChallenge({
               </div>
             </fieldset>
           </div>
-        )}
+        )} */}
 
-        <form
-          method="POST"
+        {/* <form
           class="mx-auto mt-4 max-w-xl "
           onSubmit={(e) => {
             e.preventDefault();
@@ -295,6 +340,8 @@ export function AskChallenge({
                 switch (config.address_type) {
                   case "email":
                     return i18n.str`Email`;
+                  case "address":
+                    return i18n.str`Address`;
                   case "phone":
                     return i18n.str`Phone`;
                 }
@@ -311,6 +358,8 @@ export function AskChallenge({
                   switch (config.address_type) {
                     case "email":
                       return "email";
+                    case "address":
+                      return "address";
                     case "phone":
                       return "phone";
                   }
@@ -329,69 +378,34 @@ export function AskChallenge({
               />
             </div>
           </div>
+        </form> */}
+        <div class="mx-auto mt-4 max-w-xl ">
+          <FormUI
+            design={design}
+            model={form.model}
+            onSubmit={onSend?.onClick}
+          />
+        </div>
 
-          {lastStatus.fix_address || addrIndex !== undefined ? undefined : (
-            <div class="sm:col-span-2">
-              <label
-                for="repeat-address"
-                class="block text-sm font-semibold leading-6 text-gray-900"
-              >
-                {(function (): TranslatedString {
-                  switch (config.address_type) {
-                    case "email":
-                      return i18n.str`Repeat email`;
-                    case "phone":
-                      return i18n.str`Repeat phone`;
-                  }
-                })()}
-              </label>
-              <div class="mt-2.5">
-                <input
-                  type="text"
-                  name="repeat-address"
-                  id="repeat-address"
-                  value={repeat ?? ""}
-                  onChange={(e) => {
-                    setRepeat(e.currentTarget.value);
-                  }}
-                  autocomplete={(function (): string {
-                    switch (config.address_type) {
-                      case "email":
-                        return "email";
-                      case "phone":
-                        return "phone";
-                    }
-                  })()}
-                  class="block w-full rounded-md border-0 px-3.5 py-2 text-gray-900 shadow-sm ring-1 ring-inset ring-gray-300 placeholder:text-gray-400 focus:ring-2 focus:ring-inset focus:ring-indigo-600 sm:text-sm sm:leading-6"
-                />
-                <ShowInputErrorLabel
-                  message={errors?.repeat}
-                  isDirty={repeat !== undefined}
-                />
-              </div>
-            </div>
-          )}
-
-          {lastStatus === undefined ? undefined : (
-            <p class="mt-2 text-sm leading-6 text-gray-400">
-              {lastStatus.changes_left < 1 ? (
-                <i18n.Translate>
-                  You can&#39;t change the contact address anymore.
-                </i18n.Translate>
-              ) : lastStatus.changes_left === 1 ? (
-                <i18n.Translate>
-                  You can change the contact address one last time.
-                </i18n.Translate>
-              ) : (
-                <i18n.Translate>
-                  You can change the contact address {lastStatus.changes_left}{" "}
-                  more times.
-                </i18n.Translate>
-              )}
-            </p>
-          )}
-
-          <div class="flex items-center justify-between py-2">
+        {lastStatus === undefined ? undefined : (
+          <p class="mt-2 text-sm leading-6 text-gray-400">
+            {lastStatus.changes_left < 1 ? (
+              <i18n.Translate>
+                You can&#39;t change the contact address anymore.
+              </i18n.Translate>
+            ) : lastStatus.changes_left === 1 ? (
+              <i18n.Translate>
+                You can change the contact address one last time.
+              </i18n.Translate>
+            ) : (
+              <i18n.Translate>
+                You can change the contact address {lastStatus.changes_left}{" "}
+                more times.
+              </i18n.Translate>
+            )}
+          </p>
+        )}
+        {/* <div class="flex items-center justify-between py-2">
             <span class="flex flex-grow flex-col">
               <span
                 class="text-sm text-black font-medium leading-6 "
@@ -421,8 +435,7 @@ export function AskChallenge({
                 class="translate-x-5 data-[enabled=false]:translate-x-0 pointer-events-none inline-block h-5 w-5 transform rounded-full bg-white shadow ring-0 transition duration-200 ease-in-out"
               ></span>
             </button>
-          </div>
-        </form>
+          </div> */}
         <div class="mx-auto mt-4 max-w-xl ">
           {!prevAddr ? (
             <div class="mt-10">
@@ -436,6 +449,9 @@ export function AskChallenge({
                   switch (config.address_type) {
                     case "email":
                       return i18n.str`Send email`;
+                    case "postal":
+                    case "postal-ch":
+                      return i18n.str`Send letter`;
                     case "phone":
                       return i18n.str`Send SMS`;
                   }
@@ -453,9 +469,18 @@ export function AskChallenge({
                 {(function (): TranslatedString {
                   switch (config.address_type) {
                     case "email":
-                      return i18n.str`Change email`;
+                      return lastStatus.fix_address
+                        ? i18n.str`Confirm email`
+                        : i18n.str`Change email`;
+                    case "postal":
+                    case "postal-ch":
+                      return lastStatus.fix_address
+                        ? i18n.str`Confirm address`
+                        : i18n.str`Change address`;
                     case "phone":
-                      return i18n.str`Change phone`;
+                      return lastStatus.fix_address
+                        ? i18n.str`Confirm phone`
+                        : i18n.str`Change phone`;
                   }
                 })()}
               </Button>
@@ -474,3 +499,250 @@ export function undefinedIfEmpty<T extends object>(obj: T): T | undefined {
     ? obj
     : undefined;
 }
+
+const ADDRESS_EXAMPLE_INTERNATIONAL = `Street name 1
+2. OG xxxx 
+12345 City_name 
+country_name `;
+
+const ADDRESS_EXAMPLE_CH = `Street name 1
+5. OG xxxx 
+12345 City_name 
+country_name `;
+
+export const EMAIL_REGEX = /^[\w-.]+@([\w-]+\.)+[\w-]{2,4}$/;
+export const PHONE_REGEX =
+  /^(\+\d{1,2}\s?)?\(?\d{3}\)?[\s.-]?\d{3}[\s.-]?\d{4}$/;
+export const CONTACT_REGEX = /.*/;
+export const ZIPCODE_REGEX = /.*/;
+export const ADDR_LINES_REGEX = /.*/;
+
+function getRestriction(
+  i18n: InternationalizationAPI,
+  serverConfig: ChallengerApi.Restriction | undefined,
+  fallback?: RegExp,
+): { regex: undefined | RegExp; hint: TranslatedString } {
+  const regexText =
+    serverConfig && serverConfig.regex ? serverConfig.regex : undefined;
+  const hint =
+    serverConfig && serverConfig.hint
+      ? (serverConfig.hint as TranslatedString)
+      : i18n.str`Invalid field`;
+
+  let regex;
+  if (regexText) {
+    try {
+      regex = new RegExp(regexText);
+    } catch (e) {
+      console.error(`Invalid server regular expression configuration. Server restriction
+          is "${regexText}" but it didn't compile: ${String(e)}`);
+      // return (
+      //   <Attention title={i18n.str`Server configuration error`} type="danger">
+      //     <i18n.Translate>
+      //       Invalid server regular expression configuration. Server
+      //       restriction is "{regexText}" but it didn't compile: {String(e)}
+      //     </i18n.Translate>
+      //   </Attention>
+      // );
+      regex = fallback;
+    }
+  } else {
+    regex = fallback;
+  }
+  return { regex, hint };
+}
+function getFormDesignBasedOnAddressType(
+  i18n: InternationalizationAPI,
+  type: ChallengerApi.ChallengerTermsOfServiceResponse["address_type"],
+  restrictions: Record<string, ChallengerApi.Restriction | undefined>,
+  prevValue: Record<string, string>,
+  read_only: boolean,
+): FormDesign {
+  switch (type) {
+    case "email":
+      return {
+        type: "single-column",
+        fields: [
+          {
+            type: "text",
+            id: TalerFormAttributes.CONTACT_EMAIL,
+            required: true,
+            label: i18n.str`Email`,
+            disabled: read_only,
+            validator(text) {
+              const restriction = getRestriction(
+                i18n,
+                restrictions[TalerFormAttributes.CONTACT_EMAIL],
+                EMAIL_REGEX,
+              );
+              if (restriction.regex && !restriction.regex.test(text)) {
+                return restriction.hint;
+              }
+              // const prev = prevValue[TalerFormAttributes.CONTACT_EMAIL];
+              // if (prev === text) {
+              //   return i18n.str`Can't use the same address`;
+              // }
+              return undefined;
+            },
+          },
+        ],
+      };
+    case "phone":
+      return {
+        type: "single-column",
+        fields: [
+          {
+            type: "text",
+            id: TalerFormAttributes.CONTACT_PHONE,
+            required: true,
+            label: i18n.str`Phone`,
+            disabled: read_only,
+            validator(text) {
+              const restriction = getRestriction(
+                i18n,
+                restrictions[TalerFormAttributes.CONTACT_PHONE],
+                PHONE_REGEX,
+              );
+              if (restriction.regex && !restriction.regex.test(text)) {
+                return restriction.hint;
+              }
+              // const prev = prevValue[TalerFormAttributes.CONTACT_PHONE];
+              // if (prev === text) {
+              //   return i18n.str`Can't use the same number`;
+              // }
+              return undefined;
+            },
+          },
+        ],
+      };
+    case "postal":
+      return {
+        type: "single-column",
+        fields: [
+          {
+            type: "text",
+            id: TalerFormAttributes.CONTACT_NAME,
+            required: true,
+            disabled: read_only,
+            label: i18n.str`Contact name`,
+            placeholder: i18n.str`Person full name or name of the business`,
+            validator(text) {
+              const restriction = getRestriction(
+                i18n,
+                restrictions[TalerFormAttributes.CONTACT_NAME],
+                CONTACT_REGEX,
+              );
+              if (restriction.regex && !restriction.regex.test(text)) {
+                return restriction.hint;
+              }
+              return undefined;
+            },
+          },
+          {
+            type: "textArea",
+            id: TalerFormAttributes.ADDRESS_LINES,
+            required: true,
+            disabled: read_only,
+            label: i18n.str`Address`,
+            placeholder: ADDRESS_EXAMPLE_INTERNATIONAL,
+            validator(text) {
+              const restriction = getRestriction(
+                i18n,
+                restrictions[TalerFormAttributes.ADDRESS_LINES],
+                ADDR_LINES_REGEX,
+              );
+              if (restriction.regex && !restriction.regex.test(text)) {
+                return restriction.hint;
+              }
+              return undefined;
+            },
+          },
+          {
+            id: TalerFormAttributes.ADDRESS_COUNTRY,
+            label: i18n.str`Country`,
+            type: "selectOne",
+            disabled: read_only,
+            choices: countryNameList(i18n),
+            required: true,
+            preferredChoiceVals: ["CH", "DE"],
+            validator(text) {
+              const restriction = getRestriction(
+                i18n,
+                restrictions[TalerFormAttributes.ADDRESS_COUNTRY],
+              );
+              if (restriction.regex && !restriction.regex.test(text)) {
+                return restriction.hint;
+              }
+              return undefined;
+            },
+          },
+        ],
+      };
+
+    case "postal-ch":
+      return {
+        type: "single-column",
+        fields: [
+          {
+            type: "text",
+            id: TalerFormAttributes.CONTACT_NAME,
+            required: true,
+            disabled: read_only,
+            label: i18n.str`Contact name`,
+            placeholder: i18n.str`Your full name`,
+            validator(text) {
+              const restriction = getRestriction(
+                i18n,
+                restrictions[TalerFormAttributes.CONTACT_NAME],
+              );
+              if (restriction.regex && !restriction.regex.test(text)) {
+                return restriction.hint;
+              }
+              return undefined;
+            },
+          },
+          {
+            type: "textArea",
+            id: TalerFormAttributes.ADDRESS_LINES,
+            required: true,
+            disabled: read_only,
+            label: i18n.str`Swiss address`,
+            help: i18n.str`Make sure that this address is for Switzerland`,
+            placeholder: ADDRESS_EXAMPLE_CH,
+            validator(text) {
+              const restriction = getRestriction(
+                i18n,
+                restrictions[TalerFormAttributes.ADDRESS_LINES],
+              );
+              if (restriction.regex && !restriction.regex.test(text)) {
+                return restriction.hint;
+              }
+              return undefined;
+            },
+          },
+          {
+            id: TalerFormAttributes.ADDRESS_COUNTRY,
+            label: i18n.str`Country`,
+            type: "selectOne",
+            choices: countryNameList(i18n),
+            required: true,
+            disabled: true,
+            preferredChoiceVals: ["CH"],
+            validator(text) {
+              const restriction = getRestriction(
+                i18n,
+                restrictions[TalerFormAttributes.ADDRESS_COUNTRY],
+              );
+              if (restriction.regex && !restriction.regex.test(text)) {
+                return restriction.hint;
+              }
+              return undefined;
+            },
+          },
+        ],
+      };
+    default: {
+      assertUnreachable(type);
+    }
+  }
+}
