@@ -16,12 +16,17 @@
 import { Attention, useTranslationContext } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
 import { useSessionState } from "../hooks/session.js";
+import { useEffect } from "preact/hooks";
 
 export function CallengeCompleted(): VNode {
   const { state } = useSessionState();
 
   const { i18n } = useTranslationContext();
   
+  useEffect(() => {
+    window.location.href = state?.completedURL ?? "#"
+  },[])
+
   return (
     <div class="m-4">
       <Attention
