@@ -13,6 +13,9 @@
  You should have received a copy of the GNU General Public License along with
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
+import { default as tw_typo } from '@tailwindcss/typography';
+import { default as tw_form } from '@tailwindcss/forms';
+
 export default {
   content: {
     relative: true,
@@ -24,5 +27,5 @@ export default {
   theme: {
     extend: {},
   },
-  plugins: [require("@tailwindcss/typography"), require("@tailwindcss/forms")],
+  plugins: [tw_typo, tw_form],
 };
