@@ -213,7 +213,7 @@ export interface Backend {
     indexName: string,
   ): void;
 
-  rollback(btx: DatabaseTransaction): void;
+  rollback(btx: DatabaseTransaction): Promise<void>;
 
   // FIXME: Should probably not be async
   commit(btx: DatabaseTransaction): Promise<void>;
