@@ -1,8 +1,10 @@
 import test from "ava";
-import { idbFactory } from "./wptsupport.js";
+import { initTestIndexedDB, useTestIndexedDb } from "./wptsupport.js";
+
+test.before("test DB initialization", initTestIndexedDB);
 
 test("WPT idbfactory-cmp*.html", async (t) => {
-  const indexedDB = idbFactory;
+  const indexedDB = useTestIndexedDb();
   var greater = indexedDB.cmp(2, 1);
   var equal = indexedDB.cmp(2, 2);
   var less = indexedDB.cmp(1, 2);
