@@ -1,15 +1,15 @@
 export type Sqlite3Database = {
   internalDbHandle: any;
-  exec(sqlStr: string): void;
-  prepare(stmtStr: string): Sqlite3Statement;
-  close(): void;
+  exec(sqlStr: string): Promise<void>;
+  prepare(stmtStr: string): Promise<Sqlite3Statement>;
+  close(): Promise<void>;
 };
 export type Sqlite3Statement = {
   internalStatement: any;
 
-  run(params?: BindParams): RunResult;
-  getAll(params?: BindParams): ResultRow[];
-  getFirst(params?: BindParams): ResultRow | undefined;
+  run(params?: BindParams): Promise<RunResult>;
+  getAll(params?: BindParams): Promise<ResultRow[]>;
+  getFirst(params?: BindParams): Promise<ResultRow | undefined>;
 };
 
 export interface RunResult {
@@ -30,5 +30,5 @@ export type ResultRow = Record<string, Sqlite3Value>;
  * to be used by our IndexedDB sqlite3 backend.
  */
 export interface Sqlite3Interface {
-  open(filename: string): Sqlite3Database;
+  open(filename: string): Promise<Sqlite3Database>;
 }
