@@ -16,7 +16,7 @@
 import { createSqliteBackend } from "./SqliteBackend.js";
 import { BridgeIDBFactory } from "./bridge-idb.js";
 import { IDBFactory } from "./idbtypes.js";
-import { createNodeSqlite3Impl } from "./node-sqlite3-impl.js";
+import { createNodeHelperSqlite3Impl } from "./node-helper-sqlite3-impl.js";
 
 let idbFactory: IDBFactory | undefined = undefined;
 
@@ -24,7 +24,11 @@ export async function initTestIndexedDB(): Promise<void> {
   // const backend = new MemoryBackend();
   // backend.enableTracing = true;
 
-  const sqlite3Impl = await createNodeSqlite3Impl();
+  const sqlite3Impl = await createNodeHelperSqlite3Impl({
+    enableTracing: false,
+  });
+
+  // const sqlite3Impl = await createNodeBetterSqlite3Impl();
 
   const backend = await createSqliteBackend(sqlite3Impl, {
     filename: ":memory:",
