@@ -15,18 +15,21 @@
  */
 
 /**
-*
-* @author Sebastian Javier Marchano (sebasjm)
-*/
-import { h, VNode } from 'preact';
-import { FooterBar } from '../styled';
+ *
+ * @author Sebastian Javier Marchano (sebasjm)
+ */
+import { h, VNode } from "preact";
+import { FooterBar } from "../styled/index.js";
 
 export function Footer(): VNode {
-  return <FooterBar>
-    <p>
-      <a href="https://taler.net/">Learn more about GNU Taler on our website.</a>
-      <p>Copyright &copy; 2014&mdash;2021 Taler Systems SA</p>
-    </p>
-  </FooterBar>
+  return (
+    <FooterBar>
+      <p>
+        <a href="https://taler.net/">
+          Learn more about GNU Taler on our website.
+        </a>
+        <p>Copyright &copy; 2014&mdash;2021 Taler Systems SA</p>
+      </p>
+    </FooterBar>
+  );
 }
-
