@@ -30,6 +30,9 @@ export function AdminRoutes(): VNode {
       <Route
         path={AdminPaths.list_instances}
         component={InstanceListPage}
+        onChangePassword={(id) => {
+          route(`/instance/${id}/password`);
+        }}
         onCreate={() => {
           route(AdminPaths.new_instance);
         }}
