@@ -29,6 +29,7 @@ import {
 } from "@gnu-taler/taler-util";
 import {
   BrowserHashNavigationProvider,
+  CommonPreferenceProvider,
   ConfigResultFail,
   MerchantApiProvider,
   TalerWalletIntegrationBrowserProvider,
@@ -84,6 +85,7 @@ import {
   fetchSettings,
 } from "./settings.js";
 import { revalidateInstanceCategories } from "./hooks/category.js";
+import { revalidateInstanceAccessTokens } from "./hooks/access-tokens.js";
 const WITH_LOCAL_STORAGE_CACHE = false;
 
 export function Application(): VNode {
@@ -96,57 +98,65 @@ export function Application(): VNode {
   const baseUrl = getInitialBackendBaseURL(settings.backendBaseURL);
   return (
     <SettingsProvider value={settings}>
-      <TranslationProvider
-        source={strings}
-        completeness={{
-          es: strings["es"].completeness,
-          de: strings["de"].completeness,
-        }}
-      >
-        <MerchantApiProvider
-          baseUrl={new URL("./", baseUrl)}
-          frameOnError={OnConfigError}
-          evictors={{
-            management: swrCacheEvictor,
+      <CommonPreferenceProvider showDebug={false}>
+        <TranslationProvider
+          source={strings}
+          completeness={{
+            uk: strings["uk"].completeness,
+            tr: strings["tr"].completeness,
+            // ru: strings["ru"].completeness,
+            sv: strings["sv"].completeness,
+            it: strings["it"].completeness,
+            fr: strings["fr"].completeness,
+            es: strings["es"].completeness,
+            de: strings["de"].completeness,
           }}
         >
-          <SessionContextProvider>
-            <SWRConfig
-              value={{
-                provider: WITH_LOCAL_STORAGE_CACHE
-                  ? localStorageProvider
-                  : undefined,
-                // normally, do not revalidate
-                revalidateOnFocus: false,
-                revalidateOnReconnect: false,
-                revalidateIfStale: false,
-                revalidateOnMount: undefined,
-                focusThrottleInterval: undefined,
+          <MerchantApiProvider
+            baseUrl={new URL("./", baseUrl)}
+            frameOnError={OnConfigError}
+            evictors={{
+              management: swrCacheEvictor,
+            }}
+          >
+            <SessionContextProvider>
+              <SWRConfig
+                value={{
+                  provider: WITH_LOCAL_STORAGE_CACHE
+                    ? localStorageProvider
+                    : undefined,
+                  // normally, do not revalidate
+                  revalidateOnFocus: false,
+                  revalidateOnReconnect: false,
+                  revalidateIfStale: false,
+                  revalidateOnMount: undefined,
+                  focusThrottleInterval: undefined,
 
-                // normally, do not refresh
-                refreshInterval: undefined,
-                dedupingInterval: 2000,
-                refreshWhenHidden: false,
-                refreshWhenOffline: false,
+                  // normally, do not refresh
+                  refreshInterval: undefined,
+                  dedupingInterval: 2000,
+                  refreshWhenHidden: false,
+                  refreshWhenOffline: false,
 
-                // ignore errors
-                shouldRetryOnError: false,
-                errorRetryCount: 0,
-                errorRetryInterval: undefined,
+                  // ignore errors
+                  shouldRetryOnError: false,
+                  errorRetryCount: 0,
+                  errorRetryInterval: undefined,
 
-                // do not go to loading again if already has data
-                keepPreviousData: true,
-              }}
-            >
-              <TalerWalletIntegrationBrowserProvider>
-                <BrowserHashNavigationProvider>
-                  <Routing />
-                </BrowserHashNavigationProvider>
-              </TalerWalletIntegrationBrowserProvider>
-            </SWRConfig>
-          </SessionContextProvider>
-        </MerchantApiProvider>
-      </TranslationProvider>
+                  // do not go to loading again if already has data
+                  keepPreviousData: true,
+                }}
+              >
+                <TalerWalletIntegrationBrowserProvider>
+                  <BrowserHashNavigationProvider>
+                    <Routing />
+                  </BrowserHashNavigationProvider>
+                </TalerWalletIntegrationBrowserProvider>
+              </SWRConfig>
+            </SessionContextProvider>
+          </MerchantApiProvider>
+        </TranslationProvider>
+      </CommonPreferenceProvider>
     </SettingsProvider>
   );
 }
@@ -199,7 +209,7 @@ function OnConfigError({
   state,
 }: {
   state:
-    | ConfigResultFail<TalerMerchantApi.TalerMerchantConfigResponse>
+    | ConfigResultFail<TalerMerchantApi.MerchantVersionResponse>
     | undefined;
 }): VNode {
   const { i18n } = useTranslationContext();
@@ -383,9 +393,20 @@ const swrCacheEvictor = new (class
         await Promise.all([revalidateTokenFamilies()]);
         return;
       }
+      case TalerMerchantInstanceCacheEviction.CREATE_ACCESSTOKEN: {
+        await Promise.all([revalidateInstanceAccessTokens()]);
+        return;
+      }
+      case TalerMerchantInstanceCacheEviction.DELETE_ACCESSTOKEN: {
+        await Promise.all([revalidateInstanceAccessTokens()]);
+        return;
+      }
       case TalerMerchantInstanceCacheEviction.LAST: {
         return;
       }
+      default: {
+        assertUnreachable(op);
+      }
     }
   }
 })();
