@@ -21,6 +21,7 @@
 
 import {
   AbsoluteTime,
+  HttpStatusCode,
   TalerError,
   TranslatedString,
 } from "@gnu-taler/taler-util";
@@ -37,13 +38,18 @@ import {
 } from "./components/menu/index.js";
 import { useSessionContext } from "./context/session.js";
 import { useInstanceBankAccounts } from "./hooks/bank.js";
-import { useInstanceKYCDetails } from "./hooks/instance.js";
+import { useInstanceKYCDetailsLongPolling } from "./hooks/instance.js";
 import { usePreference } from "./hooks/preference.js";
 import InstanceCreatePage from "./paths/admin/create/index.js";
 import InstanceListPage from "./paths/admin/list/index.js";
+import AccessTokenCreatePage from "./paths/instance/accessTokens/create/index.js";
+import AccessTokenListPage from "./paths/instance/accessTokens/list/index.js";
 import BankAccountCreatePage from "./paths/instance/accounts/create/index.js";
 import BankAccountListPage from "./paths/instance/accounts/list/index.js";
 import BankAccountUpdatePage from "./paths/instance/accounts/update/index.js";
+import CreateCategory from "./paths/instance/categories/create/index.js";
+import ListCategories from "./paths/instance/categories/list/index.js";
+import UpdateCategory from "./paths/instance/categories/update/index.js";
 import ListKYCPage from "./paths/instance/kyc/list/index.js";
 import OrderCreatePage from "./paths/instance/orders/create/index.js";
 import OrderDetailsPage from "./paths/instance/orders/details/index.js";
@@ -51,6 +57,10 @@ import OrderListPage from "./paths/instance/orders/list/index.js";
 import ValidatorCreatePage from "./paths/instance/otp_devices/create/index.js";
 import ValidatorListPage from "./paths/instance/otp_devices/list/index.js";
 import ValidatorUpdatePage from "./paths/instance/otp_devices/update/index.js";
+import PasswordPage, {
+  AdminPassword as InstanceAdminTokenPage,
+  Props as InstanceAdminTokenProps,
+} from "./paths/instance/password/index.js";
 import ProductCreatePage from "./paths/instance/products/create/index.js";
 import ProductListPage from "./paths/instance/products/list/index.js";
 import ProductUpdatePage from "./paths/instance/products/update/index.js";
@@ -59,7 +69,6 @@ import TemplateListPage from "./paths/instance/templates/list/index.js";
 import TemplateQrPage from "./paths/instance/templates/qr/index.js";
 import TemplateUpdatePage from "./paths/instance/templates/update/index.js";
 import TemplateUsePage from "./paths/instance/templates/use/index.js";
-import TokenPage from "./paths/instance/token/index.js";
 import TokenFamilyCreatePage from "./paths/instance/tokenfamilies/create/index.js";
 import TokenFamilyListPage from "./paths/instance/tokenfamilies/list/index.js";
 import TokenFamilyUpdatePage from "./paths/instance/tokenfamilies/update/index.js";
@@ -73,16 +82,18 @@ import WebhookCreatePage from "./paths/instance/webhooks/create/index.js";
 import WebhookListPage from "./paths/instance/webhooks/list/index.js";
 import WebhookUpdatePage from "./paths/instance/webhooks/update/index.js";
 import { LoginPage } from "./paths/login/index.js";
+import { NewAccount } from "./paths/newAccount/index.js";
 import { Settings } from "./paths/settings/index.js";
 import { Notification } from "./utils/types.js";
-import ListCategories from "./paths/instance/categories/list/index.js";
-import CreateCategory from "./paths/instance/categories/create/index.js";
-import UpdateCategory from "./paths/instance/categories/update/index.js";
+import { ResetAccount } from "./paths/resetAccount/index.js";
 
 export enum InstancePaths {
   error = "/error",
   settings = "/settings",
-  token = "/token",
+  password = "/password",
+
+  access_token_list = "/access-token",
+  access_token_new = "/access-token/new",
 
   bank_list = "/bank",
   bank_update = "/bank/:bid/update",
@@ -124,12 +135,16 @@ export enum InstancePaths {
   otp_devices_new = "/otp-devices/new",
 
   interface = "/interface",
+
+  newAccount = "/account/new",
+  resetAccount = "/account/reset/:id",
 }
 
 export enum AdminPaths {
   list_instances = "/instances",
   new_instance = "/instance/new",
   update_instance = "/instance/:id/update",
+  update_instance_auth = "/instance/:id/token",
 }
 
 export interface Props {}
@@ -145,7 +160,8 @@ export const publicPages = {
 
 const history = createHashHistory();
 export function Routing(_p: Props): VNode {
-  const { state } = useSessionContext();
+  const { state, config } = useSessionContext();
+  const { i18n } = useTranslationContext();
 
   type GlobalNotifState =
     | (Notification & { to: string | undefined })
@@ -159,8 +175,18 @@ export function Routing(_p: Props): VNode {
   const now = AbsoluteTime.now();
 
   const instance = useInstanceBankAccounts();
+
+  if (!instance) {
+    return (
+      <Fragment>
+        <NotConnectedAppMenu title={i18n.str`Welcome!`} />
+        <Loading />
+      </Fragment>
+    );
+  }
+
   const accounts =
-    !instance || instance instanceof TalerError || instance.type === "fail"
+    instance instanceof TalerError || instance.type === "fail"
       ? undefined
       : instance.body;
   const shouldWarnAboutMissingBankAccounts =
@@ -170,13 +196,32 @@ export function Routing(_p: Props): VNode {
     (AbsoluteTime.isNever(preference.hideMissingAccountUntil) ||
       AbsoluteTime.cmp(now, preference.hideMissingAccountUntil) > 0);
 
-  const shouldLogin = state.status === "loggedOut";
+  const unauthorized =
+    !(instance instanceof TalerError) &&
+    instance.type === "fail" &&
+    instance.case === HttpStatusCode.Unauthorized;
+
+  const shouldLogin = state.status === "loggedOut" || unauthorized;
 
   if (shouldLogin) {
     return (
       <Fragment>
-        <NotConnectedAppMenu title="Welcome!" />
-        <LoginPage />
+        <NotConnectedAppMenu title={i18n.str`Welcome!`} />
+        <Router history={history}>
+          <Route
+            path={InstancePaths.newAccount}
+            component={NewAccount}
+            onCancel={() => route(InstancePaths.order_list)}
+            onCreated={() => route(InstancePaths.order_list)}
+          />
+          <Route
+            path={InstancePaths.resetAccount}
+            component={ResetAccount}
+            onCancel={() => route(InstancePaths.order_list)}
+            onReseted={() => route(InstancePaths.order_list)}
+          />
+          <Route default component={() => <LoginPage showCreateAccount={config.have_self_provisioning} />} />
+        </Router>
       </Fragment>
     );
   }
@@ -251,6 +296,9 @@ export function Routing(_p: Props): VNode {
           <Route
             path={AdminPaths.list_instances}
             component={InstanceListPage}
+            onChangePassword={(id: string): void => {
+              route(`/instance/${id}/token`);
+            }}
             onCreate={() => {
               route(AdminPaths.new_instance);
             }}
@@ -279,6 +327,16 @@ export function Routing(_p: Props): VNode {
             }}
           />
         )}
+        {state.isAdmin && (
+          <Route
+            path={AdminPaths.update_instance_auth}
+            component={AdminInstanceUpdateTokenPage}
+            onCancel={() => route(AdminPaths.list_instances)}
+            onChange={() => {
+              route(AdminPaths.list_instances);
+            }}
+          />
+        )}
         {/**
          * Update instance page
          */}
@@ -296,8 +354,8 @@ export function Routing(_p: Props): VNode {
          * Update instance page
          */}
         <Route
-          path={InstancePaths.token}
-          component={TokenPage}
+          path={InstancePaths.password}
+          component={PasswordPage}
           onChange={() => {
             route(`/`);
           }}
@@ -404,6 +462,26 @@ export function Routing(_p: Props): VNode {
             route(InstancePaths.bank_list);
           }}
         />
+        {/**
+         * Sessions pages
+         */}
+        <Route
+          path={InstancePaths.access_token_list}
+          component={AccessTokenListPage}
+          onCreate={() => {
+            route(InstancePaths.access_token_new);
+          }}
+        />
+        <Route
+          path={InstancePaths.access_token_new}
+          component={AccessTokenCreatePage}
+          onConfirm={() => {
+            route(InstancePaths.access_token_list);
+          }}
+          onBack={() => {
+            route(InstancePaths.access_token_list);
+          }}
+        />
         {/**
          * Order pages
          */}
@@ -610,7 +688,13 @@ export function Routing(_p: Props): VNode {
           }}
         />
 
-        <Route path={InstancePaths.kyc} component={ListKYCPage} />
+        <Route
+          path={InstancePaths.kyc}
+          component={ListKYCPage}
+          // onSelect={(id: string) => {
+          //   route(InstancePaths.bank_update.replace(":bid", id));
+          // }}
+        />
         <Route path={InstancePaths.interface} component={Settings} />
         {/**
          * Example pages
@@ -640,6 +724,17 @@ function AdminInstanceUpdatePage({
   );
 }
 
+function AdminInstanceUpdateTokenPage({
+  id,
+  ...rest
+}: { id: string } & InstanceAdminTokenProps): VNode {
+  return (
+    <Fragment>
+      <InstanceAdminTokenPage {...rest} instanceId={id} />
+    </Fragment>
+  );
+}
+
 function BankAccountBanner(): VNode {
   const { i18n } = useTranslationContext();
 
@@ -677,7 +772,7 @@ function BankAccountBanner(): VNode {
 }
 
 function KycBanner(): VNode {
-  const kycStatus = useInstanceKYCDetails();
+  const kycStatus = useInstanceKYCDetailsLongPolling();
   const { i18n } = useTranslationContext();
   // const today = format(new Date(), dateFormatForSettings(settings));
   const [prefs, updatePref] = usePreference();
@@ -688,8 +783,10 @@ function KycBanner(): VNode {
     kycStatus !== undefined &&
     !(kycStatus instanceof TalerError) &&
     kycStatus.type === "ok" &&
-    !!kycStatus.body;
-
+    !!kycStatus.body &&
+    kycStatus.body.kyc_data.findIndex(
+      (d) => d.payto_kycauths !== undefined || d.access_token !== undefined,
+    ) !== -1;
   const hidden = AbsoluteTime.cmp(now, prefs.hideKycUntil) < 1;
   if (hidden || !needsToBeShown) return <Fragment />;
 
@@ -705,8 +802,9 @@ function KycBanner(): VNode {
           <div>
             <p>
               <i18n.Translate>
-                Some transfer are on hold until a KYC process is completed. Go
-                to the KYC section in the left panel for more information
+                Some transfers are on hold until the KYC process is completed.
+                Visit the KYC section in the left-hand menu for more
+                information.
               </i18n.Translate>
             </p>
             <div class="buttons is-right">
