@@ -26,13 +26,13 @@ import { useTranslationContext } from "@gnu-taler/web-util/browser";
 
 type Props = {
   children: ComponentChildren;
-  disabled: boolean;
+  disabled?: boolean;
   onClick?: () => Promise<void>;
   [rest: string]: any;
 };
 
 export function AsyncButton({ onClick, disabled, children, ...rest }: Props) {
-  const { isSlow, isLoading, request, cancel } = useAsync(onClick);
+  const { isSlow, isLoading, request, cancel, error } = useAsync(onClick);
   const { i18n } = useTranslationContext();
   if (isSlow) {
     return <LoadingModal onCancel={cancel} />;
@@ -50,6 +50,12 @@ export function AsyncButton({ onClick, disabled, children, ...rest }: Props) {
       <button class="button is-success" onClick={request} disabled={disabled}>
         {children}
       </button>
+
+      {!error ? undefined : (
+        <div class="message is-danger" style={{padding:2}}>
+          Unexpected error: {error}
+        </div>
+      )}
     </span>
   );
 }
