@@ -123,8 +123,13 @@ export function InputArray<T>({
             )}
           </div>
           {help}
-          {error && <p class="help is-danger"> {error} </p>}
-          
+          {error && (
+            <p class="help is-danger" style={{ fontSize: 16 }}>
+              {" "}
+              {error}{" "}
+            </p>
+          )}
+
           {suggestions.length > 0 ? (
             <div>
               <DropdownList
@@ -153,7 +158,9 @@ export function InputArray<T>({
                 class="tag is-medium is-danger is-delete mb-0"
                 onClick={() => {
                   onChange(array.filter((f) => f !== v) as T[keyof T]);
-                  setCurrentValue(getSuggestion ? (v as any).description : toStr(v));
+                  setCurrentValue(
+                    getSuggestion ? (v as any).description : toStr(v),
+                  );
                 }}
               />
             </div>
