@@ -18,6 +18,7 @@
  *
  * @author Sebastian Javier Marchano (sebasjm)
  */
+import { Duration } from "@gnu-taler/taler-util";
 import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { formatDuration, intervalToDuration } from "date-fns";
 import { ComponentChildren, h, VNode } from "preact";
@@ -25,7 +26,6 @@ import { useState } from "preact/hooks";
 import { SimpleModal } from "../modal/index.js";
 import { DurationPicker } from "../picker/DurationPicker.js";
 import { InputProps, useField } from "./useField.js";
-import { Duration } from "@gnu-taler/taler-util";
 
 export interface Props<T> extends InputProps<T> {
   expand?: boolean;
@@ -33,6 +33,7 @@ export interface Props<T> extends InputProps<T> {
   withForever?: boolean;
   side?: ComponentChildren;
   withoutClear?: boolean;
+  useProtocolDuration?: boolean;
 }
 
 export function InputDuration<T>({
@@ -46,6 +47,7 @@ export function InputDuration<T>({
   withForever,
   withoutClear,
   side,
+  useProtocolDuration,
 }: Props<keyof T>): VNode {
   const [opened, setOpened] = useState(false);
   const { i18n } = useTranslationContext();
@@ -53,7 +55,7 @@ export function InputDuration<T>({
   const { error, required, value: anyValue, onChange } = useField<T>(name);
   let strValue = "";
   const value: Duration =
-    anyValue.d_us !== undefined
+    anyValue && anyValue.d_us !== undefined
       ? Duration.fromTalerProtocolDuration(anyValue)
       : anyValue;
   if (!value) {
@@ -86,6 +88,8 @@ export function InputDuration<T>({
                 return i18n.str`${value}min`;
               case "xSeconds":
                 return i18n.str`${value}sec`;
+              default:
+                throw Error("not reached");
             }
           },
           localize: {
@@ -147,7 +151,11 @@ export function InputDuration<T>({
               </a>
             </div>
           </div>
-          {error && <p class="help is-danger">{error}</p>}
+          {error && (
+            <p class="help is-danger" style={{ fontSize: 16 }}>
+              {error}
+            </p>
+          )}
           <span class="has-text-grey">{help}</span>
         </div>
 
@@ -182,7 +190,12 @@ export function InputDuration<T>({
             minutes
             value={!value || value.d_ms === "forever" ? 0 : value.d_ms}
             onChange={(v) => {
-              onChange({ d_ms: v } as any);
+              let duration: any = { d_ms: v};
+              if (useProtocolDuration === true) {
+                onChange(Duration.toTalerProtocolDuration(duration) as any);
+              } else {
+                onChange(duration);
+              }
             }}
           />
         </SimpleModal>
