@@ -27,13 +27,6 @@ export function InputLocation({ name }: { name: string }) {
   return (
     <>
       <Input name={`${name}.country`} label={i18n.str`Country`} />
-      <Input
-        name={`${name}.address_lines`}
-        inputType="multiline"
-        label={i18n.str`Address`}
-        toStr={(v: string[] | undefined) => (!v ? "" : v.join("\n"))}
-        fromStr={(v: string) => v.split("\n")}
-      />
       <Input
         name={`${name}.building_number`}
         label={i18n.str`Building number`}
@@ -48,6 +41,13 @@ export function InputLocation({ name }: { name: string }) {
         name={`${name}.country_subdivision`}
         label={i18n.str`Country subdivision`}
       />
+      <Input
+        name={`${name}.address_lines`}
+        inputType="multiline"
+        label={i18n.str`Additional address lines`}
+        toStr={(v: string[] | undefined) => (!v ? "" : v.join("\n"))}
+        fromStr={(v: string) => v.split("\n")}
+      />
     </>
   );
 }
