@@ -19,7 +19,9 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 import {
+  InternationalizationAPI,
   PaytoUri,
+  TranslatedString,
   parsePaytoUri,
   stringifyPaytoUri,
 } from "@gnu-taler/taler-util";
@@ -28,19 +30,21 @@ import { Fragment, VNode, h } from "preact";
 import { useEffect, useState } from "preact/hooks";
 import { COUNTRY_TABLE } from "../../utils/constants.js";
 import { undefinedIfEmpty } from "../../utils/table.js";
-import { FormErrors, FormProvider } from "./FormProvider.js";
+import { FormErrors, FormProvider, TalerForm } from "./FormProvider.js";
 import { Input } from "./Input.js";
 import { InputGroup } from "./InputGroup.js";
 import { InputSelector } from "./InputSelector.js";
 import { InputProps, useField } from "./useField.js";
+import { useSettingsContext } from "../../context/settings.js";
+import { useSessionContext } from "../../context/session.js";
 
-export interface Props<T> extends InputProps<T> {}
+export interface Props<T> extends InputProps<T> { }
 
 // type Entity = PaytoUriGeneric
 // https://datatracker.ietf.org/doc/html/rfc8905
 type Entity = {
   // iban, bitcoin, x-taler-bank. it defined the format
-  target: string;
+  target: string | undefined;
   // path1 if the first field to be used
   path1?: string;
   // path2 if the second field to be used, optional
@@ -53,7 +57,7 @@ type Entity = {
     amount?: string;
     instruction?: string;
     [name: string]: string | undefined;
-  };
+  } & TalerForm;
 };
 
 function isEthereumAddress(address: string) {
@@ -76,20 +80,20 @@ function checkAddressChecksum(_address: string) {
 function validateBitcoin_path1(
   addr: string,
   i18n: ReturnType<typeof useTranslationContext>["i18n"],
-): string | undefined {
+): TranslatedString | undefined {
   try {
     const valid = /^(bc1|[13])[a-zA-HJ-NP-Z0-9]{25,39}$/.test(addr);
     if (valid) return undefined;
   } catch (e) {
     console.log(e);
   }
-  return i18n.str`This is not a valid bitcoin address.`;
+  return i18n.str`This is not a valid Bitcoin address.`;
 }
 
 function validateEthereum_path1(
   addr: string,
   i18n: ReturnType<typeof useTranslationContext>["i18n"],
-): string | undefined {
+): TranslatedString | undefined {
   try {
     const valid = isEthereumAddress(addr);
     if (valid) return undefined;
@@ -100,19 +104,25 @@ function validateEthereum_path1(
 }
 
 /**
- * validates
+ * validates "[host]:[port]/[path]/" where:
+ * host: can be localhost, bank.com
+ * port: any number
+ * path: may include subpath
+ *
+ * for example
+ * localhost
  * bank.com/
  * bank.com
  * bank.com/path
  * bank.com/path/subpath/
  */
 const DOMAIN_REGEX =
-  /^[a-zA-Z0-9][a-zA-Z0-9-_]{1,61}[a-zA-Z0-9-_](?:\.[a-zA-Z0-9-_]{2,})+(:[0-9]+)?(\/[a-zA-Z0-9-.]+)*\/?$/;
+  /^[a-zA-Z0-9][a-zA-Z0-9-_]{1,61}[a-zA-Z0-9-_](?:\.[a-zA-Z0-9-_]{2,})*(:[0-9]+)?(\/[a-zA-Z0-9-.]+)*\/?$/;
 
 function validateTalerBank_path1(
   addr: string,
   i18n: ReturnType<typeof useTranslationContext>["i18n"],
-): string | undefined {
+): TranslatedString | undefined {
   try {
     const valid = DOMAIN_REGEX.test(addr);
     if (valid) return undefined;
@@ -139,12 +149,11 @@ function validateTalerBank_path1(
 function validateIBAN_path1(
   iban: string,
   i18n: ReturnType<typeof useTranslationContext>["i18n"],
-): string | undefined {
+): TranslatedString | undefined {
   // Check total length
-  if (iban.length < 4)
-    return i18n.str`IBAN numbers usually have more that 4 digits`;
+  if (iban.length < 4) return i18n.str`IBANs usually have more than 4 digits.`;
   if (iban.length > 34)
-    return i18n.str`IBAN numbers usually have less that 34 digits`;
+    return i18n.str`IBANs usually have fewer than 34 digits.`;
 
   const A_code = "A".charCodeAt(0);
   const Z_code = "Z".charCodeAt(0);
@@ -152,7 +161,7 @@ function validateIBAN_path1(
   // check supported country
   const code = IBAN.substr(0, 2);
   const found = code in COUNTRY_TABLE;
-  if (!found) return i18n.str`IBAN country code not found`;
+  if (!found) return i18n.str`The IBAN's country code could not be retrieved.`;
 
   // 2.- Move the four initial characters to the end of the string
   const step2 = IBAN.substr(4) + iban.substr(0, 4);
@@ -177,10 +186,26 @@ function validateIBAN_path1(
 
   const checksum = calculate_iban_checksum(step3);
   if (checksum !== 1)
-    return i18n.str`IBAN number is invalid, checksum is wrong`;
+    return i18n.str`The IBAN is invalid because the checksum is wrong.`;
   return undefined;
 }
 
+const allTargets = ["iban", "bitcoin", "ethereum", "x-taler-bank"];
+
+function checkServerRegex(i18n: InternationalizationAPI, payto: string | undefined, serverReg: string | undefined) {
+  if (!payto || !serverReg) return undefined
+  try {
+    const validator = new RegExp(serverReg)
+    if (!validator.test(payto)) {
+      return i18n.str`This account is not allowed.`
+    } else {
+      return undefined
+    }
+  } catch (e) {
+    return undefined
+  }
+}
+
 export function InputPaytoForm<T>({
   name,
   readonly,
@@ -189,22 +214,28 @@ export function InputPaytoForm<T>({
 }: Props<keyof T>): VNode {
   const { value: initialValueStr, onChange } = useField<T>(name);
 
+  const { config } = useSessionContext();
+
   const initialPayto = parsePaytoUri(initialValueStr ?? "");
   const { i18n } = useTranslationContext();
 
-  const targets = [
-    i18n.str`Choose one...`,
-    "iban",
-    "bitcoin",
-    "ethereum",
-    "x-taler-bank",
-  ];
-  const noTargetValue = targets[0];
+  const supportedWireMethods =
+    config.payment_target_types === "*"
+      ? undefined
+      : config.payment_target_types.trim().split(" ");
+
+  // only the one supported by the server
+  const targets =
+    !supportedWireMethods || !supportedWireMethods.length
+      ? allTargets
+      : allTargets.filter((t) => supportedWireMethods.indexOf(t) !== -1);
+
   const defaultTarget: Entity = {
-    target: noTargetValue,
+    target: targets.length ? targets[0] : "",
     params: {},
   };
 
+  // FIXME: use new paytos API and EBICS is not supported
   const paths = !initialPayto ? [] : initialPayto.targetPath.split("/");
   const initialPath1 = paths.length >= 1 ? paths[0] : undefined;
   const initialPath2 = paths.length >= 2 ? paths[1] : undefined;
@@ -212,42 +243,32 @@ export function InputPaytoForm<T>({
     initialPayto === undefined
       ? defaultTarget
       : {
-          target: initialPayto.targetType,
-          params: initialPayto.params,
-          path1: initialPath1,
-          path2: initialPath2,
-        };
+        target: initialPayto.targetType,
+        params: initialPayto.params,
+        path1: initialPath1,
+        path2: initialPath2,
+      };
+
   const [value, setValue] = useState<Partial<Entity>>(initial);
-  useEffect(() => {
-    const nv = parsePaytoUri(initialValueStr ?? "");
-    const paths = !initialPayto ? [] : initialPayto.targetPath.split("/");
-    if (nv !== undefined && nv.isKnown) {
-      if (nv.targetType === "iban" && paths.length >= 2) {
-        //FIXME: workaround EBIC not supported
-        paths[0] = paths[1];
-        delete paths[1];
-      }
-      setValue({
-        target: nv.targetType,
-        params: nv.params,
-        path1: paths.length >= 1 ? paths[0] : undefined,
-        path2: paths.length >= 2 ? paths[1] : undefined,
-      });
-    }
-  }, [initialValueStr]);
 
   const errors = undefinedIfEmpty<FormErrors<Entity>>({
-    target: value.target === noTargetValue ? i18n.str`Required` : undefined,
+    target: value.target === undefined ? i18n.str`Required` : undefined,
     path1: !value.path1
       ? i18n.str`Required`
       : value.target === "iban"
-        ? validateIBAN_path1(value.path1, i18n)
+        ? validateIBAN_path1(cleanupPath1(value.path1, value.target), i18n)
         : value.target === "bitcoin"
-          ? validateBitcoin_path1(value.path1, i18n)
+          ? validateBitcoin_path1(cleanupPath1(value.path1, value.target), i18n)
           : value.target === "ethereum"
-            ? validateEthereum_path1(value.path1, i18n)
+            ? validateEthereum_path1(
+              cleanupPath1(value.path1, value.target),
+              i18n,
+            )
             : value.target === "x-taler-bank"
-              ? validateTalerBank_path1(value.path1, i18n)
+              ? validateTalerBank_path1(
+                cleanupPath1(value.path1, value.target),
+                i18n,
+              )
               : undefined,
     path2:
       value.target === "x-taler-bank"
@@ -264,42 +285,57 @@ export function InputPaytoForm<T>({
 
   const hasErrors = errors !== undefined;
 
-  const path1WithSlash =
-    value.path1 && !value.path1.endsWith("/") ? value.path1 + "/" : value.path1;
+  const path1WithSlash = !value.path1
+    ? undefined
+    : cleanupPath1(value.path1, value.target);
   const pto =
     hasErrors || !value.target
       ? undefined
       : ({
-          targetType: value.target,
-          targetPath: value.path2
-            ? `${path1WithSlash}${value.path2}`
-            : value.path1 ?? "",
-          params: value.params ?? {},
-          isKnown: false as const,
-        } as PaytoUri);
+        targetType: value.target,
+        targetPath: value.path2
+          ? `${path1WithSlash}${value.path2}`
+          : path1WithSlash ?? "",
+        params: value.params ?? {},
+        isKnown: false as const,
+      } as PaytoUri);
 
   const str = !pto ? undefined : stringifyPaytoUri(pto);
 
+  const regexError = checkServerRegex(i18n, str, config.payment_target_regex)
+
   useEffect(() => {
     onChange(str as T[keyof T]);
   }, [str]);
 
+  if (!targets.length) {
+    return (
+      <i18n.Translate>
+        None of the supported wire method of the server are currently supported
+        by this app. Server settings: {supportedWireMethods?.join(",") ?? "-"}
+      </i18n.Translate>
+    );
+  }
   return (
     <InputGroup name="payto" label={label} fixed tooltip={tooltip}>
       <FormProvider<Entity>
         name="tax"
-        errors={errors}
+        errors={regexError  ? {path1: regexError} : errors}
         object={value}
         valueHandler={setValue}
       >
-        <InputSelector<Entity>
-          name="target"
-          label={i18n.str`Type`}
-          tooltip={i18n.str`Method to use for wire transfer`}
-          values={targets}
-          readonly={readonly}
-          toStr={(v) => (v === noTargetValue ? i18n.str`Choose one...` : v)}
-        />
+        {targets.length === 1 ? undefined : (
+          <InputSelector<Entity>
+            name="target"
+            label={i18n.str`Wire method`}
+            tooltip={i18n.str`Select the method you want to use to transfer your earnings to your business account.`}
+            values={targets}
+            readonly={readonly}
+            toStr={(v) =>
+              v === undefined ? i18n.str`Select a wire method...` : v
+            }
+          />
+        )}
 
         {value.target === "ach" && (
           <Fragment>
@@ -307,13 +343,13 @@ export function InputPaytoForm<T>({
               name="path1"
               label={i18n.str`Routing`}
               readonly={readonly}
-              tooltip={i18n.str`Routing number.`}
+              tooltip={i18n.str`Routing number`}
             />
             <Input<Entity>
               name="path2"
               label={i18n.str`Account`}
               readonly={readonly}
-              tooltip={i18n.str`Account number.`}
+              tooltip={i18n.str`Account number`}
             />
           </Fragment>
         )}
@@ -323,7 +359,7 @@ export function InputPaytoForm<T>({
               name="path1"
               label={i18n.str`Code`}
               readonly={readonly}
-              tooltip={i18n.str`Business Identifier Code.`}
+              tooltip={i18n.str`Business Identifier Code`}
             />
           </Fragment>
         )}
@@ -332,9 +368,9 @@ export function InputPaytoForm<T>({
             <Input<Entity>
               name="path1"
               label={i18n.str`IBAN`}
-              tooltip={i18n.str`International Bank Account Number.`}
+              tooltip={i18n.str`International Bank Account Number`}
               readonly={readonly}
-              placeholder="DE1231231231"
+              placeholder="e.g. DE12 0000 1111 2222 3333 00"
               inputExtra={{ style: { textTransform: "uppercase" } }}
             />
           </Fragment>
@@ -345,7 +381,7 @@ export function InputPaytoForm<T>({
               name="path1"
               readonly={readonly}
               label={i18n.str`Account`}
-              tooltip={i18n.str`Unified Payment Interface.`}
+              tooltip={i18n.str`Unified Payment Interface`}
             />
           </Fragment>
         )}
@@ -355,7 +391,7 @@ export function InputPaytoForm<T>({
               name="path1"
               readonly={readonly}
               label={i18n.str`Address`}
-              tooltip={i18n.str`Bitcoin protocol.`}
+              tooltip={i18n.str`Bitcoin protocol`}
             />
           </Fragment>
         )}
@@ -365,7 +401,7 @@ export function InputPaytoForm<T>({
               name="path1"
               readonly={readonly}
               label={i18n.str`Address`}
-              tooltip={i18n.str`Ethereum protocol.`}
+              tooltip={i18n.str`Ethereum protocol`}
             />
           </Fragment>
         )}
@@ -375,7 +411,7 @@ export function InputPaytoForm<T>({
               name="path1"
               readonly={readonly}
               label={i18n.str`Address`}
-              tooltip={i18n.str`Interledger protocol.`}
+              tooltip={i18n.str`Interledger protocol`}
             />
           </Fragment>
         )}
@@ -397,12 +433,13 @@ export function InputPaytoForm<T>({
                 }
                 return v;
               }}
-              tooltip={i18n.str`Bank host.`}
+              tooltip={i18n.str`Bank host`}
               help={
                 <Fragment>
                   <div>
                     <i18n.Translate>
-                      Without scheme and may include subpath:
+                      Enter the data without a scheme. A subpath may be
+                      included:
                     </i18n.Translate>
                   </div>
                   <div>bank.com/</div>
@@ -414,7 +451,7 @@ export function InputPaytoForm<T>({
               name="path2"
               readonly={readonly}
               label={i18n.str`Account`}
-              tooltip={i18n.str`Bank account.`}
+              tooltip={i18n.str`Bank account`}
             />
           </Fragment>
         )}
@@ -422,15 +459,15 @@ export function InputPaytoForm<T>({
         {/**
          * Show additional fields apart from the payto
          */}
-        {value.target !== noTargetValue && (
+        {value.target !== undefined && (
           <Fragment>
             <Input
               name="params.receiver-name"
               readonly={readonly}
-              label={i18n.str`Owner's name`}
+              label={i18n.str`Name of the account holder`}
               placeholder="John Doe"
               tooltip={i18n.str`Legal name of the person holding the account.`}
-              help={i18n.str`It should match the bank account name.`}
+              help={i18n.str`Thissupports Taler Revenue API then you can addthe name of the bank account holder.`}
             />
           </Fragment>
         )}
@@ -438,3 +475,14 @@ export function InputPaytoForm<T>({
     </InputGroup>
   );
 }
+function cleanupPath1(str: string, type?: string) {
+  if (type === "iban") {
+    // we tolerate country in any case
+    // and don't care about any space
+    return str.replace(/\s/g, "").toUpperCase();
+  }
+  if (type === "x-taler-bank") {
+    return !str.endsWith("/") ? str + "/" : str;
+  }
+  return str;
+}
