@@ -27,7 +27,7 @@ import { InputWithAddon } from "./InputWithAddon.js";
 import { TranslatedString } from "@gnu-taler/taler-util";
 
 type Entity = {
-  id: string,
+  id: string;
   description: string;
   image?: string;
   extra?: string;
@@ -64,13 +64,13 @@ export function InputSearchOnList<T extends Entity>({
   if (selected) {
     return (
       <article class="media">
-        {withImage &&
+        {withImage && (
           <figure class="media-left">
             <p class="image is-128x128">
               <img src={selected.image ? selected.image : emptyImage} />
             </p>
           </figure>
-        }
+        )}
         <div class="media-content">
           <div class="content">
             <p class="media-meta">
@@ -85,7 +85,7 @@ export function InputSearchOnList<T extends Entity>({
                 class="button is-info"
                 onClick={() => onChange(undefined)}
               >
-                clear
+                <i18n.Translate>Clear</i18n.Translate>
               </button>
             </div>
           </div>
@@ -133,7 +133,12 @@ interface DropdownListProps<T extends Entity> {
   withImage: boolean;
 }
 
-export function DropdownList<T extends Entity>({ name, onSelect, list, withImage }: DropdownListProps<T>) {
+export function DropdownList<T extends Entity>({
+  name,
+  onSelect,
+  list,
+  withImage,
+}: DropdownListProps<T>) {
   const { i18n } = useTranslationContext();
   if (!name) {
     /* FIXME
@@ -146,8 +151,10 @@ export function DropdownList<T extends Entity>({ name, onSelect, list, withImage
       </div>
     );
   }
+
+  const MATCH_NAME = new RegExp(name, "i");
   const filtered = list.filter(
-    (p) => p.id.includes(name) || p.description.includes(name),
+    (p) => MATCH_NAME.test(p.id) || MATCH_NAME.test(p.description),
   );
 
   return (
@@ -162,7 +169,7 @@ export function DropdownList<T extends Entity>({ name, onSelect, list, withImage
           {!filtered.length ? (
             <div class="dropdown-item">
               <i18n.Translate>
-                no match found with that description or id
+                No match found for that description or ID.
               </i18n.Translate>
             </div>
           ) : (
@@ -174,7 +181,7 @@ export function DropdownList<T extends Entity>({ name, onSelect, list, withImage
                 style={{ cursor: "pointer" }}
               >
                 <article class="media">
-                  {withImage &&
+                  {withImage && (
                     <div class="media-left">
                       <div class="image" style={{ minWidth: 64 }}>
                         <img
@@ -183,11 +190,14 @@ export function DropdownList<T extends Entity>({ name, onSelect, list, withImage
                         />
                       </div>
                     </div>
-                  }
+                  )}
                   <div class="media-content">
                     <div class="content">
                       <p>
-                        <strong>{p.id}</strong> {p.extra !== undefined ? <small>{p.extra}</small> : undefined}
+                        <strong>{p.id}</strong>{" "}
+                        {p.extra !== undefined ? (
+                          <small>{p.extra}</small>
+                        ) : undefined}
                         <br />
                         {p.description}
                       </p>
