@@ -110,8 +110,7 @@ export function InputTaxes<T>({ name, label }: Props<keyof T>): VNode {
           tooltip={i18n.str`Taxes can be in currencies that differ from the main currency used by the merchant.`}
         >
           <i18n.Translate>
-            Enter currency and value separated with a colon, e.g.
-            &quot;USD:2.3&quot;.
+            Enter the currency and value separated by a colon (e.g., &quot;USD:2.3&quot;).
           </i18n.Translate>
         </Input>
 
