@@ -97,7 +97,11 @@ export function InputToggle<T>({
             </label>
             <p>{help}</p>
           </p>
-          {error && <p class="help is-danger">{error}</p>}
+          {error && (
+            <p class="help is-danger" style={{ fontSize: 16 }}>
+              {error}
+            </p>
+          )}
         </div>
         {side}
       </div>
