@@ -50,7 +50,7 @@ export function DefaultInstanceFormFields({
           addonBefore={new URL("instances/", state.backendUrl.href).href}
           readonly={readonlyId}
           label={i18n.str`Identifier`}
-          tooltip={i18n.str`Name of the instance in URLs. The 'default' instance is special in that it is used to administer other instances.`}
+          tooltip={i18n.str`Name of the instance in URLs. The 'admin' instance is special in that it is used to administer other instances.`}
         />
       )}
 
@@ -68,6 +68,12 @@ export function DefaultInstanceFormFields({
             tooltip={i18n.str`Contact email`}
           />
 
+          <Input<Entity>
+            name="phone_number"
+            label={i18n.str`Phone number`}
+            tooltip={i18n.str`Contact phone`}
+          />
+
           <Input<Entity>
             name="website"
             label={i18n.str`Website URL`}
@@ -99,7 +105,7 @@ export function DefaultInstanceFormFields({
           <InputToggle<Entity>
             name="use_stefan"
             label={i18n.str`Pay transaction fee`}
-            tooltip={i18n.str`Assume the cost of the transaction of let the user pay for it.`}
+            tooltip={i18n.str`Cover the transaction cost or pass it on to the user.`}
           />
 
           <InputDuration<Entity>
