@@ -58,14 +58,12 @@ export function NavigationBar({ onMobileMenu, title }: Props): VNode {
       <div class="navbar-menu ">
         <a
           class="navbar-start is-justify-content-center is-flex-grow-1"
-          href="https://taler.net"
+          href="https://tutorials.taler.net/merchant-backoffice"
+          target="_blank"
+          rel="noreferrer"
         >
           <img src={logo} style={{ height: 35, margin: 10 }} />
         </a>
-        <div class="navbar-end">
-          <div class="navbar-item" style={{ paddingTop: 4, paddingBottom: 4 }}>
-          </div>
-        </div>
       </div>
     </nav>
   );
