@@ -19,13 +19,17 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { TalerError } from "@gnu-taler/taler-util";
+import {
+  getMerchantAccountKycStatusSimplified,
+  MerchantAccountKycStatusSimplified,
+  TalerError,
+} from "@gnu-taler/taler-util";
 import { useTranslationContext } from "@gnu-taler/web-util/browser";
-import { Fragment, VNode, h } from "preact";
+import { Fragment, h, VNode } from "preact";
 import { useSessionContext } from "../../context/session.js";
-import { useInstanceKYCDetails } from "../../hooks/instance.js";
-import { LangSelector } from "./LangSelector.js";
+import { useInstanceKYCDetailsLongPolling } from "../../hooks/instance.js";
 import { usePreference } from "../../hooks/preference.js";
+import { LangSelector } from "./LangSelector.js";
 
 // const GIT_HASH = typeof __GIT_HASH__ !== "undefined" ? __GIT_HASH__ : undefined;
 const VERSION = typeof __VERSION__ !== "undefined" ? __VERSION__ : undefined;
@@ -37,14 +41,23 @@ interface Props {
 export function Sidebar({ mobile }: Props): VNode {
   const { i18n } = useTranslationContext();
   const { state, logOut, config } = useSessionContext();
-  const kycStatus = useInstanceKYCDetails();
+  const kycStatus = useInstanceKYCDetailsLongPolling();
   const [pref] = usePreference();
 
-  const needKYC =
+  const allKycData =
     kycStatus !== undefined &&
     !(kycStatus instanceof TalerError) &&
     kycStatus.type === "ok" &&
-    !!kycStatus.body;
+    !!kycStatus.body
+      ? kycStatus.body.kyc_data
+      : [];
+
+  const simplifiedKycStatus = allKycData.reduce((prev, cur) => {
+    const st = getMerchantAccountKycStatusSimplified(cur.status);
+    if (st > prev) return st;
+    return prev;
+  }, MerchantAccountKycStatusSimplified.OK);
+
   const isLoggedIn = state.status === "loggedIn";
   const hasToken = isLoggedIn && state.token !== undefined;
 
@@ -81,7 +94,7 @@ export function Sidebar({ mobile }: Props): VNode {
           <Fragment>
             <ul class="menu-list">
               <li>
-                <a href={"/orders"} class="has-icon">
+                <a href={"#/orders"} class="has-icon">
                   <span class="icon">
                     <i class="mdi mdi-cash-register" />
                   </span>
@@ -91,7 +104,7 @@ export function Sidebar({ mobile }: Props): VNode {
                 </a>
               </li>
               <li>
-                <a href={"/inventory"} class="has-icon">
+                <a href={"#/inventory"} class="has-icon">
                   <span class="icon">
                     <i class="mdi mdi-shopping" />
                   </span>
@@ -101,7 +114,7 @@ export function Sidebar({ mobile }: Props): VNode {
                 </a>
               </li>
               <li>
-                <a href={"/category"} class="has-icon">
+                <a href={"#/category"} class="has-icon">
                   <span class="icon">
                     <i class="mdi mdi-label-outline" />
                   </span>
@@ -111,17 +124,17 @@ export function Sidebar({ mobile }: Props): VNode {
                 </a>
               </li>
               <li>
-                <a href={"/transfers"} class="has-icon">
+                <a href={"#/transfers"} class="has-icon">
                   <span class="icon">
                     <i class="mdi mdi-arrow-left-right" />
                   </span>
                   <span class="menu-item-label">
-                    <i18n.Translate>Transfers</i18n.Translate>
+                    <i18n.Translate>Wire transfers</i18n.Translate>
                   </span>
                 </a>
               </li>
               <li>
-                <a href={"/templates"} class="has-icon">
+                <a href={"#/templates"} class="has-icon">
                   <span class="icon">
                     <i class="mdi mdi-qrcode" />
                   </span>
@@ -132,9 +145,9 @@ export function Sidebar({ mobile }: Props): VNode {
               </li>
               {pref.developerMode ? (
                 <li>
-                  <a href={"/tokenfamilies"} class="has-icon">
+                  <a href={"#/tokenfamilies"} class="has-icon">
                     <span class="icon">
-                      <i class="mdi mdi-key" />
+                      <i class="mdi mdi-clock" />
                     </span>
                     <span class="menu-item-label">
                       <i18n.Translate>Token Families</i18n.Translate>
@@ -142,9 +155,46 @@ export function Sidebar({ mobile }: Props): VNode {
                   </a>
                 </li>
               ) : undefined}
-              {needKYC && (
-                <li>
-                  <a href={"/kyc"} class="has-icon">
+              {!allKycData.length ? undefined : (
+                <li
+                  class={
+                    simplifiedKycStatus ===
+                    MerchantAccountKycStatusSimplified.WARNING
+                      ? "is-warning"
+                      : simplifiedKycStatus ===
+                          MerchantAccountKycStatusSimplified.ERROR
+                        ? "is-error"
+                        : simplifiedKycStatus ===
+                            MerchantAccountKycStatusSimplified.ACTION_REQUIRED
+                          ? "is-warning"
+                          : undefined
+                  }
+                >
+                  <a
+                    href={"#/kyc"}
+                    class="has-icon"
+                    style={
+                      simplifiedKycStatus ===
+                      MerchantAccountKycStatusSimplified.WARNING
+                        ? {
+                            backgroundColor: "darkorange",
+                            color: "black",
+                          }
+                        : simplifiedKycStatus ===
+                            MerchantAccountKycStatusSimplified.ERROR
+                          ? {
+                              backgroundColor: "darkred",
+                              color: "black",
+                            }
+                          : simplifiedKycStatus ===
+                              MerchantAccountKycStatusSimplified.ACTION_REQUIRED
+                            ? {
+                                backgroundColor: "darkorange",
+                                color: "black",
+                              }
+                            : undefined
+                    }
+                  >
                     <span class="icon">
                       <i class="mdi mdi-account-check" />
                     </span>
@@ -158,7 +208,7 @@ export function Sidebar({ mobile }: Props): VNode {
             </p>
             <ul class="menu-list">
               <li>
-                <a href={"/bank"} class="has-icon">
+                <a href={"#/bank"} class="has-icon">
                   <span class="icon">
                     <i class="mdi mdi-bank" />
                   </span>
@@ -168,7 +218,7 @@ export function Sidebar({ mobile }: Props): VNode {
                 </a>
               </li>
               <li>
-                <a href={"/otp-devices"} class="has-icon">
+                <a href={"#/otp-devices"} class="has-icon">
                   <span class="icon">
                     <i class="mdi mdi-lock" />
                   </span>
@@ -178,7 +228,7 @@ export function Sidebar({ mobile }: Props): VNode {
                 </a>
               </li>
               <li>
-                <a href={"/webhooks"} class="has-icon">
+                <a href={"#/webhooks"} class="has-icon">
                   <span class="icon">
                     <i class="mdi mdi-webhook" />
                   </span>
@@ -188,7 +238,7 @@ export function Sidebar({ mobile }: Props): VNode {
                 </a>
               </li>
               <li>
-                <a href={"/settings"} class="has-icon">
+                <a href={"#/settings"} class="has-icon">
                   <span class="icon">
                     <i class="mdi mdi-square-edit-outline" />
                   </span>
@@ -198,12 +248,22 @@ export function Sidebar({ mobile }: Props): VNode {
                 </a>
               </li>
               <li>
-                <a href={"/token"} class="has-icon">
+                <a href={"#/password"} class="has-icon">
                   <span class="icon">
                     <i class="mdi mdi-security" />
                   </span>
                   <span class="menu-item-label">
-                    <i18n.Translate>Access token</i18n.Translate>
+                    <i18n.Translate>Password</i18n.Translate>
+                  </span>
+                </a>
+              </li>
+              <li>
+                <a href={"#/access-token"} class="has-icon">
+                  <span class="icon">
+                    <i class="mdi mdi-key" />
+                  </span>
+                  <span class="menu-item-label">
+                    <i18n.Translate>Access tokens</i18n.Translate>
                   </span>
                 </a>
               </li>
@@ -215,7 +275,7 @@ export function Sidebar({ mobile }: Props): VNode {
         </p>
         <ul class="menu-list">
           <li>
-            <a class="has-icon is-state-info is-hoverable" href="/interface">
+            <a class="has-icon is-state-info is-hoverable" href="#/interface">
               <span class="icon">
                 <i class="mdi mdi-newspaper" />
               </span>
@@ -246,7 +306,7 @@ export function Sidebar({ mobile }: Props): VNode {
                 <i18n.Translate>Instances</i18n.Translate>
               </p>
               <li>
-                <a href={"/instance/new"} class="has-icon">
+                <a href={"#/instance/new"} class="has-icon">
                   <span class="icon">
                     <i class="mdi mdi-plus" />
                   </span>
@@ -256,7 +316,7 @@ export function Sidebar({ mobile }: Props): VNode {
                 </a>
               </li>
               <li>
-                <a href={"/instances"} class="has-icon">
+                <a href={"#/instances"} class="has-icon">
                   <span class="icon">
                     <i class="mdi mdi-format-list-bulleted" />
                   </span>
