@@ -23,6 +23,7 @@ import { NavigationBar } from "./NavigationBar.js";
 import { Sidebar } from "./SideBar.js";
 import { useSessionContext } from "../../context/session.js";
 import { useNavigationContext } from "@gnu-taler/web-util/browser";
+import { LangSelector } from "./LangSelector.js";
 
 function getInstanceTitle(path: string, id: string): string {
   switch (path) {
@@ -51,7 +52,7 @@ function getInstanceTitle(path: string, id: string): string {
     case InstancePaths.category_update:
       return `${id}: Update category`;
     case InstancePaths.transfers_list:
-      return `${id}: Transfers`;
+      return `${id}: Wire transfers`;
     case InstancePaths.transfers_new:
       return `${id}: New transfer`;
     case InstancePaths.webhooks_list:
@@ -82,8 +83,13 @@ function getInstanceTitle(path: string, id: string): string {
       return `${id}: New token family`;
     case InstancePaths.token_family_update:
       return `${id}: Update token family`;
-    default:
+    case InstancePaths.access_token_list:
+      return `${id}: Access tokens`;
+    case InstancePaths.access_token_new:
+      return `${id}: New access token`;
+    default: {
       return "";
+    }
   }
 }
 
@@ -228,6 +234,19 @@ export function NotConnectedAppMenu({
         onMobileMenu={() => setMobileOpen(!mobileOpen)}
         title={title}
       />
+      <div
+        style={{
+          width: "100%",
+          display: "flex",
+          justifyContent: "space-between",
+          paddingTop:8,
+          paddingRight: 8,
+          
+        }}
+      >
+        <div />
+        <LangSelector />
+      </div>
     </div>
   );
 }
