@@ -21,6 +21,7 @@
 
 import { h, VNode } from "preact";
 import { MessageType, Notification } from "../../utils/types.js";
+import { assertUnreachable } from "@gnu-taler/taler-util";
 
 interface Props {
   notifications: Notification[];
@@ -37,8 +38,9 @@ function messageStyle(type: MessageType): string {
       return "message is-danger";
     case "SUCCESS":
       return "message is-success";
-    default:
-      return "message";
+    default: {
+      assertUnreachable(type);
+    }
   }
 }
 
