@@ -26,7 +26,7 @@ import {
   TalerMerchantApi,
 } from "@gnu-taler/taler-util";
 import { Loading, useTranslationContext } from "@gnu-taler/web-util/browser";
-import { h } from "preact";
+import { Fragment, h } from "preact";
 import { useCallback, useEffect, useState } from "preact/hooks";
 import { useSessionContext } from "../../context/session.js";
 import { undefinedIfEmpty } from "../../utils/table.js";
@@ -41,6 +41,7 @@ import { InputWithAddon } from "../form/InputWithAddon.js";
 import { InputArray } from "../form/InputArray.js";
 import { useInstanceCategories } from "../../hooks/category.js";
 import { ErrorLoadingMerchant } from "../ErrorLoadingMerchant.js";
+import { usePreference } from "../../hooks/preference.js";
 
 type Entity = TalerMerchantApi.ProductDetail & {
   product_id: string;
@@ -55,6 +56,8 @@ interface Props {
 export function ProductForm({ onSubscribe, initial, alreadyExist }: Props) {
   const { i18n } = useTranslationContext();
   const { state, lib } = useSessionContext();
+  const [preference] = usePreference();
+
   // FIXME: if the category list is big the will bring a lot of info
   // we could find a lazy way to add up on searches
   const categoriesResult = useInstanceCategories();
@@ -112,6 +115,7 @@ export function ProductForm({ onSubscribe, initial, alreadyExist }: Props) {
 
   const errors = undefinedIfEmpty({
     product_id: !value.product_id ? i18n.str`Required` : undefined,
+    product_name: !value.product_name ? i18n.str`Required` : undefined,
     description: !value.description ? i18n.str`Required` : undefined,
     unit: !value.unit ? i18n.str`Required` : undefined,
     price: !value.price
@@ -173,11 +177,23 @@ export function ProductForm({ onSubscribe, initial, alreadyExist }: Props) {
         object={value}
         valueHandler={valueHandler}
       >
+        {/**
+         * If the user press enter on any text field it will the browser will trigger
+         * the first button that it found.
+         *
+         * In this form the InputImage will be triggered and this is unwanted.
+         *
+         * As a workaround we have this non-action button which will prevent loading/unloading
+         * the image when the enter key is pressed accidentally.
+         */}
+        <button style={{ visibility: "hidden" }} />
         {alreadyExist ? undefined : (
           <InputWithAddon<Entity>
             name="product_id"
             addonBefore={new URL("product/", state.backendUrl.href).href}
             label={i18n.str`ID`}
+            addonBeforeLarge={state.backendUrl.href.length > 50}
+            expand
             tooltip={i18n.str`Product identification to use in URLs (for internal use only).`}
           />
         )}
@@ -186,6 +202,11 @@ export function ProductForm({ onSubscribe, initial, alreadyExist }: Props) {
           label={i18n.str`Image`}
           tooltip={i18n.str`Illustration of the product for customers.`}
         />
+        <Input<Entity>
+          name="product_name"
+          label={i18n.str`Name`}
+          tooltip={i18n.str`Product name.`}
+        />
         <Input<Entity>
           name="description"
           inputType="multiline"
