@@ -19,14 +19,11 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { TalerMerchantApi } from "@gnu-taler/taler-util";
+import { MerchantAuthMethod, TalerMerchantApi } from "@gnu-taler/taler-util";
 import { useMerchantApiContext } from "@gnu-taler/web-util/browser";
 import * as tests from "@gnu-taler/web-util/testing";
 import { expect } from "chai";
-import {
-  useBackendInstances,
-  useInstanceDetails,
-} from "./instance.js";
+import { useBackendInstances, useInstanceDetails } from "./instance.js";
 import { ApiMockEnvironment } from "./testing.js";
 import {
   API_CREATE_INSTANCE,
@@ -51,7 +48,7 @@ describe("instance api interaction with details", () => {
     const hookBehavior = await tests.hookBehaveLikeThis(
       () => {
         // const api = useInstanceAPI();
-        const { lib: api } = useMerchantApiContext()
+        const { lib: api } = useMerchantApiContext();
         const query = useInstanceDetails();
         return { query, api };
       },
@@ -119,7 +116,7 @@ describe("instance api interaction with details", () => {
 
     const hookBehavior = await tests.hookBehaveLikeThis(
       () => {
-        const { lib: api } = useMerchantApiContext()
+        const { lib: api } = useMerchantApiContext();
         const query = useInstanceDetails();
         return { query, api };
       },
@@ -144,7 +141,7 @@ describe("instance api interaction with details", () => {
           env.addRequestExpectation(API_UPDATE_CURRENT_INSTANCE_AUTH, {
             request: {
               method: "token",
-              token: "secret",
+              password: "secret",
             } as TalerMerchantApi.InstanceAuthConfigurationMessage,
           });
           // env.addRequestExpectation(API_NEW_LOGIN, {
@@ -155,7 +152,7 @@ describe("instance api interaction with details", () => {
           //       "d_us": "forever",
           //     },
           //     refreshable: true,
-          //   },            
+          //   },
           // });
           env.addRequestExpectation(API_GET_CURRENT_INSTANCE, {
             response: {
@@ -205,7 +202,7 @@ describe("instance api interaction with details", () => {
 
     const hookBehavior = await tests.hookBehaveLikeThis(
       () => {
-        const { lib: api } = useMerchantApiContext()
+        const { lib: api } = useMerchantApiContext();
         const query = useInstanceDetails();
         return { query, api };
       },
@@ -230,20 +227,23 @@ describe("instance api interaction with details", () => {
           // });
           env.addRequestExpectation(API_UPDATE_CURRENT_INSTANCE_AUTH, {
             request: {
-              method: "external",
+              method: "token",
+              password: "asd"
             } as TalerMerchantApi.InstanceAuthConfigurationMessage,
           });
           env.addRequestExpectation(API_GET_CURRENT_INSTANCE, {
             response: {
               name: "instance_name",
-              auth: {
-                method: "external",
-              },
+              // auth: {
+              //   method: "token",
+              //   password: "asd"
+              // }
             } as TalerMerchantApi.QueryInstancesResponse,
           });
 
           api.instance.updateCurrentInstanceAuthentication(undefined, {
-            method: "external"
+            method: MerchantAuthMethod.TOKEN,
+            password: "asd"
           });
         },
         ({ query, api }) => {
@@ -338,7 +338,7 @@ describe("instance admin api interaction with listing", () => {
 
     const hookBehavior = await tests.hookBehaveLikeThis(
       () => {
-        const { lib: api } = useMerchantApiContext()
+        const { lib: api } = useMerchantApiContext();
         const query = useBackendInstances();
         return { query, api };
       },
@@ -382,7 +382,7 @@ describe("instance admin api interaction with listing", () => {
 
           api.instance.createInstance(undefined, {
             name: "other_name",
-          } as TalerMerchantApi.InstanceConfigurationMessage)
+          } as TalerMerchantApi.InstanceConfigurationMessage);
         },
         ({ query, api }) => {
           expect(env.assertJustExpectedRequestWereMade()).deep.eq({
@@ -416,7 +416,7 @@ describe("instance admin api interaction with listing", () => {
       response: {
         instances: [
           {
-            id: "default",
+            id: "admin",
             name: "instance_name",
           } as TalerMerchantApi.Instance,
           {
@@ -429,7 +429,7 @@ describe("instance admin api interaction with listing", () => {
 
     const hookBehavior = await tests.hookBehaveLikeThis(
       () => {
-        const { lib: api } = useMerchantApiContext()
+        const { lib: api } = useMerchantApiContext();
         const query = useBackendInstances();
         return { query, api };
       },
@@ -448,7 +448,7 @@ describe("instance admin api interaction with listing", () => {
           // expect(query.data).deep.equals({
           //   instances: [
           //     {
-          //       id: "default",
+          //       id: "admin",
           //       name: "instance_name",
           //     },
           //     {
@@ -463,7 +463,7 @@ describe("instance admin api interaction with listing", () => {
             response: {
               instances: [
                 {
-                  id: "default",
+                  id: "admin",
                   name: "instance_name",
                 } as TalerMerchantApi.Instance,
               ],
@@ -482,7 +482,7 @@ describe("instance admin api interaction with listing", () => {
           // expect(query.data).deep.equals({
           //   instances: [
           //     {
-          //       id: "default",
+          //       id: "admin",
           //       name: "instance_name",
           //     },
           //   ],
@@ -521,7 +521,7 @@ describe("instance admin api interaction with listing", () => {
 
     // expect(result.current.query.data).equals({
     //   instances: [{
-    //     id: 'default',
+    //     id: 'admin',
     //     name: 'instance_name'
     //   }, {
     //     id: 'the_id',
@@ -540,7 +540,7 @@ describe("instance admin api interaction with listing", () => {
     // env.addRequestExpectation(API_LIST_INSTANCES, {
     //   response: {
     //     instances: [{
-    //       id: 'default',
+    //       id: 'admin',
     //       name: 'instance_name'
     //     } as TalerMerchantApi.Instance]
     //   },
@@ -557,7 +557,7 @@ describe("instance admin api interaction with listing", () => {
 
     // expect(result.current.query.data).equals({
     //   instances: [{
-    //     id: 'default',
+    //     id: 'admin',
     //     name: 'instance_name'
     //   }]
     // });
@@ -570,7 +570,7 @@ describe("instance admin api interaction with listing", () => {
       response: {
         instances: [
           {
-            id: "default",
+            id: "admin",
             name: "instance_name",
           } as TalerMerchantApi.Instance,
           {
@@ -583,7 +583,7 @@ describe("instance admin api interaction with listing", () => {
 
     const hookBehavior = await tests.hookBehaveLikeThis(
       () => {
-        const { lib: api } = useMerchantApiContext()
+        const { lib: api } = useMerchantApiContext();
         const query = useBackendInstances();
         return { query, api };
       },
@@ -602,7 +602,7 @@ describe("instance admin api interaction with listing", () => {
           // expect(query.data).deep.equals({
           //   instances: [
           //     {
-          //       id: "default",
+          //       id: "admin",
           //       name: "instance_name",
           //     },
           //     {
@@ -621,14 +621,14 @@ describe("instance admin api interaction with listing", () => {
             response: {
               instances: [
                 {
-                  id: "default",
+                  id: "admin",
                   name: "instance_name",
                 } as TalerMerchantApi.Instance,
               ],
             },
           });
 
-          api.instance.deleteInstance(undefined, "the_id", { purge: true })
+          api.instance.deleteInstance(undefined, "the_id", { purge: true });
         },
         ({ query, api }) => {
           expect(env.assertJustExpectedRequestWereMade()).deep.eq({
@@ -640,7 +640,7 @@ describe("instance admin api interaction with listing", () => {
           // expect(query.data).deep.equals({
           //   instances: [
           //     {
-          //       id: "default",
+          //       id: "admin",
           //       name: "instance_name",
           //     },
           //   ],
@@ -671,7 +671,7 @@ describe("instance management api interaction with listing", () => {
 
     const hookBehavior = await tests.hookBehaveLikeThis(
       () => {
-        const { lib: api } = useMerchantApiContext()
+        const { lib: api } = useMerchantApiContext();
         const query = useBackendInstances();
         return { query, api };
       },
