@@ -28,13 +28,12 @@ export function revalidateInstanceOtpDevices() {
   );
 }
 export function useInstanceOtpDevices() {
-  const { state: session } = useSessionContext();
-  const { lib: { instance } } = useSessionContext();
+  const { state, lib } = useSessionContext();
 
   // const [offset, setOffset] = useState<string | undefined>();
 
   async function fetcher([token, _bid]: [AccessToken, string]) {
-    return await instance.listOtpDevices(token, {
+    return await lib.instance.listOtpDevices(token, {
       // limit: PAGINATED_LIST_REQUEST,
       // offset: bid,
       // order: "dec",
@@ -44,7 +43,7 @@ export function useInstanceOtpDevices() {
   const { data, error } = useSWR<
     TalerMerchantManagementResultByMethod<"listOtpDevices">,
     TalerHttpError
-  >([session.token, "offset", "listOtpDevices"], fetcher);
+  >([state.token, "offset", "listOtpDevices"], fetcher);
 
   if (error) return error;
   if (data === undefined) return undefined;
@@ -62,17 +61,16 @@ export function revalidateOtpDeviceDetails() {
   );
 }
 export function useOtpDeviceDetails(deviceId: string) {
-  const { state: session } = useSessionContext();
-  const { lib: { instance } } = useSessionContext();
+  const { state, lib } = useSessionContext();
 
   async function fetcher([dId, token]: [string, AccessToken]) {
-    return await instance.getOtpDeviceDetails(token, dId);
+    return await lib.instance.getOtpDeviceDetails(token, dId);
   }
 
   const { data, error } = useSWR<
     TalerMerchantManagementResultByMethod<"getOtpDeviceDetails">,
     TalerHttpError
-  >([deviceId, session.token, "getOtpDeviceDetails"], fetcher);
+  >([deviceId, state.token, "getOtpDeviceDetails"], fetcher);
 
   if (data) return data;
   if (error) return error;
