@@ -16,10 +16,14 @@
 import { PAGINATED_LIST_REQUEST } from "../utils/constants.js";
 
 // FIX default import https://github.com/microsoft/TypeScript/issues/49189
-import { AccessToken, TalerHttpError, TalerMerchantManagementResultByMethod } from "@gnu-taler/taler-util";
+import {
+  AccessToken,
+  TalerHttpError,
+  TalerMerchantManagementResultByMethod,
+} from "@gnu-taler/taler-util";
 import _useSWR, { SWRHook, mutate } from "swr";
 import { useSessionContext } from "../context/session.js";
-import { buildPaginatedResult } from "./webhooks.js";
+import { buildPaginatedResult } from "@gnu-taler/web-util/browser";
 const useSWR = _useSWR as unknown as SWRHook;
 
 export interface InstanceTransferFilter {
@@ -37,13 +41,18 @@ export function revalidateInstanceTransfers() {
 }
 export function useInstanceTransfers(
   args?: InstanceTransferFilter,
-  updatePosition: (id: string | undefined) => void = (() => { }),
+  updatePosition: (id: string | undefined) => void = () => {},
 ) {
   const { state, lib } = useSessionContext();
 
   // const [offset, setOffset] = useState<string | undefined>(args?.position);
 
-  async function fetcher([token, o, p, v]: [AccessToken, string, string, boolean]) {
+  async function fetcher([token, o, p, v]: [
+    AccessToken,
+    string,
+    string,
+    boolean,
+  ]) {
     return await lib.instance.listWireTransfers(token, {
       paytoURI: p,
       verified: v,
@@ -56,12 +65,26 @@ export function useInstanceTransfers(
   const { data, error } = useSWR<
     TalerMerchantManagementResultByMethod<"listWireTransfers">,
     TalerHttpError
-  >([state.token, args?.position, args?.payto_uri, args?.verified, "listWireTransfers"], fetcher);
+  >(
+    [
+      state.token,
+      args?.position,
+      args?.payto_uri,
+      args?.verified,
+      "listWireTransfers",
+    ],
+    fetcher,
+  );
 
   if (error) return error;
   if (data === undefined) return undefined;
   if (data.type !== "ok") return data;
 
-  return buildPaginatedResult(data.body.transfers, args?.position, updatePosition, (d) => String(d.transfer_serial_id))
-
+  return buildPaginatedResult(
+    data.body.transfers,
+    args?.position,
+    updatePosition,
+    (d) => String(d.transfer_serial_id),
+    PAGINATED_LIST_REQUEST,
+  );
 }
