@@ -53,6 +53,10 @@ function createExample<Props>(
               num_fractional_trailing_zero_digits: 1,
             }
           },
+          have_donau: false,
+          mandatory_tan_channels: [],
+          payment_target_regex: "*",
+          payment_target_types: "*",
           exchanges: [],
           name: "taler-merchant"
         },
