@@ -26,7 +26,7 @@ import {
   TalerMerchantApi,
   TranslatedString,
   assertUnreachable,
-  parsePaytoUri
+  parsePaytoUri,
 } from "@gnu-taler/taler-util";
 import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
@@ -35,59 +35,70 @@ import { AsyncButton } from "../../../../components/exception/AsyncButton.js";
 import {
   FormErrors,
   FormProvider,
+  TalerForm,
 } from "../../../../components/form/FormProvider.js";
 import { Input } from "../../../../components/form/Input.js";
 import { InputPaytoForm } from "../../../../components/form/InputPaytoForm.js";
 import { InputSelector } from "../../../../components/form/InputSelector.js";
 import { InputToggle } from "../../../../components/form/InputToggle.js";
-import {
-  CompareAccountsModal,
-  ImportingAccountModal,
-} from "../../../../components/modal/index.js";
+import { CompareAccountsModal } from "../../../../components/modal/index.js";
 import { undefinedIfEmpty } from "../../../../utils/table.js";
 import { safeConvertURL } from "../update/UpdatePage.js";
-import { testRevenueAPI } from "./index.js";
+import { TestRevenueErrorType, testRevenueAPI } from "./index.js";
+import { usePreference } from "../../../../hooks/preference.js";
+import { useSettingsContext } from "../../../../context/settings.js";
 
-type Entity = TalerMerchantApi.AccountAddDetails & { verified?: boolean };
+type Entity = TalerMerchantApi.AccountAddDetails & {
+  verified?: boolean;
+} & TalerForm;
 
 interface Props {
   onCreate: (d: TalerMerchantApi.AccountAddDetails) => Promise<void>;
   onBack?: () => void;
 }
 
-const accountAuthType = ["none", "basic"];
-
 export function CreatePage({ onCreate, onBack }: Props): VNode {
   const { i18n } = useTranslationContext();
 
-  const [importing, setImporting] = useState(false);
-  const [state, setState] = useState<Partial<Entity>>({});
+  const [{ developerMode }] = usePreference();
+  const accountAuthType = developerMode
+    ? ["none", "basic", "bearer"]
+    : ["none", "basic"];
+
+  const [state, setState] = useState<Partial<Entity>>({
+    credit_facade_credentials: {
+      type: "basic",
+      password: "",
+      username: "",
+    },
+  });
   const facadeURL = safeConvertURL(state.credit_facade_url);
 
   const [revenuePayto, setRevenuePayto] = useState<PaytoUri | undefined>(
-    // parsePaytoUri("payto://x-taler-bank/asd.com:1010/asd/pepe"),
     undefined,
   );
   const [testError, setTestError] = useState<TranslatedString | undefined>(
     undefined,
   );
-  const errors: FormErrors<Entity> = {
+  const errors = undefinedIfEmpty<FormErrors<Entity>>({
     payto_uri: !state.payto_uri ? i18n.str`Required` : undefined,
 
-    credit_facade_credentials: !state.credit_facade_credentials
-      ? undefined
-      : undefinedIfEmpty({
-          username:
-            state.credit_facade_credentials.type === "basic" &&
-            !state.credit_facade_credentials.username
-              ? i18n.str`Required`
-              : undefined,
-          password:
-            state.credit_facade_credentials.type === "basic" &&
-            !state.credit_facade_credentials.password
-              ? i18n.str`Required`
-              : undefined,
-        }),
+    credit_facade_credentials: undefinedIfEmpty(
+      !state.credit_facade_credentials || !state.credit_facade_url
+        ? undefined
+        : {
+            username:
+              state.credit_facade_credentials.type === "basic" &&
+              !state.credit_facade_credentials.username
+                ? i18n.str`Required`
+                : undefined,
+            password:
+              state.credit_facade_credentials.type === "basic" &&
+              !state.credit_facade_credentials.password
+                ? i18n.str`Required`
+                : undefined,
+          },
+    ) as any,
     credit_facade_url: !state.credit_facade_url
       ? undefined
       : !facadeURL
@@ -99,11 +110,9 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
             : facadeURL.hash
               ? i18n.str`URL must not hash param`
               : undefined,
-  };
+  });
 
-  const hasErrors = Object.keys(errors).some(
-    (k) => (errors as Record<string, unknown>)[k] !== undefined,
-  );
+  const hasErrors = errors !== undefined;
 
   const submitForm = () => {
     if (hasErrors) return Promise.reject();
@@ -169,7 +178,17 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
           }
           case HttpStatusCode.NotFound: {
             setTestError(
-              i18n.str`The endpoint doesn't seems to be a Taler Revenue API.`,
+              i18n.str`The endpoint does not seem to be a Taler Revenue API.`,
+            );
+            setState({
+              ...state,
+              verified: undefined,
+            });
+            return;
+          }
+          case TestRevenueErrorType.CANT_VALIDATE: {
+            setTestError(
+              i18n.str`The request was made correctly, but the bank's server did not respond with the appropriate value for 'credit_account', so we cannot confirm that it is the same bank account.`,
             );
             setState({
               ...state,
@@ -211,86 +230,94 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
                 name="payto_uri"
                 label={i18n.str`Account details`}
               />
-              <div class="message-body" style={{ marginBottom: 10 }}>
-                <p>
-                  <i18n.Translate>
-                    If the bank supports Taler Revenue API then you can add the
-                    endpoint URL below to keep the revenue information in sync.
-                  </i18n.Translate>
-                </p>
-              </div>
-              <Input<Entity>
-                name="credit_facade_url"
-                label={i18n.str`Endpoint URL`}
-                help="https://bank.demo.taler.net/accounts/${USERNAME}/taler-revenue/"
-                expand
-                tooltip={i18n.str`From where the merchant can download information about incoming wire transfers to this account`}
-              />
-              <InputSelector
-                name="credit_facade_credentials.type"
-                label={i18n.str`Auth type`}
-                tooltip={i18n.str`Choose the authentication type for the account info URL`}
-                values={accountAuthType}
-                toStr={(str) => {
-                  if (str === "none") return "Without authentication";
-                  return "Username and password";
-                }}
-              />
-              {state.credit_facade_credentials?.type === "basic" ? (
+              {!developerMode ? undefined : (
                 <Fragment>
-                  <Input
-                    name="credit_facade_credentials.username"
-                    label={i18n.str`Username`}
-                    tooltip={i18n.str`Username to access the account information.`}
+                  <div class="message-body" style={{ marginBottom: 10 }}>
+                    <p>
+                      <i18n.Translate>
+                        If the bank supports Taler Revenue API then you can add
+                        the endpoint URL below to keep the revenue information
+                        in sync.
+                      </i18n.Translate>
+                    </p>
+                  </div>
+                  <Input<Entity>
+                    name="credit_facade_url"
+                    label={i18n.str`Endpoint URL`}
+                    help="https://bank.demo.taler.net/accounts/${USERNAME}/taler-revenue/"
+                    expand
+                    tooltip={i18n.str`From where the merchant can download information about incoming wire transfers to this account`}
                   />
-                  <Input
-                    name="credit_facade_credentials.password"
-                    inputType="password"
-                    label={i18n.str`Password`}
-                    tooltip={i18n.str`Password to access the account information.`}
+                  <InputSelector
+                    name="credit_facade_credentials.type"
+                    label={i18n.str`Auth type`}
+                    tooltip={i18n.str`Choose the authentication type for the account info URL`}
+                    values={accountAuthType}
+                    toStr={(str) => {
+                      if (str === "none")
+                        return i18n.str`Without authentication`;
+                      if (str === "bearer") return i18n.str`With token`;
+                      return "With username and password";
+                    }}
+                  />
+                  {state.credit_facade_credentials?.type === "basic" ? (
+                    <Fragment>
+                      <Input
+                        name="credit_facade_credentials.username"
+                        label={i18n.str`Username`}
+                        tooltip={i18n.str`Username to access the account information.`}
+                      />
+                      <Input
+                        name="credit_facade_credentials.password"
+                        inputType="password"
+                        label={i18n.str`Password`}
+                        tooltip={i18n.str`Password to access the account information.`}
+                      />
+                    </Fragment>
+                  ) : undefined}
+                  {state.credit_facade_credentials?.type === "bearer" ? (
+                    <Fragment>
+                      <Input
+                        name="credit_facade_credentials.token"
+                        label={i18n.str`Token`}
+                        inputType="password"
+                        tooltip={i18n.str`Access token to access the account information.`}
+                      />
+                    </Fragment>
+                  ) : undefined}
+                  <InputToggle<Entity>
+                    label={i18n.str`Match`}
+                    tooltip={i18n.str`Check if the information matches the server info.`}
+                    name="verified"
+                    readonly
+                    threeState
+                    help={
+                      testError !== undefined
+                        ? testError
+                        : state.verified === undefined
+                          ? i18n.str`Not verified`
+                          : state.verified
+                            ? i18n.str`Last test was successful.`
+                            : i18n.str`Last test failed.`
+                    }
+                    side={
+                      <button
+                        class="button is-info"
+                        data-tooltip={i18n.str`Compare info from server with account form`}
+                        disabled={!state.credit_facade_url}
+                        onClick={async () => {
+                          await testAccountInfo();
+                        }}
+                      >
+                        <i18n.Translate>Test</i18n.Translate>
+                      </button>
+                    }
                   />
                 </Fragment>
-              ) : undefined}
-              <InputToggle<Entity>
-                label={i18n.str`Match`}
-                tooltip={i18n.str`Check where the information match against the server info.`}
-                name="verified"
-                readonly
-                threeState
-                help={
-                  testError !== undefined
-                    ? testError
-                    : state.verified === undefined
-                      ? i18n.str`Not verified`
-                      : state.verified
-                        ? i18n.str`Last test was ok`
-                        : i18n.str`Last test failed`
-                }
-                side={
-                  <button
-                    class="button is-info"
-                    data-tooltip={i18n.str`Compare info from server with account form`}
-                    disabled={!state.credit_facade_url}
-                    onClick={async () => {
-                      await testAccountInfo();
-                    }}
-                  >
-                    <i18n.Translate>Test</i18n.Translate>
-                  </button>
-                }
-              />
+              )}
             </FormProvider>
 
             <div class="buttons is-right mt-5">
-              <button
-                class="button is-info"
-                onClick={() => {
-                  setImporting(true);
-                }}
-              >
-                <i18n.Translate>Import from bank</i18n.Translate>
-              </button>
-
               {onBack && (
                 <button class="button" onClick={onBack}>
                   <i18n.Translate>Cancel</i18n.Translate>
@@ -300,7 +327,7 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
                 disabled={hasErrors}
                 data-tooltip={
                   hasErrors
-                    ? i18n.str`Need to complete marked fields`
+                    ? i18n.str`Please complete the marked fields`
                     : i18n.str`Confirm operation`
                 }
                 onClick={submitForm}
@@ -312,34 +339,6 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
           <div class="column" />
         </div>
       </section>
-      {!importing ? undefined : (
-        <ImportingAccountModal
-          onCancel={() => {
-            setImporting(false);
-          }}
-          onConfirm={(ac) => {
-            const u = new URL(ac.infoURL);
-            const user = u.username;
-            const pwd = u.password;
-            u.password = "";
-            u.username = "";
-            const credit_facade_url = u.href;
-            setState({
-              payto_uri: ac.accountURI,
-              credit_facade_credentials:
-                user || pwd
-                  ? {
-                      type: "basic",
-                      password: pwd,
-                      username: user,
-                    }
-                  : undefined,
-              credit_facade_url,
-            });
-            setImporting(false);
-          }}
-        />
-      )}
       {!revenuePayto ? undefined : (
         <CompareAccountsModal
           onCancel={() => {
