@@ -20,12 +20,18 @@
  */
 
 import {
+  AccountAddDetails,
+  BankAccountDetail,
+  ChallengeResponse,
   HttpStatusCode,
   TalerError,
   TalerMerchantApi,
   assertUnreachable,
 } from "@gnu-taler/taler-util";
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
+import {
+  useChallengeHandler,
+  useTranslationContext,
+} from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
 import { useState } from "preact/hooks";
 import { ErrorLoadingMerchant } from "../../../../components/ErrorLoadingMerchant.js";
@@ -38,6 +44,7 @@ import { LoginPage } from "../../../login/index.js";
 import { NotFoundPageOrAdminCreate } from "../../../notfound/index.js";
 import { UpdatePage } from "./UpdatePage.js";
 import { WithId } from "../../../../declaration.js";
+import { SolveMFAChallenges } from "../../../../components/SolveMFA.js";
 
 export type Entity = TalerMerchantApi.AccountPatchDetails & WithId;
 
@@ -54,6 +61,7 @@ export default function UpdateValidator({
   const { state, lib } = useSessionContext();
   const result = useBankAccountDetails(bid);
   const [notif, setNotif] = useState<Notification | undefined>(undefined);
+  const mfa = useChallengeHandler();
 
   const { i18n } = useTranslationContext();
 
@@ -75,79 +83,102 @@ export default function UpdateValidator({
     }
   }
 
+  async function doUpdateImpl(request: TalerMerchantApi.AccountPatchDetails) {
+    return lib.instance
+      .updateBankAccount(state.token, bid, request)
+      .then((resp) => {
+        if (resp.type === "fail") {
+          setNotif({
+            message: i18n.str`Could not update account`,
+            type: "ERROR",
+            description: resp.detail?.hint,
+          });
+          return;
+        }
+        onConfirm();
+      })
+      .catch((error) => {
+        setNotif({
+          message: i18n.str`Could not update account`,
+          type: "ERROR",
+          description: error instanceof Error ? error.message : String(error),
+        });
+      });
+  }
+
+  const [doReplace, repeatReplace] = mfa.withMfaHandler(
+    ({ challengeIds, onChallengeRequired }) =>
+      async function doReplaceImpl(
+        prev: BankAccountDetail,
+        next: AccountAddDetails,
+      ) {
+        try {
+          const resp = await lib.instance.addBankAccount(state.token, next, {
+            challengeIds,
+          });
+          if (resp.type === "fail") {
+            if (resp.case === HttpStatusCode.Accepted) {
+              onChallengeRequired(resp.body);
+              return;
+            }
+            setNotif({
+              message: i18n.str`Could not create account`,
+              type: "ERROR",
+              description: resp.detail?.hint,
+            });
+            return;
+          }
+        } catch (error) {
+          setNotif({
+            message: i18n.str`Could not create account`,
+            type: "ERROR",
+            description: error instanceof Error ? error.message : String(error),
+          });
+          return;
+        }
+        try {
+          const resp = await lib.instance.deleteBankAccount(
+            state.token,
+            prev.h_wire,
+          );
+          if (resp.type === "fail") {
+            setNotif({
+              message: i18n.str`Could not delete account`,
+              type: "ERROR",
+              description: resp.detail?.hint,
+            });
+            return;
+          }
+        } catch (error) {
+          setNotif({
+            message: i18n.str`Could not delete account`,
+            type: "ERROR",
+            description: error instanceof Error ? error.message : String(error),
+          });
+          return;
+        }
+        onConfirm();
+      },
+  );
+
+  if (mfa.pendingChallenge) {
+    return (
+      <SolveMFAChallenges
+        currentChallenge={mfa.pendingChallenge}
+        onCompleted={repeatReplace}
+        onCancel={mfa.doCancelChallenge}
+      />
+    );
+  }
+
   return (
     <Fragment>
       <NotificationCard notification={notif} />
       <UpdatePage
         account={{ ...result.body, id: bid }}
         onBack={onBack}
-        onUpdate={async (request) => {
-          return lib.instance
-            .updateBankAccount(state.token, bid, request)
-            .then((updated) => {
-              if (updated.type === "fail") {
-                setNotif({
-                  message: i18n.str`Could not update account`,
-                  type: "ERROR",
-                  description: updated.detail.hint,
-                });
-                return;
-              }
-              onConfirm();
-            })
-            .catch((error) => {
-              setNotif({
-                message: i18n.str`Could not update account`,
-                type: "ERROR",
-                description: error instanceof Error ? error.message : String(error),
-              });
-            });
-        }}
-        onReplace={async (prev, next) => {
-          try {
-            const created = await lib.instance.addBankAccount(
-              state.token,
-              next,
-            );
-            if (created.type === "fail") {
-              setNotif({
-                message: i18n.str`Could not create account`,
-                type: "ERROR",
-                description: created.detail.hint,
-              });
-              return;
-            }
-          } catch (error) {
-            setNotif({
-              message: i18n.str`Could not create account`,
-              type: "ERROR",
-              description: error instanceof Error ? error.message : String(error),
-            });
-            return;
-          }
-          try {
-            const deleted = await lib.instance.deleteBankAccount(
-              state.token,
-              prev.h_wire,
-            );
-            if (deleted.type === "fail") {
-              setNotif({
-                message: i18n.str`Could not delete account`,
-                type: "ERROR",
-                description: deleted.detail.hint,
-              });
-              return;
-            }
-          } catch (error) {
-            setNotif({
-              message: i18n.str`Could not delete account`,
-              type: "ERROR",
-              description: error instanceof Error ? error.message : String(error),
-            });
-            return;
-          }
-          onConfirm();
-        }}
+        onUpdate={doUpdateImpl}
+        onReplace={doReplace}
       />
     </Fragment>
   );
