@@ -34,6 +34,7 @@ import { useEffect, useState } from "preact/hooks";
 import {
   FormErrors,
   FormProvider,
+  TalerForm,
 } from "../../../../components/form/FormProvider.js";
 import { Input } from "../../../../components/form/Input.js";
 import { InputCurrency } from "../../../../components/form/InputCurrency.js";
@@ -47,12 +48,12 @@ import { InventoryProductForm } from "../../../../components/product/InventoryPr
 import { NonInventoryProductFrom } from "../../../../components/product/NonInventoryProductForm.js";
 import { ProductList } from "../../../../components/product/ProductList.js";
 import { useSessionContext } from "../../../../context/session.js";
+import { WithId } from "../../../../declaration.js";
 import { usePreference } from "../../../../hooks/preference.js";
 import { rate } from "../../../../utils/amount.js";
 import { undefinedIfEmpty } from "../../../../utils/table.js";
-import { WithId } from "../../../../declaration.js";
 
-interface Props {
+export interface Props {
   onCreate: (d: TalerMerchantApi.PostOrderRequest) => void;
   onBack?: () => void;
   instanceConfig: InstanceConfig;
@@ -74,6 +75,10 @@ function with_defaults(
   const defaultWireDeadline = Duration.fromTalerProtocolDuration(
     config.default_wire_transfer_delay,
   );
+  const defaultRefundDeadline = Duration.min(
+    defaultWireDeadline,
+    Duration.fromSpec({ days: 15 }),
+  );
 
   return {
     inventoryProducts: {},
@@ -83,7 +88,7 @@ function with_defaults(
       max_fee: undefined,
       createToken: true,
       pay_deadline: defaultPayDeadline,
-      refund_deadline: defaultPayDeadline,
+      refund_deadline: defaultRefundDeadline,
       wire_transfer_deadline: defaultWireDeadline,
     },
     shipping: {},
@@ -99,26 +104,27 @@ export interface ProductMap {
   [id: string]: ProductAndQuantity;
 }
 
-interface Pricing {
+interface Pricing extends TalerForm {
   products_price: string;
   order_price: string;
   summary: string;
 }
-interface Shipping {
+interface Shipping extends TalerForm {
   delivery_date?: Date;
   delivery_location?: TalerMerchantApi.Location;
-  fullfilment_url?: string;
+  fulfillment_url?: string;
+  fulfillment_message?: string;
 }
-interface Payments {
+interface Payments extends TalerForm {
   refund_deadline: Duration;
   pay_deadline: Duration;
   wire_transfer_deadline: Duration;
-  auto_refund_deadline: Duration;
+  auto_refund_deadline?: Duration;
   max_fee?: string;
   createToken: boolean;
   minimum_age?: number;
 }
-interface Entity {
+interface Entity extends TalerForm {
   inventoryProducts: ProductMap;
   products: TalerMerchantApi.Product[];
   pricing: Partial<Pricing>;
@@ -127,6 +133,15 @@ interface Entity {
   extra: Record<string, string>;
 }
 
+function isInvalidUrl(url: string) {
+  try {
+    const asd = new URL("./", url);
+    return false;
+  } catch (e) {
+    return true;
+  }
+}
+
 export function CreatePage({
   onCreate,
   onBack,
@@ -148,7 +163,7 @@ export function CreatePage({
     : Amounts.parse(value.pricing.order_price);
 
   const errors = undefinedIfEmpty<FormErrors<Entity>>({
-    pricing: undefinedIfEmpty({
+    pricing: undefinedIfEmpty<FormErrors<Pricing>>({
       summary: !value.pricing?.summary ? i18n.str`Required` : undefined,
       order_price: !value.pricing?.order_price
         ? i18n.str`Required`
@@ -166,7 +181,7 @@ export function CreatePage({
               value.payments.refund_deadline,
               value.payments.pay_deadline,
             ) === -1
-          ? i18n.str`Refund deadline can't be before pay deadline`
+          ? i18n.str`The refund deadline cannot be earlier than the payment deadline.`
           : value.payments.wire_transfer_deadline &&
               Duration.cmp(
                 value.payments.wire_transfer_deadline,
@@ -203,6 +218,19 @@ export function CreatePage({
         : !isFuture(value.shipping.delivery_date)
           ? i18n.str`Must be in the future`
           : undefined,
+      fulfillment_message:
+        !!value.shipping?.fulfillment_message &&
+        !!value.shipping?.fulfillment_url
+          ? i18n.str`Either fulfillment url or fulfillment message must be specified.`
+          : undefined,
+      fulfillment_url:
+        !!value.shipping?.fulfillment_message &&
+        !!value.shipping?.fulfillment_url
+          ? i18n.str`Either fulfillment url or fulfillment message must be specified.`
+          : !!value.shipping?.fulfillment_url &&
+              isInvalidUrl(value.shipping.fulfillment_url)
+            ? i18n.str`is not a valid URL`
+            : undefined,
     }),
   });
   const hasErrors = errors !== undefined;
@@ -213,7 +241,7 @@ export function CreatePage({
     const summary = order.pricing?.summary;
     if (!value.payments) return;
     if (!value.shipping) return;
-    if (!price || ! summary) return;
+    if (!price || !summary) return;
 
     const request: TalerMerchantApi.PostOrderRequest = {
       order: {
@@ -249,7 +277,8 @@ export function CreatePage({
           ? { t_s: value.shipping.delivery_date.getTime() / 1000 }
           : undefined,
         delivery_location: value.shipping.delivery_location,
-        fulfillment_url: value.shipping.fullfilment_url,
+        fulfillment_url: value.shipping.fulfillment_url,
+        fulfillment_message: value.shipping.fulfillment_message,
         minimum_age: value.payments.minimum_age,
       },
       inventory_products: inventoryList.map((p) => ({
@@ -336,6 +365,7 @@ export function CreatePage({
     parsedPrice ?? Amounts.zeroOfCurrency(config.currency),
     totalPrice.amount,
   );
+  const discountOrRiseRounded = Math.round((discountOrRise - 1) * 100);
 
   const minAgeByProducts = inventoryList.reduce(
     (cur, prev) =>
@@ -452,20 +482,18 @@ export function CreatePage({
                 <Fragment>
                   <InputCurrency
                     name="pricing.products_price"
-                    label={i18n.str`Total price`}
+                    label={i18n.str`Products price sum`}
                     readonly
                     tooltip={i18n.str`Total product price added up`}
                   />
                   <InputCurrency
                     name="pricing.order_price"
-                    label={i18n.str`Total price`}
+                    label={i18n.str`Order price`}
                     addonAfter={
-                      discountOrRise > 0 &&
-                      (discountOrRise < 1
-                        ? `discount of %${Math.round(
-                            (1 - discountOrRise) * 100,
-                          )}`
-                        : `rise of %${Math.round((discountOrRise - 1) * 100)}`)
+                      discountOrRiseRounded > 0 &&
+                      (discountOrRiseRounded < 1
+                        ? `discount of ${discountOrRiseRounded}%`
+                        : `rise of ${discountOrRiseRounded}%`)
                     }
                     tooltip={i18n.str`Amount to be paid by the customer`}
                   />
@@ -485,10 +513,10 @@ export function CreatePage({
                 tooltip={i18n.str`Title of the order to be shown to the customer`}
               />
 
-              {pref.advanceOrderMode && (
+              {(pref.advanceOrderMode || errors?.shipping) && (
                 <InputGroup
                   name="shipping"
-                  label={i18n.str`Shipping and Fulfillment`}
+                  label={i18n.str`Shipping and fulfillment`}
                   initialActive
                 >
                   <InputDate
@@ -496,30 +524,37 @@ export function CreatePage({
                     label={i18n.str`Delivery date`}
                     tooltip={i18n.str`Deadline for physical delivery assured by the merchant.`}
                   />
-                  {value.shipping?.delivery_date && (
-                    <InputGroup
-                      name="shipping.delivery_location"
-                      label={i18n.str`Location`}
-                      tooltip={i18n.str`Address where the products will be delivered`}
-                    >
-                      <InputLocation name="shipping.delivery_location" />
-                    </InputGroup>
-                  )}
+                  <InputGroup
+                    name="shipping.delivery_location"
+                    label={i18n.str`Delivery location`}
+                    tooltip={i18n.str`Address where the products will be delivered`}
+                  >
+                    <InputLocation name="shipping.delivery_location" />
+                  </InputGroup>
                   <Input
-                    name="shipping.fullfilment_url"
+                    name="shipping.fulfillment_url"
                     label={i18n.str`Fulfillment URL`}
                     tooltip={i18n.str`URL to which the user will be redirected after successful payment.`}
                   />
+                  <Input
+                    name="shipping.fulfillment_message"
+                    label={i18n.str`Fulfillment message`}
+                    tooltip={i18n.str`Message shown to the customer after paying for the order.`}
+                  />
                 </InputGroup>
               )}
 
-              {(pref.advanceOrderMode || requiresSomeTalerOptions) && (
+              {(pref.advanceOrderMode ||
+                requiresSomeTalerOptions ||
+                errors?.payments) && (
                 <InputGroup
                   name="payments"
                   label={i18n.str`Taler payment options`}
                   tooltip={i18n.str`Override default Taler payment settings for this order`}
                 >
-                  {(pref.advanceOrderMode || noDefault_payDeadline) && (
+                  {(pref.advanceOrderMode ||
+                    noDefault_payDeadline ||
+                    errors?.payments?.pay_deadline !== undefined) && (
                     <InputDuration
                       name="payments.pay_deadline"
                       label={i18n.str`Payment time`}
@@ -528,7 +563,7 @@ export function CreatePage({
                       }
                       withForever
                       withoutClear
-                      tooltip={i18n.str`Time for the customer to pay for the offer before it expires. Inventory products will be reserved until this deadline. Time start to run after the order is created.`}
+                      tooltip={i18n.str`Time for the customer to pay before the offer expires. Inventory products will be reserved until this deadline. The time starts after the order is created.`}
                       side={
                         <span>
                           <button
@@ -551,7 +586,8 @@ export function CreatePage({
                       }
                     />
                   )}
-                  {pref.advanceOrderMode && (
+                  {(pref.advanceOrderMode ||
+                    errors?.payments?.refund_deadline !== undefined) && (
                     <InputDuration
                       name="payments.refund_deadline"
                       label={i18n.str`Refund time`}
@@ -584,7 +620,9 @@ export function CreatePage({
                       }
                     />
                   )}
-                  {(pref.advanceOrderMode || noDefault_wireDeadline) && (
+                  {(pref.advanceOrderMode ||
+                    noDefault_wireDeadline ||
+                    errors?.payments?.wire_transfer_deadline !== undefined) && (
                     <InputDuration
                       name="payments.wire_transfer_deadline"
                       label={i18n.str`Wire transfer time`}
@@ -618,7 +656,8 @@ export function CreatePage({
                       }
                     />
                   )}
-                  {pref.advanceOrderMode && (
+                  {(pref.advanceOrderMode ||
+                    errors?.payments?.auto_refund_deadline !== undefined) && (
                     <InputDuration
                       name="payments.auto_refund_deadline"
                       label={i18n.str`Auto-refund time`}
@@ -632,28 +671,31 @@ export function CreatePage({
                     />
                   )}
 
-                  {pref.advanceOrderMode && (
+                  {(pref.advanceOrderMode ||
+                    errors?.payments?.max_fee !== undefined) && (
                     <InputCurrency
                       name="payments.max_fee"
                       label={i18n.str`Maximum fee`}
                       tooltip={i18n.str`Maximum fees the merchant is willing to cover for this order. Higher deposit fees must be covered in full by the consumer.`}
                     />
                   )}
-                  {pref.advanceOrderMode && (
+                  {(pref.advanceOrderMode ||
+                    errors?.payments?.createToken !== undefined) && (
                     <InputToggle
                       name="payments.createToken"
                       label={i18n.str`Create token`}
-                      tooltip={i18n.str`If the order ID is easy to guess the token will prevent user to steal orders from others.`}
+                      tooltip={i18n.str`If the order ID is easy to guess, the token will prevent other users from claiming the order.`}
                     />
                   )}
-                  {pref.advanceOrderMode && (
+                  {(pref.advanceOrderMode ||
+                    errors?.payments?.minimum_age !== undefined) && (
                     <InputNumber
                       name="payments.minimum_age"
                       label={i18n.str`Minimum age required`}
                       tooltip={i18n.str`Any value greater than 0 will limit the coins able be used to pay this contract. If empty the age restriction will be defined by the products`}
                       help={
                         minAgeByProducts > 0
-                          ? i18n.str`Min age defined by the producs is ${minAgeByProducts}`
+                          ? i18n.str`Minimum age defined by the products is ${minAgeByProducts}`
                           : i18n.str`No product with age restriction in this order`
                       }
                     />
@@ -661,7 +703,7 @@ export function CreatePage({
                 </InputGroup>
               )}
 
-              {pref.advanceOrderMode && (
+              {(pref.advanceOrderMode || errors?.extra !== undefined) && (
                 <InputGroup
                   name="extra"
                   label={i18n.str`Additional information`}
@@ -766,6 +808,7 @@ export function CreatePage({
 function asProduct(p: ProductAndQuantity): TalerMerchantApi.Product {
   return {
     product_id: p.product.id,
+    product_name: p.product.product_name,
     image: p.product.image,
     price: p.product.price,
     unit: p.product.unit,
@@ -796,3 +839,17 @@ function DeadlineHelp({ duration }: { duration?: Duration }): VNode {
     </i18n.Translate>
   );
 }
+
+function getAll(s: object): string[] {
+  return Object.entries(s).flatMap(([key, value]) => {
+    if (typeof value === "object")
+      return getAll(value).map((v) => `${key}.${v}`);
+    if (!value) return [];
+    return key;
+  });
+}
+
+function describeMissingFields(errors: object | undefined): string[] {
+  if (!errors) return [];
+  return getAll(errors);
+}
