@@ -13,7 +13,13 @@
  You should have received a copy of the GNU General Public License along with
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
-import { HttpStatusCode, TalerError, assertUnreachable } from "@gnu-taler/taler-util";
+import {
+  HttpStatusCode,
+  OrderVersion,
+  TalerError,
+  TalerErrorCode,
+  assertUnreachable,
+} from "@gnu-taler/taler-util";
 import { Loading, useTranslationContext } from "@gnu-taler/web-util/browser";
 import { VNode, h } from "preact";
 import { ErrorLoadingMerchant } from "../../../../components/ErrorLoadingMerchant.js";
@@ -34,62 +40,86 @@ export function OrderCreatedSuccessfully({
   onConfirm,
   onCreateAnother,
 }: Props): VNode {
-  const result = useOrderDetails(entity.response.order_id)
+  const result = useOrderDetails(entity.response.order_id);
   const { i18n } = useTranslationContext();
 
-  if (!result) return <Loading />
+  if (!result) return <Loading />;
   if (result instanceof TalerError) {
-    return <ErrorLoadingMerchant error={result} />
+    return <ErrorLoadingMerchant error={result} />;
   }
+
+  if (
+    entity.request.order.version !== undefined &&
+    entity.request.order.version !== OrderVersion.V0
+  ) {
+    return <div>Unsupported order version {entity.request.order.version}</div>;
+  }
+
   if (result.type === "fail") {
-    switch(result.case) {
-      case HttpStatusCode.NotFound: {
-        return <NotFoundPageOrAdminCreate />
+    switch (result.case) {
+      case TalerErrorCode.MERCHANT_GENERIC_INSTANCE_UNKNOWN: {
+        return <NotFoundPageOrAdminCreate />;
       }
-      case HttpStatusCode.BadGateway: {
-        return <div>Failed to obtain a response from the exchange</div>;
-      }
-      case HttpStatusCode.GatewayTimeout: {
-        return (
-          <div>The merchant's interaction with the exchange took too long</div>
-        );
+      case TalerErrorCode.MERCHANT_GENERIC_ORDER_UNKNOWN: {
+        return <i18n.Translate>Order unkown</i18n.Translate>;
       }
       case HttpStatusCode.Unauthorized: {
-        return <LoginPage />
+        return <LoginPage />;
       }
       default: {
-        assertUnreachable(result)
+        assertUnreachable(result);
       }
     }
   }
 
-  const url = result.body.order_status === "unpaid" ?
-    result.body.taler_pay_uri :
-    result.body.contract_terms.fulfillment_url
+  const url =
+    result.body.order_status === "unpaid"
+      ? result.body.taler_pay_uri
+      : result.body.contract_terms.fulfillment_url;
 
   return (
     <CreatedSuccessfully
       onConfirm={onConfirm}
       onCreateAnother={onCreateAnother}
     >
-      <div class="field is-horizontal">
-        <div class="field-label is-normal">
-          <label class="label">
-            <i18n.Translate>Amount</i18n.Translate>
-          </label>
-        </div>
-        <div class="field-body is-flex-grow-3">
-          <div class="field">
-            <p class="control">
-              <input
-                class="input"
-                readonly
-                value={entity.request.order.amount}
-              />
-            </p>
+      {!entity.request.order.version ? (
+        <div class="field is-horizontal">
+          <div class="field-label is-normal">
+            <label class="label">
+              <i18n.Translate>Amount</i18n.Translate>
+            </label>
+          </div>
+          <div class="field-body is-flex-grow-3">
+            <div class="field">
+              <p class="control">
+                <input
+                  class="input"
+                  readonly
+                  value={entity.request.order.amount}
+                />
+              </p>
+            </div>
           </div>
         </div>
-      </div>
+      ) : (
+        <div class="field is-horizontal">
+          <div class="field-label is-normal">
+            <label class="label">
+              <i18n.Translate>Choices</i18n.Translate>
+            </label>
+          </div>
+          <div class="field-body is-flex-grow-3">
+            <div class="field">
+              <p class="control">
+                <i18n.Translate>
+                  This feature is not yet supported
+                </i18n.Translate>
+              </p>
+            </div>
+          </div>
+        </div>
+      )}
+
       <div class="field is-horizontal">
         <div class="field-label is-normal">
           <label class="label">
