@@ -22,12 +22,12 @@
 import {
   AmountJson,
   Amounts,
+  HostPortPath,
+  MerchantContractVersion,
   TalerMerchantApi,
   stringifyRefundUri,
 } from "@gnu-taler/taler-util";
-import {
-  useTranslationContext
-} from "@gnu-taler/web-util/browser";
+import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { format, formatDistance } from "date-fns";
 import { Fragment, VNode, h } from "preact";
 import { useState } from "preact/hooks";
@@ -50,9 +50,9 @@ import { RefundModal } from "../list/Table.js";
 import { Event, Timeline } from "./Timeline.js";
 
 type Entity = TalerMerchantApi.MerchantOrderStatusResponse;
-type CT = TalerMerchantApi.ContractTerms;
+type CT = TalerMerchantApi.MerchantContractTermsV0;
 
-interface Props {
+export interface Props {
   onBack: () => void;
   selected: Entity;
   id: string;
@@ -69,7 +69,7 @@ function ContractTerms({ value }: { value: CT }) {
   const { i18n } = useTranslationContext();
 
   return (
-    <InputGroup name="contract_terms" label={i18n.str`Contract Terms`}>
+    <InputGroup name="contract_terms" label={i18n.str`Contract terms`}>
       <FormProvider<CT> object={value} valueHandler={null}>
         <Input<CT>
           readonly
@@ -107,7 +107,7 @@ function ContractTerms({ value }: { value: CT }) {
           readonly
           name="refund_deadline"
           label={i18n.str`Refund deadline`}
-          tooltip={i18n.str`After this deadline has passed no refunds will be accepted`}
+          tooltip={i18n.str`After this deadline has passed, no refunds will be accepted.`}
         />
         <InputDate<CT>
           readonly
@@ -127,15 +127,13 @@ function ContractTerms({ value }: { value: CT }) {
           label={i18n.str`Delivery date`}
           tooltip={i18n.str`Time indicating when the order should be delivered`}
         />
-        {value.delivery_date && (
-          <InputGroup
-            name="delivery_location"
-            label={i18n.str`Location`}
-            tooltip={i18n.str`Where the order will be delivered`}
-          >
-            <InputLocation name="payments.delivery_location" />
-          </InputGroup>
-        )}
+        <InputGroup
+          name="delivery_location"
+          label={i18n.str`Delivery location`}
+          tooltip={i18n.str`Where the order will be delivered`}
+        >
+          <InputLocation name="delivery_location" />
+        </InputGroup>
         <InputDuration<CT>
           readonly
           name="auto_refund"
@@ -160,6 +158,12 @@ function ClaimedPage({
   id: string;
   order: TalerMerchantApi.CheckPaymentClaimedResponse;
 }) {
+  if (order.contract_terms.version !== undefined &&
+    order.contract_terms.version !== MerchantContractVersion.V0
+  ) {
+    return <div>Unsupported contract version {order.contract_terms.version}</div>;
+  }
+
   const now = new Date();
   const refundable =
     order.contract_terms.refund_deadline.t_s !== "never" &&
@@ -251,7 +255,7 @@ function ClaimedPage({
                             <i18n.Translate>Claimed at</i18n.Translate>:
                           </b>{" "}
                           {order.contract_terms.timestamp.t_s === "never"
-                            ? "never"
+                            ? i18n.str`Never`
                             : format(
                                 new Date(
                                   order.contract_terms.timestamp.t_s * 1000,
@@ -303,7 +307,7 @@ function ClaimedPage({
               </div>
             </section>
 
-            {order.contract_terms.products.length ? (
+            {order.contract_terms.products?.length ? (
               <Fragment>
                 <div class="title">
                   <i18n.Translate>Product list</i18n.Translate>
@@ -313,6 +317,9 @@ function ClaimedPage({
             ) : undefined}
 
             {value.contract_terms && (
+              value.contract_terms.version === undefined
+                || value.contract_terms.version === MerchantContractVersion.V0
+            ) && (
               <ContractTerms value={value.contract_terms} />
             )}
           </div>
@@ -331,6 +338,12 @@ function PaidPage({
   order: TalerMerchantApi.CheckPaymentPaidResponse;
   onRefund: (id: string) => void;
 }) {
+  if (order.contract_terms.version !== undefined &&
+    order.contract_terms.version !== MerchantContractVersion.V0
+  ) {
+    return <div>Unsupported contract version {order.contract_terms.version}</div>;
+  }
+
   const now = new Date();
   const refundable =
     order.contract_terms.refund_deadline.t_s !== "never" &&
@@ -433,7 +446,7 @@ function PaidPage({
   const { state } = useSessionContext();
 
   const refundurl = stringifyRefundUri({
-    merchantBaseUrl: state.backendUrl.href,
+    merchantBaseUrl: state.backendUrl.href as HostPortPath,
     orderId: order.contract_terms.order_id,
   });
   const { i18n } = useTranslationContext();
@@ -596,7 +609,7 @@ function PaidPage({
               </div>
             </section>
 
-            {order.contract_terms.products.length ? (
+            {order.contract_terms.products?.length ? (
               <Fragment>
                 <div class="title">
                   <i18n.Translate>Product list</i18n.Translate>
@@ -606,6 +619,9 @@ function PaidPage({
             ) : undefined}
 
             {value.contract_terms && (
+              value.contract_terms.version === undefined
+                || value.contract_terms.version === MerchantContractVersion.V0
+            ) && (
               <ContractTerms value={value.contract_terms} />
             )}
           </div>
@@ -671,7 +687,7 @@ function UnpaidPage({
                       <i18n.Translate>Created at</i18n.Translate>:
                     </b>{" "}
                     {order.creation_time.t_s === "never"
-                      ? "never"
+                      ? i18n.str`Never`
                       : format(
                           new Date(order.creation_time.t_s * 1000),
                           datetimeFormatForSettings(settings),
