@@ -19,10 +19,8 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { Amounts, TalerMerchantApi } from "@gnu-taler/taler-util";
-import {
-  useTranslationContext
-} from "@gnu-taler/web-util/browser";
+import { Amounts, AmountString, MerchantContractVersion, TalerMerchantApi } from "@gnu-taler/taler-util";
+import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { format } from "date-fns";
 import { VNode, h } from "preact";
 import { StateUpdater, useState } from "preact/hooks";
@@ -164,7 +162,7 @@ function Table({
                   style={{ cursor: "pointer" }}
                 >
                   {i.timestamp.t_s === "never"
-                    ? "never"
+                    ? i18n.str`Never`
                     : format(
                         new Date(i.timestamp.t_s * 1000),
                         datetimeFormatForSettings(settings),
@@ -210,9 +208,11 @@ function Table({
         </tbody>
       </table>
       {onLoadMoreAfter && (
-        <button class="button is-fullwidth" 
-        data-tooltip={i18n.str`Load more orders after the last one`}
-        onClick={onLoadMoreAfter}>
+        <button
+          class="button is-fullwidth"
+          data-tooltip={i18n.str`Load more orders after the last one`}
+          onClick={onLoadMoreAfter}
+        >
           <i18n.Translate>Load next page</i18n.Translate>
         </button>
       )}
@@ -255,6 +255,17 @@ export function RefundModal({
   const { i18n } = useTranslationContext();
   // const [errors, setErrors] = useState<FormErrors<State>>({});
 
+  let amount: AmountString;
+  if (order.order_status === "paid") {
+    if (order.contract_terms.version !== undefined &&
+      order.contract_terms.version !== MerchantContractVersion.V0
+    ) {
+      return <div>Unsupported contract version {order.contract_terms.version}</div>;
+    }
+
+    amount = order.contract_terms.amount;
+  }
+
   const refunds = (
     order.order_status === "paid" ? order.refund_details : []
   ).reduce(mergeRefunds, []);
@@ -268,7 +279,7 @@ export function RefundModal({
     );
   const orderPrice =
     order.order_status === "paid"
-      ? Amounts.parseOrThrow(order.contract_terms.amount)
+      ? Amounts.parseOrThrow(amount!)
       : undefined;
   const totalRefundable = !orderPrice
     ? Amounts.zeroOfCurrency(totalRefunded.currency)
@@ -290,7 +301,7 @@ export function RefundModal({
       : !Amounts.parse(form.refund)
         ? i18n.str`Invalid`
         : Amounts.cmp(totalRefundable, Amounts.parse(form.refund)!) === -1
-          ? i18n.str`This value exceed the refundable amount`
+          ? i18n.str`This value exceeds the refundable amount.`
           : undefined,
   };
   const hasErrors = Object.keys(errors).some(
@@ -351,7 +362,7 @@ export function RefundModal({
                       <tr key={r.timestamp.t_s}>
                         <td>
                           {r.timestamp.t_s === "never"
-                            ? "never"
+                            ? i18n.str`Never`
                             : format(
                                 new Date(r.timestamp.t_s * 1000),
                                 datetimeFormatForSettings(settings),
