@@ -58,7 +58,7 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
     otp_device_id: !state.otp_device_id
       ? i18n.str`Required`
       : !/[a-zA-Z0-9]*/.test(state.otp_device_id)
-        ? i18n.str`Invalid. Only characters and numbers`
+        ? i18n.str`Invalid. Please enter letters and numbers only.`
         : undefined,
     otp_algorithm: !state.otp_algorithm ? i18n.str`Required` : undefined,
     otp_key: !state.otp_key
@@ -66,12 +66,12 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
       : !isRfc3548Base32Charset(state.otp_key)
         ? i18n.str`Just letters and numbers from 2 to 7`
         : state.otp_key.length !== 32
-          ? i18n.str`Size of the key must be 32`
+          ? i18n.str`The size of the key must be 32 characters`
           : undefined,
     otp_device_description: !state.otp_device_description
       ? i18n.str`Required`
       : !/[a-zA-Z0-9]*/.test(state.otp_device_description)
-        ? i18n.str`Invalid. Only characters and numbers`
+        ? i18n.str`Invalid. Please enter letters and numbers only.`
         : undefined,
   });
 
@@ -96,7 +96,7 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
               <Input<Entity>
                 name="otp_device_id"
                 label={i18n.str`ID`}
-                tooltip={i18n.str`Internal id on the system`}
+                tooltip={i18n.str`Internal ID on the system`}
               />
               <Input<Entity>
                 name="otp_device_description"
@@ -106,7 +106,7 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
               <InputSelector<Entity>
                 name="otp_algorithm"
                 label={i18n.str`Verification algorithm`}
-                tooltip={i18n.str`Algorithm to use to verify transaction in offline mode`}
+                tooltip={i18n.str`Algorithm used to verify transactions in offline mode`}
                 values={algorithms}
                 toStr={(v) => algorithmsNames[v]}
                 fromStr={(v) => Number(v)}
@@ -118,8 +118,8 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
                     name="otp_key"
                     label={i18n.str`Device key`}
                     inputType={showKey ? "text" : "password"}
-                    help={i18n.str`Be sure to be very hard to guess or use the random generator`}
-                    tooltip={i18n.str`Your device need to have exactly the same value`}
+                    help={i18n.str`Be sure to choose a password that is hard to guess, or use the random generator.`}
+                    tooltip={i18n.str`Your device needs to match exactly the same value`}
                     fromStr={(v) => v.toUpperCase()}
                     addonAfterAction={() => {
                       setShowKey(!showKey);
@@ -163,7 +163,7 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
                 disabled={hasErrors}
                 data-tooltip={
                   hasErrors
-                    ? i18n.str`Need to complete marked fields`
+                    ? i18n.str`Please complete the marked fields`
                     : i18n.str`Confirm operation`
                 }
                 onClick={submitForm}
