@@ -65,7 +65,7 @@ export default function CreateValidator({ onConfirm, onBack }: Props): VNode {
                 setNotif({
                   message: i18n.str`Could not add device`,
                   type: "ERROR",
-                  description: resp.detail.hint,
+                  description: resp.detail?.hint,
                 });
               }
             })
