@@ -54,7 +54,7 @@ export function CardTable({
           <span class="icon">
             <i class="mdi mdi-lock" />
           </span>
-          <i18n.Translate>OTP Devices</i18n.Translate>
+          <i18n.Translate>OTP devices</i18n.Translate>
         </p>
         <div class="card-header-icon" aria-label="more options">
           <span
@@ -188,7 +188,7 @@ function EmptyTable(): VNode {
       </p>
       <p>
         <i18n.Translate>
-          There is no devices yet, add more pressing the + sign
+          There are no devices to list yet, add more by pressing the + sign
         </i18n.Translate>
       </p>
     </div>
