@@ -26,7 +26,6 @@ import {
   assertUnreachable
 } from "@gnu-taler/taler-util";
 import {
-  useMerchantApiContext,
   useTranslationContext
 } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
@@ -37,10 +36,11 @@ import { NotificationCard } from "../../../../components/menu/index.js";
 import { useSessionContext } from "../../../../context/session.js";
 import { useOtpDeviceDetails } from "../../../../hooks/otp.js";
 import { Notification } from "../../../../utils/types.js";
+import { LoginPage } from "../../../login/index.js";
 import { NotFoundPageOrAdminCreate } from "../../../notfound/index.js";
 import { CreatedSuccessfully } from "../create/CreatedSuccessfully.js";
 import { UpdatePage } from "./UpdatePage.js";
-import { LoginPage } from "../../../login/index.js";
+import { WithId } from "../../../../declaration.js";
 
 export type Entity = TalerMerchantApi.OtpDevicePatchDetails & WithId;
 
@@ -58,8 +58,7 @@ export default function UpdateValidator({
   const [notif, setNotif] = useState<Notification | undefined>(undefined);
   const [keyUpdated, setKeyUpdated] =
     useState<TalerMerchantApi.OtpDeviceAddDetails | null>(null);
-  const { lib } = useMerchantApiContext();
-  const { state } = useSessionContext();
+  const { state, lib } = useSessionContext();
 
   const { i18n } = useTranslationContext();
 
@@ -136,9 +135,9 @@ export default function UpdateValidator({
             })
             .catch((error) => {
               setNotif({
-                message: i18n.str`could not update template`,
+                message: i18n.str`Could not update template`,
                 type: "ERROR",
-                description: error.message,
+                description: error instanceof Error ? error.message : String(error),
               });
             });
         }}
