@@ -30,7 +30,7 @@ type Entity = TalerMerchantApi.ProductAddDetail & {
   product_id: string;
 };
 
-interface Props {
+export interface Props {
   onCreate: (d: Entity) => Promise<void>;
   onBack?: () => void;
 }
@@ -63,7 +63,7 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
                 onClick={submitForm}
                 data-tooltip={
                   !submitForm
-                    ? i18n.str`Need to complete marked fields`
+                    ? i18n.str`Please complete the marked fields`
                     : "confirm operation"
                 }
                 disabled={!submitForm}
