@@ -82,7 +82,7 @@ export function UpdatePage({ product, onUpdate, onBack }: Props): VNode {
                 onClick={submitForm}
                 data-tooltip={
                   !submitForm
-                    ? i18n.str`Need to complete marked fields`
+                    ? i18n.str`Please complete the marked fields`
                     : "confirm operation"
                 }
                 disabled={!submitForm}
