@@ -90,7 +90,7 @@ export default function UpdateProduct({
                 setNotif({
                   message: i18n.str`Could not update product`,
                   type: "ERROR",
-                  description: resp.detail.hint,
+                  description: resp.detail?.hint,
                 });
 
               }
