@@ -20,6 +20,7 @@
  */
 
 import {
+  HostPortPath,
   TalerMerchantApi,
   stringifyPayTemplateUri,
 } from "@gnu-taler/taler-util";
@@ -40,7 +41,7 @@ export function QrPage({ id: templateId, onBack }: Props): VNode {
   const { i18n } = useTranslationContext();
   const { state } = useSessionContext();
 
-  const merchantBaseUrl = state.backendUrl.href;
+  const merchantBaseUrl = state.backendUrl.href as HostPortPath;
 
   const payTemplateUri = stringifyPayTemplateUri({
     merchantBaseUrl,
