@@ -30,6 +30,7 @@ import {
 } from "../../../../components/form/FormProvider.js";
 import { Input } from "../../../../components/form/Input.js";
 import { InputCurrency } from "../../../../components/form/InputCurrency.js";
+import { InputWithAddon } from "../../../../components/form/InputWithAddon.js";
 
 type Entity = TalerMerchantApi.TemplateContractDetails;
 
@@ -55,8 +56,10 @@ export function UsePage({ id, template, onCreateOrder, onBack }: Props): VNode {
   });
 
   const errors: FormErrors<Entity> = {
-    amount: !state.amount ? i18n.str`Amount is required` : undefined,
-    summary: !state.summary ? i18n.str`Order summary is required` : undefined,
+    amount: !state.amount ? i18n.str`An amount is required` : undefined,
+    summary: !state.summary
+      ? i18n.str`An order summary is required`
+      : undefined,
   };
 
   const hasErrors = Object.keys(errors).some(
@@ -83,7 +86,7 @@ export function UsePage({ id, template, onCreateOrder, onBack }: Props): VNode {
               <div class="level-left">
                 <div class="level-item">
                   <span class="is-size-4">
-                    <i18n.Translate>New order for template</i18n.Translate>:{" "}
+                    <i18n.Translate>New order from template</i18n.Translate>:{" "}
                     <b>{id}</b>
                   </span>
                 </div>
@@ -101,12 +104,20 @@ export function UsePage({ id, template, onCreateOrder, onBack }: Props): VNode {
               valueHandler={setState}
               errors={errors}
             >
-              <InputCurrency<Entity>
+              <InputWithAddon<Entity>
                 name="amount"
                 label={i18n.str`Amount`}
-                readonly={!!template.template_contract.amount}
+                addonBefore={state.currency}
+                inputType="number"
+                toStr={(v?: AmountString) => v?.split(":")[1] || ""}
+                fromStr={(v: string) =>
+                  !v ? undefined : `${state.currency}:${v}`
+                }
+                inputExtra={{ min: 0, step: 0.001 }}
                 tooltip={i18n.str`Amount of the order`}
+                readonly={!!template.template_contract.amount}
               />
+
               <Input<Entity>
                 name="summary"
                 inputType="multiline"
@@ -126,7 +137,7 @@ export function UsePage({ id, template, onCreateOrder, onBack }: Props): VNode {
                 disabled={hasErrors}
                 data-tooltip={
                   hasErrors
-                    ? i18n.str`Need to complete marked fields`
+                    ? i18n.str`Please complete the marked fields`
                     : "confirm operation"
                 }
                 onClick={submitForm}
