@@ -19,18 +19,19 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { HttpStatusCode, TalerError, TalerMerchantApi, assertUnreachable } from "@gnu-taler/taler-util";
 import {
-  useTranslationContext
-} from "@gnu-taler/web-util/browser";
+  HttpStatusCode,
+  TalerError,
+  TalerMerchantApi,
+  assertUnreachable,
+} from "@gnu-taler/taler-util";
+import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
 import { useState } from "preact/hooks";
 import { ErrorLoadingMerchant } from "../../../../components/ErrorLoadingMerchant.js";
 import { Loading } from "../../../../components/exception/loading.js";
 import { NotificationCard } from "../../../../components/menu/index.js";
-import {
-  useTemplateDetails
-} from "../../../../hooks/templates.js";
+import { useTemplateDetails } from "../../../../hooks/templates.js";
 import { Notification } from "../../../../utils/types.js";
 import { LoginPage } from "../../../login/index.js";
 import { NotFoundPageOrAdminCreate } from "../../../notfound/index.js";
@@ -54,20 +55,20 @@ export default function TemplateUsePage({
   const [notif, setNotif] = useState<Notification | undefined>(undefined);
   const { i18n } = useTranslationContext();
 
-  if (!result) return <Loading />
+  if (!result) return <Loading />;
   if (result instanceof TalerError) {
-    return <ErrorLoadingMerchant error={result} />
+    return <ErrorLoadingMerchant error={result} />;
   }
   if (result.type === "fail") {
-    switch(result.case) {
+    switch (result.case) {
       case HttpStatusCode.NotFound: {
-        return <NotFoundPageOrAdminCreate />
+        return <NotFoundPageOrAdminCreate />;
       }
       case HttpStatusCode.Unauthorized: {
-        return <LoginPage />
+        return <LoginPage />;
       }
       default: {
-        assertUnreachable(result)
+        assertUnreachable(result);
       }
     }
   }
@@ -79,29 +80,52 @@ export default function TemplateUsePage({
         template={result.body}
         id={tid}
         onBack={onBack}
-        onCreateOrder={(
-          request: TalerMerchantApi.UsingTemplateDetails,
-        ) => {
-          return lib.instance.useTemplateCreateOrder(tid, request)
-            .then((res) => {
-              if (res.type === "ok") {
-                onOrderCreated(res.body.order_id)
+        onCreateOrder={(request: TalerMerchantApi.UsingTemplateDetails) => {
+          return lib.instance
+            .useTemplateCreateOrder(tid, request)
+            .then((resp) => {
+              if (resp.type === "ok") {
+                onOrderCreated(resp.body.order_id);
               } else {
-                setNotif({
-                  message: i18n.str`Could not create order from template`,
-                  type: "ERROR",
-                  description:
-                    res.case === HttpStatusCode.Gone
-                      ? i18n.str`No more stock for product with id "${res.body.product_id}".`
-                      : res.detail.hint,
-                });
+                switch (resp.case) {
+                  case HttpStatusCode.UnavailableForLegalReasons: {
+                    setNotif({
+                      message: i18n.str`Could not create order`,
+                      type: "ERROR",
+                      description: i18n.str`No exchange would accept a payment because of KYC requirements.`,
+                    });
+                    return;
+                  }
+                  case HttpStatusCode.Unauthorized:
+                  case HttpStatusCode.NotFound:
+                  case HttpStatusCode.Conflict: {
+                    setNotif({
+                      message: i18n.str`Could not create order`,
+                      type: "ERROR",
+                      description: resp.detail?.hint,
+                    });
+                    return;
+                  }
+                  case HttpStatusCode.Gone: {
+                    setNotif({
+                      message: i18n.str`Could not create order`,
+                      type: "ERROR",
+                      description: i18n.str`No more stock for product with ID "${resp.body.product_id}".`,
+                    });
+                    return;
+                  }
+                  default: {
+                    assertUnreachable(resp);
+                  }
+                }
               }
             })
             .catch((error) => {
               setNotif({
                 message: i18n.str`Could not create order from template`,
                 type: "ERROR",
-                description: error instanceof Error ? error.message : String(error),
+                description:
+                  error instanceof Error ? error.message : String(error),
               });
             });
         }}
