@@ -67,7 +67,7 @@ export default function CreateTransfer({ onConfirm, onBack }: Props): VNode {
                 setNotif({
                   message: i18n.str`Could not inform transfer`,
                   type: "ERROR",
-                  description: resp.detail.hint,
+                  description: resp.detail?.hint,
                 });
               }
             })
