@@ -21,6 +21,7 @@
 
 import { FunctionalComponent, h } from "preact";
 import { UpdatePage as TestedComponent } from "./UpdatePage.js";
+import { MerchantAuthMethod } from "@gnu-taler/taler-util";
 
 export default {
   title: "Pages/Instance/Update",
@@ -43,9 +44,8 @@ function createExample<Props>(
 export const Example = createExample(TestedComponent, {
   selected: {
     name: "name",
-    auth: { method: "external" },
+    auth: { method: MerchantAuthMethod.TOKEN },
     address: {},
-    user_type: "business",
     use_stefan: true,
     jurisdiction: {},
     default_pay_delay: {
