@@ -27,6 +27,7 @@ import { AsyncButton } from "../../../components/exception/AsyncButton.js";
 import {
   FormErrors,
   FormProvider,
+  TalerForm,
 } from "../../../components/form/FormProvider.js";
 import { DefaultInstanceFormFields } from "../../../components/instance/DefaultInstanceFormFields.js";
 import { useSessionContext } from "../../../context/session.js";
@@ -35,9 +36,9 @@ import { undefinedIfEmpty } from "../../../utils/table.js";
 export type Entity = Omit<Omit<TalerMerchantApi.InstanceReconfigurationMessage, "default_pay_delay">, "default_wire_transfer_delay"> & {
   default_pay_delay: Duration,
   default_wire_transfer_delay: Duration,
-};
+} & TalerForm;
 
-interface Props {
+export interface Props {
   onUpdate: (d: TalerMerchantApi.InstanceReconfigurationMessage) => void;
   selected: TalerMerchantApi.QueryInstancesResponse;
   isLoading: boolean;
@@ -54,7 +55,7 @@ function convert(
     default_pay_delay: Duration.fromTalerProtocolDuration(default_pay_delay),
     default_wire_transfer_delay: Duration.fromTalerProtocolDuration(default_wire_transfer_delay),
   };
-  return { ...defaults, ...rest };
+  return { ...defaults, ...rest } as Entity;
 }
 
 export function UpdatePage({
@@ -70,11 +71,6 @@ export function UpdatePage({
 
   const errors = undefinedIfEmpty<FormErrors<Entity>>({
     name: !value.name ? i18n.str`Required` : undefined,
-    user_type: !value.user_type
-      ? i18n.str`Required`
-      : value.user_type !== "business" && value.user_type !== "individual"
-        ? i18n.str`Must be business or individual`
-        : undefined,
     default_pay_delay: !value.default_pay_delay
       ? i18n.str`Required`
       : !!value.default_wire_transfer_delay &&
@@ -155,7 +151,7 @@ export function UpdatePage({
                 onClick={submit}
                 data-tooltip={
                   hasErrors
-                    ? i18n.str`Need to complete marked fields`
+                    ? i18n.str`Please complete the marked fields`
                     : i18n.str`Confirm operation`
                 }
                 disabled={hasErrors}
