@@ -57,7 +57,7 @@ export default function CreateWebhook({ onConfirm, onBack }: Props): VNode {
                 setNotif({
                   message: i18n.str`Could not create the webhook`,
                   type: "ERROR",
-                  description: resp.detail.hint,
+                  description: resp.detail?.hint,
                 });
               }
             })
