@@ -85,14 +85,14 @@ export default function ListWebhooks({ onCreate, onSelect }: Props): VNode {
             .then((resp) => {
               if (resp.type === "ok") {
                 setNotif({
-                  message: i18n.str`Webhook delete successfully`,
+                  message: i18n.str`Webhook deleted successfully`,
                   type: "SUCCESS",
                 });
               } else {
                 setNotif({
                   message: i18n.str`Could not delete the webhook`,
                   type: "ERROR",
-                  description: resp.detail.hint,
+                  description: resp.detail?.hint,
                 });
               }
             })
