@@ -50,8 +50,8 @@ export function NotFoundPageOrAdminCreate(): VNode {
       <Fragment>
         <NotificationCard
           notification={{
-            message: i18n.str`No 'default' instance configured yet.`,
-            description: i18n.str`Create a 'default' instance to begin using the merchant backoffice.`,
+            message: i18n.str`No 'admin' instance configured yet.`,
+            description: i18n.str`Create an 'admin' instance to begin using the merchant backoffice.`,
             type: "INFO",
           }}
         />
