@@ -26,17 +26,8 @@ import { InputToggle } from "../../components/form/InputToggle.js";
 import { LangSelector } from "../../components/menu/LangSelector.js";
 import { Preferences, usePreference } from "../../hooks/preference.js";
 
-function getBrowserLang(): string | undefined {
-  if (typeof window === "undefined") return undefined;
-  if (window.navigator.languages) return window.navigator.languages[0];
-  if (window.navigator.language) return window.navigator.language;
-  return undefined;
-}
-
 export function Settings({ onClose }: { onClose?: () => void }): VNode {
   const { i18n } = useTranslationContext();
-  const borwserLang = getBrowserLang();
-  const { update } = useLang(undefined, {});
 
   const [value, , updateValue] = usePreference();
   const errors: FormErrors<Preferences> = {};
@@ -83,19 +74,6 @@ export function Settings({ onClose }: { onClose?: () => void }): VNode {
                   </div>
                   <div class="field field-body has-addons is-flex-grow-3">
                     <LangSelector />
-                    &nbsp;
-                    {borwserLang !== undefined && (
-                      <button
-                        data-tooltip={i18n.str`Generate random secret key`}
-                        class="button is-info mr-2"
-                        onClick={(e) => {
-                          update(borwserLang.substring(0, 2));
-                          e.preventDefault()
-                        }}
-                      >
-                        <i18n.Translate>Set default</i18n.Translate>
-                      </button>
-                    )}
                   </div>
                 </div>
                 <InputToggle<Preferences>
@@ -132,7 +110,7 @@ export function Settings({ onClose }: { onClose?: () => void }): VNode {
                 />
                 <InputToggle<Preferences>
                   label={i18n.str`Developer mode`}
-                  tooltip={i18n.str`Shows more options and tools which are not intended for general audience.`}
+                  tooltip={i18n.str`Shows more options and tools that are not intended for a general audience.`}
                   name="developerMode"
                 />
               </FormProvider>
