@@ -19,17 +19,19 @@
  */
 import {
   AmountString,
-  NotificationType,
   TransactionMajorState,
   TransactionMinorState,
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
-import { GlobalTestState, generateRandomPayto } from "../harness/harness.js";
+import { defaultCoinConfig } from "../harness/denomStructures.js";
 import {
   createSimpleTestkudosEnvironmentV3,
   withdrawViaBankV3,
-} from "../harness/helpers.js";
-import { defaultCoinConfig } from "../harness/denomStructures.js";
+} from "../harness/environments.js";
+import {
+  GlobalTestState,
+  getTestHarnessPaytoForLabel,
+} from "../harness/harness.js";
 
 /**
  * Run test for basic, bank-integrated withdrawal and payment.
@@ -64,39 +66,35 @@ export async function runAgeRestrictionsDepositTest(t: GlobalTestState) {
 
   const depositTxId = dgIdResp.transactionId;
 
-  const depositTrack = walletClient.waitForNotificationCond(
-    (n) =>
-      n.type == NotificationType.TransactionStateTransition &&
-      n.transactionId == depositTxId &&
-      n.newTxState.major == TransactionMajorState.Pending &&
-      n.newTxState.minor == TransactionMinorState.Track,
-  );
-
-  const depositDone = walletClient.waitForNotificationCond(
-    (n) =>
-      n.type == NotificationType.TransactionStateTransition &&
-      n.transactionId == depositTxId &&
-      n.newTxState.major == TransactionMajorState.Done,
-  );
-
   const depositGroupResult = await walletClient.client.call(
     WalletApiOperation.CreateDepositGroup,
     {
       amount: "TESTKUDOS:10" as AmountString,
-      depositPaytoUri: generateRandomPayto("foo"),
+      depositPaytoUri: getTestHarnessPaytoForLabel("foo"),
       transactionId: depositTxId,
     },
   );
 
   t.assertDeepEqual(depositGroupResult.transactionId, depositTxId);
 
-  await depositTrack;
+  await walletClient.call(WalletApiOperation.TestingWaitTransactionState, {
+    transactionId: depositTxId,
+    txState: {
+      major: TransactionMajorState.Finalizing,
+      minor: TransactionMinorState.Track,
+    },
+  });
 
   await exchange.runAggregatorOnceWithTimetravel({
     timetravelMicroseconds: 1000 * 1000 * 60 * 60 * 3,
   });
 
-  await depositDone;
+  await walletClient.call(WalletApiOperation.TestingWaitTransactionState, {
+    transactionId: depositTxId,
+    txState: {
+      major: TransactionMajorState.Done,
+    },
+  });
 
   const transactions = await walletClient.client.call(
     WalletApiOperation.GetTransactions,
