@@ -20,13 +20,13 @@
 import { AmountString, TalerMerchantApi } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { defaultCoinConfig } from "../harness/denomStructures.js";
-import { GlobalTestState } from "../harness/harness.js";
 import {
   createSimpleTestkudosEnvironmentV3,
   createWalletDaemonWithClient,
   makeTestPaymentV2,
   withdrawViaBankV3,
-} from "../harness/helpers.js";
+} from "../harness/environments.js";
+import { GlobalTestState } from "../harness/harness.js";
 
 /**
  * Run test for basic, bank-integrated withdrawal and payment.
@@ -39,6 +39,7 @@ export async function runAgeRestrictionsMerchantTest(t: GlobalTestState) {
     bankClient,
     exchange,
     merchant,
+    merchantAdminAccessToken,
     exchangeBankAccount,
   } = await createSimpleTestkudosEnvironmentV3(
     t,
@@ -85,6 +86,7 @@ export async function runAgeRestrictionsMerchantTest(t: GlobalTestState) {
     await makeTestPaymentV2(t, {
       walletClient: walletClientZero,
       merchant,
+      merchantAdminAccessToken,
       order,
     });
     await walletClientZero.call(
@@ -112,7 +114,7 @@ export async function runAgeRestrictionsMerchantTest(t: GlobalTestState) {
       minimum_age: 9,
     };
 
-    await makeTestPaymentV2(t, { walletClient, merchant, order });
+    await makeTestPaymentV2(t, { walletClient, merchant, order, merchantAdminAccessToken });
     await walletClient.call(
       WalletApiOperation.TestingWaitTransactionsFinal,
       {},
@@ -137,7 +139,7 @@ export async function runAgeRestrictionsMerchantTest(t: GlobalTestState) {
       fulfillment_url: "taler://fulfillment-success/thx",
     };
 
-    await makeTestPaymentV2(t, { walletClient, merchant, order });
+    await makeTestPaymentV2(t, { walletClient, merchant, order, merchantAdminAccessToken });
     await walletClient.call(
       WalletApiOperation.TestingWaitTransactionsFinal,
       {},
@@ -162,7 +164,7 @@ export async function runAgeRestrictionsMerchantTest(t: GlobalTestState) {
       minimum_age: 9,
     };
 
-    await makeTestPaymentV2(t, { walletClient, merchant, order });
+    await makeTestPaymentV2(t, { walletClient, merchant, order, merchantAdminAccessToken });
     await walletClient.call(
       WalletApiOperation.TestingWaitTransactionsFinal,
       {},
@@ -172,3 +174,4 @@ export async function runAgeRestrictionsMerchantTest(t: GlobalTestState) {
 
 runAgeRestrictionsMerchantTest.suites = ["wallet"];
 runAgeRestrictionsMerchantTest.timeoutMs = 120 * 1000;
+runAgeRestrictionsMerchantTest.experimental = false;
