@@ -33,7 +33,7 @@ import {
   createSimpleTestkudosEnvironmentV3,
   createWalletDaemonWithClient,
   withdrawViaBankV3,
-} from "../harness/helpers.js";
+} from "../harness/environments.js";
 
 /**
  * Run test for basic, bank-integrated withdrawal and payment.
