@@ -17,14 +17,17 @@
 /**
  * Imports.
  */
+import {
+  succeedOrThrow,
+  TalerMerchantInstanceHttpClient,
+} from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
-import { URL } from "url";
-import { GlobalTestState } from "../harness/harness.js";
+import { URL } from "node:url";
 import {
   createSimpleTestkudosEnvironmentV3,
   withdrawViaBankV3,
-} from "../harness/helpers.js";
-import { MerchantApiClient } from "@gnu-taler/taler-util";
+} from "../harness/environments.js";
+import { GlobalTestState } from "../harness/harness.js";
 
 /**
  * Run test for the merchant's order lifecycle.
@@ -35,7 +38,7 @@ import { MerchantApiClient } from "@gnu-taler/taler-util";
 export async function runClaimLoopTest(t: GlobalTestState) {
   // Set up test environment
 
-  const { walletClient, bankClient, exchange, merchant } =
+  const { walletClient, bankClient, exchange, merchant, merchantAdminAccessToken } =
     await createSimpleTestkudosEnvironmentV3(t);
 
   await withdrawViaBankV3(t, {
@@ -45,22 +48,27 @@ export async function runClaimLoopTest(t: GlobalTestState) {
     amount: "TESTKUDOS:20",
   });
 
-  const merchantClient = new MerchantApiClient(merchant.makeInstanceBaseUrl());
+  const merchantClient = new TalerMerchantInstanceHttpClient(
+    merchant.makeInstanceBaseUrl(),
+  );
 
   // Set up order.
-  const orderResp = await merchantClient.createOrder({
-    order: {
-      summary: "Buy me!",
-      amount: "TESTKUDOS:5",
-      fulfillment_url: "taler://fulfillment-success/thx",
-    },
-  });
+  const orderResp = succeedOrThrow(
+    await merchantClient.createOrder(merchantAdminAccessToken, {
+      order: {
+        summary: "Buy me!",
+        amount: "TESTKUDOS:5",
+        fulfillment_url: "taler://fulfillment-success/thx",
+      },
+    }),
+  );
 
   // Query private order status before claiming it.
-  let orderStatusBefore = await merchantClient.queryPrivateOrderStatus({
-    orderId: orderResp.order_id,
-  });
+  let orderStatusBefore = succeedOrThrow(
+    await merchantClient.getOrderDetails(merchantAdminAccessToken, orderResp.order_id),
+  );
   t.assertTrue(orderStatusBefore.order_status === "unpaid");
+
   let statusUrlBefore = new URL(orderStatusBefore.order_status_url);
 
   // Make wallet claim the unpaid order.
@@ -71,9 +79,9 @@ export async function runClaimLoopTest(t: GlobalTestState) {
   });
 
   // Query private order status after claiming it.
-  let orderStatusAfter = await merchantClient.queryPrivateOrderStatus({
-    orderId: orderResp.order_id,
-  });
+  let orderStatusAfter = succeedOrThrow(
+    await merchantClient.getOrderDetails(merchantAdminAccessToken, orderResp.order_id),
+  );
   t.assertTrue(orderStatusAfter.order_status === "claimed");
 
   await t.shutdown();
