@@ -22,7 +22,7 @@ import { GlobalTestState } from "../harness/harness.js";
 import {
   createSimpleTestkudosEnvironmentV3,
   withdrawViaBankV3,
-} from "../harness/helpers.js";
+} from "../harness/environments.js";
 
 /**
  * Run test for refreshe after a payment.
