@@ -19,11 +19,11 @@
  */
 import {
   AbsoluteTime,
-  codecForExchangeKeysJson,
+  codecForExchangeKeysResponse,
   DenominationPubKey,
   DenomKeyType,
   Duration,
-  ExchangeKeysJson,
+  ExchangeKeysResponse,
   Logger,
   TalerCorebankApiClient,
 } from "@gnu-taler/taler-util";
@@ -33,9 +33,9 @@ import {
 } from "@gnu-taler/taler-util/http";
 import { makeNoFeeCoinConfig } from "../harness/denomStructures.js";
 import {
-    BankService,
+  BankService,
   ExchangeService,
-  generateRandomPayto,
+  getTestHarnessPaytoForLabel,
   GlobalTestState,
   MerchantService,
   setupDb,
@@ -44,7 +44,7 @@ import {
   applyTimeTravelV2,
   createWalletDaemonWithClient,
   withdrawViaBankV3,
-} from "../harness/helpers.js";
+} from "../harness/environments.js";
 
 const logger = new Logger("test-exchange-timetravel.ts");
 
@@ -53,7 +53,7 @@ interface DenomInfo {
   expireDeposit: string;
 }
 
-function getDenomInfoFromKeys(ek: ExchangeKeysJson): DenomInfo[] {
+function getDenomInfoFromKeys(ek: ExchangeKeysResponse): DenomInfo[] {
   const denomInfos: DenomInfo[] = [];
   for (const denomGroup of ek.denominations) {
     switch (denomGroup.cipher) {
@@ -120,20 +120,24 @@ export async function runExchangeTimetravelTest(t: GlobalTestState) {
 
   const merchant = await MerchantService.create(t, {
     name: "testmerchant-1",
-    currency: "TESTKUDOS",
     httpPort: 8083,
     database: db.connStr,
   });
 
   let receiverName = "Exchange";
   let exchangeBankUsername = "exchange";
-  let exchangeBankPassword = "mypw";
-  let exchangePaytoUri = generateRandomPayto(exchangeBankUsername);
+  let exchangeBankPassword = "mypw-password";
+  let exchangePaytoUri = getTestHarnessPaytoForLabel(exchangeBankUsername);
 
   await exchange.addBankAccount("1", {
-    accountName: exchangeBankUsername,
-    accountPassword: exchangeBankPassword,
-    wireGatewayApiBaseUrl: new URL("accounts/exchange/taler-wire-gateway/", bank.baseUrl).href,
+    wireGatewayAuth: {
+      username: exchangeBankUsername,
+      password: exchangeBankPassword,
+    },
+    wireGatewayApiBaseUrl: new URL(
+      "accounts/exchange/taler-wire-gateway/",
+      bank.baseUrl,
+    ).href,
     accountPaytoUri: exchangePaytoUri,
   });
 
@@ -146,7 +150,7 @@ export async function runExchangeTimetravelTest(t: GlobalTestState) {
   const bankClient = new TalerCorebankApiClient(bank.corebankApiBaseUrl, {
     auth: {
       username: "admin",
-      password: "adminpw",
+      password: "admin-password",
     },
   });
 
@@ -168,17 +172,17 @@ export async function runExchangeTimetravelTest(t: GlobalTestState) {
   await merchant.start();
   await merchant.pingUntilAvailable();
 
-  await merchant.addInstanceWithWireAccount({
-    id: "default",
+  const { accessToken: adminAccessToken } = await merchant.addInstanceWithWireAccount({
+    id: "admin",
     name: "Default Instance",
-    paytoUris: [generateRandomPayto("merchant-default")],
+    paytoUris: [getTestHarnessPaytoForLabel("merchant-default")],
   });
 
   await merchant.addInstanceWithWireAccount({
     id: "minst1",
     name: "minst1",
-    paytoUris: [generateRandomPayto("minst1")],
-  });
+    paytoUris: [getTestHarnessPaytoForLabel("minst1")],
+  }, {adminAccessToken});
 
   console.log("setup done!");
 
@@ -199,7 +203,7 @@ export async function runExchangeTimetravelTest(t: GlobalTestState) {
   const keysResp1 = await http.fetch(exchange.baseUrl + "keys");
   const keys1 = await readSuccessResponseJsonOrThrow(
     keysResp1,
-    codecForExchangeKeysJson(),
+    codecForExchangeKeysResponse(),
   );
   console.log(
     "keys 1 (before time travel):",
@@ -221,7 +225,7 @@ export async function runExchangeTimetravelTest(t: GlobalTestState) {
   const keysResp2 = await http.fetch(exchange.baseUrl + "keys");
   const keys2 = await readSuccessResponseJsonOrThrow(
     keysResp2,
-    codecForExchangeKeysJson(),
+    codecForExchangeKeysResponse(),
   );
   console.log(
     "keys 2 (after time travel):",
