@@ -20,7 +20,7 @@
 import { AmountString, j2s } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { GlobalTestState } from "../harness/harness.js";
-import { createSimpleTestkudosEnvironmentV2 } from "../harness/helpers.js";
+import { createSimpleTestkudosEnvironmentV2 } from "../harness/environments.js";
 
 /**
  * Run test for forced denom/coin selection.
@@ -28,7 +28,7 @@ import { createSimpleTestkudosEnvironmentV2 } from "../harness/helpers.js";
 export async function runForcedSelectionTest(t: GlobalTestState) {
   // Set up test environment
 
-  const { walletClient, bank, exchange, merchant } =
+  const { walletClient, bank, exchange, merchant, merchantAdminAccessToken } =
     await createSimpleTestkudosEnvironmentV2(t);
 
   await walletClient.call(WalletApiOperation.AddExchange, {
@@ -58,6 +58,7 @@ export async function runForcedSelectionTest(t: GlobalTestState) {
   const payResp = await walletClient.call(WalletApiOperation.TestPay, {
     amount: "TESTKUDOS:3" as AmountString,
     merchantBaseUrl: merchant.makeInstanceBaseUrl(),
+    merchantAuthToken: merchantAdminAccessToken,
     summary: "bla",
     forcedCoinSel: {
       coins: [
