@@ -23,7 +23,7 @@ import {
   createSimpleTestkudosEnvironmentV3,
   makeTestPaymentV2,
   withdrawViaBankV3,
-} from "../harness/helpers.js";
+} from "../harness/environments.js";
 import { TalerMerchantApi } from "@gnu-taler/taler-util";
 
 /**
@@ -32,7 +32,7 @@ import { TalerMerchantApi } from "@gnu-taler/taler-util";
 export async function runPaymentForgettableTest(t: GlobalTestState) {
   // Set up test environment
 
-  const { walletClient, bankClient, exchange, merchant } =
+  const { walletClient, bankClient, exchange, merchant, merchantAdminAccessToken } =
     await createSimpleTestkudosEnvironmentV3(t);
 
   // Withdraw digital cash into the wallet.
@@ -59,7 +59,7 @@ export async function runPaymentForgettableTest(t: GlobalTestState) {
       },
     };
 
-    await makeTestPaymentV2(t, { walletClient, merchant, order });
+    await makeTestPaymentV2(t, { walletClient, merchant, order, merchantAdminAccessToken });
   }
 
   console.log("testing with forgettable field without hash");
@@ -77,7 +77,7 @@ export async function runPaymentForgettableTest(t: GlobalTestState) {
       },
     };
 
-    await makeTestPaymentV2(t, { walletClient, merchant, order });
+    await makeTestPaymentV2(t, { walletClient, merchant, order, merchantAdminAccessToken });
   }
 
   await walletClient.call(WalletApiOperation.TestingWaitTransactionsFinal, {});
