@@ -17,13 +17,17 @@
 /**
  * Imports.
  */
-import { MerchantApiClient, PreparePayResultType } from "@gnu-taler/taler-util";
+import {
+  PreparePayResultType,
+  succeedOrThrow,
+  TalerMerchantInstanceHttpClient,
+} from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
-import { GlobalTestState } from "../harness/harness.js";
 import {
   createSimpleTestkudosEnvironmentV3,
   withdrawViaBankV3,
-} from "../harness/helpers.js";
+} from "../harness/environments.js";
+import { GlobalTestState } from "../harness/harness.js";
 
 /**
  * Test the wallet-core payment API, especially that repeated operations
@@ -32,7 +36,7 @@ import {
 export async function runPaymentIdempotencyTest(t: GlobalTestState) {
   // Set up test environment
 
-  const { walletClient, bankClient, exchange, merchant } =
+  const { walletClient, bankClient, exchange, merchant, merchantAdminAccessToken } =
     await createSimpleTestkudosEnvironmentV3(t);
 
   // Withdraw digital cash into the wallet.
@@ -46,21 +50,25 @@ export async function runPaymentIdempotencyTest(t: GlobalTestState) {
 
   await wres.withdrawalFinishedCond;
 
-  const merchantClient = new MerchantApiClient(merchant.makeInstanceBaseUrl());
+  const merchantClient = new TalerMerchantInstanceHttpClient(
+    merchant.makeInstanceBaseUrl(),
+  );
 
   // Set up order.
 
-  const orderResp = await merchantClient.createOrder({
-    order: {
-      summary: "Buy me!",
-      amount: "TESTKUDOS:5",
-      fulfillment_url: "taler://fulfillment-success/thx",
-    },
-  });
+  const orderResp = succeedOrThrow(
+    await merchantClient.createOrder(merchantAdminAccessToken, {
+      order: {
+        summary: "Buy me!",
+        amount: "TESTKUDOS:5",
+        fulfillment_url: "taler://fulfillment-success/thx",
+      },
+    }),
+  );
 
-  let orderStatus = await merchantClient.queryPrivateOrderStatus({
-    orderId: orderResp.order_id,
-  });
+  let orderStatus = succeedOrThrow(
+    await merchantClient.getOrderDetails(merchantAdminAccessToken, orderResp.order_id),
+  );
 
   t.assertTrue(orderStatus.order_status === "unpaid");
 
@@ -89,12 +97,12 @@ export async function runPaymentIdempotencyTest(t: GlobalTestState) {
     preparePayResultRep.status === PreparePayResultType.PaymentPossible,
   );
 
-  const proposalId = preparePayResult.proposalId;
+  const proposalTransactionId = preparePayResult.transactionId;
 
   const confirmPayResult = await walletClient.call(
     WalletApiOperation.ConfirmPay,
     {
-      proposalId: proposalId,
+      transactionId: proposalTransactionId,
     },
   );
 
@@ -104,9 +112,9 @@ export async function runPaymentIdempotencyTest(t: GlobalTestState) {
 
   // Check if payment was successful.
 
-  orderStatus = await merchantClient.queryPrivateOrderStatus({
-    orderId: orderResp.order_id,
-  });
+  orderStatus = succeedOrThrow(
+    await merchantClient.getOrderDetails(merchantAdminAccessToken, orderResp.order_id),
+  );
 
   t.assertTrue(orderStatus.order_status === "paid");
 
