@@ -23,23 +23,24 @@ import {
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { CoinConfig } from "../harness/denomStructures.js";
-import {
-  BankService,
-  ExchangeService,
-  GlobalTestState,
-  MerchantService,
-  WalletCli,
-  WalletClient,
-  delayMs,
-  generateRandomPayto,
-  setupDb,
-} from "../harness/harness.js";
 import {
   SimpleTestEnvironmentNg3,
   createWalletDaemonWithClient,
   makeTestPaymentV2,
   withdrawViaBankV3,
-} from "../harness/helpers.js";
+} from "../harness/environments.js";
+import {
+  BankService,
+  ExchangeService,
+  GlobalTestState,
+  HarnessExchangeBankAccount,
+  MerchantService,
+  WalletCli,
+  WalletClient,
+  delayMs,
+  getTestHarnessPaytoForLabel,
+  setupDb,
+} from "../harness/harness.js";
 
 async function revokeAllWalletCoins(req: {
   walletClient: WalletClient;
@@ -85,25 +86,26 @@ async function createTestEnvironment(
 
   const merchant = await MerchantService.create(t, {
     name: "testmerchant-1",
-    currency: "TESTKUDOS",
     httpPort: 8083,
     database: db.connStr,
   });
 
   let receiverName = "Exchange";
   let exchangeBankUsername = "exchange";
-  let exchangeBankPassword = "mypw";
-  let exchangePaytoUri = generateRandomPayto(exchangeBankUsername);
+  let exchangeBankPassword = "mypw-password";
+  let exchangePaytoUri = getTestHarnessPaytoForLabel(exchangeBankUsername);
 
-  await exchange.addBankAccount("1", {
-    accountName: exchangeBankUsername,
-    accountPassword: exchangeBankPassword,
+  const exchangeBankAccount: HarnessExchangeBankAccount = {
+    wireGatewayAuth: {
+      username: exchangeBankUsername,
+      password: exchangeBankPassword,
+    },
     wireGatewayApiBaseUrl: new URL(
       "accounts/exchange/taler-wire-gateway/",
       bank.baseUrl,
     ).href,
     accountPaytoUri: exchangePaytoUri,
-  });
+  };
 
   bank.setSuggestedExchange(exchange, exchangePaytoUri);
 
@@ -114,7 +116,7 @@ async function createTestEnvironment(
   const bankClient = new TalerCorebankApiClient(bank.corebankApiBaseUrl, {
     auth: {
       username: "admin",
-      password: "adminpw",
+      password: "admin-password",
     },
   });
 
@@ -150,17 +152,17 @@ async function createTestEnvironment(
   await merchant.start();
   await merchant.pingUntilAvailable();
 
-  await merchant.addInstanceWithWireAccount({
-    id: "default",
+  const { accessToken: adminAccessToken } = await merchant.addInstanceWithWireAccount({
+    id: "admin",
     name: "Default Instance",
-    paytoUris: [generateRandomPayto("merchant-default")],
+    paytoUris: [getTestHarnessPaytoForLabel("merchant-default")],
   });
 
   await merchant.addInstanceWithWireAccount({
     id: "minst1",
     name: "minst1",
-    paytoUris: [generateRandomPayto("minst1")],
-  });
+    paytoUris: [getTestHarnessPaytoForLabel("minst1")],
+  }, {adminAccessToken});
 
   console.log("setup done!");
 
@@ -178,14 +180,11 @@ async function createTestEnvironment(
     exchange,
     merchant,
     walletClient,
+    merchantAdminAccessToken: adminAccessToken,
     walletService,
+    bank,
     bankClient,
-    exchangeBankAccount: {
-      accountName: "",
-      accountPassword: "",
-      accountPaytoUri: "",
-      wireGatewayApiBaseUrl: "",
-    },
+    exchangeBankAccount,
   };
 }
 
@@ -195,7 +194,7 @@ async function createTestEnvironment(
 export async function runRevocationTest(t: GlobalTestState) {
   // Set up test environment
 
-  const { walletClient, bankClient, exchange, merchant } =
+  const { walletClient, bankClient, exchange, merchant, merchantAdminAccessToken } =
     await createTestEnvironment(t);
 
   // Withdraw digital cash into the wallet.
@@ -226,7 +225,7 @@ export async function runRevocationTest(t: GlobalTestState) {
     fulfillment_url: "taler://fulfillment-success/thx",
   } satisfies TalerMerchantApi.Order;
 
-  await makeTestPaymentV2(t, { walletClient, merchant, order });
+  await makeTestPaymentV2(t, { walletClient, merchant, order, merchantAdminAccessToken });
 
   await walletClient.call(WalletApiOperation.ClearDb, {});
 
@@ -259,7 +258,7 @@ export async function runRevocationTest(t: GlobalTestState) {
     console.log("wallet balance", bal);
   }
 
-  await makeTestPaymentV2(t, { walletClient, merchant, order });
+  await makeTestPaymentV2(t, { walletClient, merchant, order, merchantAdminAccessToken });
 }
 
 runRevocationTest.timeoutMs = 120000;
