@@ -17,14 +17,17 @@
 /**
  * Imports.
  */
-import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
-import { GlobalTestState } from "../harness/harness.js";
 import {
-  withdrawViaBankV2,
+  TalerCorebankApiClient,
+  TalerMerchantApi,
+} from "@gnu-taler/taler-util";
+import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
+import {
   makeTestPaymentV2,
   useSharedTestkudosEnvironment,
-} from "../harness/helpers.js";
-import { TalerMerchantApi } from "@gnu-taler/taler-util";
+  withdrawViaBankV3,
+} from "../harness/environments.js";
+import { GlobalTestState } from "../harness/harness.js";
 
 /**
  * Run test for basic, bank-integrated withdrawal and payment.
@@ -32,14 +35,15 @@ import { TalerMerchantApi } from "@gnu-taler/taler-util";
 export async function runSimplePaymentTest(t: GlobalTestState) {
   // Set up test environment
 
-  const { walletClient, bank, exchange, merchant } =
+  const { walletClient, bank, exchange, merchant, merchantAdminAccessToken } =
     await useSharedTestkudosEnvironment(t);
 
   // Withdraw digital cash into the wallet.
+  const bankClient = new TalerCorebankApiClient(bank.baseUrl);
 
-  await withdrawViaBankV2(t, {
+  await withdrawViaBankV3(t, {
     walletClient,
-    bank,
+    bankClient,
     exchange,
     amount: "TESTKUDOS:20",
   });
@@ -52,7 +56,7 @@ export async function runSimplePaymentTest(t: GlobalTestState) {
     fulfillment_url: "taler://fulfillment-success/thx",
   } satisfies TalerMerchantApi.Order;
 
-  await makeTestPaymentV2(t, { walletClient, merchant, order });
+  await makeTestPaymentV2(t, { walletClient, merchant, order, merchantAdminAccessToken });
   await walletClient.call(WalletApiOperation.TestingWaitTransactionsFinal, {});
 }
 
