@@ -17,14 +17,14 @@
 /**
  * Imports.
  */
+import { TalerCorebankApiClient, TalerMerchantApi } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
-import { GlobalTestState } from "../harness/harness.js";
 import {
-  withdrawViaBankV2,
   makeTestPaymentV2,
   useSharedTestkudosEnvironment,
-} from "../harness/helpers.js";
-import { TalerMerchantApi } from "@gnu-taler/taler-util";
+  withdrawViaBankV3,
+} from "../harness/environments.js";
+import { GlobalTestState } from "../harness/harness.js";
 
 /**
  * Test stored backup wallet-core API.
@@ -32,14 +32,16 @@ import { TalerMerchantApi } from "@gnu-taler/taler-util";
 export async function runStoredBackupsTest(t: GlobalTestState) {
   // Set up test environment
 
-  const { walletClient, bank, exchange, merchant } =
+  const { walletClient, bank, exchange, merchant, merchantAdminAccessToken } =
     await useSharedTestkudosEnvironment(t);
 
   // Withdraw digital cash into the wallet.
 
-  const wres = await withdrawViaBankV2(t, {
+  const bankClient = new TalerCorebankApiClient(bank.baseUrl);
+
+  const wres = await withdrawViaBankV3(t, {
     walletClient,
-    bank,
+    bankClient,
     exchange,
     amount: "TESTKUDOS:20",
   });
@@ -65,7 +67,7 @@ export async function runStoredBackupsTest(t: GlobalTestState) {
     fulfillment_url: "taler://fulfillment-success/thx",
   } satisfies TalerMerchantApi.Order;
 
-  await makeTestPaymentV2(t, { walletClient, merchant, order });
+  await makeTestPaymentV2(t, { walletClient, merchant, order, merchantAdminAccessToken });
   await walletClient.call(WalletApiOperation.TestingWaitTransactionsFinal, {});
 
   const txn1 = await walletClient.call(WalletApiOperation.GetTransactions, {});
