@@ -37,8 +37,8 @@ import {
   topupReserveWithBank,
   withdrawCoin,
 } from "@gnu-taler/taler-wallet-core/dbless";
+import { createSimpleTestkudosEnvironmentV2 } from "../harness/environments.js";
 import { GlobalTestState, harnessHttpLib } from "../harness/harness.js";
-import { createSimpleTestkudosEnvironmentV2 } from "../harness/helpers.js";
 
 /**
  * Run test for basic, bank-integrated withdrawal and payment.
@@ -54,6 +54,10 @@ export async function runWalletDblessTest(t: GlobalTestState) {
   );
   const cryptoApi = cryptiDisp.cryptoApi;
 
+  const merchantPair = await cryptoApi.createEddsaKeypair({});
+  const merchantPub = merchantPair.pub;
+  const merchantPriv = merchantPair.priv;
+
   try {
     // Withdraw digital cash into the wallet.
 
@@ -85,11 +89,8 @@ export async function runWalletDblessTest(t: GlobalTestState) {
     console.log(exchangeInfo);
 
     await checkReserve(http, exchange.baseUrl, reserveKeyPair.pub);
-    const denomselAllowLate = false;
 
-    const d1 = findDenomOrThrow(exchangeInfo, "TESTKUDOS:8" as AmountString, {
-      denomselAllowLate,
-    });
+    const d1 = findDenomOrThrow(exchangeInfo, "TESTKUDOS:8" as AmountString);
 
     const coin = await withdrawCoin({
       http,
@@ -103,12 +104,11 @@ export async function runWalletDblessTest(t: GlobalTestState) {
     });
 
     const wireSalt = encodeCrock(getRandomBytes(16));
-    const merchantPub = encodeCrock(getRandomBytes(32));
     const contractTermsHash = encodeCrock(getRandomBytes(64));
 
     await depositCoin({
       contractTermsHash,
-      merchantPub,
+      merchantPriv,
       wireSalt,
       amount: "TESTKUDOS:4" as AmountString,
       coin: coin,
@@ -120,7 +120,7 @@ export async function runWalletDblessTest(t: GlobalTestState) {
     // Idempotency
     await depositCoin({
       contractTermsHash,
-      merchantPub,
+      merchantPriv,
       wireSalt,
       amount: "TESTKUDOS:4" as AmountString,
       coin: coin,
@@ -130,12 +130,8 @@ export async function runWalletDblessTest(t: GlobalTestState) {
     });
 
     const refreshDenoms = [
-      findDenomOrThrow(exchangeInfo, "TESTKUDOS:1" as AmountString, {
-        denomselAllowLate,
-      }),
-      findDenomOrThrow(exchangeInfo, "TESTKUDOS:1" as AmountString, {
-        denomselAllowLate,
-      }),
+      findDenomOrThrow(exchangeInfo, "TESTKUDOS:1" as AmountString),
+      findDenomOrThrow(exchangeInfo, "TESTKUDOS:1" as AmountString),
     ];
 
     await refreshCoin({
