@@ -17,19 +17,24 @@
 /**
  * Imports.
  */
-import { NotificationType, TalerCorebankApiClient, WalletNotification } from "@gnu-taler/taler-util";
+import {
+  NotificationType,
+  TalerCorebankApiClient,
+  TalerWireGatewayAuth,
+  WalletNotification,
+} from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { CoinConfig, defaultCoinConfig } from "../harness/denomStructures.js";
+import { withdrawViaBankV3 } from "../harness/environments.js";
 import {
   BankService,
   ExchangeService,
   GlobalTestState,
   WalletClient,
   WalletService,
-  generateRandomPayto,
+  getTestHarnessPaytoForLabel,
   setupDb,
 } from "../harness/harness.js";
-import { withdrawViaBankV3 } from "../harness/helpers.js";
 
 export async function runWalletObservabilityTest(t: GlobalTestState) {
   // Set up test environment
@@ -52,13 +57,20 @@ export async function runWalletObservabilityTest(t: GlobalTestState) {
 
   let receiverName = "Exchange";
   let exchangeBankUsername = "exchange";
-  let exchangeBankPassword = "mypw";
-  let exchangePaytoUri = generateRandomPayto(exchangeBankUsername);
+  let exchangeBankPassword = "mypw-password";
+  let exchangePaytoUri = getTestHarnessPaytoForLabel(exchangeBankUsername);
+
+  const wireGatewayAuth: TalerWireGatewayAuth = {
+    username: exchangeBankUsername,
+    password: exchangeBankPassword,
+  };
 
   await exchange.addBankAccount("1", {
-    accountName: exchangeBankUsername,
-    accountPassword: exchangeBankPassword,
-    wireGatewayApiBaseUrl: new URL("accounts/exchange/taler-wire-gateway/", bank.baseUrl).href,
+    wireGatewayAuth,
+    wireGatewayApiBaseUrl: new URL(
+      "accounts/exchange/taler-wire-gateway/",
+      bank.baseUrl,
+    ).href,
     accountPaytoUri: exchangePaytoUri,
   });
 
@@ -71,7 +83,7 @@ export async function runWalletObservabilityTest(t: GlobalTestState) {
   const bankClient = new TalerCorebankApiClient(bank.corebankApiBaseUrl, {
     auth: {
       username: "admin",
-      password: "adminpw",
+      password: "admin-password",
     },
   });
 
