@@ -14,32 +14,51 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
-import { CancellationToken, Logger, minimatch } from "@gnu-taler/taler-util";
+import {
+  CancellationToken,
+  Logger,
+  minimatch,
+  setGlobalLogLevelFromString,
+} from "@gnu-taler/taler-util";
 import * as child_process from "child_process";
 import { spawnSync } from "child_process";
-import * as fs from "fs";
-import * as os from "os";
-import * as path from "path";
-import url from "url";
+import * as fs from "node:fs";
+import * as os from "node:os";
+import * as path from "node:path";
+import url from "node:url";
+import { getSharedTestDir } from "../harness/environments.js";
 import {
   GlobalTestState,
-  TestRunResult,
   runTestWithState,
   shouldLingerInTest,
+  TestRunResult,
 } from "../harness/harness.js";
-import { getSharedTestDir } from "../harness/helpers.js";
+import { runAccountRestrictionsTest } from "./test-account-restrictions.js";
 import { runAgeRestrictionsDepositTest } from "./test-age-restrictions-deposit.js";
 import { runAgeRestrictionsMerchantTest } from "./test-age-restrictions-merchant.js";
 import { runAgeRestrictionsMixedMerchantTest } from "./test-age-restrictions-mixed-merchant.js";
 import { runAgeRestrictionsPeerTest } from "./test-age-restrictions-peer.js";
 import { runBankApiTest } from "./test-bank-api.js";
+import { runBankWopTest } from "./test-bank-wop.js";
 import { runClaimLoopTest } from "./test-claim-loop.js";
 import { runClauseSchnorrTest } from "./test-clause-schnorr.js";
 import { runCurrencyScopeTest } from "./test-currency-scope.js";
+import { runDenomLostComplexTest } from "./test-denom-lost-complex.js";
 import { runDenomLostTest } from "./test-denom-lost.js";
 import { runDenomUnofferedTest } from "./test-denom-unoffered.js";
+import { runDepositFaultTest } from "./test-deposit-fault.js";
+import { runDepositLargeTest } from "./test-deposit-large.js";
+import { runDepositMergeTest } from "./test-deposit-merge.js";
+import { runDepositTooLargeTest } from "./test-deposit-too-large.js";
 import { runDepositTest } from "./test-deposit.js";
+import { runDonauCharityManagementTest } from "./test-donau-charity-management.js";
+import { runDonauCompatTest } from "./test-donau-compat.js";
+import { runDonauIdempotencyTest } from "./test-donau-idempotency.js";
+import { runDonauMinusTTest } from "./test-donau-minus-t.js";
+import { runDonauMultiTest } from "./test-donau-multi.js";
+import { runDonauTest } from "./test-donau.js";
 import { runExchangeDepositTest } from "./test-exchange-deposit.js";
+import { runExchangeKycAuthTest } from "./test-exchange-kyc-auth.js";
 import { runExchangeManagementFaultTest } from "./test-exchange-management-fault.js";
 import { runExchangeManagementTest } from "./test-exchange-management.js";
 import { runExchangeMasterPubChangeTest } from "./test-exchange-master-pub-change.js";
@@ -47,16 +66,42 @@ import { runExchangePurseTest } from "./test-exchange-purse.js";
 import { runExchangeTimetravelTest } from "./test-exchange-timetravel.js";
 import { runFeeRegressionTest } from "./test-fee-regression.js";
 import { runForcedSelectionTest } from "./test-forced-selection.js";
+import { runKnownAccountsTest } from "./test-known-accounts.js";
+import { runKycAmpFailureTest } from "./test-kyc-amp-failure.js";
+import { runKycAmpTimeoutTest } from "./test-kyc-amp-timeout.js";
+import { runKycBalanceWithdrawalChangeManualTest } from "./test-kyc-balance-withdrawal-change-manual.js";
 import { runKycBalanceWithdrawalTest } from "./test-kyc-balance-withdrawal.js";
+import { runKycChallengerTest } from "./test-kyc-challenger.js";
+import { runKycDecisionAttrTest } from "./test-kyc-decision-attr.js";
+import { runKycDecisionEventsTest } from "./test-kyc-decision-events.js";
+import { runKycDecisionsTest } from "./test-kyc-decisions.js";
+import { runKycDepositAggregateImplicitAuthTest } from "./test-kyc-deposit-aggregate-implicit-auth.js";
 import { runKycDepositAggregateTest } from "./test-kyc-deposit-aggregate.js";
+import { runKycDepositDepositKyctransferTest } from "./test-kyc-deposit-deposit-kyctransfer.js";
+import { runKycDepositDepositTest } from "./test-kyc-deposit-deposit.js";
+import { runKycDepositKycauthTest } from "./test-kyc-deposit-kycauth.js";
 import { runKycExchangeWalletTest } from "./test-kyc-exchange-wallet.js";
+import { runKycFailRecoverSimpleTest } from "./test-kyc-fail-recover-simple.js";
+import { runKycFormBadMeasureTest } from "./test-kyc-form-bad-measure.js";
+import { runKycFormCompressionTest } from "./test-kyc-form-compression.js";
 import { runKycFormWithdrawalTest } from "./test-kyc-form-withdrawal.js";
+import { runKycMerchantActivateBankAccountTest } from "./test-kyc-merchant-activate-bank-account.js";
+import { runKycMerchantAggregateTest } from "./test-kyc-merchant-aggregate.js";
+import { runKycMerchantDepositFormTest } from "./test-kyc-merchant-deposit-form.js";
+import { runKycMerchantDepositRewriteTest } from "./test-kyc-merchant-deposit-rewrite.js";
+import { runKycMerchantDepositTest } from "./test-kyc-merchant-deposit.js";
 import { runKycNewMeasureTest } from "./test-kyc-new-measure.js";
+import { runKycNewMeasuresProgTest } from "./test-kyc-new-measures-prog.js";
 import { runKycPeerPullTest } from "./test-kyc-peer-pull.js";
 import { runKycPeerPushTest } from "./test-kyc-peer-push.js";
+import { runKycSkipExpirationTest } from "./test-kyc-skip-expiration.js";
 import { runKycThresholdWithdrawalTest } from "./test-kyc-threshold-withdrawal.js";
-import { runKycTest } from "./test-kyc.js";
+import { runKycTwoFormsTest } from "./test-kyc-two-forms.js";
+import { runKycWalletDepositAbortTest } from "./test-kyc-wallet-deposit-abort.js";
+import { runKycWithdrawalVerbotenTest } from "./test-kyc-withdrawal-verboten.js";
 import { runLibeufinBankTest } from "./test-libeufin-bank.js";
+import { runLibeufinConversionTest } from "./test-libeufin-conversion.js";
+import { runMerchantAcctselTest } from "./test-merchant-acctsel.js";
 import { runMerchantCategoriesTest } from "./test-merchant-categories.js";
 import { runMerchantExchangeConfusionTest } from "./test-merchant-exchange-confusion.js";
 import { runMerchantInstancesDeleteTest } from "./test-merchant-instances-delete.js";
@@ -64,6 +109,10 @@ import { runMerchantInstancesUrlsTest } from "./test-merchant-instances-urls.js"
 import { runMerchantInstancesTest } from "./test-merchant-instances.js";
 import { runMerchantLongpollingTest } from "./test-merchant-longpolling.js";
 import { runMerchantRefundApiTest } from "./test-merchant-refund-api.js";
+import { runMerchantSelfProvisionActivationAndLoginTest } from "./test-merchant-self-provision-activation-and-login.js";
+import { runMerchantSelfProvisionActivationTest } from "./test-merchant-self-provision-activation.js";
+import { runMerchantSelfProvisionForgotPasswordTest } from "./test-merchant-self-provision-forgot-password.js";
+import { runMerchantSelfProvisionInactiveAccountPermissionsTest } from "./test-merchant-self-provision-inactive-account-permissions.js";
 import { runMerchantSpecPublicOrdersTest } from "./test-merchant-spec-public-orders.js";
 import { runMultiExchangeTest } from "./test-multiexchange.js";
 import { runOtpTest } from "./test-otp.js";
@@ -76,6 +125,7 @@ import { runPaymentFaultTest } from "./test-payment-fault.js";
 import { runPaymentForgettableTest } from "./test-payment-forgettable.js";
 import { runPaymentIdempotencyTest } from "./test-payment-idempotency.js";
 import { runPaymentMultipleTest } from "./test-payment-multiple.js";
+import { runPaymentShareIdempotencyTest } from "./test-payment-share-idempotency.js";
 import { runPaymentShareTest } from "./test-payment-share.js";
 import { runPaymentTemplateTest } from "./test-payment-template.js";
 import { runPaymentTransientTest } from "./test-payment-transient.js";
@@ -83,20 +133,30 @@ import { runPaymentZeroTest } from "./test-payment-zero.js";
 import { runPaymentTest } from "./test-payment.js";
 import { runPaywallFlowTest } from "./test-paywall-flow.js";
 import { runPeerPullLargeTest } from "./test-peer-pull-large.js";
+import { runPeerPullTest } from "./test-peer-pull.js";
 import { runPeerPushLargeTest } from "./test-peer-push-large.js";
+import { runPeerPushTest } from "./test-peer-push.js";
 import { runPeerRepairTest } from "./test-peer-repair.js";
-import { runPeerToPeerPullTest } from "./test-peer-to-peer-pull.js";
-import { runPeerToPeerPushTest } from "./test-peer-to-peer-push.js";
 import { runRefundAutoTest } from "./test-refund-auto.js";
 import { runRefundGoneTest } from "./test-refund-gone.js";
 import { runRefundIncrementalTest } from "./test-refund-incremental.js";
 import { runRefundTest } from "./test-refund.js";
+import { runRepurchaseTest } from "./test-repurchase.js";
 import { runRevocationTest } from "./test-revocation.js";
 import { runSimplePaymentTest } from "./test-simple-payment.js";
 import { runStoredBackupsTest } from "./test-stored-backups.js";
 import { runTimetravelAutorefreshTest } from "./test-timetravel-autorefresh.js";
 import { runTimetravelWithdrawTest } from "./test-timetravel-withdraw.js";
+import { runTopsAmlBasicTest } from "./test-tops-aml-basic.js";
+import { runTopsAmlCustomAddrPostalTest } from "./test-tops-aml-custom-addr-postal.js";
+import { runTopsAmlCustomAddrSmsTest } from "./test-tops-aml-custom-addr-sms.js";
+import { runTopsAmlKyxNaturalTest } from "./test-tops-aml-kyx-natural.js";
+import { runTopsAmlLegiTest } from "./test-tops-aml-legi.js";
+import { runTopsAmlMeasuresTest } from "./test-tops-aml-measures.js";
+import { runTopsChallengerTwiceTest } from "./test-tops-challenger-twice.js";
+import { runTopsPeerTest } from "./test-tops-peer.js";
 import { runTermOfServiceFormatTest } from "./test-tos-format.js";
+import { runUtilMerchantClientTest } from "./test-util-merchant-client.js";
 import { runWalletBackupBasicTest } from "./test-wallet-backup-basic.js";
 import { runWalletBackupDoublespendTest } from "./test-wallet-backup-doublespend.js";
 import { runWalletBalanceNotificationsTest } from "./test-wallet-balance-notifications.js";
@@ -113,6 +173,9 @@ import { runWalletDblessTest } from "./test-wallet-dbless.js";
 import { runWalletDd48Test } from "./test-wallet-dd48.js";
 import { runWalletDenomExpireTest } from "./test-wallet-denom-expire.js";
 import { runWalletDevExperimentsTest } from "./test-wallet-dev-experiments.js";
+import { runWalletDevexpFakeprotoverTest } from "./test-wallet-devexp-fakeprotover.js";
+import { runWalletExchangeMigrationExistingTest } from "./test-wallet-exchange-migration-existing.js";
+import { runWalletExchangeMigrationTest } from "./test-wallet-exchange-migration.js";
 import { runWalletExchangeUpdateTest } from "./test-wallet-exchange-update.js";
 import { runWalletGenDbTest } from "./test-wallet-gendb.js";
 import { runWalletInsufficientBalanceTest } from "./test-wallet-insufficient-balance.js";
@@ -121,11 +184,16 @@ import { runWalletNotificationsTest } from "./test-wallet-notifications.js";
 import { runWalletObservabilityTest } from "./test-wallet-observability.js";
 import { runWalletRefreshErrorsTest } from "./test-wallet-refresh-errors.js";
 import { runWalletRefreshTest } from "./test-wallet-refresh.js";
+import { runWalletTokensDiscountTest } from "./test-wallet-tokens-discount.js";
+import { runWalletTokensTest } from "./test-wallet-tokens.js";
+import { runWalletTransactionsTest } from "./test-wallet-transactions.js";
 import { runWalletWirefeesTest } from "./test-wallet-wirefees.js";
 import { runWallettestingTest } from "./test-wallettesting.js";
 import { runWithdrawalAbortBankTest } from "./test-withdrawal-abort-bank.js";
 import { runWithdrawalAmountTest } from "./test-withdrawal-amount.js";
 import { runWithdrawalBankIntegratedTest } from "./test-withdrawal-bank-integrated.js";
+import { runWithdrawalCashacceptorTest } from "./test-withdrawal-cashacceptor.js";
+import { runWithdrawalConflictTest } from "./test-withdrawal-conflict.js";
 import { runWithdrawalConversionTest } from "./test-withdrawal-conversion.js";
 import { runWithdrawalExternalTest } from "./test-withdrawal-external.js";
 import { runWithdrawalFakebankTest } from "./test-withdrawal-fakebank.js";
@@ -135,6 +203,7 @@ import { runWithdrawalHandoverTest } from "./test-withdrawal-handover.js";
 import { runWithdrawalHugeTest } from "./test-withdrawal-huge.js";
 import { runWithdrawalIdempotentTest } from "./test-withdrawal-idempotent.js";
 import { runWithdrawalManualTest } from "./test-withdrawal-manual.js";
+import { runWithdrawalPrepareTest } from "./test-withdrawal-prepare.js";
 
 /**
  * Test runner.
@@ -152,21 +221,18 @@ interface TestMainFunction {
 }
 
 const allTests: TestMainFunction[] = [
-  runAgeRestrictionsMerchantTest,
-  runAgeRestrictionsMixedMerchantTest,
-  runAgeRestrictionsPeerTest,
-  runAgeRestrictionsDepositTest,
   runBankApiTest,
   runClaimLoopTest,
   runClauseSchnorrTest,
   runDenomUnofferedTest,
   runDepositTest,
+  runDepositMergeTest,
   runSimplePaymentTest,
   runExchangeManagementFaultTest,
   runExchangeTimetravelTest,
   runFeeRegressionTest,
   runForcedSelectionTest,
-  runKycTest,
+  runKycChallengerTest,
   runExchangePurseTest,
   runExchangeDepositTest,
   runMerchantExchangeConfusionTest,
@@ -183,6 +249,7 @@ const allTests: TestMainFunction[] = [
   runPaymentMultipleTest,
   runPaymentTest,
   runPaymentShareTest,
+  runPaymentShareIdempotencyTest,
   runPaymentTemplateTest,
   runPaymentAbortTest,
   runPaymentTransientTest,
@@ -192,8 +259,8 @@ const allTests: TestMainFunction[] = [
   runMultiExchangeTest,
   runWalletBalanceTest,
   runPaywallFlowTest,
-  runPeerToPeerPullTest,
-  runPeerToPeerPushTest,
+  runPeerPullTest,
+  runPeerPushTest,
   runRefundAutoTest,
   runRefundGoneTest,
   runRefundIncrementalTest,
@@ -233,6 +300,8 @@ const allTests: TestMainFunction[] = [
   runWalletDevExperimentsTest,
   runWalletBalanceZeroTest,
   runWalletInsufficientBalanceTest,
+  runWalletTokensTest,
+  runWalletTokensDiscountTest,
   runWalletWirefeesTest,
   runDenomLostTest,
   runWalletDenomExpireTest,
@@ -250,6 +319,10 @@ const allTests: TestMainFunction[] = [
   runWithdrawalFlexTest,
   runExchangeMasterPubChangeTest,
   runMerchantCategoriesTest,
+  runMerchantSelfProvisionActivationTest,
+  runMerchantSelfProvisionActivationAndLoginTest,
+  runMerchantSelfProvisionForgotPasswordTest,
+  runMerchantSelfProvisionInactiveAccountPermissionsTest,
   runWithdrawalExternalTest,
   runWithdrawalIdempotentTest,
   runKycThresholdWithdrawalTest,
@@ -260,11 +333,72 @@ const allTests: TestMainFunction[] = [
   runKycFormWithdrawalTest,
   runKycBalanceWithdrawalTest,
   runKycNewMeasureTest,
+  runKycSkipExpirationTest,
+  runKycTwoFormsTest,
+  runKycFailRecoverSimpleTest,
+  runKycDepositDepositTest,
+  runKycDepositKycauthTest,
+  runKycMerchantDepositTest,
+  runKycMerchantDepositRewriteTest,
+  runKycMerchantAggregateTest,
+  runKycDepositDepositKyctransferTest,
+  runWithdrawalPrepareTest,
+  runAccountRestrictionsTest,
+  runRepurchaseTest,
+  runWalletTransactionsTest,
+  runKycWithdrawalVerbotenTest,
+  runKycMerchantActivateBankAccountTest,
+  runKycDecisionsTest,
+  runKnownAccountsTest,
+  runKycDepositAggregateImplicitAuthTest,
+  runKycAmpTimeoutTest,
+  runKycAmpFailureTest,
+  runWithdrawalCashacceptorTest,
+  runWithdrawalConflictTest,
+  runBankWopTest,
+  runKycNewMeasuresProgTest,
+  runKycDecisionAttrTest,
+  runDepositFaultTest,
+  runAgeRestrictionsMerchantTest,
+  runAgeRestrictionsMixedMerchantTest,
+  runAgeRestrictionsPeerTest,
+  runAgeRestrictionsDepositTest,
+  runKycDecisionEventsTest,
+  runWalletDevexpFakeprotoverTest,
+  runTopsAmlBasicTest,
+  runTopsAmlCustomAddrPostalTest,
+  runTopsAmlCustomAddrSmsTest,
+  runTopsAmlKyxNaturalTest,
+  runTopsAmlMeasuresTest,
+  runTopsAmlLegiTest,
+  runTopsChallengerTwiceTest,
+  runKycFormBadMeasureTest,
+  runKycBalanceWithdrawalChangeManualTest,
+  runUtilMerchantClientTest,
+  runKycWalletDepositAbortTest,
+  runKycMerchantDepositFormTest,
+  runExchangeKycAuthTest,
+  runTopsPeerTest,
+  runWalletExchangeMigrationTest,
+  runWalletExchangeMigrationExistingTest,
+  runKycFormCompressionTest,
+  runDepositLargeTest,
+  runDepositTooLargeTest,
+  runMerchantAcctselTest,
+  runLibeufinConversionTest,
+  runDenomLostComplexTest,
+  runDonauCompatTest,
+  runDonauTest,
+  runDonauMinusTTest,
+  runDonauCharityManagementTest,
+  runDonauMultiTest,
+  runDonauIdempotencyTest,
 ];
 
 export interface TestRunSpec {
   includePattern?: string;
   suiteSpec?: string;
+  testDir?: string;
   dryRun?: boolean;
   failFast?: boolean;
   waitOnFail?: boolean;
@@ -282,7 +416,7 @@ export interface TestInfo {
 function updateCurrentSymlink(testDir: string): void {
   const currLink = path.join(
     os.tmpdir(),
-    `taler-integrationtests-${os.userInfo().username}-current`,
+    `taler-integrationtests-latest-${os.userInfo().username}`,
   );
   try {
     fs.unlinkSync(currLink);
@@ -344,10 +478,20 @@ export async function runTests(spec: TestRunSpec) {
   } else {
     logger.info("keeping shared test environment");
   }
-
-  const testRootDir = fs.mkdtempSync(
-    path.join(os.tmpdir(), "taler-integrationtests-"),
-  );
+  let testRootDir: string;
+  if (spec.testDir != null) {
+    testRootDir = spec.testDir;
+    try {
+      fs.rmSync(testRootDir, { recursive: true, force: true });
+    } catch (e) {
+      // Ignore
+    }
+    fs.mkdirSync(testRootDir);
+  } else {
+    testRootDir = fs.mkdtempSync(
+      path.join(os.tmpdir(), "taler-integrationtests-"),
+    );
+  }
   updateCurrentSymlink(testRootDir);
   console.log(`testsuite root directory: ${testRootDir}`);
 
@@ -374,10 +518,23 @@ export async function runTests(spec: TestRunSpec) {
     suites = new Set(spec.suiteSpec.split(",").map((x) => x.trim()));
   }
 
+  const filteredTests: TestMainFunction[] = [];
+
+  const patterns = spec.includePattern?.split(",");
+
   for (const [n, testCase] of allTests.entries()) {
     const testName = getTestName(testCase);
-    if (spec.includePattern && !minimatch(testName, spec.includePattern)) {
-      continue;
+    if (patterns) {
+      let matched = false;
+      for (const pat of patterns) {
+        if (minimatch(testName, pat)) {
+          matched = true;
+          break;
+        }
+      }
+      if (!matched) {
+        continue;
+      }
     }
 
     if (testCase.experimental && !spec.includeExperimental) {
@@ -391,7 +548,15 @@ export async function runTests(spec: TestRunSpec) {
         continue;
       }
     }
+    filteredTests.push(testCase);
+  }
 
+  console.log(`selected ${filteredTests.length} tests`);
+
+  let numFailed = 0;
+
+  for (const [n, testCase] of filteredTests.entries()) {
+    const testName = getTestName(testCase);
     if (spec.dryRun) {
       console.log(`dry run: would run test ${testName}`);
       continue;
@@ -407,6 +572,7 @@ export async function runTests(spec: TestRunSpec) {
     currentChild = child_process.fork(myFilename, ["__TWCLI_TESTWORKER"], {
       env: {
         TWCLI_RUN_TEST_INSTRUCTION: JSON.stringify(testInstr),
+        TWCLI_LOGLEVEL: logger.getGlobalLogLevel(),
         ...process.env,
       },
       stdio: ["pipe", "pipe", "pipe", "ipc"],
@@ -438,10 +604,17 @@ export async function runTests(spec: TestRunSpec) {
       ? Math.max(testCase.timeoutMs, defaultTimeout)
       : defaultTimeout;
 
+    let progressText = `${n + 1} of ${filteredTests.length}`;
+    if (numFailed > 0) {
+      progressText = progressText + `, failed ${numFailed}`;
+    }
+
     if (spec.noTimeout) {
-      console.log(`running ${testName}, no timeout`);
+      console.log(`running ${testName} (${progressText}), no timeout`);
     } else {
-      console.log(`running ${testName} with timeout ${testTimeoutMs}ms`);
+      console.log(
+        `running ${testName} (${progressText}) with timeout ${testTimeoutMs}ms`,
+      );
     }
 
     const token = spec.noTimeout
@@ -508,8 +681,22 @@ export async function runTests(spec: TestRunSpec) {
       }
     }
 
+    if (result.status === "fail") {
+      numFailed++;
+    }
+
     harnessLogStream.close();
 
+    const stepsFile = `${testDir}/steps.txt`;
+    if (spec.verbosity > 0 && fs.existsSync(stepsFile)) {
+      let stepsLog = fs.readFileSync(stepsFile, {
+        encoding: "utf-8",
+      });
+      console.log("-- test steps --");
+      console.log(stepsLog.trim());
+      console.log("-- end --");
+    }
+
     console.log(`parent: got result ${JSON.stringify(result)}`);
 
     testResults.push(result);
@@ -548,6 +735,7 @@ export function reportAndQuit(
     console.log("test suite was interrupted");
   }
   console.log(`See ${resultsFile} for details`);
+  console.log(`Logs can be found in ${testRootDir}`);
   console.log(`Skipped: ${numSkip}/${numTotal}`);
   console.log(`Failed: ${numFail}/${numTotal}`);
   console.log(`Passed: ${numPass}/${numTotal}`);
@@ -571,8 +759,10 @@ export function getTestInfo(): TestInfo[] {
 
 const runTestInstrStr = process.env["TWCLI_RUN_TEST_INSTRUCTION"];
 if (runTestInstrStr && process.argv.includes("__TWCLI_TESTWORKER")) {
+  setGlobalLogLevelFromString(process.env["TWCLI_LOGLEVEL"] ?? "INFO");
   // Test will call taler-wallet-cli, so we must not propagate this variable.
   delete process.env["TWCLI_RUN_TEST_INSTRUCTION"];
+  delete process.env["TWCLI_LOGLEVEL"];
   const { testRootDir, testName } = JSON.parse(
     runTestInstrStr,
   ) as RunTestChildInstruction;
