@@ -23,8 +23,9 @@
  * Imports.
  */
 import { AmountJson, Amounts } from "./amounts.js";
-import { decodeCrock } from "./taler-crypto.js";
-import * as segwit from "./segwit_addr.js";
+import { opFixedSuccess, opKnownFailure } from "./operation.js";
+import { BtAddrString } from "./payto.js";
+import { BitcoinSewgit } from "./segwit_addr.js";
 
 function buf2hex(buffer: Uint8Array) {
   // buffer is an ArrayBuffer
@@ -36,19 +37,25 @@ function buf2hex(buffer: Uint8Array) {
 const hext2buf = (hexString: string) =>
   new Uint8Array(hexString.match(/.{1,2}/g)!.map((byte) => parseInt(byte, 16)));
 
-export function generateFakeSegwitAddress(
-  reservePub: string | undefined,
-  addr: string,
-): string[] {
-  if (!reservePub) return [];
-  let pub;
-  try {
-    pub = decodeCrock(reservePub);
-  } catch {
-    // pub = new Uint8Array(0)
-  }
-  if (!pub || pub.length !== 32) return [];
+export enum GenerateSegwitAddrError {
+  /**
+   * The reserve pub used to generate the segwith is invalid
+   */
+  WRONG_RESERVE_PUB = "wrong-reserve-pub",
+  /**
+   * The net prefix of the addr is unsupported
+   */
+  WRONG_PREFIX = "wrong-prefix",
+  /**
+   * The process of generating a segwit failed
+   */
+  INVALID_SEGWIT = "invalid-segwit",
+}
 
+export function generateFakeSegwitAddress(
+  pub: Uint8Array,
+  addr: string,
+){
   const first_rnd = new Uint8Array(4);
   first_rnd.set(pub.subarray(0, 4));
   const second_rnd = new Uint8Array(4);
@@ -73,12 +80,20 @@ export function generateFakeSegwitAddress(
         : addr[0] === "b" && addr[1] == "c"
           ? "bc"
           : undefined;
-  if (prefix === undefined) throw new Error("unknown bitcoin net");
+  if (prefix === undefined) {
+    return opKnownFailure(GenerateSegwitAddrError.WRONG_PREFIX);
+  }
 
-  const addr1 = segwit.default.encode(prefix, 0, first_part);
-  const addr2 = segwit.default.encode(prefix, 0, second_part);
-
-  return [addr1, addr2];
+  const addr1 = BitcoinSewgit.encode(prefix, 0, Array.from(first_part));
+  if (addr1.type === "fail") {
+    return opKnownFailure(GenerateSegwitAddrError.INVALID_SEGWIT);
+  }
+  const addr2 = BitcoinSewgit.encode(prefix, 0, Array.from(second_part));
+  if (addr2.type === "fail") {
+    return opKnownFailure(GenerateSegwitAddrError.INVALID_SEGWIT);
+  }
+  const result: [BtAddrString, BtAddrString] = [addr1.body, addr2.body];
+  return opFixedSuccess(result);
 }
 
 // https://github.com/bitcoin/bitcoin/blob/master/src/policy/policy.cpp
