@@ -29,12 +29,16 @@ import {
   PaymentInsufficientBalanceDetails,
   TalerErrorCode,
   TalerErrorDetail,
+  TransactionState,
   TransactionType,
 } from "@gnu-taler/taler-util";
 
 type empty = Record<string, never>;
 
 export interface DetailsMap {
+  [TalerErrorCode.WALLET_PAY_MERCHANT_KYC_MISSING]: {
+    exchangeResponse: any;
+  };
   [TalerErrorCode.WALLET_PENDING_OPERATION_FAILED]: {
     innerError: TalerErrorDetail;
     transactionId?: string;
@@ -54,7 +58,7 @@ export interface DetailsMap {
   };
   [TalerErrorCode.WALLET_ORDER_ALREADY_PAID]: {
     orderId: string;
-    fulfillmentUrl: string;
+    fulfillmentUrl: string | undefined;
   };
   [TalerErrorCode.WALLET_CONTRACT_TERMS_MALFORMED]: empty;
   [TalerErrorCode.WALLET_CONTRACT_TERMS_SIGNATURE_INVALID]: {
@@ -178,10 +182,18 @@ export interface DetailsMap {
   [TalerErrorCode.WALLET_EXCHANGE_ENTRY_UPDATE_CONFLICT]: {
     detail?: string;
   };
+  [TalerErrorCode.WALLET_REQUEST_TRANSACTION_STATE_UNSUPPORTED]: {
+    message?: string;
+    txState: TransactionState;
+    debugStateNum?: number;
+  };
 }
 
 type ErrBody<Y> = Y extends keyof DetailsMap ? DetailsMap[Y] : empty;
 
+export type TypedTalerErrorDetail<Y> = TalerErrorDetail &
+  (Y extends keyof DetailsMap ? DetailsMap[Y] : {});
+
 export function makeErrorDetail<C extends TalerErrorCode>(
   code: C,
   detail: ErrBody<C>,
@@ -351,6 +363,12 @@ export function getErrorDetailFromException(e: any): TalerErrorDetail {
   return err;
 }
 
+/**
+ * This function should not be called at runtime.
+ * Useful on switch/case to detect that all path has been contemplated.
+ * 
+ * @param x 
+ */
 export function assertUnreachable(x: never): never {
   throw new Error("Didn't expect to get here");
 }
