@@ -21,19 +21,8 @@
 /**
  * Imports.
  */
-import { AmountJson } from "./amounts.js";
-import * as Amounts from "./amounts.js";
 import { URL } from "./url.js";
 
-/**
- * Show an amount in a form suitable for the user.
- * FIXME:  In the future, this should consider currency-specific
- * settings such as significant digits or currency symbols.
- */
-export function amountToPretty(amount: AmountJson): string {
-  const x = amount.value + amount.fraction / Amounts.amountFractionalBase;
-  return `${x} ${amount.currency}`;
-}
 
 /**
  * Canonicalize a base url, typically for the exchange.
