@@ -18,17 +18,18 @@
  * Imports.
  */
 import { AmountJson, Amounts } from "../amounts.js";
-import { HttpRequestLibrary, readTalerErrorResponse } from "../http-common.js";
+import { HttpRequestLibrary } from "../http-common.js";
 import { HttpStatusCode } from "../http-status-codes.js";
 import { createPlatformHttpLib } from "../http.js";
 import { LibtoolVersion } from "../libtool-version.js";
 import {
   FailCasesByMethod,
   ResultByMethod,
+  carefullyParseConfig,
   opEmptySuccess,
   opKnownHttpFailure,
   opSuccessFromHttp,
-  opUnknownFailure,
+  opUnknownHttpFailure,
 } from "../operation.js";
 import { TalerErrorCode } from "../taler-error-codes.js";
 import {
@@ -36,6 +37,7 @@ import {
   codecForCashinConversionResponse,
   codecForCashoutConversionResponse,
   codecForConversionBankConfig,
+  codecForConversionRate,
 } from "../types-taler-bank-conversion.js";
 import { AccessToken } from "../types-taler-common.js";
 import { codecForTalerErrorDetail } from "../types-taler-wallet.js";
@@ -60,7 +62,7 @@ export enum TalerBankConversionCacheEviction {
  * The API is used by the wallets.
  */
 export class TalerBankConversionHttpClient {
-  public readonly PROTOCOL_VERSION = "0:0:0";
+  public static readonly PROTOCOL_VERSION = "0:0:0";
 
   httpLib: HttpRequestLibrary;
   cacheEvictor: CacheEvictor<TalerBankConversionCacheEviction>;
@@ -74,7 +76,7 @@ export class TalerBankConversionHttpClient {
     this.cacheEvictor = cacheEvictor ?? nullEvictor;
   }
 
-  isCompatible(version: string): boolean {
+  static isCompatible(version: string): boolean {
     const compare = LibtoolVersion.compare(this.PROTOCOL_VERSION, version);
     return compare?.compatible ?? false;
   }
@@ -90,11 +92,40 @@ export class TalerBankConversionHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccessFromHttp(resp, codecForConversionBankConfig());
+        return carefullyParseConfig(
+          "taler-conversion-info",
+          TalerBankConversionHttpClient.PROTOCOL_VERSION,
+          resp,
+          codecForConversionBankConfig(),
+        );
       case HttpStatusCode.NotImplemented:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
+        return opUnknownHttpFailure(resp);
+    }
+  }
+
+  /**
+   * https://docs.taler.net/core/api-bank-conversion-info.html#get--rate
+   *
+   */
+  async getRate(auth: AccessToken | undefined) {
+    const url = new URL(`rate`, this.baseUrl);
+    const headers: Record<string, string> = {};
+    if (auth) {
+      headers.Authorization = makeBearerTokenAuthHeader(auth);
+    }
+    const resp = await this.httpLib.fetch(url.href, {
+      method: "GET",
+      headers,
+    });
+    switch (resp.status) {
+      case HttpStatusCode.Ok:
+        return opSuccessFromHttp(resp, codecForConversionRate());
+      case HttpStatusCode.NotImplemented:
+        return opKnownHttpFailure(resp.status, resp);
+      default:
+        return opUnknownHttpFailure(resp);
     }
   }
 
@@ -102,7 +133,10 @@ export class TalerBankConversionHttpClient {
    * https://docs.taler.net/core/api-bank-conversion-info.html#get--cashin-rate
    *
    */
-  async getCashinRate(conversion: { debit?: AmountJson; credit?: AmountJson }) {
+  async getCashinRate(
+    auth: AccessToken | undefined,
+    conversion: { debit?: AmountJson; credit?: AmountJson },
+  ) {
     const url = new URL(`cashin-rate`, this.baseUrl);
     if (conversion.debit) {
       url.searchParams.set("amount_debit", Amounts.stringify(conversion.debit));
@@ -113,8 +147,13 @@ export class TalerBankConversionHttpClient {
         Amounts.stringify(conversion.credit),
       );
     }
+    const headers: Record<string, string> = {};
+    if (auth) {
+      headers.Authorization = makeBearerTokenAuthHeader(auth);
+    }
     const resp = await this.httpLib.fetch(url.href, {
       method: "GET",
+      headers,
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
@@ -130,7 +169,7 @@ export class TalerBankConversionHttpClient {
           case TalerErrorCode.GENERIC_CURRENCY_MISMATCH:
             return opKnownHttpFailure(resp.status, resp);
           default:
-            return opUnknownFailure(resp, body);
+            return opUnknownHttpFailure(resp, details);
         }
       }
       case HttpStatusCode.Conflict:
@@ -138,7 +177,7 @@ export class TalerBankConversionHttpClient {
       case HttpStatusCode.NotImplemented:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
+        return opUnknownHttpFailure(resp);
     }
   }
 
@@ -146,10 +185,13 @@ export class TalerBankConversionHttpClient {
    * https://docs.taler.net/core/api-bank-conversion-info.html#get--cashout-rate
    *
    */
-  async getCashoutRate(conversion: {
-    debit?: AmountJson;
-    credit?: AmountJson;
-  }) {
+  async getCashoutRate(
+    auth: AccessToken | undefined,
+    conversion: {
+      debit?: AmountJson;
+      credit?: AmountJson;
+    },
+  ) {
     const url = new URL(`cashout-rate`, this.baseUrl);
     if (conversion.debit) {
       url.searchParams.set("amount_debit", Amounts.stringify(conversion.debit));
@@ -160,8 +202,13 @@ export class TalerBankConversionHttpClient {
         Amounts.stringify(conversion.credit),
       );
     }
+    const headers: Record<string, string> = {};
+    if (auth) {
+      headers.Authorization = makeBearerTokenAuthHeader(auth);
+    }
     const resp = await this.httpLib.fetch(url.href, {
       method: "GET",
+      headers,
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
@@ -177,7 +224,7 @@ export class TalerBankConversionHttpClient {
           case TalerErrorCode.GENERIC_CURRENCY_MISMATCH:
             return opKnownHttpFailure(resp.status, resp);
           default:
-            return opUnknownFailure(resp, body);
+            return opUnknownHttpFailure(resp, details);
         }
       }
       case HttpStatusCode.Conflict:
@@ -185,7 +232,7 @@ export class TalerBankConversionHttpClient {
       case HttpStatusCode.NotImplemented:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
+        return opUnknownHttpFailure(resp);
     }
   }
 
@@ -207,14 +254,14 @@ export class TalerBankConversionHttpClient {
         this.cacheEvictor.notifySuccess(
           TalerBankConversionCacheEviction.UPDATE_RATE,
         );
-        return opEmptySuccess(resp);
+        return opEmptySuccess();
       }
       case HttpStatusCode.Unauthorized:
         return opKnownHttpFailure(resp.status, resp);
       case HttpStatusCode.NotImplemented:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
+        return opUnknownHttpFailure(resp);
     }
   }
 }
