@@ -14,28 +14,30 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
-import {
-  HttpRequestLibrary,
-  makeBasicAuthHeader,
-  readTalerErrorResponse,
-} from "../http-common.js";
+import { HttpRequestLibrary } from "../http-common.js";
 import { HttpStatusCode } from "../http-status-codes.js";
 import { createPlatformHttpLib } from "../http.js";
 import { LibtoolVersion } from "../libtool-version.js";
 import {
+  carefullyParseConfig,
   FailCasesByMethod,
-  ResultByMethod,
   opFixedSuccess,
   opKnownHttpFailure,
   opSuccessFromHttp,
-  opUnknownFailure,
+  opUnknownHttpFailure,
+  ResultByMethod,
 } from "../operation.js";
 import { LongPollParams, PaginationParams } from "../types-taler-common.js";
 import {
   codecForRevenueConfig,
   codecForRevenueIncomingHistory,
 } from "../types-taler-revenue.js";
-import { addLongPollingParam, addPaginationParams } from "./utils.js";
+import {
+  addLongPollingParam,
+  addPaginationParams,
+  BasicOrTokenAuth,
+  createAuthorizationHeader,
+} from "./utils.js";
 
 export type TalerBankRevenueResultByMethod<
   prop extends keyof TalerRevenueHttpClient,
@@ -44,10 +46,6 @@ export type TalerBankRevenueErrorsByMethod<
   prop extends keyof TalerRevenueHttpClient,
 > = FailCasesByMethod<TalerRevenueHttpClient, prop>;
 
-type UsernameAndPassword = {
-  username: string;
-  password: string;
-};
 /**
  * The API is used by the merchant (or other parties) to query
  * for incoming transactions to their account.
@@ -62,9 +60,9 @@ export class TalerRevenueHttpClient {
     this.httpLib = httpClient ?? createPlatformHttpLib();
   }
 
-  public readonly PROTOCOL_VERSION = "0:0:0";
+  public static readonly PROTOCOL_VERSION = "1:0:0";
 
-  isCompatible(version: string): boolean {
+  static isCompatible(version: string): boolean {
     const compare = LibtoolVersion.compare(this.PROTOCOL_VERSION, version);
     return compare?.compatible ?? false;
   }
@@ -73,25 +71,28 @@ export class TalerRevenueHttpClient {
    * https://docs.taler.net/core/api-bank-revenue.html#get--config
    *
    */
-  async getConfig(auth?: UsernameAndPassword) {
+  async getConfig(auth?: BasicOrTokenAuth) {
     const url = new URL(`config`, this.baseUrl);
     const resp = await this.httpLib.fetch(url.href, {
       method: "GET",
       headers: {
-        Authorization: auth
-          ? makeBasicAuthHeader(auth.username, auth.password)
-          : undefined,
+        Authorization: createAuthorizationHeader(auth),
       },
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccessFromHttp(resp, codecForRevenueConfig());
+        return carefullyParseConfig(
+          "taler-revenue",
+          TalerRevenueHttpClient.PROTOCOL_VERSION,
+          resp,
+          codecForRevenueConfig(),
+        );
       case HttpStatusCode.Unauthorized:
         return opKnownHttpFailure(resp.status, resp);
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
+        return opUnknownHttpFailure(resp);
     }
   }
   /**
@@ -100,7 +101,7 @@ export class TalerRevenueHttpClient {
    * @returns
    */
   async getHistory(
-    auth?: UsernameAndPassword,
+    auth?: BasicOrTokenAuth,
     params?: PaginationParams & LongPollParams,
   ) {
     const url = new URL(`history`, this.baseUrl);
@@ -109,9 +110,7 @@ export class TalerRevenueHttpClient {
     const resp = await this.httpLib.fetch(url.href, {
       method: "GET",
       headers: {
-        Authorization: auth
-          ? makeBasicAuthHeader(auth.username, auth.password)
-          : undefined,
+        Authorization: createAuthorizationHeader(auth),
       },
     });
     switch (resp.status) {
@@ -130,7 +129,7 @@ export class TalerRevenueHttpClient {
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
+        return opUnknownHttpFailure(resp);
     }
   }
 }
