@@ -14,11 +14,7 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
-import {
-  HttpRequestLibrary,
-  makeBasicAuthHeader,
-  readTalerErrorResponse,
-} from "../http-common.js";
+import { HttpRequestLibrary, makeBasicAuthHeader } from "../http-common.js";
 import { HttpStatusCode } from "../http-status-codes.js";
 import { createPlatformHttpLib } from "../http.js";
 import {
@@ -27,10 +23,11 @@ import {
   opFixedSuccess,
   opKnownHttpFailure,
   opSuccessFromHttp,
-  opUnknownFailure,
+  opUnknownHttpFailure,
 } from "../operation.js";
 import {
   codecForAddIncomingResponse,
+  codecForBankWireTransferList,
   codecForIncomingHistory,
   codecForOutgoingHistory,
   codecForTransferResponse,
@@ -39,6 +36,7 @@ import { addLongPollingParam, addPaginationParams } from "./utils.js";
 
 import { LongPollParams, PaginationParams } from "../types-taler-common.js";
 import * as TalerWireGatewayApi from "../types-taler-wire-gateway.js";
+import { carefullyParseConfig, LibtoolVersion } from "../index.js";
 
 export type TalerWireGatewayResultByMethod<
   prop extends keyof TalerWireGatewayHttpClient,
@@ -47,6 +45,11 @@ export type TalerWireGatewayErrorsByMethod<
   prop extends keyof TalerWireGatewayHttpClient,
 > = FailCasesByMethod<TalerWireGatewayHttpClient, prop>;
 
+export interface TalerWireGatewayAuth {
+  username: string;
+  password: string;
+}
+
 /**
  * The API is used by the exchange to trigger transactions and query
  * incoming transactions, as well as by the auditor to query incoming
@@ -56,47 +59,64 @@ export type TalerWireGatewayErrorsByMethod<
  */
 export class TalerWireGatewayHttpClient {
   httpLib: HttpRequestLibrary;
+  public static readonly PROTOCOL_VERSION = "4:0:0";
 
   constructor(
     readonly baseUrl: string,
-    readonly username: string,
-    httpClient?: HttpRequestLibrary,
+    options: {
+      httpClient?: HttpRequestLibrary;
+    } = {},
   ) {
-    this.httpLib = httpClient ?? createPlatformHttpLib();
+    this.httpLib = options.httpClient ?? createPlatformHttpLib();
   }
-  // public readonly PROTOCOL_VERSION = "4:0:0";
-  // isCompatible(version: string): boolean {
-  //   const compare = LibtoolVersion.compare(this.PROTOCOL_VERSION, version)
-  //   return compare?.compatible ?? false
-  // }
 
-  // /**
-  //  * https://docs.taler.net/core/api-corebank.html#config
-  //  *
-  //  */
-  // async getConfig() {
-  //   const url = new URL(`config`, this.baseUrl);
-  //   const resp = await this.httpLib.fetch(url.href, {
-  //     method: "GET"
-  //   });
-  //   switch (resp.status) {
-  //     case HttpStatusCode.Ok: return opSuccess(resp, codecForCoreBankConfig())
-  //     default: return opUnknownFailure(resp, await readTalerErrorResponse(resp))
-  //   }
-  // }
+  static isCompatible(version: string): boolean {
+    const compare = LibtoolVersion.compare(this.PROTOCOL_VERSION, version);
+    return compare?.compatible ?? false;
+  }
+
+  /**
+   * https://docs.taler.net/core/api-bank-wire.html#get--config
+   *
+   */
+  async getConfig() {
+    const url = new URL(`config`, this.baseUrl);
+    const resp = await this.httpLib.fetch(url.href, {
+      method: "GET",
+    });
+    switch (resp.status) {
+      case HttpStatusCode.Ok:
+        return carefullyParseConfig(
+          "taler-wire-gateway",
+          TalerWireGatewayHttpClient.PROTOCOL_VERSION,
+          resp,
+          TalerWireGatewayApi.codecForWireConfigResponse(),
+        );
+      case HttpStatusCode.NotFound:
+        return opKnownHttpFailure(resp.status, resp);
+      default:
+        return opUnknownHttpFailure(resp);
+    }
+  }
 
   /**
    * https://docs.taler.net/core/api-bank-wire.html#post--transfer
    *
    */
-  async transfer(auth: string, body: TalerWireGatewayApi.TransferRequest) {
+  async makeWireTransfer(req: {
+    body: TalerWireGatewayApi.TransferRequest;
+    auth: { username: string; password: string };
+  }) {
     const url = new URL(`transfer`, this.baseUrl);
     const resp = await this.httpLib.fetch(url.href, {
       method: "POST",
       headers: {
-        Authorization: makeBasicAuthHeader(this.username, auth),
+        Authorization: makeBasicAuthHeader(
+          req.auth.username,
+          req.auth.password,
+        ),
       },
-      body,
+      body: req.body,
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
@@ -112,7 +132,86 @@ export class TalerWireGatewayHttpClient {
       case HttpStatusCode.Conflict:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
+        return opUnknownHttpFailure(resp);
+    }
+  }
+
+  /**
+   * https://docs.taler.net/core/api-bank-wire.html#get--transfers
+   *
+   */
+  async getTransfers(req: {
+    params?: {
+      status?: TalerWireGatewayApi.WireTransferStatus;
+    } & PaginationParams;
+    auth: TalerWireGatewayAuth;
+  }) {
+    const url = new URL(`transfers`, this.baseUrl);
+    if (req.params) {
+      if (req.params.status) {
+        url.searchParams.set("status", req.params.status);
+      }
+    }
+    addPaginationParams(url, req.params);
+    const resp = await this.httpLib.fetch(url.href, {
+      method: "GET",
+      headers: {
+        Authorization: makeBasicAuthHeader(
+          req.auth.username,
+          req.auth.password,
+        ),
+      },
+    });
+    switch (resp.status) {
+      case HttpStatusCode.Ok:
+        return opSuccessFromHttp(resp, codecForBankWireTransferList());
+      // FIXME: account should not be returned or make it optional
+      case HttpStatusCode.NoContent:
+        return opFixedSuccess({
+          transfers: [],
+          debit_account: undefined,
+        });
+      // FIXME: show more details in docs
+      case HttpStatusCode.BadRequest:
+        return opKnownHttpFailure(resp.status, resp);
+      case HttpStatusCode.Unauthorized:
+        return opKnownHttpFailure(resp.status, resp);
+      // FIXME: show more details in docs
+      case HttpStatusCode.NotFound:
+        return opKnownHttpFailure(resp.status, resp);
+      default:
+        return opUnknownHttpFailure(resp);
+    }
+  }
+
+  /**
+   * https://docs.taler.net/core/api-bank-wire.html#get--transfers-$ROW_ID
+   *
+   */
+  async getTransferStatus(req: { auth: TalerWireGatewayAuth; rowId?: number }) {
+    const url = new URL(`transfers/${req.rowId}`, this.baseUrl);
+    const resp = await this.httpLib.fetch(url.href, {
+      method: "GET",
+      headers: {
+        Authorization: makeBasicAuthHeader(
+          req.auth.username,
+          req.auth.password,
+        ),
+      },
+    });
+    switch (resp.status) {
+      case HttpStatusCode.Ok:
+        return opSuccessFromHttp(resp, codecForBankWireTransferList());
+      //FIXME: account should not be returned or make it optional
+      case HttpStatusCode.BadRequest:
+        return opKnownHttpFailure(resp.status, resp);
+      case HttpStatusCode.Unauthorized:
+        return opKnownHttpFailure(resp.status, resp);
+      //FIXME: show more details in docs
+      case HttpStatusCode.NotFound:
+        return opKnownHttpFailure(resp.status, resp);
+      default:
+        return opUnknownHttpFailure(resp);
     }
   }
 
@@ -120,17 +219,20 @@ export class TalerWireGatewayHttpClient {
    * https://docs.taler.net/core/api-bank-wire.html#get--history-incoming
    *
    */
-  async getHistoryIncoming(
-    auth: string,
-    params?: PaginationParams & LongPollParams,
-  ) {
+  async getHistoryIncoming(req: {
+    params?: PaginationParams & LongPollParams;
+    auth: TalerWireGatewayAuth;
+  }) {
     const url = new URL(`history/incoming`, this.baseUrl);
-    addPaginationParams(url, params);
-    addLongPollingParam(url, params);
+    addPaginationParams(url, req.params);
+    addLongPollingParam(url, req.params);
     const resp = await this.httpLib.fetch(url.href, {
       method: "GET",
       headers: {
-        Authorization: makeBasicAuthHeader(this.username, auth),
+        Authorization: makeBasicAuthHeader(
+          req.auth.username,
+          req.auth.password,
+        ),
       },
     });
     switch (resp.status) {
@@ -151,7 +253,7 @@ export class TalerWireGatewayHttpClient {
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
+        return opUnknownHttpFailure(resp);
     }
   }
 
@@ -159,17 +261,20 @@ export class TalerWireGatewayHttpClient {
    * https://docs.taler.net/core/api-bank-wire.html#get--history-outgoing
    *
    */
-  async getHistoryOutgoing(
-    auth: string,
-    params?: PaginationParams & LongPollParams,
-  ) {
+  async getHistoryOutgoing(req: {
+    auth: TalerWireGatewayAuth;
+    params?: PaginationParams & LongPollParams;
+  }) {
     const url = new URL(`history/outgoing`, this.baseUrl);
-    addPaginationParams(url, params);
-    addLongPollingParam(url, params);
+    addPaginationParams(url, req.params);
+    addLongPollingParam(url, req.params);
     const resp = await this.httpLib.fetch(url.href, {
       method: "GET",
       headers: {
-        Authorization: makeBasicAuthHeader(this.username, auth),
+        Authorization: makeBasicAuthHeader(
+          req.auth.username,
+          req.auth.password,
+        ),
       },
     });
     switch (resp.status) {
@@ -190,7 +295,7 @@ export class TalerWireGatewayHttpClient {
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
+        return opUnknownHttpFailure(resp);
     }
   }
 
@@ -198,17 +303,20 @@ export class TalerWireGatewayHttpClient {
    * https://docs.taler.net/core/api-bank-wire.html#post--admin-add-incoming
    *
    */
-  async addIncoming(
-    auth: string,
-    body: TalerWireGatewayApi.AddIncomingRequest,
-  ) {
+  async addIncoming(req: {
+    body: TalerWireGatewayApi.AddIncomingRequest;
+    auth: { username: string; password: string };
+  }) {
     const url = new URL(`admin/add-incoming`, this.baseUrl);
     const resp = await this.httpLib.fetch(url.href, {
       method: "POST",
       headers: {
-        Authorization: makeBasicAuthHeader(this.username, auth),
+        Authorization: makeBasicAuthHeader(
+          req.auth.username,
+          req.auth.password,
+        ),
       },
-      body,
+      body: req.body,
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
@@ -224,7 +332,42 @@ export class TalerWireGatewayHttpClient {
       case HttpStatusCode.Conflict:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
+        return opUnknownHttpFailure(resp);
+    }
+  }
+
+  /**
+   * https://docs.taler.net/core/api-bank-wire.html#post--admin-add-kycauth
+   *
+   */
+  async addKycAuth(req: {
+    body: TalerWireGatewayApi.AddKycauthRequest;
+    auth: { username: string; password: string };
+  }) {
+    const url = new URL(`admin/add-kycauth`, this.baseUrl);
+    const resp = await this.httpLib.fetch(url.href, {
+      method: "POST",
+      headers: {
+        Authorization: makeBasicAuthHeader(
+          req.auth.username,
+          req.auth.password,
+        ),
+      },
+      body: req.body,
+    });
+    switch (resp.status) {
+      case HttpStatusCode.Ok:
+        return opSuccessFromHttp(resp, codecForAddIncomingResponse());
+      //FIXME: show more details in docs
+      case HttpStatusCode.BadRequest:
+        return opKnownHttpFailure(resp.status, resp);
+      case HttpStatusCode.Unauthorized:
+        return opKnownHttpFailure(resp.status, resp);
+      //FIXME: show more details in docs
+      case HttpStatusCode.NotFound:
+        return opKnownHttpFailure(resp.status, resp);
+      default:
+        return opUnknownHttpFailure(resp);
     }
   }
 }
