@@ -53,6 +53,8 @@ export function singular(
   ...values: any[]
 ): TranslatedString {
   const s = toI18nString(stringSeq);
+  // jed throws a Error when key is empty
+  if (!s) return "" as TranslatedString;
   const tr = jed
     .translate(s)
     .ifPlural(1, s)
@@ -60,6 +62,18 @@ export function singular(
   return tr;
 }
 
+function withContext(ctx: string): typeof singular {
+  return function (t: TemplateStringsArray, ...v: any[]): TranslatedString {
+    const s = toI18nString(t);
+    const tr = jed
+      .translate(s)
+      .withContext(ctx)
+      .ifPlural(1, s)
+      .fetch(...v);
+    return tr;
+  };
+}
+
 /**
  * Internationalize a string template without serializing
  */
@@ -79,14 +93,18 @@ export function translate(
 export function Translate({
   children,
   debug,
+  context: ctx,
 }: {
   children: any;
   debug?: boolean;
+  context?: string;
 }): any {
   const c = [].concat(children);
   const s = stringifyArray(c);
   if (!s) return [];
-  const translation: TranslatedString = jed.ngettext(s, s, 1);
+  const translation: TranslatedString = ctx
+    ? jed.npgettext(ctx, s, s, 1)
+    : jed.ngettext(s, s, 1);
   if (debug) {
     console.log("looking for ", s, "got", translation);
   }
@@ -154,8 +172,12 @@ function stringifyArray(children: Array<any>): string {
   return s;
 }
 
+export type InternationalizationAPI = typeof i18n;
+export type Translator = (i18n: InternationalizationAPI) => TranslatedString;
+
 export const i18n = {
   str: singular,
+  ctx: withContext,
   singular,
   Translate,
   translate,
