@@ -32,12 +32,9 @@ import {
 
 export enum NotificationType {
   BalanceChange = "balance-change",
+  BankAccountChange = "bank-account-change",
   BackupOperationError = "backup-error",
   TransactionStateTransition = "transaction-state-transition",
-  /**
-   * @deprecated
-   */
-  WithdrawalOperationTransition = "withdrawal-operation-transition",
   ExchangeStateTransition = "exchange-state-transition",
   Idle = "idle",
   TaskObservabilityEvent = "task-observability-event",
@@ -52,9 +49,31 @@ export interface ErrorInfoSummary {
 
 export interface TransactionStateTransitionNotification {
   type: NotificationType.TransactionStateTransition;
+
+  /**
+   * Identifier of the affected transaction.
+   */
   transactionId: string;
+
+  /**
+   * State before the transition.
+   */
   oldTxState: TransactionState;
+
+  /**
+   * State after the transition.
+   */
   newTxState: TransactionState;
+
+  /**
+   * Internal ID of the new state.
+   * Must not be used by the UI, only used for testing.
+   */
+  newStId: number;
+
+  /**
+   * Short summary of the error for an error transition.
+   */
   errorInfo?: ErrorInfoSummary;
 
   /**
@@ -85,8 +104,10 @@ export interface ExchangeStateTransitionNotification {
 
   /**
    * New state of the exchange.
+   *
+   * If missing, exchange got deleted.
    */
-  newExchangeState: ExchangeEntryState;
+  newExchangeState?: ExchangeEntryState;
 
   /**
    * Summary of the error that occurred when trying to update the exchange entry,
@@ -95,9 +116,30 @@ export interface ExchangeStateTransitionNotification {
   errorInfo?: ErrorInfoSummary;
 }
 
+/**
+ * Transaction emitted when a bank account changes.
+ */
+export interface BankAccountChangeNotification {
+  type: NotificationType.BankAccountChange;
+
+  /**
+   * ID of the affected bank account.
+   */
+  bankAccountId: string;
+}
+
 export interface BalanceChangeNotification {
   type: NotificationType.BalanceChange;
 
+  /**
+   * If set to true, the balance change is internal
+   * to the wallet and not visible to the user.
+   *
+   * (For example when the material balance changes via a refresh, but
+   * the available balance stays the same.)
+   */
+  isInternal?: boolean;
+
   /**
    * Transaction ID of the transaction that caused the balance update.
    *
@@ -238,16 +280,6 @@ export interface BackupOperationErrorNotification {
   type: NotificationType.BackupOperationError;
   error: TalerErrorDetail;
 }
-/**
- * This notification is required to signal UI that
- * the withdrawal operation changed the state.
- *
- * https://bugs.gnunet.org/view.php?id=8099
- */
-export interface WithdrawalOperationTransitionNotification {
-  type: NotificationType.WithdrawalOperationTransition;
-  uri: string;
-}
 
 export interface IdleNotification {
   type: NotificationType.Idle;
@@ -255,7 +287,7 @@ export interface IdleNotification {
 
 export type WalletNotification =
   | BalanceChangeNotification
-  | WithdrawalOperationTransitionNotification
+  | BankAccountChangeNotification
   | BackupOperationErrorNotification
   | ExchangeStateTransitionNotification
   | TransactionStateTransitionNotification
