@@ -37,23 +37,92 @@ import {
   DenomKeyType,
   DenominationPubKey,
 } from "./types-taler-exchange.js";
+import { TokenEnvelope, TokenIssuePublicKey } from "./types-taler-merchant.js";
+import { PayWalletData } from "./types-taler-wallet.js";
 
-export type Flavor<T, FlavorT extends string> = T & {
-  _flavor?: `taler.${FlavorT}`;
+const isEddsaPubP: unique symbol = Symbol("isEddsaPubP");
+type FlavorEddsaPubP = {
+  readonly flavor?: typeof isEddsaPubP;
+  readonly _size?: 32;
 };
 
-export type FlavorP<T, FlavorT extends string, S extends number> = T & {
-  _flavor?: `taler.${FlavorT}`;
-  _size?: S;
+const isEddsaPrivP: unique symbol = Symbol("isEddsaPrivP");
+type FlavorEddsaPrivP = {
+  readonly flavor?: typeof isEddsaPrivP;
+  readonly _size?: 32;
 };
 
-export function getRandomBytes(n: number): Uint8Array {
-  return nacl.randomBytes(n);
-}
+const isEddsaSigP: unique symbol = Symbol("isEddsaSigP");
+type FlavorEddsaSigP = {
+  readonly flavor?: typeof isEddsaSigP;
+  readonly _size?: 64;
+};
 
-export function getRandomBytesF<T extends number, N extends string>(
-  n: T,
-): FlavorP<Uint8Array, N, T> {
+const isEdx25519PublicKey: unique symbol = Symbol("isEdx25519PublicKey");
+type FlavorEdx25519PublicKey = {
+  readonly flavor?: typeof isEdx25519PublicKey;
+  readonly _size?: 32;
+};
+
+const isEdx25519PrivateKey: unique symbol = Symbol("isEdx25519PrivateKey");
+type FlavorEdx25519PrivateKey = {
+  readonly flavor?: typeof isEdx25519PrivateKey;
+  readonly _size?: 64;
+};
+
+const isEcdhePrivP: unique symbol = Symbol("isEcdhePrivP");
+type FlavorEcdhePrivP = {
+  readonly flavor?: typeof isEcdhePrivP;
+  readonly _size?: 32;
+};
+
+const isEdx25519Signature: unique symbol = Symbol("isEdx25519Signature");
+type FlavorEdx25519Signature = {
+  readonly flavor?: typeof isEdx25519Signature;
+  readonly _size?: 64;
+};
+
+const isEdx25519PublicKeyEnc: unique symbol = Symbol("isEdx25519PublicKeyEnc");
+type FlavorEdx25519PublicKeyEnc = {
+  readonly [isEdx25519PublicKeyEnc]?: true;
+};
+
+const isEdx25519PrivateKeyEnc: unique symbol = Symbol(
+  "isEdx25519PrivateKeyEnc",
+);
+type FlavorEdx25519PrivateKeyEnc = {
+  readonly [isEdx25519PrivateKeyEnc]?: true;
+};
+
+const isEncryptionNonce: unique symbol = Symbol("isEncryptionNone");
+type FlavorEncryptionNonceP = {
+  readonly flavor?: typeof isEncryptionNonce;
+};
+
+type Sized<T> = { readonly _size?: T };
+
+export type EddsaPubP = Uint8Array & FlavorEddsaPubP;
+export type EddsaPrivP = Uint8Array & FlavorEddsaPrivP;
+export type EddsaSigP = Uint8Array & FlavorEddsaSigP;
+
+export type EcdhePrivP = Uint8Array & FlavorEcdhePrivP;
+
+export type OpaqueData = Uint8Array;
+export type Edx25519PublicKey = Uint8Array & FlavorEdx25519PublicKey;
+export type Edx25519PrivateKey = Uint8Array & FlavorEdx25519PrivateKey;
+export type Edx25519Signature = Uint8Array & FlavorEdx25519Signature;
+
+export type Edx25519PublicKeyEnc = string & FlavorEdx25519PublicKeyEnc;
+export type Edx25519PrivateKeyEnc = string & FlavorEdx25519PrivateKeyEnc;
+
+export type EncryptionNonceP = Uint8Array & FlavorEncryptionNonceP;
+
+export type PursePublicKey = EddsaPubP;
+
+export type ContractPrivateKey = EcdhePrivP;
+export type MergePrivateKeyP = Uint8Array & EddsaPrivP;
+
+export function getRandomBytes<N extends number>(n: N): Uint8Array & Sized<N> {
   return nacl.randomBytes(n);
 }
 
@@ -317,8 +386,8 @@ export function keyExchangeEddsaEcdh(
 }
 
 export function keyExchangeEcdhEddsa(
-  ecdhPriv: Uint8Array & MaterialEcdhePriv,
-  eddsaPub: Uint8Array & MaterialEddsaPub,
+  ecdhPriv: Uint8Array & FlavorEcdhePrivP,
+  eddsaPub: Uint8Array & FlavorEddsaPubP,
 ): Uint8Array {
   if (tart) {
     return tart.keyExchangeEcdhEddsa(ecdhPriv, eddsaPub);
@@ -850,6 +919,62 @@ export function hashDenomPub(pub: DenominationPubKey): Uint8Array {
   }
 }
 
+/**
+ * Hash a token issue public key.
+ */
+export function hashTokenIssuePub(pub: TokenIssuePublicKey): Uint8Array {
+  if (pub.cipher === DenomKeyType.Rsa) {
+    const dec = decodeCrock(pub.rsa_pub);
+    return hash(dec);
+  } else if (pub.cipher === DenomKeyType.ClauseSchnorr) {
+    const dec = decodeCrock(pub.cs_pub);
+    return hash(dec);
+  } else {
+    throw Error(
+      `unsupported cipher (${
+        (pub as TokenIssuePublicKey).cipher
+      }), unable to hash`,
+    );
+  }
+}
+
+/**
+ * Hash a token envelope.
+ */
+export function hashTokenEv(
+  tokenEv: TokenEnvelope,
+  tokenIssuePubHash: HashCodeString,
+): Uint8Array {
+  const hashContext = createHashContext();
+  hashContext.update(decodeCrock(tokenIssuePubHash));
+  hashTokenEvInner(tokenEv, hashContext);
+  return hashContext.finish();
+}
+
+export function hashTokenEvInner(
+  tokenEv: TokenEnvelope,
+  hashState: TalerHashState,
+): void {
+  const hashInputBuf = new ArrayBuffer(4);
+  const uint8ArrayBuf = new Uint8Array(hashInputBuf);
+  const dv = new DataView(hashInputBuf);
+  dv.setUint32(0, DenomKeyType.toIntTag(tokenEv.cipher));
+  hashState.update(uint8ArrayBuf);
+  switch (tokenEv.cipher) {
+    case DenomKeyType.Rsa:
+      hashState.update(decodeCrock(tokenEv.rsa_blinded_planchet));
+      return;
+    default:
+      throw new Error();
+  }
+}
+
+export function hashPayWalletData(walletData: PayWalletData): Uint8Array {
+  const canon = canonicalJson(walletData) + "\0";
+  const bytes = stringToBytes(canon);
+  return hash(bytes);
+}
+
 export function eddsaSign(msg: Uint8Array, eddsaPriv: Uint8Array): Uint8Array {
   if (tart) {
     return tart.eddsaSign(msg, eddsaPriv);
@@ -860,8 +985,8 @@ export function eddsaSign(msg: Uint8Array, eddsaPriv: Uint8Array): Uint8Array {
 
 export function eddsaVerify(
   msg: Uint8Array,
-  sig: Uint8Array,
-  eddsaPub: Uint8Array,
+  sig: EddsaSigP,
+  eddsaPub: EddsaPubP,
 ): boolean {
   if (tart) {
     return tart.eddsaVerify(msg, sig, eddsaPub);
@@ -995,6 +1120,7 @@ export enum TalerSignaturePurpose {
   WALLET_ACCOUNT_MERGE = 1214,
   WALLET_PURSE_ECONTRACT = 1216,
   WALLET_PURSE_DELETE = 1220,
+  WALLET_TOKEN_USE = 1222,
   WALLET_COIN_HISTORY = 1209,
   EXCHANGE_CONFIRM_RECOUP = 1039,
   EXCHANGE_CONFIRM_RECOUP_REFRESH = 1041,
@@ -1056,18 +1182,6 @@ export function buildSigPS(purposeNum: number): SignaturePurposeBuilder {
   return new SignaturePurposeBuilder(purposeNum);
 }
 
-export type OpaqueData = Flavor<Uint8Array, any>;
-export type Edx25519PublicKey = FlavorP<Uint8Array, "Edx25519PublicKey", 32>;
-export type Edx25519PrivateKey = FlavorP<Uint8Array, "Edx25519PrivateKey", 64>;
-export type Edx25519Signature = FlavorP<Uint8Array, "Edx25519Signature", 64>;
-
-export type Edx25519PublicKeyEnc = FlavorP<string, "Edx25519PublicKeyEnc", 32>;
-export type Edx25519PrivateKeyEnc = FlavorP<
-  string,
-  "Edx25519PrivateKeyEnc",
-  64
->;
-
 /**
  * Convert a big integer to a fixed-size, little-endian array.
  */
@@ -1124,9 +1238,9 @@ export namespace Edx25519 {
   ): Promise<OpaqueData> {
     const res = kdfKw({
       outputLength: 64,
-      salt: seed,
       ikm: pub,
-      info: stringToBytes("edx25519-derivation"),
+      salt: stringToBytes("edx25519-derivation"),
+      info: seed,
     });
 
     return res;
@@ -1436,12 +1550,9 @@ export namespace AgeRestriction {
   }
 }
 
-// FIXME: make it a branded type!
-export type EncryptionNonce = FlavorP<Uint8Array, "EncryptionNonce", 24>;
-
 async function deriveKey(
   keySeed: OpaqueData,
-  nonce: EncryptionNonce,
+  nonce: EncryptionNonceP,
   salt: string,
 ): Promise<Uint8Array> {
   return kdfKw({
@@ -1453,7 +1564,7 @@ async function deriveKey(
 }
 
 export async function encryptWithDerivedKey(
-  nonce: EncryptionNonce,
+  nonce: EncryptionNonceP,
   keySeed: OpaqueData,
   plaintext: OpaqueData,
   salt: string,
@@ -1486,44 +1597,15 @@ enum ContractFormatTag {
   PaymentRequest = 1,
 }
 
-type MaterialEddsaPub = {
-  _materialType?: "eddsa-pub";
-  _size?: 32;
-};
-
-type MaterialEddsaPriv = {
-  _materialType?: "ecdhe-priv";
-  _size?: 32;
-};
-
-type MaterialEcdhePub = {
-  _materialType?: "ecdhe-pub";
-  _size?: 32;
-};
-
-type MaterialEcdhePriv = {
-  _materialType?: "ecdhe-priv";
-  _size?: 32;
-};
-
-type PursePublicKey = FlavorP<Uint8Array, "PursePublicKey", 32> &
-  MaterialEddsaPub;
-
-type ContractPrivateKey = FlavorP<Uint8Array, "ContractPrivateKey", 32> &
-  MaterialEcdhePriv;
-
-type MergePrivateKey = FlavorP<Uint8Array, "MergePrivateKey", 32> &
-  MaterialEddsaPriv;
-
 const mergeSalt = "p2p-merge-contract";
 const depositSalt = "p2p-deposit-contract";
 
 export function encryptContractForMerge(
   pursePub: PursePublicKey,
   contractPriv: ContractPrivateKey,
-  mergePriv: MergePrivateKey,
+  mergePriv: MergePrivateKeyP,
   contractTerms: any,
-  nonce: EncryptionNonce,
+  nonce: EncryptionNonceP,
 ): Promise<OpaqueData> {
   const contractTermsCanon = canonicalJson(contractTerms) + "\0";
   const contractTermsBytes = stringToBytes(contractTermsCanon);
@@ -1542,7 +1624,7 @@ export function encryptContractForDeposit(
   pursePub: PursePublicKey,
   contractPriv: ContractPrivateKey,
   contractTerms: any,
-  nonce: EncryptionNonce,
+  nonce: EncryptionNonceP,
 ): Promise<OpaqueData> {
   const contractTermsCanon = canonicalJson(contractTerms) + "\0";
   const contractTermsBytes = stringToBytes(contractTermsCanon);
@@ -1603,7 +1685,7 @@ export async function decryptContractForDeposit(
   };
 }
 
-export function amountToBuffer(amount: AmountLike): Uint8Array {
+export function bufferFromAmount(amount: AmountLike): Uint8Array {
   const amountJ = Amounts.jsonifyAmount(amount);
   const buffer = new ArrayBuffer(8 + 4 + 12);
   const dvbuf = new DataView(buffer);
@@ -1624,18 +1706,23 @@ export function amountToBuffer(amount: AmountLike): Uint8Array {
   return u8buf;
 }
 
+const foreverNum = 2n ** 64n - 1n;
+
 export function timestampRoundedToBuffer(
   ts: TalerProtocolTimestamp,
 ): Uint8Array {
   const b = new ArrayBuffer(8);
   const v = new DataView(b);
+  const numVal =
+    ts.t_s === "never" ? foreverNum : BigInt(ts.t_s) * 1000n * 1000n;
   // The buffer we sign over represents the timestamp in microseconds.
   if (typeof v.setBigUint64 !== "undefined") {
-    const s = BigInt(ts.t_s) * BigInt(1000 * 1000);
-    v.setBigUint64(0, s);
+    v.setBigUint64(0, numVal);
   } else {
     const s =
-      ts.t_s === "never" ? bigint.zero : bigint(ts.t_s).multiply(1000 * 1000);
+      ts.t_s === "never"
+        ? bigint(foreverNum)
+        : bigint(ts.t_s).multiply(1000 * 1000);
     const arr = s.toArray(2 ** 8).value;
     let offset = 8 - arr.length;
     for (let i = 0; i < arr.length; i++) {
@@ -1662,3 +1749,10 @@ export function durationRoundedToBuffer(ts: TalerProtocolDuration): Uint8Array {
   }
   return new Uint8Array(b);
 }
+
+export function toHexString(byteArray: Uint8Array) {
+  return byteArray.reduce(
+    (output, elem) => output + ("0" + elem.toString(16)).slice(-2),
+    "",
+  );
+}
