@@ -97,6 +97,18 @@ export namespace TalerPreciseTimestamp {
   }
 }
 
+export namespace TalerProtocolDuration {
+  export function fromSpec(d: DurationUnitSpec) {
+    return Duration.toTalerProtocolDuration(Duration.fromSpec(d));
+  }
+
+  export function forever(): TalerProtocolDuration {
+    return {
+      d_us: "forever",
+    };
+  }
+}
+
 export namespace TalerProtocolTimestamp {
   export function now(): TalerProtocolTimestamp {
     return AbsoluteTime.toProtocolTimestamp(AbsoluteTime.now());
@@ -136,6 +148,7 @@ export namespace TalerProtocolTimestamp {
     }
     return { t_s: Math.min(t1.t_s, t2.t_s) };
   }
+
   export function max(
     t1: TalerProtocolTimestamp,
     t2: TalerProtocolTimestamp,
@@ -172,6 +185,15 @@ export function setDangerousTimetravel(dt: number): void {
   timeshift = dt;
 }
 
+export interface DurationUnitSpec {
+  seconds?: number;
+  minutes?: number;
+  hours?: number;
+  days?: number;
+  months?: number;
+  years?: number;
+}
+
 export namespace Duration {
   export function toMilliseconds(d: Duration): number {
     if (d.d_ms === "forever") {
@@ -220,12 +242,24 @@ export namespace Duration {
       }
 
       if (s[i] === "s") {
+        if (s.startsWith("seconds", i)) {
+          i += "seconds".length - 1;
+        }
         dMs += 1000 * Number.parseInt(currentNum, 10);
       } else if (s[i] === "m") {
+        if (s.startsWith("minutes", i)) {
+          i += "minutes".length - 1;
+        }
         dMs += 60 * 1000 * Number.parseInt(currentNum, 10);
       } else if (s[i] === "h") {
+        if (s.startsWith("hours", i)) {
+          i += "hours".length - 1;
+        }
         dMs += 60 * 60 * 1000 * Number.parseInt(currentNum, 10);
       } else if (s[i] === "d") {
+        if (s.startsWith("days", i)) {
+          i += "days".length - 1;
+        }
         dMs += 24 * 60 * 60 * 1000 * Number.parseInt(currentNum, 10);
       } else {
         throw Error("invalid duration, unsupported unit");
@@ -280,14 +314,12 @@ export namespace Duration {
     return Math.ceil(d.d_ms / 1000 / 60 / 60 / 24 / 365);
   }
 
-  export function fromSpec(spec: {
-    seconds?: number;
-    minutes?: number;
-    hours?: number;
-    days?: number;
-    months?: number;
-    years?: number;
-  }): Duration {
+  /**
+   * Construct a duration from a specification of the individual units.
+   *
+   * Returns a zero duration if none of the units were specified.
+   */
+  export function fromSpec(spec: DurationUnitSpec): Duration {
     let d_ms = 0;
     d_ms += (spec.seconds ?? 0) * SECONDS;
     d_ms += (spec.minutes ?? 0) * MINUTES;
@@ -298,6 +330,52 @@ export namespace Duration {
     return { d_ms };
   }
 
+  export function fromSpecOrUndefined(
+    spec: DurationUnitSpec,
+  ): Duration | undefined {
+    if (
+      spec.seconds == undefined &&
+      spec.minutes == undefined &&
+      spec.hours == undefined &&
+      spec.days == undefined &&
+      spec.months == undefined &&
+      spec.years == undefined
+    ) {
+      return undefined;
+    }
+
+    return Duration.fromSpec(spec);
+  }
+
+  export function toSpec({ d_ms }: Duration):
+    | {
+        seconds: number;
+        minutes: number;
+        hours: number;
+        days: number;
+        month: number;
+        years: number;
+      }
+    | undefined {
+    if (d_ms === "forever") return undefined;
+    const ms = d_ms > 0 ? d_ms : 0;
+    const Y_rest = ms % YEARS;
+    const M_rest = Y_rest % MONTHS;
+    const D_rest = M_rest % DAYS;
+    const h_rest = D_rest % HOURS;
+    const m_rest = h_rest % MINUTES;
+    const millis = m_rest % SECONDS;
+
+    return {
+      years: (ms - Y_rest) / YEARS,
+      month: (Y_rest - M_rest) / MONTHS,
+      days: (M_rest - D_rest) / DAYS,
+      hours: (D_rest - h_rest) / HOURS,
+      minutes: (h_rest - m_rest) / MINUTES,
+      seconds: (m_rest - millis) / SECONDS,
+    };
+  }
+
   export function getForever(): Duration {
     return { d_ms: "forever" };
   }
@@ -605,7 +683,9 @@ export function durationAdd(d1: Duration, d2: Duration): Duration {
 export const codecForAbsoluteTime: Codec<AbsoluteTime> = {
   decode(x: any, c?: Context): AbsoluteTime {
     if (x === undefined) {
-      throw Error(`got undefined and expected absolute time at ${renderContext(c)}`);
+      throw Error(
+        `got undefined and expected absolute time at ${renderContext(c)}`,
+      );
     }
     const t_ms = x.t_ms;
     if (typeof t_ms === "string") {
@@ -623,7 +703,9 @@ export const codecForTimestamp: Codec<TalerProtocolTimestamp> = {
   decode(x: any, c?: Context): TalerProtocolTimestamp {
     // Compatibility, should be removed soon.
     if (x === undefined) {
-      throw Error(`got undefined and expected timestamp at ${renderContext(c)}`);
+      throw Error(
+        `got undefined and expected timestamp at ${renderContext(c)}`,
+      );
     }
     const t_ms = x.t_ms;
     if (typeof t_ms === "string") {
@@ -676,3 +758,19 @@ export const codecForDuration: Codec<TalerProtocolDuration> = {
     throw Error(`expected duration at ${renderContext(c)}`);
   },
 };
+
+export const codecForDurationMs: Codec<Duration> = {
+  decode(x: any, c?: Context): Duration {
+    const d_ms = x.d_ms;
+    if (typeof d_ms === "string") {
+      if (d_ms === "forever") {
+        return { d_ms: "forever" };
+      }
+      throw Error(`expected duration at ${renderContext(c)}`);
+    }
+    if (typeof d_ms === "number") {
+      return { d_ms };
+    }
+    throw Error(`expected duration at ${renderContext(c)}`);
+  },
+};
