@@ -40,7 +40,7 @@ import {
   codecForString,
   codecOptional,
 } from "./codec.js";
-import { codecForEither } from "./index.js";
+import { EddsaPrivP } from "./taler-crypto.js";
 import {
   TalerProtocolDuration,
   TalerProtocolTimestamp,
@@ -81,12 +81,12 @@ export type ClaimToken = string;
 // EdDSA and ECDHE public keys always point on Curve25519
 // and represented  using the standard 256 bits Ed25519 compact format,
 // converted to Crockford Base32.
-export type EddsaPublicKey = string;
+export type EddsaPublicKey = EddsaPublicKeyString;
 
 // EdDSA and ECDHE public keys always point on Curve25519
 // and represented  using the standard 256 bits Ed25519 compact format,
 // converted to Crockford Base32.
-export type EddsaPrivateKey = string;
+export type EddsaPrivateKey = EddsaPrivateKeyString;
 
 // Edx25519 public keys are points on Curve25519 and represented using the
 // standard 256 bits Ed25519 compact format converted to Crockford
@@ -154,13 +154,15 @@ export interface CurrencySpecification {
   // e.g.  "0 => €" or "3 => k€". For BTC, would be "0 => BTC, -3 => mBTC".
   // Communicates the currency symbol to be used.
   alt_unit_names: { [log10: string]: string };
+
+  common_amounts?: AmountString[];
 }
 
 export interface InternationalizedString {
   [lang_tag: string]: string;
 }
 
-export type RsaPublicKeySring = string;
+export type RsaPublicKeyString = string;
 export type AgeMask = number;
 
 // The string must be a data URL according to RFC 2397
@@ -177,29 +179,6 @@ export type ImageDataUrl = string;
  */
 export type Cs25519Point = string;
 
-/**
- * Response from the bank.
- */
-export class WithdrawOperationStatusResponse {
-  status: "selected" | "aborted" | "confirmed" | "pending";
-
-  selection_done: boolean;
-
-  transfer_done: boolean;
-
-  aborted: boolean;
-
-  amount: string | undefined;
-
-  sender_wire?: string;
-
-  suggested_exchange?: string;
-
-  confirm_transfer_url?: string;
-
-  wire_types: string[];
-}
-
 export type LitAmountString = `${string}:${number}`;
 
 export type LibtoolVersionString = string;
@@ -212,9 +191,12 @@ export type AmountString =
   | LitAmountString;
 // export type AmountString = string;
 export type Base32String = string;
-export type EddsaSignatureString = string;
-export type EddsaPublicKeyString = string;
-export type EddsaPrivateKeyString = string;
+declare const __eddsasign_str: unique symbol;
+export type EddsaSignatureString = string; // & { [__eddsasign_str]: true };
+declare const __eddsapub_str: unique symbol;
+export type EddsaPublicKeyString = string; // & { [__eddsapub_str]: true };
+declare const __eddsapriv_str: unique symbol;
+export type EddsaPrivateKeyString = string; // & { [__eddsapriv_str]: true };
 export type CoinPublicKeyString = string;
 
 // FIXME: implement this codec
@@ -225,32 +207,19 @@ export const codecForLibtoolVersion = codecForString;
 export const codecForCurrencyName = codecForString;
 // FIXME: implement this codec
 export const codecForDecimalNumber = codecForString;
+// FIXME: implement this codec
+export const codecForEddsaPublicKey =
+  codecForString as () => Codec<EddsaPublicKeyString>;
+// FIXME: implement this codec
+export const codecForEddsaPrivateKey =
+  codecForString as () => Codec<EddsaPrivateKeyString>;
+// FIXME: implement this codec
+export const codecForEddsaSignature =
+  codecForString as () => Codec<EddsaSignatureString>;
 
 export const codecForInternationalizedString =
   (): Codec<InternationalizedString> => codecForMap(codecForString());
 
-export const codecForWithdrawOperationStatusResponse =
-  (): Codec<WithdrawOperationStatusResponse> =>
-    buildCodecForObject<WithdrawOperationStatusResponse>()
-      .property(
-        "status",
-        codecForEither(
-          codecForConstString("selected"),
-          codecForConstString("confirmed"),
-          codecForConstString("aborted"),
-          codecForConstString("pending"),
-        ),
-      )
-      .property("selection_done", codecForBoolean())
-      .property("transfer_done", codecForBoolean())
-      .property("aborted", codecForBoolean())
-      .property("amount", codecOptional(codecForString()))
-      .property("sender_wire", codecOptional(codecForString()))
-      .property("suggested_exchange", codecOptional(codecForString()))
-      .property("confirm_transfer_url", codecOptional(codecForString()))
-      .property("wire_types", codecForList(codecForString()))
-      .build("WithdrawOperationStatusResponse");
-
 export const codecForCurrencySpecificiation =
   (): Codec<CurrencySpecification> =>
     buildCodecForObject<CurrencySpecification>()
@@ -259,6 +228,11 @@ export const codecForCurrencySpecificiation =
       .property("num_fractional_normal_digits", codecForNumber())
       .property("num_fractional_trailing_zero_digits", codecForNumber())
       .property("alt_unit_names", codecForMap(codecForString()))
+      .property(
+        "common_amounts",
+        codecOptional(codecForList(codecForAmountString())),
+      )
+      .deprecatedProperty("currency")
       .build("CurrencySpecification");
 
 export interface TalerCommonConfigResponse {
@@ -269,6 +243,7 @@ export interface TalerCommonConfigResponse {
 export const codecForTalerCommonConfigResponse =
   (): Codec<TalerCommonConfigResponse> =>
     buildCodecForObject<TalerCommonConfigResponse>()
+      .allowExtra()
       .property("name", codecForString())
       .property("version", codecForString())
       .build("TalerCommonConfigResponse");
@@ -291,21 +266,6 @@ export type HashCodeString = string;
 
 export type WireSalt = string;
 
-export interface WalletKycUuid {
-  // UUID that the wallet should use when initiating
-  // the KYC check.
-  requirement_row: number;
-
-  // Hash of the payto:// account URI for the wallet.
-  h_payto: string;
-}
-
-export const codecForWalletKycUuid = (): Codec<WalletKycUuid> =>
-  buildCodecForObject<WalletKycUuid>()
-    .property("requirement_row", codecForNumber())
-    .property("h_payto", codecForString())
-    .build("WalletKycUuid");
-
 export interface MerchantUsingTemplateDetails {
   summary?: string;
   amount?: AmountString;
@@ -377,8 +337,8 @@ export interface DenominationExpiredMessage {
 export const codecForDenominationExpiredMessage = () =>
   buildCodecForObject<DenominationExpiredMessage>()
     .property("code", codecForNumber())
-    .property("exchange_sig", codecForString())
-    .property("exchange_pub", codecForString())
+    .property("exchange_sig", codecForEddsaSignature())
+    .property("exchange_pub", codecForEddsaPublicKey())
     .property("h_denom_pub", codecForString())
     .property("timestamp", codecForTimestamp)
     .property("oper", codecForString())
@@ -402,10 +362,15 @@ export const codecForCoinHistoryResponse = () =>
     .property("history", codecForAny())
     .build("CoinHistoryResponse");
 
+export type BankTokenScope =
+  | "readonly"
+  | "readwrite"
+  | "revenue"
+  | "wiregateway";
 export interface TokenRequest {
   // Service-defined scope for the token.
   // Typical scopes would be "readonly" or "readwrite".
-  scope: string;
+  scope: BankTokenScope;
 
   // Server may impose its own upper bound
   // on the token validity duration
@@ -428,17 +393,49 @@ export interface TokenSuccessResponse {
   // Opque access token.
   access_token: AccessToken;
 }
-export interface TokenSuccessResponseMerchant {
-  // Expiration determined by the server.
-  // Can be based on the token_duration
-  // from the request, but ultimately the
-  // server decides the expiration.
+
+export interface TokenInfos {
+  tokens: TokenInfo[];
+}
+
+export interface TokenInfo {
+  // Time when the token was created.
+  creation_time: Timestamp;
+
+  // Time when the token expires.
   expiration: Timestamp;
 
-  // Opque access token.
-  token: AccessToken;
+  // Scope for the token.
+  scope: string;
+
+  // Is the token refreshable into a new token during its
+  // validity?
+  // Refreshable tokens effectively provide indefinite
+  // access if they are refreshed in time.
+  refreshable: boolean;
+
+  // Optional token description
+  description?: string;
+
+  // Opaque unique ID used for pagination.
+  serial: Integer;
 }
 
+export const codecForTokenInfo = (): Codec<TokenInfo> =>
+  buildCodecForObject<TokenInfo>()
+    .property("creation_time", codecForTimestamp)
+    .property("expiration", codecForTimestamp)
+    .property("scope", codecForString())
+    .property("refreshable", codecForBoolean())
+    .property("description", codecOptional(codecForString()))
+    .property("serial", codecForNumber())
+    .build("TokenInfo");
+
+export const codecForTokenInfoList = (): Codec<TokenInfos> =>
+  buildCodecForObject<TokenInfos>()
+    .property("tokens", codecForList(codecForTokenInfo()))
+    .build("TokenInfoList");
+
 //FIXME: implement this codec
 export const codecForAccessToken = codecForString as () => Codec<AccessToken>;
 export const codecForTokenSuccessResponse = (): Codec<TokenSuccessResponse> =>
@@ -447,13 +444,6 @@ export const codecForTokenSuccessResponse = (): Codec<TokenSuccessResponse> =>
     .property("expiration", codecForTimestamp)
     .build("TalerAuthentication.TokenSuccessResponse");
 
-export const codecForTokenSuccessResponseMerchant =
-  (): Codec<TokenSuccessResponseMerchant> =>
-    buildCodecForObject<TokenSuccessResponseMerchant>()
-      .property("token", codecForAccessToken())
-      .property("expiration", codecForTimestamp)
-      .build("TalerAuthentication.TokenSuccessResponseMerchant");
-
 // FIXME: implement this codec
 export const codecForURN = codecForString;
 
@@ -530,35 +520,17 @@ declare const opaque_OfficerId: unique symbol;
 export type OfficerId = string & { [opaque_OfficerId]: true };
 
 declare const opaque_OfficerSigningKey: unique symbol;
-/**
- * Private key for AML officer
- */
-export type SigningKey = Uint8Array & { [opaque_OfficerSigningKey]: true };
 
 export interface OfficerAccount {
   id: OfficerId;
-  signingKey: SigningKey;
+  signingKey: EddsaPrivP;
 }
 
-
-declare const opaque_ReserveAccount: unique symbol;
-/**
- * Sealed private key for AML officer
- */
-export type LockedReserve = string & { [opaque_ReserveAccount]: true };
-
-declare const opaque_ReserveId: unique symbol;
-/**
- * Public key for AML officer
- */
-export type ReserveId = string & { [opaque_ReserveId]: true };
-
 export interface ReserveAccount {
-  id: ReserveId;
-  signingKey: SigningKey;
+  id: EddsaPublicKeyString;
+  signingKey: EddsaPrivP;
 }
 
-
 export type PaginationParams = {
   /**
    * row identifier as the starting point of the query
