@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2019 Taler Systems S.A.
+ (C) 2019-2025 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -23,11 +23,15 @@ import {
   AgeRestriction,
   Amounts,
   AmountString,
+  assertUnreachable,
+  ChoiceSelectionDetailType,
   codecForList,
   codecForString,
+  ContractTermsUtil,
   CoreApiResponse,
   Duration,
   encodeCrock,
+  ExchangeTosStatus,
   getErrorDetailFromException,
   getRandomBytes,
   InitRequest,
@@ -43,6 +47,8 @@ import {
   summarizeTalerErrorDetail,
   TalerUriAction,
   TransactionIdStr,
+  TransactionMajorState,
+  TransactionMinorState,
   WalletNotification,
 } from "@gnu-taler/taler-util";
 import { clk } from "@gnu-taler/taler-util/clk";
@@ -57,7 +63,6 @@ import {
 import { createPlatformHttpLib } from "@gnu-taler/taler-util/http";
 import { JsonMessage, runRpcServer } from "@gnu-taler/taler-util/twrpc";
 import {
-  AccessStats,
   createNativeWalletHost2,
   nativeCrypto,
   Wallet,
@@ -95,10 +100,6 @@ setUnhandledRejectionHandler((error: any) => {
 const defaultWalletDbPath = pathHomedir() + "/" + ".talerwalletdb.sqlite3";
 const defaultWalletCoreSocket = pathHomedir() + "/" + ".wallet-core.sock";
 
-function assertUnreachable(x: never): never {
-  throw new Error("Didn't expect to get here");
-}
-
 async function doPay(
   wallet: WalletCoreApiClient,
   payUrl: string,
@@ -113,6 +114,7 @@ async function doPay(
     processExit(1);
     return;
   }
+  let choiceIndex: number | undefined;
   if (result.status === PreparePayResultType.AlreadyConfirmed) {
     if (result.paid) {
       console.log("already paid!");
@@ -121,42 +123,75 @@ async function doPay(
     }
     processExit(0);
     return;
-  }
-  if (result.status === "payment-possible") {
+  } else if (result.status === PreparePayResultType.ChoiceSelection) {
+    console.log(`choices:`);
+    const choices = await wallet.call(WalletApiOperation.GetChoicesForPayment, {
+      transactionId: result.transactionId,
+    });
+    console.log(j2s(choices));
+    choiceIndex = await askChoice(choices.choices.length);
+    const myChoice = choices.choices[choiceIndex];
+    if (myChoice.status !== ChoiceSelectionDetailType.PaymentPossible) {
+      console.log("insufficient balance for choice");
+      processExit(1);
+    }
     console.log("paying ...");
+    console.log("contract", result.contractTerms);
+    console.log("raw amount:", myChoice.amountRaw);
+    console.log("effective amount:", myChoice.amountEffective);
+  } else if (result.status === "payment-possible") {
+    console.log("paying ...");
+    console.log("contract", result.contractTerms);
+    console.log("raw amount:", result.amountRaw);
+    console.log("effective amount:", result.amountEffective);
   } else {
     throw Error("not reached");
   }
-  console.log("contract", result.contractTerms);
-  console.log("raw amount:", result.amountRaw);
-  console.log("effective amount:", result.amountEffective);
-  let pay;
+  let pay: boolean;
   if (options.alwaysYes) {
     pay = true;
   } else {
-    while (true) {
-      const yesNoResp = (await clk.prompt("Pay? [Y/n]")).toLowerCase();
-      if (yesNoResp === "" || yesNoResp === "y" || yesNoResp === "yes") {
-        pay = true;
-        break;
-      } else if (yesNoResp === "n" || yesNoResp === "no") {
-        pay = false;
-        break;
-      } else {
-        console.log("please answer y/n");
-      }
-    }
+    pay = await askYesNo();
   }
 
   if (pay) {
     await wallet.call(WalletApiOperation.ConfirmPay, {
-      proposalId: result.proposalId,
+      transactionId: result.transactionId,
+      choiceIndex: choiceIndex,
+      useDonau: true,
     });
   } else {
     console.log("not paying");
   }
 }
 
+async function askChoice(n: number): Promise<number> {
+  while (true) {
+    const choice = Number.parseInt(
+      await clk.prompt(`Select choice (0-${n - 1}):`),
+    );
+    if (choice >= 0 && choice < n) {
+      return choice;
+    } else {
+      console.log("Please enter a valid choice.");
+    }
+  }
+}
+
+async function askYesNo(): Promise<boolean> {
+  while (true) {
+    const yesNoResp = (await clk.prompt("Pay? [Y/n]")).toLowerCase();
+    if (yesNoResp === "" || yesNoResp === "y" || yesNoResp === "yes") {
+      return true;
+    } else if (yesNoResp === "n" || yesNoResp === "no") {
+      return false;
+      break;
+    } else {
+      console.log("please answer y/n");
+    }
+  }
+}
+
 function applyVerbose(verbose: boolean): void {
   // TODO
 }
@@ -175,6 +210,9 @@ export const walletCli = clk
   .maybeOption("walletDbFile", ["--wallet-db"], clk.STRING, {
     help: "Location of the wallet database file",
   })
+  .maybeOption("features", ["--features"], clk.STRING, {
+    help: "Comma-separated list of feature flags to enable.",
+  })
   .maybeOption("walletConnection", ["--wallet-connection"], clk.STRING, {
     help: "Connect to an RPC wallet",
   })
@@ -249,7 +287,6 @@ export interface WalletContext {
 
 interface CreateWalletResult {
   wallet: Wallet;
-  getStats: () => AccessStats;
 }
 
 async function createLocalWallet(
@@ -266,7 +303,9 @@ async function createLocalWallet(
     persistentStoragePath: dbPath !== ":memory:" ? dbPath : undefined,
     httpLib: myHttpLib,
     notifyHandler: (n) => {
-      logger.info(`wallet notification: ${j2s(n)}`);
+      if (logger.shouldLogTrace()) {
+        logger.trace(`wallet notification: ${j2s(n)}`);
+      }
       if (notificationHandler) {
         notificationHandler(n);
       }
@@ -275,7 +314,9 @@ async function createLocalWallet(
   });
 
   applyVerbose(walletCliArgs.wallet.verbose);
-  const res = { wallet: wh.wallet, getStats: wh.getDbStats };
+  const res = { wallet: wh.wallet };
+
+  const features = (walletCliArgs.wallet.features ?? "").split(",");
 
   if (args.noInit) {
     return res;
@@ -289,12 +330,12 @@ async function createLocalWallet(
           lazyTaskLoop: args.lazyTaskLoop,
           testing: {
             devModeActive: checkEnvFlag("TALER_WALLET_DEV_MODE"),
-            denomselAllowLate: checkEnvFlag(
-              "TALER_WALLET_DEBUG_DENOMSEL_ALLOW_LATE",
-            ),
             emitObservabilityEvents: observabilityEventFile != null,
             skipDefaults: walletCliArgs.wallet.skipDefaults,
           },
+          features: {
+            enableV1Contracts: true,
+          },
         },
       } satisfies InitRequest,
     );
@@ -335,12 +376,26 @@ async function withWallet<T>(
     writeObservabilityLog(notif);
   };
 
+  let walletSocketPath: string | undefined = undefined;
+
+  const connEnvName = "TALER_WALLET_CONNECTION";
+
   if (walletCliArgs.wallet.walletConnection) {
+    walletSocketPath = walletCliArgs.wallet.walletConnection;
+    logger.info(`using wallet socket from command line (${walletSocketPath})`);
+  } else if (!!process.env[connEnvName]) {
+    walletSocketPath = process.env[connEnvName];
+    logger.info(
+      `using wallet socket from ${connEnvName} (${walletSocketPath})`,
+    );
+  }
+
+  if (walletSocketPath) {
     logger.info("creating remote wallet");
     const w = await createRemoteWallet({
       name: "wallet",
       notificationHandler: onNotif,
-      socketFilename: walletCliArgs.wallet.walletConnection,
+      socketFilename: walletSocketPath,
     });
     const ctx: WalletContext = {
       makeCoreApiRequest(operation, payload) {
@@ -365,7 +420,11 @@ async function withWallet<T>(
     await wh.wallet.client.call(WalletApiOperation.Shutdown, {});
     if (process.env.TALER_WALLET_DBSTATS) {
       console.log("database stats:");
-      console.log(j2s(wh.getStats()));
+      const stats = await wh.wallet.client.call(
+        WalletApiOperation.TestingGetDbStats,
+        {},
+      );
+      console.log(j2s(stats));
     }
     return result;
   }
@@ -421,6 +480,8 @@ walletCli
         if (resp.type === "error") {
           if (args.api.expectSuccess) {
             processExit(EXIT_API_ERROR);
+          } else {
+            logger.warn("api request resulted in error response");
           }
         }
       } catch (e) {
@@ -550,12 +611,14 @@ transactionsCli
   .requiredArgument("transactionId", clk.STRING, {
     help: "Identifier of the transaction to delete",
   })
+  .flag("includeContractTerms", ["--include-contract-terms"])
   .action(async (args) => {
     await withWallet(args, { lazyTaskLoop: true }, async (wallet) => {
       const tx = await wallet.client.call(
         WalletApiOperation.GetTransactionById,
         {
           transactionId: args.lookup.transactionId,
+          includeContractTerms: args.lookup.includeContractTerms ?? false,
         },
       );
       console.log(j2s(tx));
@@ -681,6 +744,131 @@ withdrawCli
     });
   });
 
+async function cliHandleTos(
+  wallet: WalletContext,
+  exchangeBaseUrl: string,
+): Promise<boolean> {
+  while (1) {
+    const exch = await wallet.client.call(
+      WalletApiOperation.GetExchangeEntryByUrl,
+      {
+        exchangeBaseUrl,
+      },
+    );
+    if (exch.tosStatus === ExchangeTosStatus.Proposed) {
+      const res = await readlinePrompt(
+        `Accept terms of service of exchange ${exchangeBaseUrl}? [y/N/info]: `,
+      );
+      switch (res.toLowerCase()) {
+        case "":
+        case "n": {
+          return false;
+        }
+        case "y": {
+          await wallet.client.call(WalletApiOperation.SetExchangeTosAccepted, {
+            exchangeBaseUrl,
+          });
+          return true;
+        }
+        case "info": {
+          const tosText = await wallet.client.call(
+            WalletApiOperation.GetExchangeTos,
+            {
+              exchangeBaseUrl,
+            },
+          );
+          console.log(tosText.content);
+        }
+      }
+    }
+  }
+  throw Error("not reached");
+}
+
+/**
+ * Handle the user action of scanning a taler://pay-push/ URI.
+ */
+async function cliPeerPushCredit(
+  wallet: WalletContext,
+  uri: string,
+): Promise<void> {
+  const prepRes = await wallet.client.call(
+    WalletApiOperation.PreparePeerPushCredit,
+    {
+      talerUri: uri,
+    },
+  );
+  const txDet = await wallet.client.call(
+    WalletApiOperation.GetTransactionById,
+    {
+      transactionId: prepRes.transactionId,
+      includeContractTerms: true,
+    },
+  );
+  if (txDet.txState.major === TransactionMajorState.Done) {
+    console.log("Payment already done in transaction history.");
+    return;
+  }
+  if (
+    txDet.txState.major === TransactionMajorState.Dialog &&
+    txDet.txState.minor === TransactionMinorState.Proposed
+  ) {
+    while (true) {
+      const res = await readlinePrompt(
+        `Accept payment of ${prepRes.amountEffective}? [y/N/info/delete]: `,
+      );
+      let done = false;
+      switch (res.toLowerCase()) {
+        case "y": {
+          const tosOk = await cliHandleTos(wallet, prepRes.exchangeBaseUrl);
+          if (!tosOk) {
+            console.log(
+              "ToS needs to be accepted before payment can be accepted",
+            );
+            done = true;
+            break;
+          }
+          await wallet.client.call(WalletApiOperation.ConfirmPeerPushCredit, {
+            transactionId: prepRes.transactionId,
+          });
+          console.log(
+            "peer-push-credit confirmed, waiting for transaction to be final...",
+          );
+          await wallet.client.call(
+            WalletApiOperation.TestingWaitTransactionState,
+            {
+              transactionId: prepRes.transactionId,
+              txState: [
+                { major: TransactionMajorState.Done },
+                { major: TransactionMajorState.Failed, minor: "*" },
+                { major: TransactionMajorState.Aborted, minor: "*" },
+              ],
+            },
+          );
+          done = true;
+          break;
+        }
+        case "":
+        case "n": {
+          done = true;
+          break;
+        }
+        case "info": {
+          console.log(`${j2s(txDet)}`);
+          break;
+        }
+        default: {
+          console.log("not understood");
+          break;
+        }
+      }
+      if (done) {
+        break;
+      }
+    }
+  }
+}
+
 walletCli
   .subcommand("handleUri", "handle-uri", {
     help: "Handle a taler:// URI.",
@@ -714,6 +902,37 @@ walletCli
             talerRefundUri: uri,
           });
           break;
+        case TalerUriAction.PayPush: {
+          await cliPeerPushCredit(wallet, uri);
+          break;
+        }
+        case TalerUriAction.WithdrawExchange: {
+          const exchangeBaseUrl = parsedTalerUri.exchangeBaseUrl;
+          await wallet.client.call(WalletApiOperation.UpdateExchangeEntry, {
+            exchangeBaseUrl,
+          });
+          const exch = await wallet.client.call(
+            WalletApiOperation.GetExchangeEntryByUrl,
+            {
+              exchangeBaseUrl,
+            },
+          );
+          // FIXME: Maybe prompt for this?
+          await wallet.client.call(WalletApiOperation.SetExchangeTosAccepted, {
+            exchangeBaseUrl,
+          });
+          const res = await readlinePrompt(`Amount (in ${exch.currency}): `);
+          const amount = Amounts.stringify(Amounts.parseOrThrow(res));
+          const w = await wallet.client.call(
+            WalletApiOperation.AcceptManualWithdrawal,
+            {
+              amount,
+              exchangeBaseUrl,
+            },
+          );
+          console.log(`Transaction ID: ${w.transactionId}`);
+          break;
+        }
         case TalerUriAction.Withdraw: {
           const withdrawInfo = await wallet.client.call(
             WalletApiOperation.GetWithdrawalDetailsForUri,
@@ -837,6 +1056,21 @@ exchangesCli
     });
   });
 
+exchangesCli
+  .subcommand("exchangesListCmd", "trusted", {
+    help: "List trusted global currency exchanges.",
+  })
+  .action(async (args) => {
+    console.log("Listing exchanges ...");
+    await withWallet(args, { lazyTaskLoop: true }, async (wallet) => {
+      const exchanges = await wallet.client.call(
+        WalletApiOperation.ListGlobalCurrencyExchanges,
+        {},
+      );
+      console.log(JSON.stringify(exchanges, undefined, 2));
+    });
+  });
+
 exchangesCli
   .subcommand("exchangesUpdateCmd", "update", {
     help: "Update or add an exchange by base URL.",
@@ -883,7 +1117,7 @@ exchangesCli
   .action(async (args) => {
     await withWallet(args, { lazyTaskLoop: true }, async (wallet) => {
       await wallet.client.call(WalletApiOperation.AddExchange, {
-        exchangeBaseUrl: args.exchangesAddCmd.url,
+        uri: args.exchangesAddCmd.url,
       });
     });
   });
@@ -1019,6 +1253,24 @@ backupCli.subcommand("importDb", "import-db").action(async (args) => {
   });
 });
 
+const bankAccountsCli = walletCli.subcommand(
+  "bankAccountArgs",
+  "bank-accounts",
+  {
+    help: "Subcommands for managing known bank accounts.",
+  },
+);
+
+bankAccountsCli.subcommand("listArgs", "list").action(async (args) => {
+  await withWallet(args, { lazyTaskLoop: true }, async (wallet) => {
+    const resp = await wallet.client.call(
+      WalletApiOperation.ListBankAccounts,
+      {},
+    );
+    console.log(`Bank accounts: ${j2s(resp)}`);
+  });
+});
+
 const depositCli = walletCli.subcommand("depositArgs", "deposit", {
   help: "Subcommands for depositing money to payto:// accounts",
 });
@@ -1040,6 +1292,20 @@ depositCli
     });
   });
 
+depositCli
+  .subcommand("checkDepositArgs", "check")
+  .requiredArgument("amount", clk.AMOUNT)
+  .requiredArgument("targetPayto", clk.STRING)
+  .action(async (args) => {
+    await withWallet(args, { lazyTaskLoop: true }, async (wallet) => {
+      const resp = await wallet.client.call(WalletApiOperation.CheckDeposit, {
+        amount: args.checkDepositArgs.amount,
+        depositPaytoUri: args.checkDepositArgs.targetPayto,
+      });
+      console.log(`Check deposit result: ${j2s(resp)}`);
+    });
+  });
+
 const peerCli = walletCli.subcommand("peerArgs", "p2p", {
   help: "Subcommands for peer-to-peer payments.",
 });
@@ -1580,6 +1846,9 @@ advancedCli
         case PreparePayResultType.PaymentPossible:
           console.log("payment possible");
           break;
+        case PreparePayResultType.ChoiceSelection:
+          console.log("choice selection");
+          break;
         default:
           assertUnreachable(res);
       }
@@ -1603,12 +1872,12 @@ advancedCli
   .subcommand("payConfirm", "pay-confirm", {
     help: "Confirm payment proposed by a merchant.",
   })
-  .requiredArgument("proposalId", clk.STRING)
+  .requiredArgument("transactionId", clk.STRING)
   .maybeOption("sessionIdOverride", ["--session-id"], clk.STRING)
   .action(async (args) => {
     await withWallet(args, { lazyTaskLoop: true }, async (wallet) => {
       await wallet.client.call(WalletApiOperation.ConfirmPay, {
-        proposalId: args.payConfirm.proposalId,
+        transactionId: args.payConfirm.transactionId as TransactionIdStr,
         sessionId: args.payConfirm.sessionIdOverride,
       });
     });
@@ -1697,6 +1966,15 @@ advancedCli
     });
   });
 
+advancedCli
+  .subcommand("hashContractTerms", "hash-contract-terms")
+  .action(async (args) => {
+    const data = await read(process.stdin);
+    const ct = JSON.parse(data);
+    const ctHash = ContractTermsUtil.hashContractTerms(ct);
+    console.log(ctHash);
+  });
+
 advancedCli
   .subcommand("coins", "list-coins", {
     help: "List coins.",
@@ -1706,6 +1984,7 @@ advancedCli
       const coins = await wallet.client.call(WalletApiOperation.DumpCoins, {});
       for (const coin of coins.coins) {
         console.log(`coin ${coin.coinPub}`);
+        console.log(` value ${coin.denomValue}`);
         console.log(` exchange ${coin.exchangeBaseUrl}`);
         console.log(` denomPubHash ${coin.denomPubHash}`);
         console.log(` status ${coin.coinStatus}`);
@@ -1734,21 +2013,56 @@ const testCli = walletCli.subcommand("testingArgs", "testing", {
 
 testCli
   .subcommand("withdrawTestkudos", "withdraw-testkudos")
+  .flag("wait", ["--wait"])
   .action(async (args) => {
     await withWallet(args, { lazyTaskLoop: true }, async (wallet) => {
-      await wallet.client.call(WalletApiOperation.WithdrawTestkudos, {});
+      const resp = await wallet.client.call(
+        WalletApiOperation.WithdrawTestkudos,
+        {},
+      );
+      if (args.withdrawTestkudos.wait) {
+        await wallet.client.call(
+          WalletApiOperation.TestingWaitTransactionState,
+          {
+            transactionId: resp.transactionId,
+            txState: {
+              major: TransactionMajorState.Done,
+            },
+          },
+        );
+      }
     });
   });
 
-testCli.subcommand("withdrawKudos", "withdraw-kudos").action(async (args) => {
-  await withWallet(args, { lazyTaskLoop: true }, async (wallet) => {
-    await wallet.client.call(WalletApiOperation.WithdrawTestBalance, {
-      amount: "KUDOS:50" as AmountString,
-      corebankApiBaseUrl: "https://bank.demo.taler.net/",
-      exchangeBaseUrl: "https://exchange.demo.taler.net/",
+testCli
+  .subcommand("withdrawKudos", "withdraw-kudos")
+  .flag("wait", ["--wait"])
+  .maybeOption("amount", ["--amount"], clk.AMOUNT, {
+    help: "Amount to withdraw",
+  })
+  .action(async (args) => {
+    await withWallet(args, { lazyTaskLoop: true }, async (wallet) => {
+      const resp = await wallet.client.call(
+        WalletApiOperation.WithdrawTestBalance,
+        {
+          amount: (args.withdrawKudos.amount ?? "KUDOS:50") as AmountString,
+          corebankApiBaseUrl: "https://bank.demo.taler.net/",
+          exchangeBaseUrl: "https://exchange.demo.taler.net/",
+        },
+      );
+      if (args.withdrawKudos.wait) {
+        await wallet.client.call(
+          WalletApiOperation.TestingWaitTransactionState,
+          {
+            transactionId: resp.transactionId,
+            txState: {
+              major: TransactionMajorState.Done,
+            },
+          },
+        );
+      }
     });
   });
-});
 
 class PerfTimer {
   tStarted: bigint | undefined;
