@@ -1,6 +1,7 @@
 /*
  This file is part of GNU Taler
  (C) 2022 GNUnet e.V.
+ (C) 2025 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -14,9 +15,6 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
-/**
- * Imports.
- */
 import {
   AbsoluteTime,
   AmountJson,
@@ -46,6 +44,7 @@ import {
   checkLogicInvariant,
   durationMul,
 } from "@gnu-taler/taler-util";
+import { HttpRequestOptions, HttpResponse } from "@gnu-taler/taler-util/http";
 import {
   BackupProviderRecord,
   CoinHistoryRecord,
@@ -69,6 +68,7 @@ import {
 } from "./db.js";
 import { ReadyExchangeSummary } from "./exchanges.js";
 import { createRefreshGroup } from "./refresh.js";
+import { BalanceEffect } from "./transactions.js";
 import { WalletExecutionContext, getDenomInfo } from "./wallet.js";
 
 const logger = new Logger("operations/common.ts");
@@ -83,6 +83,11 @@ export interface CoinsSpendInfo {
   transactionId: TransactionIdStr;
 }
 
+export interface TokensSpendInfo {
+  tokenPubs: string[];
+  transactionId: TransactionIdStr;
+}
+
 export async function makeCoinsVisible(
   wex: WalletExecutionContext,
   tx: WalletDbReadWriteTransaction<["coins", "coinAvailability"]>,
@@ -255,6 +260,34 @@ export async function spendCoins(
   );
 }
 
+export async function spendTokens(
+  tx: WalletDbReadWriteTransaction<["tokens", "purchases"]>,
+  tsi: TokensSpendInfo,
+): Promise<void> {
+  if (tsi.tokenPubs.length === 0) {
+    return;
+  }
+
+  for (const pub of tsi.tokenPubs) {
+    const token = await tx.tokens.get(pub);
+    if (!token) {
+      throw Error(`token allocated for payment doesn't exist anymore`);
+    }
+
+    if (token.transactionId) {
+      if (token.transactionId !== tsi.transactionId) {
+        throw Error(`token already being spent in a different transaction`);
+      } else {
+        return;
+      }
+    }
+
+    logger.trace(`spending token ${token.tokenUsePub} in ${tsi.transactionId}`);
+    token.transactionId = tsi.transactionId;
+    await tx.tokens.put(token);
+  }
+}
+
 export enum TombstoneTag {
   DeleteWithdrawalGroup = "delete-withdrawal-group",
   DeleteReserve = "delete-reserve",
@@ -450,6 +483,16 @@ export namespace TaskRunResult {
       type: TaskRunResultType.NetworkRequired,
     };
   }
+  /**
+   * Network connection is required to complete the task.
+   * When network is back, the transaction will be retried.
+   */
+  export function error(detail: TalerErrorDetail): TaskRunResult {
+    return {
+      type: TaskRunResultType.Error,
+      errorDetail: detail,
+    };
+  }
 }
 
 export interface TaskRunFinishedResult {
@@ -600,6 +643,7 @@ export function getAutoRefreshExecuteThreshold(d: {
 
 export enum PendingTaskType {
   ExchangeUpdate = "exchange-update",
+  ExchangeAutoRefresh = "exchange-auto-refresh",
   ExchangeWalletKyc = "exchange-wallet-kyc",
   Purchase = "purchase",
   Refresh = "refresh",
@@ -623,6 +667,7 @@ export type ParsedTaskIdentifier =
       withdrawalGroupId: string;
     }
   | { tag: PendingTaskType.ExchangeUpdate; exchangeBaseUrl: string }
+  | { tag: PendingTaskType.ExchangeAutoRefresh; exchangeBaseUrl: string }
   | { tag: PendingTaskType.ExchangeWalletKyc; exchangeBaseUrl: string }
   | { tag: PendingTaskType.Backup; backupProviderBaseUrl: string }
   | { tag: PendingTaskType.Deposit; depositGroupId: string }
@@ -650,6 +695,8 @@ export function parseTaskIdentifier(x: string): ParsedTaskIdentifier {
       return { tag: type, depositGroupId: rest[0] };
     case PendingTaskType.ExchangeUpdate:
       return { tag: type, exchangeBaseUrl: decodeURIComponent(rest[0]) };
+    case PendingTaskType.ExchangeAutoRefresh:
+      return { tag: type, exchangeBaseUrl: decodeURIComponent(rest[0]) };
     case PendingTaskType.ExchangeWalletKyc:
       return { tag: type, exchangeBaseUrl: decodeURIComponent(rest[0]) };
     case PendingTaskType.PeerPullCredit:
@@ -685,6 +732,8 @@ export function constructTaskIdentifier(p: ParsedTaskIdentifier): TaskIdStr {
       return `${p.tag}:${encodeURIComponent(p.exchangeBaseUrl)}` as TaskIdStr;
     case PendingTaskType.ExchangeWalletKyc:
       return `${p.tag}:${encodeURIComponent(p.exchangeBaseUrl)}` as TaskIdStr;
+    case PendingTaskType.ExchangeAutoRefresh:
+      return `${p.tag}:${encodeURIComponent(p.exchangeBaseUrl)}` as TaskIdStr;
     case PendingTaskType.PeerPullDebit:
       return `${p.tag}:${p.peerPullDebitId}` as TaskIdStr;
     case PendingTaskType.PeerPushCredit:
@@ -722,6 +771,13 @@ export namespace TaskIdentifiers {
       exchBaseUrl,
     )}` as TaskIdStr;
   }
+  export function forExchangeAutoRefreshFromUrl(
+    exchBaseUrl: string,
+  ): TaskIdStr {
+    return `${PendingTaskType.ExchangeAutoRefresh}:${encodeURIComponent(
+      exchBaseUrl,
+    )}` as TaskIdStr;
+  }
   export function forRefresh(
     refreshGroupRecord: RefreshGroupRecord,
   ): TaskIdStr {
@@ -774,7 +830,11 @@ export enum TransitionResultType {
 
 export type TransitionResult<R> =
   | { type: TransitionResultType.Stay }
-  | { type: TransitionResultType.Transition; rec: R }
+  | {
+      type: TransitionResultType.Transition;
+      rec: R;
+      balanceEffect: BalanceEffect;
+    }
   | { type: TransitionResultType.Delete };
 
 export const TransitionResult = {
@@ -784,10 +844,14 @@ export const TransitionResult = {
   delete<T>(): TransitionResult<T> {
     return { type: TransitionResultType.Delete };
   },
-  transition<T>(rec: T): TransitionResult<T> {
+  transition<T>(
+    rec: T,
+    balanceEffect: BalanceEffect = BalanceEffect.Any,
+  ): TransitionResult<T> {
     return {
       type: TransitionResultType.Transition,
       rec,
+      balanceEffect,
     };
   },
 };
@@ -799,16 +863,21 @@ export const TransitionResult = {
 export interface TransactionContext {
   get taskId(): TaskIdStr | undefined;
   get transactionId(): TransactionIdStr;
-  abortTransaction(): Promise<void>;
+  abortTransaction(reason?: TalerErrorDetail): Promise<void>;
   suspendTransaction(): Promise<void>;
   resumeTransaction(): Promise<void>;
-  failTransaction(): Promise<void>;
+  failTransaction(reason?: TalerErrorDetail): Promise<void>;
   deleteTransaction(): Promise<void>;
   lookupFullTransaction(
     tx: WalletDbAllStoresReadOnlyTransaction,
+    args?: LookupFullTransactionOpts,
   ): Promise<Transaction | undefined>;
 }
 
+export interface LookupFullTransactionOpts {
+  includeContractTerms?: boolean;
+}
+
 declare const __taskIdStr: unique symbol;
 export type TaskIdStr = string & { [__taskIdStr]: true };
 
@@ -916,8 +985,15 @@ export async function genericWaitForStateVal<T>(
 }
 
 export function requireExchangeTosAcceptedOrThrow(
+  wex: WalletExecutionContext,
   exchange: ReadyExchangeSummary,
 ): void {
+  if (wex.ws.refcntIgnoreTos > 0) {
+    logger.warn(
+      `ignoring ToS acceptance state, refcntIgnoreTos=${wex.ws.refcntIgnoreTos}`,
+    );
+    return;
+  }
   switch (exchange.tosStatus) {
     case ExchangeTosStatus.Accepted:
     case ExchangeTosStatus.MissingTos:
@@ -982,3 +1058,32 @@ export async function runWithClientCancellation<R, T>(
     }
   }
 }
+
+/**
+ * Run a queued longpool fetch with cancellation token and timeout_ms
+ */
+export async function cancelableLongPoll(
+  wex: WalletExecutionContext,
+  url: URL,
+  opt?: HttpRequestOptions,
+): Promise<HttpResponse> {
+  const longPoll = async (timeoutMs: number) => {
+    url.searchParams.set("timeout_ms", `${timeoutMs}`);
+    return cancelableFetch(wex, url, opt);
+  };
+  return wex.ws.longpollQueue.run(url, wex.cancellationToken, longPoll);
+}
+
+/**
+ * Fetch with cancellation token
+ */
+export async function cancelableFetch(
+  wex: WalletExecutionContext,
+  url: URL,
+  opt?: HttpRequestOptions,
+): Promise<HttpResponse> {
+  return wex.http.fetch(url.href, {
+    ...opt,
+    cancellationToken: wex.cancellationToken,
+  });
+}
