@@ -30,10 +30,10 @@ import {
   AmountJson,
   Amounts,
   AmountString,
-  amountToBuffer,
   BlindedDenominationSignature,
   bufferForUint32,
   bufferForUint64,
+  bufferFromAmount,
   buildSigPS,
   canonicalJson,
   CoinDepositPermission,
@@ -47,8 +47,10 @@ import {
   durationRoundedToBuffer,
   ecdhGetPublic,
   eddsaGetPublic,
+  EddsaPrivateKeyString,
   EddsaPublicKeyString,
   eddsaSign,
+  EddsaSignatureString,
   eddsaVerify,
   encodeCrock,
   encryptContractForDeposit,
@@ -62,12 +64,16 @@ import {
   hashCoinEvInner,
   hashCoinPub,
   hashDenomPub,
+  hashTokenEv,
+  hashTokenIssuePub,
   hashTruncate32,
   kdf,
   kdfKw,
   keyExchangeEcdhEddsa,
   Logger,
   MakeSyncSignatureRequest,
+  MerchantContractOutputType,
+  PayWalletData,
   PlanchetCreationRequest,
   PlanchetUnblindInfo,
   PurseDeposit,
@@ -77,17 +83,21 @@ import {
   rsaBlind,
   rsaUnblind,
   rsaVerify,
-  setupTipPlanchet,
+  SignTokenUseRequest,
+  Slate,
+  SlateCreationRequest,
+  SlateUnblindInfo,
   stringToBytes,
   TalerProtocolTimestamp,
   TalerSignaturePurpose,
   timestampRoundedToBuffer,
-  UnblindedSignature,
+  toHexString,
+  TokenEnvelope,
+  TokenIssueBlindSig,
+  UnblindedDenominationSignature,
   WireFee,
   WithdrawalPlanchet,
 } from "@gnu-taler/taler-util";
-// FIXME: Crypto should not use DB Types!
-import { DenominationRecord, timestampProtocolFromDb } from "../db.js";
 import {
   CreateRecoupRefreshReqRequest,
   CreateRecoupReqRequest,
@@ -96,15 +106,17 @@ import {
   DecryptContractRequest,
   DecryptContractResponse,
   DerivedRefreshSession,
-  DerivedTipPlanchet,
+  DerivedRefreshSessionV2,
   DeriveRefreshSessionRequest,
-  DeriveTipRequest,
+  DeriveRefreshSessionRequestV2,
   EncryptContractForDepositRequest,
   EncryptContractForDepositResponse,
   EncryptContractRequest,
   EncryptContractResponse,
   SignCoinHistoryRequest,
   SignCoinHistoryResponse,
+  SignContractTermsHashRequest,
+  SignContractTermsHashResponse,
   SignDeletePurseRequest,
   SignDeletePurseResponse,
   SignPurseMergeRequest,
@@ -122,6 +134,17 @@ import {
 
 const logger = new Logger("cryptoImplementation.ts");
 
+let debugDumpSigs = false;
+
+/**
+ * Enable dumping of signature blobs to stderr.
+ *
+ * Global option, must only be used for debugging.
+ */
+export function setDebugDumpSigs(b: boolean) {
+  debugDumpSigs = b;
+}
+
 /**
  * Interface for (asynchronous) cryptographic operations that
  * Taler uses.
@@ -133,12 +156,16 @@ export interface TalerCryptoInterface {
    */
   createPlanchet(req: PlanchetCreationRequest): Promise<WithdrawalPlanchet>;
 
+  signWithdrawal(req: SignWithdrawalRequest): Promise<SignWithdrawalResponse>;
+
   eddsaSign(req: EddsaSignRequest): Promise<EddsaSignResponse>;
 
   /**
-   * Create a planchet used for tipping, including the private keys.
+   * Create a pre-token (a.k.a. slate) of a given token family.
    */
-  createTipPlanchet(req: DeriveTipRequest): Promise<DerivedTipPlanchet>;
+  createSlate(req: SlateCreationRequest): Promise<Slate>;
+
+  signTokenUse(req: SignTokenUseRequest): Promise<EddsaSigningResult>;
 
   signTrackTransaction(
     req: SignTrackTransactionRequest,
@@ -170,18 +197,24 @@ export interface TalerCryptoInterface {
     req: ContractTermsValidationRequest,
   ): Promise<ValidationResult>;
 
-  createEddsaKeypair(req: {}): Promise<EddsaKeypair>;
+  createEddsaKeypair(req: {}): Promise<EddsaKeyPairStrings>;
 
   eddsaGetPublic(req: EddsaGetPublicRequest): Promise<EddsaGetPublicResponse>;
 
   unblindDenominationSignature(
     req: UnblindDenominationSignatureRequest,
-  ): Promise<UnblindedSignature>;
+  ): Promise<UnblindedDenominationSignature>;
+
+  unblindTokenIssueSignature(
+    req: UnblindTokenIssueSignatureRequest,
+  ): Promise<UnblindedDenominationSignature>;
 
   rsaUnblind(req: RsaUnblindRequest): Promise<RsaUnblindResponse>;
 
   rsaVerify(req: RsaVerificationRequest): Promise<ValidationResult>;
 
+  rsaVerifyDirect(req: RsaVerificationRequest): Promise<ValidationResult>;
+
   rsaBlind(req: RsaBlindRequest): Promise<RsaBlindResponse>;
 
   signDepositPermission(
@@ -192,6 +225,10 @@ export interface TalerCryptoInterface {
     req: DeriveRefreshSessionRequest,
   ): Promise<DerivedRefreshSession>;
 
+  deriveRefreshSessionV2(
+    req: DeriveRefreshSessionRequestV2,
+  ): Promise<DerivedRefreshSessionV2>;
+
   hashString(req: HashStringRequest): Promise<HashStringResult>;
 
   signCoinLink(req: SignCoinLinkRequest): Promise<EddsaSigningResult>;
@@ -202,6 +239,10 @@ export interface TalerCryptoInterface {
     req: SetupRefreshPlanchetRequest,
   ): Promise<FreshCoinEncoded>;
 
+  setupRefreshPlanchetV2(
+    req: SetupRefreshPlanchetRequestV2,
+  ): Promise<FreshCoinEncoded>;
+
   setupWithdrawalPlanchet(
     req: SetupWithdrawalPlanchetRequest,
   ): Promise<FreshCoinEncoded>;
@@ -265,6 +306,10 @@ export interface TalerCryptoInterface {
   signWalletKycAuth(
     req: SignWalletKycAuthRequest,
   ): Promise<SignWalletKycAuthResponse>;
+
+  signContractTermsHash(
+    req: SignContractTermsHashRequest,
+  ): Promise<SignContractTermsHashResponse>;
 }
 
 /**
@@ -283,9 +328,12 @@ export const nullCrypto: TalerCryptoInterface = {
   eddsaSign: function (req: EddsaSignRequest): Promise<EddsaSignResponse> {
     throw new Error("Function not implemented.");
   },
-  createTipPlanchet: function (
-    req: DeriveTipRequest,
-  ): Promise<DerivedTipPlanchet> {
+  createSlate: function (req: SlateCreationRequest): Promise<Slate> {
+    throw new Error("Function not implemented.");
+  },
+  signTokenUse: function (
+    req: SignTokenUseRequest,
+  ): Promise<EddsaSigningResult> {
     throw new Error("Function not implemented.");
   },
   signTrackTransaction: function (
@@ -333,15 +381,22 @@ export const nullCrypto: TalerCryptoInterface = {
   ): Promise<ValidationResult> {
     throw new Error("Function not implemented.");
   },
-  createEddsaKeypair: function (req: unknown): Promise<EddsaKeypair> {
+  createEddsaKeypair: function (req: unknown): Promise<EddsaKeyPairStrings> {
     throw new Error("Function not implemented.");
   },
-  eddsaGetPublic: function (req: EddsaGetPublicRequest): Promise<EddsaKeypair> {
+  eddsaGetPublic: function (
+    req: EddsaGetPublicRequest,
+  ): Promise<EddsaKeyPairStrings> {
     throw new Error("Function not implemented.");
   },
   unblindDenominationSignature: function (
     req: UnblindDenominationSignatureRequest,
-  ): Promise<UnblindedSignature> {
+  ): Promise<UnblindedDenominationSignature> {
+    throw new Error("Function not implemented.");
+  },
+  unblindTokenIssueSignature: function (
+    req: UnblindTokenIssueSignatureRequest,
+  ): Promise<UnblindedDenominationSignature> {
     throw new Error("Function not implemented.");
   },
   rsaUnblind: function (req: RsaUnblindRequest): Promise<RsaUnblindResponse> {
@@ -378,6 +433,11 @@ export const nullCrypto: TalerCryptoInterface = {
   ): Promise<FreshCoinEncoded> {
     throw new Error("Function not implemented.");
   },
+  setupRefreshPlanchetV2: function (
+    req: SetupRefreshPlanchetRequestV2,
+  ): Promise<FreshCoinEncoded> {
+    throw new Error("Function not implemented.");
+  },
   rsaBlind: function (req: RsaBlindRequest): Promise<RsaBlindResponse> {
     throw new Error("Function not implemented.");
   },
@@ -469,6 +529,26 @@ export const nullCrypto: TalerCryptoInterface = {
   ): Promise<SignWalletKycAuthResponse> {
     throw new Error("Function not implemented.");
   },
+  signContractTermsHash: function (
+    req: SignContractTermsHashRequest,
+  ): Promise<SignContractTermsHashResponse> {
+    throw new Error("Function not implemented.");
+  },
+  signWithdrawal: function (
+    req: SignWithdrawalRequest,
+  ): Promise<SignWithdrawalResponse> {
+    throw new Error("Function not implemented.");
+  },
+  deriveRefreshSessionV2: function (
+    req: DeriveRefreshSessionRequestV2,
+  ): Promise<DerivedRefreshSessionV2> {
+    throw new Error("Function not implemented.");
+  },
+  rsaVerifyDirect: function (
+    req: RsaVerificationRequest,
+  ): Promise<ValidationResult> {
+    throw new Error("Function not implemented.");
+  },
 };
 
 export type WithArg<X> = X extends (req: infer T) => infer R
@@ -492,11 +572,43 @@ export interface SetupRefreshPlanchetRequest {
   coinNumber: number;
 }
 
+export interface SetupRefreshPlanchetRequestV2 {
+  /**
+   * Refresh planchet secret, derived from the
+   * secret signature made by the old coin.
+   */
+  refreshPlanchetSecret: string;
+  coinNumber: number;
+}
+
 export interface SetupWithdrawalPlanchetRequest {
   secretSeed: string;
   coinNumber: number;
 }
 
+export interface SignWithdrawalRequest {
+  reservePriv: EddsaPrivateKeyString;
+  /**
+   * Total amount without fees.
+   */
+  amount: AmountString;
+
+  /**
+   * Total withdrawal fee.
+   */
+  fee: AmountString;
+
+  coinEvs: CoinEnvelope[];
+
+  denomsPubHashes: HashCodeString[];
+
+  // FIXME: Age restriction stuff
+}
+
+export interface SignWithdrawalResponse {
+  sig: EddsaSignatureString;
+}
+
 export interface SignPurseCreationRequest {
   pursePriv: string;
   purseExpiration: TalerProtocolTimestamp;
@@ -519,8 +631,9 @@ export interface SpendCoinDetails {
   coinPub: string;
   coinPriv: string;
   contribution: AmountString;
+  feeDeposit: AmountString;
   denomPubHash: string;
-  denomSig: UnblindedSignature;
+  denomSig: UnblindedDenominationSignature;
   ageCommitmentProof: AgeCommitmentProof | undefined;
 }
 
@@ -574,8 +687,18 @@ export interface GlobalFeesValidationRequest {
 }
 
 export interface DenominationValidationRequest {
-  denom: DenominationRecord;
+  stampStart: TalerProtocolTimestamp;
+  stampExpireWithdraw: TalerProtocolTimestamp;
+  stampExpireDeposit: TalerProtocolTimestamp;
+  stampExpireLegal: TalerProtocolTimestamp;
+  value: AmountJson;
+  feeWithdraw: AmountJson;
+  feeDeposit: AmountJson;
+  feeRefresh: AmountJson;
+  feeRefund: AmountJson;
+  denomPubHash: string;
   masterPub: string;
+  masterSig: string;
 }
 
 export interface PaymentSignatureValidationRequest {
@@ -600,7 +723,7 @@ export interface WireAccountValidationRequest {
   creditRestrictions?: any[];
 }
 
-export interface EddsaKeypair {
+export interface EddsaKeyPairStrings {
   priv: string;
   pub: string;
 }
@@ -626,6 +749,11 @@ export interface UnblindDenominationSignatureRequest {
   evSig: BlindedDenominationSignature;
 }
 
+export interface UnblindTokenIssueSignatureRequest {
+  slate: SlateUnblindInfo;
+  evSig: TokenIssueBlindSig;
+}
+
 export interface FreshCoinEncoded {
   coinPub: string;
   coinPriv: string;
@@ -730,6 +858,36 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
     };
   },
 
+  async setupRefreshPlanchetV2(
+    tci: TalerCryptoInterfaceR,
+    req: SetupRefreshPlanchetRequestV2,
+  ): Promise<FreshCoinEncoded> {
+    const planchetMasterSecret = decodeCrock(req.refreshPlanchetSecret);
+
+    const coinPriv = kdfKw({
+      ikm: planchetMasterSecret,
+      outputLength: 32,
+      salt: stringToBytes("coin"),
+    });
+
+    const bks = kdfKw({
+      ikm: planchetMasterSecret,
+      outputLength: 32,
+      salt: stringToBytes("bks"),
+    });
+
+    const coinPrivEnc = encodeCrock(coinPriv);
+    const coinPubRes = await tci.eddsaGetPublic(tci, {
+      priv: coinPrivEnc,
+    });
+
+    return {
+      bks: encodeCrock(bks),
+      coinPriv: coinPrivEnc,
+      coinPub: coinPubRes.pub,
+    };
+  },
+
   async setupWithdrawalPlanchet(
     tci: TalerCryptoInterfaceR,
     req: SetupWithdrawalPlanchetRequest,
@@ -801,7 +959,7 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
       const withdrawRequest = buildSigPS(
         TalerSignaturePurpose.WALLET_RESERVE_WITHDRAW,
       )
-        .put(amountToBuffer(amountWithFee))
+        .put(bufferFromAmount(amountWithFee))
         .put(denomPubHash)
         .put(evHash)
         .build();
@@ -830,43 +988,105 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
     }
   },
 
-  async createTipPlanchet(
+  async createSlate(
     tci: TalerCryptoInterfaceR,
-    req: DeriveTipRequest,
-  ): Promise<DerivedTipPlanchet> {
-    if (req.denomPub.cipher !== DenomKeyType.Rsa) {
-      throw Error(`unsupported cipher (${req.denomPub.cipher})`);
+    req: SlateCreationRequest,
+  ): Promise<Slate> {
+    if (req.tokenIssuePub.cipher !== DenomKeyType.Rsa) {
+      throw Error(`unsupported cipher (${req.tokenIssuePub.cipher})`);
     }
-    const fc = await setupTipPlanchet(
-      decodeCrock(req.secretSeed),
-      req.denomPub,
-      req.planchetIndex,
-    );
-    const maybeAch = fc.ageCommitmentProof
-      ? AgeRestriction.hashCommitment(fc.ageCommitmentProof.commitment)
-      : undefined;
-    const denomPub = decodeCrock(req.denomPub.rsa_public_key);
-    const coinPubHash = hashCoinPub(encodeCrock(fc.coinPub), maybeAch);
-    const blindResp = await tci.rsaBlind(tci, {
-      bks: encodeCrock(fc.bks),
-      hm: encodeCrock(coinPubHash),
-      pub: encodeCrock(denomPub),
+
+    // token generation
+    const info = stringToBytes("taler-payment-token-derivation");
+    const saltArrBuf = new ArrayBuffer(4);
+    const salt = new Uint8Array(saltArrBuf);
+    const saltDataView = new DataView(saltArrBuf);
+    saltDataView.setUint32(0, req.outputIndex);
+    const secretSeedDec = decodeCrock(req.secretSeed);
+    const out = kdf(64, secretSeedDec, salt, info);
+    const tokenPriv = out.slice(0, 32);
+    const bks = encodeCrock(out.slice(32, 64));
+    const tokenPrivEnc = encodeCrock(tokenPriv);
+    const tokenPubRes = await tci.eddsaGetPublic(tci, {
+      priv: tokenPrivEnc,
     });
-    const coinEv = {
+
+    // token hashing
+    const tokenPubHash = hash(decodeCrock(tokenPubRes.pub));
+
+    // token blinding
+    const blindResp = await tci.rsaBlind(tci, {
+      bks: bks,
+      hm: encodeCrock(tokenPubHash),
+      pub: req.tokenIssuePub.rsa_pub,
+    });
+
+    // token envelope
+    const tokenEv: TokenEnvelope = {
       cipher: DenomKeyType.Rsa,
       rsa_blinded_planchet: blindResp.blinded,
     };
-    const tipPlanchet: DerivedTipPlanchet = {
-      blindingKey: encodeCrock(fc.bks),
-      coinEv,
-      coinEvHash: encodeCrock(
-        hashCoinEv(coinEv, encodeCrock(hashDenomPub(req.denomPub))),
-      ),
-      coinPriv: encodeCrock(fc.coinPriv),
-      coinPub: encodeCrock(fc.coinPub),
-      ageCommitmentProof: fc.ageCommitmentProof,
+
+    // token issue public key hash
+    const tokenIssuePubHash = hashTokenIssuePub(req.tokenIssuePub);
+    const evHash = hashTokenEv(tokenEv, encodeCrock(tokenIssuePubHash));
+
+    const choice = req.contractTerms.choices[req.outputIndex];
+    const tokenEvs: TokenEnvelope[] = [];
+    for (const output of choice.outputs) {
+      if (output.type !== MerchantContractOutputType.Token) {
+        continue;
+      }
+
+      const slug = output.token_family_slug;
+      const family = req.contractTerms.token_families[slug];
+      tokenEvs.push(
+        ...family.keys.map((key) => {
+          let tokenEv: TokenEnvelope;
+          if (key.cipher === DenomKeyType.Rsa) {
+            tokenEv = {
+              cipher: DenomKeyType.Rsa,
+              rsa_blinded_planchet: key.rsa_pub,
+            };
+          } else {
+            throw Error(`unsupported cipher (${req.tokenIssuePub.cipher})`);
+          }
+          return tokenEv;
+        }),
+      );
+    }
+
+    // wallet data object with envelopes
+    const tokenWalletData: PayWalletData = {
+      choice_index: req.choiceIndex,
+      tokens_evs: tokenEvs,
     };
-    return tipPlanchet;
+
+    return {
+      tokenPub: tokenPubRes.pub,
+      tokenPriv: tokenPrivEnc,
+      tokenIssuePub: req.tokenIssuePub,
+      tokenIssuePubHash: encodeCrock(tokenIssuePubHash),
+      tokenWalletData,
+      tokenEv,
+      tokenEvHash: encodeCrock(evHash),
+      blindingKey: bks,
+    };
+  },
+
+  async signTokenUse(
+    tci: TalerCryptoInterfaceR,
+    req: SignTokenUseRequest,
+  ): Promise<EddsaSigningResult> {
+    const tokenUseRequest = buildSigPS(TalerSignaturePurpose.WALLET_TOKEN_USE)
+      .put(decodeCrock(req.contractTermsHash))
+      .put(decodeCrock(req.walletDataHash))
+      .build();
+
+    return await tci.eddsaSign(tci, {
+      msg: encodeCrock(tokenUseRequest),
+      priv: req.tokenUsePriv,
+    });
   },
 
   async signTrackTransaction(
@@ -971,8 +1191,8 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
       .put(hash(stringToBytes(type + "\0")))
       .put(timestampRoundedToBuffer(wf.startStamp))
       .put(timestampRoundedToBuffer(wf.endStamp))
-      .put(amountToBuffer(wf.wireFee))
-      .put(amountToBuffer(wf.closingFee))
+      .put(bufferFromAmount(wf.wireFee))
+      .put(bufferFromAmount(wf.closingFee))
       .build();
     const sig = decodeCrock(wf.sig);
     const pub = decodeCrock(masterPub);
@@ -992,9 +1212,9 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
       .put(timestampRoundedToBuffer(gf.end_date))
       .put(durationRoundedToBuffer(gf.purse_timeout))
       .put(durationRoundedToBuffer(gf.history_expiration))
-      .put(amountToBuffer(Amounts.parseOrThrow(gf.history_fee)))
-      .put(amountToBuffer(Amounts.parseOrThrow(gf.account_fee)))
-      .put(amountToBuffer(Amounts.parseOrThrow(gf.purse_fee)))
+      .put(bufferFromAmount(Amounts.parseOrThrow(gf.history_fee)))
+      .put(bufferFromAmount(Amounts.parseOrThrow(gf.account_fee)))
+      .put(bufferFromAmount(Amounts.parseOrThrow(gf.purse_fee)))
       .put(bufferForUint32(gf.purse_account_limit))
       .build();
     const sig = decodeCrock(gf.master_sig);
@@ -1009,35 +1229,22 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
     tci: TalerCryptoInterfaceR,
     req: DenominationValidationRequest,
   ): Promise<ValidationResult> {
-    const { masterPub, denom } = req;
-    const value: AmountJson = Amounts.parseOrThrow(denom.value);
+    const value: AmountJson = Amounts.parseOrThrow(req.value);
     const p = buildSigPS(TalerSignaturePurpose.MASTER_DENOMINATION_KEY_VALIDITY)
-      .put(decodeCrock(masterPub))
-      .put(timestampRoundedToBuffer(timestampProtocolFromDb(denom.stampStart)))
-      .put(
-        timestampRoundedToBuffer(
-          timestampProtocolFromDb(denom.stampExpireWithdraw),
-        ),
-      )
-      .put(
-        timestampRoundedToBuffer(
-          timestampProtocolFromDb(denom.stampExpireDeposit),
-        ),
-      )
-      .put(
-        timestampRoundedToBuffer(
-          timestampProtocolFromDb(denom.stampExpireLegal),
-        ),
-      )
-      .put(amountToBuffer(value))
-      .put(amountToBuffer(denom.fees.feeWithdraw))
-      .put(amountToBuffer(denom.fees.feeDeposit))
-      .put(amountToBuffer(denom.fees.feeRefresh))
-      .put(amountToBuffer(denom.fees.feeRefund))
-      .put(decodeCrock(denom.denomPubHash))
+      .put(decodeCrock(req.masterPub))
+      .put(timestampRoundedToBuffer(req.stampStart))
+      .put(timestampRoundedToBuffer(req.stampExpireWithdraw))
+      .put(timestampRoundedToBuffer(req.stampExpireDeposit))
+      .put(timestampRoundedToBuffer(req.stampExpireLegal))
+      .put(bufferFromAmount(value))
+      .put(bufferFromAmount(req.feeWithdraw))
+      .put(bufferFromAmount(req.feeDeposit))
+      .put(bufferFromAmount(req.feeRefresh))
+      .put(bufferFromAmount(req.feeRefund))
+      .put(decodeCrock(req.denomPubHash))
       .build();
-    const sig = decodeCrock(denom.masterSig);
-    const pub = decodeCrock(masterPub);
+    const sig = decodeCrock(req.masterSig);
+    const pub = decodeCrock(req.masterPub);
     const res = eddsaVerify(p, sig, pub);
     return { valid: res };
   },
@@ -1081,7 +1288,9 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
   /**
    * Create a new EdDSA key pair.
    */
-  async createEddsaKeypair(tci: TalerCryptoInterfaceR): Promise<EddsaKeypair> {
+  async createEddsaKeypair(
+    tci: TalerCryptoInterfaceR,
+  ): Promise<EddsaKeyPairStrings> {
     const eddsaPriv = encodeCrock(getRandomBytes(32));
     const eddsaPubRes = await tci.eddsaGetPublic(tci, {
       priv: eddsaPriv,
@@ -1095,7 +1304,7 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
   async eddsaGetPublic(
     tci: TalerCryptoInterfaceR,
     req: EddsaGetPublicRequest,
-  ): Promise<EddsaKeypair> {
+  ): Promise<EddsaKeyPairStrings> {
     return {
       priv: req.priv,
       pub: encodeCrock(eddsaGetPublic(decodeCrock(req.priv))),
@@ -1105,7 +1314,7 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
   async unblindDenominationSignature(
     tci: TalerCryptoInterfaceR,
     req: UnblindDenominationSignatureRequest,
-  ): Promise<UnblindedSignature> {
+  ): Promise<UnblindedDenominationSignature> {
     if (req.evSig.cipher === DenomKeyType.Rsa) {
       if (req.planchet.denomPub.cipher !== DenomKeyType.Rsa) {
         throw new Error(
@@ -1126,6 +1335,29 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
     }
   },
 
+  async unblindTokenIssueSignature(
+    tci: TalerCryptoInterfaceR,
+    req: UnblindTokenIssueSignatureRequest,
+  ): Promise<UnblindedDenominationSignature> {
+    if (req.evSig.cipher === DenomKeyType.Rsa) {
+      if (req.slate.tokenIssuePub.cipher !== DenomKeyType.Rsa) {
+        throw new Error("slate cipher does not match blind signature cipher");
+      }
+      const { sig } = await tci.rsaUnblind(tci, {
+        bk: req.slate.blindingKey,
+        blindedSig: req.evSig.blinded_rsa_signature,
+        pk: req.slate.tokenIssuePub.rsa_pub,
+      });
+
+      return {
+        cipher: DenomKeyType.Rsa,
+        rsa_signature: sig,
+      };
+    } else {
+      throw Error(`unblinding for cipher ${req.evSig.cipher} not implemented`);
+    }
+  },
+
   /**
    * Unblind a blindly signed value.
    */
@@ -1142,7 +1374,8 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
   },
 
   /**
-   * Unblind a blindly signed value.
+   * Verify an RSA signature, where the signature
+   * is over the hash of the message.
    */
   async rsaVerify(
     tci: TalerCryptoInterfaceR,
@@ -1157,6 +1390,22 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
     };
   },
 
+  /**
+   * Verify an RSA signature.
+   */
+  async rsaVerifyDirect(
+    tci: TalerCryptoInterfaceR,
+    req: RsaVerificationRequest,
+  ): Promise<ValidationResult> {
+    return {
+      valid: rsaVerify(
+        decodeCrock(req.hm),
+        decodeCrock(req.sig),
+        decodeCrock(req.pk),
+      ),
+    };
+  },
+
   /**
    * Generate updated coins (to store in the database)
    * and deposit permissions for each given coin.
@@ -1186,8 +1435,9 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
       // All zeros.
       hAgeCommitment = new Uint8Array(32);
     }
-    // FIXME: Actually allow passing user data here!
-    const walletDataHash = new Uint8Array(64);
+    const walletDataHash = depositInfo.walletDataHash
+      ? decodeCrock(depositInfo.walletDataHash)
+      : new Uint8Array(64);
     let d: Uint8Array;
     if (depositInfo.denomKeyType === DenomKeyType.Rsa) {
       d = buildSigPS(TalerSignaturePurpose.WALLET_COIN_DEPOSIT)
@@ -1198,8 +1448,8 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
         .put(decodeCrock(depositInfo.denomPubHash))
         .put(timestampRoundedToBuffer(depositInfo.timestamp))
         .put(timestampRoundedToBuffer(depositInfo.refundDeadline))
-        .put(amountToBuffer(depositInfo.spendAmount))
-        .put(amountToBuffer(depositInfo.feeDeposit))
+        .put(bufferFromAmount(depositInfo.spendAmount))
+        .put(bufferFromAmount(depositInfo.feeDeposit))
         .put(decodeCrock(depositInfo.merchantPub))
         .put(walletDataHash)
         .build();
@@ -1211,6 +1461,13 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
       priv: depositInfo.coinPriv,
     });
 
+    if (debugDumpSigs) {
+      logger.info(
+        `Dumping deposit signature for ct=${depositInfo.contractTermsHash}`,
+      );
+      logger.info(`Deposit signature: ${toHexString(d)}`);
+    }
+
     if (depositInfo.denomKeyType === DenomKeyType.Rsa) {
       const s: CoinDepositPermission = {
         coin_pub: depositInfo.coinPub,
@@ -1241,6 +1498,209 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
     }
   },
 
+  async deriveRefreshSessionV2(
+    tci: TalerCryptoInterfaceR,
+    req: DeriveRefreshSessionRequestV2,
+  ): Promise<DerivedRefreshSessionV2> {
+    const {
+      newCoinDenoms,
+      feeRefresh,
+      kappa,
+      meltCoinDenomPubHash,
+      meltCoinPriv,
+      meltCoinPub,
+      sessionPublicSeed,
+    } = req;
+
+    const currency = Amounts.currencyOf(newCoinDenoms[0].value);
+    let valueWithFee = Amounts.zeroOfCurrency(currency);
+
+    // Melt fee of old coin.
+    valueWithFee = Amounts.add(valueWithFee, feeRefresh).amount;
+
+    // Coin value and withdrawal fees for new coins.
+    for (const ncd of newCoinDenoms) {
+      const t = Amounts.add(ncd.value, ncd.feeWithdraw).amount;
+      valueWithFee = Amounts.add(
+        valueWithFee,
+        Amounts.mult(t, ncd.count).amount,
+      ).amount;
+    }
+
+    const planchetsForGammas: RefreshPlanchetInfo[][] = [];
+
+    const noncesBytes = kdfKw({
+      outputLength: 64 * kappa,
+      salt: stringToBytes("refresh-kappa-nonces"),
+      ikm: decodeCrock(sessionPublicSeed),
+    });
+
+    const numDenoms = newCoinDenoms.length;
+    let numCoins = 0;
+    const coinDenomsHc = createHashContext();
+    for (let i = 0; i < numDenoms; i++) {
+      for (let j = 0; j < newCoinDenoms[i].count; j++) {
+        coinDenomsHc.update(decodeCrock(newCoinDenoms[i].denomPubHash));
+        numCoins++;
+      }
+    }
+    const coinDenomsHash = coinDenomsHc.finish();
+
+    const signatures: EddsaSignatureString[] = [];
+
+    const sessionHc = createHashContext();
+
+    sessionHc.update(decodeCrock(sessionPublicSeed));
+    const blindingSeed = new Uint8Array(32);
+    // For CS, we'd need to also read th real blinding_seed into sessionHc.
+    sessionHc.update(blindingSeed);
+    sessionHc.update(decodeCrock(meltCoinPub));
+    sessionHc.update(bufferFromAmount(valueWithFee));
+
+    if (debugDumpSigs) {
+      logger.info(`hashing blinding seed: ${toHexString(blindingSeed)}`);
+      logger.info(
+        `hashing session seed: ${toHexString(decodeCrock(sessionPublicSeed))}`,
+      );
+      logger.info(
+        `hashing melt coin pub: ${toHexString(decodeCrock(meltCoinPub))}`,
+      );
+      logger.info(
+        `hashing amount: ${toHexString(bufferFromAmount(valueWithFee))}`,
+      );
+    }
+
+    for (let i = 0; i < kappa; i++) {
+      const planchets: RefreshPlanchetInfo[] = [];
+      const nonce = noncesBytes.slice(i * 64, i * 64 + 64);
+
+      const planchetsHc = createHashContext();
+
+      const coinLink = buildSigPS(TalerSignaturePurpose.WALLET_COIN_LINK)
+        .put(nonce)
+        .put(coinDenomsHash)
+        .put(bufferForUint32(i))
+        .build();
+      const sig = await tci.eddsaSign(tci, {
+        msg: encodeCrock(coinLink),
+        priv: meltCoinPriv,
+      });
+      signatures.push(sig.sig);
+
+      const planchetSecretsBytes = kdfKw({
+        outputLength: 32 * numCoins,
+        salt: stringToBytes("refresh-planchet-secret"),
+        ikm: decodeCrock(sig.sig),
+      });
+
+      for (let j = 0; j < numDenoms; j++) {
+        const denomSel = newCoinDenoms[j];
+        for (let k = 0; k < newCoinDenoms[j].count; k++) {
+          const coinIndex = planchets.length;
+          const myPlanchetSecret = planchetSecretsBytes.slice(
+            32 * coinIndex,
+            32 * coinIndex + 32,
+          );
+          let fresh: FreshCoinEncoded = await tci.setupRefreshPlanchetV2(tci, {
+            coinNumber: coinIndex,
+            refreshPlanchetSecret: encodeCrock(myPlanchetSecret),
+          });
+          let newAc: AgeCommitmentProof | undefined = undefined;
+          let newAch: HashCodeString | undefined = undefined;
+          if (req.meltCoinAgeCommitmentProof) {
+            const ageCommitmentSalt = kdfKw({
+              ikm: myPlanchetSecret,
+              outputLength: 64,
+              salt: stringToBytes("age commitment"),
+            });
+            newAc = await AgeRestriction.commitmentDerive(
+              req.meltCoinAgeCommitmentProof,
+              ageCommitmentSalt,
+            );
+            newAch = AgeRestriction.hashCommitment(newAc.commitment);
+          }
+          const coinPubHash = hashCoinPub(fresh.coinPub, newAch);
+          if (denomSel.denomPub.cipher !== DenomKeyType.Rsa) {
+            throw Error("unsupported cipher, can't create refresh session");
+          }
+          const blindResult = await tci.rsaBlind(tci, {
+            bks: fresh.bks,
+            hm: encodeCrock(coinPubHash),
+            pub: denomSel.denomPub.rsa_public_key,
+          });
+          const coinEv: CoinEnvelope = {
+            cipher: DenomKeyType.Rsa,
+            rsa_blinded_planchet: blindResult.blinded,
+          };
+          const coinEvHash = hashCoinEv(
+            coinEv,
+            encodeCrock(hashDenomPub(denomSel.denomPub)),
+          );
+          const planchet: RefreshPlanchetInfo = {
+            blindingKey: fresh.bks,
+            coinEv,
+            coinPriv: fresh.coinPriv,
+            coinPub: fresh.coinPub,
+            coinEvHash: encodeCrock(coinEvHash),
+            maxAge: req.meltCoinMaxAge,
+            ageCommitmentProof: newAc,
+          };
+          planchets.push(planchet);
+          planchetsHc.update(coinEvHash);
+        }
+      }
+
+      const planchetsHash = planchetsHc.finish();
+      if (debugDumpSigs) {
+        logger.info(`planchets hash ${i}: ${toHexString(planchetsHash)}`);
+      }
+      sessionHc.update(planchetsHash);
+
+      planchetsForGammas.push(planchets);
+    }
+
+    const sessionHash = sessionHc.finish();
+    let confirmData: Uint8Array;
+    let hAgeCommitment: Uint8Array;
+    if (req.meltCoinAgeCommitmentProof) {
+      hAgeCommitment = decodeCrock(
+        AgeRestriction.hashCommitment(
+          req.meltCoinAgeCommitmentProof.commitment,
+        ),
+      );
+    } else {
+      hAgeCommitment = new Uint8Array(32);
+    }
+    confirmData = buildSigPS(TalerSignaturePurpose.WALLET_COIN_MELT)
+      .put(sessionHash)
+      .put(decodeCrock(meltCoinDenomPubHash))
+      .put(hAgeCommitment)
+      .put(bufferFromAmount(valueWithFee))
+      .put(bufferFromAmount(feeRefresh))
+      .build();
+
+    if (debugDumpSigs) {
+      logger.info(`melt signature blob:`);
+      logger.info(toHexString(confirmData));
+      logger.info(`valueWithFee: ${Amounts.stringify(valueWithFee)}`);
+      logger.info(`meltFee: ${Amounts.stringify(feeRefresh)}`);
+    }
+
+    const confirmSigResp = await tci.eddsaSign(tci, {
+      msg: encodeCrock(confirmData),
+      priv: meltCoinPriv,
+    });
+
+    return {
+      signatures,
+      planchets: planchetsForGammas,
+      confirmSig: confirmSigResp.sig,
+      hash: encodeCrock(sessionHash),
+      meltCoinPub: meltCoinPub,
+      meltValueWithFee: valueWithFee,
+    };
+  },
+
   async deriveRefreshSession(
     tci: TalerCryptoInterfaceR,
     req: DeriveRefreshSessionRequest,
@@ -1298,7 +1758,7 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
     }
 
     sessionHc.update(decodeCrock(meltCoinPub));
-    sessionHc.update(amountToBuffer(valueWithFee));
+    sessionHc.update(bufferFromAmount(valueWithFee));
 
     for (let i = 0; i < kappa; i++) {
       const planchets: RefreshPlanchetInfo[] = [];
@@ -1378,8 +1838,8 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
       .put(sessionHash)
       .put(decodeCrock(meltCoinDenomPubHash))
       .put(hAgeCommitment)
-      .put(amountToBuffer(valueWithFee))
-      .put(amountToBuffer(meltFee))
+      .put(bufferFromAmount(valueWithFee))
+      .put(bufferFromAmount(meltFee))
       .build();
 
     const confirmSigResp = await tci.eddsaSign(tci, {
@@ -1391,7 +1851,7 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
       confirmSig: confirmSigResp.sig,
       hash: encodeCrock(sessionHash),
       meltCoinPub: meltCoinPub,
-      planchetsForGammas: planchetsForGammas,
+      planchets: planchetsForGammas,
       transferPrivs,
       transferPubs,
       meltValueWithFee: valueWithFee,
@@ -1491,7 +1951,7 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
   ): Promise<EddsaSigningResult> {
     const sigBlob = buildSigPS(TalerSignaturePurpose.WALLET_PURSE_CREATE)
       .put(timestampRoundedToBuffer(req.purseExpiration))
-      .put(amountToBuffer(Amounts.parseOrThrow(req.purseAmount)))
+      .put(bufferFromAmount(Amounts.parseOrThrow(req.purseAmount)))
       .put(decodeCrock(req.hContractTerms))
       .put(decodeCrock(req.mergePub))
       .put(bufferForUint32(req.minAge))
@@ -1517,7 +1977,7 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
         maybeAch = new Uint8Array(32);
       }
       const sigBlob = buildSigPS(TalerSignaturePurpose.WALLET_PURSE_DEPOSIT)
-        .put(amountToBuffer(Amounts.parseOrThrow(c.contribution)))
+        .put(bufferFromAmount(Amounts.parseOrThrow(c.contribution)))
         .put(decodeCrock(c.denomPubHash))
         .put(maybeAch)
         .put(decodeCrock(req.pursePub))
@@ -1634,8 +2094,8 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
       TalerSignaturePurpose.WALLET_ACCOUNT_MERGE,
     )
       .put(timestampRoundedToBuffer(req.purseExpiration))
-      .put(amountToBuffer(Amounts.parseOrThrow(req.purseAmount)))
-      .put(amountToBuffer(Amounts.parseOrThrow(req.purseFee)))
+      .put(bufferFromAmount(Amounts.parseOrThrow(req.purseAmount)))
+      .put(bufferFromAmount(Amounts.parseOrThrow(req.purseFee)))
       .put(decodeCrock(req.contractTermsHash))
       .put(decodeCrock(req.pursePub))
       .put(timestampRoundedToBuffer(req.mergeTimestamp))
@@ -1679,8 +2139,8 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
       TalerSignaturePurpose.WALLET_ACCOUNT_MERGE,
     )
       .put(timestampRoundedToBuffer(req.purseExpiration))
-      .put(amountToBuffer(Amounts.parseOrThrow(req.purseAmount)))
-      .put(amountToBuffer(Amounts.parseOrThrow(req.purseFee)))
+      .put(bufferFromAmount(Amounts.parseOrThrow(req.purseAmount)))
+      .put(bufferFromAmount(Amounts.parseOrThrow(req.purseFee)))
       .put(decodeCrock(req.contractTermsHash))
       .put(decodeCrock(req.pursePub))
       .put(timestampRoundedToBuffer(req.mergeTimestamp))
@@ -1702,7 +2162,7 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
 
     const purseSigBlob = buildSigPS(TalerSignaturePurpose.WALLET_PURSE_CREATE)
       .put(timestampRoundedToBuffer(req.purseExpiration))
-      .put(amountToBuffer(Amounts.parseOrThrow(req.purseAmount)))
+      .put(bufferFromAmount(Amounts.parseOrThrow(req.purseAmount)))
       .put(decodeCrock(req.contractTermsHash))
       .put(decodeCrock(mergePub))
       // FIXME: add age!
@@ -1728,7 +2188,7 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
       .put(decodeCrock(req.contractTermsHash))
       .put(decodeCrock(req.coinPub))
       .put(bufferForUint64(req.rtransactionId))
-      .put(amountToBuffer(req.refundAmount))
+      .put(bufferFromAmount(req.refundAmount))
       .build();
     const refundSigResp = await tci.eddsaSign(tci, {
       msg: encodeCrock(refundSigBlob),
@@ -1792,7 +2252,7 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
     req: SignWalletAccountSetupRequest,
   ): Promise<SignWalletAccountSetupResponse> {
     const sigData = buildSigPS(TalerSignaturePurpose.WALLET_ACCOUNT_SETUP)
-      .put(amountToBuffer(req.threshold))
+      .put(bufferFromAmount(req.threshold))
       .build();
     const sigResp = await tci.eddsaSign(tci, {
       msg: encodeCrock(sigData),
@@ -1815,6 +2275,53 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
       sig: sigResp.sig,
     };
   },
+  async signContractTermsHash(
+    tci: TalerCryptoInterfaceR,
+    req: SignContractTermsHashRequest,
+  ): Promise<SignContractTermsHashResponse> {
+    const sigData = buildSigPS(TalerSignaturePurpose.MERCHANT_CONTRACT)
+      .put(decodeCrock(req.contractTermsHash))
+      .build();
+    const sigRes = await tci.eddsaSign(tci, {
+      msg: encodeCrock(sigData),
+      priv: req.merchantPriv,
+    });
+    return {
+      sig: sigRes.sig,
+    };
+  },
+
+  async signWithdrawal(
+    tci: TalerCryptoInterfaceR,
+    req: SignWithdrawalRequest,
+  ): Promise<SignWithdrawalResponse> {
+    const hc = createHashContext();
+
+    for (let i = 0; i < req.coinEvs.length; i++) {
+      hc.update(hashCoinEv(req.coinEvs[i], req.denomsPubHashes[i]));
+    }
+
+    const hPlanchets = hc.finish();
+
+    // Clause Schnorr blinding seed
+    // Wallet doesn't fully support CS yet.
+    const blindingSeed = new Uint8Array(32);
+
+    const withdrawRequest = buildSigPS(
+      TalerSignaturePurpose.WALLET_RESERVE_WITHDRAW,
+    )
+      .put(bufferFromAmount(req.amount))
+      .put(bufferFromAmount(req.fee))
+      .put(hPlanchets)
+      .put(blindingSeed)
+      .put(bufferForUint32(0)) // max_age_group
+      .put(bufferForUint32(0)) // age mask
+      .build();
+    const sig = eddsaSign(withdrawRequest, decodeCrock(req.reservePriv));
+    return {
+      sig: encodeCrock(sig),
+    };
+  },
 };
 
 export interface EddsaSignRequest {
