@@ -18,8 +18,8 @@
  * Imports
  */
 import { Logger } from "@gnu-taler/taler-util";
-import os from "os";
-import url from "url";
+import os from "node:os";
+import url from "node:url";
 import { nativeCryptoR } from "../cryptoImplementation.js";
 import { CryptoWorkerFactory } from "./crypto-dispatcher.js";
 import { CryptoWorker } from "./cryptoWorkerInterface.js";
