@@ -44,7 +44,7 @@ const logger = new Logger("denomSelection.ts");
 export function selectWithdrawalDenominations(
   amountAvailable: AmountJson,
   denoms: DenominationRecord[],
-  denomselAllowLate: boolean = false,
+  opts: { limitCoins?: number } = {},
 ): DenomSelectionState {
   let remaining = Amounts.copy(amountAvailable);
 
@@ -58,7 +58,7 @@ export function selectWithdrawalDenominations(
   let earliestDepositExpiration: AbsoluteTime | undefined;
   let hasDenomWithAgeRestriction = false;
 
-  denoms = denoms.filter((d) => isWithdrawableDenom(d, denomselAllowLate));
+  denoms = denoms.filter((d) => isWithdrawableDenom(d));
   denoms.sort((d1, d2) => Amounts.cmp(d2.value, d1.value));
 
   if (logger.shouldLogTrace()) {
@@ -67,10 +67,15 @@ export function selectWithdrawalDenominations(
     );
   }
 
+  let totalCount = 0;
+
   for (const d of denoms) {
     const cost = Amounts.add(d.value, d.fees.feeWithdraw).amount;
     const res = Amounts.divmod(remaining, cost);
-    const count = res.quotient;
+    let count = res.quotient;
+    if (opts.limitCoins != null && totalCount + count > opts.limitCoins) {
+      count = opts.limitCoins - totalCount;
+    }
     remaining = Amounts.sub(remaining, Amounts.mult(cost, count).amount).amount;
     if (count > 0) {
       totalCoinValue = Amounts.add(
@@ -123,9 +128,6 @@ export function selectWithdrawalDenominations(
     selectedDenoms,
     totalCoinValue: Amounts.stringify(totalCoinValue),
     totalWithdrawCost: Amounts.stringify(totalWithdrawCost),
-    earliestDepositExpiration: AbsoluteTime.toProtocolTimestamp(
-      earliestDepositExpiration,
-    ),
     hasDenomWithAgeRestriction,
   };
 }
@@ -134,7 +136,6 @@ export function selectForcedWithdrawalDenominations(
   amountAvailable: AmountJson,
   denoms: DenominationRecord[],
   forcedDenomSel: ForcedDenomSel,
-  denomselAllowLate: boolean,
 ): DenomSelectionState {
   const selectedDenoms: {
     count: number;
@@ -146,7 +147,7 @@ export function selectForcedWithdrawalDenominations(
   let earliestDepositExpiration: AbsoluteTime | undefined;
   let hasDenomWithAgeRestriction = false;
 
-  denoms = denoms.filter((d) => isWithdrawableDenom(d, denomselAllowLate));
+  denoms = denoms.filter((d) => isWithdrawableDenom(d));
   denoms.sort((d1, d2) => Amounts.cmp(d2.value, d1.value));
 
   for (const fds of forcedDenomSel.denoms) {
@@ -191,9 +192,6 @@ export function selectForcedWithdrawalDenominations(
     selectedDenoms,
     totalCoinValue: Amounts.stringify(totalCoinValue),
     totalWithdrawCost: Amounts.stringify(totalWithdrawCost),
-    earliestDepositExpiration: AbsoluteTime.toProtocolTimestamp(
-      earliestDepositExpiration,
-    ),
     hasDenomWithAgeRestriction,
   };
 }
