@@ -26,7 +26,6 @@ import {
   CoinInfo,
   convertDepositAmountForAvailableCoins,
   convertWithdrawalAmountFromAvailableCoins,
-  getMaxDepositAmountForAvailableCoins,
 } from "./instructedAmountConversion.js";
 
 function makeCurrencyHelper(currency: string) {
@@ -254,76 +253,6 @@ test("deposit with wire fee raw 2", (t) => {
  *
  */
 
-test("deposit max 35", (t) => {
-  const coinList: Coin[] = [
-    [kudos`2`, 5],
-    [kudos`5`, 5],
-  ];
-  const result = getMaxDepositAmountForAvailableCoins(
-    {
-      list: coinList.map(([v, t]) => defaultFeeConfig(v, t)),
-      exchanges: {
-        "2": {
-          wireFee: kudos`0.00`,
-          purseFee: kudos`0.00`,
-          creditDeadline: AbsoluteTime.never(),
-          debitDeadline: AbsoluteTime.never(),
-        },
-      },
-    },
-    "KUDOS",
-  );
-  t.is(Amounts.stringifyValue(result.raw), "34.9");
-  t.is(Amounts.stringifyValue(result.effective), "35");
-});
-
-test("deposit max 35 with wirefee", (t) => {
-  const coinList: Coin[] = [
-    [kudos`2`, 5],
-    [kudos`5`, 5],
-  ];
-  const result = getMaxDepositAmountForAvailableCoins(
-    {
-      list: coinList.map(([v, t]) => defaultFeeConfig(v, t)),
-      exchanges: {
-        "2": {
-          wireFee: kudos`1`,
-          purseFee: kudos`0.00`,
-          creditDeadline: AbsoluteTime.never(),
-          debitDeadline: AbsoluteTime.never(),
-        },
-      },
-    },
-    "KUDOS",
-  );
-  t.is(Amounts.stringifyValue(result.raw), "33.9");
-  t.is(Amounts.stringifyValue(result.effective), "35");
-});
-
-test("deposit max repeated denom", (t) => {
-  const coinList: Coin[] = [
-    [kudos`2`, 1],
-    [kudos`2`, 1],
-    [kudos`5`, 1],
-  ];
-  const result = getMaxDepositAmountForAvailableCoins(
-    {
-      list: coinList.map(([v, t]) => defaultFeeConfig(v, t)),
-      exchanges: {
-        "2": {
-          wireFee: kudos`0.00`,
-          purseFee: kudos`0.00`,
-          creditDeadline: AbsoluteTime.never(),
-          debitDeadline: AbsoluteTime.never(),
-        },
-      },
-    },
-    "KUDOS",
-  );
-  t.is(Amounts.stringifyValue(result.raw), "8.97");
-  t.is(Amounts.stringifyValue(result.effective), "9");
-});
-
 /**
  * Making a withdrawal with effective amount
  *
@@ -663,42 +592,6 @@ test("demo: withdraw raw 25", (t) => {
   //shows fee = 0.2
 });
 
-test("demo: deposit max after withdraw raw 25", (t) => {
-  const coinList: Coin[] = [
-    [kudos`0.1`, 8],
-    [kudos`1`, 0],
-    [kudos`2`, 2],
-    [kudos`5`, 0],
-    [kudos`10`, 2],
-  ];
-  const result = getMaxDepositAmountForAvailableCoins(
-    {
-      list: coinList.map(([v, t]) => defaultFeeConfig(v, t)),
-      exchanges: {
-        one: {
-          wireFee: kudos`0.01`,
-          purseFee: kudos`0.00`,
-          creditDeadline: AbsoluteTime.never(),
-          debitDeadline: AbsoluteTime.never(),
-        },
-      },
-    },
-    "KUDOS",
-  );
-  t.is(Amounts.stringifyValue(result.effective), "24.8");
-  t.is(Amounts.stringifyValue(result.raw), "24.67");
-
-  // 8 x  0.1
-  // 2 x  0.2
-  // 2 x 10.0
-  // total effective 24.8
-  // deposit fee 12 x 0.01 = 0.12
-  // wire fee 0.01
-  // total raw: 24.8 - 0.13 = 24.67
-
-  // current wallet impl fee 0.14
-});
-
 test("demo: withdraw raw 13", (t) => {
   const coinList: Coin[] = [
     [kudos`0.1`, 0],
@@ -729,39 +622,3 @@ test("demo: withdraw raw 13", (t) => {
   //current wallet impl: hides the left in reserve fee
   //shows fee = 0.2
 });
-
-test("demo: deposit max after withdraw raw 13", (t) => {
-  const coinList: Coin[] = [
-    [kudos`0.1`, 8],
-    [kudos`1`, 0],
-    [kudos`2`, 1],
-    [kudos`5`, 0],
-    [kudos`10`, 1],
-  ];
-  const result = getMaxDepositAmountForAvailableCoins(
-    {
-      list: coinList.map(([v, t]) => defaultFeeConfig(v, t)),
-      exchanges: {
-        one: {
-          wireFee: kudos`0.01`,
-          purseFee: kudos`0.00`,
-          creditDeadline: AbsoluteTime.never(),
-          debitDeadline: AbsoluteTime.never(),
-        },
-      },
-    },
-    "KUDOS",
-  );
-  t.is(Amounts.stringifyValue(result.effective), "12.8");
-  t.is(Amounts.stringifyValue(result.raw), "12.69");
-
-  // 8 x  0.1
-  // 1 x  0.2
-  // 1 x 10.0
-  // total effective 12.8
-  // deposit fee 10 x 0.01 = 0.10
-  // wire fee 0.01
-  // total raw: 12.8 - 0.11 = 12.69
-
-  // current wallet impl fee 0.14
-});
