@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2024 Taler Systems SA
+ (C) 2024-2025 Taler Systems SA
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -33,6 +33,7 @@ import {
   DbAccess,
   DbReadOnlyTransaction,
   DbReadWriteTransaction,
+  StoreMap,
   StoreNames,
 } from "./query.js";
 import { TaskScheduler } from "./shepherd.js";
@@ -44,7 +45,7 @@ export class ObservableTaskScheduler implements TaskScheduler {
   constructor(
     private impl: TaskScheduler,
     private oc: ObservabilityContext,
-  ) {}
+  ) { }
 
   private taskDepCache = new Set<string>();
 
@@ -126,11 +127,11 @@ export function getCallerInfo(up: number = 2): string {
   return identifies.slice(up, up + 2).join("/");
 }
 
-export class ObservableDbAccess<StoreMap> implements DbAccess<StoreMap> {
+export class ObservableDbAccess<Stores extends StoreMap> implements DbAccess<Stores> {
   constructor(
-    private impl: DbAccess<StoreMap>,
+    private impl: DbAccess<Stores>,
     private oc: ObservabilityContext,
-  ) {}
+  ) { }
   idbHandle(): IDBDatabase {
     return this.impl.idbHandle();
   }
@@ -140,7 +141,7 @@ export class ObservableDbAccess<StoreMap> implements DbAccess<StoreMap> {
       label?: string;
     },
     txf: (
-      tx: DbReadWriteTransaction<StoreMap, StoreNames<StoreMap>[]>,
+      tx: DbReadWriteTransaction<Stores, StoreNames<Stores>[]>,
     ) => Promise<T>,
   ): Promise<T> {
     const location = getCallerInfo();
@@ -173,7 +174,7 @@ export class ObservableDbAccess<StoreMap> implements DbAccess<StoreMap> {
       label?: string;
     },
     txf: (
-      tx: DbReadOnlyTransaction<StoreMap, StoreNames<StoreMap>[]>,
+      tx: DbReadOnlyTransaction<Stores, StoreNames<Stores>[]>,
     ) => Promise<T>,
   ): Promise<T> {
     const location = getCallerInfo();
@@ -201,12 +202,12 @@ export class ObservableDbAccess<StoreMap> implements DbAccess<StoreMap> {
     }
   }
 
-  async runReadWriteTx<T, StoreNameArray extends StoreNames<StoreMap>[]>(
+  async runReadWriteTx<T, StoreNameArray extends StoreNames<Stores>[]>(
     opts: {
       storeNames: StoreNameArray;
       label?: string;
     },
-    txf: (tx: DbReadWriteTransaction<StoreMap, StoreNameArray>) => Promise<T>,
+    txf: (tx: DbReadWriteTransaction<Stores, StoreNameArray>) => Promise<T>,
   ): Promise<T> {
     const location = getCallerInfo();
     this.oc.observe({
@@ -233,12 +234,12 @@ export class ObservableDbAccess<StoreMap> implements DbAccess<StoreMap> {
     }
   }
 
-  async runReadOnlyTx<T, StoreNameArray extends StoreNames<StoreMap>[]>(
+  async runReadOnlyTx<T, StoreNameArray extends StoreNames<Stores>[]>(
     opts: {
       storeNames: StoreNameArray;
       label?: string;
     },
-    txf: (tx: DbReadOnlyTransaction<StoreMap, StoreNameArray>) => Promise<T>,
+    txf: (tx: DbReadOnlyTransaction<Stores, StoreNameArray>) => Promise<T>,
   ): Promise<T> {
     const location = getCallerInfo();
     try {
