@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2022-2024 Taler Systems S.A.
+ (C) 2022-2025 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -20,9 +20,6 @@
  * paying for an invoice the wallet received from another wallet.
  */
 
-/**
- * Imports.
- */
 import {
   AcceptPeerPullPaymentResponse,
   Amounts,
@@ -32,17 +29,18 @@ import {
   ExchangePurseDeposits,
   HttpStatusCode,
   Logger,
-  NotificationType,
   ObservabilityEventType,
   PeerContractTerms,
   PreparePeerPullDebitRequest,
   PreparePeerPullDebitResponse,
+  PurseConflict,
   RefreshReason,
+  ScopeType,
   SelectedProspectiveCoin,
   TalerError,
   TalerErrorCode,
+  TalerErrorDetail,
   TalerPreciseTimestamp,
-  TalerProtocolViolationError,
   Transaction,
   TransactionAction,
   TransactionIdStr,
@@ -50,30 +48,24 @@ import {
   TransactionMinorState,
   TransactionState,
   TransactionType,
+  WalletNotification,
   assertUnreachable,
   checkDbInvariant,
   checkLogicInvariant,
-  codecForAny,
-  codecForExchangeGetContractResponse,
   codecForPeerContractTerms,
   decodeCrock,
   eddsaGetPublic,
   encodeCrock,
   getRandomBytes,
   j2s,
+  makeTalerErrorDetail,
   parsePayPullUri,
 } from "@gnu-taler/taler-util";
-import {
-  HttpResponse,
-  readSuccessResponseJsonOrThrow,
-  readTalerErrorResponse,
-} from "@gnu-taler/taler-util/http";
 import { PreviousPayCoins, selectPeerCoins } from "./coinSelection.js";
 import {
   PendingTaskType,
   TaskIdStr,
   TaskRunResult,
-  TaskRunResultType,
   TransactionContext,
   TransitionResultType,
   constructTaskIdentifier,
@@ -85,25 +77,27 @@ import {
   RefreshOperationStatus,
   WalletDbAllStoresReadOnlyTransaction,
   WalletDbReadWriteTransaction,
-  WalletStoresV1,
   timestampPreciseFromDb,
   timestampPreciseToDb,
 } from "./db.js";
+import { getExchangeScopeInfo, getScopeForAllExchanges } from "./exchanges.js";
 import {
-  codecForExchangePurseStatus,
   getTotalPeerPaymentCost,
+  isPurseDeposited,
   queryCoinInfosForSelection,
+  recordCreate,
+  recordDelete,
+  recordTransition,
+  recordTransitionStatus,
+  recordUpdateMeta,
 } from "./pay-peer-common.js";
-import { DbReadWriteTransaction, StoreNames } from "./query.js";
 import { createRefreshGroup } from "./refresh.js";
 import {
   constructTransactionIdentifier,
   isUnsuccessfulTransaction,
-  notifyTransition,
   parseTransactionIdentifier,
 } from "./transactions.js";
-import { WalletExecutionContext } from "./wallet.js";
-import { getScopeForAllExchanges } from "./exchanges.js";
+import { WalletExecutionContext, walletExchangeClient } from "./wallet.js";
 
 const logger = new Logger("pay-peer-pull-debit.ts");
 
@@ -111,13 +105,13 @@ const logger = new Logger("pay-peer-pull-debit.ts");
  * Common context for a peer-pull-debit transaction.
  */
 export class PeerPullDebitTransactionContext implements TransactionContext {
-  wex: WalletExecutionContext;
   readonly transactionId: TransactionIdStr;
   readonly taskId: TaskIdStr;
-  peerPullDebitId: string;
 
-  constructor(wex: WalletExecutionContext, peerPullDebitId: string) {
-    this.wex = wex;
+  constructor(
+    public wex: WalletExecutionContext,
+    public peerPullDebitId: string,
+  ) {
     this.transactionId = constructTransactionIdentifier({
       tag: TransactionType.PeerPullDebit,
       peerPullDebitId,
@@ -126,25 +120,24 @@ export class PeerPullDebitTransactionContext implements TransactionContext {
       tag: PendingTaskType.PeerPullDebit,
       peerPullDebitId,
     });
-    this.peerPullDebitId = peerPullDebitId;
   }
 
-  async updateTransactionMeta(
+  readonly store = "peerPullDebit";
+  readonly recordId = this.peerPullDebitId;
+  readonly recordState = (rec: PeerPullPaymentIncomingRecord) => ({
+    txState: computePeerPullDebitTransactionState(rec),
+    stId: rec.status,
+  });
+  readonly recordMeta = (rec: PeerPullPaymentIncomingRecord) => ({
+    transactionId: this.transactionId,
+    status: rec.status,
+    timestamp: rec.timestampCreated,
+    currency: Amounts.currencyOf(rec.amount),
+    exchanges: [rec.exchangeBaseUrl],
+  });
+  updateTransactionMeta = (
     tx: WalletDbReadWriteTransaction<["peerPullDebit", "transactionsMeta"]>,
-  ): Promise<void> {
-    const ppdRec = await tx.peerPullDebit.get(this.peerPullDebitId);
-    if (!ppdRec) {
-      await tx.transactionsMeta.delete(this.transactionId);
-      return;
-    }
-    await tx.transactionsMeta.put({
-      transactionId: this.transactionId,
-      status: ppdRec.status,
-      timestamp: ppdRec.timestampCreated,
-      currency: Amounts.currencyOf(ppdRec.amount),
-      exchanges: [ppdRec.exchangeBaseUrl],
-    });
-  }
+  ) => recordUpdateMeta(this, tx);
 
   /**
    * Get the full transaction details for the transaction.
@@ -167,6 +160,7 @@ export class PeerPullDebitTransactionContext implements TransactionContext {
     return {
       type: TransactionType.PeerPullDebit,
       txState,
+      stId: pi.status,
       scopes: await getScopeForAllExchanges(tx, [pi.exchangeBaseUrl]),
       txActions: computePeerPullDebitTransactionActions(pi),
       amountEffective: isUnsuccessfulTransaction(txState)
@@ -179,96 +173,63 @@ export class PeerPullDebitTransactionContext implements TransactionContext {
       info: {
         expiration: contractTerms.purse_expiration,
         summary: contractTerms.summary,
+        iconId: contractTerms.icon_id,
       },
+      abortReason: pi.abortReason,
+      failReason: pi.failReason,
       timestamp: timestampPreciseFromDb(pi.timestampCreated),
-      transactionId: constructTransactionIdentifier({
-        tag: TransactionType.PeerPullDebit,
-        peerPullDebitId: pi.peerPullDebitId,
-      }),
+      transactionId: this.transactionId,
       ...(ort?.lastError ? { error: ort.lastError } : {}),
     };
   }
 
   async deleteTransaction(): Promise<void> {
-    const transactionId = this.transactionId;
-    const ws = this.wex;
-    const peerPullDebitId = this.peerPullDebitId;
-    await ws.db.runReadWriteTx(
-      { storeNames: ["peerPullDebit", "tombstones", "transactionsMeta"] },
-      async (tx) => {
-        const debit = await tx.peerPullDebit.get(peerPullDebitId);
-        if (debit) {
-          await tx.peerPullDebit.delete(peerPullDebitId);
-          await this.updateTransactionMeta(tx);
-          await tx.tombstones.put({ id: transactionId });
-        }
-      },
+    const res = await this.wex.db.runReadWriteTx(
+      { storeNames: ["peerPullDebit", "transactionsMeta"] },
+      this.deleteTransactionInTx.bind(this),
     );
+    for (const notif of res.notifs) {
+      this.wex.ws.notify(notif);
+    }
+  }
+
+  async deleteTransactionInTx(
+    tx: WalletDbReadWriteTransaction<["peerPullDebit", "transactionsMeta"]>,
+  ): Promise<{ notifs: WalletNotification[] }> {
+    return recordDelete(this, tx);
   }
 
   async suspendTransaction(): Promise<void> {
-    const taskId = this.taskId;
-    const transactionId = this.transactionId;
-    const wex = this.wex;
-    const peerPullDebitId = this.peerPullDebitId;
-    const transitionInfo = await wex.db.runReadWriteTx(
-      { storeNames: ["peerPullDebit", "transactionsMeta"] },
-      async (tx) => {
-        const pullDebitRec = await tx.peerPullDebit.get(peerPullDebitId);
-        if (!pullDebitRec) {
-          logger.warn(`peer pull debit ${peerPullDebitId} not found`);
-          return;
-        }
-        let newStatus: PeerPullDebitRecordStatus | undefined = undefined;
-        switch (pullDebitRec.status) {
-          case PeerPullDebitRecordStatus.DialogProposed:
-            break;
-          case PeerPullDebitRecordStatus.Done:
-            break;
-          case PeerPullDebitRecordStatus.PendingDeposit:
-            newStatus = PeerPullDebitRecordStatus.SuspendedDeposit;
-            break;
-          case PeerPullDebitRecordStatus.SuspendedDeposit:
-            break;
-          case PeerPullDebitRecordStatus.Aborted:
-            break;
-          case PeerPullDebitRecordStatus.AbortingRefresh:
-            newStatus = PeerPullDebitRecordStatus.SuspendedAbortingRefresh;
-            break;
-          case PeerPullDebitRecordStatus.Failed:
-            break;
-          case PeerPullDebitRecordStatus.SuspendedAbortingRefresh:
-            break;
-          default:
-            assertUnreachable(pullDebitRec.status);
-        }
-        if (newStatus != null) {
-          const oldTxState = computePeerPullDebitTransactionState(pullDebitRec);
-          pullDebitRec.status = newStatus;
-          const newTxState = computePeerPullDebitTransactionState(pullDebitRec);
-          await tx.peerPullDebit.put(pullDebitRec);
-          await this.updateTransactionMeta(tx);
-          return {
-            oldTxState,
-            newTxState,
-          };
-        }
-        return undefined;
-      },
-    );
-    notifyTransition(wex, transactionId, transitionInfo);
-    wex.taskScheduler.stopShepherdTask(taskId);
+    await recordTransition(this, {}, async (rec) => {
+      switch (rec.status) {
+        case PeerPullDebitRecordStatus.DialogProposed:
+        case PeerPullDebitRecordStatus.Done:
+        case PeerPullDebitRecordStatus.SuspendedDeposit:
+        case PeerPullDebitRecordStatus.Aborted:
+        case PeerPullDebitRecordStatus.Failed:
+        case PeerPullDebitRecordStatus.SuspendedAbortingRefresh:
+          return TransitionResultType.Stay;
+        case PeerPullDebitRecordStatus.PendingDeposit:
+          rec.status = PeerPullDebitRecordStatus.SuspendedDeposit;
+          return TransitionResultType.Transition;
+        case PeerPullDebitRecordStatus.AbortingRefresh:
+          rec.status = PeerPullDebitRecordStatus.SuspendedAbortingRefresh;
+          return TransitionResultType.Transition;
+        default:
+          assertUnreachable(rec.status);
+      }
+    });
+    this.wex.taskScheduler.stopShepherdTask(this.taskId);
   }
 
   async resumeTransaction(): Promise<void> {
-    const ctx = this;
-    await ctx.transition(async (pi) => {
-      switch (pi.status) {
+    await recordTransition(this, {}, async (rec) => {
+      switch (rec.status) {
         case PeerPullDebitRecordStatus.SuspendedDeposit:
-          pi.status = PeerPullDebitRecordStatus.PendingDeposit;
+          rec.status = PeerPullDebitRecordStatus.PendingDeposit;
           return TransitionResultType.Transition;
         case PeerPullDebitRecordStatus.SuspendedAbortingRefresh:
-          pi.status = PeerPullDebitRecordStatus.AbortingRefresh;
+          rec.status = PeerPullDebitRecordStatus.AbortingRefresh;
           return TransitionResultType.Transition;
         case PeerPullDebitRecordStatus.Aborted:
         case PeerPullDebitRecordStatus.AbortingRefresh:
@@ -282,16 +243,16 @@ export class PeerPullDebitTransactionContext implements TransactionContext {
     this.wex.taskScheduler.startShepherdTask(this.taskId);
   }
 
-  async failTransaction(): Promise<void> {
-    const ctx = this;
-    await ctx.transition(async (pi) => {
-      switch (pi.status) {
+  async failTransaction(reason?: TalerErrorDetail): Promise<void> {
+    await recordTransition(this, {}, async (rec) => {
+      switch (rec.status) {
         case PeerPullDebitRecordStatus.SuspendedDeposit:
         case PeerPullDebitRecordStatus.PendingDeposit:
         case PeerPullDebitRecordStatus.AbortingRefresh:
         case PeerPullDebitRecordStatus.SuspendedAbortingRefresh:
           // FIXME: Should we also abort the corresponding refresh session?!
-          pi.status = PeerPullDebitRecordStatus.Failed;
+          rec.status = PeerPullDebitRecordStatus.Failed;
+          rec.failReason = reason;
           return TransitionResultType.Transition;
         default:
           return TransitionResultType.Stay;
@@ -300,12 +261,11 @@ export class PeerPullDebitTransactionContext implements TransactionContext {
     this.wex.taskScheduler.stopShepherdTask(this.taskId);
   }
 
-  async abortTransaction(): Promise<void> {
-    const ctx = this;
-    await ctx.transitionExtra(
+  async abortTransaction(reason?: TalerErrorDetail): Promise<void> {
+    await recordTransition(
+      this,
       {
         extraStores: [
-          "coinAvailability",
           "coinAvailability",
           "coinHistory",
           "coins",
@@ -337,7 +297,7 @@ export class PeerPullDebitTransactionContext implements TransactionContext {
         }
 
         const refresh = await createRefreshGroup(
-          ctx.wex,
+          this.wex,
           tx,
           currency,
           coinPubs,
@@ -347,103 +307,36 @@ export class PeerPullDebitTransactionContext implements TransactionContext {
 
         pi.status = PeerPullDebitRecordStatus.AbortingRefresh;
         pi.abortRefreshGroupId = refresh.refreshGroupId;
+        pi.abortReason = reason;
         return TransitionResultType.Transition;
       },
     );
-  }
-
-  async transition(
-    f: (rec: PeerPullPaymentIncomingRecord) => Promise<TransitionResultType>,
-  ): Promise<void> {
-    return this.transitionExtra(
-      {
-        extraStores: [],
-      },
-      f,
-    );
-  }
-
-  async transitionExtra<
-    StoreNameArray extends Array<StoreNames<typeof WalletStoresV1>> = [],
-  >(
-    opts: { extraStores: StoreNameArray },
-    f: (
-      rec: PeerPullPaymentIncomingRecord,
-      tx: DbReadWriteTransaction<
-        typeof WalletStoresV1,
-        ["peerPullDebit", "transactionsMeta", ...StoreNameArray]
-      >,
-    ) => Promise<TransitionResultType>,
-  ): Promise<void> {
-    const wex = this.wex;
-    const extraStores = opts.extraStores ?? [];
-    const transitionInfo = await wex.db.runReadWriteTx(
-      { storeNames: ["peerPullDebit", "transactionsMeta", ...extraStores] },
-      async (tx) => {
-        const pi = await tx.peerPullDebit.get(this.peerPullDebitId);
-        if (!pi) {
-          throw Error("peer pull payment not found anymore");
-        }
-        const oldTxState = computePeerPullDebitTransactionState(pi);
-        const res = await f(pi, tx);
-        switch (res) {
-          case TransitionResultType.Transition: {
-            await tx.peerPullDebit.put(pi);
-            await this.updateTransactionMeta(tx);
-            const newTxState = computePeerPullDebitTransactionState(pi);
-            return {
-              oldTxState,
-              newTxState,
-            };
-          }
-          case TransitionResultType.Delete: {
-            await tx.peerPullDebit.delete(this.peerPullDebitId);
-            await this.updateTransactionMeta(tx);
-            return {
-              oldTxState,
-              newTxState: {
-                major: TransactionMajorState.None,
-              },
-            };
-          }
-          default:
-            return undefined;
-        }
-      },
-    );
-    wex.taskScheduler.stopShepherdTask(this.taskId);
-    notifyTransition(wex, this.transactionId, transitionInfo);
-    wex.taskScheduler.startShepherdTask(this.taskId);
+    this.wex.taskScheduler.stopShepherdTask(this.taskId);
+    this.wex.taskScheduler.startShepherdTask(this.taskId);
   }
 }
 
 async function handlePurseCreationConflict(
   ctx: PeerPullDebitTransactionContext,
   peerPullInc: PeerPullPaymentIncomingRecord,
-  resp: HttpResponse,
+  conflict: PurseConflict,
 ): Promise<TaskRunResult> {
-  const ws = ctx.wex;
-  const errResp = await readTalerErrorResponse(resp);
-  if (errResp.code !== TalerErrorCode.EXCHANGE_GENERIC_INSUFFICIENT_FUNDS) {
+  if (conflict.code !== TalerErrorCode.EXCHANGE_GENERIC_INSUFFICIENT_FUNDS) {
     await ctx.failTransaction();
     return TaskRunResult.finished();
   }
 
-  // FIXME: Properly parse!
-  const brokenCoinPub = (errResp as any).coin_pub;
+  const brokenCoinPub = conflict.coin_pub;
   logger.trace(`excluded broken coin pub=${brokenCoinPub}`);
-
-  if (!brokenCoinPub) {
-    // FIXME: Details!
-    throw new TalerProtocolViolationError();
-  }
-
   const instructedAmount = Amounts.parseOrThrow(peerPullInc.amount);
+  const currency = instructedAmount.currency;
+  const exchangeBaseUrl = peerPullInc.exchangeBaseUrl;
 
   const sel = peerPullInc.coinSel;
-  if (!sel) {
-    throw Error("invalid state (coin selection expected)");
-  }
+  checkDbInvariant(
+    !!sel,
+    `no coin selected for peer pull deposit ${peerPullInc.pursePub}`,
+  );
 
   const repair: PreviousPayCoins = [];
 
@@ -456,9 +349,14 @@ async function handlePurseCreationConflict(
     }
   }
 
-  const coinSelRes = await selectPeerCoins(ws, {
+  const coinSelRes = await selectPeerCoins(ctx.wex, {
     instructedAmount,
     repair,
+    restrictScope: {
+      type: ScopeType.Exchange,
+      currency,
+      url: exchangeBaseUrl,
+    },
   });
 
   switch (coinSelRes.type) {
@@ -478,36 +376,69 @@ async function handlePurseCreationConflict(
   }
 
   const totalAmount = await getTotalPeerPaymentCost(
-    ws,
+    ctx.wex,
     coinSelRes.result.coins,
   );
 
-  await ws.db.runReadWriteTx(
-    { storeNames: ["peerPullDebit", "transactionsMeta"] },
-    async (tx) => {
-      const myPpi = await tx.peerPullDebit.get(peerPullInc.peerPullDebitId);
-      if (!myPpi) {
-        return;
+  await recordTransition(ctx, {}, async (rec) => {
+    switch (rec.status) {
+      case PeerPullDebitRecordStatus.PendingDeposit:
+      case PeerPullDebitRecordStatus.SuspendedDeposit: {
+        const sel = coinSelRes.result;
+        rec.coinSel = {
+          coinPubs: sel.coins.map((x) => x.coinPub),
+          contributions: sel.coins.map((x) => x.contribution),
+          totalCost: Amounts.stringify(totalAmount),
+        };
+        return TransitionResultType.Transition;
       }
-      switch (myPpi.status) {
-        case PeerPullDebitRecordStatus.PendingDeposit:
-        case PeerPullDebitRecordStatus.SuspendedDeposit: {
-          const sel = coinSelRes.result;
-          myPpi.coinSel = {
-            coinPubs: sel.coins.map((x) => x.coinPub),
-            contributions: sel.coins.map((x) => x.contribution),
-            totalCost: Amounts.stringify(totalAmount),
-          };
-          break;
-        }
-        default:
-          return;
-      }
-      await tx.peerPullDebit.put(myPpi);
-      await ctx.updateTransactionMeta(tx);
-    },
+      default:
+        return TransitionResultType.Stay;
+    }
+  });
+  return TaskRunResult.progress();
+}
+
+async function processPeerPullDebitDialogProposed(
+  wex: WalletExecutionContext,
+  pullIni: PeerPullPaymentIncomingRecord,
+): Promise<TaskRunResult> {
+  const ctx = new PeerPullDebitTransactionContext(wex, pullIni.peerPullDebitId);
+  const exchangeClient = walletExchangeClient(pullIni.exchangeBaseUrl, wex);
+  const resp = await exchangeClient.getPurseStatusAtDeposit(
+    pullIni.pursePub,
+    true,
   );
-  return TaskRunResult.backoff();
+
+  switch (resp.case) {
+    case "ok":
+      break;
+    case HttpStatusCode.Gone:
+      // Exchange says that purse doesn't exist anymore => expired!
+      await recordTransitionStatus(
+        ctx,
+        PeerPullDebitRecordStatus.DialogProposed,
+        PeerPullDebitRecordStatus.Aborted,
+      );
+      return TaskRunResult.finished();
+    case HttpStatusCode.NotFound:
+      await ctx.failTransaction(resp.detail);
+      return TaskRunResult.finished();
+    default:
+      assertUnreachable(resp);
+  }
+
+  if (isPurseDeposited(resp.body)) {
+    logger.info("purse completed by another wallet");
+    await recordTransitionStatus(
+      ctx,
+      PeerPullDebitRecordStatus.DialogProposed,
+      PeerPullDebitRecordStatus.Aborted,
+    );
+    return TaskRunResult.finished();
+  }
+
+  return TaskRunResult.longpollReturnedPending();
 }
 
 async function processPeerPullDebitPendingDeposit(
@@ -518,16 +449,22 @@ async function processPeerPullDebitPendingDeposit(
     wex,
     peerPullInc.peerPullDebitId,
   );
+  const { pursePub, coinSel } = peerPullInc;
 
-  const pursePub = peerPullInc.pursePub;
+  const exchangeBaseUrl = peerPullInc.exchangeBaseUrl;
 
-  const coinSel = peerPullInc.coinSel;
-
-  if (!coinSel) {
+  // This can happen when there was a prospective coin selection.
+  if (coinSel == null) {
     const instructedAmount = Amounts.parseOrThrow(peerPullInc.amount);
+    const currency = instructedAmount.currency;
 
     const coinSelRes = await selectPeerCoins(wex, {
       instructedAmount,
+      restrictScope: {
+        type: ScopeType.Exchange,
+        currency,
+        url: exchangeBaseUrl,
+      },
     });
     if (logger.shouldLogTrace()) {
       logger.trace(`selected p2p coins (pull): ${j2s(coinSelRes)}`);
@@ -544,6 +481,8 @@ async function processPeerPullDebitPendingDeposit(
           },
         );
       case "prospective":
+        // Coin selection is *still* only prospective!
+        // FIXME: Really report this as error?
         throw Error("insufficient balance (locked behind refresh)");
       case "success":
         coins = coinSelRes.result.coins;
@@ -552,35 +491,28 @@ async function processPeerPullDebitPendingDeposit(
         assertUnreachable(coinSelRes);
     }
 
-    const peerPullDebitId = peerPullInc.peerPullDebitId;
     const totalAmount = await getTotalPeerPaymentCost(wex, coins);
 
     // FIXME: Missing notification here!
-
-    const transitionDone = await wex.db.runReadWriteTx(
+    const info = await recordTransition(
+      ctx,
       {
-        storeNames: [
+        extraStores: [
           "coinAvailability",
           "coinHistory",
           "coins",
           "denominations",
           "exchanges",
-          "peerPullDebit",
           "refreshGroups",
           "refreshSessions",
-          "transactionsMeta",
         ],
       },
-      async (tx) => {
-        const pi = await tx.peerPullDebit.get(peerPullDebitId);
-        if (!pi) {
-          return false;
-        }
-        if (pi.status !== PeerPullDebitRecordStatus.PendingDeposit) {
-          return false;
-        }
-        if (pi.coinSel) {
-          return false;
+      async (rec, tx) => {
+        if (
+          rec.status !== PeerPullDebitRecordStatus.PendingDeposit ||
+          rec.coinSel != null
+        ) {
+          return TransitionResultType.Stay;
         }
         await spendCoins(wex, tx, {
           transactionId: ctx.transactionId,
@@ -590,33 +522,27 @@ async function processPeerPullDebitPendingDeposit(
           ),
           refreshReason: RefreshReason.PayPeerPull,
         });
-        pi.coinSel = {
+        rec.coinSel = {
           coinPubs: coinSelRes.result.coins.map((x) => x.coinPub),
           contributions: coinSelRes.result.coins.map((x) => x.contribution),
           totalCost: Amounts.stringify(totalAmount),
         };
-        await tx.peerPullDebit.put(pi);
-        await ctx.updateTransactionMeta(tx);
-        return true;
+        return TransitionResultType.Transition;
       },
     );
-    if (transitionDone) {
+    if (info != null) {
       return TaskRunResult.progress();
     } else {
       return TaskRunResult.backoff();
     }
   }
-
-  const purseDepositUrl = new URL(
-    `purses/${pursePub}/deposit`,
-    peerPullInc.exchangeBaseUrl,
-  );
+  const exchangeClient = walletExchangeClient(peerPullInc.exchangeBaseUrl, wex);
 
   // FIXME: We could skip batches that we've already submitted.
 
   const coins = await queryCoinInfosForSelection(wex, coinSel);
 
-  const maxBatchSize = 100;
+  const maxBatchSize = 64;
 
   for (let i = 0; i < coins.length; i += maxBatchSize) {
     const batchSize = Math.min(maxBatchSize, coins.length - i);
@@ -640,48 +566,38 @@ async function processPeerPullDebitPendingDeposit(
     if (logger.shouldLogTrace()) {
       logger.trace(`purse deposit payload: ${j2s(depositPayload)}`);
     }
-
-    const httpResp = await wex.http.fetch(purseDepositUrl.href, {
-      method: "POST",
-      body: depositPayload,
-      cancellationToken: wex.cancellationToken,
-    });
-
-    switch (httpResp.status) {
-      case HttpStatusCode.Ok: {
-        const resp = await readSuccessResponseJsonOrThrow(
-          httpResp,
-          codecForAny(),
-        );
-        logger.trace(`purse deposit response: ${j2s(resp)}`);
+    const resp = await exchangeClient.depositIntoPurse(
+      pursePub,
+      depositPayload,
+    );
+    switch (resp.case) {
+      case "ok":
         continue;
-      }
       case HttpStatusCode.Gone: {
-        await ctx.abortTransaction();
-        return TaskRunResult.backoff();
-      }
-      case HttpStatusCode.Conflict: {
-        return handlePurseCreationConflict(ctx, peerPullInc, httpResp);
-      }
-      default: {
-        const errResp = await readTalerErrorResponse(httpResp);
-        return {
-          type: TaskRunResultType.Error,
-          errorDetail: errResp,
-        };
+        await ctx.abortTransaction(
+          makeTalerErrorDetail(
+            TalerErrorCode.WALLET_PEER_PULL_DEBIT_PURSE_GONE,
+            {},
+          ),
+        );
+        return TaskRunResult.finished();
       }
+      case HttpStatusCode.Conflict:
+        return handlePurseCreationConflict(ctx, peerPullInc, resp.body);
+      case HttpStatusCode.Forbidden:
+      case HttpStatusCode.NotFound:
+        await ctx.failTransaction(resp.detail);
+        return TaskRunResult.finished();
+      default:
+        assertUnreachable(resp);
     }
   }
-
   // All batches succeeded, we can transition!
-
-  await ctx.transition(async (r) => {
-    if (r.status !== PeerPullDebitRecordStatus.PendingDeposit) {
-      return TransitionResultType.Stay;
-    }
-    r.status = PeerPullDebitRecordStatus.Done;
-    return TransitionResultType.Transition;
-  });
+  await recordTransitionStatus(
+    ctx,
+    PeerPullDebitRecordStatus.PendingDeposit,
+    PeerPullDebitRecordStatus.Done,
+  );
   return TaskRunResult.finished();
 }
 
@@ -693,41 +609,32 @@ async function processPeerPullDebitAbortingRefresh(
   const abortRefreshGroupId = peerPullInc.abortRefreshGroupId;
   checkLogicInvariant(!!abortRefreshGroupId);
   const ctx = new PeerPullDebitTransactionContext(wex, peerPullDebitId);
-  const transitionInfo = await wex.db.runReadWriteTx(
-    { storeNames: ["peerPullDebit", "refreshGroups", "transactionsMeta"] },
-    async (tx) => {
+  await recordTransition(
+    ctx,
+    { extraStores: ["refreshGroups"] },
+    async (rec, tx) => {
       const refreshGroup = await tx.refreshGroups.get(abortRefreshGroupId);
-      let newOpState: PeerPullDebitRecordStatus | undefined;
-      if (!refreshGroup) {
+      if (refreshGroup == null) {
         // Maybe it got manually deleted? Means that we should
         // just go into failed.
         logger.warn("no aborting refresh group found for deposit group");
-        newOpState = PeerPullDebitRecordStatus.Failed;
+        rec.status = PeerPullDebitRecordStatus.Failed;
+        return TransitionResultType.Transition;
       } else {
-        if (refreshGroup.operationStatus === RefreshOperationStatus.Finished) {
-          newOpState = PeerPullDebitRecordStatus.Aborted;
-        } else if (
-          refreshGroup.operationStatus === RefreshOperationStatus.Failed
-        ) {
-          newOpState = PeerPullDebitRecordStatus.Failed;
+        switch (refreshGroup.operationStatus) {
+          case RefreshOperationStatus.Finished:
+            rec.status = PeerPullDebitRecordStatus.Aborted;
+            return TransitionResultType.Transition;
+          case RefreshOperationStatus.Failed: {
+            rec.status = PeerPullDebitRecordStatus.Failed;
+            return TransitionResultType.Transition;
+          }
+          default:
+            return TransitionResultType.Stay;
         }
       }
-      if (newOpState) {
-        const newDg = await tx.peerPullDebit.get(peerPullDebitId);
-        if (!newDg) {
-          return;
-        }
-        const oldTxState = computePeerPullDebitTransactionState(newDg);
-        newDg.status = newOpState;
-        const newTxState = computePeerPullDebitTransactionState(newDg);
-        await tx.peerPullDebit.put(newDg);
-        await ctx.updateTransactionMeta(tx);
-        return { oldTxState, newTxState };
-      }
-      return undefined;
     },
   );
-  notifyTransition(wex, ctx.transactionId, transitionInfo);
   // FIXME: Shouldn't this be finished in some cases?!
   return TaskRunResult.backoff();
 }
@@ -742,54 +649,64 @@ export async function processPeerPullDebit(
 
   const peerPullInc = await wex.db.runReadOnlyTx(
     { storeNames: ["peerPullDebit"] },
-    async (tx) => {
-      return tx.peerPullDebit.get(peerPullDebitId);
-    },
+    async (tx) => tx.peerPullDebit.get(peerPullDebitId),
   );
   if (!peerPullInc) {
     throw Error("peer pull debit not found");
   }
 
   switch (peerPullInc.status) {
+    case PeerPullDebitRecordStatus.DialogProposed:
+      return processPeerPullDebitDialogProposed(wex, peerPullInc);
     case PeerPullDebitRecordStatus.PendingDeposit:
-      return await processPeerPullDebitPendingDeposit(wex, peerPullInc);
+      return processPeerPullDebitPendingDeposit(wex, peerPullInc);
     case PeerPullDebitRecordStatus.AbortingRefresh:
-      return await processPeerPullDebitAbortingRefresh(wex, peerPullInc);
+      return processPeerPullDebitAbortingRefresh(wex, peerPullInc);
+    case PeerPullDebitRecordStatus.Done:
+    case PeerPullDebitRecordStatus.Aborted:
+    case PeerPullDebitRecordStatus.Failed:
+    case PeerPullDebitRecordStatus.SuspendedAbortingRefresh:
+    case PeerPullDebitRecordStatus.SuspendedDeposit:
+      return TaskRunResult.finished();
+    default:
+      assertUnreachable(peerPullInc.status);
   }
-  return TaskRunResult.finished();
 }
 
 export async function confirmPeerPullDebit(
   wex: WalletExecutionContext,
   req: ConfirmPeerPullDebitRequest,
 ): Promise<AcceptPeerPullPaymentResponse> {
-  let peerPullDebitId: string;
-  const parsedTx = parseTransactionIdentifier(req.transactionId);
-  if (!parsedTx || parsedTx.tag !== TransactionType.PeerPullDebit) {
+  const parsed = parseTransactionIdentifier(req.transactionId);
+  if (!parsed || parsed.tag !== TransactionType.PeerPullDebit) {
     throw Error("invalid peer-pull-debit transaction identifier");
   }
-  peerPullDebitId = parsedTx.peerPullDebitId;
 
   const peerPullInc = await wex.db.runReadOnlyTx(
     { storeNames: ["peerPullDebit"] },
-    async (tx) => {
-      return tx.peerPullDebit.get(peerPullDebitId);
-    },
+    async (tx) => tx.peerPullDebit.get(parsed.peerPullDebitId),
   );
 
-  if (!peerPullInc) {
+  if (peerPullInc == null) {
     throw Error(
       `can't accept unknown incoming p2p pull payment (${req.transactionId})`,
     );
   }
 
-  const ctx = new PeerPullDebitTransactionContext(wex, peerPullDebitId);
-  const transactionId = ctx.transactionId;
+  const ctx = new PeerPullDebitTransactionContext(wex, parsed.peerPullDebitId);
+
+  const exchangeBaseUrl = peerPullInc.exchangeBaseUrl;
 
   const instructedAmount = Amounts.parseOrThrow(peerPullInc.amount);
+  const currency = instructedAmount.currency;
 
   const coinSelRes = await selectPeerCoins(wex, {
     instructedAmount,
+    restrictScope: {
+      type: ScopeType.Exchange,
+      currency,
+      url: exchangeBaseUrl,
+    },
   });
   if (logger.shouldLogTrace()) {
     logger.trace(`selected p2p coins (pull): ${j2s(coinSelRes)}`);
@@ -817,60 +734,47 @@ export async function confirmPeerPullDebit(
 
   const totalAmount = await getTotalPeerPaymentCost(wex, coins);
 
-  // FIXME: Missing notification here!
-
-  await wex.db.runReadWriteTx(
+  await recordTransition(
+    ctx,
     {
-      storeNames: [
+      extraStores: [
         "coinAvailability",
         "coinHistory",
         "coins",
         "denominations",
         "exchanges",
-        "peerPullDebit",
         "refreshGroups",
         "refreshSessions",
-        "transactionsMeta",
       ],
     },
-    async (tx) => {
-      const pi = await tx.peerPullDebit.get(peerPullDebitId);
-      if (!pi) {
-        throw Error();
-      }
-      if (pi.status !== PeerPullDebitRecordStatus.DialogProposed) {
-        return;
+    async (rec, tx) => {
+      if (rec.status !== PeerPullDebitRecordStatus.DialogProposed) {
+        return TransitionResultType.Stay;
       }
       if (coinSelRes.type == "success") {
         await spendCoins(wex, tx, {
-          transactionId,
+          transactionId: ctx.transactionId,
           coinPubs: coinSelRes.result.coins.map((x) => x.coinPub),
           contributions: coinSelRes.result.coins.map((x) =>
             Amounts.parseOrThrow(x.contribution),
           ),
           refreshReason: RefreshReason.PayPeerPull,
         });
-        pi.coinSel = {
+        rec.coinSel = {
           coinPubs: coinSelRes.result.coins.map((x) => x.coinPub),
           contributions: coinSelRes.result.coins.map((x) => x.contribution),
           totalCost: Amounts.stringify(totalAmount),
         };
       }
-      pi.status = PeerPullDebitRecordStatus.PendingDeposit;
-      await ctx.updateTransactionMeta(tx);
-      await tx.peerPullDebit.put(pi);
+      rec.status = PeerPullDebitRecordStatus.PendingDeposit;
+      return TransitionResultType.Transition;
     },
   );
-
-  wex.ws.notify({
-    type: NotificationType.BalanceChange,
-    hintTransactionId: transactionId,
-  });
-
+  wex.taskScheduler.stopShepherdTask(ctx.taskId);
   wex.taskScheduler.startShepherdTask(ctx.taskId);
 
   return {
-    transactionId,
+    transactionId: ctx.transactionId,
   };
 }
 
@@ -889,7 +793,16 @@ export async function preparePeerPullDebit(
   }
 
   const existing = await wex.db.runReadOnlyTx(
-    { storeNames: ["peerPullDebit", "contractTerms"] },
+    {
+      storeNames: [
+        "peerPullDebit",
+        "contractTerms",
+        "exchangeDetails",
+        "exchanges",
+        "globalCurrencyAuditors",
+        "globalCurrencyExchanges",
+      ],
+    },
     async (tx) => {
       const peerPullDebitRecord =
         await tx.peerPullDebit.indexes.byExchangeAndContractPriv.get([
@@ -905,7 +818,18 @@ export async function preparePeerPullDebit(
       if (!contractTerms) {
         return;
       }
-      return { peerPullDebitRecord, contractTerms };
+      const currency = Amounts.currencyOf(peerPullDebitRecord.amount);
+      const scopeInfo = await getExchangeScopeInfo(
+        tx,
+        peerPullDebitRecord.exchangeBaseUrl,
+        currency,
+      );
+      return {
+        peerPullDebitRecord,
+        contractTerms,
+        scopeInfo,
+        exchangeBaseUrl: peerPullDebitRecord.exchangeBaseUrl,
+      };
     },
   );
 
@@ -915,7 +839,8 @@ export async function preparePeerPullDebit(
       amountRaw: existing.peerPullDebitRecord.amount,
       amountEffective: existing.peerPullDebitRecord.totalCostEstimated,
       contractTerms: existing.contractTerms.contractTermsRaw,
-      peerPullDebitId: existing.peerPullDebitRecord.peerPullDebitId,
+      scopeInfo: existing.scopeInfo,
+      exchangeBaseUrl: existing.exchangeBaseUrl,
       transactionId: constructTransactionIdentifier({
         tag: TransactionType.PeerPullDebit,
         peerPullDebitId: existing.peerPullDebitRecord.peerPullDebitId,
@@ -926,32 +851,47 @@ export async function preparePeerPullDebit(
   const exchangeBaseUrl = uri.exchangeBaseUrl;
   const contractPriv = uri.contractPriv;
   const contractPub = encodeCrock(eddsaGetPublic(decodeCrock(contractPriv)));
+  const exchangeClient = walletExchangeClient(exchangeBaseUrl, wex);
 
-  const getContractUrl = new URL(`contracts/${contractPub}`, exchangeBaseUrl);
-
-  const contractHttpResp = await wex.http.fetch(getContractUrl.href);
-
-  const contractResp = await readSuccessResponseJsonOrThrow(
-    contractHttpResp,
-    codecForExchangeGetContractResponse(),
-  );
-
-  const pursePub = contractResp.purse_pub;
+  const contractResp = await exchangeClient.getContract(contractPub);
+  switch (contractResp.case) {
+    case "ok":
+      break;
+    case HttpStatusCode.NotFound:
+      // FIXME: appropriated error code
+      throw Error("unknown P2P contract");
+    default:
+      assertUnreachable(contractResp);
+  }
+  const pursePub = contractResp.body.purse_pub;
 
   const dec = await wex.cryptoApi.decryptContractForDeposit({
-    ciphertext: contractResp.econtract,
+    ciphertext: contractResp.body.econtract,
     contractPriv: contractPriv,
     pursePub: pursePub,
   });
 
-  const getPurseUrl = new URL(`purses/${pursePub}/merge`, exchangeBaseUrl);
+  const resp = await exchangeClient.getPurseStatusAtMerge(pursePub);
+  switch (resp.case) {
+    case "ok":
+      break;
+    case HttpStatusCode.Gone:
+      throw TalerError.fromDetail(
+        TalerErrorCode.WALLET_PEER_PULL_DEBIT_PURSE_GONE,
+        {},
+      );
+    case HttpStatusCode.NotFound:
+      // FIXME: appropriated error code
+      throw Error("unknown peer pull debit");
+    default:
+      assertUnreachable(resp);
+  }
 
-  const purseHttpResp = await wex.http.fetch(getPurseUrl.href);
-
-  const purseStatus = await readSuccessResponseJsonOrThrow(
-    purseHttpResp,
-    codecForExchangePurseStatus(),
-  );
+  if (isPurseDeposited(resp.body)) {
+    logger.info("purse completed by another wallet");
+    // FIXME: appropriated error code
+    throw Error("peer pull debit already completed");
+  }
 
   const peerPullDebitId = encodeCrock(getRandomBytes(32));
 
@@ -971,9 +911,15 @@ export async function preparePeerPullDebit(
   // FIXME: Why don't we compute the totalCost here?!
 
   const instructedAmount = Amounts.parseOrThrow(contractTerms.amount);
+  const currency = Amounts.currencyOf(instructedAmount);
 
   const coinSelRes = await selectPeerCoins(wex, {
     instructedAmount,
+    restrictScope: {
+      type: ScopeType.Exchange,
+      currency,
+      url: exchangeBaseUrl,
+    },
   });
   if (logger.shouldLogTrace()) {
     logger.trace(`selected p2p coins (pull): ${j2s(coinSelRes)}`);
@@ -1002,15 +948,18 @@ export async function preparePeerPullDebit(
   const totalAmount = await getTotalPeerPaymentCost(wex, coins);
 
   const ctx = new PeerPullDebitTransactionContext(wex, peerPullDebitId);
-
-  await wex.db.runReadWriteTx(
-    { storeNames: ["peerPullDebit", "contractTerms", "transactionsMeta"] },
+  await recordCreate(
+    ctx,
+    {
+      extraStores: ["contractTerms"],
+      label: "create-transaction-peer-pull-credit",
+    },
     async (tx) => {
       await tx.contractTerms.put({
         h: contractTermsHash,
         contractTermsRaw: contractTerms,
       });
-      await tx.peerPullDebit.add({
+      return {
         peerPullDebitId,
         contractPriv: contractPriv,
         exchangeBaseUrl: exchangeBaseUrl,
@@ -1020,21 +969,23 @@ export async function preparePeerPullDebit(
         amount: contractTerms.amount,
         status: PeerPullDebitRecordStatus.DialogProposed,
         totalCostEstimated: Amounts.stringify(totalAmount),
-      });
-      await ctx.updateTransactionMeta(tx);
+      };
     },
   );
+  wex.taskScheduler.startShepherdTask(ctx.taskId);
+
+  const scopeInfo = await wex.db.runAllStoresReadOnlyTx({}, (tx) => {
+    return getExchangeScopeInfo(tx, exchangeBaseUrl, currency);
+  });
 
   return {
     amount: contractTerms.amount,
     amountEffective: Amounts.stringify(totalAmount),
     amountRaw: contractTerms.amount,
     contractTerms: contractTerms,
-    peerPullDebitId,
-    transactionId: constructTransactionIdentifier({
-      tag: TransactionType.PeerPullDebit,
-      peerPullDebitId: peerPullDebitId,
-    }),
+    scopeInfo,
+    exchangeBaseUrl,
+    transactionId: ctx.transactionId,
   };
 }
 
