@@ -1,6 +1,7 @@
 /*
  This file is part of TALER
  (C) 2016 GNUnet e.V.
+ (C) 2025 Taler Systems S.A.
 
  TALER is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -42,6 +43,7 @@ import {
   Logger,
   openPromise,
   safeStringifyException,
+  WalletNotification,
 } from "@gnu-taler/taler-util";
 
 const logger = new Logger("query.ts");
@@ -85,7 +87,10 @@ const logExtra = false;
 
 let idbRequestPromId = 1;
 
-function requestToPromise(req: IDBRequest): Promise<any> {
+function requestToPromise(
+  req: IDBRequest,
+  internalContext: InternalTransactionContext,
+): Promise<any> {
   const myId = idbRequestPromId++;
   if (logExtra) {
     logger.trace(`started db request ${myId}`);
@@ -102,6 +107,14 @@ function requestToPromise(req: IDBRequest): Promise<any> {
       if (logExtra) {
         logger.trace(`finished db request ${myId} with error`);
       }
+      if (internalContext.isAborted) {
+        reject(
+          new TransactionAbortedError(
+            internalContext.abortExn?.message ?? "Aborted",
+          ),
+        );
+        return;
+      }
       if (
         req.error != null &&
         "name" in req.error &&
@@ -110,7 +123,10 @@ function requestToPromise(req: IDBRequest): Promise<any> {
         logger.warn("DB request failed, transaction aborted");
       } else {
         logger.error(`error in DB request: ${req.error}`);
-        logger.error(`Request failed: ${stack}`);
+        if (req.error && "stack" in req.error) {
+          logger.error(`Stack: ${req.error.stack}`);
+        }
+        logger.error(`Request failed: ${stack.stack ?? stack}`);
       }
       reject(req.error);
     };
@@ -313,7 +329,7 @@ export function openDatabase(
           cause: req.error,
         });
       }
-      logger.info(
+      logger.trace(
         `handling upgradeneeded event on ${databaseName} from ${e.oldVersion} to ${e.newVersion}`,
       );
       onUpgradeNeeded(db, e.oldVersion, newVersion, transaction);
@@ -330,7 +346,7 @@ export interface IndexDescriptor {
 }
 
 export interface StoreDescriptor<RecordType> {
-  _dummy: undefined & RecordType;
+  _dummy: RecordType;
   keyPath?: IDBKeyPath | IDBKeyPath[];
   autoIncrement?: boolean;
   /**
@@ -532,47 +548,31 @@ type ValidateKeyPath<T, P> = P extends `${infer PX extends keyof T &
 
 // foo({x: [0,1,2]}, "x.0");
 
-export type StoreNames<StoreMap> = StoreMap extends {
-  [P in keyof StoreMap]: StoreWithIndexes<infer SN1, infer SD1, infer IM1>;
-}
-  ? keyof StoreMap
-  : unknown;
-
+export type StoreMap = { [Store: string]: StoreWithIndexes<any, any, any> };
+export type StoreNames<Stores extends StoreMap> = keyof Stores;
 export type DbReadWriteTransaction<
-  StoreMap,
-  StoresArr extends Array<StoreNames<StoreMap>>,
-> = StoreMap extends {
-  [P in string]: StoreWithIndexes<infer _SN1, infer _SD1, infer _IM1>;
-}
-  ? {
-      [X in StoresArr[number] &
-        keyof StoreMap]: StoreMap[X] extends StoreWithIndexes<
-        infer _StoreName,
-        infer RecordType,
-        infer IndexMap
-      >
-        ? StoreReadWriteAccessor<RecordType, IndexMap>
-        : unknown;
-    }
-  : never;
+  Stores extends StoreMap,
+  StoresArr extends Array<StoreNames<Stores>>,
+> = {
+  [X in StoresArr[number]]: StoreReadWriteAccessor<
+    Stores[X]["store"]["_dummy"],
+    Stores[X]["indexMap"]
+  >;
+} & {
+  notify: (w: WalletNotification) => void;
+};
 
 export type DbReadOnlyTransaction<
-  StoreMap,
-  StoresArr extends Array<StoreNames<StoreMap>>,
-> = StoreMap extends {
-  [P in string]: StoreWithIndexes<infer _SN1, infer _SD1, infer _IM1>;
-}
-  ? {
-      [X in StoresArr[number] &
-        keyof StoreMap]: StoreMap[X] extends StoreWithIndexes<
-        infer _StoreName,
-        infer RecordType,
-        infer IndexMap
-      >
-        ? StoreReadOnlyAccessor<RecordType, IndexMap>
-        : unknown;
-    }
-  : never;
+  Stores extends StoreMap,
+  StoresArr extends Array<StoreNames<Stores>>,
+> = {
+  [X in StoresArr[number]]: StoreReadOnlyAccessor<
+    Stores[X]["store"]["_dummy"],
+    Stores[X]["indexMap"]
+  >;
+} & {
+  notify: (w: WalletNotification) => void;
+};
 
 /**
  * Convert the type of an array to a union of the contents.
@@ -591,22 +591,32 @@ function runTx<Arg, Res>(
   tx: IDBTransaction,
   arg: Arg,
   f: (t: Arg, t2: IDBTransaction) => Promise<Res>,
-  triggerContext: InternalTriggerContext,
-  cancellationToken: CancellationToken,
+  internalContext: InternalTransactionContext,
 ): Promise<Res> {
   // Create stack trace in case we need to to print later where
   // the transaction was started.
   const stack = Error("Failed transaction was started here.");
 
+  const cancellationToken = internalContext.cancellationToken;
+
   const unregisterOnCancelled = cancellationToken.onCancelled(() => {
+    logger.trace("aborting transaction due to cancellation");
+    if (!internalContext.isAborted) {
+      internalContext.isAborted = true;
+      const abortExn = new CancellationToken.CancellationError(
+        cancellationToken.reason,
+      );
+      internalContext.abortExn = abortExn;
+    }
     tx.abort();
   });
 
+  cancellationToken.throwIfCancelled();
+
   return new Promise((resolve, reject) => {
     let funResult: any = undefined;
     let gotFunResult = false;
     let transactionException: any = undefined;
-    let aborted = false;
     tx.oncomplete = () => {
       logger.trace("transaction completed");
       // This is a fatal error: The transaction completed *before*
@@ -619,12 +629,19 @@ function runTx<Arg, Res>(
         const msg =
           "BUG: transaction closed before transaction function returned";
         logger.error(msg);
-        logger.error(`${stack.stack}`);
+        logger.error(`${stack.stack ?? stack}`);
         reject(Error(msg));
       } else {
         resolve(funResult);
       }
-      triggerContext.handleAfterCommit();
+      internalContext.handleAfterCommit();
+      // Notify here.
+      if (
+        internalContext.notifications.length > 0 &&
+        internalContext.applyNotifications
+      ) {
+        internalContext.applyNotifications(internalContext.notifications);
+      }
       unregisterOnCancelled();
     };
     tx.onerror = () => {
@@ -636,7 +653,7 @@ function runTx<Arg, Res>(
         return;
       }
       logger.error("error in transaction");
-      logger.error(`${stack.stack}`);
+      logger.error(`${stack.stack ?? stack}`);
       const txError = tx.error;
       if (txError) {
         reject(txError);
@@ -647,9 +664,10 @@ function runTx<Arg, Res>(
     tx.onabort = () => {
       logger.trace("transaction was aborted");
       if (cancellationToken.isCancelled) {
-        reject(
-          new CancellationToken.CancellationError(cancellationToken.reason),
+        const abortExn = new CancellationToken.CancellationError(
+          cancellationToken.reason,
         );
+        reject(abortExn);
         return;
       }
       let msg: string;
@@ -660,11 +678,13 @@ function runTx<Arg, Res>(
       } else {
         msg = "Transaction aborted (no DB error)";
       }
-      aborted = true;
+      const abortExn = new TransactionAbortedError(msg);
+      internalContext.isAborted = true;
+      internalContext.abortExn = abortExn;
       unregisterOnCancelled();
       logger.error(msg);
-      logger.error(`${stack.stack}`);
-      reject(new TransactionAbortedError(msg));
+      logger.error(`${stack.stack ?? stack}`);
+      reject(abortExn);
     };
     const resP = Promise.resolve().then(() => f(arg, tx));
     resP
@@ -674,6 +694,10 @@ function runTx<Arg, Res>(
         funResult = result;
       })
       .catch((e) => {
+        if (cancellationToken.isCancelled) {
+          logger.trace("Ignoring failed transaction due to cancellation.");
+          return;
+        }
         if (e == TransactionAbort) {
           logger.trace("aborting transaction");
           tx.abort();
@@ -682,7 +706,7 @@ function runTx<Arg, Res>(
         } else {
           transactionException = e;
           logger.error(`Transaction failed: ${safeStringifyException(e)}`);
-          logger.error(`${stack}`);
+          logger.error(`${stack.stack ?? stack}`);
           tx.abort();
         }
       })
@@ -692,85 +716,24 @@ function runTx<Arg, Res>(
   });
 }
 
-function makeReadContext(
+/**
+ * Create a transaction handle that will be passed
+ * to the main handler for the transaction.
+ */
+function makeTxClientContext(
   tx: IDBTransaction,
   storePick: { [n: string]: StoreWithIndexes<any, any, any> },
-  triggerContext: InternalTriggerContext,
+  internalContext: InternalTransactionContext,
 ): any {
-  const ctx: { [s: string]: StoreReadOnlyAccessor<any, any> } = {};
-  for (const storeAlias in storePick) {
-    const indexes: { [s: string]: IndexReadOnlyAccessor<any> } = {};
-    const swi = storePick[storeAlias];
-    const storeName = swi.storeName;
-    for (const indexAlias in storePick[storeAlias].indexMap) {
-      const indexDescriptor: IndexDescriptor =
-        storePick[storeAlias].indexMap[indexAlias];
-      const indexName = indexDescriptor.name;
-      indexes[indexAlias] = {
-        get(key) {
-          triggerContext.storesAccessed.add(storeName);
-          const req = tx.objectStore(storeName).index(indexName).get(key);
-          return requestToPromise(req);
-        },
-        iter(query) {
-          triggerContext.storesAccessed.add(storeName);
-          const req = tx
-            .objectStore(storeName)
-            .index(indexName)
-            .openCursor(query);
-          return new ResultStream<any>(req);
-        },
-        getAll(query, count) {
-          triggerContext.storesAccessed.add(storeName);
-          const req = tx
-            .objectStore(storeName)
-            .index(indexName)
-            .getAll(query, count);
-          return requestToPromise(req);
-        },
-        getAllKeys(query, count) {
-          triggerContext.storesAccessed.add(storeName);
-          const req = tx
-            .objectStore(storeName)
-            .index(indexName)
-            .getAllKeys(query, count);
-          return requestToPromise(req);
-        },
-        count(query) {
-          triggerContext.storesAccessed.add(storeName);
-          const req = tx.objectStore(storeName).index(indexName).count(query);
-          return requestToPromise(req);
-        },
-      };
-    }
-    ctx[storeAlias] = {
-      indexes,
-      get(key) {
-        triggerContext.storesAccessed.add(storeName);
-        const req = tx.objectStore(storeName).get(key);
-        return requestToPromise(req);
-      },
-      getAll(query, count) {
-        triggerContext.storesAccessed.add(storeName);
-        const req = tx.objectStore(storeName).getAll(query, count);
-        return requestToPromise(req);
-      },
-      iter(query) {
-        triggerContext.storesAccessed.add(storeName);
-        const req = tx.objectStore(storeName).openCursor(query);
-        return new ResultStream<any>(req);
-      },
-    };
-  }
-  return ctx;
-}
-
-function makeWriteContext(
-  tx: IDBTransaction,
-  storePick: { [n: string]: StoreWithIndexes<any, any, any> },
-  triggerContext: InternalTriggerContext,
-): any {
-  const ctx: { [s: string]: StoreReadWriteAccessor<any, any> } = {};
+  const ctx: {
+    [s: string]:
+      | StoreReadWriteAccessor<any, any>
+      | ((notif: WalletNotification) => void);
+  } = {
+    notify(notif: WalletNotification): void {
+      internalContext.scheduleNotification(notif);
+    },
+  };
   for (const storeAlias in storePick) {
     const indexes: { [s: string]: IndexReadWriteAccessor<any> } = {};
     const swi = storePick[storeAlias];
@@ -781,12 +744,14 @@ function makeWriteContext(
       const indexName = indexDescriptor.name;
       indexes[indexAlias] = {
         get(key) {
-          triggerContext.storesAccessed.add(storeName);
+          internalContext.throwIfInactive();
+          internalContext.storesAccessed.add(storeName);
           const req = tx.objectStore(storeName).index(indexName).get(key);
-          return requestToPromise(req);
+          return requestToPromise(req, internalContext);
         },
         iter(query) {
-          triggerContext.storesAccessed.add(storeName);
+          internalContext.throwIfInactive();
+          internalContext.storesAccessed.add(storeName);
           const req = tx
             .objectStore(storeName)
             .index(indexName)
@@ -794,68 +759,86 @@ function makeWriteContext(
           return new ResultStream<any>(req);
         },
         getAll(query, count) {
-          triggerContext.storesAccessed.add(storeName);
+          internalContext.throwIfInactive();
+          internalContext.storesAccessed.add(storeName);
           const req = tx
             .objectStore(storeName)
             .index(indexName)
             .getAll(query, count);
-          return requestToPromise(req);
+          return requestToPromise(req, internalContext);
         },
         getAllKeys(query, count) {
-          triggerContext.storesAccessed.add(storeName);
+          internalContext.throwIfInactive();
+          internalContext.storesAccessed.add(storeName);
           const req = tx
             .objectStore(storeName)
             .index(indexName)
             .getAllKeys(query, count);
-          return requestToPromise(req);
+          return requestToPromise(req, internalContext);
         },
         count(query) {
-          triggerContext.storesAccessed.add(storeName);
+          internalContext.throwIfInactive();
+          internalContext.storesAccessed.add(storeName);
           const req = tx.objectStore(storeName).index(indexName).count(query);
-          return requestToPromise(req);
+          return requestToPromise(req, internalContext);
         },
       };
     }
     ctx[storeAlias] = {
       indexes,
       get(key) {
-        triggerContext.storesAccessed.add(storeName);
+        internalContext.throwIfInactive();
+        internalContext.storesAccessed.add(storeName);
         const req = tx.objectStore(storeName).get(key);
-        return requestToPromise(req);
+        return requestToPromise(req, internalContext);
       },
       getAll(query, count) {
-        triggerContext.storesAccessed.add(storeName);
+        internalContext.throwIfInactive();
+        internalContext.storesAccessed.add(storeName);
         const req = tx.objectStore(storeName).getAll(query, count);
-        return requestToPromise(req);
+        return requestToPromise(req, internalContext);
       },
       iter(query) {
-        triggerContext.storesAccessed.add(storeName);
+        internalContext.throwIfInactive();
+        internalContext.storesAccessed.add(storeName);
         const req = tx.objectStore(storeName).openCursor(query);
         return new ResultStream<any>(req);
       },
       async add(r, k) {
-        triggerContext.storesAccessed.add(storeName);
-        triggerContext.storesModified.add(storeName);
+        internalContext.throwIfInactive();
+        if (!internalContext.allowWrite) {
+          throw Error("attempting write in a read-only transaction");
+        }
+        internalContext.storesAccessed.add(storeName);
+        internalContext.storesModified.add(storeName);
         const req = tx.objectStore(storeName).add(r, k);
-        const key = await requestToPromise(req);
+        const key = await requestToPromise(req, internalContext);
         return {
           key: key,
         };
       },
       async put(r, k) {
-        triggerContext.storesAccessed.add(storeName);
-        triggerContext.storesModified.add(storeName);
+        internalContext.throwIfInactive();
+        if (!internalContext.allowWrite) {
+          throw Error("attempting write in a read-only transaction");
+        }
+        internalContext.storesAccessed.add(storeName);
+        internalContext.storesModified.add(storeName);
         const req = tx.objectStore(storeName).put(r, k);
-        const key = await requestToPromise(req);
+        const key = await requestToPromise(req, internalContext);
         return {
           key: key,
         };
       },
       delete(k) {
-        triggerContext.storesAccessed.add(storeName);
-        triggerContext.storesModified.add(storeName);
+        internalContext.throwIfInactive();
+        if (!internalContext.allowWrite) {
+          throw Error("attempting write in a read-only transaction");
+        }
+        internalContext.storesAccessed.add(storeName);
+        internalContext.storesModified.add(storeName);
         const req = tx.objectStore(storeName).delete(k);
-        return requestToPromise(req);
+        return requestToPromise(req, internalContext);
       },
     };
   }
@@ -865,7 +848,7 @@ function makeWriteContext(
 /**
  * Handle for typed access to a database.
  */
-export interface DbAccess<StoreMap> {
+export interface DbAccess<Stores extends StoreMap> {
   /**
    * The underlying IndexedDB database handle.
    *
@@ -887,7 +870,7 @@ export interface DbAccess<StoreMap> {
       label?: string;
     },
     txf: (
-      tx: DbReadWriteTransaction<StoreMap, Array<StoreNames<StoreMap>>>,
+      tx: DbReadWriteTransaction<Stores, Array<StoreNames<Stores>>>,
     ) => Promise<T>,
   ): Promise<T>;
 
@@ -904,7 +887,7 @@ export interface DbAccess<StoreMap> {
       label?: string;
     },
     txf: (
-      tx: DbReadOnlyTransaction<StoreMap, Array<StoreNames<StoreMap>>>,
+      tx: DbReadOnlyTransaction<Stores, Array<StoreNames<Stores>>>,
     ) => Promise<T>,
   ): Promise<T>;
 
@@ -916,12 +899,12 @@ export interface DbAccess<StoreMap> {
    * Waiting for macrotasks results in an autocommit and
    * a subsequent exception thrown by this function.
    */
-  runReadWriteTx<T, StoreNameArray extends Array<StoreNames<StoreMap>>>(
+  runReadWriteTx<T, StoreNameArray extends Array<StoreNames<Stores>>>(
     opts: {
       storeNames: StoreNameArray;
       label?: string;
     },
-    txf: (tx: DbReadWriteTransaction<StoreMap, StoreNameArray>) => Promise<T>,
+    txf: (tx: DbReadWriteTransaction<Stores, StoreNameArray>) => Promise<T>,
   ): Promise<T>;
 
   /**
@@ -932,12 +915,12 @@ export interface DbAccess<StoreMap> {
    * Waiting for macrotasks results in an autocommit and
    * a subsequent exception thrown by this function.
    */
-  runReadOnlyTx<T, StoreNameArray extends Array<StoreNames<StoreMap>>>(
+  runReadOnlyTx<T, StoreNameArray extends Array<StoreNames<Stores>>>(
     opts: {
       storeNames: StoreNameArray;
       label?: string;
     },
-    txf: (tx: DbReadOnlyTransaction<StoreMap, StoreNameArray>) => Promise<T>,
+    txf: (tx: DbReadOnlyTransaction<Stores, StoreNameArray>) => Promise<T>,
   ): Promise<T>;
 }
 
@@ -959,17 +942,32 @@ export interface TriggerSpec {
   // beforeCommit<State>? (tx: Transaction, s: State | undefined) => Promise<void>;
 }
 
-class InternalTriggerContext {
+/**
+ * Additional state we store for every IndexedDB transaction opened
+ * via the query helper.
+ */
+class InternalTransactionContext {
+  isAborted = false;
   storesScope: Set<string>;
   storesAccessed: Set<string> = new Set();
   storesModified: Set<string> = new Set();
+  allowWrite: boolean;
+  abortExn: TransactionAbortedError | undefined;
+  notifications: WalletNotification[] = [];
 
   constructor(
-    private triggerSpec: TriggerSpec,
-    private mode: IDBTransactionMode,
-    scope: string[],
+    private readonly triggerSpec: TriggerSpec,
+    private readonly mode: IDBTransactionMode,
+    readonly scope: string[],
+    public readonly cancellationToken: CancellationToken,
+    public readonly applyNotifications?: (notifs: WalletNotification[]) => void,
   ) {
     this.storesScope = new Set(scope);
+    this.allowWrite = mode === "readwrite" || mode === "versionchange";
+  }
+
+  scheduleNotification(notif: WalletNotification): void {
+    this.notifications.push(notif);
   }
 
   handleAfterCommit() {
@@ -982,6 +980,13 @@ class InternalTriggerContext {
       });
     }
   }
+
+  throwIfInactive() {
+    this.cancellationToken.throwIfCancelled();
+    if (this.isAborted) {
+      throw this.abortExn;
+    }
+  }
 }
 
 /**
@@ -989,12 +994,13 @@ class InternalTriggerContext {
  *
  * A store map is the metadata that describes the store.
  */
-export class DbAccessImpl<StoreMap> implements DbAccess<StoreMap> {
+export class DbAccessImpl<Stores extends StoreMap> implements DbAccess<Stores> {
   constructor(
     private db: IDBDatabase,
-    private stores: StoreMap,
+    private stores: Stores,
     private triggers: TriggerSpec = {},
     private cancellationToken: CancellationToken,
+    private applyNotifications?: (notifs: WalletNotification[]) => void,
   ) {}
 
   idbHandle(): IDBDatabase {
@@ -1006,7 +1012,7 @@ export class DbAccessImpl<StoreMap> implements DbAccess<StoreMap> {
       label?: string;
     },
     txf: (
-      tx: DbReadWriteTransaction<StoreMap, Array<StoreNames<StoreMap>>>,
+      tx: DbReadWriteTransaction<Stores, Array<StoreNames<Stores>>>,
     ) => Promise<T>,
   ): Promise<T> {
     this.cancellationToken.throwIfCancelled();
@@ -1019,20 +1025,20 @@ export class DbAccessImpl<StoreMap> implements DbAccess<StoreMap> {
       accessibleStores[swi.storeName] = swi;
     }
     const mode = "readwrite";
-    const triggerContext = new InternalTriggerContext(
+    const triggerContext = new InternalTransactionContext(
       this.triggers,
       mode,
       strStoreNames,
+      this.cancellationToken,
+      this.applyNotifications,
     );
     const tx = this.db.transaction(strStoreNames, mode);
-    const writeContext = makeWriteContext(tx, accessibleStores, triggerContext);
-    return await runTx(
+    const writeContext = makeTxClientContext(
       tx,
-      writeContext,
-      txf,
+      accessibleStores,
       triggerContext,
-      this.cancellationToken,
     );
+    return await runTx(tx, writeContext, txf, triggerContext);
   }
 
   async runAllStoresReadOnlyTx<T>(
@@ -1040,7 +1046,7 @@ export class DbAccessImpl<StoreMap> implements DbAccess<StoreMap> {
       label?: string;
     },
     txf: (
-      tx: DbReadOnlyTransaction<StoreMap, Array<StoreNames<StoreMap>>>,
+      tx: DbReadOnlyTransaction<Stores, Array<StoreNames<Stores>>>,
     ) => Promise<T>,
   ): Promise<T> {
     this.cancellationToken.throwIfCancelled();
@@ -1053,28 +1059,28 @@ export class DbAccessImpl<StoreMap> implements DbAccess<StoreMap> {
       accessibleStores[swi.storeName] = swi;
     }
     const mode = "readonly";
-    const triggerContext = new InternalTriggerContext(
+    const internalContext = new InternalTransactionContext(
       this.triggers,
       mode,
       strStoreNames,
+      this.cancellationToken,
+      this.applyNotifications,
     );
     const tx = this.db.transaction(strStoreNames, mode);
-    const writeContext = makeReadContext(tx, accessibleStores, triggerContext);
-    const res = await runTx(
+    const writeContext = makeTxClientContext(
       tx,
-      writeContext,
-      txf,
-      triggerContext,
-      this.cancellationToken,
+      accessibleStores,
+      internalContext,
     );
+    const res = await runTx(tx, writeContext, txf, internalContext);
     return res;
   }
 
-  async runReadWriteTx<T, StoreNameArray extends Array<StoreNames<StoreMap>>>(
+  async runReadWriteTx<T, StoreNameArray extends Array<StoreNames<Stores>>>(
     opts: {
       storeNames: StoreNameArray;
     },
-    txf: (tx: DbReadWriteTransaction<StoreMap, StoreNameArray>) => Promise<T>,
+    txf: (tx: DbReadWriteTransaction<Stores, StoreNameArray>) => Promise<T>,
   ): Promise<T> {
     this.cancellationToken.throwIfCancelled();
     const accessibleStores: { [x: string]: StoreWithIndexes<any, any, any> } =
@@ -1086,28 +1092,28 @@ export class DbAccessImpl<StoreMap> implements DbAccess<StoreMap> {
       accessibleStores[swi.storeName] = swi;
     }
     const mode = "readwrite";
-    const triggerContext = new InternalTriggerContext(
+    const triggerContext = new InternalTransactionContext(
       this.triggers,
       mode,
       strStoreNames,
+      this.cancellationToken,
+      this.applyNotifications,
     );
     const tx = this.db.transaction(strStoreNames, mode);
-    const writeContext = makeWriteContext(tx, accessibleStores, triggerContext);
-    const res = await runTx(
+    const writeContext = makeTxClientContext(
       tx,
-      writeContext,
-      txf,
+      accessibleStores,
       triggerContext,
-      this.cancellationToken,
     );
+    const res = await runTx(tx, writeContext, txf, triggerContext);
     return res;
   }
 
-  async runReadOnlyTx<T, StoreNameArray extends Array<StoreNames<StoreMap>>>(
+  async runReadOnlyTx<T, StoreNameArray extends Array<StoreNames<Stores>>>(
     opts: {
       storeNames: StoreNameArray;
     },
-    txf: (tx: DbReadOnlyTransaction<StoreMap, StoreNameArray>) => Promise<T>,
+    txf: (tx: DbReadOnlyTransaction<Stores, StoreNameArray>) => Promise<T>,
   ): Promise<T> {
     this.cancellationToken.throwIfCancelled();
     const accessibleStores: { [x: string]: StoreWithIndexes<any, any, any> } =
@@ -1119,20 +1125,20 @@ export class DbAccessImpl<StoreMap> implements DbAccess<StoreMap> {
       accessibleStores[swi.storeName] = swi;
     }
     const mode = "readonly";
-    const triggerContext = new InternalTriggerContext(
+    const triggerContext = new InternalTransactionContext(
       this.triggers,
       mode,
       strStoreNames,
+      this.cancellationToken,
+      this.applyNotifications,
     );
     const tx = this.db.transaction(strStoreNames, mode);
-    const readContext = makeReadContext(tx, accessibleStores, triggerContext);
-    const res = await runTx(
+    const readContext = makeTxClientContext(
       tx,
-      readContext,
-      txf,
+      accessibleStores,
       triggerContext,
-      this.cancellationToken,
     );
+    const res = await runTx(tx, readContext, txf, triggerContext);
     return res;
   }
 }
