@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2019-2020 Taler Systems SA
+ (C) 2019-2025 Taler Systems SA
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -34,6 +34,7 @@ import {
   TransactionIdStr,
   TransactionType,
   URL,
+  WalletNotification,
   checkDbInvariant,
   codecForRecoupConfirmation,
   codecForReserveStatus,
@@ -47,6 +48,7 @@ import {
   TaskIdStr,
   TaskRunResult,
   TransactionContext,
+  cancelableFetch,
   constructTaskIdentifier,
 } from "./common.js";
 import {
@@ -67,7 +69,7 @@ import { constructTransactionIdentifier } from "./transactions.js";
 import { WalletExecutionContext, getDenomInfo } from "./wallet.js";
 import { internalCreateWithdrawalGroup } from "./withdraw.js";
 
-export const logger = new Logger("operations/recoup.ts");
+const logger = new Logger("operations/recoup.ts");
 
 /**
  * Store a recoup group record in the database after marking
@@ -174,7 +176,7 @@ async function recoupRefreshCoin(
   );
   logger.trace(`making recoup request for ${coin.coinPub}`);
 
-  const resp = await wex.http.fetch(reqUrl.href, {
+  const resp = await cancelableFetch(wex, reqUrl, {
     method: "POST",
     body: recoupRequest,
   });
@@ -284,7 +286,7 @@ export async function recoupWithdrawCoin(
   });
   const reqUrl = new URL(`/coins/${coin.coinPub}/recoup`, coin.exchangeBaseUrl);
   logger.trace(`requesting recoup via ${reqUrl.href}`);
-  const resp = await wex.http.fetch(reqUrl.href, {
+  const resp = await cancelableFetch(wex, reqUrl, {
     method: "POST",
     body: recoupRequest,
   });
@@ -411,7 +413,7 @@ export async function processRecoupGroup(
     );
     logger.info(`querying reserve status for recoup via ${reserveUrl}`);
 
-    const resp = await wex.http.fetch(reserveUrl.href);
+    const resp = await cancelableFetch(wex, reserveUrl);
 
     const result = await readSuccessResponseJsonOrThrow(
       resp,
@@ -532,8 +534,38 @@ export class RecoupTransactionContext implements TransactionContext {
     throw new Error("Method not implemented.");
   }
 
-  deleteTransaction(): Promise<void> {
-    throw new Error("Method not implemented.");
+  async deleteTransaction(): Promise<void> {
+    const res = await this.wex.db.runReadWriteTx(
+      {
+        storeNames: [
+          "recoupGroups",
+          "tombstones",
+          "exchanges",
+          "transactionsMeta",
+        ],
+      },
+      async (tx) => {
+        return this.deleteTransactionInTx(tx);
+      },
+    );
+    for (const notif of res.notifs) {
+      this.wex.ws.notify(notif);
+    }
+  }
+
+  async deleteTransactionInTx(
+    tx: WalletDbReadWriteTransaction<
+      ["recoupGroups", "tombstones", "exchanges", "transactionsMeta"]
+    >,
+  ): Promise<{ notifs: WalletNotification[] }> {
+    const notifs: WalletNotification[] = [];
+    const rec = await tx.recoupGroups.get(this.recoupGroupId);
+    if (!rec) {
+      return { notifs };
+    }
+    await tx.recoupGroups.delete(this.recoupGroupId);
+    await this.updateTransactionMeta(tx);
+    return { notifs };
   }
 
   lookupFullTransaction(
