@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2020 Taler Systems S.A.
+ (C) 2020-2025 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -42,11 +42,14 @@ import {
   parsePaytoUri,
   PreparePayResultType,
   TalerCorebankApiClient,
+  TestingWaitTransactionRequest,
   TestPayArgs,
   TestPayResult,
+  TransactionIdStr,
   TransactionMajorState,
   TransactionMinorState,
   TransactionState,
+  TransactionStatePattern,
   TransactionType,
   URL,
   WithdrawTestBalanceRequest,
@@ -58,10 +61,7 @@ import {
 import { getBalances } from "./balance.js";
 import { genericWaitForState } from "./common.js";
 import { createDepositGroup } from "./deposits.js";
-import {
-  acceptExchangeTermsOfService,
-  fetchFreshExchange,
-} from "./exchanges.js";
+import { fetchFreshExchange } from "./exchanges.js";
 import {
   confirmPay,
   preparePayForUri,
@@ -78,7 +78,11 @@ import {
 } from "./pay-peer-push-credit.js";
 import { initiatePeerPushDebit } from "./pay-peer-push-debit.js";
 import { getRefreshesForTransaction } from "./refresh.js";
-import { getTransactionById, getTransactions } from "./transactions.js";
+import {
+  getTransactionById,
+  getTransactions,
+  parseTransactionIdentifier,
+} from "./transactions.js";
 import type { WalletExecutionContext } from "./wallet.js";
 import { acceptBankIntegratedWithdrawal } from "./withdraw.js";
 
@@ -93,7 +97,7 @@ export interface WithdrawTestBalanceResult {
   /**
    * Transaction ID of the newly created withdrawal transaction.
    */
-  transactionId: string;
+  transactionId: TransactionIdStr;
 
   /**
    * Account of the user registered for the withdrawal.
@@ -104,6 +108,18 @@ export interface WithdrawTestBalanceResult {
 export async function withdrawTestBalance(
   wex: WalletExecutionContext,
   req: WithdrawTestBalanceRequest,
+): Promise<WithdrawTestBalanceResult> {
+  wex.ws.refcntIgnoreTos += 1;
+  try {
+    return await withdrawTestBalanceImpl(wex, req);
+  } finally {
+    wex.ws.refcntIgnoreTos -= 1;
+  }
+}
+
+async function withdrawTestBalanceImpl(
+  wex: WalletExecutionContext,
+  req: WithdrawTestBalanceRequest,
 ): Promise<WithdrawTestBalanceResult> {
   const amount = req.amount;
   const exchangeBaseUrl = req.exchangeBaseUrl;
@@ -125,13 +141,21 @@ export async function withdrawTestBalance(
     amount,
   );
 
-  await fetchFreshExchange(wex, req.exchangeBaseUrl);
-  await acceptExchangeTermsOfService(wex, req.exchangeBaseUrl);
-
   const acceptResp = await acceptBankIntegratedWithdrawal(wex, {
     talerWithdrawUri: wresp.taler_withdraw_uri,
     selectedExchange: exchangeBaseUrl,
     forcedDenomSel: req.forcedDenomSel,
+    isForeignAccount: req.useForeignAccount,
+  });
+
+  // We need to wait until the wallet sent the reserve information
+  // to the bank, otherwise the confirmation in the bank would fail.
+  await waitTransactionState(wex, {
+    transactionId: acceptResp.transactionId,
+    txState: {
+      major: TransactionMajorState.Pending,
+      minor: TransactionMinorState.BankConfirmTransfer,
+    },
   });
 
   await corebankClient.confirmWithdrawalOperation(bankUser.username, {
@@ -145,7 +169,7 @@ export async function withdrawTestBalance(
 }
 
 /**
- * FIXME: User MerchantApiClient instead.
+ * FIXME: User TalerMerchantInstanceHttpClient instead.
  */
 function getMerchantAuthHeader(m: MerchantBackendInfo): Record<string, string> {
   if (m.authToken) {
@@ -157,7 +181,7 @@ function getMerchantAuthHeader(m: MerchantBackendInfo): Record<string, string> {
 }
 
 /**
- * FIXME: User MerchantApiClient instead.
+ * FIXME: User TalerMerchantInstanceHttpClient instead.
  */
 async function refund(
   http: HttpRequestLibrary,
@@ -189,7 +213,7 @@ async function refund(
 }
 
 /**
- * FIXME: User MerchantApiClient instead.
+ * FIXME: User TalerMerchantInstanceHttpClient instead.
  */
 async function createOrder(
   http: HttpRequestLibrary,
@@ -223,7 +247,7 @@ async function createOrder(
 }
 
 /**
- * FIXME: User MerchantApiClient instead.
+ * FIXME: User TalerMerchantInstanceHttpClient instead.
  */
 async function checkPayment(
   http: HttpRequestLibrary,
@@ -276,11 +300,9 @@ async function makePayment(
     throw Error("payment not possible");
   }
 
-  const confirmPayResult = await confirmPay(
-    wex,
-    preparePayResult.transactionId,
-    undefined,
-  );
+  const confirmPayResult = await confirmPay(wex, {
+    transactionId: preparePayResult.transactionId,
+  });
 
   logger.trace("confirmPayResult", confirmPayResult);
 
@@ -301,6 +323,18 @@ async function makePayment(
 export async function runIntegrationTest(
   wex: WalletExecutionContext,
   args: IntegrationTestArgs,
+): Promise<void> {
+  wex.ws.refcntIgnoreTos += 1;
+  try {
+    return await runIntegrationTestImpl(wex, args);
+  } finally {
+    wex.ws.refcntIgnoreTos -= 1;
+  }
+}
+
+async function runIntegrationTestImpl(
+  wex: WalletExecutionContext,
+  args: IntegrationTestArgs,
 ): Promise<void> {
   logger.info("running test with arguments", args);
 
@@ -571,40 +605,92 @@ async function waitUntilTransactionPendingReady(
   wex: WalletExecutionContext,
   transactionId: string,
 ): Promise<void> {
-  return await waitTransactionState(wex, transactionId, {
-    major: TransactionMajorState.Pending,
-    minor: TransactionMinorState.Ready,
+  return await waitTransactionState(wex, {
+    transactionId: transactionId as TransactionIdStr,
+    txState: {
+      major: TransactionMajorState.Pending,
+      minor: TransactionMinorState.Ready,
+    },
   });
 }
 
+function matchState(
+  st: TransactionState,
+  pat: TransactionStatePattern,
+): boolean {
+  return (
+    (pat.major === "*" || st.major === pat.major) &&
+    (pat.minor === "*" || st.minor === pat.minor)
+  );
+}
+
 /**
  * Wait until a transaction is in a particular state.
  */
 export async function waitTransactionState(
   wex: WalletExecutionContext,
-  transactionId: string,
-  txState: TransactionState,
+  req: TestingWaitTransactionRequest,
 ): Promise<void> {
+  const transactionId = req.transactionId;
+  const txState = req.txState;
+  const logId = req.logId ?? "none";
   logger.info(
     `starting waiting for ${transactionId} to be in ${JSON.stringify(
       txState,
-    )})`,
+    )}) (start logId: ${logId})`,
   );
-  await genericWaitForState(wex, {
+  let timeoutPromise;
+  if (req.timeout != null) {
+    const durationMs = Duration.fromSpec(req.timeout).d_ms;
+    checkLogicInvariant(durationMs !== "forever");
+    timeoutPromise = new Promise((resolve, reject) => {
+      wex.ws.timerGroup.after(durationMs, () => {
+        // Cancel the waiter.
+        wex.cts?.cancel();
+        reject(new Error(`waiting timed out (timeout logId: ${logId}`));
+      });
+    });
+  } else {
+    // No timeout => never resolve!
+    timeoutPromise = new Promise(() => {});
+  }
+  const waitPromise = genericWaitForState(wex, {
     async checkState() {
       const tx = await getTransactionById(wex, {
         transactionId,
       });
-      return (
-        tx.txState.major === txState.major && tx.txState.minor === txState.minor
+      logger.info(
+        `current state for ${transactionId} is ${JSON.stringify(
+          tx.txState,
+        )} (update logId: ${logId})`,
       );
+      if (req.requireError) {
+        if (tx.error == null) {
+          return false;
+        }
+      }
+      if (Array.isArray(txState)) {
+        for (const myState of txState) {
+          if (matchState(tx.txState, myState)) {
+            return true;
+          }
+        }
+        return false;
+      } else if (typeof txState === "number") {
+        return tx.stId === txState;
+      } else {
+        return matchState(tx.txState, txState);
+      }
     },
-    filterNotification(notif) {
-      return notif.type === NotificationType.TransactionStateTransition;
-    },
+    filterNotification: (notif) =>
+      notif.type === NotificationType.TransactionStateTransition &&
+      notif.transactionId === transactionId,
   });
+  await Promise.race([timeoutPromise, waitPromise]);
   logger.info(
-    `done waiting for ${transactionId} to be in ${JSON.stringify(txState)}`,
+    `done waiting for ${transactionId} to be in ${JSON.stringify(
+      txState,
+    )} (done logId: ${logId})`,
   );
 }
 
@@ -629,6 +715,18 @@ export async function waitUntilTransactionFinal(
 export async function runIntegrationTest2(
   wex: WalletExecutionContext,
   args: IntegrationTestV2Args,
+): Promise<void> {
+  wex.ws.refcntIgnoreTos += 1;
+  try {
+    return await runIntegrationTest2Impl(wex, args);
+  } finally {
+    wex.ws.refcntIgnoreTos -= 1;
+  }
+}
+
+async function runIntegrationTest2Impl(
+  wex: WalletExecutionContext,
+  args: IntegrationTestV2Args,
 ): Promise<void> {
   await wex.taskScheduler.ensureRunning();
   logger.info("running test with arguments", args);
@@ -859,19 +957,19 @@ export async function testPay(
   if (result.status !== PreparePayResultType.PaymentPossible) {
     throw Error(`unexpected prepare pay status: ${result.status}`);
   }
-  const r = await confirmPay(
-    wex,
-    result.transactionId,
-    undefined,
-    args.forcedCoinSel,
-  );
+  const r = await confirmPay(wex, {
+    transactionId: result.transactionId,
+    forcedCoinSel: args.forcedCoinSel,
+  });
   if (r.type != ConfirmPayResultType.Done) {
     throw Error("payment not done");
   }
   const purchase = await wex.db.runReadOnlyTx(
     { storeNames: ["purchases"] },
     async (tx) => {
-      return tx.purchases.get(result.proposalId);
+      const parsedTx = parseTransactionIdentifier(r.transactionId);
+      checkLogicInvariant(parsedTx?.tag === TransactionType.Payment);
+      return tx.purchases.get(parsedTx.proposalId);
     },
   );
   checkLogicInvariant(!!purchase);
