@@ -17,15 +17,22 @@
 /**
  * Imports.
  */
-import { GlobalIDB, IDBKeyRange } from "@gnu-taler/idb-bridge";
+import {
+  BridgeIDBKeyRange,
+  GlobalIDB,
+  IDBKeyRange,
+} from "@gnu-taler/idb-bridge";
 import {
   AbsoluteTime,
   Amounts,
   assertUnreachable,
+  GetTransactionsV2Request,
   j2s,
   Logger,
+  makeTalerErrorDetail,
   NotificationType,
   ScopeType,
+  TalerErrorCode,
   Transaction,
   TransactionByIdRequest,
   TransactionIdStr,
@@ -34,6 +41,7 @@ import {
   TransactionsResponse,
   TransactionState,
   TransactionType,
+  WalletNotification,
 } from "@gnu-taler/taler-util";
 import {
   constructTaskIdentifier,
@@ -42,8 +50,15 @@ import {
   TransactionContext,
 } from "./common.js";
 import {
+  DbPreciseTimestamp,
+  OPERATION_STATUS_DONE_FIRST,
+  OPERATION_STATUS_DONE_LAST,
   OPERATION_STATUS_NONFINAL_FIRST,
   OPERATION_STATUS_NONFINAL_LAST,
+  PurchaseStatus,
+  timestampPreciseToDb,
+  TransactionMetaRecord,
+  WalletDbAllStoresReadOnlyTransaction,
   WalletDbAllStoresReadWriteTransaction,
 } from "./db.js";
 import { DepositTransactionContext } from "./deposits.js";
@@ -63,7 +78,10 @@ import { WithdrawTransactionContext } from "./withdraw.js";
 const logger = new Logger("taler-wallet-core:transactions.ts");
 
 function shouldSkipCurrency(
-  transactionsRequest: TransactionsRequest | undefined,
+  transactionsRequest:
+    | TransactionsRequest
+    | GetTransactionsV2Request
+    | undefined,
   currency: string,
   exchangesInTransaction: string[],
 ): boolean {
@@ -91,7 +109,6 @@ function shouldSkipCurrency(
         assertUnreachable(transactionsRequest.scopeInfo);
     }
   }
-  // FIXME: remove next release
   if (transactionsRequest?.currency) {
     return (
       transactionsRequest.currency.toLowerCase() !== currency.toLowerCase()
@@ -143,7 +160,9 @@ export async function getTransactionById(
     case TransactionType.Refund: {
       const ctx = await getContextForTransaction(wex, req.transactionId);
       const txDetails = await wex.db.runAllStoresReadOnlyTx({}, async (tx) =>
-        ctx.lookupFullTransaction(tx),
+        ctx.lookupFullTransaction(tx, {
+          includeContractTerms: req.includeContractTerms,
+        }),
       );
       if (!txDetails) {
         throw Error("transaction not found");
@@ -163,6 +182,261 @@ export function isUnsuccessfulTransaction(state: TransactionState): boolean {
   );
 }
 
+function checkFilterIncludes(
+  req: GetTransactionsV2Request | undefined,
+  mtx: TransactionMetaRecord,
+): boolean {
+  if (shouldSkipCurrency(req, mtx.currency, mtx.exchanges)) {
+    return false;
+  }
+
+  const parsedTx = parseTransactionIdentifier(mtx.transactionId);
+  if (
+    parsedTx?.tag === TransactionType.Refresh &&
+    !(req?.includeRefreshes || req?.includeAll)
+  ) {
+    return false;
+  }
+
+  if (!req?.includeAll) {
+    if (
+      parsedTx?.tag === TransactionType.Payment &&
+      mtx.status === PurchaseStatus.DoneRepurchaseDetected
+    ) {
+      return false;
+    }
+  }
+
+  let included: boolean;
+
+  const filter = req?.filterByState;
+  switch (filter) {
+    case "done":
+      included =
+        mtx.status >= OPERATION_STATUS_DONE_FIRST &&
+        mtx.status <= OPERATION_STATUS_DONE_LAST;
+      break;
+    case "final":
+      included = !(
+        mtx.status >= OPERATION_STATUS_NONFINAL_FIRST &&
+        mtx.status <= OPERATION_STATUS_NONFINAL_LAST
+      );
+      break;
+    case "nonfinal":
+      included =
+        mtx.status >= OPERATION_STATUS_NONFINAL_FIRST &&
+        mtx.status <= OPERATION_STATUS_NONFINAL_LAST;
+      break;
+    case undefined:
+      included = true;
+      break;
+    default:
+      assertUnreachable(filter);
+  }
+  return included;
+}
+
+async function addFiltered(
+  wex: WalletExecutionContext,
+  tx: WalletDbAllStoresReadOnlyTransaction,
+  req: GetTransactionsV2Request | undefined,
+  target: Transaction[],
+  source: TransactionMetaRecord[],
+): Promise<number> {
+  let numAdded: number = 0;
+  for (const mtx of source) {
+    if (req?.limit != null && target.length >= Math.abs(req.limit)) {
+      break;
+    }
+    if (checkFilterIncludes(req, mtx)) {
+      const ctx = await getContextForTransaction(wex, mtx.transactionId);
+      const txDetails = await ctx.lookupFullTransaction(tx);
+      // FIXME: This means that in some cases we can return fewer transactions
+      // than requested.
+      if (!txDetails) {
+        continue;
+      }
+      numAdded += 1;
+      target.push(txDetails);
+    }
+  }
+  return numAdded;
+}
+
+/**
+ * Sort transactions in-place according to the request.
+ */
+function sortTransactions(
+  req: GetTransactionsV2Request | undefined,
+  transactions: Transaction[],
+): void {
+  let sortSign: number;
+  if (req?.limit != null && req.limit < 0) {
+    sortSign = -1;
+  } else {
+    sortSign = 1;
+  }
+  const txCmp = (h1: Transaction, h2: Transaction) => {
+    // Order transactions by timestamp.  Newest transactions come first.
+    const tsCmp = AbsoluteTime.cmp(
+      AbsoluteTime.fromPreciseTimestamp(h1.timestamp),
+      AbsoluteTime.fromPreciseTimestamp(h2.timestamp),
+    );
+    // If the timestamp is exactly the same, order by transaction type.
+    if (tsCmp === 0) {
+      return Math.sign(txOrder[h1.type] - txOrder[h2.type]);
+    }
+    return sortSign * tsCmp;
+  };
+  transactions.sort(txCmp);
+}
+
+async function findOffsetTransaction(
+  tx: WalletDbAllStoresReadOnlyTransaction,
+  req?: GetTransactionsV2Request,
+): Promise<TransactionMetaRecord | undefined> {
+  let forwards = req?.limit == null || req.limit >= 0;
+  let closestTimestamp: DbPreciseTimestamp | undefined = undefined;
+  if (req?.offsetTransactionId) {
+    const res = await tx.transactionsMeta.get(req.offsetTransactionId);
+    if (res) {
+      return res;
+    }
+    if (req.offsetTimestamp) {
+      closestTimestamp = timestampPreciseToDb(req.offsetTimestamp);
+    } else {
+      throw Error(
+        "offset transaction not found and no offset timestamp specified",
+      );
+    }
+  } else if (req?.offsetTimestamp) {
+    const dbStamp = timestampPreciseToDb(req.offsetTimestamp);
+    const res = await tx.transactionsMeta.indexes.byTimestamp.get(dbStamp);
+    if (res) {
+      return res;
+    }
+    closestTimestamp = timestampPreciseToDb(req.offsetTimestamp);
+  } else {
+    return undefined;
+  }
+
+  // We didn't find a precise offset transaction.
+  // This must mean that it was deleted.
+  // Depending on the direction, find the prev/next
+  // transaction and use it as an offset.
+
+  if (forwards) {
+    // We don't want to skip transactions in pagination,
+    // so get the transaction before the timestamp
+
+    // Slow query, but should not happen often!
+    const recs = await tx.transactionsMeta.indexes.byTimestamp.getAll(
+      BridgeIDBKeyRange.upperBound(closestTimestamp, false),
+    );
+    if (recs.length > 0) {
+      return recs[recs.length - 1];
+    }
+    return undefined;
+  } else {
+    // Likewise, get the transaction after the timestamp
+    const recs = await tx.transactionsMeta.indexes.byTimestamp.getAll(
+      BridgeIDBKeyRange.lowerBound(closestTimestamp, false),
+      1,
+    );
+    return recs[0];
+  }
+}
+
+export async function getTransactionsV2(
+  wex: WalletExecutionContext,
+  transactionsRequest?: GetTransactionsV2Request,
+): Promise<TransactionsResponse> {
+  // The implementation of this function is optimized for
+  // the common fast path of requesting transactions
+  // in an ascending order.
+  // Other requests are more difficult to implement
+  // in a performant way due to IndexedDB limitations.
+
+  const resultTransactions: Transaction[] = [];
+
+  await wex.db.runAllStoresReadOnlyTx({}, async (tx) => {
+    let forwards =
+      transactionsRequest?.limit == null || transactionsRequest.limit >= 0;
+    let limit =
+      transactionsRequest?.limit != null
+        ? Math.abs(transactionsRequest.limit)
+        : undefined;
+    let offsetMtx = await findOffsetTransaction(tx, transactionsRequest);
+
+    if (limit == null && offsetMtx == null) {
+      // Fast path for returning *everything* that matches the filter.
+      // FIXME: We could use the DB for filtering here
+      const res = await tx.transactionsMeta.indexes.byStatus.getAll();
+      await addFiltered(wex, tx, transactionsRequest, resultTransactions, res);
+    } else if (!forwards) {
+      // Descending, backwards request.
+      // Slow implementation. Doing it properly would require using cursors,
+      // which are also slow in IndexedDB.
+      const res = await tx.transactionsMeta.indexes.byTimestamp.getAll();
+      res.reverse();
+      let start: number;
+      if (offsetMtx != null) {
+        const needleTxId = offsetMtx.transactionId;
+        start = res.findIndex((x) => x.transactionId === needleTxId);
+        if (start < 0) {
+          throw Error("offset transaction not found");
+        }
+      } else {
+        start = 0;
+      }
+      for (let i = start; i < res.length; i++) {
+        if (limit != null && resultTransactions.length >= limit) {
+          break;
+        }
+        await addFiltered(wex, tx, transactionsRequest, resultTransactions, [
+          res[i],
+        ]);
+      }
+    } else {
+      // Forward request
+      let query: BridgeIDBKeyRange | undefined = undefined;
+      if (offsetMtx != null) {
+        query = GlobalIDB.KeyRange.lowerBound(offsetMtx.timestamp, true);
+      }
+      while (true) {
+        const res = await tx.transactionsMeta.indexes.byTimestamp.getAll(
+          query,
+          limit,
+        );
+        if (res.length === 0) {
+          break;
+        }
+        await addFiltered(
+          wex,
+          tx,
+          transactionsRequest,
+          resultTransactions,
+          res,
+        );
+        if (limit != null && resultTransactions.length >= limit) {
+          break;
+        }
+        // Continue after last result
+        query = BridgeIDBKeyRange.lowerBound(
+          res[res.length - 1].timestamp,
+          true,
+        );
+      }
+    }
+  });
+
+  sortTransactions(transactionsRequest, resultTransactions);
+
+  return {
+    transactions: resultTransactions,
+  };
+}
+
 /**
  * Retrieve the full event history for this wallet.
  */
@@ -275,7 +549,7 @@ export async function rematerializeTransactions(
   wex: WalletExecutionContext,
   tx: WalletDbAllStoresReadWriteTransaction,
 ): Promise<void> {
-  logger.info("re-materializing transactions");
+  logger.trace("re-materializing transactions");
 
   const allTxMeta = await tx.transactionsMeta.getAll();
   for (const txMeta of allTxMeta) {
@@ -598,7 +872,12 @@ export async function failTransaction(
   transactionId: string,
 ): Promise<void> {
   const ctx = await getContextForTransaction(wex, transactionId);
-  await ctx.failTransaction();
+  await ctx.failTransaction(
+    makeTalerErrorDetail(
+      TalerErrorCode.WALLET_TRANSACTION_ABANDONED_BY_USER,
+      {},
+    ),
+  );
 }
 
 /**
@@ -631,19 +910,67 @@ export async function abortTransaction(
   transactionId: string,
 ): Promise<void> {
   const ctx = await getContextForTransaction(wex, transactionId);
-  await ctx.abortTransaction();
+  await ctx.abortTransaction(
+    makeTalerErrorDetail(TalerErrorCode.WALLET_TRANSACTION_ABORTED_BY_USER, {}),
+  );
 }
 
 export interface TransitionInfo {
+  // FIXME: Get rid in favor of oldStId
   oldTxState: TransactionState;
+  // FIXME: Get rid in favor of newStId
   newTxState: TransactionState;
+
+  balanceEffect: BalanceEffect;
+
+  newStId: number;
+  oldStId: number;
+}
+
+export enum BalanceEffect {
+  /**
+   * There is no effect on the balance.
+   */
+  None = 0,
+  /**
+   * Balance change is internal and should not
+   * change the user-visible balance.
+   */
+  PreserveUserVisible = 1,
+  /**
+   * The balance preserves the user-visible balance,
+   * but might affect user-visible flags.
+   */
+  Flags = 2,
+  /**
+   * Balance change can have any effect.
+   */
+  Any = 3,
 }
 
 /**
- * Notify of a state transition if necessary.
+ * Call the given notification function with a
+ * balance effect notification, if necessary.
  */
-export function notifyTransition(
-  wex: WalletExecutionContext,
+export function applyNotifyBalanceEffect(
+  notify: (n: WalletNotification) => void,
+  transactionId: string,
+  balanceEffect: BalanceEffect,
+): void {
+  if (balanceEffect > BalanceEffect.None) {
+    notify({
+      type: NotificationType.BalanceChange,
+      hintTransactionId: transactionId,
+      isInternal: balanceEffect <= BalanceEffect.PreserveUserVisible,
+    });
+  }
+}
+
+/**
+ * Notify of a state transition and balance change if necessary.
+ */
+export function applyNotifyTransition(
+  notify: (n: WalletNotification) => void,
   transactionId: string,
   transitionInfo: TransitionInfo | undefined,
   experimentalUserData: any = undefined,
@@ -651,16 +978,24 @@ export function notifyTransition(
   if (
     transitionInfo &&
     !(
+      transitionInfo.oldStId === transitionInfo.newStId &&
       transitionInfo.oldTxState.major === transitionInfo.newTxState.major &&
       transitionInfo.oldTxState.minor === transitionInfo.newTxState.minor
     )
   ) {
-    wex.ws.notify({
+    notify({
       type: NotificationType.TransactionStateTransition,
       oldTxState: transitionInfo.oldTxState,
       newTxState: transitionInfo.newTxState,
       transactionId,
       experimentalUserData,
+      newStId: transitionInfo.newStId,
     });
+
+    applyNotifyBalanceEffect(
+      notify,
+      transactionId,
+      transitionInfo.balanceEffect,
+    );
   }
 }
