@@ -14,12 +14,15 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
+import { TalerExchangeHttpClient } from "@gnu-taler/taler-util";
+
 /**
  * Protocol version spoken with the exchange.
  *
  * Uses libtool's current:revision:age versioning.
  */
-export const WALLET_EXCHANGE_PROTOCOL_VERSION = "17:0:0";
+export const WALLET_EXCHANGE_PROTOCOL_VERSION =
+  TalerExchangeHttpClient.SUPPORTED_EXCHANGE_PROTOCOL_VERSION;
 
 /**
  * Protocol version spoken with the merchant.
