@@ -149,7 +149,9 @@ export const Pages = {
 
   defaultCta: pageDefinition<{ uri: CrockEncodedString }>("/taler-uri/:uri"),
   // FIXME: mem leak problems
-  defaultCtaSimple: pageDefinition<{ uri: CrockEncodedString }>("/taler-uri-simple/:uri"),
+  defaultCtaSimple: pageDefinition<{ uri: CrockEncodedString }>(
+    "/taler-uri-simple/:uri",
+  ),
   cta: pageDefinition<{ action: CrockEncodedString }>("/cta/:action"),
   ctaPay: "/cta/pay",
   ctaPayTemplate: "/cta/pay/template",
@@ -158,7 +160,7 @@ export const Pages = {
   ctaWithdraw: "/cta/withdraw",
   ctaDeposit: pageDefinition<{
     scope: CrockEncodedString;
-    account: CrockEncodedString,
+    account: CrockEncodedString;
   }>("/cta/deposit/:scope/:account"),
   ctaExperiment: "/cta/experiment",
   ctaAddExchange: "/cta/add/exchange",
@@ -171,10 +173,16 @@ export const Pages = {
   ctaInvoicePay: "/cta/invoice/pay",
   ctaTransferPickup: "/cta/transfer/pickup",
 
-  ctaWithdrawManual: pageDefinition<{
+  ctaWithdrawManualForScope: pageDefinition<{
     scope: CrockEncodedString;
     amount?: string;
-  }>("/cta/manual-withdraw/:scope/:amount?"),
+  }>("/cta/scope-withdraw/:scope/:amount?"),
+  ctaWithdrawTransferResult: "/cta/transfer-result",
+  ctaWithdrawManual: pageDefinition<{amount?: string;}>("/cta/manual-withdraw/:amount?"),
+  paytoQrs: pageDefinition<{ payto: CrockEncodedString }>("/payto/qrs/:payto?"),
+  paytoBanks: pageDefinition<{ payto: CrockEncodedString }>(
+    "/payto/banks/:payto?",
+  ),
 };
 
 const talerUriActionToPageName: {
@@ -190,6 +198,7 @@ const talerUriActionToPageName: {
   [TalerUriAction.WithdrawExchange]: "ctaWithdrawManual",
   [TalerUriAction.DevExperiment]: "ctaExperiment",
   [TalerUriAction.AddExchange]: "ctaAddExchange",
+  [TalerUriAction.WithdrawalTransferResult]: "ctaWithdrawTransferResult",
 };
 
 export function getPathnameForTalerURI(talerUri: string): string | undefined {
@@ -277,11 +286,17 @@ export function WalletNavBar({ path }: { path?: WalletNavBarOptions }): VNode {
   return (
     <NavigationHeaderHolder>
       <NavigationHeader>
-        <a href={`#${Pages.balance}`} class={path === "balance" ? "active" : ""}>
+        <a
+          href={`#${Pages.balance}`}
+          class={path === "balance" ? "active" : ""}
+        >
           <i18n.Translate>Balance</i18n.Translate>
         </a>
         <EnabledBySettings name="backup">
-          <a href={`#${Pages.backup}`} class={path === "backup" ? "active" : ""}>
+          <a
+            href={`#${Pages.backup}`}
+            class={path === "backup" ? "active" : ""}
+          >
             <i18n.Translate>Backup</i18n.Translate>
           </a>
         </EnabledBySettings>
