@@ -17,7 +17,7 @@
 import { Amounts, ScopeInfo, ScopeType, WalletBalance } from "@gnu-taler/taler-util";
 import { Fragment, VNode, h } from "preact";
 import {
-  TableWithRoundRows as TableWithRoundedRows
+  TableWithRoundRows
 } from "./styled/index.js";
 
 export function BalanceTable({
@@ -29,7 +29,7 @@ export function BalanceTable({
 }): VNode {
   return (
     <Fragment>
-      <TableWithRoundedRows>
+      <TableWithRoundRows>
         {balances.map((entry, idx) => {
           const av = Amounts.parseOrThrow(entry.available);
 
@@ -50,7 +50,7 @@ export function BalanceTable({
                 {Amounts.stringifyValue(av, 2)}
                 <div style={{ fontSize: "small", color: "grey" }}>
                   {entry.scopeInfo.type === ScopeType.Exchange ||
-                  entry.scopeInfo.type === ScopeType.Auditor
+                    entry.scopeInfo.type === ScopeType.Auditor
                     ? entry.scopeInfo.url
                     : undefined}
                 </div>
@@ -58,7 +58,7 @@ export function BalanceTable({
             </tr>
           );
         })}
-      </TableWithRoundedRows>
+      </TableWithRoundRows>
     </Fragment>
   );
 }
