@@ -19,14 +19,15 @@ import {
   stringifyPaytoUri,
   TranslatedString,
 } from "@gnu-taler/taler-util";
+import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { styled } from "@linaria/react";
 import { Fragment, h, VNode } from "preact";
 import { useState } from "preact/hooks";
+import arrowDown from "../svg/chevron-down.inline.svg";
 import {
   ExtraLargeText,
   LargeText,
-  SmallBoldText,
-  SmallLightText,
+  SmallBoldText
 } from "./styled/index.js";
 
 export type Kind = "positive" | "negative" | "neutral";
@@ -96,11 +97,8 @@ const CollasibleBox = styled.div`
     }
   }
 `;
-import arrowDown from "../svg/chevron-down.inline.svg";
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
 
-export function PartCollapsible({ text, title, big, showSign }: Props): VNode {
-  const Text = big ? ExtraLargeText : LargeText;
+export function PartCollapsible({ text, title }: Props): VNode {
   const [collapsed, setCollapsed] = useState(true);
 
   return (
