@@ -18,6 +18,7 @@
  *
  * @author Sebastian Javier Marchano (sebasjm)
  */
+import { AmountString, MerchantContractTerms } from "@gnu-taler/taler-util";
 import * as tests from "@gnu-taler/web-util/testing";
 import {
   ErrorView,
@@ -25,42 +26,42 @@ import {
   LoadingView,
   ShowView,
 } from "./ShowFullContractTermPopup.js";
-import { AmountString, WalletContractData } from "@gnu-taler/taler-util";
 
 export default {
   title: "ShowFullContractTermPopup",
 };
 
-const cd: WalletContractData = {
+const cd: MerchantContractTerms = {
+  products: [],
+  nonce: "123",
   amount: "ARS:2" as AmountString,
-  contractTermsHash:
-    "92X0KSJPZ8XS2XECCGFWTCGW8XMFCXTT2S6WHZDP6H9Y3TSKMTHY94WXEWDERTNN5XWCYGW4VN5CF2D4846HXTW7P06J4CZMHCWKC9G",
-  fulfillmentUrl: "",
-  merchantBaseUrl: "https://merchant-backend.taler.ar/",
-  merchantPub: "JZYHJ13M91GMSQMT75J8Q6ZN0QP8XF8CRHR7K5MMWYE8JQB6AAPG",
-  merchantSig:
-    "0YA1WETV15R6K8QKS79QA3QMT16010F42Q49VSKYQ71HVQKAG0A4ZJCA4YTKHE9EA5SP156TJSKZEJJJ87305N6PS80PC48RNKYZE08",
-  orderId: "2022.220-0281XKKB8W7YE",
+  merchant_base_url: "https://merchant-backend.taler.ar/",
+  merchant_pub: "JZYHJ13M91GMSQMT75J8Q6ZN0QP8XF8CRHR7K5MMWYE8JQB6AAPG",
+  order_id: "2022.220-0281XKKB8W7YE",
   summary: "w",
-  payDeadline: {
+  pay_deadline: {
     t_s: 1660002673,
   },
-  refundDeadline: {
+  refund_deadline: {
     t_s: 1660002673,
   },
-  allowedExchanges: [
+  exchanges: [
     {
-      exchangeBaseUrl: "https://exchange.taler.ar/",
-      exchangePub: "1C2EYE90PYDNVRTQ25A3PA0KW5W4WPAJNNQHVHV49PT6W5CERFV0",
+      master_pub: "1C2EYE90PYDNVRTQ25A3PA0KW5W4WPAJNNQHVHV49PT6W5CERFV0",
+      url: "https://exchange.taler.ar/",
+      priority: 1,
     },
   ],
   timestamp: {
     t_s: 1659972710,
   },
-  wireMethod: "x-taler-bank",
-  wireInfoHash:
+  wire_transfer_deadline: {
+    t_s: 1659972710,
+  },
+  wire_method: "x-taler-bank",
+  h_wire:
     "QDT28374ZHYJ59WQFZ3TW1D5WKJVDYHQT86VHED3TNMB15ANJSKXDYPPNX01348KDYCX6T4WXA5A8FJJ8YWNEB1JW726C1JPKHM89DR",
-  maxDepositFee: "ARS:1" as AmountString,
+  max_fee: "ARS:1" as AmountString,
   merchant: {
     name: "Default",
     address: {
@@ -71,8 +72,8 @@ const cd: WalletContractData = {
     },
   },
   // products: [],
-  autoRefund: undefined,
-  summaryI18n: undefined,
+  auto_refund: undefined,
+  summary_i18n: undefined,
   // deliveryDate: undefined,
   // deliveryLocation: undefined,
 };
