@@ -37,9 +37,9 @@ import { TextField } from "../mui/TextField.js";
 import { SafeHandler } from "../mui/handlers.js";
 import { WxApiType } from "../wxApi.js";
 import { WalletActivityTrack } from "../wxBackend.js";
+import { Checkbox } from "./Checkbox.js";
 import { Modal } from "./Modal.js";
 import { Time } from "./Time.js";
-import { Checkbox } from "./Checkbox.js";
 
 const OPEN_ACTIVITY_HEIGHT_PX = 250;
 const CLOSE_ACTIVITY_HEIGHT_PX = 40;
@@ -312,34 +312,34 @@ function ShowExchangeStateTransition({ events }: MoreInfoPRops): VNode {
       <dt>Exchange</dt>
       <dd>{not.exchangeBaseUrl}</dd>
       {not.oldExchangeState &&
-        not.newExchangeState.exchangeEntryStatus !==
+        not.newExchangeState?.exchangeEntryStatus !==
           not.oldExchangeState?.exchangeEntryStatus && (
           <Fragment>
             <dt>Entry status</dt>
             <dd>
               from {not.oldExchangeState.exchangeEntryStatus} to{" "}
-              {not.newExchangeState.exchangeEntryStatus}
+              {not.newExchangeState?.exchangeEntryStatus}
             </dd>
           </Fragment>
         )}
       {not.oldExchangeState &&
-        not.newExchangeState.exchangeUpdateStatus !==
+        not.newExchangeState?.exchangeUpdateStatus !==
           not.oldExchangeState?.exchangeUpdateStatus && (
           <Fragment>
             <dt>Update status</dt>
             <dd>
               from {not.oldExchangeState.exchangeUpdateStatus} to{" "}
-              {not.newExchangeState.exchangeUpdateStatus}
+              {not.newExchangeState?.exchangeUpdateStatus}
             </dd>
           </Fragment>
         )}
       {not.oldExchangeState &&
-        not.newExchangeState.tosStatus !== not.oldExchangeState?.tosStatus && (
+        not.newExchangeState?.tosStatus !== not.oldExchangeState?.tosStatus && (
           <Fragment>
             <dt>Tos status</dt>
             <dd>
               from {not.oldExchangeState.tosStatus} to{" "}
-              {not.newExchangeState.tosStatus}
+              {not.newExchangeState?.tosStatus}
             </dd>
           </Fragment>
         )}
@@ -848,9 +848,8 @@ export function ObservabilityEventsTable(): VNode {
                           (not.events[0] as RequestProgressNotification)
                             .requestId
                         })`;
-                      case NotificationType.WithdrawalOperationTransition: {
-                        return `---`;
-                      }
+                      case NotificationType.BankAccountChange:
+                        return i18n.str`Bank account info changed`;
                       default: {
                         assertUnreachable(not.type);
                       }
@@ -933,9 +932,8 @@ export function ObservabilityEventsTable(): VNode {
                     />
                   );
                 }
-                case NotificationType.WithdrawalOperationTransition: {
-                  return <div>not implemented</div>;
-                }
+                default:
+                  return null;
               }
             })()}
           </details>
