@@ -17,13 +17,12 @@
 import {
   AbsoluteTime,
   Amounts,
-  MerchantContractTerms as ContractTerms,
   Duration,
-  PreparePayResultType,
-  TranslatedString,
+  TranslatedString
 } from "@gnu-taler/taler-util";
 import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
+import { EnabledBySettings } from "../../components/EnabledBySettings.js";
 import { Part } from "../../components/Part.js";
 import { PaymentButtons } from "../../components/PaymentButtons.js";
 import { ShowFullContractTermPopup } from "../../components/ShowFullContractTermPopup.js";
@@ -31,41 +30,30 @@ import { Time } from "../../components/Time.js";
 import {
   AgeSign,
   SuccessBox,
+  TableWithRoundRows,
   WarningBox,
 } from "../../components/styled/index.js";
 import { MerchantDetails } from "../../wallet/Transaction.js";
 import { State } from "./index.js";
-import { EnabledBySettings } from "../../components/EnabledBySettings.js";
 
-type SupportedStates =
+export type PaymentStates =
   | State.Ready
   | State.Confirmed
-  | State.NoBalanceForCurrency
   | State.NoEnoughBalance;
 
-export function BaseView(state: SupportedStates): VNode {
+const inFiveMinutes = AbsoluteTime.addDuration(
+  AbsoluteTime.now(),
+  Duration.fromSpec({ minutes: 5 }),
+);
+
+export function BaseView(state: PaymentStates): VNode {
   const { i18n } = useTranslationContext();
 
-  const contractTerms: ContractTerms = state.payStatus.contractTerms;
 
-  const effective =
-    "amountEffective" in state.payStatus
-      ? state.payStatus.amountEffective
-        ? Amounts.parseOrThrow(state.payStatus.amountEffective)
-        : Amounts.zeroOfCurrency(state.amount.currency)
-      : state.amount;
+  const willExpireSoon = AbsoluteTime.cmp(state.expiration, inFiveMinutes) === -1;
+
+  // const choices = state.contractTerms.version && state.contractTerms.version === 1 ? state.contractTerms.choices : [];
 
-  const expiration = !contractTerms.pay_deadline
-    ? undefined
-    : AbsoluteTime.fromProtocolTimestamp(contractTerms.pay_deadline);
-  const inFiveMinutes = AbsoluteTime.addDuration(
-    AbsoluteTime.now(),
-    Duration.fromSpec({ minutes: 5 }),
-  );
-  const willExpireSoon =
-    !expiration || expiration.t_ms === "never"
-      ? undefined
-      : AbsoluteTime.cmp(expiration, inFiveMinutes) === -1;
   return (
     <Fragment>
       <ShowImportantMessage state={state} />
@@ -73,7 +61,7 @@ export function BaseView(state: SupportedStates): VNode {
       <section style={{ textAlign: "left" }}>
         <Part
           title={
-            contractTerms.minimum_age ? (
+            state.minimum_age ? (
               <Fragment>
                 <i18n.Translate>Purchase</i18n.Translate>
                 &nbsp;
@@ -81,59 +69,88 @@ export function BaseView(state: SupportedStates): VNode {
                   size={20}
                   title={i18n.str`This purchase is age restricted.`}
                 >
-                  {contractTerms.minimum_age}+
+                  {state.minimum_age}+
                 </AgeSign>
               </Fragment>
             ) : (
               <i18n.Translate>Purchase</i18n.Translate>
             )
           }
-          text={contractTerms.summary as TranslatedString}
-          kind="neutral"
-        />
-        <Part
-          title={i18n.str`Merchant`}
-          text={<MerchantDetails merchant={contractTerms.merchant} />}
+          text={state.summary as TranslatedString}
           kind="neutral"
         />
+        {!state.merchant ? undefined :
+          <Part
+            title={i18n.str`Merchant`}
+            text={<MerchantDetails merchant={state.merchant} />}
+            kind="neutral"
+          />
+        }
         {willExpireSoon && (
           <Part
             title={i18n.str`Expires at`}
-            text={<Time timestamp={expiration} format="HH:mm" />}
+            text={<Time timestamp={state.expiration} format="HH:mm" />}
             kind="neutral"
           />
         )}
+        {state.status === "confirmed" || state.choices === undefined ? undefined :
+          <TableWithRoundRows>
+            {state.choices.list.map((entry, idx) => {
+              const selected = state.choices!.index === idx;
+              const av = Amounts.parseOrThrow(entry.amount);
+
+              return (
+                <tr
+                  key={idx}
+                  onClick={() => state.choices!.select(idx)}
+                  style={{ cursor: !selected ? "pointer" : "default" }}
+                >
+                  <td style={{ borderWidth: selected ? 4 : 1 }}>{av.currency}</td>
+                  <td
+                    style={{
+                      fontSize: "2em",
+                      textAlign: "left",
+                      borderWidth: selected ? 4 : 1,
+                    }}
+                  >
+                    {Amounts.stringifyValue(av, 2)}
+                  </td>
+                  <td style={{
+                    borderWidth: selected ? 4 : 1,
+                    width: "100%",
+                    textAlign: "right",
+                  }}>{entry.description}</td>
+
+                </tr>
+              );
+            })}
+          </TableWithRoundRows>
+        }
       </section>
       <EnabledBySettings name="advancedMode">
         <section style={{ textAlign: "left" }}>
           <ShowFullContractTermPopup
-            transactionId={state.payStatus.transactionId}
+            transactionId={state.transactionId}
           />
         </section>
       </EnabledBySettings>
-      <PaymentButtons
-        amount={effective}
-        payStatus={state.payStatus}
-        uri={state.uri}
-        payHandler={state.status === "ready" ? state.payHandler : undefined}
-        goToWalletManualWithdraw={state.goToWalletManualWithdraw}
-      />
+
+      <PaymentButtons paymentState={state} />
     </Fragment>
   );
 }
 
-function ShowImportantMessage({ state }: { state: SupportedStates }): VNode {
+function ShowImportantMessage({ state }: { state: PaymentStates }): VNode {
   const { i18n } = useTranslationContext();
-  const { payStatus } = state;
-  if (payStatus.status === PreparePayResultType.AlreadyConfirmed) {
-    if (payStatus.paid) {
-      if (payStatus.contractTerms.fulfillment_url) {
+  if (state.status === "confirmed") {
+    if (state.paid) {
+      if (state.message) {
         return (
           <SuccessBox>
             <i18n.Translate>
               Already paid, you are going to be redirected to{" "}
-              <a href={payStatus.contractTerms.fulfillment_url}>
-                {payStatus.contractTerms.fulfillment_url}
+              <a href={state.message}>
+                {state.message}
               </a>
             </i18n.Translate>
           </SuccessBox>
