@@ -20,7 +20,7 @@
  */
 
 import { Amounts } from "@gnu-taler/taler-util";
-import beer from "../../../static-dev/beer.png";
+import beer from "../../../static-stories/beer.png";
 import * as tests from "@gnu-taler/web-util/testing";
 import { IgnoredView, ReadyView } from "./views.js";
 export default {
