@@ -21,6 +21,8 @@ import {
   ExchangeFullDetails,
   ExchangeListItem,
   NotificationType,
+  ScopeInfo,
+  ScopeType,
   TransactionMajorState,
   TransactionMinorState,
   parseWithdrawExchangeUri,
@@ -37,44 +39,53 @@ import { PropsFromParams, PropsFromURI, State } from "./index.js";
 
 export function useComponentStateFromParams({
   talerExchangeWithdrawUri: maybeTalerUri,
-  amount,
   scope,
+  amount: paramsAmount,
   cancel,
   onSuccess,
 }: PropsFromParams): RecursiveState<State> {
   const api = useBackendContext();
   const { i18n } = useTranslationContext();
-  const paramsAmount = amount ? Amounts.parse(amount) : undefined;
+  // const paramsAmount = amount ? Amounts.parse(amount) : undefined;
   const [updatedExchangeByUser, setUpdatedExchangeByUser] = useState<string>();
   const uriInfoHook = useAsyncAsHook(async () => {
-    const exchanges = await api.wallet.call(
-      WalletApiOperation.ListExchanges,
-      {},
-    );
     const uri = maybeTalerUri
       ? parseWithdrawExchangeUri(maybeTalerUri)
       : undefined;
-    const exchangeByTalerUri = updatedExchangeByUser ?? uri?.exchangeBaseUrl;
+
+    const exchangeUrlByScope =
+      scope?.type === ScopeType.Exchange ? scope.url : undefined;
+    const exchangeByScopeOrTalerUri =
+      updatedExchangeByUser ?? exchangeUrlByScope ?? uri?.exchangeBaseUrl;
 
     let ex: ExchangeFullDetails | undefined;
-    if (exchangeByTalerUri) {
+    if (exchangeByScopeOrTalerUri) {
       await api.wallet.call(WalletApiOperation.AddExchange, {
-        exchangeBaseUrl: exchangeByTalerUri,
+        exchangeBaseUrl: exchangeByScopeOrTalerUri,
       });
       const info = await api.wallet.call(
         WalletApiOperation.GetExchangeDetailedInfo,
         {
-          exchangeBaseUrl: exchangeByTalerUri,
+          exchangeBaseUrl: exchangeByScopeOrTalerUri,
         },
       );
 
       ex = info.exchange;
     }
+
+    const exchanges = await api.wallet.call(WalletApiOperation.ListExchanges, {
+      filterByScope: scope,
+    });
+
     const chosenAmount =
       !uri || !uri.amount ? undefined : Amounts.parse(uri.amount);
     return { amount: chosenAmount, exchanges, exchange: ex };
   });
 
+  useEffect(() => {
+    uriInfoHook?.retry();
+  }, [paramsAmount]);
+
   if (!uriInfoHook) return { status: "loading", error: undefined };
 
   if (uriInfoHook.hasError) {
@@ -88,75 +99,23 @@ export function useComponentStateFromParams({
     };
   }
 
-  useEffect(() => {
-    uriInfoHook?.retry();
-  }, [amount]);
-
-  const exchangeByTalerUri = uriInfoHook.response.exchange?.exchangeBaseUrl;
+  const currency = uriInfoHook.response.exchange?.currency ?? scope?.currency;
+  const exchangeByUrl = uriInfoHook.response.exchange?.exchangeBaseUrl;
   const exchangeList = uriInfoHook.response.exchanges.exchanges;
 
   const maybeAmount = uriInfoHook.response.amount ?? paramsAmount;
+  if (!currency) {
+    return {
+      status: "error",
+      error: alertFromError(
+        i18n,
+        i18n.str`Could not load the list of exchanges`,
+        {} as any,
+      ),
+    };
+  }
 
-  // if (!maybeAmount) {
-  //   const exchangeBaseUrl =
-  //     uriInfoHook.response.exchange?.exchangeBaseUrl ??
-  //     (exchangeList.length > 0 ? exchangeList[0].exchangeBaseUrl : undefined);
-  //   const currency =
-  //     uriInfoHook.response.exchange?.currency ??
-  //     (exchangeList.length > 0 ? exchangeList[0].currency : undefined);
-
-  //   if (!exchangeBaseUrl) {
-  //     return {
-  //       status: "error",
-  //       error: {
-  //         message: i18n.str`Can't withdraw from exchange`,
-  //         description: i18n.str`Missing base URL`,
-  //         cause: undefined,
-  //         context: {},
-  //         type: "error",
-  //       },
-  //     };
-  //   }
-  //   if (!currency) {
-  //     return {
-  //       status: "error",
-  //       error: {
-  //         message: i18n.str`Can't withdraw from exchange`,
-  //         description: i18n.str`Missing unknown currency`,
-  //         cause: undefined,
-  //         context: {},
-  //         type: "error",
-  //       },
-  //     };
-  //   }
-  //   return () => {
-  //     const { pushAlertOnError } = useAlertContext();
-  //     const [amount, setAmount] = useState<AmountJson>(
-  //       Amounts.zeroOfCurrency(currency),
-  //     );
-  //     const isValid = Amounts.isNonZero(amount);
-  //     return {
-  //       status: "select-amount",
-  //       currency,
-  //       exchangeBaseUrl,
-  //       error: undefined,
-  //       confirm: {
-  //         onClick: isValid
-  //           ? pushAlertOnError(async () => {
-  //               onAmountChanged(Amounts.stringify(amount));
-  //             })
-  //           : undefined,
-  //       },
-  //       amount: {
-  //         value: amount,
-  //         onInput: pushAlertOnError(async (e) => {
-  //           setAmount(e);
-  //         }),
-  //       },
-  //     };
-  //   };
-  // }
-  const chosenAmount = maybeAmount ?? Amounts.zeroOfCurrency(scope.currency);
+  const chosenAmount = maybeAmount ?? Amounts.zeroOfCurrency(currency);
 
   async function doManualWithdraw(
     exchange: string,
@@ -180,6 +139,12 @@ export function useComponentStateFromParams({
     };
   }
 
+  const exchangeScope = scope ?? {
+    type: ScopeType.Exchange,
+    currency: currency,
+    url: exchangeByUrl!,
+  };
+
   return () =>
     exchangeSelectionState(
       doManualWithdraw,
@@ -193,10 +158,11 @@ export function useComponentStateFromParams({
         bankFee: Amounts.zeroOfCurrency(chosenAmount.currency),
         editableAmount: true,
         editableExchange: true,
-        exchange: exchangeByTalerUri,
+        exchange: exchangeByUrl,
         exchangeList: exchangeList,
       },
       setUpdatedExchangeByUser,
+      exchangeScope,
     );
 }
 
@@ -312,10 +278,7 @@ export function useComponentStateFromURI({
     };
   }
 
-  if (
-    uriInfoHook.response.txInfo &&
-    uriInfoHook.response.status !== "pending"
-  ) {
+  if (uriInfoHook.response.status !== "pending") {
     const info = uriInfoHook.response.txInfo;
     const anotherWallet =
       info.txState.major === TransactionMajorState.Aborted &&
@@ -329,7 +292,22 @@ export function useComponentStateFromURI({
       error: undefined,
     };
   }
+  if (
+    uriInfoHook.response.txInfo.txState.major === TransactionMajorState.Failed
+  ) {
+    return {
+      status: "already-completed",
+      thisWallet: true,
+      redirectToTx: () => onSuccess(uriInfoHook.response.txInfo.transactionId),
+      error: undefined,
+      operationState: "aborted",
+    };
+  }
 
+  const exchangeScope: ScopeInfo = {
+    type: ScopeType.Global,
+    currency: bwi.currency,
+  };
   return useCallback(() => {
     return exchangeSelectionState(
       doManagedWithdraw,
@@ -347,6 +325,7 @@ export function useComponentStateFromURI({
         exchangeList: bwi.possibleExchanges,
       },
       setUpdatedExchangeByUser,
+      exchangeScope,
     );
   }, []);
 }
@@ -375,10 +354,11 @@ function exchangeSelectionState(
   talerWithdrawUri: string | undefined,
   wInfo: WithdrawalInfo,
   onExchangeUpdated: (ex: string) => void,
+  scope: ScopeInfo,
 ): RecursiveState<State> {
   const api = useBackendContext();
   const selectedExchange = useSelectedExchange({
-    currency: wInfo.currency,
+    scope,
     defaultExchange: wInfo.exchange,
     list: wInfo.exchangeList,
   });
