@@ -39,10 +39,10 @@ import {
   getAmountWithFee,
 } from "../../wallet/Transaction.js";
 import { State } from "./index.js";
-import { Amounts } from "@gnu-taler/taler-util";
 
 export function FinalStateOperation(state: State.AlreadyCompleted): VNode {
   const { i18n } = useTranslationContext();
+
   // document.location.href = res.confirmTransferUrl
   if (state.thisWallet) {
     switch (state.operationState) {
@@ -280,44 +280,3 @@ function WithdrawWithMobile({
   );
 }
 
-export function SelectAmountView({
-  amount,
-  exchangeBaseUrl,
-  confirm,
-}: State.SelectAmount): VNode {
-  const { i18n } = useTranslationContext();
-  return (
-    <Fragment>
-      <section style={{ textAlign: "left" }}>
-        <Part
-          title={
-            <div
-              style={{
-                display: "flex",
-                alignItems: "center",
-              }}
-            >
-              <i18n.Translate>Exchange</i18n.Translate>
-            </div>
-          }
-          text={<ExchangeDetails exchange={exchangeBaseUrl} />}
-          kind="neutral"
-          big
-        />
-        <Grid container columns={2} justifyContent="space-between">
-          <AmountField label={i18n.str`Amount`} required handler={amount} />
-        </Grid>
-      </section>
-      <section>
-        <Button
-          variant="contained"
-          color="info"
-          disabled={!confirm.onClick}
-          onClick={confirm.onClick}
-        >
-          <i18n.Translate>See details</i18n.Translate>
-        </Button>
-      </section>
-    </Fragment>
-  );
-}
