@@ -48,14 +48,13 @@ export type HookResponseWithRetry<T> =
 
 export function useAsyncAsHook<T>(
   fn: () => Promise<T | false>,
-  deps?: any[],
+  deps?: unknown[],
 ): HookResponseWithRetry<T> {
   const [result, setHookResponse] = useState<HookResponse<T>>(undefined);
 
   const args = useMemo(
     () => ({
       fn,
-      // eslint-disable-next-line react-hooks/exhaustive-deps
     }),
     deps || [],
   );
