@@ -14,22 +14,13 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
-import { buildStorageKey, useLocalStorage } from "@gnu-taler/web-util/browser";
-import { Settings, defaultSettings } from "../platform/api.js";
 import {
   Codec,
   buildCodecForObject,
   codecForBoolean,
 } from "@gnu-taler/taler-util";
-
-function parse_json_or_undefined<T>(str: string | undefined): T | undefined {
-  if (str === undefined) return undefined;
-  try {
-    return JSON.parse(str);
-  } catch {
-    return undefined;
-  }
-}
+import { buildStorageKey, useLocalStorage } from "@gnu-taler/web-util/browser";
+import { Settings, defaultSettings } from "../platform/api.js";
 
 export const codecForSettings = (): Codec<Settings> =>
   buildCodecForObject<Settings>()
@@ -46,6 +37,7 @@ export const codecForSettings = (): Codec<Settings> =>
     .property("showExchangeManagement", codecForBoolean())
     .property("selectTosFormat", codecForBoolean())
     .property("showWalletActivity", codecForBoolean())
+    .property("walletEnableV1Contracts", codecForBoolean())
     .build("Settings");
 
 const SETTINGS_KEY = buildStorageKey("wallet-settings", codecForSettings());
