@@ -16,7 +16,8 @@
 
 import { BackgroundPlatformAPI } from "./api.js";
 
-export let platform: BackgroundPlatformAPI = undefined as any;
+// it should never be undefined :)
+export let platform: BackgroundPlatformAPI = undefined!;
 export function setupPlatform(impl: BackgroundPlatformAPI): void {
   platform = impl;
 }
