@@ -234,6 +234,9 @@ function openWalletURIFromPopup(uri: TalerUri): void {
     case TalerUriAction.DevExperiment:
       logger.warn(`taler://dev-experiment URIs are not allowed in headers`);
       return;
+    case TalerUriAction.WithdrawalTransferResult:
+      logger.warn(`taler://withdrawal-transfer-result URIs are not allowed in headers`);
+      return;
     default: {
       const error: never = uri;
       logger.warn(
