@@ -20,21 +20,24 @@
  * @author sebasjm
  */
 
+import { ScopeInfo, stringifyScopeInfoShort } from "@gnu-taler/taler-util";
 import {
   TranslationProvider,
   useTranslationContext,
+  encodeCrockForURI,
+  decodeCrockFromURI,
 } from "@gnu-taler/web-util/browser";
 import { createHashHistory } from "history";
-import { ComponentChildren, Fragment, h, VNode } from "preact";
-import { route, Route, Router } from "preact-router";
+import { ComponentChildren, Fragment, VNode, h } from "preact";
+import { Route, Router, route } from "preact-router";
 import { useEffect, useState } from "preact/hooks";
+import { Pages, PopupNavBar, PopupNavBarOptions } from "../NavigationBar.js";
 import PendingTransactions from "../components/PendingTransactions.js";
 import { PopupBox } from "../components/styled/index.js";
 import { AlertProvider } from "../context/alert.js";
 import { IoCProviderForRuntime } from "../context/iocContext.js";
 import { useTalerActionURL } from "../hooks/useTalerActionURL.js";
 import { strings } from "../i18n/strings.js";
-import { Pages, PopupNavBar, PopupNavBarOptions } from "../NavigationBar.js";
 import { platform } from "../platform/foreground.js";
 import { BackupPage } from "../wallet/BackupPage.js";
 import { ProviderDetailPage } from "../wallet/ProviderDetailPage.js";
@@ -59,7 +62,7 @@ function ApplicationView(): VNode {
 
   useEffect(() => {
     if (actionUri) {
-      route(Pages.cta({ action: encodeURIComponent(actionUri) }));
+      route(Pages.cta({ action: encodeCrockForURI(actionUri) }));
     }
   }, [actionUri]);
 
@@ -68,7 +71,7 @@ function ApplicationView(): VNode {
   }
 
   function redirectToURL(str: string): void {
-    platform.openNewURLFromPopup(new URL(str))
+    platform.openNewURLFromPopup(new URL(str));
   }
 
   return (
@@ -76,14 +79,34 @@ function ApplicationView(): VNode {
       <Route
         path={Pages.balance}
         component={() => (
-          <PopupTemplate path="balance" goToTransaction={redirectToTxInfo} goToURL={redirectToURL}>
+          <PopupTemplate
+            path="balance"
+            goToTransaction={redirectToTxInfo}
+            goToURL={redirectToURL}
+          >
             <BalancePage
-              goToWalletManualWithdraw={() => redirectTo(Pages.receiveCash({}))}
-              goToWalletDeposit={(currency: string) =>
-                redirectTo(Pages.sendCash({ amount: `${currency}:0` }))
+              goToWalletManualWithdraw={(scope?: ScopeInfo) => {
+                return redirectTo(
+                  Pages.receiveCash({
+                    scope: !scope
+                      ? undefined
+                      : encodeCrockForURI(stringifyScopeInfoShort(scope)),
+                  }),
+                );
+              }}
+              goToWalletDeposit={(scope: ScopeInfo) =>
+                redirectTo(
+                  Pages.sendCash({
+                    scope: encodeCrockForURI(stringifyScopeInfoShort(scope)),
+                  }),
+                )
               }
-              goToWalletHistory={(currency: string) =>
-                redirectTo(Pages.balanceHistory({ currency }))
+              goToWalletHistory={(scope: ScopeInfo) =>
+                redirectTo(
+                  Pages.balanceHistory({
+                    scope: encodeCrockForURI(stringifyScopeInfoShort(scope)),
+                  }),
+                )
               }
             />
           </PopupTemplate>
@@ -98,7 +121,7 @@ function ApplicationView(): VNode {
           return (
             <PopupTemplate goToURL={redirectToURL}>
               <TalerActionFound
-                url={decodeURIComponent(action)}
+                url={decodeCrockFromURI(action)}
                 onDismiss={() => {
                   setDismissed(true);
                   return redirectTo(Pages.balance);
@@ -112,7 +135,11 @@ function ApplicationView(): VNode {
       <Route
         path={Pages.backup}
         component={() => (
-          <PopupTemplate path="backup" goToTransaction={redirectToTxInfo} goToURL={redirectToURL}>
+          <PopupTemplate
+            path="backup"
+            goToTransaction={redirectToTxInfo}
+            goToURL={redirectToURL}
+          >
             <BackupPage
               onAddProvider={() => redirectTo(Pages.backupProviderAdd)}
             />
@@ -127,9 +154,9 @@ function ApplicationView(): VNode {
               onPayProvider={(uri: string) =>
                 redirectTo(`${Pages.ctaPay}?talerPayUri=${uri}`)
               }
-              onWithdraw={(amount: string) =>
-                redirectTo(Pages.receiveCash({ amount }))
-              }
+              onWithdraw={async (_amount: string) => {
+                // redirectTo(Pages.receiveCash({ amount }))
+              }}
               pid={pid}
               onBack={() => redirectTo(Pages.backup)}
             />
@@ -219,7 +246,10 @@ function PopupTemplate({
 }): VNode {
   return (
     <Fragment>
-      <PendingTransactions goToTransaction={goToTransaction} goToURL={goToURL} />
+      <PendingTransactions
+        goToTransaction={goToTransaction}
+        goToURL={goToURL}
+      />
       <PopupNavBar path={path} />
       <PopupBox>
         <AlertProvider>{children}</AlertProvider>
