@@ -14,6 +14,8 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
+import { encodeCrockForURI } from "@gnu-taler/web-util/browser";
+
 /**
  * WARNING
  *
@@ -22,10 +24,10 @@
  */
 (() => {
   const logger = {
-    debug: (...msg: any[]) => { },
-    info: (...msg: any[]) =>
+    debug: (..._msg: unknown[]) => { },
+    info: (...msg: unknown[]) =>
       console.log(`${new Date().toISOString()} TALER`, ...msg),
-    error: (...msg: any[]) =>
+    error: (...msg: unknown[]) =>
       console.error(`${new Date().toISOString()} TALER`, ...msg),
   };
 
@@ -76,13 +78,17 @@
         return undefined;
       }
       const host = `${config.protocol}//${config.hostname}`;
-      const path = `static/wallet.html#/taler-uri/${encodeURIComponent(uri)}`;
+      // FIXME: mem leak problems
+      // const path = `static/wallet.html#/taler-uri/${encodeCrockForURI(uri)}`;
+      const path = `static/wallet.html#/taler-uri-simple/${encodeURIComponent(uri)}`;
       return `${host}/${path}`;
     }
 
     function anchorOnClick(ev: MouseEvent) {
       if (!(ev.currentTarget instanceof Element)) {
-        logger.debug(`onclick: registered in a link that is not an HTML element`);
+        logger.debug(
+          `onclick: registered in a link that is not an HTML element`,
+        );
         return;
       }
       const hrefAttr = ev.currentTarget.attributes.getNamedItem("href");
@@ -95,7 +101,9 @@
         targetAttr && targetAttr.value ? targetAttr.value : "_self";
       const page = convertURIToWebExtensionPath(hrefAttr.value);
       if (!page) {
-        logger.debug(`onclick: could not convert "${hrefAttr.value}" into path`);
+        logger.debug(
+          `onclick: could not convert "${hrefAttr.value}" into path`,
+        );
         return;
       }
       // we can use window.open, but maybe some browser will block it?
@@ -118,7 +126,7 @@
 
     function checkForNewAnchors(
       mutations: MutationRecord[],
-      observer: MutationObserver,
+      _observer: MutationObserver,
     ) {
       mutations.forEach((mut) => {
         if (mut.type === "childList") {
@@ -137,7 +145,7 @@
      * Register the anchor handler when found
      */
     function registerProtocolHandler() {
-      if (document.body) overrideAllAnchor(document.body)
+      if (document.body) overrideAllAnchor(document.body);
       new MutationObserver(checkForNewAnchors).observe(document, {
         childList: true,
         subtree: true,
@@ -179,7 +187,8 @@
     };
 
     if (apiEnabled) {
-      //@ts-ignore
+      // @ts-expect-error we now that `taler` doesn't exist. 
+      // we are creating the property
       window.taler = taler;
     }
 
@@ -196,5 +205,4 @@
   }
 
   start();
-})()
-
+})();
