@@ -14,14 +14,25 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
-import { NotificationType, TalerBankIntegrationHttpClient, TalerCoreBankHttpClient, TalerRevenueHttpClient, TalerWireGatewayHttpClient, WalletNotification } from "@gnu-taler/taler-util";
+import {
+  NotificationType,
+  TalerBankIntegrationHttpClient,
+  TalerCoreBankHttpClient,
+  TalerRevenueHttpClient,
+  TalerWireGatewayHttpClient,
+  WalletNotification,
+} from "@gnu-taler/taler-util";
 import {
   WalletCoreApiClient,
   WalletCoreOpKeys,
   WalletCoreRequestType,
   WalletCoreResponseType,
 } from "@gnu-taler/taler-wallet-core";
-import { ApiContextProvider, TranslationProvider, defaultRequestHandler } from "@gnu-taler/web-util/browser";
+import {
+  ApiContextProvider,
+  TranslationProvider,
+  defaultRequestHandler,
+} from "@gnu-taler/web-util/browser";
 import {
   ComponentChildren,
   FunctionalComponent,
@@ -115,9 +126,7 @@ export function createWalletApiMock(): {
       },
     }),
     listener: {
-      trigger: () => {
-
-      },
+      trigger: () => {},
       onUpdateNotification(
         mTypes: NotificationType[],
         callback: ((d: WalletNotification) => void) | undefined,
@@ -189,9 +198,9 @@ export function createWalletApiMock(): {
       request: defaultRequestHandler,
       bankCore: new TalerCoreBankHttpClient("/"),
       bankIntegration: new TalerBankIntegrationHttpClient("/"),
-      bankWire: new TalerWireGatewayHttpClient("/",""),
-      bankRevenue: new TalerRevenueHttpClient("/"),    
-    }
+      bankWire: new TalerWireGatewayHttpClient("/"),
+      bankRevenue: new TalerRevenueHttpClient("/"),
+    };
     children = create(ApiContextProvider, { value, children }, children);
     children = create(
       TranslationProvider,
