@@ -14,7 +14,7 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
-import { ExchangeEntryStatus, TalerExchangeHttpClient, canonicalizeBaseUrl, opKnownFailureWithBody } from "@gnu-taler/taler-util";
+import { ExchangeEntryStatus, TalerExchangeHttpClient, canonicalizeBaseUrl, opKnownFailure, opKnownFailureWithBody, succeedOrThrow } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { BrowserFetchHttpLib } from "@gnu-taler/web-util/browser";
 import { useCallback, useEffect, useState } from "preact/hooks";
@@ -22,7 +22,7 @@ import { useBackendContext } from "../../context/backend.js";
 import { useAsyncAsHook } from "../../hooks/useAsyncAsHook.js";
 import { withSafe } from "../../mui/handlers.js";
 import { RecursiveState } from "../../utils/index.js";
-import { CheckExchangeErrors, Props, State } from "./index.js";
+import { Props, State } from "./index.js";
 
 function urlFromInput(str: string): URL {
   let result: URL;
@@ -59,11 +59,11 @@ export function useComponentState({ onBack, currency, noDebounce }: Props): Recu
       const checkExchangeBaseUrl_memo = useCallback(async function checkExchangeBaseUrl(str: string) {
         const baseUrl = urlFromInput(str)
         if (baseUrl.protocol !== "http:" && baseUrl.protocol !== "https:") {
-          return opKnownFailureWithBody<CheckExchangeErrors>("invalid-protocol", undefined)
+          return opKnownFailure("invalid-protocol"as const)
         }
         const found = used.findIndex((e) => e.exchangeBaseUrl === baseUrl.href);
         if (found !== -1) {
-          return opKnownFailureWithBody<CheckExchangeErrors>("already-active", undefined);
+          return opKnownFailure("already-active"as const);
         }
 
         /**
@@ -81,16 +81,12 @@ export function useComponentState({ onBack, currency, noDebounce }: Props): Recu
          * │                   Types have separate declarations of a private property '_isCancelled'.
          *
          */
-        const api = new TalerExchangeHttpClient(baseUrl.href, new BrowserFetchHttpLib() as any);
-        const config = await api.getConfig()
-        if (config.type === "fail") {
-          return opKnownFailureWithBody<CheckExchangeErrors>("not-found", undefined)
-        }
-        if (!api.isCompatible(config.body.version)) {
-          return opKnownFailureWithBody<CheckExchangeErrors>("invalid-version", config.body.version)
-        }
-        if (currency !== undefined && currency !== config.body.currency) {
-          return opKnownFailureWithBody<CheckExchangeErrors>("invalid-currency", config.body.currency)
+        const api = new TalerExchangeHttpClient(baseUrl.href, {
+          httpClient: new BrowserFetchHttpLib(),
+        });
+        const config = succeedOrThrow(await api.getConfig())
+        if (currency !== undefined && currency !== config.currency) {
+          return opKnownFailureWithBody("invalid-currency"as const, config.currency)
         }
         const keys = await api.getKeys()
         return keys
