@@ -19,8 +19,6 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import * as tests from "@gnu-taler/web-util/testing";
-import { ConfirmView, VerifyView } from "./views.js";
 
 export default {
   title: "example",
