@@ -29,9 +29,8 @@ import {
   parsePaytoUri,
   parseScopeInfoShort,
   parseTalerUri,
-  stringifyPaytoUri,
   stringifyScopeInfoShort,
-  stringifyTalerUri,
+  stringifyTalerUri
 } from "@gnu-taler/taler-util";
 import {
   TranslationProvider,
@@ -50,8 +49,10 @@ import {
   getPathnameForTalerURI,
 } from "../NavigationBar.js";
 import { AlertView, CurrentAlerts } from "../components/CurrentAlerts.js";
+import { EnabledBySettings } from "../components/EnabledBySettings.js";
 import { LogoHeader } from "../components/LogoHeader.js";
 import PendingTransactions from "../components/PendingTransactions.js";
+import { WalletActivity } from "../components/WalletActivity.js";
 import {
   LinkPrimary,
   RedBanner,
@@ -62,6 +63,7 @@ import {
 import { AlertProvider } from "../context/alert.js";
 import { IoCProviderForRuntime } from "../context/iocContext.js";
 import { DepositPage as DepositPageCTA } from "../cta/Deposit/index.js";
+import { DevExperimentPage } from "../cta/DevExperiment/index.js";
 import { InvoiceCreatePage } from "../cta/InvoiceCreate/index.js";
 import { InvoicePayPage } from "../cta/InvoicePay/index.js";
 import { PaymentPage } from "../cta/Payment/index.js";
@@ -79,22 +81,20 @@ import { strings } from "../i18n/strings.js";
 import CloseIcon from "../svg/close_24px.inline.svg";
 import { AddBackupProviderPage } from "./AddBackupProvider/index.js";
 import { AddExchange } from "./AddExchange/index.js";
+import { ConfirmAddExchangeView } from "./AddExchange/views.js";
 import { BackupPage } from "./BackupPage.js";
 import { DepositPage } from "./DepositPage/index.js";
 import { DestinationSelectionPage } from "./DestinationSelection/index.js";
 import { DeveloperPage } from "./DeveloperPage.js";
 import { HistoryPage } from "./History.js";
+import { ManageAccountPage } from "./ManageAccount/index.js";
 import { NotificationsPage } from "./Notifications/index.js";
 import { ProviderDetailPage } from "./ProviderDetailPage.js";
 import { QrReaderPage } from "./QrReader.js";
 import { SettingsPage } from "./Settings.js";
+import { SupportedBanksForAccount } from "./SupportedBanksForAccount.js";
 import { TransactionPage } from "./Transaction.js";
 import { WelcomePage } from "./Welcome.js";
-import { WalletActivity } from "../components/WalletActivity.js";
-import { EnabledBySettings } from "../components/EnabledBySettings.js";
-import { DevExperimentPage } from "../cta/DevExperiment/index.js";
-import { ConfirmAddExchangeView } from "./AddExchange/views.js";
-import { ManageAccountPage } from "./ManageAccount/index.js";
 
 export function Application(): VNode {
   const { i18n } = useTranslationContext();
@@ -131,7 +131,9 @@ export function Application(): VNode {
                   onDetected={(talerActionUrl: TalerUri) => {
                     redirectTo(
                       Pages.defaultCta({
-                        uri: encodeCrockForURI(stringifyTalerUri(talerActionUrl)),
+                        uri: encodeCrockForURI(
+                          stringifyTalerUri(talerActionUrl),
+                        ),
                       }),
                     );
                   }}
@@ -188,14 +190,18 @@ export function Application(): VNode {
                   goToWalletDeposit={(scope: ScopeInfo) =>
                     redirectTo(
                       Pages.sendCash({
-                        scope: encodeCrockForURI(stringifyScopeInfoShort(scope)),
+                        scope: encodeCrockForURI(
+                          stringifyScopeInfoShort(scope),
+                        ),
                       }),
                     )
                   }
                   goToWalletManualWithdraw={(scope?: ScopeInfo) =>
                     redirectTo(
                       Pages.receiveCash({
-                        scope: !scope ? undefined : encodeCrockForURI(stringifyScopeInfoShort(scope)),
+                        scope: !scope
+                          ? undefined
+                          : encodeCrockForURI(stringifyScopeInfoShort(scope)),
                       }),
                     )
                   }
@@ -221,14 +227,18 @@ export function Application(): VNode {
                   goToWalletDeposit={(scope: ScopeInfo) =>
                     redirectTo(
                       Pages.sendCash({
-                        scope: encodeCrockForURI(stringifyScopeInfoShort(scope)),
+                        scope: encodeCrockForURI(
+                          stringifyScopeInfoShort(scope),
+                        ),
                       }),
                     )
                   }
                   goToWalletManualWithdraw={(scope?: ScopeInfo) =>
                     redirectTo(
                       Pages.receiveCash({
-                        scope: !scope ? undefined : encodeCrockForURI(stringifyScopeInfoShort(scope)),
+                        scope: !scope
+                          ? undefined
+                          : encodeCrockForURI(stringifyScopeInfoShort(scope)),
                       }),
                     )
                   }
@@ -238,11 +248,7 @@ export function Application(): VNode {
           />
           <Route
             path={Pages.sendCash.pattern}
-            component={({
-              scope,
-            }: {
-              scope?: string;
-            }) => {
+            component={({ scope }: { scope?: string }) => {
               if (!scope) return <Redirect to={Pages.balanceHistory({})} />;
               const s = parseScopeInfoShort(decodeCrockFromURI(scope));
               if (!s) return <Redirect to={Pages.balanceHistory({})} />;
@@ -256,7 +262,7 @@ export function Application(): VNode {
                       redirectTo(
                         Pages.ctaDeposit({
                           scope: encodeCrockForURI(stringifyScopeInfoShort(s)),
-                          account: encodeCrockForURI(stringifyPaytoUri(p)),
+                          account: encodeCrockForURI(p),
                         }),
                       )
                     }
@@ -281,26 +287,32 @@ export function Application(): VNode {
           />
           <Route
             path={Pages.bankManange.pattern}
-            component={({
-              scope,
-            }: {
-              scope?: string;
-            }) => {
-              const s = !scope ? undefined : parseScopeInfoShort(decodeCrockFromURI(scope));
-              if (!s) return <div>missing scope</div>
+            component={({ scope }: { scope?: string }) => {
+              const s = !scope
+                ? undefined
+                : parseScopeInfoShort(decodeCrockFromURI(scope));
+              if (!s) return <div>missing scope</div>;
 
               return (
                 <WalletTemplate path="balance" goToURL={redirectToURL}>
                   <ManageAccountPage
                     scope={s}
-                    onAccountAdded={(account) => redirectTo(Pages.ctaDeposit({
-                      scope: encodeCrockForURI(stringifyScopeInfoShort(s)),
-                      account: encodeCrockForURI(stringifyPaytoUri(account)),
-                    }))}
+                    onAccountSelected={(account) =>
+                      redirectTo(
+                        Pages.ctaDeposit({
+                          scope: encodeCrockForURI(stringifyScopeInfoShort(s)),
+                          account: encodeCrockForURI(account),
+                        }),
+                      )
+                    }
                     onCancel={() => {
-                      redirectTo(Pages.balanceHistory({
-                        scope: encodeCrockForURI(stringifyScopeInfoShort(s)),
-                      }))
+                      redirectTo(
+                        Pages.balanceHistory({
+                          scope: !scope
+                            ? undefined
+                            : encodeCrockForURI(stringifyScopeInfoShort(s)),
+                        }),
+                      );
                     }}
                   />
                 </WalletTemplate>
@@ -329,11 +341,7 @@ export function Application(): VNode {
           />
           <Route
             path={Pages.receiveCash.pattern}
-            component={({
-              scope,
-            }: {
-              scope?: string;
-            }) => {
+            component={({ scope }: { scope?: string }) => {
               const s = !scope
                 ? undefined
                 : parseScopeInfoShort(decodeCrockFromURI(scope));
@@ -343,14 +351,14 @@ export function Application(): VNode {
                   <DestinationSelectionPage
                     type="get"
                     scope={s}
-                    goToWalletManualWithdraw={(s,) =>
+                    goToWalletManualWithdraw={(s) =>
                       redirectTo(
-                        Pages.ctaWithdrawManual({
+                        Pages.ctaWithdrawManualForScope({
                           scope: encodeCrockForURI(stringifyScopeInfoShort(s)),
                         }),
                       )
                     }
-                    goToWalletWalletInvoice={(s,) =>
+                    goToWalletWalletInvoice={(s) =>
                       redirectTo(
                         Pages.ctaInvoiceCreate({
                           scope: encodeCrockForURI(stringifyScopeInfoShort(s)),
@@ -369,10 +377,12 @@ export function Application(): VNode {
               <WalletTemplate path="balance" goToURL={redirectToURL}>
                 <TransactionPage
                   tid={tid}
-                  goToWalletHistory={(scope: ScopeInfo) =>
+                  goToWalletHistory={(scope?: ScopeInfo) =>
                     redirectTo(
                       Pages.balanceHistory({
-                        scope: encodeCrockForURI(stringifyScopeInfoShort(scope)),
+                        scope: !scope
+                          ? undefined
+                          : encodeCrockForURI(stringifyScopeInfoShort(scope)),
                       }),
                     )
                   }
@@ -383,34 +393,36 @@ export function Application(): VNode {
 
           <Route
             path={Pages.balanceDeposit.pattern}
-            component={({
-              scope,
-            }: {
-              scope: string;
-            }) => {
-              const s = parseScopeInfoShort(decodeCrockFromURI(scope))
+            component={({ scope }: { scope: string }) => {
+              const s = parseScopeInfoShort(decodeCrockFromURI(scope));
               if (!s) {
-                return <div>missing scope</div>
+                return <div>missing scope</div>;
               }
-              return <WalletTemplate path="balance" goToURL={redirectToURL}>
-                <DepositPage
-                  scope={s}
-                  onCancel={(scope: ScopeInfo) => {
-                    redirectTo(
-                      Pages.balanceHistory({
-                        scope: encodeCrockForURI(stringifyScopeInfoShort(scope)),
-                      }),
-                    );
-                  }}
-                  onSuccess={(scope: ScopeInfo) => {
-                    redirectTo(
-                      Pages.balanceHistory({
-                        scope: encodeCrockForURI(stringifyScopeInfoShort(scope)),
-                      }),
-                    );
-                  }}
-                />
-              </WalletTemplate>
+              return (
+                <WalletTemplate path="balance" goToURL={redirectToURL}>
+                  <DepositPage
+                    scope={s}
+                    onCancel={(scope: ScopeInfo) => {
+                      redirectTo(
+                        Pages.balanceHistory({
+                          scope: encodeCrockForURI(
+                            stringifyScopeInfoShort(scope),
+                          ),
+                        }),
+                      );
+                    }}
+                    onSuccess={(scope: ScopeInfo) => {
+                      redirectTo(
+                        Pages.balanceHistory({
+                          scope: encodeCrockForURI(
+                            stringifyScopeInfoShort(scope),
+                          ),
+                        }),
+                      );
+                    }}
+                  />
+                </WalletTemplate>
+              );
             }}
           />
 
@@ -579,7 +591,9 @@ export function Application(): VNode {
             component={({ talerUri }: { talerUri: string }) => (
               <CallToActionTemplate title={i18n.str`Digital cash withdrawal`}>
                 <WithdrawPageFromURI
-                  talerWithdrawUri={decodeCrockFromURI(talerUri)}
+                  talerWithdrawUri={
+                    !talerUri ? undefined : decodeCrockFromURI(talerUri)
+                  }
                   cancel={() => redirectTo(Pages.balance)}
                   onSuccess={(tid: string) =>
                     redirectTo(Pages.balanceTransaction({ tid }))
@@ -591,13 +605,39 @@ export function Application(): VNode {
           <Route
             path={Pages.ctaWithdrawManual.pattern}
             component={({
-              scope,
+              // scope,
               amount,
               talerUri,
             }: {
-              scope: string;
+              // scope: string;
               amount: string;
               talerUri: string;
+            }) => {
+              return (
+                <CallToActionTemplate title={i18n.str`Digital cash withdrawal`}>
+                  <WithdrawPageFromParams
+                    scope={undefined}
+                    talerExchangeWithdrawUri={
+                      !talerUri ? undefined : decodeCrockFromURI(talerUri)
+                    }
+                    amount={Amounts.parse(amount)}
+                    cancel={() => redirectTo(Pages.balance)}
+                    onSuccess={(tid: string) =>
+                      redirectTo(Pages.balanceTransaction({ tid }))
+                    }
+                  />
+                </CallToActionTemplate>
+              );
+            }}
+          />
+          <Route
+            path={Pages.ctaWithdrawManualForScope.pattern}
+            component={({
+              scope,
+              amount,
+            }: {
+              scope: string;
+              amount: string;
             }) => {
               if (!scope) return <Redirect to={Pages.balanceHistory({})} />;
               const s = parseScopeInfoShort(decodeCrockFromURI(scope));
@@ -606,9 +646,9 @@ export function Application(): VNode {
               return (
                 <CallToActionTemplate title={i18n.str`Digital cash withdrawal`}>
                   <WithdrawPageFromParams
+                    talerExchangeWithdrawUri={undefined}
                     scope={s}
-                    talerExchangeWithdrawUri={talerUri}
-                    amount={amount}
+                    amount={Amounts.parse(amount)}
                     cancel={() => redirectTo(Pages.balance)}
                     onSuccess={(tid: string) =>
                       redirectTo(Pages.balanceTransaction({ tid }))
@@ -624,30 +664,31 @@ export function Application(): VNode {
               scope,
               account,
             }: {
-              scope: string,
+              scope: string;
               account: string;
             }) => {
-              const s = parseScopeInfoShort(decodeCrockFromURI(scope))
+              const s = parseScopeInfoShort(decodeCrockFromURI(scope));
               if (!s) {
-                return <div>missing scope</div>
+                return <div>missing scope</div>;
               }
-              const p = parsePaytoUri(decodeCrockFromURI(account))
+              const p = parsePaytoUri(decodeCrockFromURI(account));
               if (!p) {
-                return <div>missing account</div>
+                return <div>missing account</div>;
               }
 
-              return <CallToActionTemplate title={i18n.str`Digital cash deposit`} >
-                <DepositPageCTA
-                  scope={s}
-                  account={p}
-                  cancel={() => redirectTo(Pages.balance)}
-                  onSuccess={(tid: string) =>
-                    redirectTo(Pages.balanceTransaction({ tid }))
-                  }
-                />
-              </CallToActionTemplate>
-            }
-            }
+              return (
+                <CallToActionTemplate title={i18n.str`Digital cash deposit`}>
+                  <DepositPageCTA
+                    scope={s}
+                    account={p}
+                    cancel={() => redirectTo(Pages.balance)}
+                    onSuccess={(tid: string) =>
+                      redirectTo(Pages.balanceTransaction({ tid }))
+                    }
+                  />
+                </CallToActionTemplate>
+              );
+            }}
           />
           <Route
             path={Pages.ctaInvoiceCreate.pattern}
@@ -679,42 +720,60 @@ export function Application(): VNode {
           />
           <Route
             path={Pages.ctaInvoicePay}
-            component={({ talerUri }: { talerUri: string }) => (
-              <CallToActionTemplate title={i18n.str`Digital cash invoice`}>
-                <InvoicePayPage
-                  talerPayPullUri={decodeCrockFromURI(talerUri)}
-                  goToWalletManualWithdraw={(_amount?: string) =>
-                    // FIXME: use receiveCashForInvoice
-                    redirectTo(Pages.receiveCash({}))
-                  }
-                  onClose={() => redirectTo(Pages.balance)}
-                  onSuccess={(tid: string) =>
-                    redirectTo(Pages.balanceTransaction({ tid }))
-                  }
-                />
-              </CallToActionTemplate>
-            )}
+            component={({ talerUri }: { talerUri: string }) => {
+              const uri = decodeCrockFromURI(talerUri);
+              if (!uri) {
+                return <div>missing taler uri</div>;
+              }
+
+              return (
+                <CallToActionTemplate title={i18n.str`Digital cash invoice`}>
+                  <InvoicePayPage
+                    talerPayPullUri={uri}
+                    goToWalletManualWithdraw={(_amount?: string) =>
+                      // FIXME: use receiveCashForInvoice
+                      redirectTo(Pages.receiveCash({}))
+                    }
+                    onClose={() => redirectTo(Pages.balance)}
+                    onSuccess={(tid: string) =>
+                      redirectTo(Pages.balanceTransaction({ tid }))
+                    }
+                  />
+                </CallToActionTemplate>
+              );
+            }}
           />
           <Route
             path={Pages.ctaTransferPickup}
-            component={({ talerUri }: { talerUri: string }) => (
-              <CallToActionTemplate title={i18n.str`Digital cash transfer`}>
-                <TransferPickupPage
-                  talerPayPushUri={decodeCrockFromURI(talerUri)}
-                  onClose={() => redirectTo(Pages.balance)}
-                  onSuccess={(tid: string) =>
-                    redirectTo(Pages.balanceTransaction({ tid }))
-                  }
-                />
-              </CallToActionTemplate>
-            )}
+            component={({ talerUri }: { talerUri: string }) => {
+              const uri = decodeCrockFromURI(talerUri);
+              if (!uri) {
+                return <div>missing taler uri</div>;
+              }
+
+              return (
+                <CallToActionTemplate title={i18n.str`Digital cash transfer`}>
+                  <TransferPickupPage
+                    talerPayPushUri={uri}
+                    onClose={() => redirectTo(Pages.balance)}
+                    onSuccess={(tid: string) =>
+                      redirectTo(Pages.balanceTransaction({ tid }))
+                    }
+                  />
+                </CallToActionTemplate>
+              );
+            }}
           />
           <Route
             path={Pages.ctaRecovery}
             component={({ talerRecoveryUri }: { talerRecoveryUri: string }) => (
               <CallToActionTemplate title={i18n.str`Digital cash recovery`}>
                 <RecoveryPage
-                  talerRecoveryUri={decodeCrockFromURI(talerRecoveryUri)}
+                  talerRecoveryUri={
+                    !talerRecoveryUri
+                      ? undefined
+                      : decodeCrockFromURI(talerRecoveryUri)
+                  }
                   onCancel={() => redirectTo(Pages.balance)}
                   onSuccess={() => redirectTo(Pages.backup)}
                 />
@@ -726,7 +785,9 @@ export function Application(): VNode {
             component={({ talerUri }: { talerUri: string }) => (
               <CallToActionTemplate title={i18n.str`Development experiment`}>
                 <DevExperimentPage
-                  talerExperimentUri={decodeCrockFromURI(talerUri)}
+                  talerExperimentUri={
+                    !talerUri ? undefined : decodeCrockFromURI(talerUri)
+                  }
                   onCancel={() => redirectTo(Pages.balanceHistory({}))}
                   onSuccess={() => redirectTo(Pages.balanceHistory({}))}
                 />
@@ -736,7 +797,9 @@ export function Application(): VNode {
           <Route
             path={Pages.ctaAddExchange}
             component={({ talerUri }: { talerUri: string }) => {
-              const tUri = parseTalerUri(decodeCrockFromURI(talerUri));
+              const tUri = parseTalerUri(
+                decodeCrockFromURI(talerUri).toLowerCase(),
+              );
               const baseUrl =
                 tUri?.type === TalerUriAction.AddExchange
                   ? tUri.exchangeBaseUrl
@@ -758,6 +821,41 @@ export function Application(): VNode {
               );
             }}
           />
+          <Route
+            path={Pages.paytoBanks.pattern}
+            component={({ payto }: { payto: string }) => {
+              const pUri = parsePaytoUri(
+                decodeCrockFromURI(payto).toLowerCase(),
+              );
+              if (!pUri) {
+                redirectTo(Pages.balanceHistory({}));
+                return <div>invalid uri {pUri}</div>;
+              }
+              return (
+                <WalletTemplate goToURL={redirectToURL}>
+                  <SupportedBanksForAccount account={pUri} />
+                </WalletTemplate>
+              );
+            }}
+          />
+          <Route
+            path={Pages.paytoQrs.pattern}
+            component={({ payto }: { payto: string }) => {
+              const pUri = parsePaytoUri(
+                decodeCrockFromURI(payto).toLowerCase(),
+              );
+              if (!pUri) {
+                redirectTo(Pages.balanceHistory({}));
+                return <div>invalid uri {pUri}</div>;
+              }
+              return (
+                <WalletTemplate goToURL={redirectToURL}>
+                  {/* <AllQrsForAccount account={pUri} /> */}
+                  <pre>{JSON.stringify({ title: "QRS", pUri })}</pre>
+                </WalletTemplate>
+              );
+            }}
+          />
           {/**
            * NOT FOUND
            * all redirects should be at the end
@@ -776,7 +874,7 @@ export function Application(): VNode {
           <WalletActivity />
         </EnabledBySettings>
       </IoCProviderForRuntime>
-    </TranslationProvider >
+    </TranslationProvider>
   );
 }
 
