@@ -16,6 +16,7 @@
 
 import {
   AbsoluteTime,
+  HostPortPath,
   ProviderInfo,
   ProviderPaymentPaid,
   ProviderPaymentStatus,
@@ -136,7 +137,7 @@ export function BackupPage({ onAddProvider }: Props): VNode {
     );
     const str = stringifyRestoreUri({
       walletRootPriv: r.walletRootPriv,
-      providers: r.providers.map((p) => p.url),
+      providers: r.providers.map((p) => p.url as HostPortPath),
     });
     setRecoveryInfo(str);
   }
