@@ -23,10 +23,8 @@ import {
   AmountResponse,
   Amounts,
   AmountString,
-  parsePaytoUri,
   ScopeInfo,
   ScopeType,
-  stringifyPaytoUri
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import * as tests from "@gnu-taler/web-util/testing";
@@ -45,9 +43,8 @@ const withoutFee = (value: number): AmountResponse => ({
 
 const defaultScope: ScopeInfo = {
   type: ScopeType.Global,
-  currency
-}
-
+  currency,
+};
 
 const withSomeFee = (value: number, fee: number): AmountResponse => ({
   effectiveAmount: `${currency}:${value}` as AmountString,
@@ -56,7 +53,12 @@ const withSomeFee = (value: number, fee: number): AmountResponse => ({
 describe("DepositPage states", () => {
   it("should have status 'no-enough-balance' when balance is empty", async () => {
     const { handler, TestingContext } = createWalletApiMock();
-    const props = { scope: defaultScope, amount, onCancel: nullFunction, onSuccess: nullFunction };
+    const props = {
+      scope: defaultScope,
+      amount,
+      onCancel: nullFunction,
+      onSuccess: nullFunction,
+    };
 
     handler.addWalletCallResponse(WalletApiOperation.GetBalances, undefined, {
       balances: [
@@ -72,7 +74,7 @@ describe("DepositPage states", () => {
       ],
     });
     handler.addWalletCallResponse(
-      WalletApiOperation.ListKnownBankAccounts,
+      WalletApiOperation.ListBankAccounts,
       undefined,
       {
         accounts: [],
@@ -99,7 +101,11 @@ describe("DepositPage states", () => {
 
   it("should have status 'no-accounts' when balance is not empty and accounts is empty", async () => {
     const { handler, TestingContext } = createWalletApiMock();
-    const props = { scope: defaultScope, onCancel: nullFunction, onSuccess: nullFunction };
+    const props = {
+      scope: defaultScope,
+      onCancel: nullFunction,
+      onSuccess: nullFunction,
+    };
 
     handler.addWalletCallResponse(WalletApiOperation.GetBalances, undefined, {
       balances: [
@@ -115,7 +121,7 @@ describe("DepositPage states", () => {
       ],
     });
     handler.addWalletCallResponse(
-      WalletApiOperation.ListKnownBankAccounts,
+      WalletApiOperation.ListBankAccounts,
       undefined,
       {
         accounts: [],
@@ -141,21 +147,27 @@ describe("DepositPage states", () => {
   });
 
   const ibanPayto = {
-    uri: parsePaytoUri("payto://iban/ES8877998399652238")!,
-    kyc_completed: false,
-    currency: "EUR",
-    alias: "my iban account",
+    bankAccountId: "acct:123",
+    paytoUri: "payto://iban/ES8877998399652238",
+    kycCompleted: false,
+    currencies: ["EUR"],
+    label: "my iban account",
   };
   const talerBankPayto = {
-    uri: parsePaytoUri("payto://x-taler-bank/ES8877998399652238")!,
-    kyc_completed: false,
-    currency: "EUR",
-    alias: "my taler account",
+    bankAccountId: "acct:456",
+    paytoUri: "payto://x-taler-bank/ES8877998399652238",
+    kycCompleted: false,
+    currencies: ["EUR"],
+    label: "my taler account",
   };
 
   it("should have status 'ready' but unable to deposit ", async () => {
     const { handler, TestingContext } = createWalletApiMock();
-    const props = { scope: defaultScope, onCancel: nullFunction, onSuccess: nullFunction };
+    const props = {
+      scope: defaultScope,
+      onCancel: nullFunction,
+      onSuccess: nullFunction,
+    };
 
     handler.addWalletCallResponse(WalletApiOperation.GetBalances, undefined, {
       balances: [
@@ -171,7 +183,7 @@ describe("DepositPage states", () => {
       ],
     });
     handler.addWalletCallResponse(
-      WalletApiOperation.ListKnownBankAccounts,
+      WalletApiOperation.ListBankAccounts,
       undefined,
       {
         accounts: [ibanPayto],
@@ -197,7 +209,7 @@ describe("DepositPage states", () => {
           if (state.status !== "ready") expect.fail();
           expect(state.cancelHandler.onClick).not.undefined;
           expect(state.currency).eq(currency);
-          expect(state.account.value).eq(stringifyPaytoUri(ibanPayto.uri));
+          expect(state.account.value).eq(ibanPayto.paytoUri);
           expect(state.amount.value).deep.eq(Amounts.parseOrThrow("EUR:0"));
           expect(state.depositHandler.onClick).undefined;
         },
@@ -211,7 +223,11 @@ describe("DepositPage states", () => {
 
   it("should not be able to deposit more than the balance ", async () => {
     const { handler, TestingContext } = createWalletApiMock();
-    const props = { scope: defaultScope, onCancel: nullFunction, onSuccess: nullFunction };
+    const props = {
+      scope: defaultScope,
+      onCancel: nullFunction,
+      onSuccess: nullFunction,
+    };
 
     handler.addWalletCallResponse(WalletApiOperation.GetBalances, undefined, {
       balances: [
@@ -227,7 +243,7 @@ describe("DepositPage states", () => {
       ],
     });
     handler.addWalletCallResponse(
-      WalletApiOperation.ListKnownBankAccounts,
+      WalletApiOperation.ListBankAccounts,
       undefined,
       {
         accounts: [talerBankPayto, ibanPayto],
@@ -245,7 +261,7 @@ describe("DepositPage states", () => {
       withoutFee(0),
     );
 
-    const accountSelected = stringifyPaytoUri(ibanPayto.uri);
+    const accountSelected = ibanPayto.paytoUri;
 
     const hookBehavior = await tests.hookBehaveLikeThis(
       useComponentState,
@@ -261,7 +277,7 @@ describe("DepositPage states", () => {
           if (state.status !== "ready") expect.fail();
           expect(state.cancelHandler.onClick).not.undefined;
           expect(state.currency).eq(currency);
-          expect(state.account.value).eq(stringifyPaytoUri(talerBankPayto.uri));
+          expect(state.account.value).eq(talerBankPayto.paytoUri);
           expect(state.amount.value).deep.eq(Amounts.parseOrThrow("EUR:0"));
           expect(state.depositHandler.onClick).undefined;
           expect(state.totalFee).deep.eq(Amounts.parseOrThrow(`${currency}:0`));
@@ -297,7 +313,11 @@ describe("DepositPage states", () => {
 
   it("should calculate the fee upon entering amount ", async () => {
     const { handler, TestingContext } = createWalletApiMock();
-    const props = { scope: defaultScope, onCancel: nullFunction, onSuccess: nullFunction };
+    const props = {
+      scope: defaultScope,
+      onCancel: nullFunction,
+      onSuccess: nullFunction,
+    };
 
     handler.addWalletCallResponse(WalletApiOperation.GetBalances, undefined, {
       balances: [
@@ -313,7 +333,7 @@ describe("DepositPage states", () => {
       ],
     });
     handler.addWalletCallResponse(
-      WalletApiOperation.ListKnownBankAccounts,
+      WalletApiOperation.ListBankAccounts,
       undefined,
       {
         accounts: [talerBankPayto, ibanPayto],
@@ -327,15 +347,15 @@ describe("DepositPage states", () => {
     handler.addWalletCallResponse(
       WalletApiOperation.ConvertDepositAmount,
       undefined,
-      withSomeFee(10,3),
+      withSomeFee(10, 3),
     );
     handler.addWalletCallResponse(
       WalletApiOperation.ConvertDepositAmount,
       undefined,
-      withSomeFee(10,3),
+      withSomeFee(10, 3),
     );
 
-    const accountSelected = stringifyPaytoUri(ibanPayto.uri);
+    const accountSelected = ibanPayto.paytoUri;
 
     const hookBehavior = await tests.hookBehaveLikeThis(
       useComponentState,
@@ -351,7 +371,7 @@ describe("DepositPage states", () => {
           if (state.status !== "ready") expect.fail();
           expect(state.cancelHandler.onClick).not.undefined;
           expect(state.currency).eq(currency);
-          expect(state.account.value).eq(stringifyPaytoUri(talerBankPayto.uri));
+          expect(state.account.value).eq(talerBankPayto.paytoUri);
           expect(state.amount.value).deep.eq(Amounts.parseOrThrow("EUR:0"));
           expect(state.depositHandler.onClick).undefined;
           expect(state.totalFee).deep.eq(Amounts.parseOrThrow(`${currency}:0`));
