@@ -18,7 +18,9 @@ import {
   AbsoluteTime,
   Amounts,
   CoinStatus,
+  ExchangeListItem,
   ExchangeTosStatus,
+  HostPortPath,
   LogLevel,
   NotificationType,
   ScopeType,
@@ -69,11 +71,239 @@ type SplitedCoinInfo = {
   usable: CalculatedCoinfInfo[];
 };
 
+const listenAllEvents = Array.from<NotificationType>({ length: 1 });
+
+function ExchangeActions({}: {}): VNode {
+  const [time, reload] = useState<number>();
+  const { i18n } = useTranslationContext();
+  const api = useBackendContext();
+  const hook = useAsyncAsHook(async () => {
+    const list = await api.wallet.call(WalletApiOperation.ListExchanges, {});
+    return { exchanges: list.exchanges };
+  }, [time]);
+  const exchangeList = hook && !hook.hasError ? hook.response.exchanges : [];
+
+  if (!exchangeList || !exchangeList.length) {
+    return (
+      <div>
+        <i18n.Translate>No exchange yet</i18n.Translate>
+      </div>
+    );
+  }
+  return (
+    <table>
+      <thead>
+        <tr>
+          <th>
+            <i18n.Translate>Currency</i18n.Translate>
+          </th>
+          <th>
+            <i18n.Translate>URL</i18n.Translate>
+          </th>
+          <th>
+            <i18n.Translate>Status</i18n.Translate>
+          </th>
+          <th>
+            <i18n.Translate>Terms of Service</i18n.Translate>
+          </th>
+          <th>
+            <i18n.Translate>Last Update</i18n.Translate>
+          </th>
+          <th>
+            <i18n.Translate>Actions</i18n.Translate>
+          </th>
+        </tr>
+      </thead>
+      <tbody>
+        {exchangeList.map((e, idx) => {
+          function TosStatus(): VNode {
+            switch (e.tosStatus) {
+              case ExchangeTosStatus.Accepted:
+                return (
+                  <SuccessText>
+                    <i18n.Translate>ok</i18n.Translate>
+                  </SuccessText>
+                );
+              case ExchangeTosStatus.Pending:
+                return (
+                  <WarningText>
+                    <i18n.Translate>pending</i18n.Translate>
+                  </WarningText>
+                );
+              case ExchangeTosStatus.Proposed:
+                return <i18n.Translate>proposed</i18n.Translate>;
+              default:
+                return (
+                  <DestructiveText>
+                    <i18n.Translate>
+                      unknown (exchange status should be updated)
+                    </i18n.Translate>
+                  </DestructiveText>
+                );
+            }
+          }
+          const uri = !e.masterPub
+            ? undefined
+            : stringifyWithdrawExchange({
+                exchangeBaseUrl: e.exchangeBaseUrl as HostPortPath,
+              });
+          return (
+            <tr key={idx}>
+              <td>
+                <a
+                  href={
+                    !uri
+                      ? undefined
+                      : `#${Pages.defaultCta({
+                          uri: encodeCrockForURI(uri),
+                        })}`
+                  }
+                >
+                  {e.scopeInfo
+                    ? `${e.scopeInfo.currency} (${
+                        e.scopeInfo.type === ScopeType.Global
+                          ? "global"
+                          : "regional"
+                      })`
+                    : e.currency}
+                </a>
+              </td>
+              <td>
+                <a
+                  href={new URL(`/keys`, e.exchangeBaseUrl).href}
+                  target="_blank"
+                  rel="noreferrer"
+                >
+                  {e.exchangeBaseUrl}
+                </a>
+              </td>
+              <td>
+                {e.exchangeEntryStatus} / {e.exchangeUpdateStatus}
+              </td>
+              <td>
+                <TosStatus />
+              </td>
+              <td>
+                {e.lastUpdateTimestamp
+                  ? AbsoluteTime.toIsoString(
+                      AbsoluteTime.fromPreciseTimestamp(e.lastUpdateTimestamp),
+                    )
+                  : "never"}
+              </td>
+              <td>
+                <button
+                  onClick={async () => {
+                    await api.wallet.call(
+                      WalletApiOperation.UpdateExchangeEntry,
+                      {
+                        exchangeBaseUrl: e.exchangeBaseUrl,
+                        force: true,
+                      },
+                    );
+                    reload(new Date().getTime());
+                  }}
+                >
+                  <i18n.Translate>Reload</i18n.Translate>
+                </button>
+                <button
+                  onClick={async () => {
+                    await api.wallet.call(WalletApiOperation.DeleteExchange, {
+                      exchangeBaseUrl: e.exchangeBaseUrl,
+                    });
+                    reload(new Date().getTime());
+                  }}
+                >
+                  <i18n.Translate>Delete</i18n.Translate>
+                </button>
+                <button
+                  onClick={async () => {
+                    await api.wallet.call(WalletApiOperation.DeleteExchange, {
+                      exchangeBaseUrl: e.exchangeBaseUrl,
+                      purge: true,
+                    });
+                    reload(new Date().getTime());
+                  }}
+                >
+                  <i18n.Translate>Purge</i18n.Translate>
+                </button>
+                {e.scopeInfo && e.masterPub && e.currency ? (
+                  e.scopeInfo.type === ScopeType.Global ? (
+                    <button
+                      onClick={async () => {
+                        await api.wallet.call(
+                          WalletApiOperation.RemoveGlobalCurrencyExchange,
+                          {
+                            exchangeBaseUrl: e.exchangeBaseUrl,
+                            currency: e.currency!,
+                            exchangeMasterPub: e.masterPub!,
+                          },
+                        );
+                        reload(new Date().getTime());
+                      }}
+                    >
+                      <i18n.Translate>Make regional</i18n.Translate>
+                    </button>
+                  ) : e.scopeInfo.type === ScopeType.Auditor ? undefined : e
+                      .scopeInfo.type === ScopeType.Exchange ? (
+                    <button
+                      onClick={async () => {
+                        await api.wallet.call(
+                          WalletApiOperation.AddGlobalCurrencyExchange,
+                          {
+                            exchangeBaseUrl: e.exchangeBaseUrl,
+                            currency: e.currency!,
+                            exchangeMasterPub: e.masterPub!,
+                          },
+                        );
+                        reload(new Date().getTime());
+                      }}
+                    >
+                      <i18n.Translate>Make global</i18n.Translate>
+                    </button>
+                  ) : undefined
+                ) : undefined}
+                <button
+                  disabled={e.tosStatus !== ExchangeTosStatus.Accepted}
+                  onClick={async () => {
+                    await api.wallet.call(
+                      WalletApiOperation.SetExchangeTosForgotten,
+                      {
+                        exchangeBaseUrl: e.exchangeBaseUrl,
+                      },
+                    );
+                    reload(new Date().getTime());
+                  }}
+                >
+                  <i18n.Translate>Forget ToS</i18n.Translate>
+                </button>
+              </td>
+            </tr>
+          );
+        })}
+      </tbody>
+    </table>
+  );
+}
+
 export function DeveloperPage(): VNode {
   const { i18n } = useTranslationContext();
   const [downloadedDatabase, setDownloadedDatabase] = useState<
     { time: Date; content: string } | undefined
   >(undefined);
+  const [tagName, setTagName] = useState("");
+  const [experiment, setExperiment] = useState("");
+  const [logLevel, setLogLevel] = useState("info");
+  const api = useBackendContext();
+  const fileRef = useRef<HTMLInputElement>(null);
+  const [settings, updateSettings] = useSettings();
+  const { safely } = useAlertContext();
+  useEffect(() => {
+    return api.listener.onUpdateNotification(listenAllEvents, (ev) => {
+      console.log("event", ev);
+      return hook?.retry();
+    });
+  });
+
   async function onExportDatabase(): Promise<void> {
     const db = await api.wallet.call(WalletApiOperation.ExportDb, {});
     const content = JSON.stringify(db);
@@ -82,19 +312,12 @@ export function DeveloperPage(): VNode {
       content,
     });
   }
-  const api = useBackendContext();
 
-  const fileRef = useRef<HTMLInputElement>(null);
   async function onImportDatabase(str: string): Promise<void> {
     await api.wallet.call(WalletApiOperation.ImportDb, {
       dump: JSON.parse(str),
     });
   }
-  const [settings, updateSettings] = useSettings();
-  const { safely } = useAlertContext();
-
-  const listenAllEvents = Array.from<NotificationType>({ length: 1 });
-  // listenAllEvents.includes = () => true
 
   const hook = useAsyncAsHook(async () => {
     const list = await api.wallet.call(WalletApiOperation.ListExchanges, {});
@@ -105,13 +328,6 @@ export function DeveloperPage(): VNode {
   const exchangeList = hook && !hook.hasError ? hook.response.exchanges : [];
   const coins = hook && !hook.hasError ? hook.response.coins.coins : [];
 
-  useEffect(() => {
-    return api.listener.onUpdateNotification(listenAllEvents, (ev) => {
-      console.log("event", ev);
-      return hook?.retry();
-    });
-  });
-
   const currencies: { [ex: string]: string } = {};
   const money_by_exchange = coins.reduce(
     (prev, cur) => {
@@ -138,8 +354,6 @@ export function DeveloperPage(): VNode {
     },
   );
 
-  const [tagName, setTagName] = useState("");
-  const [logLevel, setLogLevel] = useState("info");
   return (
     <div>
       <p>
@@ -275,199 +489,8 @@ export function DeveloperPage(): VNode {
       <SubTitle>
         <i18n.Translate>Exchange Entries</i18n.Translate>
       </SubTitle>
-      {!exchangeList || !exchangeList.length ? (
-        <div>
-          <i18n.Translate>No exchange yet</i18n.Translate>
-        </div>
-      ) : (
-        <Fragment>
-          <table>
-            <thead>
-              <tr>
-                <th>
-                  <i18n.Translate>Currency</i18n.Translate>
-                </th>
-                <th>
-                  <i18n.Translate>URL</i18n.Translate>
-                </th>
-                <th>
-                  <i18n.Translate>Status</i18n.Translate>
-                </th>
-                <th>
-                  <i18n.Translate>Terms of Service</i18n.Translate>
-                </th>
-                <th>
-                  <i18n.Translate>Last Update</i18n.Translate>
-                </th>
-                <th>
-                  <i18n.Translate>Actions</i18n.Translate>
-                </th>
-              </tr>
-            </thead>
-            <tbody>
-              {exchangeList.map((e, idx) => {
-                function TosStatus(): VNode {
-                  switch (e.tosStatus) {
-                    case ExchangeTosStatus.Accepted:
-                      return (
-                        <SuccessText>
-                          <i18n.Translate>ok</i18n.Translate>
-                        </SuccessText>
-                      );
-                    case ExchangeTosStatus.Pending:
-                      return (
-                        <WarningText>
-                          <i18n.Translate>pending</i18n.Translate>
-                        </WarningText>
-                      );
-                    case ExchangeTosStatus.Proposed:
-                      return <i18n.Translate>proposed</i18n.Translate>;
-                    default:
-                      return (
-                        <DestructiveText>
-                          <i18n.Translate>
-                            unknown (exchange status should be updated)
-                          </i18n.Translate>
-                        </DestructiveText>
-                      );
-                  }
-                }
-                const uri = !e.masterPub
-                  ? undefined
-                  : stringifyWithdrawExchange({
-                      exchangeBaseUrl: e.exchangeBaseUrl,
-                    });
-                return (
-                  <tr key={idx}>
-                    <td>
-                      <a
-                        href={
-                          !uri
-                            ? undefined
-                            : `#${Pages.defaultCta({ uri: encodeCrockForURI(uri) })}`
-                        }
-                      >
-                        {e.scopeInfo
-                          ? `${e.scopeInfo.currency} (${
-                              e.scopeInfo.type === ScopeType.Global
-                                ? "global"
-                                : "regional"
-                            })`
-                          : e.currency}
-                      </a>
-                    </td>
-                    <td>
-                      <a
-                        href={new URL(`/keys`, e.exchangeBaseUrl).href}
-                        target="_blank"
-                        rel="noreferrer"
-                      >
-                        {e.exchangeBaseUrl}
-                      </a>
-                    </td>
-                    <td>
-                      {e.exchangeEntryStatus} / {e.exchangeUpdateStatus}
-                    </td>
-                    <td>
-                      <TosStatus />
-                    </td>
-                    <td>
-                      {e.lastUpdateTimestamp
-                        ? AbsoluteTime.toIsoString(
-                            AbsoluteTime.fromPreciseTimestamp(
-                              e.lastUpdateTimestamp,
-                            ),
-                          )
-                        : "never"}
-                    </td>
-                    <td>
-                      <button
-                        onClick={() => {
-                          api.wallet.call(
-                            WalletApiOperation.UpdateExchangeEntry,
-                            {
-                              exchangeBaseUrl: e.exchangeBaseUrl,
-                              force: true,
-                            },
-                          );
-                        }}
-                      >
-                        Reload
-                      </button>
-                      <button
-                        onClick={() => {
-                          api.wallet.call(WalletApiOperation.DeleteExchange, {
-                            exchangeBaseUrl: e.exchangeBaseUrl,
-                          });
-                        }}
-                      >
-                        Delete
-                      </button>
-                      <button
-                        onClick={() => {
-                          api.wallet.call(WalletApiOperation.DeleteExchange, {
-                            exchangeBaseUrl: e.exchangeBaseUrl,
-                            purge: true,
-                          });
-                        }}
-                      >
-                        Purge
-                      </button>
-                      {e.scopeInfo && e.masterPub && e.currency ? (
-                        e.scopeInfo.type === ScopeType.Global ? (
-                          <button
-                            onClick={() => {
-                              api.wallet.call(
-                                WalletApiOperation.RemoveGlobalCurrencyExchange,
-                                {
-                                  exchangeBaseUrl: e.exchangeBaseUrl,
-                                  currency: e.currency!,
-                                  exchangeMasterPub: e.masterPub!,
-                                },
-                              );
-                            }}
-                          >
-                            Make regional
-                          </button>
-                        ) : e.scopeInfo.type ===
-                          ScopeType.Auditor ? undefined : e.scopeInfo.type ===
-                          ScopeType.Exchange ? (
-                          <button
-                            onClick={() => {
-                              api.wallet.call(
-                                WalletApiOperation.AddGlobalCurrencyExchange,
-                                {
-                                  exchangeBaseUrl: e.exchangeBaseUrl,
-                                  currency: e.currency!,
-                                  exchangeMasterPub: e.masterPub!,
-                                },
-                              );
-                            }}
-                          >
-                            Make global
-                          </button>
-                        ) : undefined
-                      ) : undefined}
-                      <button
-                        onClick={() => {
-                          api.wallet.call(
-                            WalletApiOperation.SetExchangeTosForgotten,
-                            {
-                              exchangeBaseUrl: e.exchangeBaseUrl,
-                            },
-                          );
-                        }}
-                      >
-                        Forget ToS
-                      </button>
-                    </td>
-                  </tr>
-                );
-              })}
-            </tbody>
-          </table>
-        </Fragment>
-      )}
+      <ExchangeActions />
+
       <div style={{ display: "flex", justifyContent: "space-between" }}>
         <div />
         <LinkPrimary href={`#${Pages.settingsExchangeAdd({})}`}>
@@ -512,6 +535,31 @@ export function DeveloperPage(): VNode {
         </Button>
       </Paper>
 
+      <Paper style={{ padding: 10, margin: 10 }}>
+        <h3>Dev Experiment</h3>
+        <div>
+          <TextField
+            label="Experiment name"
+            placeholder="taler://dev-experiment"
+            variant="filled"
+            // error={subject.error}
+            required
+            value={experiment}
+            onChange={setExperiment}
+          />
+        </div>
+        <Button
+          variant="contained"
+          onClick={async () => {
+            await api.wallet.call(WalletApiOperation.ApplyDevExperiment, {
+              devExperimentUri: `taler://dev-experiment/${experiment}`,
+            });
+          }}
+        >
+          Apply
+        </Button>
+      </Paper>
+
       <br />
       <p>
         <i18n.Translate>Coins</i18n.Translate>:
