@@ -15,26 +15,31 @@
  */
 
 import {
-  buildPayto,
-  KnownBankAccountsInfo,
+  BtAddrString,
+  HostPortPath,
+  IbanString,
+  InternationalizationAPI,
+  OperationOk,
+  parseIban,
+  ParseIbanError,
+  parsePaytoUri,
+  Paytos,
   PaytoUriBitcoin,
   PaytoUriIBAN,
   PaytoUriTalerBank,
-  stringifyPaytoUri,
-  validateIban,
+  TranslatedString,
+  WalletBankAccountInfo
 } from "@gnu-taler/taler-util";
 import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { styled } from "@linaria/react";
 import { Fragment, h, VNode } from "preact";
 import { useState } from "preact/hooks";
 import { ErrorMessage } from "../../components/ErrorMessage.js";
-import { SubTitle, SvgIcon } from "../../components/styled/index.js";
+import { SubTitle } from "../../components/styled/index.js";
 import { Button } from "../../mui/Button.js";
 import { TextFieldHandler } from "../../mui/handlers.js";
 import { TextField } from "../../mui/TextField.js";
-import checkIcon from "../../svg/check_24px.inline.svg";
 import deleteIcon from "../../svg/delete_24px.inline.svg";
-import warningIcon from "../../svg/warning_24px.inline.svg";
 import { State } from "./index.js";
 
 type AccountType = "bitcoin" | "x-taler-bank" | "iban";
@@ -44,8 +49,9 @@ type ComponentFormByAccountType = {
 
 type ComponentListByAccountType = {
   [type in AccountType]: (props: {
-    list: KnownBankAccountsInfo[];
-    onDelete: (a: KnownBankAccountsInfo) => Promise<void>;
+    list: WalletBankAccountInfo[];
+    onDelete: (a: WalletBankAccountInfo) => Promise<void>;
+    onSelect: (ac: WalletBankAccountInfo) => void;
   }) => VNode;
 };
 
@@ -85,6 +91,7 @@ export function ReadyView({
   deleteAccount,
   onCancel,
   uri,
+  selectAccount,
 }: State.Ready): VNode {
   const { i18n } = useTranslationContext();
 
@@ -168,7 +175,14 @@ export function ReadyView({
       <section>
         {Object.entries(accountByType).map(([type, list]) => {
           const Table = tableComponentByAccountType[type as AccountType];
-          return <Table key={type} list={list} onDelete={deleteAccount} />;
+          return (
+            <Table
+              key={type}
+              list={list}
+              onDelete={deleteAccount}
+              onSelect={selectAccount}
+            />
+          );
         })}
       </section>
     </Fragment>
@@ -177,10 +191,12 @@ export function ReadyView({
 
 function IbanTable({
   list,
+  onSelect,
   onDelete,
 }: {
-  list: KnownBankAccountsInfo[];
-  onDelete: (ac: KnownBankAccountsInfo) => void;
+  list: WalletBankAccountInfo[];
+  onDelete: (ac: WalletBankAccountInfo) => void;
+  onSelect: (ac: WalletBankAccountInfo) => void;
 }): VNode {
   const { i18n } = useTranslationContext();
   if (list.length === 0) return <Fragment />;
@@ -195,9 +211,9 @@ function IbanTable({
             <th>
               <i18n.Translate>Alias</i18n.Translate>
             </th>
-            <th>
+            {/* <th>
               <i18n.Translate>Bank Id</i18n.Translate>
-            </th>
+            </th> */}
             <th>
               <i18n.Translate>Int. Account Number</i18n.Translate>
             </th>
@@ -212,15 +228,23 @@ function IbanTable({
         </thead>
         <tbody>
           {list.map((account) => {
-            const p = account.uri as PaytoUriIBAN;
+            const p = parsePaytoUri(account.paytoUri) as PaytoUriIBAN;
             return (
-              <tr key={account.alias}>
-                <td>{account.alias}</td>
-                <td>{p.bic}</td>
+              <tr
+                key={account.label}
+                onClick={(e) => {
+                  onSelect(account);
+                }}
+                style={{
+                  cursor: "pointer",
+                }}
+              >
+                <td>{account.label}</td>
+                {/* <td>{p.bic}</td> */}
                 <td>{p.iban}</td>
                 <td>{p.params["receiver-name"]}</td>
-                <td class="kyc">
-                  {account.kyc_completed ? (
+                {/* <td class="kyc">
+                  {account.kycCompleted ? (
                     <SvgIcon
                       title={i18n.str`KYC done`}
                       dangerouslySetInnerHTML={{ __html: checkIcon }}
@@ -233,7 +257,7 @@ function IbanTable({
                       color="orange"
                     />
                   )}
-                </td>
+                </td> */}
                 <td class="actions">
                   <Button
                     variant="outlined"
@@ -258,8 +282,9 @@ function TalerBankTable({
   list,
   onDelete,
 }: {
-  list: KnownBankAccountsInfo[];
-  onDelete: (ac: KnownBankAccountsInfo) => void;
+  list: WalletBankAccountInfo[];
+  onDelete: (ac: WalletBankAccountInfo) => void;
+  onSelect: (ac: WalletBankAccountInfo) => void;
 }): VNode {
   const { i18n } = useTranslationContext();
   if (list.length === 0) return <Fragment />;
@@ -288,14 +313,14 @@ function TalerBankTable({
         </thead>
         <tbody>
           {list.map((account) => {
-            const p = account.uri as PaytoUriTalerBank;
+            const p = parsePaytoUri(account.paytoUri) as PaytoUriTalerBank;
             return (
-              <tr key={account.alias}>
-                <td>{account.alias}</td>
+              <tr key={account.label}>
+                <td>{account.label}</td>
                 <td>{p.host}</td>
                 <td>{p.account}</td>
-                <td class="kyc">
-                  {account.kyc_completed ? (
+                {/* <td class="kyc">
+                  {account.kycCompleted ? (
                     <SvgIcon
                       title={i18n.str`KYC done`}
                       dangerouslySetInnerHTML={{ __html: checkIcon }}
@@ -308,7 +333,7 @@ function TalerBankTable({
                       color="orange"
                     />
                   )}
-                </td>
+                </td> */}
                 <td class="actions">
                   <Button
                     variant="outlined"
@@ -332,9 +357,11 @@ function TalerBankTable({
 function BitcoinTable({
   list,
   onDelete,
+  onSelect,
 }: {
-  list: KnownBankAccountsInfo[];
-  onDelete: (ac: KnownBankAccountsInfo) => void;
+  list: WalletBankAccountInfo[];
+  onDelete: (ac: WalletBankAccountInfo) => void;
+  onSelect: (ac: WalletBankAccountInfo) => void;
 }): VNode {
   const { i18n } = useTranslationContext();
   if (list.length === 0) return <Fragment />;
@@ -360,13 +387,13 @@ function BitcoinTable({
         </thead>
         <tbody>
           {list.map((account) => {
-            const p = account.uri as PaytoUriBitcoin;
+            const p = parsePaytoUri(account.paytoUri) as PaytoUriBitcoin;
             return (
-              <tr key={account.alias}>
-                <td>{account.alias}</td>
+              <tr key={account.label}>
+                <td>{account.label}</td>
                 <td>{p.targetPath}</td>
-                <td class="kyc">
-                  {account.kyc_completed ? (
+                {/* <td class="kyc">
+                  {account.kycCompleted ? (
                     <SvgIcon
                       title={i18n.str`KYC done`}
                       dangerouslySetInnerHTML={{ __html: checkIcon }}
@@ -379,7 +406,7 @@ function BitcoinTable({
                       color="orange"
                     />
                   )}
-                </td>
+                </td> */}
                 <td class="actions">
                   <Button
                     variant="outlined"
@@ -421,8 +448,8 @@ function BitcoinAddressAccount({ field }: { field: TextFieldHandler }): VNode {
         onChange={(v) => {
           setValue(v);
           if (!errors && field.onInput) {
-            const p = buildPayto("bitcoin", v, undefined);
-            field.onInput(stringifyPaytoUri(p));
+            const p = Paytos.createBitcoin(v as BtAddrString, undefined);
+            field.onInput(Paytos.toFullString(p));
           }
         }}
       />
@@ -431,7 +458,9 @@ function BitcoinAddressAccount({ field }: { field: TextFieldHandler }): VNode {
 }
 
 function undefinedIfEmpty<T extends object>(obj: T): T | undefined {
-  return Object.keys(obj).some((k) => (obj as Record<string,unknown>)[k] !== undefined)
+  return Object.keys(obj).some(
+    (k) => (obj as Record<string, unknown>)[k] !== undefined,
+  )
     ? obj
     : undefined;
 }
@@ -463,8 +492,8 @@ function TalerBankAddressAccount({
         onChange={(v) => {
           setHost(v);
           if (!errors && field.onInput && account) {
-            const p = buildPayto("x-taler-bank", v, account);
-            field.onInput(stringifyPaytoUri(p));
+            const p = Paytos.createTalerBank(v as HostPortPath, account);
+            field.onInput(Paytos.toFullString(p));
           }
         }}
       />
@@ -478,15 +507,36 @@ function TalerBankAddressAccount({
         onChange={(v) => {
           setAccount(v || "");
           if (!errors && field.onInput && host) {
-            const p = buildPayto("x-taler-bank", host, v);
-            field.onInput(stringifyPaytoUri(p));
+            const p = Paytos.createTalerBank(host as HostPortPath, v);
+            field.onInput(Paytos.toFullString(p));
           }
         }}
       />
     </Fragment>
   );
 }
+type FailCasesOf<T extends (...args: any) => any> = Exclude<
+  ReturnType<T>,
+  OperationOk<any>
+>;
 
+function translateIbanError(
+  result: FailCasesOf<typeof parseIban>,
+  i18n: InternationalizationAPI,
+): TranslatedString {
+  switch (result.case) {
+    case ParseIbanError.UNSUPPORTED_COUNTRY:
+      return i18n.str`Unsupported country.`;
+    case ParseIbanError.TOO_LONG:
+      return i18n.str`IBANs have fewer than 34 characters.`;
+    case ParseIbanError.TOO_SHORT:
+      return i18n.str`IBANs have more than 4 characters.`;
+    case ParseIbanError.INVALID_CHARSET:
+      return i18n.str`It should only contain numbers and letters.`;
+    case ParseIbanError.INVALID_CHECKSUM:
+      return i18n.str`The checksum is wrong.`;
+  }
+}
 //Taken from libeufin and libeufin took it from the ISO20022 XSD schema
 // const bicRegex = /^[A-Z]{6}[A-Z2-9][A-NP-Z0-9]([A-Z0-9]{3})?$/;
 // const ibanRegex = /^[A-Z]{2}[0-9]{2}[a-zA-Z0-9]{1,30}$/;
@@ -496,21 +546,20 @@ function IbanAddressAccount({ field }: { field: TextFieldHandler }): VNode {
   // const [bic, setBic] = useState<string | undefined>(undefined);
   const [iban, setIban] = useState<string | undefined>(undefined);
   const [name, setName] = useState<string | undefined>(undefined);
-  const bic = ""
-  const errorsFN = (iban:string | undefined, name: string | undefined) => undefinedIfEmpty({
-    // bic: !bic
-    //   ? undefined
-    //   : !bicRegex.test(bic)
-    //   ? i18n.str`Invalid bic`
-    //   : undefined,
-    iban: !iban
-      ? i18n.str`Can't be empty`
-      : validateIban(iban).type === "invalid"
-      ? i18n.str`Invalid iban`
-      : undefined,
-    name: !name ? i18n.str`Can't be empty` : undefined,
-  });
-  const errors = errorsFN(iban, name)
+  const bic = "";
+  const errorsFN = (iban: string | undefined, name: string | undefined) =>
+    undefinedIfEmpty({
+      // bic: !bic
+      //   ? undefined
+      //   : !bicRegex.test(bic)
+      //   ? i18n.str`Invalid bic`
+      //   : undefined,
+      iban: !iban
+        ? i18n.str`Required`
+        : validateIBAN(iban, i18n),
+      name: !name ? i18n.str`Can't be empty` : undefined,
+    });
+  const errors = errorsFN(iban, name);
 
   function sendUpdateIfNoErrors(
     bic: string | undefined,
@@ -519,11 +568,11 @@ function IbanAddressAccount({ field }: { field: TextFieldHandler }): VNode {
   ): void {
     if (!field.onInput) return;
     if (!errorsFN(iban, name)) {
-      const p = buildPayto("iban", iban, bic);
+      const p = Paytos.createIban(iban as IbanString, bic);
       p.params["receiver-name"] = name;
-      field.onInput(stringifyPaytoUri(p));
+      field.onInput(Paytos.toFullString(p));
     } else {
-      field.onInput("")
+      field.onInput("");
     }
   }
   return (
@@ -599,3 +648,17 @@ function CustomFieldByAccountType({
     </div>
   );
 }
+
+function validateIBAN(
+  iban: string,
+  i18n: ReturnType<typeof useTranslationContext>["i18n"],
+): TranslatedString | undefined {
+  if (!iban) {
+    return i18n.str`Required`;
+  }
+  const result = parseIban(iban);
+  if (result.type === "ok") {
+    return undefined;
+  }
+  return translateIbanError(result, i18n);
+}
