@@ -264,6 +264,10 @@ function AdvanceSettings(): VNode {
       label: i18n.str`Show wallet activity`,
       description: i18n.str`Show the wallet notification and observability event in the UI.`,
     },
+    walletEnableV1Contracts: {
+      label: i18n.str`Enable v1 contract`,
+      description: i18n.str`v1 is under development and wallet-core converts v1 contract to v0. Enable this options for testing`,
+    }
   };
   return (
     <Fragment>
