@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2022 Taler Systems S.A.
+ (C) 2022-2025 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -19,6 +19,7 @@ import {
   AmountJson,
   Amounts,
   AmountString,
+  assertUnreachable,
   DenomLossEventType,
   Duration,
   MerchantInfo,
@@ -27,7 +28,6 @@ import {
   parsePaytoUri,
   PaytoUri,
   ScopeInfo,
-  ScopeType,
   stringifyPaytoUri,
   TalerErrorCode,
   TalerPreciseTimestamp,
@@ -66,7 +66,6 @@ import {
   SmallLightText,
   SubTitle,
   SvgIcon,
-  WarningBox,
 } from "../components/styled/index.js";
 import { Time } from "../components/Time.js";
 import { alertFromError, useAlertContext } from "../context/alert.js";
@@ -77,11 +76,10 @@ import { Button } from "../mui/Button.js";
 import { SafeHandler } from "../mui/handlers.js";
 import { Pages } from "../NavigationBar.js";
 import refreshIcon from "../svg/refresh_24px.inline.svg";
-import { assertUnreachable } from "../utils/index.js";
 
 interface Props {
   tid: string;
-  goToWalletHistory: (scope: ScopeInfo) => Promise<void>;
+  goToWalletHistory: (scope?: ScopeInfo) => Promise<void>;
 }
 
 export function TransactionPage({ tid, goToWalletHistory }: Props): VNode {
@@ -119,12 +117,8 @@ export function TransactionPage({ tid, goToWalletHistory }: Props): VNode {
     );
   }
 
-  const currency = Amounts.parse(state.response.amountEffective)!.currency;
   const txScope = !state.response.scopes.length
-    ? {
-        type: ScopeType.Global as const,
-        currency,
-      }
+    ? undefined
     : state.response.scopes[0];
 
   return (
@@ -236,6 +230,100 @@ function TransactionTemplate({
 
   const showButton = getShowButtonStates(transaction);
 
+  let pendingOrKycInfo: VNode | undefined = undefined;
+
+  switch (transaction.txState.major) {
+    case TransactionMajorState.Pending: {
+      switch (transaction.txState.minor) {
+        case TransactionMinorState.KycAuthRequired: {
+          pendingOrKycInfo = (
+            <AlertView
+              alert={{
+                type: "warning",
+                message: i18n.str`Bank account confirmation required for the transaction to complete.`,
+                description: (
+                  <div>
+                    <p>
+                      Your account:{" "}
+                      <code>
+                        {transaction.kycAuthTransferInfo?.debitPaytoUri}
+                      </code>
+                    </p>
+                    <p>
+                      Deposit account public key:{" "}
+                      <code>{transaction.kycAuthTransferInfo?.accountPub}</code>
+                    </p>
+                    <p>
+                      Payment targets:{" "}
+                      <ul>
+                        {transaction.kycAuthTransferInfo?.creditPaytoUris.map(
+                          (x) => (
+                            <li>
+                              <code>{x}</code>
+                            </li>
+                          ),
+                        )}
+                      </ul>
+                    </p>
+                  </div>
+                ),
+              }}
+            />
+          );
+          break;
+        }
+        case TransactionMinorState.MergeKycRequired:
+        case TransactionMinorState.KycRequired:
+        case TransactionMinorState.BalanceKycRequired: {
+          pendingOrKycInfo = (
+            <AlertView
+              alert={{
+                type: "warning",
+                message: i18n.str`KYC check required for the transaction to complete.`,
+                description: !transaction.kycUrl ? (
+                  i18n.str`No additional information has been provided.`
+                ) : (
+                  <div>
+                    <i18n.Translate>
+                      Follow this link to the{` `}
+                      <a
+                        rel="noreferrer"
+                        target="_bank"
+                        href={transaction.kycUrl}
+                      >
+                        KYC status page.
+                      </a>
+                    </i18n.Translate>
+                  </div>
+                ),
+              }}
+            />
+          );
+          break;
+        }
+        default: {
+          // Plain pending, no KYC.
+          pendingOrKycInfo = (
+            <InfoBox>
+              <div style={{ justifyContent: "center", lineHeight: "25px" }}>
+                <i18n.Translate>
+                  This transaction is pending to complete.
+                </i18n.Translate>
+                <Link onClick={onRetry} style={{ padding: 0 }}>
+                  <SvgIcon
+                    title={i18n.str`Retry`}
+                    dangerouslySetInnerHTML={{ __html: refreshIcon }}
+                    color="black"
+                  />
+                </Link>
+              </div>
+            </InfoBox>
+          );
+        }
+      }
+    }
+  }
+
   return (
     <Fragment>
       <section style={{ padding: 8, textAlign: "center" }}>
@@ -251,57 +339,20 @@ function TransactionTemplate({
             )}
           />
         ) : undefined}
-        {transaction.txState.major === TransactionMajorState.Pending &&
-          (transaction.txState.minor === TransactionMinorState.KycRequired ? (
-            <AlertView
-              alert={{
-                type: "warning",
-                message: i18n.str`KYC check required for the transaction to complete.`,
-                description:
-                  transaction.kycUrl &&
-                  typeof transaction.kycUrl === "string" ? (
-                    <div>
-                      <i18n.Translate>
-                        Follow this link to the{` `}
-                        <a
-                          rel="noreferrer"
-                          target="_bank"
-                          href={transaction.kycUrl}
-                        >
-                          KYC verifier.
-                        </a>
-                      </i18n.Translate>
-                    </div>
-                  ) : (
-                    i18n.str`No additional information has been provided.`
-                  ),
-              }}
-            />
-          ) : (
-            <WarningBox>
-              <div style={{ justifyContent: "center", lineHeight: "25px" }}>
-                <i18n.Translate>
-                  This transaction is not completed
-                </i18n.Translate>
-                <Link onClick={onRetry} style={{ padding: 0 }}>
-                  <SvgIcon
-                    title={i18n.str`Retry`}
-                    dangerouslySetInnerHTML={{ __html: refreshIcon }}
-                    color="black"
-                  />
-                </Link>
-              </div>
-            </WarningBox>
-          ))}
+        {pendingOrKycInfo}
         {transaction.txState.major === TransactionMajorState.Aborted && (
           <InfoBox>
             <i18n.Translate>This transaction was aborted.</i18n.Translate>
           </InfoBox>
         )}
         {transaction.txState.major === TransactionMajorState.Failed && (
-          <ErrorBox>
-            <i18n.Translate>This transaction failed.</i18n.Translate>
-          </ErrorBox>
+          <ErrorAlertView
+            error={alertFromError(
+              i18n,
+              i18n.str`This transaction failed.`,
+              transaction.error!,
+            )}
+          />
         )}
         {confirmBeforeForget ? (
           <Overlay>
@@ -906,6 +957,16 @@ export function TransactionView({
           <i18n.Translate>Invoice</i18n.Translate>
         </Header>
 
+        {transaction.txState.major === TransactionMajorState.Pending &&
+          transaction.txState.minor === TransactionMinorState.Ready &&
+          transaction.talerUri &&
+          !transaction.error && (
+            <Part
+              title={i18n.str`URI`}
+              text={<ShowQrWithCopy text={transaction.talerUri} startOpen />}
+              kind="neutral"
+            />
+          )}
         {transaction.info.summary ? (
           <Part
             title={i18n.str`Subject`}
@@ -918,16 +979,6 @@ export function TransactionView({
           text={transaction.exchangeBaseUrl as TranslatedString}
           kind="neutral"
         />
-        {transaction.txState.major === TransactionMajorState.Pending &&
-          transaction.txState.minor === TransactionMinorState.Ready &&
-          transaction.talerUri &&
-          !transaction.error && (
-            <Part
-              title={i18n.str`URI`}
-              text={<ShowQrWithCopy text={transaction.talerUri} />}
-              kind="neutral"
-            />
-          )}
         <Part
           title={i18n.str`Details`}
           text={
@@ -1005,6 +1056,13 @@ export function TransactionView({
           <i18n.Translate>Transfer</i18n.Translate>
         </Header>
 
+        {transaction.talerUri && (
+          <Part
+            title={i18n.str`URI`}
+            text={<ShowQrWithCopy text={transaction.talerUri} startOpen />}
+            kind="neutral"
+          />
+        )}
         {transaction.info.summary ? (
           <Part
             title={i18n.str`Subject`}
@@ -1017,13 +1075,6 @@ export function TransactionView({
           text={transaction.exchangeBaseUrl as TranslatedString}
           kind="neutral"
         />
-        {transaction.talerUri && (
-          <Part
-            title={i18n.str`URI`}
-            text={<ShowQrWithCopy text={transaction.talerUri} />}
-            kind="neutral"
-          />
-        )}
         <Part
           title={i18n.str`Details`}
           text={
@@ -1939,8 +1990,14 @@ function NicePayto({ payto }: { payto: PaytoUri }): VNode {
   return <Fragment>{stringifyPaytoUri(payto)}</Fragment>;
 }
 
-function ShowQrWithCopy({ text }: { text: string }): VNode {
-  const [showing, setShowing] = useState(false);
+function ShowQrWithCopy({
+  text,
+  startOpen,
+}: {
+  text: string;
+  startOpen?: boolean;
+}): VNode {
+  const [showing, setShowing] = useState(startOpen);
   const { i18n } = useTranslationContext();
   async function copy(): Promise<void> {
     navigator.clipboard.writeText(text);
