@@ -44,6 +44,7 @@ import {
 import { HttpRequestLibrary } from "@gnu-taler/taler-util/http";
 import {
   DbAccess,
+  HttpFactory,
   SynchronousCryptoWorkerFactoryPlain,
   Wallet,
   WalletApiOperation,
@@ -54,6 +55,7 @@ import {
   importDb,
 } from "@gnu-taler/taler-wallet-core";
 import { BrowserFetchHttpLib } from "@gnu-taler/web-util/browser";
+import { BridgeIDBFactory } from "../../idb-bridge/src/bridge-idb.js";
 import { MessageFromFrontend, MessageResponse } from "./platform/api.js";
 import { platform } from "./platform/background.js";
 import { ExtensionOperations } from "./taler-wallet-interaction-loader.js";
@@ -152,9 +154,6 @@ function convertWalletActivityNotification(
         groupId,
       };
     }
-    case NotificationType.WithdrawalOperationTransition: {
-      return undefined;
-    }
     case NotificationType.ExchangeStateTransition: {
       const groupId = `${event.type}:${event.exchangeBaseUrl}`;
       const found = knownEvents.find((a) => a.groupId === groupId);
@@ -215,6 +214,7 @@ function convertWalletActivityNotification(
       };
     }
   }
+  return undefined;
 }
 
 function addNewWalletActivityNotification(
@@ -460,8 +460,19 @@ async function reinitWallet(): Promise<void> {
   const settings = await platform.getSettingsFromStorage();
   logger.info("Setting up wallet");
   const wallet = await Wallet.create(
-    indexedDB as any,
-    httpFactory as any,
+    {
+      idbFactory: indexedDB as any as BridgeIDBFactory,
+      readBackupJson(path) {
+        throw Error("DB file export not supported for platform webext");
+      },
+      exportToFile() {
+        throw Error("DB file export not supported for platform webext");
+      },
+      getStats() {
+        throw Error("DB statistics not supported for platform webext");
+      },
+    },
+    httpFactory as any as HttpFactory,
     timer,
     cryptoWorker,
   );
@@ -474,6 +485,7 @@ async function reinitWallet(): Promise<void> {
         },
         features: {
           allowHttp: settings.walletAllowHttp,
+          enableV1Contracts: settings.walletEnableV1Contracts,
         },
       },
     });
