@@ -11,7 +11,7 @@ interface Props {
 export function Attention({ type = "info", title, children, onClose, timeout = Duration.getForever() }: Props): VNode {
 
   return <div class={`group attention-${type} mt-2 shadow-lg`}>
-    {timeout.d_ms === "forever" ? undefined : <style>{`
+    {/* {timeout.d_ms === "forever" ? undefined : <style>{`
     .progress {
         animation: notificationTimeoutBar ${Math.round(timeout.d_ms / 1000)}s ease-in-out;
         animation-fill-mode:both; 
@@ -22,7 +22,7 @@ export function Attention({ type = "info", title, children, onClose, timeout = D
       100% { width: 100%; }
     }
   `}</style>
-    }
+    } */}
 
     <div data-timed={timeout.d_ms !== "forever"} class="rounded-md data-[timed=true]:rounded-b-none group-[.attention-info]:bg-blue-50 group-[.attention-low]:bg-gray-100 group-[.attention-warning]:bg-yellow-50 group-[.attention-danger]:bg-red-50 group-[.attention-success]:bg-green-50 p-4 shadow">
       <div class="flex">
