@@ -35,7 +35,7 @@ export function Loading(): VNode {
 
 function Spinner(): VNode {
   return (
-    <div class="lds-ring" style={{ margin: "auto" }}>
+    <div class="lds-ring m-auto" >
       <div />
       <div />
       <div />
