@@ -25,5 +25,5 @@ export function ShowInputErrorLabel({
 }): VNode {
   if (message && isDirty)
     return <div class="text-base" style={{ color: "red" }}>{message}</div>;
-  return <div class="text-base" style={{ }}> </div>;
+  return <div class="text-base" > </div>;
 }
