@@ -1,4 +1,5 @@
 export * as utils from "./utils.js";
+export { onComponentUnload, preconnectAs, Preconnect } from "./utils.js";
 export * from "./Attention.js";
 export * from "./CopyButton.js";
 export * from "./ErrorLoading.js";
@@ -11,3 +12,5 @@ export * from "./ShowInputErrorLabel.js";
 export * from "./NotificationBanner.js";
 export * from "./ToastBanner.js";
 export * from "./Time.js";
+export * from "./RenderAmount.js";
+export * from "./Pagination.js";
