@@ -1,6 +1,9 @@
 import { TranslatedString } from "@gnu-taler/taler-util";
-import { VNode, h } from "preact";
-import { LabelWithTooltipMaybeRequired, RenderAddon } from "./InputLine.js";
+import { Fragment, VNode, h } from "preact";
+import {
+  LabelWithTooltipMaybeRequired,
+  RenderAddon,
+} from "./fields/InputLine.js";
 import { Addon } from "./FormProvider.js";
 
 interface Props {
@@ -9,11 +12,22 @@ interface Props {
   help?: TranslatedString;
   before?: Addon;
   after?: Addon;
+  hidden?: boolean;
 }
 
-export function Caption({ before, after, label, tooltip, help }: Props): VNode {
+export function Caption({
+  hidden,
+  before,
+  after,
+  label,
+  tooltip,
+  help,
+}: Props): VNode {
+  if (hidden) {
+    return <Fragment />;
+  }
   return (
-    <div class="sm:col-span-6 flex">
+    <div class="sm:col-span-6">
       {before !== undefined && <RenderAddon addon={before} />}
       <LabelWithTooltipMaybeRequired label={label} tooltip={tooltip} />
       {after !== undefined && <RenderAddon addon={after} />}
