@@ -1,18 +1,14 @@
 import esbuild, { PluginBuild } from "esbuild";
 import linaria from "@linaria/esbuild";
-import fs from "fs";
-import path from "path";
+import fs from "node:fs";
+import path from "node:path";
 import postcss from "postcss";
 import sass from "sass";
 import postcssrc from "postcss-load-config";
 
-// this should give us the current directory where
-// the project is being built
-const BASE = process.cwd();
-
 type Assets = {
   base: string;
-  files: string[];
+  files: (string | { orig: string; dest: string })[];
 };
 
 export function getFilesInDirectory(startPath: string, regex?: RegExp): Assets {
@@ -44,24 +40,9 @@ export function getFilesInDirectory(startPath: string, regex?: RegExp): Assets {
   };
 }
 
-let GIT_ROOT = BASE;
-while (!fs.existsSync(path.join(GIT_ROOT, ".git")) && GIT_ROOT !== "/") {
-  GIT_ROOT = path.join(GIT_ROOT, "../");
-}
-if (GIT_ROOT === "/") {
-  // eslint-disable-next-line no-undef
-  console.log("not found");
-  // eslint-disable-next-line no-undef
-  process.exit(1);
-}
-const GIT_HASH = git_hash();
-
-const buf = fs.readFileSync(path.join(BASE, "package.json"));
-let _package = JSON.parse(buf.toString("utf-8"));
-
-function git_hash() {
+function git_hash(root: string) {
   const rev = fs
-    .readFileSync(path.join(GIT_ROOT, ".git", "HEAD"))
+    .readFileSync(path.join(root, ".git", "HEAD"))
     .toString()
     .trim()
     .split(/.*[: ]/)
@@ -69,7 +50,10 @@ function git_hash() {
   if (rev.indexOf("/") === -1) {
     return rev;
   } else {
-    return fs.readFileSync(path.join(GIT_ROOT, ".git", rev)).toString().trim();
+    return fs
+      .readFileSync(path.join(root, ".git", rev))
+      .toString()
+      .trim();
   }
 }
 
@@ -89,13 +73,17 @@ function copyFilesPlugin(assets: Assets | Assets[]) {
       }
       build.onEnd(() => {
         list.forEach((as) => {
-          for (const file of as.files) {
-            const destination = path.join(outDir, path.relative(as.base, file));
+          for (const fileName of as.files) {
+            const { orig, dest } =
+              typeof fileName === "string"
+                ? { orig: fileName, dest: fileName }
+                : fileName;
+            const destination = path.join(outDir, path.relative(as.base, dest));
             const dirname = path.dirname(destination);
             if (!fs.existsSync(dirname)) {
               fs.mkdirSync(dirname, { recursive: true });
             }
-            fs.copyFileSync(file, destination);
+            fs.copyFileSync(orig, destination);
           }
         });
       });
@@ -121,50 +109,48 @@ const sassPlugin: esbuild.Plugin = {
   },
 };
 
-
 /**
- * Problem: 
+ * Problem:
  *   No loader is configured for ".node" files: ../../node_modules/.pnpm/fsevents@2.3.3/node_modules/fsevents/fsevents.node
- * 
+ *
  * Reference:
  *   https://github.com/evanw/esbuild/issues/1051#issuecomment-806325487
  */
 const nativeNodeModulesPlugin: esbuild.Plugin = {
-  name: 'native-node-modules',
+  name: "native-node-modules",
   setup(build) {
-    // If a ".node" file is imported within a module in the "file" namespace, resolve 
+    // If a ".node" file is imported within a module in the "file" namespace, resolve
     // it to an absolute path and put it into the "node-file" virtual namespace.
-    build.onResolve({ filter: /\.node$/, namespace: 'file' }, args => ({
+    build.onResolve({ filter: /\.node$/, namespace: "file" }, (args) => ({
       path: require.resolve(args.path, { paths: [args.resolveDir] }),
-      namespace: 'node-file',
-    }))
+      namespace: "node-file",
+    }));
 
     // Files in the "node-file" virtual namespace call "require()" on the
     // path from esbuild of the ".node" file in the output directory.
-    build.onLoad({ filter: /.*/, namespace: 'node-file' }, args => ({
+    build.onLoad({ filter: /.*/, namespace: "node-file" }, (args) => ({
       contents: `
         import path from ${JSON.stringify(args.path)}
         try { module.exports = require(path) }
         catch {}
       `,
-    }))
+    }));
 
     // If a ".node" file is imported within a module in the "node-file" namespace, put
     // it in the "file" namespace where esbuild's default loading behavior will handle
     // it. It is already an absolute path since we resolved it to one above.
-    build.onResolve({ filter: /\.node$/, namespace: 'node-file' }, args => ({
+    build.onResolve({ filter: /\.node$/, namespace: "node-file" }, (args) => ({
       path: args.path,
-      namespace: 'file',
-    }))
+      namespace: "file",
+    }));
 
     // Tell esbuild's default loading behavior to use the "file" loader for
     // these ".node" files.
-    let opts = build.initialOptions
-    opts.loader = opts.loader || {}
-    opts.loader['.node'] = 'file'
+    let opts = build.initialOptions;
+    opts.loader = opts.loader || {};
+    opts.loader[".node"] = "file";
   },
-}
-
+};
 
 const postCssPlugin: esbuild.Plugin = {
   name: "custom-build-postcss",
@@ -227,25 +213,69 @@ const defaultEsBuildConfig: esbuild.BuildOptions = {
     react: "preact/compat",
     "react-dom": "preact/compat",
   },
-  define: {
-    __VERSION__: `"${_package.version}"`,
-    __GIT_HASH__: `"${GIT_HASH}"`,
-  },
 };
-
+import nodePath from "node:path";
+import nodeUrl from "node:url";
 export interface BuildParams {
   type: "development" | "test" | "production";
+  /**
+   * Assign this to import.meta so the script can be called from any directory
+   */
+  importMeta?: ImportMeta;
   source: {
     assets: Assets | Assets[];
-    js: string[];
+    js: string[] | Record<string, string>;
   };
   public?: string;
+  /**
+   * Location directory of the output. Can be redirected with INSTALL_DIR environment
+   */
   destination: string;
   css: "sass" | "postcss" | "linaria";
   linariaPlugin?: () => esbuild.Plugin;
 }
 
-export function computeConfig(params: BuildParams): esbuild.BuildOptions {
+function getPackageAndGitRoot(meta: undefined | ImportMeta) {
+  if (meta) {
+    const root = nodePath.dirname(nodeUrl.fileURLToPath(meta.url));
+    process.chdir(root);
+  }
+  const baseDir = process.cwd();
+
+  let GIT_ROOT = baseDir;
+  while (!fs.existsSync(path.join(GIT_ROOT, ".git")) && GIT_ROOT !== "/") {
+    GIT_ROOT = path.join(GIT_ROOT, "../");
+  }
+  if (GIT_ROOT === "/") {
+    return { version: 'not defined', hash: 'not defined', baseDir }
+  }
+  const hash = git_hash(GIT_ROOT);
+
+  const buf = fs.readFileSync(path.join(baseDir, "package.json"));
+  const version = JSON.parse(buf.toString("utf-8")).version;
+  return { version, hash, baseDir };
+}
+
+export function computeConfig(params: BuildParams): {
+  esBuildOptions: esbuild.BuildOptions;
+  version: string;
+} {
+  const {
+    version,
+    hash: GIT_HASH,
+    baseDir,
+  } = getPackageAndGitRoot(params.importMeta);
+
+  if (!params.type) {
+    throw Error(
+      `missing build type, it should be "test", "development" or "production"`,
+    );
+  }
+
+  if (!params.destination) {
+    throw Error(`missing destination folder`);
+  }
+
   const plugins: Array<esbuild.Plugin> = [
     copyFilesPlugin(params.source.assets),
   ];
@@ -274,57 +304,72 @@ export function computeConfig(params: BuildParams): esbuild.BuildOptions {
       }
     }
   }
-  if (!params.type) {
-    throw Error(
-      `missing build type, it should be "test", "development" or "production"`,
-    );
-  }
 
-  // if (!params.source.js) {
-  //   throw Error(`no javascript entry points, nothing to compile?`);
-  // }
-  if (!params.destination) {
-    throw Error(`missing destination folder`);
-  }
-
-  return {
+  const esBuildOptions: esbuild.BuildOptions = {
     ...defaultEsBuildConfig,
+    absWorkingDir: baseDir,
     entryPoints: params.source.js,
     publicPath: params.public,
-    outdir: params.destination,
+    outdir: process.env.INSTALL_DIR ?? params.destination,
     treeShaking: true,
     minify: false, //params.type === "production",
     sourcemap: true, //params.type !== "production",
     define: {
-      ...defaultEsBuildConfig.define,
       "process.env.NODE_ENV": JSON.stringify(params.type),
+      __VERSION__: `"${version}"`,
+      __GIT_HASH__: `"${GIT_HASH}"`,
     },
     plugins,
   };
+
+  return { esBuildOptions, version };
 }
 
 /**
  * Build sources for prod environment
  */
 export async function build(config: BuildParams) {
-  const res = await esbuild.build(computeConfig(config));
-  fs.writeFileSync(`${config.destination}/version.txt`, `${_package.version}`);
+  const { esBuildOptions, version } = computeConfig(config);
+  const res = await esbuild.build(esBuildOptions);
+  fs.writeFileSync(`${esBuildOptions.outdir}/version.txt`, version);
   return res;
 }
 
 const LIVE_RELOAD_SCRIPT =
   "./node_modules/@gnu-taler/web-util/lib/live-reload.mjs";
+const LIVE_RELOAD_SCRIPT_LOCALLY = "./lib/live-reload.mjs";
 
 /**
  * Do startup for development environment
+ *
+ * To be used from web-utils project
  */
-export function initializeDev(
+export function initializeDevOnWebUtils(
   config: BuildParams,
 ): () => Promise<esbuild.BuildResult> {
   function buildDevelopment() {
-    const result = computeConfig(config);
-    result.inject = [LIVE_RELOAD_SCRIPT];
-    return esbuild.build(result);
+    const { esBuildOptions } = computeConfig(config);
+    esBuildOptions.inject = [LIVE_RELOAD_SCRIPT_LOCALLY];
+    return esbuild.build(esBuildOptions);
+  }
+  return buildDevelopment;
+}
+
+/**
+ * Do startup for development environment
+ *
+ * To be used when web-utils is a library
+ */
+export function initializeDev(
+  config: BuildParams,
+  options: { dontDoLiveReload?: boolean } = {},
+): () => Promise<esbuild.BuildResult> {
+  function buildDevelopment() {
+    const { esBuildOptions } = computeConfig(config);
+    if (!options.dontDoLiveReload) {
+      esBuildOptions.inject = [LIVE_RELOAD_SCRIPT];
+    }
+    return esbuild.build(esBuildOptions);
   }
   return buildDevelopment;
 }
